# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

from openerp.osv import fields, osv
from StringIO import StringIO
from openerp.tools.translate import _
from xlwt import easyxf
import datetime
import openerp.netsvc
import base64
import time
import logging
import calendar
from lxml import etree
from dateutil.relativedelta import relativedelta
from types import ListType

_logger = logging.getLogger('usi.generic.report')

class report_excel_output(osv.osv_memory):
    _name = 'report.excel.output'
    _description = "Excel Report Output"
    
    _columns = {
        'name': fields.char('Filename', 256, readonly=True),
        'data': fields.binary('File', readonly=True, required=True),
        'date': fields.datetime('Creation Date', readonly=True, required=True)
    }
    
    _defaults = {
        'date': lambda *a:time.strftime('%Y-%m-%d %H:%M:%S')
    }
report_excel_output()

class Abstract_Report_Excel(osv.osv_memory):
    _name = 'abstract.report.excel'
    _description = 'Abstract Report Excel'
    
    _columns = {
        'save': fields.boolean('Save to document storage'),
    }
    
    def get_easyxf_styles(self):
        styledict = {
            'title_xf': easyxf('font: bold on, height 250; align: wrap off, vert centre, horiz left;'),
            'too': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz center,rota 90; borders: top thin, left thin, bottom thin, right thin, top_colour gray50,bottom_colour gray50,left_colour gray50,right_colour gray50; pattern: pattern solid, fore_color pale_blue'),
            'small_title_xf': easyxf('font: bold on, height 190; align: wrap off, vert centre, horiz left;'),
            'heading_xf': easyxf('font: bold on, height 180; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gold'),
            'heading_white_xf': easyxf('font: bold on, height 180; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin'),
            'heading_grey_text_center_bold_xf': easyxf('font: bold on, height 180; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gray25'),
            'text_xf': easyxf('font: bold off, height 160; align: wrap on, vert centre, horiz left; borders: top thin, left thin, bottom thin, right thin;'),
            'text_right_xf': easyxf('font: bold off, height 160; align: wrap on, vert centre, horiz right; borders: top thin, left thin, bottom thin, right thin;'),
            'text_bold_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz left; borders: top thin, left thin, bottom thin, right thin;'),
            'text_bold_right_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz right; borders: top thin, left thin, bottom thin, right thin;'),
            'text_center_bold_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin;'),
            'text_center_xf': easyxf('font: bold off, height 160; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin;'),
            'number_xf': easyxf('font: height 160; align: horz right; borders: top thin, left thin, bottom thin, right thin;', num_format_str='#,##0.##'),
            'number_bold_xf': easyxf('font: height 160, bold on; align: horz right; borders: top thin, left thin, bottom thin, right thin;', num_format_str='#,##0.##'),
            'grey_text_bold_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz left; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gray25'),
            'grey_text_bold_right_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz right; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gray25'),
            'grey_text_center_bold_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gray25'),
            'grey_number_bold_xf': easyxf('font: height 160, bold on; align: horz right; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gray25', num_format_str='#,##0.##'),
            'gold_text_bold_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz left; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gold'),
            'gold_text_bold_right_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz right; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gold'),
            'gold_text_center_bold_xf': easyxf('font: bold on, height 160; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gold'),
            'gold_number_bold_xf': easyxf('font: height 160, bold on; align: horz right; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour gold', num_format_str='#,##0.##'),
        }
        return styledict
    
    def get_log_message(self, cr, uid, ids, context=None):
        return u"Unimplemented log"
    
    def force_make_attachment(self, cr, uid, datas, directory_name, 
            attache_name, context=None):
        ''' Excel тайланг шууд баримтын серверт хадгална.
        '''
        if context is None:
            context = {}
        if attache_name:
            aname = "%s_%s.xls" % (attache_name, time.strftime('%Y%m%d_%H%M'))
        else :
            aname = '%s_%s.xls' % (self._name.replace('.','_'), time.strftime('%Y%m%d%H%M%S'))
        directory = False
        if directory_name:
            directory_ids = self.pool.get('document.directory').search(cr, 1, 
                        [('name','=',directory_name)], context=context)
            if directory_ids :
                directory = directory_ids[0]
        try:
            model_name = self._name
            res_id = False
            if 'generator_id' in context:
                model_name = 'report.schedule.generator'
                res_id = context['generator_id']
            document_name = context.get('force_document_name', context.get('attache_name', aname))
            self.pool.get('ir.attachment').create(cr, uid, {
                'name': document_name+' [%s]' % time.strftime('%Y%m%d%H%M%S'),
                'datas': datas,
                'datas_fname': aname,
                'res_model': model_name,
                'res_id': res_id,
                'parent_id': directory
                }, context=context)
            if context.get('other_users',[]):
                for uid2 in context['other_users']:
                    self.pool.get('ir.attachment').create(cr, uid2, {
                        'name': document_name+' [%s]' % time.strftime('%Y%m%d%H%M%S'),
                        'datas': datas,
                        'datas_fname': aname,
                        'res_model': model_name,
                        'res_id': res_id,
                        'parent_id': directory
                    }, context=context)
        except Exception:
            #TODO: should probably raise a proper osv_except instead, shouldn't we? see LP bug #325632
            logging.getLogger('report').error('Could not create saved report attachment', exc_info=True)
        return aname
    
    def print_report(self, cr, uid, ids, data, context=None):
        if context is None:
            context = {}
        data = self.prepare_data(cr, uid, ids, context=context)
        
        body = self.get_log_message(cr, uid, ids, context=context)
        message = u"[Тайлан][PDF][PROCESSING] %s" % body
        context.update({'report_log':True})
        log_id = self.log(cr, uid, ids[0], message, context=context)
        data['log_body'] = body
        data['log_id'] = log_id
        data['log_start'] = time.strftime('%Y-%m-%d %H:%M:%S')
        _logger.debug(message)
        
        return {
            'type': 'ir.actions.report.xml',
            'report_name': data['report_name'],
            'datas': data,
            'context': context
        }
    
    def export_report(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        body = self.get_log_message(cr, uid, ids, context=context)
        if 'generator_id' in context:
            message = u"[Автомат тайлан][XLS][PROCESSING] %s" % body
            prefix = u"[Автомат тайлан][XLS][PROCESSING]"
        else :
            message = u"[Тайлан][XLS][PROCESSING] %s" % body
            prefix = u"[Тайлан][XLS][PROCESSING]"
        _logger.debug(message)
        context.update({'report_log':True})
        log_id = self.log(cr, uid, ids[0], message, context=context)
        
        d1 = datetime.datetime.now()
        res = self.get_export_data(cr, uid, ids, context=context)
        
        book = res['data'] # Workbook object instance.
        buffer = StringIO()
        book.save(buffer)
        buffer.seek(0)
        
        out = base64.encodestring(buffer.getvalue())
        buffer.close()
        
        # Force save if needed.
        if res.get('force_save', False) == True and res.get('directory_name', False):
            context.update({'force_document_name': body[:128]})
            filename = self.force_make_attachment(cr, uid, out, res['directory_name'], 
                        res.get('attache_name',False), context=context)
        else :
            filename = self._name.replace('.', '_')
            filename = "%s_%s.xls" % (filename, time.strftime('%Y%m%d_%H%M'),)
            if 'xlsx' in res:
                filename += 'x' # Office 2007
        
        excel_id = self.pool.get('report.excel.output').create(cr, uid, {
                                'data':out,
                                'name':filename
        }, context=context)
        
        mod_obj = self.pool.get('ir.model.data')
        form_res = mod_obj.get_object_reference(cr, uid, 'hr_biometric_machine', 'action_excel_output_view')
        form_id = form_res and form_res[1] or False
        
        dta = datetime.datetime.now() - d1
        if dta.seconds > 60 :
            tm = '%sm %ss' %(dta.seconds / 60, dta.seconds % 60)
        else :
            tm = '%ss' % dta.seconds
        if 'generator_id' in context:
            prefix = u"[Автомат тайлан][XLS][COMPLETE (%s)]" % (tm,)
        else :
            prefix = u"[Тайлан][XLS][COMPLETE (%s)]" % (tm,)
        message = u"%s %s" % (prefix, body)
        #self.pool.get('res.log').write(cr, uid, [log_id], {'name': message,'create_date':time.strftime('%Y-%m-%d %H:%M:%S')})
        _logger.debug(message)
        
#        return {
#            'name': _('Export Result'),
#            'view_type': 'form',
#            'view_mode': 'tree,form',
#            'res_model': 'report.excel.output',
#            'view_id': False,
#            'type': 'ir.actions.act_window',
#            #'search_view_id': res_id,
#            'domain': "[('id', 'in', ["+ str(excel_id) +"])]"
#        }
        
        return {
            'name': _('Export Result'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'report.excel.output',
            'res_id': excel_id,
            'view_id': False,
            'views': [(form_id, 'form')],
            'context': context,
            'type': 'ir.actions.act_window',
            'target':'new',
            'nodestroy': True,
        }
    
    def get_export_data(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Programming Error!'), _('Unimplemented method.'))

Abstract_Report_Excel()

#def update_date(dd, k, new):
#    if k == 'start':
#        if 'date_start' in dd:
#            dd['date_start'] = new
#        elif 'date_from' in dd:
#            dd['date_from'] = new
#        elif 'start_date' in dd:
#            dd['start_date'] = new
#        elif 'from_date' in dd:
#            dd['from_date'] = new
#    else :
#        if 'date_stop' in dd:
#            dd['date_stop'] = new
#        elif 'date_to' in dd:
#            dd['date_to'] = new
#        elif 'stop_date' in dd:
#            dd['stop_date'] = new
#        elif 'to_date' in dd:
#            dd['to_date'] = new
#
#def get_period(dd):
#    if 'period_stop' in dd:
#        return dd['period_stop']
#    elif 'period_to' in dd:
#        return dd['period_to']
#    elif 'stop_period' in dd:
#        return dd['stop_period']
#    elif 'to_period' in dd:
#        return dd['to_period']
#
#def get_date(dd):
#    if 'date_stop' in dd:
#        return dd['date_stop']
#    elif 'date_to' in dd:
#        return dd['date_to']
#    elif 'stop_date' in dd:
#        return dd['stop_date']
#    elif 'to_date' in dd:
#        return dd['to_date']
#    return False
#
#def update_period(dd, k, new):
#    if k == 'start':
#        if 'period_start' in dd:
#            dd['period_start'] = new
#        elif 'period_from' in dd:
#            dd['period_from'] = new
#        elif 'start_period' in dd:
#            dd['start_period'] = new
#        elif 'from_period' in dd:
#            dd['from_period'] = new
#    else :
#        if 'period_stop' in dd:
#            dd['period_stop'] = new
#        elif 'period_to' in dd:
#            dd['period_to'] = new
#        elif 'stop_period' in dd:
#            dd['stop_period'] = new
#        elif 'to_period' in dd:
#            dd['to_period'] = new
#
#class report_schedule_generator(osv.osv):
#    _name = 'report.schedule.generator'
#    _description = 'Report Schedule Generator'
#    logger = netsvc.Logger()
#    
#    REPORT_SELECTION = [
#        ('report.sales.gain.loss.partner', u'Борлуулалтын ашиг алдагдлын тайлан /Харилцагч/'),
#        ('report.sales.gain.loss.product', u'Борлуулалтын ашиг алдагдлын тайлан /Бараа/'),
#        ('report.sales.shop', u'Борлуулалтын тайлан /Салбар/'),
#        ('report.sales.shop.days', u'Өдрийн борлуулалтын тайлан'),
#        ('report.sales.product.qty', u'Борлуулалтын тайлан /Тоо ширхэг/'),
#        ('report.sales.day.month', u'Өдрийн захиалгын тайлан'),
#        ('report.sales.salesman.day', u'Өдрийн захиалгын тайлан-Харилцагчаар'),
#        ('report.sales.salesman.month', u'Захиалгын тайлан /Сараар/'),
#        ('report.sales.productgainandloss', u'Борлуулалтын ашиг алдагдлын тайлан'),
#        ('sales.info.graph', u'Борлуулалтын мэдээний хүснэгт'),
#        ('report.expiring.products', u'Бараа материалын дуусах хугацааны тайлан'),
#        ('report.internal.expense', u'Дотоод зарлагын тайлан'),
#        ('report.move.by_cip', u'Барааны хөдөлгөөний тайлан (Cip үнээр)'),
#        ('report.move.by_price', u'Барааны хөдөлгөөний тайлан (Худалдах үнээр)'),
#        ('report.move.by_supplier', u'Барааны хөдөлгөөний тайлан (Нийлүүлэгчээр)'),
#        ('product.compare', u'Барааны харицуулалт хийсэн тайлан'),
#        ('report.purchase', u'Татан авалтын тайлан'),
#        ('report.stock.two', u'Барааны нөөцийн тайлан 2'),
#        ('report.product.ledger', u'Бараа материалын товчоо тайлан'),
#        ('account.product.ledger.report', u'Бараа материалын дэлгэрэнгүй бүртгэл'),
#    ]
#    
#    INCRE_SELECTION = [
#        ('month','Month'),
#        ('week', 'Week'),
#        ('period', 'Period'),
#        ('day', 'Day')
#    ]
#    
#    _columns = {
#        'name':             fields.char('Name', size=256, required=True),
#        #'attache_name' :    fields.char('Attache Name', size=256, required=True),
#        'report_name':      fields.selection(REPORT_SELECTION, 'Report', size=256, required=True),
#        'user_id':          fields.many2one('res.users', 'Report Requester', required=True),
#        'active':           fields.boolean('Active'),
#        'params':           fields.text('Report Data', readonly=True),
#        'date_last':        fields.datetime('Last Generated Date', readonly=True),
#        'date_next':        fields.date('Next Date', required=True),
#        'increment':        fields.integer('Increment', required=True),
#        'increment_type':   fields.selection(INCRE_SELECTION, 'Increment Mode', required=True),
#        'fixed':            fields.boolean('Fixed start date?'),
#        'user_ids':         fields.many2many('res.users', 'report_schedule_generator_users_rel', 'generator_id', 'user_id', 'Other Users')
#    }
#    
#    _defaults = {
#        'user_id': lambda obj,cr,uid,c:uid,
#        'date_next': lambda *a: time.strftime('%Y-%m-%d'),
#        'increment_type': 'day',
#        'fixed': False
#    }
#    
#    def onchange_report_name(self, cr, uid, ids, report, context=None):
#        if report:
#            return {'value':{'name':dict(self.REPORT_SELECTION).get(report, '')}}
#        return {}
#    
#    def run_report_generator(self, cr, uid, ids, context=None):
#        ''' Автомат тайлангуудыг ажиллуулна.
#        '''
#        if context is None:
#            context = {}
#        if not ids:
#            ids = self.search(cr, uid, [], context=context)
#        generator_ids = self.search(cr, uid, [('active','=',True),
#                    ('date_next','<=',time.strftime('%Y-%m-%d')),
#                    ('id','in',ids)], context=context)
#        for gen in self.browse(cr, uid, generator_ids, context=context):
#            report_obj = self.pool.get(gen.report_name)
#            
#            wizard_data = eval(gen.params)
#            wizard_data.update({'save':True})
#            for key in wizard_data:
#                if isinstance(wizard_data.get(key), ListType):
#                    wizard_data[key] = [[6, 0, wizard_data.get(key)]]
#            generate_type = wizard_data.pop('generate_type')
#            report_wiz_id = report_obj.create(cr, gen.user_id.id, wizard_data, context=context)
#            context.update({'generator_id': gen.id})
#            context.update({'attache_name':gen.name})
#            if gen.user_ids:
#                other_users = map(lambda x:x.id,gen.user_ids)
#                context.update({'other_users':other_users})
#            if generate_type == 'excel':
#                result = report_obj.export_report(cr, gen.user_id.id, 
#                        [report_wiz_id], context=context)
#            else :
#                result = report_obj.print_report(cr, gen.user_id.id, 
#                        [report_wiz_id], context=context)
#                report_service = netsvc.LocalService('report.'+result['report_name'])
#                report_service.cr = cr
#                report_service.uid = gen.user_id.id
#                report_service.datas = result['datas']
#                report_service.context = context
#                report_service.pool = self.pool
#                report_service.lang = context.get('lang', None)
#                report_service.create(cr, uid, [], result['datas'], context)
#            
#            vals = {}
#            wizard_data = eval(gen.params)
#            if gen.increment_type <> 'period':
#                date_start = get_date(wizard_data)
#                date_start = datetime.datetime.strptime(date_start, '%Y-%m-%d')
#                date_stop = date_start
#                if gen.increment_type == 'day':
#                    date_stop += relativedelta(days=gen.increment)
#                elif gen.increment_type == 'month':
#                    date_stop += relativedelta(months=gen.increment+1)
#                    date_stop = datetime.datetime.strptime(date_stop.strftime('%Y-%m-01'), '%Y-%m-%d') - relativedelta(days=1)
#                else :#'week'
#                    date_stop += relativedelta(days=gen.increment*7)
#                date_start += relativedelta(days=1)
#                vals.update({'date_next':date_stop.strftime('%Y-%m-%d')})
#                if not gen.fixed:
#                    update_date(wizard_data, 'start', date_start.strftime('%Y-%m-%d')) # Old date
#                update_date(wizard_data, 'stop', date_stop.strftime('%Y-%m-%d'))
#            else : # by sales period
#                period_obj = self.pool.get('stock.period')
#                current_period_id = get_period(wizard_data)
#                if not current_period_id:
#                    current_period_id = period_obj.find(cr, uid, context=context)[0]
#                current_period = period_obj.browse(cr, uid, current_period_id, context)
#                next_period_id = period_obj.next(cr, uid, current_period, step=1, context=context)
#                if next_period_id:
#                    next_period = period_obj.browse(cr, uid, next_period_id, context)
#                else :
#                    next_period = current_period
#                next_period0 = next_period
#                if gen.increment > 1:
#                    next_period_id = period_obj.next(cr, uid, next_period, step=gen.increment-1, context=context)
#                    next_period = period_obj.browse(cr, uid, next_period_id, context)
#                
#                vals.update({'date_next':next_period.date_stop})
#                if not gen.fixed:
#                    update_period(wizard_data, 'start', next_period0.id)
#                update_period(wizard_data, 'stop', next_period.id)
#            
#            vals.update({'date_last': time.strftime('%Y-%m-%d %H:%M:%S'),
#                         'params':wizard_data.__repr__()})
#            self.write(cr, uid, [gen.id], vals, context=context)
#            
#        return True
#    
#report_schedule_generator()
#
#class report_schedule_generator_wizard(osv.osv_memory):
#    _name = 'report.schedule.generator.wizard'
#    _description = 'Report Schedule Generator Wizard'
#    
#    _columns = {
#        #'report_type': fields.selection([('excel','Excel'),('pdf','PDF')], 'Report Type', required=True)
#        'wizard_id': fields.integer('Wizard Object ID')
#    }
#    
#    def default_get(self, cr, uid, fields, context=None):
#        """
#         To get default values for the object.
#         @param self: The object pointer.
#         @param cr: A database cursor
#         @param uid: ID of the user currently logged in
#         @param fields: List of fields for which we want default values
#         @param context: A standard dictionary
#         @return: A dictionary with default values for all field in ``fields``
#        """
#        if context is None:
#            context = {}
#        
#        generator = self.pool.get('report.schedule.generator').browse(cr,
#                        uid, context['active_id'], context=context)
#        report_obj = self.pool.get(generator.report_name)
#        report_fields = report_obj.fields_get_keys(cr, uid, context)
#        res = report_obj.default_get(cr, generator.user_id.id, report_fields, context=context)
#        return res
#
#    def fields_view_get(self, cr, uid, view_id=None, view_type='form',
#                        context=None, toolbar=False, submenu=False):
#        """
#         Changes the view dynamically
#         @param self: The object pointer.
#         @param cr: A database cursor
#         @param uid: ID of the user currently logged in
#         @param context: A standard dictionary
#         @return: New arch of view.
#         
#        """
#        if context['active_model'] == 'report.schedule.generator':
#            generator = self.pool.get('report.schedule.generator').browse(cr,
#                            uid, context['active_id'], context=context)
#            report_obj = self.pool.get(generator.report_name)
#            report_fields = report_obj.fields_get_keys(cr, uid, context)
#            res = report_obj.fields_view_get(cr, uid, view_id=view_id, view_type=view_type, 
#                context=context, toolbar=toolbar,submenu=False)
#            doc = etree.XML(res['arch'])
#            nodes = doc.xpath('//group[@name="control_bar"]')
#            for node in nodes:
#                doc.remove(node)
#            root_nodes = doc.xpath('//form')
#            if root_nodes:
#                group = etree.SubElement(root_nodes[0], 'group', attrib={'colspan':"4",'col':"4"})
#                etree.SubElement(group, 'button', attrib={
#                        'special': "cancel",
#                        'string': _("Close"),
#                        'type': "object",
#                        'icon':"gtk-cancel",
#                        'colspan':"1"
#                })
#                etree.SubElement(group, 'label', attrib={'colspan':'1'})
#                etree.SubElement(group, 'button', attrib={
#                        'name': "print_report",
#                        'string': _("Print"),
#                        'type': "object",
#                        'icon':"gtk-print",
#                        'colspan':"1"
#                })
#                etree.SubElement(group, 'button', attrib={
#                        'name': "export_report",
#                        'string': _("Export"),
#                        'type': "object",
#                        'icon':"terp-graph",
#                        'colspan':"1"
#                })
#            res['arch'] = etree.tostring(doc)
#            
#            #report_fields = report_obj.fields_get_keys(cr, uid, context)
#            #print 'fields_get :', report_obj.fields_get(cr, uid, report_fields, context)
#            #res0 = super(report_schedule_generator_wizard, self).fields_view_get(cr, uid, 
#            #    view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
#        else :
#            res = super(report_schedule_generator_wizard, self).fields_view_get(cr, uid, 
#                view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
#        return res
#    
#    def create(self, cr, uid, vals, context=None):
#        generator = self.pool.get('report.schedule.generator').browse(cr,
#                        uid, context['active_id'], context=context)
#        report_obj = self.pool.get(generator.report_name)
#        wizard_id = report_obj.create(cr, uid, vals, context=context)
#        vals = {'wizard_id': wizard_id}
#        return super(report_schedule_generator_wizard, self).create(cr, uid, vals, context=context)
#    
#    def make_params(self, cr, uid, ids, extra, context=None):
#        if context is None:
#            context = {}
#        generator = self.pool.get('report.schedule.generator').browse(cr,
#                            uid, context['active_id'], context=context)
#        report_obj = self.pool.get(generator.report_name)
#        
#        obj = self.browse(cr, uid, ids[0], context=context)
#        data = report_obj.read(cr, uid, obj.wizard_id, context=context)
#        del data['id']
#        del data['save']
#        next_date = get_date(data)
#        incr_type = 'day'
#        if not next_date:
#            period = get_period(data)
#            if period:
#                next_date = self.pool.get('stock.period').browse(cr, uid, period).date_stop
#                incr_type = 'period'
#        if not next_date:
#            next_date = time.strftime('%Y-%m-%d')
#        data.update(extra)
#        self.pool.get('report.schedule.generator').write(cr, uid, context['active_id'], 
#               {'params':data.__repr__(), 'date_next':next_date,
#                'increment_type':incr_type}, context=context)
#        return True
#    
#    def print_report(self, cr, uid, ids, context=None):
#        self.make_params(cr, uid, ids, {'generate_type':'pdf'}, context=context)
#        
#        return {'type':'ir.actions.act_window.close'}
#    
#    def export_report(self, cr, uid, ids, context=None):
#        self.make_params(cr, uid, ids, {'generate_type':'excel'}, context=context)
#        
#        return {'type':'ir.actions.act_window.close'}
#
#report_schedule_generator_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: