# -*- coding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2012 Agile Business Group sagl (<http://www.agilebg.com>)
#    Copyright (C) 2012 Domsense srl (<http://www.domsense.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
import openerp.addons.decimal_precision as dp
from openerp.tools.translate import _
from dateutil.relativedelta import relativedelta
import time
from datetime import timedelta, datetime


class account_account(osv.osv):
    _inherit = 'account.account'
    _name = 'account.account'
    
    def get_initial_balance(self, cr, uid, ids, context=None):
        ''' Тухайн дансны тайлант хугацааны эхний үлдэгдлийг олно.
        '''
        if not context:
            context = {}
        
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        fiscalyears = fiscalyear_obj.search(cr, uid, [('date_start', '<', context['date_to']), ('date_stop', '>', context['date_from'])], limit=1)
        fiscalyear = fiscalyear_obj.browse(cr, uid, fiscalyears[0], context = context)
        # Тухайн санхүүгийн жилд эхний үлдэгдэл бичсэн санхүүгийн журналыг тодорхойлно.
        date_from = context.get('date_from', fiscalyear.date_start)
        date_to = context.get('date_to', fiscalyear.date_stop)
        
        ctx = context.copy()
        start = datetime.strptime(date_from, "%Y-%m-%d")
        ctx.update({
            'date_from': str(start),
            'date_to': date_to,
            'fiscalyear': fiscalyear.id
        })
        ctx2 = ctx.copy()
        
        if date_from <> fiscalyear.date_start :
             ctx2.update({
                 'initial_bal': True,
             })
        query = ''
        init_ctx = ctx2

        prev_fy_ids = fiscalyear_obj.search(cr, uid, [('date_stop', '<', fiscalyear.date_start)], order="date_start desc", limit=1)
        initial_bal_journal = False
        if prev_fy_ids :
            prev_fy = fiscalyear_obj.browse(cr, uid, prev_fy_ids[0], context = context)
            if prev_fy.end_journal_period_id :
                initial_bal_journal = prev_fy.end_journal_period_id.journal_id.id

        if not initial_bal_journal:
            init_ctx.update({
                 'initial_bal': True,
             })
        if date_from == fiscalyear.date_start and initial_bal_journal:
            init_ctx = ctx
            query += ' l.journal_id = %s' % initial_bal_journal

        initial_bal = self.compute_hook(cr, uid, ids, ['debit','credit'], context=init_ctx, query=query)
        return initial_bal
    
    def _compute_localised(self, cr, uid, ids, field_names, arg=None, context=None,
                  query='', query_params=()):
        """ compute the balance, debit and/or credit for the provided
        account ids
        Arguments:
        `ids`: account ids
        `field_names`: the fields to compute (a list of any of
                       'balance', 'debit' and 'credit')
        `arg`: unused fields.function stuff
        `query`: additional query filter (as a string)
        `query_params`: parameters for the provided query string
                        (__compute will handle their escaping) as a
                        tuple
        """
        mapping = {
            'balance': "COALESCE(SUM(l.debit),0) - COALESCE(SUM(l.credit), 0) as balance",
            'debit': "COALESCE(SUM(l.debit), 0) as debit",
            'credit': "COALESCE(SUM(l.credit), 0) as credit",
            # by convention, foreign_balance is 0 when the account has no secondary currency, because the amounts may be in different currencies
            'foreign_balance': "(SELECT CASE WHEN currency_id IS NULL THEN 0 ELSE COALESCE(SUM(l.amount_currency), 0) END FROM account_account WHERE id IN (l.account_id)) as foreign_balance",
        }
        #get all the necessary accounts
        children_and_consolidated = self._get_children_and_consol(cr, uid, ids, context=context)
        #compute for each account the balance/debit/credit from the move lines
        accounts = {}
        res = {}
        null_result = dict((fn, 0.0) for fn in field_names)
        if children_and_consolidated:
            aml_query = self.pool.get('account.move.line')._query_get(cr, uid, context=context)

            wheres = [""]
            if query.strip():
                wheres.append(query.strip())
            if aml_query.strip():
                wheres.append(aml_query.strip())
            filters = " AND ".join(wheres)
            # IN might not work ideally in case there are too many
            # children_and_consolidated, in that case join on a
            # values() e.g.:
            # SELECT l.account_id as id FROM account_move_line l
            # INNER JOIN (VALUES (id1), (id2), (id3), ...) AS tmp (id)
            # ON l.account_id = tmp.id
            # or make _get_children_and_consol return a query and join on that
            request = ("SELECT l.account_id as id, " +\
                       ', '.join(mapping.values()) +
                       " FROM account_move_line l" \
                       " WHERE l.account_id IN %s " \
                            + filters +
                       " GROUP BY l.account_id")
            params = (tuple(children_and_consolidated),) + query_params
            cr.execute(request, params)

            for row in cr.dictfetchall():
                accounts[row['id']] = row

            # consolidate accounts with direct children
            children_and_consolidated.reverse()
            brs = list(self.browse(cr, uid, children_and_consolidated, context=context))
            sums = {}
            currency_obj = self.pool.get('res.currency')
            while brs:
                current = brs.pop(0)
#                can_compute = True
#                for child in current.child_id:
#                    if child.id not in sums:
#                        can_compute = False
#                        try:
#                            brs.insert(0, brs.pop(brs.index(child)))
#                        except ValueError:
#                            brs.insert(0, child)
#                if can_compute:
                for fn in field_names:
                    sums.setdefault(current.id, {})[fn] = accounts.get(current.id, {}).get(fn, 0.0)
                    for child in current.child_id:
                        if child.company_id.currency_id.id == current.company_id.currency_id.id:
                            sums[current.id][fn] += sums[child.id][fn]
                        else:
                            sums[current.id][fn] += currency_obj.compute(cr, uid, child.company_id.currency_id.id, current.company_id.currency_id.id, sums[child.id][fn], context=context)

                # as we have to relay on values computed before this is calculated separately than previous fields
                if current.currency_id and current.exchange_rate and \
                            ('adjusted_balance' in field_names or 'unrealized_gain_loss' in field_names):
                    # Computing Adjusted Balance and Unrealized Gains and losses
                    # Adjusted Balance = Foreign Balance / Exchange Rate
                    # Unrealized Gains and losses = Adjusted Balance - Balance
                    adj_bal = sums[current.id].get('foreign_balance', 0.0) * current.exchange_rate
                    sums[current.id].update({'adjusted_balance': adj_bal, 'unrealized_gain_loss': adj_bal - sums[current.id].get('balance', 0.0)})

            for id in ids:
                res[id] = sums.get(id, null_result)
        else:
            for id in ids:
                res[id] = null_result
        return res
    
    def _set_credit_debit(self, cr, uid, account_id, name, value, arg, context=None):
        if context.get('config_invisible', True):
            return True

        account = self.browse(cr, uid, account_id, context=context)
        diff = value - getattr(account,name)
        if not diff:
            return True

        journal_obj = self.pool.get('account.journal')
        jids = journal_obj.search(cr, uid, [('type','=','situation'),('centralisation','=',1),('company_id','=',account.company_id.id)], context=context)
        if not jids:
            raise osv.except_osv(_('Error!'),_("You need an Opening journal with centralisation checked to set the initial balance."))

        period_obj = self.pool.get('account.period')
        pids = period_obj.search(cr, uid, [('special','=',True),('company_id','=',account.company_id.id)], context=context)
        if not pids:
            raise osv.except_osv(_('Error!'),_("There is no opening/closing period defined, please create one to set the initial balance."))

        move_obj = self.pool.get('account.move.line')
        move_id = move_obj.search(cr, uid, [
            ('journal_id','=',jids[0]),
            ('period_id','=',pids[0]),
            ('account_id','=', account_id),
            (name,'>', 0.0),
            ('name','=', _('Opening Balance'))
        ], context=context)
        if move_id:
            move = move_obj.browse(cr, uid, move_id[0], context=context)
            move_obj.write(cr, uid, move_id[0], {
                name: diff+getattr(move,name)
            }, context=context)
        else:
            if diff<0.0:
                raise osv.except_osv(_('Error!'),_("Unable to adapt the initial balance (negative value)."))
            nameinv = (name=='credit' and 'debit') or 'credit'
            move_id = move_obj.create(cr, uid, {
                'name': _('Opening Balance'),
                'account_id': account_id,
                'journal_id': jids[0],
                'period_id': pids[0],
                name: diff,
                nameinv: 0.0
            }, context=context)
        return True
    
    _columns={
              'balance': fields.function(_compute_localised, digits_compute=dp.get_precision('Account'), string='Balance', multi='balance'),
        'credit': fields.function(_compute_localised, fnct_inv=_set_credit_debit, digits_compute=dp.get_precision('Account'), string='Credit', multi='balance'),
        'debit': fields.function(_compute_localised, fnct_inv=_set_credit_debit, digits_compute=dp.get_precision('Account'), string='Debit', multi='balance'),
        'foreign_balance': fields.function(_compute_localised, digits_compute=dp.get_precision('Account'), string='Foreign Balance', multi='balance',
                                           help="Total amount (in Secondary currency) for transactions held in secondary currency for this account."),
        'adjusted_balance': fields.function(_compute_localised, digits_compute=dp.get_precision('Account'), string='Adjusted Balance', multi='balance',
                                            help="Total amount (in Company currency) for transactions held in secondary currency for this account."),
        'unrealized_gain_loss': fields.function(_compute_localised, digits_compute=dp.get_precision('Account'), string='Unrealized Gain or Loss', multi='balance',
                                                help="Value of Loss or Gain due to changes in exchange rate when doing multi-currency transactions."),

              }
    
    
    def compute_hook(self, cr, uid, ids, field_names, context=None, query=''):
         return self.__compute(cr, uid, ids, field_names, arg=None, context=context, query=query)
    
    def get_initial_balance(self, cr, uid, ids, context=None):
         ''' Тухайн дансны тайлант хугацааны эхний үлдэгдлийг олно.
         '''
         if not context:
             context = {}
         fiscalyear_obj = self.pool.get('account.fiscalyear')
         fiscalyear_id = context.get('fiscalyear', False)
         if not fiscalyear_id:
             now = time.strftime('%Y-%m-%d')
             fiscalyears = fiscalyear_obj.search(cr, uid, [('date_start', '<', now), ('date_stop', '>', now)], limit=1)
             fiscalyear_id = fiscalyears and fiscalyears[0] or False
          
         res = {}
         for id in ids :
             res.setdefault(id, {'debit':0.0, 'credit':0.0})
         if not fiscalyear_id:
             return res
          
         '''Тухайн санхүүгийн жилд эхний үлдэгдэл бичсэн санхүүгийн журналыг тодорхойлно.'''
         fiscalyear = fiscalyear_obj.browse(cr, uid, fiscalyear_id)
         fy_start = fiscalyear.date_start
         date_from = context.get('date_from', fiscalyear.date_start)
         date_to = context.get('date_to', fiscalyear.date_stop)
         ctx = context.copy()
         ctx.update({
             'date_from': date_from,
             'date_to': date_to,
             'fiscalyear': fiscalyear_id
         })
         ctx2 = ctx.copy()
         if date_from <> fy_start :
             ctx2.update({
                 'initial_bal': True,
             })
         prev_fy_ids = fiscalyear_obj.search(cr, uid, [('date_stop', '<', fiscalyear.date_start)], order="date_start desc", limit=1)
         initial_bal_journal = False
         if prev_fy_ids :
             prev_fy = fiscalyear_obj.browse(cr, uid, prev_fy_ids[0])
             if prev_fy.end_journal_period_id :
                 initial_bal_journal = prev_fy.end_journal_period_id.journal_id.id
          
         initial_bal = {}
         init_ctx = ctx2
         query = ''
         if date_from == fy_start:
             init_ctx = ctx
             if initial_bal_journal:
                 query += ' l.journal_id = %s' % initial_bal_journal
             else :
                 query += ' l.journal_id = -1'
             init_ctx.update({
                 'date_to': init_ctx['date_from']
             })
          
         initial_bal = self.compute_hook(cr, uid, ids, ['debit', 'credit'], context=init_ctx, query=query)
         dict((a['id'], a) for a in self.read(cr, uid, ids, ['debit','credit'], context=ctx2))
         if context.get('return_initial_bal_journal', False):
             initial_bal['initial_bal_journal'] = initial_bal_journal
         if context.get('return_journal_query', False):
             initial_bal['journal_query'] = query
             initial_bal['init_context'] = init_ctx
         return initial_bal
     
    def currency_equalize(self, cr, uid, ids, period, date, company, journal_id, narration, context=None):
        ''' Тухайн дансанд ханшийн тэгшитгэл хийж ханшийн өөрчлөлтөөс үүдэлтэй
            балансын зөрүүг ашиг алдагдлын дансанд бичиж үүссэн ажил гүйлгээнүүдийг
            ID бүхий list буцаана.
        '''
        if context is None:
            context = {}
        cur_obj = self.pool.get('res.currency')
        amove_line_obj = self.pool.get('account.move.line')
        amove_obj = self.pool.get('account.move')
        
        res = []
        ctx = context.copy()
        ctx['date_from'] = period.fiscalyear_id.date_start
        ctx['date_to'] = date
        ctx['state'] = 'posted'
        ctx['company_id'] = company.id
        ctx['fiscalyear'] = period.fiscalyear_id.id
        
        # Тайлант хугацааны гадаад валютаарх үлдэгдлийг тооцоолох
        cr.execute("SELECT l.account_id, COALESCE(SUM(l.amount_currency), 0) "
                   "FROM account_move_line l "
                   "LEFT JOIN account_account a ON l.account_id = a.id "
                   "LEFT JOIN account_move m ON m.id = l.move_id "
                   "WHERE l.state <> 'draft' and m.state = 'posted' "
                   "AND l.date >= %s and l.date <= %s and l.company_id = %s "
                   "AND l.account_id in %s "
                   "GROUP BY l.account_id ",
            (period.fiscalyear_id.date_start, date, company.id, tuple(ids)))
        amount_currency_dict = dict(cr.fetchall())
        
        for account in self.browse(cr, uid, ids, context=ctx) :
            if not account.currency_id or account.currency_id.id == company.currency_id.id :
                continue
            
            # Тайлант хугацааны төгрөгөөрх үлдэгдэл
            if account.user_type.report_type in ('expense', 'asset') :
                balance = account.debit - account.credit
            else :
                balance = account.credit - account.debit
            
            amount_currency = amount_currency_dict.get(account.id, 0)
            
            # Валютын дүнгээрх үлдэгдэл
            # Тухайн өдрийн ханшаар валютын үлдэгдлийг үржүүлэхэд дансны үлдэгдэл гарна.
            #  Үүнийг бодит үлдэгдэлтэй харьцуулж ханшийн зөрүү дүнг олно.
            converted_amount = cur_obj.compute(cr, uid, account.currency_id.id, company.currency_id.id,
                        amount_currency, context={'date':date, 'lang':context.get('lang', 'en_US')})
            print "kkkkkkk",converted_amount , balance
            diff_amount = -converted_amount - balance
            print "aaaaa",diff_amount
            if cur_obj.is_zero(cr, uid, company.currency_id, diff_amount) : 
                # Ханшийн зөрүү үүсээгүй болно.
                continue
                
            move_id = amove_obj.create(cr, uid, {
                        'journal_id': journal_id,
                        'period_id': period.id,
                        'date': date,
                        'narration': narration
            }, context=context)
            res.append(move_id)
            
            if diff_amount > 0 :
                # Ханшийн зөрүүний ашиг
                a = company.income_currency_exchange_account_id.id
                if not a :
                    raise osv.except_osv(_('Warning!'), _('There is no income currency rate account defined for this company : %s') % company.name)
            else :
                # Ханшийн зөрүүний алдагдал
                a = company.expense_currency_exchange_account_id.id
                if not a :
                    raise osv.except_osv(_('Warning!'), _('There is no expense currency rate account defined for this company : %s') % company.name)
            
            # Ханшийн зөрүүг журналд бичих
            amove_line_obj.create(cr, uid, {
                'name': u'%s дансны ханшийн тэгшитгэл' % account.name,
                'debit': (diff_amount > 0 and diff_amount) or 0.0,
                'credit': (diff_amount < 0 and -diff_amount) or 0.0,
                'account_id': a,
                'partner_id': False,
                'amount_currency': 0,
                'company_id': company.id,
                'currency_id': False,
                'move_id': move_id,
                'date': date,
                'journal_id': journal_id,
                'period_id': period.id
            }, context=context)
            amove_line_obj.create(cr, uid, {
                'name': u'%s дансны ханшийн тэгшитгэл' % account.name,
                'debit': (diff_amount < 0 and -diff_amount) or 0.0,
                'credit': (diff_amount > 0 and diff_amount) or 0.0,
                'account_id': account.id,
                'partner_id': False,
                'amount_currency': 0,
                'company_id': company.id,
                'currency_id': account.currency_id.id,
                'move_id': move_id,
                'date': date,
                'journal_id': journal_id,
                'period_id': period.id
            }, context=context)
            
        return res
class account_move_line(osv.osv):
    _inherit = 'account.move.line'
    
    _columns = {
                'cash_move_type':fields.many2one('account.cash.move.type', 'Cash Move Type'),
                'noat_number':fields.char('Noat number'),
                }
    
    
    def currency_equalize_on_balance(self, cr, uid, line_id, company_currency_id, context=None):
         ''' Өглөг, авлагын бичилтийн үлдэгдэл дүн дээр ханшийн тэгшитгэл хийнэ.
             @param line: Тухайн өглөг, авлагыг анх бүртгэсэн журналын бичилт
             @param company_currency_id: Тухайн компаний дансаа хөтөлдөг валют
             
             Төлсний дараах үлдэгдэл дээр ханшийн тэгшитгэл хийж хэрэгжээгүй олз, гарзыг бүртгэнэ.
         '''
         if context is None:
             context = {}
         cur_obj = self.pool.get('res.currency')
         account_move_obj = self.pool.get('account.move')
         account_move_line_obj = self.pool.get('account.move.line')
         move_id = False
         
         if context.get('target_move_id', False):
             move_id = context['target_move_id']
             balance_amount, balance_amount_currency, direction = context['target_reconciled_balance']
             line = line_id
         else :
             # Тухайн өглөг, авлагын үлдэгдлийг тодорхойлох
             line = self.browse(cr, uid, line_id, context=context)
             balance_amount_currency = line.amount_residual_currency
             balance_amount = line.amount_residual
         
         # Валютын үлдэгдэлгүй байвал тэгшитгэл хийхгүй.
         #if not balance_amount_currency :
         #    return move_id
         
         # Өнөөдрийн ханшаар авлага, өглөгийн үлдэгдлийг хөрвүүлэх
         date = context.get('p_date', time.strftime('%Y-%m-%d'))
         ctx = context.copy()
         ctx.update({'date':date})
         currency = line.currency_id
         if not currency :
             raise osv.except_osv(_('Integrity Error!'), _('%s[%s] entry line does not created as foreign currency') % (line.name, line.id))
         
         balance_virtual = cur_obj.compute(cr, uid, currency.id, company_currency_id,
                                            balance_amount_currency, context=ctx)
         # Ханшийн зөрүүг тодорхойлох
         diff = balance_amount - balance_virtual # diff > 0 = ханш буурсан
         if abs(diff) > 10 ** -4 : # Ханшийн зөрүү үүссэн гэсэн үг
             if not line.company_id.income_currency_exchange_account_id:
                 raise osv.except_osv(_('Configuration Error!'), _('There is no income currency rate account defined for this company : %s') % line.company_id.name)
             if not line.company_id.expense_currency_exchange_account_id:
                 raise osv.except_osv(_('Configuration Error!'), _('There is no expense currency rate account defined for this company : %s') % line.company_id.name)
             
             period_id = context.get('period_id', False)
             if not period_id:
                 period_id = self.pool.get('account.period').find(cr, uid, dt=date)[0]
             
             if not move_id:
                 move_id = account_move_obj.create(cr, uid, {
                         'journal_id': context.get('journal_id', line.move_id.journal_id.id),
                         'period_id': period_id,
                         'date': date,
                         'narration': u'%s ханш тэгшитгэл' % (line.move_id.ref or '',),
                 }, context=context)
             
             context.update({
                 'period_id': period_id,
                 'journal_id': context.get('journal_id', line.move_id.journal_id.id)
             })
             
             if (diff > 0 and line.debit > 0) or (diff < 0 and line.credit > 0) : # move.debit > 0 == RECEIVABLE || move.credit > 0 == PAYABLE
                 # Ханш өссөн бөгөөд өглөг бууруулж байвал валютын ханшийн өөрчлөлтийн хэрэгжээгүй гарз бүртгэнэ.
                 # Ханш буурсан бөгөөд авлага бууруулж байвал валютын ханшийн өөрчлөлтийн хэрэгжээгүй гарз бүртгэнэ
                 account_id1, account_id2 = line.company_id.expense_currency_exchange_account_id.id, line.account_id.id
                 debit1 = credit2 = abs(diff)
                 credit1 = debit2 = 0.0
                 curr1 = False
                 curr2 = line.currency_id.id
                 name = u'%s - валютын ханшийн зөрүүний хэрэгжээгүй гарз : %s' % (line.ref or u'?', currency.name)
             else :
                 # Ханш өссөн бөгөөд авлага бууруулж байвал валютын ханшийн өөрчлөлтийн хэрэгжээгүй олз бүртгэнэ.
                 # Ханш буурсан бөгөөд өглөг бууруулж байвал валютын ханшийн өөрчлөлтийн хэрэгжээгүй олз бүртгэнэ.
                 account_id1, account_id2 = line.account_id.id, line.company_id.income_currency_exchange_account_id.id
                 debit1 = credit2 = abs(diff)
                 credit1 = debit2 = 0.0
                 curr1 = line.currency_id.id
                 curr2 = False
                 name = u'%s - валютын ханшийн зөрүүний хэрэгжээгүй олз : %s' % (line.ref or u'?', currency.name)
             
             new_line_id1 = self.create(cr, uid, {
                     'name': name,
                     'debit': debit1,
                     'credit': credit1,
                     'account_id': account_id1,
                     'partner_id': line.partner_id.id,
                     'company_id': line.company_id.id,
                     'amount_currency': 0,
                     'currency_id': curr1,
                     'state': 'valid',
                     'move_id': move_id,
                     'date': date,
             }, context=context)
             new_line_id2 = self.create(cr, uid, {
                     'name': name,
                     'debit': debit2,
                     'credit': credit2,
                     'account_id': account_id2,
                     'partner_id': line.partner_id.id,
                     'company_id': line.company_id.id,
                     'amount_currency': 0,
                     'currency_id': curr2,
                     'state': 'valid',
                     'move_id': move_id,
                     'date': date,
             }, context=context)
             
             # Ханш тэгшитгэсэн тухай мэдээллийг журналын бичилтэд нэмнэ.
             if account_id1 == line.account_id.id :
                 self.reconcile_partial(cr, uid, [line.id, new_line_id1], context=context)
             else :
                 self.reconcile_partial(cr, uid, [line.id, new_line_id2], context=context)
             
         return move_id
    
class account_bank_statement_line(osv.osv):
    _inherit = 'account.bank.statement.line'
    
    _columns = {
                'cash_move_type':fields.many2one('account.cash.move.type', 'Cash Move Type'),
                }
    
    
class account_cash_move_type(osv.osv):
     _name = 'account.cash.move.type'
     _description = 'Cash Move Type'
     
     TYPE_SELECTION = [
         ('activities_income', 'Cash flows income from operating activities'),
         ('activities_expense', 'Cash flows expense from operating activities'),
         ('investing_income', 'Cash flows income from investing activities'),
         ('investing_expense', 'Cash flows expense from investing activities'),
         ('financing_income', 'Cash flows income from financing activities'),
         ('financing_expense', 'Cash flows expense from financing activities'),
         ('dummy', 'Non affects in statements')
     ]
     
     _columns = {
         'name': fields.char('Name', size=100, required=True),
         'group': fields.selection(TYPE_SELECTION, 'Group', required=True),
         'sequence': fields.integer('Sequence', help="This sequence is used when generate statement of cash flows report. Sequence is ordered into the groups."),
         'description': fields.text('Description'),
         'company_id': fields.many2one('res.company', 'Company', required=True)
     }
     
     _defaults = {
         'sequence': 1,
         'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'account.cash.move.type', context=c),
     }
     
     

 