# -*- coding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2012 Agile Business Group sagl (<http://www.agilebg.com>)
#    Copyright (C) 2012 Domsense srl (<http://www.domsense.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
import openerp.addons.decimal_precision as dp
from openerp.tools.translate import _
from dateutil.relativedelta import relativedelta
import time
from datetime import timedelta, datetime

class account_integrate(osv.osv):
    _name = 'account.integrate'
    
    _columns = {
                'company_id':fields.many2one('res.company','Company',required=True),
                'date':fields.date('Date'),
                'journal_id':fields.many2one('account.journal','Account journal',required=True),
                'integrate_ids':fields.many2many('account.account','account_integrate_rel','account_id','integrate_id','Account integrate'),
                'account_id':fields.many2one('account.account','Account',required=True),
                'user_id':fields.many2one('res.users','Create user',required=True),
                'state':fields.selection([('draft','Draft'),
                                         ('confirm','Confirm'),
                                         ('cancel','Cancel'),
                                         ],'State')
                }
    
    _defaults = {
                 'date':lambda obj, cr, uid, context: time.strftime('%Y-%m-%d'),
                 'state':'draft',
                 'user_id':lambda self, cr, uid, c:uid,
                 'company_id':lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'account.integrate', context=c),
                 }
    
    def action_confirm(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        account_obj = self.pool.get('account.account')
        period_id = self.pool.get('account.period').find(cr, uid, context=context)[0]
        total_amount = 0.0
        move_id = move_obj.create(cr, uid, {
                                             'name':'/',
                                             'period_id':period_id,
                                             'date':obj.date,
                                             'integrate_id':obj.id,
                                             'journal_id':obj.journal_id.id,
                                             'company_id':obj.company_id.id,
                                             })
        amount = 0.0
        debit = False
        for acc in obj.integrate_ids:
            if acc != []:
                if acc.debit > acc.credit:
                    amount = acc.debit - acc.credit
                    total_amount += acc.debit - acc.credit
                    debit = True
                    move_line_obj.create(cr, uid, {
                                               'name':'/',
                                               'move_id':move_id,
                                               'account_id':acc.id,
                                               'journal_id': obj.journal_id.id,
                                                'period_id': period_id,
                                                'date': obj.date,
                                                'debit':0.0,
                                                'credit':amount,
                                               })
                else:
                    amount = acc.credit - acc.debit
                    total_amount += acc.credit - acc.debit
                    debit = False
                    move_line_obj.create(cr, uid, {
                                                   'name':'/',
                                                   'move_id':move_id,
                                                   'account_id':acc.id,
                                                   'journal_id': obj.journal_id.id,
                                                    'period_id': period_id,
                                                    'date': obj.date,
                                                    'debit':amount,
                                                    'credit':0.0,
                                                   })
                    
        if debit == True:
            move_line_obj.create(cr, uid, {
                                           'name':'/',
                                            'move_id':move_id,
                                            'account_id':obj.account_id.id,
                                            'journal_id': obj.journal_id.id,
                                            'period_id': period_id,
                                            'date': obj.date,
                                            'debit':total_amount,
                                            'credit':0.0,
                                            })
        else:
            move_line_obj.create(cr, uid, {
                                           'name':'/',
                                            'move_id':move_id,
                                            'account_id':obj.account_id.id,
                                            'journal_id': obj.journal_id.id,
                                            'period_id': period_id,
                                            'date': obj.date,
                                            'debit':0.0,
                                            'credit':total_amount
                                            })
        
        self.write(cr, uid, ids, {'state':'confirm'})
        return True
    
    def action_cancel(self, cr, uid, ids,context=None):
        obj = self.browse(cr, uid, ids)
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        move_id = move_obj.search(cr, uid, [('integrate_id','=',obj.id),('state','=','draft')])
        if move_id == []:
            raise osv.except_osv(_('Warning!'),_('Ажил гүйлгээ цуцлагдаагүй байгаа тул цуцална уу!'))
        move_obj.unlink(cr, uid, move_id)
        self.write(cr, uid, ids, { 
                                  'state':'draft'
                                  })
        return True
class account_move(osv.osv):
    _inherit = 'account.move'
    
    _columns = {
                'integrate_id':fields.many2one('account.integrate','Account integrate'),
                }
    