# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

import time, openerp.tools
from lxml import etree

from openerp.osv import fields, osv
from openerp.tools.translate import _

class account_currency_equalization(osv.osv_memory):
    """
        Гадаад валютаар хийгдсэн журналын бичилтүүдэд ханшийн тэгшитгэл хийнэ.
    """
    _name = "account.currency.equalization"
    _description = "Currency Rate Equalization"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company'),
        'type': fields.selection([('liquidity', 'Liquidity Accounts'),('partner', 'Partner Balances')], 'Type', required=True),
        'date': fields.date('Date', required=True),
        'name': fields.char('Reason', size=128, required=True),
        'journal_id': fields.many2one('account.journal', 'Journal', required=True),
        'draft': fields.boolean('Draft Entry', help="Check if this box, equalization entries to be generated draft state. Then you should valid them."),
        
        'partner_id': fields.many2one('res.partner', 'Partner', help="If you select any partner then system will do equalization on only the selected partner balance."),
        'currency_id': fields.many2one('res.currency', 'Currency', help="If you select any currency then system will do equalization on only selected currency.")
    }
    
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d'),
        'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'account.currency.equalization', context=c),
        'draft': False,
        'type': 'liquidity'
    }
    
#    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
#        """
#        Returns views and fields for current model where view will depend on {view_type}.
#        @param cr: A database cursor
#        @param user: ID of the user currently logged in
#        @param view_id: list of fields, which required to read signatures
#        @param view_type: defines a view type. it can be one of (form, tree, graph, calender, gantt, search, mdx)
#        @param context: context arguments, like lang, time zone
#        @param toolbar: contains a list of reports, wizards, and links related to current model
#        
#        @return: Returns a dict that contains definition for fields, views, and toolbars
#        """
#
#        res = super(account_currency_equalization, self).fields_view_get(cr, uid, view_id, view_type, context, toolbar, submenu)
#
#        if not view_id:
#            return res
#        a_ids = self.get_accounts(cr, uid, context=context)
#        if a_ids :
#            domain = "[('id','in',["+ ','.join(map(str, a_ids)) +"])]"
#        else :
#            domain = "[]"
#        doc = etree.XML(res['arch'])
#        nodes = doc.xpath("//field[@name='denied_accounts']")
#        for node in nodes:
#            node.set('domain', domain)
#        res['arch'] = etree.tostring(doc)
#        return res
    
    def get_accounts(self, cr, uid, ids, type=1, context=None):
        ''' Ханшийн тэгшитгэл хийх шаардлагатай дансуудыг тодорхойлно.
            Үүнд : Авлага өглөгийн данс болон гадаад валюттай данс
        '''
        if context is None:
            context = {}
        wizobj = self.browse(cr, uid, ids[0], context=context)
        query = "SELECT a.id FROM account_account a WHERE a.type not in ('view','consolidation') AND company_id = %s " % wizobj.company_id.id
        if type == 1 :
            query += " AND (a.currency_id is not null and a.currency_id <> %s) " % wizobj.company_id.currency_id.id
            if wizobj.currency_id:
                query += " AND a.currency_id = %s " % wizobj.currency_id.id
        elif type == 2 :
            query += " AND (a.type in ('receivable','payable')) "
        
        cr.execute(query)
        res = cr.fetchall()
        return [r[0] for r in res]

    def action_equalize(self, cr, uid, ids, context=None):
        ''' 1. Авлага, өглөгийн дансны бүх гүйлгээг шүүж хамгийн сүүлд тэгшитгэсэн
                ханшаас одоогийн ханшний зөрүүгээр гүйлгээг тэгшитгэнэ. Ингэхдээ
                бүх өглөг, авлага төрөлд хамрагдах дансанд гүйлгээ хийсэн account.move
                үүдийг шүүж олно.
            
            2. Компаний үндсэн валютаас ялгаатай валютаар тохируулагдсан бүх дансуудыг
                олж тайлант хугацаа буюу сүүлд тэгшитгэсэн огнооноос хойшхи 
                    эхний үлдэгдэл,
                    орлого,
                    зарлага,
                    эцсийн үлдэгдэл
                зэргийг тооцоолж эцсийн үлдэгдэл дээр ханшийн тэгшитгэл хийнэ.
        '''
        if context is None:
            context = {}
        account_obj = self.pool.get('account.account')
        account_move_line_obj = self.pool.get('account.move.line')
        account_move_obj = self.pool.get('account.move')
        form = self.browse(cr, uid, ids[0], context=context)
        
        period_id = self.pool.get('account.period').find(cr, uid, dt=form.date, context=context)[0]
        period = self.pool.get('account.period').browse(cr, uid, period_id, context=context)
        
        # Тэгшитгэл хийх шаардлагатай дансууд. Мөн өглөг, авлагын дансуудыг ялгана.
        if form.type == 'liquidity':
            account_ids1 = self.get_accounts(cr, uid, ids, type=1, context=context)
            account_ids2 = []
        else :
            account_ids1 = []
            account_ids2 = self.get_accounts(cr, uid, ids, type=2, context=context)
        
        res = []
        
        # Мөнгөн хөрөнгийн гадаад валютын дансуудын балансыг тэгшитгэнэ.
        if account_ids1 :
            
            res += self.currency_equalize(cr, uid, account_ids1, period, form.date, 
                    form.company_id, form.journal_id.id, form.name, context=context)
        
        # Өглөг авлагын үлдэгдэл бүр дээр тэгшитгэнэ.
        if account_ids2 :
            # Өглөг, авлага бүртгэсэн гүйлгээг тодорхойлохдоо тухайн санхүүгийн жилд бүртгэгдсэнийг
            # тодорхойлно. Учир нь өмнөх жилийн гүйлгээнүүд бүгд хаагдсан байх ёстой.
            # Мөн зөвхөн тухайн компаний өглөг, авлагыг тэгшитгэнэ.
            partner_where = ""
            if form.partner_id:
                partner_where += " AND l.partner_id = %s " % form.partner_id.id
            if form.currency_id:
                partner_where += " AND l.currency_id = %s " % form.currency_id.id
            cr.execute("SELECT l.id FROM account_move_line l "
                       "LEFT JOIN account_move m ON m.id = l.move_id "
                       "LEFT JOIN account_account a ON a.id = l.account_id "
                       "LEFT JOIN account_period p ON p.id = l.period_id "
                       "WHERE l.account_id in %s "
                       "AND l.company_id = %s "
                       "AND p.fiscalyear_id = %s "
                       "AND m.state = 'posted' "
                       "AND a.reconcile = true "
                       "AND l.reconcile_id is null "
                       "AND l.amount_currency <> 0 "
                       + partner_where +
                       "AND l.currency_id <> %s AND l.currency_id is not null "
                       "and ((a.type = 'receivable' and l.debit > 0) or (a.type = 'payable' and l.credit > 0)) "
                       "AND l.date <= %s ",
                    (tuple(account_ids2), form.company_id.id, period.fiscalyear_id.id,
                     form.company_id.currency_id.id, form.date))
            fetched = cr.fetchall()
            context.update({
                'p_date':form.date,
                'journal_id': form.journal_id.id,
                'period_id': period_id
            })
            if fetched:
                for line_id, in fetched :
                    ret = account_move_line_obj.currency_equalize_on_balance(cr, uid, line_id, 
                                            form.company_id.currency_id.id, context=context)
                    if ret :
                        res.append(ret)
        
        # Гүйлгээнүүдийг батлах
        if len(res) > 0 and not form.draft :
            account_move_obj.post(cr, uid, res, context=context)
        
        # Бичигдсэн журналын бичилтүүдийг дэлгэцэнд харуулах
        return {
            'name': _('Equalization Entries'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'account.move',
            'view_id': False,
            'context': "{'visible_id':%(j)s, 'journal_id': %(j)d, 'search_default_journal_id':%(j)d, 'search_default_period_id':%(p)d}" % ({'j':form.journal_id.id, 'p':period_id}),
            'type': 'ir.actions.act_window',
            #'search_view_id': res_id,
            'domain': (len(res) > 0 and "[('id', 'in', ["+ ','.join(map(str, res)) +"])]") or "[('id','=',False)]"
        }
        
    def currency_equalize(self, cr, uid, ids, period, date, company, journal_id, narration, context=None):
        ''' Тухайн дансанд ханшийн тэгшитгэл хийж ханшийн өөрчлөлтөөс үүдэлтэй
            балансын зөрүүг ашиг алдагдлын дансанд бичиж үүссэн ажил гүйлгээнүүдийг
            ID бүхий list буцаана.
        '''
        if context is None:
            context = {}
        cur_obj = self.pool.get('res.currency')
        amove_line_obj = self.pool.get('account.move.line')
        amove_obj = self.pool.get('account.move')
        account_obj = self.pool.get('account.account')
        res = []
        ctx = context.copy()
        ctx['date_from'] = period.fiscalyear_id.date_start
        ctx['date_to'] = date
        ctx['state'] = 'posted'
        ctx['company_id'] = company.id
        ctx['fiscalyear'] = period.fiscalyear_id.id
        
        # Тайлант хугацааны гадаад валютаарх үлдэгдлийг тооцоолох
        cr.execute("SELECT l.account_id, COALESCE(SUM(l.amount_currency), 0) "
                   "FROM account_move_line l "
                   "LEFT JOIN account_account a ON l.account_id = a.id "
                   "LEFT JOIN account_move m ON m.id = l.move_id "
                   "WHERE l.state <> 'draft' and m.state = 'posted' "
                   "AND l.date >= %s and l.date <= %s and l.company_id = %s "
                   "AND l.account_id in %s "
                   "GROUP BY l.account_id ",
            (period.fiscalyear_id.date_start, date, company.id, tuple(ids)))
        amount_currency_dict = dict(cr.fetchall())
        
        for account in account_obj.browse(cr, uid, ids, context=ctx) :
            if not account.currency_id or account.currency_id.id == company.currency_id.id :
                continue
            
            # Тайлант хугацааны төгрөгөөрх үлдэгдэл
            if account.user_type.report_type in ('expense', 'asset') :
                balance = account.debit - account.credit
            else :
                balance = account.credit - account.debit
            
            amount_currency = amount_currency_dict.get(account.id, 0)
            # Валютын дүнгээрх үлдэгдэл
            # Тухайн өдрийн ханшаар валютын үлдэгдлийг үржүүлэхэд дансны үлдэгдэл гарна.
            #  Үүнийг бодит үлдэгдэлтэй харьцуулж ханшийн зөрүү дүнг олно.
            converted_amount = cur_obj.compute(cr, uid, account.currency_id.id, company.currency_id.id,
                        amount_currency, context={'date':date, 'lang':context.get('lang', 'en_US')})
            
            if converted_amount<0:
                if balance >0:
                    balance = balance*(-1)
   
                #converted_amount = converted_amount*(-1)
            else:
                if balance < 0:
                   balance = balance*(-1)
                converted_amount
            diff_amount = converted_amount - balance
            if cur_obj.is_zero(cr, uid, company.currency_id, diff_amount) : 
                # Ханшийн зөрүү үүсээгүй болно.
                continue
                
            move_id = amove_obj.create(cr, uid, {
                        'journal_id': journal_id,
                        'period_id': period.id,
                        'date': date,
                        'narration': narration
            }, context=context)
            res.append(move_id)
            
            if diff_amount > 0 :
                # Ханшийн зөрүүний ашиг
                a = company.income_currency_exchange_account_id.id
                if not a :
                    raise osv.except_osv(_('Warning!'), _('There is no income currency rate account defined for this company : %s') % company.name)
            else :
                # Ханшийн зөрүүний алдагдал
                a = company.expense_currency_exchange_account_id.id
                if not a :
                    raise osv.except_osv(_('Warning!'), _('There is no expense currency rate account defined for this company : %s') % company.name)
            
            # Ханшийн зөрүүг журналд бичих
            amove_line_obj.create(cr, uid, {
                'name': u'%s дансны ханшийн тэгшитгэл' % account.name,
                'debit': (diff_amount < 0 and -diff_amount) or 0.0,
                'credit': (diff_amount > 0 and diff_amount) or 0.0,
                'account_id': a,
                'partner_id': False,
                'amount_currency': 0,
                'company_id': company.id,
                'currency_id': False,
                'move_id': move_id,
                'date': date,
                'journal_id': journal_id,
                'period_id': period.id
            }, context=context)
            amove_line_obj.create(cr, uid, {
                'name': u'%s дансны ханшийн тэгшитгэл' % account.name,
                'debit': (diff_amount > 0 and diff_amount) or 0.0,
                'credit': (diff_amount < 0 and -diff_amount) or 0.0,
                'account_id': account.id,
                'partner_id': False,
                'amount_currency': 0,
                'company_id': company.id,
                'currency_id': account.currency_id.id,
                'move_id': move_id,
                'date': date,
                'journal_id': journal_id,
                'period_id': period.id
            }, context=context)
            
        return res


account_currency_equalization()



#class res_currency(osv.osv):
#    _inherit = 'res.currency'
#    
##    def _get_conversion_rate(self, cr, uid, from_currency, to_currency, context=None):
##        if context is None:
##            context = {}
##        ctx = context.copy()
##        ctx.update({'currency_rate_type_id': ctx.get('currency_rate_type_from')})
##        from_currency = self.browse(cr, uid, from_currency.id, context=ctx)
##        print "--------",to_currency.rate,from_currency.rate
##        ctx.update({'currency_rate_type_id': ctx.get('currency_rate_type_to')})
##        to_currency = self.browse(cr, uid, to_currency.id, context=ctx)
##        if from_currency.rate == 0 or to_currency.rate == 0:
##            date = context.get('date', time.strftime('%Y-%m-%d'))
##            if from_currency.rate == 0:
##                currency_symbol = from_currency.symbol
##            else:
##                currency_symbol = to_currency.symbol
##            raise osv.except_osv(_('Error'), _('No rate found \n' \
##                    'for the currency: %s \n' \
##                    'at the date: %s') % (currency_symbol, date))
##        return from_currency.rate/to_currency.rate
#
#    def compute(self, cr, uid, from_currency_id, to_currency_id, from_amount,
#                round=True, currency_rate_type_from=False, currency_rate_type_to=False, context=None):
#        if not context:
#            context = {}
#        if not from_currency_id:
#            from_currency_id = to_currency_id
#        if not to_currency_id:
#            to_currency_id = from_currency_id
#        xc = self.browse(cr, uid, [from_currency_id,to_currency_id], context=context)
#        from_currency = (xc[0].id == from_currency_id and xc[0]) or xc[1]
#        to_currency = (xc[0].id == to_currency_id and xc[0]) or xc[1]
#        if (to_currency_id == from_currency_id) and (currency_rate_type_from == currency_rate_type_to):
#            if round:
#                return self.round(cr, uid, to_currency, from_amount)
#            else:
#                return from_amount
#        else:
#            context.update({'currency_rate_type_from': currency_rate_type_from, 'currency_rate_type_to': currency_rate_type_to})
#            rate = self._get_conversion_rate(cr, uid, from_currency, to_currency, context=context)
#            print "hhhhhh",rate,from_amount
#            if round:
#                return self.round(cr, uid, to_currency, from_amount * rate)
#            else:
#                return from_amount * rate
#res_currency()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: