# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################

import time
from lxml import etree
from openerp.osv import osv, fields

def roundNumber(num, r):
    return round(num / r) * r

class account_asset_writeoff(osv.osv_memory):
    _name = 'account.asset.writeoff'
    _description = 'Asset Writeoff'

    _columns = {
        'writeoff_account': fields.many2one('account.account', 'Writeoff Account', domain=[('type','not in',('view','consolidation'))], required=True),
        'writeoff_amount': fields.float('Writeoff Amount', readonly=True),
        'depreciation_amount': fields.float('Depreciation Amount', readonly=True),
        'writeoff_journal': fields.many2one('account.journal','Writeoff Journal', required=True),
        'name': fields.char('Reason', size=128, required=True),
        'end_date':fields.date('End date',readonly=True),
        'partner_id':fields.many2one('res.partner','Partner'),
        'partner_journal_id':fields.many2one('account.journal','Partner journal'),
        'partner_expense_id':fields.many2one('account.account','Partner account'),
        'writeoff_date': fields.date('Date')
    }
    
    _defaults = {
        'writeoff_date': lambda *a: time.strftime('%Y-%m-%d')
    }

    def default_get(self, cr, uid, fields, context=None):
        """ To get default values for the object.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param fields: List of fields for which we want default values 
        @param context: A standard dictionary 
        @return: A dictionary which of fields with values. 
        """ 
        if not context:
            context = {}
        asset_obj = self.pool.get('account.asset.asset')
        res = super(account_asset_writeoff, self).default_get(cr, uid, fields, context=context)
        asset_id = context.get('active_id', False)
        asset = asset_obj.browse(cr, uid, asset_id, context=context)
        now_date = time.strftime('%Y-%m-%d')
        war = {}
        if 'end_date' in fields:
            res.update({'end_date':asset.end_date})
        if 'partner_id' in fields:
            res.update({'partner_id':asset.owner_id.id})
        if 'writeoff_amount' in fields:
            res.update({'writeoff_amount':asset.purchase_value})
        if 'depreciation_amount' in fields:
            depreciation_amount = 0
            for depr in asset.depreciation_line_ids:
                if depr.move_id:
                    depreciation_amount += depr.amount
            res.update({'depreciation_amount':depreciation_amount})
        return res
    
    def writeoff(self, cr, uid, ids, context=None):
        """ Modifies the duration of asset for calculating depreciation
        and maintains the history of old values.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: List of Ids 
        @param context: A standard dictionary 
        @return: Close the wizard. 
        """ 
        if not context:
            context = {}
        asset_obj = self.pool.get('account.asset.asset')
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        company_obj = self.pool.get('res.company')
        asset_id = context.get('active_id', False)
        asset = asset_obj.browse(cr, uid, asset_id, context=context)
        data = self.browse(cr, uid, ids[0], context=context)
        # Данснаас хасах бичилт
        move_lines = []
        asset_name_ref = asset.name_get()[0][1]
        writeoff_period = self.pool.get('account.period').find(cr, uid, dt=data.writeoff_date, context=context)[0]
        asset_credit = 0
        if data.writeoff_amount > 0:
            asset_credit += data.writeoff_amount
        depr_debit = 0
        if data.depreciation_amount > 0:
            depr_debit += data.depreciation_amount
        writeoff_debit = asset_credit - depr_debit
        if data.partner_id.id != False:
            if data.partner_id.company_id.id == data.partner_journal_id.company_id.id:
                move_id = move_obj.create(cr, uid,{
                                                   'name':asset_name_ref,
                                                   'journal_id':data.partner_journal_id.id,
                                                   'period_id':writeoff_period,
                                                   'ref': asset.code,
                                                    'date': data.writeoff_date,
                                                    'narration': data.name,
                                                   })
                move_line_obj.create(cr, uid, {
                                               'move_id':move_id,
                                               'name': asset_name_ref,
                                                'debit': 0,
                                                'credit': data.writeoff_amount,
                                                'account_id': data.partner_id.property_account_payable.id,
                                                'date': data.writeoff_date,
                                                'ref': asset.code,
                                                'partner_id': data.partner_id.id,
                                                'period_id': writeoff_period,
                                                'asset_id': asset.id,
                                                'journal_id': data.partner_journal_id.id,
                                               })
                move_line_obj.create(cr, uid, {
                                               'move_id':move_id,
                                                'name': asset_name_ref,
                                                'debit': data.writeoff_amount,
                                                'credit': 0,
                                                'account_id': data.partner_expense_id.id,
                                                'date': data.writeoff_date,
                                                'ref': asset.code,
                                                'partner_id': data.partner_id.id,
                                                'period_id': writeoff_period,
                                                'asset_id': asset.id,
                                                'journal_id': data.partner_journal_id.id,
                                               })
            else:
                raise osv.except_osv(_('Error!'),_("Эзэмшигчийн журнал буруу байна."))
            
        if asset_credit:
            move_lines.append( (0,0,{
                'name': asset_name_ref,
                'debit': 0,
                'credit': asset_credit,
                'account_id': asset.category_id.account_asset_id.id,
                'date': data.writeoff_date,
                'ref': asset.code,
                'partner_id': asset.partner_id.id,
                'period_id': writeoff_period,
                'asset_id': asset.id,
                'journal_id': data.writeoff_journal.id
            }))
        if depr_debit:
            move_lines.append( (0,0,{
                'name': asset_name_ref,
                'debit': depr_debit,
                'credit': 0,
                'account_id': asset.category_id.account_depreciation_id.id,
                'date': data.writeoff_date,
                'ref': asset.code,
                'partner_id': asset.partner_id.id,
                'period_id': writeoff_period,
                'asset_id': asset.id,
                'journal_id': data.writeoff_journal.id
            }))
        if writeoff_debit:
            move_lines.append( (0,0,{
                'name': asset_name_ref,
                'debit': writeoff_debit,
                'credit': 0,
                'account_id': data.writeoff_account.id,
                'date': data.writeoff_date,
                'ref': asset.code,
                'partner_id': asset.partner_id.id,
                'period_id': writeoff_period,
                'asset_id': asset.id,
                'journal_id': data.writeoff_journal.id
            }))
        
        if move_lines:
            move_id = move_obj.create(cr, uid, {
                'journal_id': data.writeoff_journal.id,
                'period_id': writeoff_period,
                'ref': asset.code,
                'date': data.writeoff_date,
                'narration': data.name,
                'line_id': move_lines
            }, context=context)
        
        todo_unlink_board = []
        for line in asset.depreciation_line_ids:
            if not line.move_id:
                todo_unlink_board.append(line.id)
        if todo_unlink_board:
            self.pool.get('account.asset.depreciation.line').unlink(cr, uid, todo_unlink_board, context=context)
        asset_obj.write(cr, uid, [asset.id], {'state':'close'}, context=context)
        return {'type': 'ir.actions.act_window_close'}

account_asset_writeoff()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: