# -*- coding: utf-8 -*-

import logging

from openerp import SUPERUSER_ID
from openerp import tools
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools.translate import _
from openerp.tools import *
from datetime import date, timedelta, datetime
from email import _name
import time
from openerp import netsvc
from lxml import etree
from operator import itemgetter
import openerp.addons.decimal_precision as dp
from openerp.models import FIELDS_TO_PGTYPES
from num2words import num2words

_logger = logging.getLogger(__name__)

class work_performance_contract(osv.Model):
    _name = 'work.performance.contract'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    
    #wpc = Work Performance Contract

    def wpc_report_print(self):
        """ Print the invoice and mark it as sent, so that we can see more
            easily the next step of the workflow
        """
        assert len(self) == 1, 'This option should only be used for a single id at a time.'
        self.sent = True
        return self.env['report'].get_action(self, 'ierp_contract.work_performance_contract')
    
    
    #===========================================================================
    # def _contract_amendment(self, cr, uid, ids, field_name, arg, context=None):
    #     res={}
    #     for tasks in self.browse(cr, uid, ids, dict(context, active_test=False)):
    #         res[tasks.id] = len(tasks.contract_amendment_ids)
    #     return res
    #===========================================================================
    
    
    def _get_tasks(self, cr, uid, ids, context=None):
        tasks = []
        for task in self.browse(cr, uid, ids, context=None):
            tasks += [x.id for x in task.contract_amendment_ids]
        return tasks
    
    def _get_act_window_dict(self, cr, uid, name, context=None):
        mod_obj = self.pool.get('ir.model.data')
        act_obj = self.pool.get('ir.actions.act_window')
        result = mod_obj.xmlid_to_res_id(cr, uid, name, raise_if_not_found=True)
        result = act_obj.read(cr, uid, [result], context=context)[0]
        return result
    
    def _contract_amendment(self, cr, uid, ids, field_name, arg, context=None):
        res={}
        task_bom = self.pool.get('contract.amendment')
        for boms in ids:
            nb = task_bom.search_count(cr, uid, [('wpc_id', '=', boms)], context=context)
        res[boms] = nb
        return res
    
    def action_contract_amendment(self, cr, uid, ids, context=None):
        boms = self._get_tasks(cr, uid, ids, context=context)
        result = self._get_act_window_dict(cr, uid, 'ierp_contract.action_contract_amendment', context=context)

        result['domain'] = "[('id','in',[" + ','.join(map(str, boms)) + "])]"
        return result

    def _intermediate_amount(self, cr, uid, ids, field_name, arg, context=None):
        obj = self.browse(cr, uid, ids)
        res={}
        sum = 0
        if obj.state == 'conclude' and obj.intermediate_funding_id:
            for line in obj.intermediate_funding_id:
                sum = sum + line.amount
        res[obj.id] = sum
        return res
    
    def _get_executive_id(self, cr, uid, context):
        #    Zow ajilladag funkts
        #group_obj = self.pool.get('res.groups').search(cr, uid, [('id','=',391)], context=context)
        #user_real_id = self.pool.get('res.users').search(cr, uid, [('groups_id','=',group_obj[0])], context=context)
        user_real_id = self.pool.get('res.users').search(cr, uid, [('job_type','=','executive_director')], context=context)
        real_id = False
        if len(user_real_id) == 1:
            real_id = user_real_id[0]
        else:
            real_id = False
        return real_id
    
    def _get_building_director_id(self, cr, uid, context):
        user_real_id = self.pool.get('res.users').search(cr, uid, [('job_type','=','building_director')], context=context)
        real_id = False
        if len(user_real_id) == 1:
            real_id = user_real_id[0]
        else:
            real_id = False
        return real_id
    
    def _get_project_headman_id(self, cr, uid, context):
        user_real_id = self.pool.get('res.users').search(cr, uid, [('job_type','=','project_headman')], context=context)
        real_id = False
        if len(user_real_id) == 1:
            real_id = user_real_id[0]
        else:
            real_id = False
        return real_id
    
    _columns = {
        'html_description': fields.html('HTML description', track_visibility='onchange'),#Үндсэн мэдээлэл
        'html_1': fields.html('HTML 1', track_visibility='onchange'),#Үндсэн мэдээлэл 1
        'html_2': fields.html('HTML 2', track_visibility='onchange'),#Үндсэн мэдээлэл 2
        'html_3': fields.html('HTML 3', track_visibility='onchange'),#Үндсэн мэдээлэл 3
        'html_4': fields.html('HTML 4', track_visibility='onchange'),#Үндсэн мэдээлэл 4
        'html_5': fields.html('HTML 5', track_visibility='onchange'),#Үндсэн мэдээлэл 5
        'html_6': fields.html('HTML 6', track_visibility='onchange'),#Үндсэн мэдээлэл 6
        
        'name': fields.text('Name', track_visibility='onchange'),#Гэрээнийн нэр
        'document_received_id': fields.many2one('ir.attachment', 'Document received', track_visibility='onchange', copy=False),#Ирсэн бичиг
        #'type': fields.many2one('type.of.contract','Type', track_visibility='onchange'),#Гэрээний төрөл
        'type': fields.selection([('work_performing','Work performing'),
                                  ('supply_goods','Supply goods'),
                                  ('design','Design'),
                                  ('other','Other'),],'Type', track_visibility='onchange'),#Гэрээний төрөл
        'company_id': fields.many2one('res.company','Company', track_visibility='onchange'),#Номин Констракшн-с гэрээлэгч
        'low_cost_contract': fields.boolean('Low cost contract', track_visibility='onchange', copy=False),#Бага үнийн дүнтэй гэрээ эсэх
        #, default=_get_executive_id
        
        
        'contract_amendment': fields.function(_contract_amendment, type='integer',string="Contract amendment"),
        'contract_amendment_ids':fields.one2many('contract.amendment', 'wpc_id', type='integer',string="Contract amendment"),
        
        'executive_director': fields.many2one('res.users','Executive director', track_visibility='onchange'),#Гүйцэтгэх захирал
        'project_id': fields.many2one('project.project','Project', track_visibility='onchange'),#Төсөл
        'contract_number': fields.char('Contract number', track_visibility='onchange', copy=False),#Гэрээний дугаар
        'contract_description': fields.text('Contract description', track_visibility='onchange'),#Тайлбар
        'contract_date': fields.date('Contract date', track_visibility='onchange'),#Огноо
        'contract_amount': fields.integer('Contract amount', track_visibility='onchange'),#Гэрээний дүн
        'contract_amount_letter': fields.char('Contract amount letter', track_visibility='onchange'),#Гэрээний дүн үсгээр
        'object': fields.many2one('project.building','Object', track_visibility='onchange'),#Обьект
        'task_type': fields.many2one('risk.activity','Task type', track_visibility='onchange'),#Гүйцэтгэх ажлын төрөл
        'adjustment_note': fields.text('Adjustment note', track_visibility='onchange'),#Нэмэлт тайлбар
        #'to_date': fields.date('To date'),#Гүйцэтгэх хугацаа
        'customer_material': fields.integer('Customer material', track_visibility='onchange'),#Захиалагч материалын дүн
        'performer_material': fields.integer('Performer material', track_visibility='onchange'),#Гүйцэтгэгч материалын дүн
        #'performer_material_other': fields.integer('Performer material other', track_visibility='onchange'),#Гүйцэтгэгчийн ажлын хөлс, бусад
        
        'amount_of_labor': fields.integer('Amount of labor', track_visibility='onchange'),#Ажилллах хүчний зардал /НӨАТ - гүй/
        'amount_of_hemab': fields.integer('Budget of hemab', track_visibility='onchange'),#ХЭМАБ-н зардал /НӨАТ - гүй/
        'amount_mech_customer': fields.integer('Amount mech customer', track_visibility='onchange'),#Механизмын захиалагчийн зардал /НӨАТ - тай/
        'amount_mech_performer': fields.integer('Amount mech performer', track_visibility='onchange'),#Механизмын гүйцэтгэгчийн зардал /НӨАТ - гүй/
        
        
        'amount_of_performing': fields.integer('Amount of performing', track_visibility='onchange'),#Гүйцэтгэлээрх дүн
        'amount_of_budget': fields.integer('Budget of performing', track_visibility='onchange'),#Төсвийн дүн
        'is_concluded': fields.boolean('Is concluded', track_visibility='onchange'),#Гэрээ дүгнэсэн эсэх
        'quality': fields.float('Quality', track_visibility='onchange'),#Чанар
        'quality_expiration': fields.date('Quality expiration', track_visibility='onchange'),#Чанарын хугацаа
        'intermediate_funding_id': fields.one2many('intermediate.funding','work_performance_contract_id','Intermediate funding', track_visibility='onchange'),#Дундын санхүүжилт
        #'intermediate_amount': fields.float('Intermediate_amount', track_visibility='onchange'),#Дундын санхүүжилтийн нийт дүн
        'intermediate_amount': fields.function(_intermediate_amount, type='integer',string="Intermediate_amount"),#Дундын санхүүжилтийн нийт дүн
        'user_id': fields.many2one('res.users','User', track_visibility='onchange'),#Хүсэлт гаргаж буй ажилтан
        
        'consumer_job': fields.many2one('hr.job','Consumer job', track_visibility='onchange'),#Захиалагчийн (хүн) албан тушаал
        'consumer_name': fields.many2one('res.users','Consumer name', track_visibility='onchange'),#Захиалагчийн нэр
        'consumer_signature': fields.binary('Consumer signature', track_visibility='onchange'),#Захиалагчийн гарын үсэг
        'consumer_phone': fields.char('Consumer phone', size=8, track_visibility='onchange'),#Захиалагчийн утас
        
        'headman_job': fields.many2one('hr.job','Headman job', track_visibility='onchange'),#Даргын албан тушаал
        'headman_name': fields.many2one('res.users','Headman name', track_visibility='onchange'),#Даргын нэр
        'headman_signature': fields.binary('Headman signature', track_visibility='onchange'),#Даргын гарын үсэг
        'headman_phone': fields.char('Headman phone', size=8, track_visibility='onchange'),#Даргын утас
        
        'coordinator_job': fields.many2one('hr.job','Coordinator job', track_visibility='onchange'),#Төслийн координаторын албан тушаал
        'coordinator_name': fields.many2one('res.users','Coordinator name', track_visibility='onchange'),#Төслийн координаторын нэр
        'coordinator_signature': fields.binary('Coordinator signature', track_visibility='onchange'),#Төслийн координаторын гарын үсэг
        'coordinator_phone': fields.char('Coordinator phone', size=8, track_visibility='onchange'),#Төслийн координаторын утас
        
        'contractor_company': fields.many2one('res.partner','Contractor company', track_visibility='onchange'),#Гэрээлэгч (компани)
        'contractor_address': fields.text('Contractor address', track_visibility='onchange'),#Гэрээлэгч компанийн хаяг
        'contractor_name': fields.many2one('res.partner','Contractor name', track_visibility='onchange'),#Гэрээлэгч (хүн буюу хариуцагч)
        'contractor_job': fields.char('Contractor job', track_visibility='onchange'),#Гэрээлэгч хүний албан тушаал
        'contractor_signature': fields.binary('Contractor signature', track_visibility='onchange'),#Гэрээлэгч хүний албан тушаал
        'contractor_phone': fields.char('Contractor phone', size=8, track_visibility='onchange'),#Гэрээлэгч хүний утас
        'include_agent': fields.boolean('Include agent', track_visibility='onchange'),#Төлөөлөгч сонгох
        'contractor_agent': fields.many2one('res.partner','Contractor agent', track_visibility='onchange'),#Төлөөлөгч ажилтан
        'agent_job': fields.char('Agent job', track_visibility='onchange'),#Төлөөлөгчийн албан тушаал
        'agent_phone': fields.char('Agent phone', size=8, track_visibility='onchange'),#Төлөөлөгчийн утас
        
        'legal_service': fields.char('Legal service', track_visibility='onchange'),#Хуулийн алба
        'conditions_id': fields.one2many('special.conditions.of.contract','work_performance_id','Conditions', track_visibility='onchange', ondelete='cascade' ),
        #, default=_get_conditions
        'import_conditions': fields.boolean('Import conditions', track_visibility='onchange'),#Гэрээний нөхцлийг оруулах
        'print_main_conditions': fields.boolean('Print main conditions', track_visibility='onchange'),#Гэрээний ерөнхий нөхцлийг хэвлэх
        'state': fields.selection([('request','Request'),#Хүсэлт
                                    ('control','Control'),#Хяналт
                                    ('budget','Budget'),#Төсөв
                                    ('coordinator','Coordinator'),#Координатор
                                    ('project','Project'),#Гэрээний төсөл
                                    ('law_headman','Law headman'),#Хуулийн албаны дарга
                                    ('project_headman','Project headman'),#Төслийн албаны дарга
                                    ('building_director','Building director'),#Барилга архитектур хариуцсан захирал
                                    ('executive_director','Executive director'),#Гүйцэтгэх захирал
                                    ('approved','Approved'),#Гэрээний дугаар олгогдсон
                                    ('conclude','Conclude'),#Гэрээ дүгнэх
                                    ('concluded','Concluded'),#Гэрээ дүгнэсэн
                                    ('contract_cancelled','Contract cancelled'),#Гэрээ цуцлагдсан
                                    ('cancelled','Cancelled'),#Гэрээ байгуулагдахааргүй болсон
                                    ],'State', track_visibility='onchange'),#Төлөв
        #BUDGET
        'quantity_surveyor': fields.many2one('res.users','Quantity surveyor', track_visibility='onchange'),#Төсөвчин
        'quantity_surveyor_phone': fields.char('Quantity surveyor phone', size=8, track_visibility='onchange'),#Төсөвчний утас
        'budget_date': fields.date('Budget date', track_visibility='onchange'),#Төсөв зоосон огноо
        'is_budget_completed': fields.boolean('Budget completed', track_visibility='onchange'),#Төсөв гаргасан эсэх
        'budget_attachment':fields.many2many('ir.attachment','wpc_ba_rel', 'ba_id', 'wpc_ba_id','Budget attachment', track_visibility='onchange'),#Төсвийн хавсралт
        'budget_note': fields.text('Budget note', track_visibility='onchange'),#Төсөвчний тайлбар
        
        #COORDINATOR
        'start_date_co': fields.date('Start date', track_visibility='onchange'),#Ажлыг эхлэх огноо
        'end_date_co': fields.date('End date', track_visibility='onchange'),#Ажлыг дуусгах огноо
        'quality_emp_number': fields.integer('Quality emps number', track_visibility='onchange'),#Чанарын хяналт хариуцсан ажилтны тоо
        'quality_emps':fields.text('Quality emps', track_visibility='onchange'),#Чанарын хяналт хариуцсан ажилчид
        'must_ita_number': fields.integer('Must ITA number', track_visibility='onchange'),#Ажиллавал зохих ИТА ажилтны тоо
        'must_itas':fields.text('Must ITAs', track_visibility='onchange'),#Ажиллавал зохих ИТА ажилчид
        'hemab_number': fields.integer('HEMAB number', track_visibility='onchange'),#ХЭМАБ ажилтны тоо
        'hemab_emps':fields.text('HEMAB emps', track_visibility='onchange'),#ХЭМАБ ажилчид
        'professional_emps_number': fields.integer('Professional emps number', track_visibility='onchange'),#Мэргэжлийн ажилтны тоо
        'professional_emps':fields.text('Professional emps', track_visibility='onchange'),#Мэргэжлийн ажилчид
        'equipment_emps_number': fields.integer('Equipment emps number', track_visibility='onchange'),#Тоног төхөөрөмж хариуцсан ажилтны тоо
        'equipment_emps':fields.text('Equipment emps', track_visibility='onchange'),#Тоног төхөөрөмж хариуцсан ажилчид
        #PROJECT
        'ready_project_of_contract':fields.boolean('Ready project of contract', track_visibility='onchange'),#Гэрээний төсөл бэлэн болсон
        'project_date':fields.date('Project date', track_visibility='onchange'),#Гэрээний төсөл боловсруулагдсан огноо
        'project_note':fields.text('Project note', track_visibility='onchange'),#Гэрээний мэргэжилтний тайлбар
        #LAW HEADMAN
        'law_headman': fields.many2one('res.users','Law headman', track_visibility='onchange', copy=False),#Хуулийн албаны дарга
        'law_date':fields.date('Law date', track_visibility='onchange'),#Хуулийн албаны даргын баталсан огноо
        'law_note':fields.text('Law note', track_visibility='onchange'),#Хуулийн албаны даргын тайлбар
        #PROJECT HEADMAN
        'project_headman_date':fields.date('Project headman date', track_visibility='onchange'),#Төслийн албаны даргын баталсан огноо
        'project_headman_note':fields.text('Project headman note', track_visibility='onchange'),#Төслийн албаны даргын тайлбар
        #BUILDING DIRECTOR
        'building_director_date':fields.date('Building director date', track_visibility='onchange'),#Барилга, архитектур хариуцсан захирлын баталсан огноо
        'building_director_note':fields.text('Building director note', track_visibility='onchange'),#Барилга, архитектур хариуцсан захирлын тайлбар
        #EXECUTIVE DIRECTOR
        'executive_director_date':fields.date('Executive director date', track_visibility='onchange'),#Гүйцэтгэх захирлын баталсан огноо
        'executive_director_note':fields.text('Executive director note', track_visibility='onchange'),#Гүйцэтгэх захирлын тайлбар
        #EXECUTIVE DIRECTOR
        'economist': fields.many2one('res.users','Economist', track_visibility='onchange'),#Эдийн засагч
        #CONTRACT CANCELLED
        'contract_cancelled_date':fields.date('Contract cancelled date', track_visibility='onchange'),#Гэрээ цуцалсан огноо
        'contract_cancelled_reason':fields.text('Contract cancelled reason', track_visibility='onchange'),#Гэрээ цуцлах шалтгаан
        #CANCELLED
        'cancelled_date':fields.date('Cancelled date', track_visibility='onchange'),#Гэрээ байгуулахгүй болсон огноо
        'cancelled_reason':fields.text('Cancelled reason', track_visibility='onchange'),#Гэрээ байгуулахгүй болсон шалтгаан
        
        #MAIL INFO
        'user_mail_created': fields.boolean('User mail created', track_visibility='onchange'),#Хүсэлт гаргагчид мэйл илгээсэн
        'controlled_mail_created': fields.boolean('Controlled mail created', track_visibility='onchange'),#Хянасан мэйл илгээсэн /Төсөвчинд мэйл илгээсэн/
        'budget_mail_created': fields.boolean('Budget mail created', track_visibility='onchange'),#Төсвийн мэйл илгээсэн
        'coordinator_mail_created': fields.boolean('Coordinator mail created', track_visibility='onchange'),#Координатор мэйл илгээсэн
        'contract_project_mail_created': fields.boolean('Contract project mail created', track_visibility='onchange'),#Гэрээний төсөл мэйл илгээсэн
        'headman_mail_created': fields.boolean('Headman mail created', track_visibility='onchange'),#Хуулийн албаны дарга мэйл илгээсэн
        'project_headman_mail_created': fields.boolean('Project headman mail created', track_visibility='onchange'),#Төслийн албаны дарга мэйл илгээсэн
        'building_director_mail_created': fields.boolean('Building director mail created', track_visibility='onchange'),#Барилга хариуцсан захирал мэйл илгээсэн
        'executive_director_mail_created': fields.boolean('Executive director mail created', track_visibility='onchange'),#Гүйцэтгэх захирал мэйл илгээсэн
        'approved_mail_created': fields.boolean('Approved mail created', track_visibility='onchange'),#Баталсан мэйл илгээсэн
        'law_dep_mail_created': fields.boolean('Law dep mail created', track_visibility='onchange'),#Хуулийн албанд мэйл илгээсэн
        'to_conclude_mail_created': fields.boolean('To conclude  mail created', track_visibility='onchange'),#Гэрээг дүгнэх мэйл илгээсэн
        'concluded_mail_created': fields.boolean('Concluded mail created', track_visibility='onchange'),#Гэрээ дүгнэсэн мэйл илгээсэн
        
        }
    
  #=============================================================================
  #   _sql_constraints = [
  #   ('document_received_id', 'unique("document_received_id")', 'Field document_received_id must be unique.'),
  # ]
  #=============================================================================
    _defaults = {
                'executive_director': _get_executive_id,
                'consumer_name': _get_building_director_id,
                'headman_name': _get_project_headman_id,
                'low_cost_contract' : False,
                #'document_received_id' : False,
                'import_conditions' : False,
                'include_agent' : False,
                'print_main_conditions' : False,
                'user_id':lambda self, cr, uid, context: uid,
                'contract_date' : date.today(),
                'state' : 'request',
                'is_budget_completed' : False,
                'ready_project_of_contract' : False,
                'is_concluded' : False,
                #mail ilgeesen medeelliig ilerhileh talbaruud
                'user_mail_created' : True,
                'controlled_mail_created' : True,
                'budget_mail_created' : True,
                'coordinator_mail_created' : True,
                'contract_project_mail_created' : True,
                'headman_mail_created' : True,
                'project_headman_mail_created' : True,
                'building_director_mail_created' : True,
                'executive_director_mail_created' : True,
                'approved_mail_created' : True,
                #'law_dep_mail_created' : True,
                'to_conclude_mail_created' : True,
                'concluded_mail_created' : True,
                
                #'html_description' : """&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Нэг талаас <b>“Номин Констракшн” ХХК</b> /цаашид “Захиалагч” гэх/, нөгөө талаас <b>“” ХХК</b> /цаашид “Гүйцэтгэгч” гэх/ нар / цаашид хамтад нь Талууд гэх/ дараах зүйлээр харилцан тохиролцож энэхүү Барилга угсралтын ажил гүйцэтгэх гэрээ/ цаашид “Гэрээ” гэх/-г байгуулав.""",
                
                #'html_1': """1. Энэхүү гэрээгээр <b>..................... төслийн .....................-р барилгын ...............................</b> ажлыг Иргэний, Барилгын болон бусад хууль тогтоомж, БНбД, улсын болон компанийн стандартын дагуу хийж гүйцэтгэхтэй холбогдон Захиалагч болон Гүйцэтгэгчийн эрх үүргийг тодорхойлж тэдний хооронд үүсэх харилцааг зохицуулна.""",
                'html_2': """2. Гүйцэтгэгч нь Гэрээ, гэрээний Ерөнхий болон Тусгай нөхцөл, бусад хавсралтуудын заалтуудын шаардлагыг хангаж Ажлын зураг, Төсөвт дурдсан ажлыг Ажлын зааврын дагуу графикт хугацаанд нь чанартай гүйцэтгэн Захиалагчид актаар хүлээлгэн өгөх ба ийнхүү гүйцэтгэсэн ажилтай холбогдох аливаа зөрчил, ажлын үр дүнгийн доголдлыг арилгах үүрэг, хариуцлагыг хүлээнэ.""",
                'html_3': """3. Гүйцэтгэгч нь энэхүү гэрээний Ерөнхий нөхцөлтэй танилцсан ба түүнийг зөвшөөрснөө үүгээр баталгаажуулж байна.""",
                #===============================================================
                # 'html_4': """4. Захиалагч нь Гэрээ, гэрээний Ерөнхий болон Тусгай нөхцөлийн заалтуудын шаардлагыг
                #              хангаж Гүйцэтгэгчийн ажлын үр дүнг шалган актаар хүлээн авч захиалагчаас нийлүүлэх 
                #              бараа, материал <b>...................</b> төгрөг, гүйцэтгэгчийн материал, ажлын хөлс болон 
                #              бусад зардал <b>...................</b> төгрөгийг багтаасан хавсралт төсөвт заасан нийт 
                #              <b>................ /үсгээр/</b> төгрөгийн тооцоог түүнтэй хийх үүрэг, хариуцлагыг 
                #              хүлээнэ. Энэ дүн нь Гэрээний үнэ болно.""",
                # 
                #===============================================================
                'html_5': """5. Гэрээ, түүний Ерөнхий болон Тусгай нөхцөл, Ажлын зураг, Төсөв, Ажлын заавар, ХЭМАБ-ын шаардлагыг хангах арга хэмжээний төлөвлөгөө нь энэхүү гэрээний хавсралт, салшгүй хэсэг байна.""",
                'html_6': """6. Талууд энэхүү гэрээгээр шууд зохицуулагдаагүй гэрээний үүрэгт хамаарах бусад асуудлыг Монгол улсын холбогдох хууль тогтоомжийн дагуу зохицуулна.""",
        }
    
    def verbose_format(self,cr, uid, ids, contract_amount, context=None):
        '''Мөнгөн дүнг үсгээр илэрхийлэх функц
        '''
        if type(contract_amount) not in (str, unicode) :
            contract_amount = str(contract_amount)
        result = u''
        BUTARHAI = True
        i = 0
        stramount = contract_amount.replace(',','.')
        
        if '-' in stramount:
            stramount= stramount.replace('-','')        
        if '.' in contract_amount:
            contract_amount = stramount[:stramount.find('.')]
            subamount = stramount[stramount.find('.')+1:]
        else :
            contract_amount = stramount
            subamount = u''
        length = len(contract_amount)
        if length == 0 or float(contract_amount) == 0:
            return ''
        place = 0
        try :
            while i < length :
                c = length - i
                if c % 3 == 0 :
                    c -= 3
                else :
                    while c % 3 != 0 :
                        c -= 1
                place = c / 3
                i1 = length - c
                tmp = contract_amount[i:i1]
                j = 0
                if tmp == '000' :
                    i = i1
                    continue
                while j < len(tmp) :
                    char = int(tmp[j])
                    p = len(tmp) - j
                    if char == 1 :
                        if p == 3 :
                            result += u'нэг зуун '
                        elif p == 2 :
                            result += u'арван '
                        elif p == 1 :
                            if len(result)==0:
                                result += u'нэг '
                            else:
                                result += u'нэгэн '
                    elif char == 2 :
                        if p == 3 :
                            result += u'хоёр зуун '
                        elif p == 2 :
                            result += u'хорин '
                        elif p == 1 :
                            result += u'хоёр '
                    elif char == 3 :
                        if p == 3 :
                            result += u'гурван зуун '
                        elif p == 2 :
                            result += u'гучин '
                        elif p == 1 :
                            result += u'гурван '
                    elif char == 4 :
                        if p == 3 :
                            result += u'дөрвөн зуун '
                        elif p == 2 :
                            result += u'дөчин '
                        elif p == 1 :
                            result += u'дөрвөн '
                    elif char == 5 :
                        if p == 3 :
                            result += u'таван зуун '
                        elif p == 2 :
                            result += u'тавин '
                        elif p == 1 :
                            result += u'таван '
                    elif char == 6 :
                        if p == 3 :
                            result += u'зургаан зуун ' 
                        elif p == 2 :
                            result += u'жаран '
                        elif p == 1 :
                            result += u'зургаан '
                    elif char == 7 :
                        if p == 3 :
                            result += u'долоон зуун '
                        elif p == 2 :
                            result += u'далан '
                        elif p == 1 :
                            result += u'долоон '
                    elif char == 8 :
                        if p == 3 :
                            result += u'найман зуун '
                        elif p == 2 :
                            result += u'наян '
                        elif p == 1 :
                            result += u'найман '
                    elif char == 9 :
                        if p == 3 :
                            result += u'есөн зуун '
                        elif p == 2 :
                            result += u'ерэн '
                        elif p == 1 :
                            result += u'есөн '
                    
                    j += 1
                # -------- end while j < len(tmp)
                if place == 3 :
                    result += u'тэрбум '
                elif place == 2 :
                    result += u'сая '
                elif place == 1 :
                    result += u'мянга '
                i = i1
            # ---------- end while i < len(amount)
        except Exception, e :
            return e
        
        if len(subamount) > 0 and float(subamount) > 0 :
            result2 = verbose_format(subamount)
            BUTARHAI = False
            result2 = result2.replace(u'төгрөг', u'мөнгө')
            result += u' төгрөг %s' % result2
            #result += u' %s' % result2
        if BUTARHAI:
            result += u' төгрөг'
            #result += u' '
    
        num = result
        #if u"мянга  төгрөг" in num:
        if u"тэрбум  төгрөг" in num:
            result = num.replace(u"тэрбум  төгрөг",u"тэрбум")
        elif u"сая  төгрөг" in num:
            result = num.replace(u"сая  төгрөг",u"сая")
        elif u"мянга  төгрөг" in num:
            result = num.replace(u"мянга  төгрөг",u"мянга")
        elif u"зуун  төгрөг" in num:
            result = num.replace(u"зуун  төгрөг",u"зуу")
        elif u"ерэн  төгрөг" in num:
            result = num.replace(u"ерэн  төгрөг",u"ер")
        elif u"наян  төгрөг" in num:
            result = num.replace(u"наян  төгрөг",u"ная")
        elif u"далан  төгрөг" in num:
            result = num.replace(u"далан  төгрөг",u"дал")
        elif u"жаран  төгрөг" in num:
            result = num.replace(u"жаран  төгрөг",u"жар")
        elif u"тавин  төгрөг" in num:
            result = num.replace(u"тавин  төгрөг",u"тавь")
        elif u"дөчин  төгрөг" in num:
            result = num.replace(u"дөчин  төгрөг",u"дөч")
        elif u"гучин  төгрөг" in num:
            result = num.replace(u"гучин  төгрөг",u"гуч")
        elif u"хорин  төгрөг" in num:
            result = num.replace(u"хорин  төгрөг",u"хорь")
        elif u"арван  төгрөг" in num:
            result = num.replace(u"арван  төгрөг",u"арав")
        elif u"есөн  төгрөг" in num:
            result = num.replace(u"есөн  төгрөг",u"ес")
        elif u"найман  төгрөг" in num:
            result = num.replace(u"найман  төгрөг",u"найм")
        elif u"долоон  төгрөг" in num:
            result = num.replace(u"долоон  төгрөг",u"долоо")
        elif u"зургаан  төгрөг" in num:
            result = num.replace(u"зургаан  төгрөг",u"зургаа")
        elif u"таван  төгрөг" in num:
            result = num.replace(u"таван  төгрөг",u"тав")
        elif u"дөрвөн  төгрөг" in num:
            result = num.replace(u"дөрвөн  төгрөг",u"дөрөв")
        elif u"гурван  төгрөг" in num:
            result = num.replace(u"гурван  төгрөг",u"гурав")
        elif u"хоёр  төгрөг" in num:
            result = num.replace(u"хоёр  төгрөг",u"хоёр")
        elif u"нэгэн  төгрөг" in num:
            result = num.replace(u"нэгэн  төгрөг",u"нэг")
        else:
            result = num
        #result = num
        return result
    
    def tbab_number2words(self,cr, uid, ids, contract_amount, context=None):
        number_text = num2words(contract_amount)
        oron = ''
        print 'aaaaaaaa', number_text
        return number_text
    
    def create(self, cr, uid, vals, context=None):
        res ={}
        print 'ddddddddddddddddd'
        context = dict(context or {})
        if context.get("mail_broadcast"):
            context['mail_create_nolog'] = True
            #Getting 'Employee code'
        if vals['type'] != False:
            i = ''
            create_year = 0
            this_year_by_2 = datetime.today().strftime('%YYYY-%mm-%dd')[2:4]
            filter = ''
            if vals['type'] == 'work_performing':
                comp_obj = self.pool.get('res.company')
                comp_id = comp_obj.search(cr, uid, [('id','=',vals['company_id'])])
                comp_cipher = comp_obj.browse(cr, uid, comp_id)
                prefix = comp_cipher.contract_cipher
                filter = str(prefix) + '/' + this_year_by_2 + '/'
            elif vals['type'] == 'supply_goods':
                prefix = 'БН'
                filter = str(prefix) + '/' + this_year_by_2 + '/'
            elif vals['type'] == 'design':
                prefix = 'ЗТ'
                filter = str(prefix) + '/' + this_year_by_2 + '/'
            elif vals['type'] == 'other':
                prefix = 'БГ'
                filter = str(prefix) + '/' + this_year_by_2 + '/'
            
            filter2 = '/' + this_year_by_2 + '/'
            #=======================================================================
            # cr.execute("select contract_number from hr_contract where contract_number like '"+vall+"%' order by contract_number desc limit 1")
            # cont_code  = cr.dictfetchall()
            #=======================================================================
            
            #===================================================================
            # cr.execute("select contract_number from work_performance_contract where contract_number like '%"+filter2+"%' order by contract_number")
            # contract_number_info2  = cr.dictfetchall()
            # print 'ssbsbsbs', contract_number_info2
            # for lin34 in contract_number_info2:
            #     list.append(line['contract_number'][4:])
            #     print 'yyyyyrrrrrfrfrfr', list
            #     for list2 in lisst:
            #         
            #===================================================================
            cr.execute("select contract_number from work_performance_contract where contract_number like '%"+filter2+"%' order by substr(contract_number,length(contract_number)-3,4) desc limit 1")
            contract_number_info  = cr.dictfetchall()
            print 'uuuuuuu', contract_number_info
            #code = ''
            measure = 0
            #if measure != 0:
            if contract_number_info:
                for line in contract_number_info:
                    empii = line['contract_number']
                    if empii == None:
                        print 'yyyyyyyyyy1', 
                        i = 1
                    else:
                        print 'yyyyyyyyyy2', 
                        measure = len(line['contract_number'][4:])
                        print 'bbbbbbbbbbb', measure
                        i = int(empii[measure:]) + 1
            else:
                print 'yyyyyyyyyy3', 
                i = 1
            code = u'%s'%filter+u'%s'%str(i).zfill(4)
        else:
            code = ''
        
        
        #print 'ddddddddd',code, ssscontract_number_info
        #vals.update({'contract_number':code})
        vals['contract_number'] = code
        wpc_id = super(work_performance_contract, self).create(cr, uid, vals, context=context)
        
        #self.write(cr, uid, wpc_id, {'contract_number': code})

        return wpc_id
        
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        
        #Гэрээний Менежер
        if_group_erp_manager = self.pool['res.users'].has_group(cr, uid, 'base.group_erp_manager')
        if not if_group_erp_manager:
            raise osv.except_osv(_("Буруу үйлдэл!"), _("Гэрээг устгах боломжгүй!!! Гэрээ байгуулагдахааргүй болсон эсвэл Гэрээ цуцлагдсан төлөвт шилжиж болно!"))
        #=======================================================================
        # line_obj = self.pool.get('special.conditions.of.contract')
        # for line in obj.conditions_id:
        #     line_obj.unlink(cr, uid, line.id)
        #=======================================================================
        return super(work_performance_contract, self).unlink(cr, uid, ids, context=context)
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)
        
        #Гэрээний Менежер
        if_group_contract_manager = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_manager')
        #Гэрээний Төсөвчин
        if_group_contract_quantity_surveyor = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_quantity_surveyor')
        #Гэрээний Координатор
        if_group_contract_coordinator = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_coordinator')
        #Гэрээний Мэргэжилтэн буюу Менежерийн доод талын эрх
        if_group_contract_officer = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_officer')
        #Гэрээний Албаны дарга
        if_group_contract_headman = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_headman')
        #Гэрээний Төслийн албаны дарга
        if_group_contract_project_headman = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_project_headman')
        #Гэрээний Барилга архитектур хариуцсан захирал
        if_group_contract_building_director = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_building_director')
        #Гэрээний Гүйцэтгэх захирал
        if_group_contract_executive_director = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_executive_director')
        #Гэрээний Эдийн засагч
        if_group_contract_economist = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_contract_economist')
        
        if obj.state in ('contract_cancelled','cancelled','approved'):
            if not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Менежер эрхтэй ажилтан энэхүү төлөвтэй гэрээг засч болно!"))
        if obj.state in ('request', 'control', 'project'):
            if not if_group_contract_officer:
                raise osv.except_osv(_("Эрхийн алдаа!"),
                                     _("Зөвхөн Гэрээний Мэргэжилтэн эрхтэй ажилтан энэхүү төлөвтэй гэрээг засч болно!"))
        if obj.state in ('budget','coordinator'):
            if not if_group_contract_quantity_surveyor and not if_group_contract_manager:
                print 'aaaaaaa', if_group_contract_coordinator
                if if_group_contract_coordinator:
                    print 'ddddddddd', if_group_contract_coordinator
                    vals['is_budget_completed'] = obj.is_budget_completed
                    vals['budget_date'] = obj.budget_date
                    vals['amount_of_performing'] = obj.amount_of_performing
                    vals['amount_of_budget'] = obj.amount_of_budget
                    vals['amount_of_labor'] = obj.amount_of_labor
                    vals['amount_of_hemab'] = obj.amount_of_hemab
                    vals['amount_mech_customer'] = obj.amount_mech_customer
                    vals['amount_mech_performer'] = obj.amount_mech_performer
                    vals['budget_note'] = obj.budget_note
                    vals['budget_attachment'] = obj.budget_attachment
                else:
                    raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Төсөвчин эсвэл Гэрээний Менежер эрхтэй ажилтан Төсөв хэсгийн мэдээллийг өөрчилж болно!"))
        #if obj.state == 'coordinator':
            if not if_group_contract_coordinator and not if_group_contract_manager:
                if if_group_contract_quantity_surveyor:
                    print 'ccccccccc', if_group_contract_quantity_surveyor
                    vals['start_date_co'] = obj.start_date_co
                    vals['end_date_co'] = obj.end_date_co
                    vals['quality_emp_number'] = obj.quality_emp_number
                    vals['quality_emps'] = obj.quality_emps
                    vals['must_ita_number'] = obj.must_ita_number
                    vals['must_itas'] = obj.must_itas
                    vals['hemab_number'] = obj.hemab_number
                    vals['hemab_emps'] = obj.hemab_emps
                    vals['professional_emps_number'] = obj.professional_emps_number
                    vals['professional_emps'] = obj.professional_emps
                    vals['equipment_emps_number'] = obj.equipment_emps_number
                    vals['equipment_emps'] = obj.equipment_emps
                else:
                    raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Координатор эсвэл Гэрээний Менежер эрхтэй ажилтан Координатор хэсгийн мэдээллийг өөрчилж болно!"))
        if obj.state == 'project':
            if not if_group_contract_officer and not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Мэргэжилтэн эсвэл Гэрээний Менежер эрхтэй ажилтан Гэрээний төсөл төлөвтэй гэрээг засч болно!"))
        if obj.state == 'law_headman':
            if not if_group_contract_headman and not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Албаны дарга эсвэл Гэрээний менежер эрхтэй ажилтан Хуулийн албаны дарга төлөвтэй гэрээг засч болно!"))
        if obj.state == 'project_headman':
            if not if_group_contract_project_headman and not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Төслийн албаны дарга эсвэл Гэрээний Менежер эрхтэй ажилтан Төслийн албаны дарга төлөвтэй гэрээг засч болно!"))
        if obj.state == 'building_director':
            if not if_group_contract_building_director and not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Барилга архитектур хариуцсан захирал эсвэл Гэрээний Менежер эрхтэй ажилтан Барилга архитектур хариуцсан захирал төлөвтэй гэрээг засч болно!"))
        if obj.state == 'executive_director':
            if not if_group_contract_executive_director and not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Гүйцэтгэх захирал эсвэл Гэрээний Менежер эрхтэй ажилтан Гүйцэтгэх захирал төлөвтэй гэрээг засч болно!"))
        if obj.state == 'conclude':
            if not if_group_contract_economist and not if_group_contract_manager:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний Эдийн засагч эсвэл Гэрээний Менежер эрхтэй ажилтан Гэрээ дүгнэх төлөвтэй гэрээг засч болно!"))
        if 'conditions_id' in vals:
            if not if_group_contract_officer:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний мэргэжилтнээс дээш эхртэй ажилтан Гэрээний тусгай нөхцлийг засч болно!"))
        if 'print_main_conditions' in vals:
            if not if_group_contract_officer:
                raise osv.except_osv(_("Эрхийн алдаа!"), _("Зөвхөн Гэрээний мэргэжилтнээс дээш эхртэй ажилтан Гэрээний ерөнхий нөхцлийг хэвлэх талбарыг засч болно!"))
        if 'type' in vals:
            raise osv.except_osv(_("Буруу үйлдэл!"), _("Гэрээний төрлийг засч болохгүй!"))
        
            
        #=======================================================================
        # if 'conditions_id' in vals:
        #     for record in self.browse(cr, uid, ids, context=context):
        #         print 'bbbbbbbbb', record
        #         numberi = 0
        #         for line in record.conditions_id:
        #             print 'hjjjjjjjjjj', line
        #             numberi = numberi + 1
        #             line.write({'sequence': numberi})
        #=======================================================================
        
        return super(work_performance_contract, self).write(cr, uid, ids, vals, context=context)
    
    
    def renumber_line(self, cr, uid, ids, *args):
        for record in self.browse(cr, uid, ids):
            numberi = 0
            for line in record.conditions_id:
                numberi = numberi + 1
                line.write({'sequence': numberi})
        return True


    def update_conditions(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        for record in self.browse(cr, uid, ids):
            numberi = 0
            for line in record.conditions_id:
                if line.type == 'coordinator_info':
                    text1 = 'ХУД, 11-р хороо, Их Монгол Улсын гудамж, Ривер гарден-2 хотхон 406-р барилга 2-р давхар '
                    text2 = str(obj.coordinator_job.name) + ': ' + str(obj.coordinator_name.name) + ' '
                    text3 = str('Утас:') + ' ' + str(obj.coordinator_phone)
                    value = text1 + text2 + text3
                    line.write({'values': value})
                elif line.type == 'contractor_address':
                    value = obj.contractor_address
                    line.write({'values': value})
                elif line.type == 'project_name':
                    value = str(obj.name) + ' ажил'
                    line.write({'values': value})
                elif line.type == 'coordinator_filled_numbers':
                    text1 = 'Чанарын хяналт хариуцсан ажилтан ' + str(obj.quality_emp_number) + ', '
                    text2 = 'ажиллавал зохих ИТА ' + str(obj.must_ita_number) + ', '
                    text3 = 'ХЭМАБ-ын ажилтан ' + str(obj.hemab_number) + ', '
                    text4 = 'мэргэжлийн ажилчин ' + str(obj.professional_emps_number) + ', '
                    text5 = 'тоног төхөөрөмж, багаж хэрэгсэл ' + str(obj.equipment_emps_number) + ' - ажилтны тоогоор'
                    value = text1 + text2 + text3 + text4 + text5
                    line.write({'values': value})
                elif line.type == 'coordinator_filled_dates':
                    format = '%Y-%m-%d'
                    text1 = ''
                    text2 = ''
                    text3 = ''
                    if obj.start_date_co:
                        text1 = 'Эхлэх: ' + str((datetime.strptime(str(obj.start_date_co), format)).date()) + ', '
                    else:
                        text1 = 'Эхлэх: 20.. / .. / ..' + ', '
                    if obj.end_date_co:
                        text2 = 'Дуусах: ' + str((datetime.strptime(str(obj.end_date_co), format)).date()) + ', '
                    else:
                        text2 = 'Дуусах: 20.. / .. / ..' + ', '
                    if obj.start_date_co and obj.end_date_co:
                        diff = (datetime.strptime(str(obj.end_date_co), format)).date() - (datetime.strptime(str(obj.start_date_co), format)).date()
                        text3 = 'Хоног: ' + str(diff.days)
                    else:
                        text3 = 'Хоног: ...'
                    value = text1 + text2 + text3
                    line.write({'values': value})
                elif line.type == 'material_amounts':
                    text1 = 'Захиалагчаас нийлүүлэх материал ' + str(obj.customer_material) + ' төгрөг, '
                    text2 = 'Гүйцэтгэгчээс нийлүүлэх материал ' + str(obj.performer_material) + ' төгрөг'
                    value = text1 + text2
                    line.write({'values': value})
                elif line.type == 'contract_amount':
                    text1 = obj.verbose_format(obj.contract_amount)
                    value = 'НӨАТ багтсан ' + str(obj.contract_amount) + ' / ' + str(text1) + ' / төгрөг байна.'
                    line.write({'values': value})
                elif line.type == 'hemab_amount':
                    value = 'ХЭМАБ-ын зардал нь ' + str(obj.amount_of_hemab) + ' төгрөг байна. Энэ зардлыг ХЭМАБ-ын шаардлагыг хангах арга хэмжээний төлөвлөгөөнд тусгаж Гүйцэтгэгч зориулалтын дагуу зарцуулах ба Захиалагч хянана.'
                    line.write({'values': value})
                elif line.type == 'performer_amounts_2':
                    text1 = int((obj.performer_material + obj.amount_of_labor + obj.amount_of_hemab + obj.amount_mech_performer) * 0.02)
                    value = 'Гүйцэтгэгчийн ажлын хөлс болон бусад зардлын үнийн дүнгийн 2 хувь буюу ' + str(text1) + ' төгрөг.'
                    line.write({'values': value})
                elif line.type == 'performer_amounts_5':
                    text1 = int((obj.performer_material + obj.amount_of_labor + obj.amount_of_hemab + obj.amount_mech_performer) * 0.05)
                    value = 'Гүйцэтгэгчийн ажлын хөлс болон бусад зардлын үнийн дүнгийн 5 хувь буюу ' + str(text1) + ' төгрөг, Барилга ашиглалтад орсноос хойш 48 сар'
                    line.write({'values': value})
        return True
    
    def to_control(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'control'})
        return True
    
    def to_budget(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'budget',
                                'user_mail_created' : False,
                                'controlled_mail_created' : False,
                                'budget_mail_created' : False,
                                'coordinator_mail_created' : False,
                                'contract_project_mail_created' : False,
                                'headman_mail_created' : False,
                                'project_headman_mail_created' : False,
                                'building_director_mail_created' : False,
                                'executive_director_mail_created' : False,
                                'approved_mail_created' : False,
                                'law_dep_mail_created' : False,
                                'to_conclude_mail_created' : False,
                                'concluded_mail_created' : False,})
        return True
    
    def to_coordinator(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        budget_date_now = date.today()
        if obj.is_budget_completed == False:
            raise osv.except_osv(_("Анхааруулга!"), _("Та <<Төсөв гаргасан эсэх>> -талбарыг сонгоод, дүнгүүдийг оруулна уу!"))
        else:
            if obj.amount_of_budget == 0.0:
                raise osv.except_osv(_("Анхааруулга!"), _("Төсвийн дүн 0.0 байж болохгүй, та дүнг оруулна уу!"))
            elif obj.amount_of_hemab == 0.0:
                raise osv.except_osv(_("Анхааруулга!"), _("ХЭМАБ-н дүн 0.0 байж болохгүй, та дүнг оруулна уу!"))
            else:
                if obj.budget_date:
                    budget_date_now = obj.budget_date
                else:
                    budget_date_now = date.today()
                    
                self.write(cr, uid, ids, {'budget_date': budget_date_now,
                                          'state': 'coordinator',
                                            'user_mail_created' : True,
                                            'controlled_mail_created' : True,
                                            'budget_mail_created' : True,
                                            'coordinator_mail_created' : False,
                                            'contract_project_mail_created' : True,
                                            'headman_mail_created' : True,
                                            'project_headman_mail_created' : True,
                                            'building_director_mail_created' : True,
                                            'executive_director_mail_created' : True,
                                            'approved_mail_created' : True,
                                            'law_dep_mail_created' : True,
                                            'to_conclude_mail_created' : True,
                                            'concluded_mail_created' : True})
        return True
    
    def to_project(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        if obj.low_cost_contract == False:
            if obj.quality_emp_number <= 0:
                raise osv.except_osv(_("Анхааруулга!"), _("<<Чанарын хяналт хариуцсан ажилтны тоо>> нь 1-ээс бага байж болохгүй!"))
            elif obj.must_ita_number <= 0:
                raise osv.except_osv(_("Анхааруулга!"), _("<<Ажиллавал зохих ИТА ажилтны тоо>> нь 1-ээс бага байж болохгүй!"))
            elif obj.hemab_number <= 0:
                raise osv.except_osv(_("Анхааруулга!"), _("<<ХЭМАБ ажилтны тоо>> нь 1-ээс бага байж болохгүй!"))
            elif obj.professional_emps_number <= 0:
                raise osv.except_osv(_("Анхааруулга!"), _("<<Мэргэжлийн ажилтны тоо>> нь 1 -ээс бага байж болохгүй!"))
          
        self.write(cr, uid, ids, {'state': 'project',
                                  'coordinator_mail_created' : True,
                                  'contract_project_mail_created' : False,})
        return True
    
    def to_law_headman(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        budget_date_now = date.today()
        if obj.ready_project_of_contract == False:
            raise osv.except_osv(_("Анхааруулга!"), _("Та <<Гэрээний төсөл бэлэн болсон>> -талбарыг сонгоно уу!"))
        else:
            if obj.project_date:
                budget_date_now = obj.project_date
            else:
                budget_date_now = date.today()
            
            self.write(cr, uid, ids, {'project_date': budget_date_now,
                                      'state': 'law_headman',
                                      'headman_mail_created' : False,
                                      
                                      'contract_project_mail_created' : True})
        return True

    def to_back_project(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        self.write(cr, uid, ids, {'state': 'project',
                                'contract_project_mail_created' : False})
        return True
    
    def to_project_headman(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        budget_date_now = date.today()
        if obj.law_date:
            budget_date_now = obj.law_date
        else:
            budget_date_now = date.today()
        print 'ddddd', obj.low_cost_contract, obj.state
        if obj.low_cost_contract == True and obj.state == 'law_headman':
            raise osv.except_osv(_("Анхааруулга!"), _("Бага үнийн дүнтэй гэрээ тул <<Тэргүүн дэд захирал>> -товчыг дарна уу!"))
        else:
            self.write(cr, uid, ids, {'law_headman': uid,
                                  'law_date': budget_date_now,
                                  'state': 'project_headman',
                                  'contract_project_mail_created' : True,
                                  'project_headman_mail_created' : False,
                                  
                                  'headman_mail_created' : True})
            
        #=======================================================================
        # if obj.low_cost_contract == True:
        #     self.write(cr, uid, ids, {'state': 'project_headman'})
        # else:
        #     self.write(cr, uid, ids, {'state': 'project_headman'})
        #=======================================================================
        return True
    
    def to_building_director(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        budget_date_now = date.today()
        if obj.project_headman_date:
            budget_date_now = obj.project_headman_date
        else:
            budget_date_now = date.today()
        
        print 'sssssssssssssssss', obj.low_cost_contract, obj.state
        if obj.low_cost_contract == False and obj.state == 'law_headman':
            raise osv.except_osv(_("Анхааруулга!"), _("Бага үнийн дүнтэй гэрээ биш тул <<Төсөл хариуцсан захирал>> -товчыг дарна уу!"))
        else:
            self.write(cr, uid, ids, {'project_headman_date': budget_date_now,
                                  'state': 'building_director',
                                  'project_headman_mail_created' : True,
                                  'building_director_mail_created' : False})
        
        return True
    
    def to_executive_director(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        budget_date_now = date.today()
        if obj.building_director_date:
            budget_date_now = obj.building_director_date
        else:
            budget_date_now = date.today()
        if obj.low_cost_contract == True and obj.state == 'building_director':
            raise osv.except_osv(_("Анхааруулга!"), _("Бага үнийн дүнтэй гэрээ тул <<Батлах>> -товчыг дарна уу!"))
        else:
            self.write(cr, uid, ids, {'building_director_date': budget_date_now,
                                  'state': 'executive_director',
                                  'building_director_mail_created' : True,
                                  'executive_director_mail_created' : False})
        return True
    
    def to_approved(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        budget_date_now = date.today()
        if obj.executive_director_date:
            budget_date_now = obj.executive_director_date
        else:
            budget_date_now = date.today()
        if obj.low_cost_contract == False and obj.state == 'building_director':
            raise osv.except_osv(_("Анхааруулга!"), _("Бага үнийн дүнтэй гэрээ биш тул <<Гүйцэтгэх захирал>> -товчыг дарна уу!"))
        else:
            self.write(cr, uid, ids, {'executive_director_date': budget_date_now,
                                  'state': 'approved',
                                  'executive_director_mail_created' : True,
                                  'user_mail_created' : False,
                                  'approved_mail_created' : False})
        return True
    
    def to_conclude(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'conclude',
                                  'approved_mail_created' : True,
                                  'to_conclude_mail_created' : False})
        return True
    
    def to_concluded(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        if obj.is_concluded == False:
            raise osv.except_osv(_("Анхааруулга!"), _("Та <<Гэрээ дүгнэлтийн мэдээлэл>> -хэсгийг бөглөнө үү!"))
        self.write(cr, uid, ids, {'state': 'concluded',
                                  'to_conclude_mail_created' : True,
                                  'concluded_mail_created' : False})
        return True
    
    def to_contract_cancelled(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        if obj.contract_cancelled_reason:
            pass
        else:
            raise osv.except_osv(_("Анхааруулга!"), _("Та <<Гэрээ цуцлах шалтгаан>> -талбарыг бөглөнө үү!"))
        self.write(cr, uid, ids, {'state': 'contract_cancelled',
                                'controlled_mail_created' : True,
                                'budget_mail_created' : True,
                                'coordinator_mail_created' : True,
                                'contract_project_mail_created' : True,
                                'headman_mail_created' : True,
                                'project_headman_mail_created' : True,
                                'building_director_mail_created' : True,
                                'executive_director_mail_created' : True,
                                'approved_mail_created' : True,
                                'law_dep_mail_created' : True,
                                'to_conclude_mail_created' : True,
                                'concluded_mail_created' : True,})
        
        amendment_obj = self.pool.get('contract.amendment')
        amendment_ids = amendment_obj.search(cr, uid, [('wpc_id','=',obj.id)])
        amendments = amendment_obj.browse(cr, uid, amendment_ids)
        i = 0
        for line in amendments:
            i = i + 1
            line.to_cancelled()
        print 'iiiiiiiiiiiiii', i
        
        return True
    
    def to_cancelled(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        if obj.cancelled_reason:
            pass
        else:
            raise osv.except_osv(_("Анхааруулга!"), _("Та <<Гэрээ байгуулахгүй болсон шалтгаан>> -талбарыг бөглөнө үү!"))
        self.write(cr, uid, ids, {'state': 'cancelled',
                                'controlled_mail_created' : True,
                                'budget_mail_created' : True,
                                'coordinator_mail_created' : True,
                                'contract_project_mail_created' : True,
                                'headman_mail_created' : True,
                                'project_headman_mail_created' : True,
                                'building_director_mail_created' : True,
                                'executive_director_mail_created' : True,
                                'approved_mail_created' : True,
                                'law_dep_mail_created' : True,
                                'to_conclude_mail_created' : True,
                                'concluded_mail_created' : True})
        
        amendment_obj = self.pool.get('contract.amendment')
        amendment_ids = amendment_obj.search(cr, uid, [('wpc_id','=',obj.id)])
        amendments = amendment_obj.browse(cr, uid, amendment_ids)
        i = 0
        for line in amendments:
            i = i + 1
            line.to_cancelled()
        print 'iiiiiiiiiiiiii', i
        return True
    
    
    def onchange_import_conditions(self, cr, uid, ids, import_conditions, context=None):
        res = {}
        list = []
        dis_line_ids = []
        if import_conditions == True:
            master_obj = self.pool.get('master.data.of.conditions')
            master_ids = master_obj.search(cr, uid, [('is_default','=',True)])
            masters = master_obj.browse(cr, uid, master_ids)
            conditions_line_obj = self.pool.get('special.conditions.of.contract')
            i = 0
            if masters:
                for line in masters:
                    i = i + 1
                    dis_line_ids = conditions_line_obj.create(cr, uid, {
                        #'sequence': i,
                        'sequence': line.sequence,
                        'name': line.name,
                        'code': line.code,
                        'values': line.values,
                        'type': line.type
                        })
                    list.append(dis_line_ids)
            else:
                pass
            list = list
        return {'value': {'conditions_id': list}}
    
    def onchange_low_cost_contract(self, cr, uid, ids, low_cost_contract, company_id, contractor_company, name, context=None):
        res = {}
        obj = self.browse(cr, uid, ids)[0]
        aaaa = ''
        bbbb = ''
        if low_cost_contract == False:
            text_description = 'Нэг талаас <b>...................<b/> /цаашид “Захиалагч” гэх/, нөгөө талаас<b>...................<b/> /цаашид “Гүйцэтгэгч” гэх/ нар / цаашид хамтад нь Талууд гэх/ дараах зүйлээр харилцан тохиролцож энэхүү Барилга угсралтын ажил гүйцэтгэх гэрээ/ цаашид “Гэрээ” гэх/-г байгуулав.'
            text1 = '1. Гэрээгээр <b>...................<b/> ажлыг Иргэний, Барилгын болон бусад хууль тогтоомж, БНбД, улсын болон компанийн стандартын дагуу хийж гүйцэтгэхтэй холбогдон Захиалагч болон Гүйцэтгэгчийн эрх үүргийг тодорхойлж тэдний хооронд үүсэх харилцааг зохицуулна.'
            text2 = '2. Гүйцэтгэгч нь Гэрээ, гэрээний Ерөнхий болон Тусгай нөхцөл, бусад хавсралтуудын заалтуудын шаардлагыг хангаж Ажлын зураг, Төсөвт дурдсан ажлыг Ажлын зааврын дагуу графикт хугацаанд нь чанартай гүйцэтгэн Захиалагчид актаар хүлээлгэн өгөх ба ийнхүү гүйцэтгэсэн ажилтай холбогдох аливаа зөрчил, ажлын үр дүнгийн доголдлыг арилгах үүрэг, хариуцлагыг хүлээнэ. '
            text3 = '3. Гүйцэтгэгч нь энэхүү гэрээний Ерөнхий нөхцөлтэй танилцсан ба түүнийг зөвшөөрснөө үүгээр баталгаажуулж байна.'
        else:
            text_description = '<b>Гүйцэтгэгчийн үндсэн үүрэг:<b/>  Ажлыг батлагдсан зураг, материал, ажлын тодорхойлолтын дагуу ажлыг чанартай гүйцэтгэж Захиалагчид гүйцэтгэлийн актаар хүлээлгэн өгөх ба энэхүү гүйцэтгэсэн ажилтай холбогдох аливаа зөрчил, ажлын үр дүнгийн гологдлыг өөрийн зардлаар арилгах үүрэг хүлээнэ.'
            text1 = '<b>Захиалагчийн үндсэн үүрэг:<b/>  Гүйцэтгэгчийн ажлын үр дүнг гүйцэтгэлийн актаар шалган хүлээн авч ажлын хөлсийг төлөх үүрэг хүлээнэ.'
            text2 = 'Хийгдэх ажил: ..........'
            text3 = 'Тохиролцсон бусад нөхцөл:..........'
        
        res['html_description'] = str(text_description)
        res['html_1'] = str(text1)
        res['html_2'] = str(text2)
        res['html_3'] = str(text3)
        return {'value':res}
    
    def onchange_name(self, cr, uid, ids, name, context=None):
        res = {}
        text1 = '1. Гэрээгээр '
        text2 = ' ажлыг Иргэний, Барилгын болон бусад хууль тогтоомж, БНбД, улсын болон компанийн стандартын дагуу хийж гүйцэтгэхтэй холбогдон Захиалагч болон Гүйцэтгэгчийн эрх үүргийг тодорхойлж тэдний хооронд үүсэх харилцааг зохицуулна.'
        real_name = ''
        if name:
            real_name = name
        else:
            real_name = '..........'
        res['html_1'] = str(text1) + '<b>' + str(real_name) + '</b>' + str(text2)
        return {'value':res}
    
    def onchange_company_id(self, cr, uid, ids, company_id, contractor_company, context=None):
        res = {}
        text1 = '           Нэг талаас '
        text2 = ' /цаашид “Захиалагч” гэх/, нөгөө талаас '
        text3 = ' /цаашид “Гүйцэтгэгч” гэх/ нар / цаашид хамтад нь Талууд гэх/ дараах зүйлээр харилцан тохиролцож энэхүү Барилга угсралтын ажил гүйцэтгэх гэрээ/ цаашид “Гэрээ” гэх/-г байгуулав.'
        comp_name = ''
        part_name = ''
        if company_id:
            company_obj = self.pool.get('res.company')
            company_id_s = company_obj.search(cr, uid, [('id','=',company_id)])
            company = company_obj.browse(cr, uid, company_id_s)
            comp_name = company.name
        else:
            comp_name = '..........'
            
        if contractor_company:
            partner_obj = self.pool.get('res.partner')
            partner_id = partner_obj.search(cr, uid, [('id','=',contractor_company)])
            partner = partner_obj.browse(cr, uid, partner_id)
            part_name = partner.name
        else:
            part_name = '..........'
        
        res['html_description'] = str(text1) + '<b>' + str(comp_name) + '</b>' + str(text2) + '<b>' + str(part_name) + '</b>' + str(text3)
        
        return {'value':res}
    
    def onchange_contractor_company(self, cr, uid, ids, contractor_company, company_id, context=None):
        res = {}
        text1 = '           Нэг талаас '
        text2 = ' ХХК /цаашид “Захиалагч” гэх/, нөгөө талаас '
        text3 = ' ХХК /цаашид “Гүйцэтгэгч” гэх/ нар / цаашид хамтад нь Талууд гэх/ дараах зүйлээр харилцан тохиролцож энэхүү Барилга угсралтын ажил гүйцэтгэх гэрээ/ цаашид “Гэрээ” гэх/-г байгуулав.'
        comp_name = ''
        part_name = ''
        if company_id:
            company_obj = self.pool.get('res.company')
            company_id_s = company_obj.search(cr, uid, [('id','=',company_id)])
            company = company_obj.browse(cr, uid, company_id_s)
            comp_name = company.name
        else:
            comp_name = '..........'
            
        if contractor_company:
            partner_obj = self.pool.get('res.partner')
            partner_id = partner_obj.search(cr, uid, [('id','=',contractor_company)])
            partner = partner_obj.browse(cr, uid, partner_id)
            res['contractor_address'] = partner.address_for_contract
            part_name = partner.name
        else:
            part_name = '..........'
            res['contractor_address'] = ''
        
        res['html_description'] = str(text1) + '<b>' + str(comp_name) + '</b>' + str(text2) + '<b>' + str(part_name) + '</b>' + str(text3)
        
        return {'value':res}
    
    def onchange_material_amount(self, cr, uid, ids, amount_of_budget, context=None):
        res = {}
        res['contract_amount'] = amount_of_budget
        return {'value':res}
    
    def onchange_contract_amount(self, cr, uid, ids, contract_amount, performer_material, customer_material, amount_of_labor, amount_of_hemab, amount_mech_performer, context=None):
        res = {}
        text1 = '4. Захиалагч нь Гэрээ, гэрээний Ерөнхий болон Тусгай нөхцөлийн заалтуудын шаардлагыг хангаж Гүйцэтгэгчийн ажлын үр дүнг шалган актаар хүлээн авч захиалагчаас нийлүүлэх бараа, материал '
        text2 = ' төгрөг, гүйцэтгэгчийн материал, ажлын хөлс болон бусад зардал '
        text3 = ' төгрөгийг багтаасан хавсралт төсөвт заасан нийт '
        text4 = ' <b>/үсгээр/ /НӨАТ багтсан/</b> төгрөгийн тооцоог түүнтэй хийх үүрэг, хариуцлагыг хүлээнэ. Энэ дүн нь Гэрээний үнэ болно.'
        
        res['html_4'] = str(text1) + '<b>' + str(customer_material) + '</b>' + str(text2) + '<b>' + str(performer_material + amount_of_labor + amount_of_hemab + amount_mech_performer) + '</b>' + str(text3) + '<b>' + str(contract_amount) + '</b>' + str(text4)
        return {'value':res}
    
    def onchange_consumer_name(self, cr, uid, ids, consumer_name, context=None):
        res = {}
        if consumer_name:
            user_obj = self.pool.get('res.users')
            user_id = user_obj.search(cr, uid, [('id','=',consumer_name)])
            user = user_obj.browse(cr, uid, user_id)
            
            emp_obj = self.pool.get('hr.employee')
            emp_id = emp_obj.search(cr, uid, [('user_id','=',consumer_name)])
            emp = emp_obj.browse(cr, uid, emp_id)
            
            res['consumer_job'] = emp.job_id.id
            #res['consumer_phone'] = emp.work_phone
            res['consumer_phone'] = '77002020'
        else:
            res['consumer_job'] = False
            res['consumer_phone'] = False
        return {'value':res}
    
    
    def onchange_headman_name(self, cr, uid, ids, headman_name, context=None):
        res = {}
        if headman_name:
            user_obj = self.pool.get('res.users')
            user_id = user_obj.search(cr, uid, [('id','=',headman_name)])
            user = user_obj.browse(cr, uid, user_id)
            
            emp_obj = self.pool.get('hr.employee')
            emp_id = emp_obj.search(cr, uid, [('user_id','=',headman_name)])
            emp = emp_obj.browse(cr, uid, emp_id)
            
            res['headman_job'] = emp.job_id.id
            #res['headman_phone'] = emp.work_phone
            res['headman_phone'] = '77002020'
        else:
            res['headman_job'] = False
            res['headman_phone'] = False
            
        return {'value':res}
    
    def onchange_coordinator_name(self, cr, uid, ids, coordinator_name, context=None):
        res = {}
        if coordinator_name:
            emp_obj = self.pool.get('hr.employee')
            emp_id = emp_obj.search(cr, uid, [('user_id','=',coordinator_name)])
            emp = emp_obj.browse(cr, uid, emp_id)
            
            res['coordinator_job'] = emp.job_id.id
            res['coordinator_phone'] = emp.work_phone
        else:
            res['coordinator_job'] = False
            res['coordinator_phone'] = False
            
        return {'value':res}
    
    def onchange_contractor_name(self, cr, uid, ids, contractor_name, context=None):
        res = {}
        if contractor_name:
            print 'lllllllll', contractor_name
            partner_obj = self.pool.get('res.partner')
            partner_id = partner_obj.search(cr, uid, [('id','=',contractor_name)])
            partner = partner_obj.browse(cr, uid, partner_id)
            
            res['contractor_job'] = partner.function
            res['contractor_phone'] = partner.phone
        else:
            res['contractor_job'] = False
            res['contractor_phone'] = False
            
        return {'value':res}
    
    def onchange_contractor_agent(self, cr, uid, ids, contractor_agent, context=None):
        res = {}
        if contractor_agent:
            print 'lllllllll', contractor_agent
            partner_obj = self.pool.get('res.partner')
            partner_id = partner_obj.search(cr, uid, [('id','=',contractor_agent)])
            partner = partner_obj.browse(cr, uid, partner_id)
            
            res['agent_job'] = partner.function
            res['agent_phone'] = partner.phone
        else:
            res['agent_job'] = False
            res['agent_phone'] = False
            
        return {'value':res}
    
    def onchange_quantity_surveyor(self, cr, uid, ids, quantity_surveyor, context=None):
        res = {}
        if quantity_surveyor:
            user_obj = self.pool.get('res.users')
            user_id = user_obj.search(cr, uid, [('id','=',quantity_surveyor)])
            user = user_obj.browse(cr, uid, user_id)
            
            res['quantity_surveyor_phone'] = user.phone
        else:
            res['quantity_surveyor_phone'] = False
        return {'value':res}
    
    def onchange_budget_amounts(self, cr, uid, ids, amount_of_labor, amount_of_hemab, amount_mech_performer, performer_material, amount_mech_customer, customer_material, context=None):
        res = {}
        without_vat = amount_of_labor + amount_of_hemab + amount_mech_performer + performer_material
        with_vat = amount_mech_customer + customer_material
        
        budget_total = without_vat + without_vat/10 + with_vat
        res['amount_of_budget'] = budget_total
        return {'value':res}
        

class type_of_contract(osv.Model):
    _name = 'type.of.contract'
    
    _columns = {
        'name': fields.char('Name'),#Нэр
        'code': fields.char('Code'),#Код
        #'values': fields.text('Values'),#Утга
        #'work_performance_id':fields.many2one('work.performance.contract','Work performance'),
        
        }
    _sql_constraints = [
        ('name', 'unique("name")', 'Field <name> must be unique.'),
        ('code', 'unique("code")', 'Field <code> must be unique.'),
        ]

class special_conditions_of_contract(osv.Model):
    _name = 'special.conditions.of.contract'
    
    _columns = {
        'sequence': fields.integer('Sequence'),#Дараалал
        'name': fields.text('Name'),#Нэр
        'code': fields.char('Code'),#Код
        'values': fields.text('Values'),#Утга
        'work_performance_id':fields.many2one('work.performance.contract','Work performance'),
        'type': fields.selection([('coordinator_info','Coordinator info'),
                                  ('contractor_address','Contractor address'),
                                  ('project_name','Project name'),
                                  ('coordinator_filled_numbers','Coordinator filled numbers'),
                                  ('coordinator_filled_dates','Coordinator filled dates'),
                                  ('material_amounts','Material amounts'),
                                  ('contract_amount','Contract amount'),
                                  ('hemab_amount','HEMAB amount'),
                                  ('performer_amounts_2','Performer amounts 2%'),
                                  ('performer_amounts_5','Performer amounts 5%'),
                                  ],'Type'),#Өгөгдөл татах төрөл
        
        }

class master_data_of_conditions(osv.Model):
    _name = 'master.data.of.conditions'
    
    _columns = {
        'sequence': fields.integer('sequence'),#Дараалал
        'name': fields.text('Name'),#Нэр
        'code': fields.char('Code'),#Код
        'values': fields.text('Values'),#Утга
        'is_default': fields.boolean('Default'),#Загварт дуудагдах эсэх
        'type': fields.selection([('coordinator_info','Coordinator info'),
                                  ('contractor_address','Contractor address'),
                                  ('project_name','Project name'),
                                  ('coordinator_filled_numbers','Coordinator filled numbers'),
                                  ('coordinator_filled_dates','Coordinator filled dates'),
                                  ('material_amounts','Material amounts'),
                                  ('contract_amount','Contract amount'),
                                  ('hemab_amount','HEMAB amount'),
                                  ('performer_amounts_2','Performer amounts 2%'),
                                  ('performer_amounts_5','Performer amounts 5%'),
                                  ],'Type'),#Өгөгдөл татах төрөл
        
        }

    _sql_constraints = [
        ('type', 'unique("type")', 'Field <type> must be unique.'),
        ]

class intermediate_funding(osv.Model):
    _name = 'intermediate.funding'
    _description = 'Intermediate funding'
    
    _columns = {
        'description': fields.text('Description'),#Тайлбар
        'amount': fields.integer('Amount'),#Дүн
        'date': fields.date('Date'),#Огноо
        'work_performance_contract_id': fields.many2one('work.performance.contract','Work performance contract', track_visibility='onchange'),#Ажил гүйцэтгэх гэрээ
        
        }
    _defaults = {
                'date' : date.today(),
        }


class ir_attachment(osv.osv):
    _inherit = 'ir.attachment'
    
    def name_get(self, cr, uid, ids, context=None):
        if isinstance(ids, (list, tuple)) and not len(ids):
            return []
        if isinstance(ids, (long, int)):
            ids = [ids]
        reads = self.read(cr, uid, ids, ['name','doc_number'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['doc_number']:
                name = name +' / '+ record['doc_number']
            res.append((record['id'], name))
        return res

    
    
    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=100):
        if args is None:
            args = []
        if context is None:
            context = {}
        ids = []
        if name:
            ids = self.search(cr, uid, [('name', 'ilike', name)] + args, limit=limit)
        if not ids:
            ids = self.search(cr, uid, [('doc_number', 'ilike', name)] + args, limit=limit)
        return self.name_get(cr, uid, ids, context=context)
    
    
    _columns = {
        'wpc_id': fields.one2many('work.performance.contract', 'document_received_id', 'WPC'),#Холбогдох гэрээ
        
        }
    
    #===========================================================================
    # def _constrains_wpc_id(self):
    #     if len(self.wpc_id) > 1:
    #         raise openerp.exceptions.ValidationError('Additional linkage failed. One received_paper can not be many contracts')
    #===========================================================================
    
    

class res_company(osv.osv):
    _inherit = 'res.company'
    _columns = {
               #'company_code':fields.integer('Company code', size = 4),
               #'is_swiss':fields.boolean('Is swiss'),
               'contract_cipher':fields.char('Contract cipher'),
               }
    _sql_constraints = [
        ('contract_cipher', 'unique("contract_cipher")', 'Field <contract_cipher> must be unique.'),
        ]
    

class res_partner(osv.osv):
    _inherit = 'res.partner'
    _columns = {
               'address_for_contract':fields.text('Address for contract'),
               }

class res_users_contract_inherit(osv.Model):
    _inherit= 'res.users'
    
    
    def _get_image(self, cr, uid, ids, name, args, context=None):
        result = dict.fromkeys(ids, False)
        for obj in self.browse(cr, uid, ids, context=context):
            result[obj.id] = tools.image_get_resized_images(obj.signature)
        return result

    def _set_image(self, cr, uid, id, name, value, args, context=None):
        return self.write(cr, uid, [id], {'signature': tools.image_resize_image_big(value)}, context=context)

    _columns = {
                #===============================================================
                # 'signature': fields.binary("Гарын үсэг",
                # help="This field holds the image used as photo for the employee, limited to 1024x1024px."),
                # 'signature_medium': fields.function(_get_image, fnct_inv=_set_image,
                # string="Medium-sized photo", type="binary", multi="_get_image",
                # store = {'hr.employee': (lambda self, cr, uid, ids, c={}: ids, ['signature'], 10),
                # }),
                # 'signature_small': fields.function(_get_image, fnct_inv=_set_image,
                # string="Small-sized photo", type="binary", multi="_get_image",
                # store = { 'hr.employee': (lambda self, cr, uid, ids, c={}: ids, ['signature'], 10),
                # }),
                #===============================================================
                #===============================================================
                # 'signature': fields.binary("Гарын үсэг",
                # help="This field holds the image used as photo for the employee, limited to 1024x1024px."),
                #===============================================================
                'job_type': fields.selection([('executive_director','Executive director'),
                                  ('building_director','Building director'),
                                  ('project_headman','Project headman')],'Job type'),#Албан тушаалын төрөл
                #'signature_image_tb':fields.binary(related='signature', string='Signature tb'),
                
        }
    
    
    