# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.tools.translate import _
import time

class res_company(osv.osv):
    
    _name = "res.company"
    _inherit = "res.company"
    _columns = {
        'currency_exchange_gains': fields.many2one('account.account', 'Currency Exchange Gains', domain="[('type', '!=', 'view')]",),
        'currency_exchange_losses': fields.many2one('account.account', 'Currency Exchange Losses', domain="[('type', '!=', 'view')]",),
    }
    
res_company()

class account_journal(osv.osv):
    
    _name = "account.journal"
    _inherit = "account.journal"
    _columns = {
        'type': fields.selection([
            ('cur_eq', 'Currency Rate Equalization'),
            ('cur_ex', 'Currency Exchange'),
            ('sale', 'Sale'),
            ('sale_refund', 'Sale Refund'),
            ('purchase', 'Purchase'),
            ('purchase_refund', 'Purchase Refund'),
            ('cash', 'Cash'),
            ('bank', 'Bank and Cheques'),
            ('general', 'General'),
            ('situation', 'Opening/Closing Situation')], 'Type', size=32, required=True,
                help="Select 'Sale' for customer invoices journals."\
                    " Select 'Purchase' for supplier invoices journals."\
                    " Select 'Cash' or 'Bank' for journals that are used in customer or supplier payments."\
                    " Select 'General' for miscellaneous operations journals."\
                    " Select 'Opening/Closing Situation' for entries generated for new fiscal years."),
    }
    def onchange_type(self, cr, uid, ids, type, currency, context=None):
        obj_data = self.pool.get('ir.model.data')
        user_pool = self.pool.get('res.users')
        type_map = {
            'cur_eq':'account_journal_view',
            'cur_ex':'account_journal_view',
            'sale':'account_sp_journal_view',
            'sale_refund':'account_sp_refund_journal_view',
            'purchase':'account_sp_journal_view',
            'purchase_refund':'account_sp_refund_journal_view',
            'cash':'account_journal_bank_view',
            'bank':'account_journal_bank_view',
            'general':'account_journal_view',
            'situation':'account_journal_view'
        }
        res = {}
        view_id = type_map.get(type, 'account_journal_view')
        user = user_pool.browse(cr, uid, uid)
        if type in ('cash', 'bank') and currency and user.company_id.currency_id.id != currency:
            view_id = 'account_journal_bank_view_multi'
        data_id = obj_data.search(cr, uid, [('model','=','account.journal.view'), ('name','=',view_id)])
        data = obj_data.browse(cr, uid, data_id[0], context=context)
        res.update({
            'centralisation':type == 'situation',
            'view_id':data.res_id,})
        return {'value':res}
    
account_journal()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
