# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.tools.translate import _
import time

class currency_journal_config(osv.osv):
    
    _name = 'currency.journal.config'
    _columns = {
        'currency_id': fields.many2one('res.currency', 'Currency', required=True, domain="[('name','!=','MNT')]"),
        'cur_cash_journal': fields.many2one('account.journal', 'Related Cash Journal', required=True, domain="[('type','=','cash')]"),
        'cur_bank_journal': fields.many2one('account.journal', 'Related Bank Journal', required=True, domain="[('type','=','bank')]"),
        'config_id': fields.many2one('currency.exchange.config', 'Config ID'),
    }
    _sql_constraints = [
        ('unique_currency_id_config_id', 'unique (currency_id, config_id)', 'The Currency configuration must be unique!')
    ]
    
    def onchange_currency(self, cr, uid, ids, currency_id):
        vals = {}
        vals['cur_cash_journal'] = False
        vals['cur_bank_journal'] = False
        domain = {
            'cur_cash_journal': [('type','=','cash'), ('currency','=',currency_id)],
            'cur_bank_journal': [('type','=','bank'), ('currency','=',currency_id)]
        }
        return {'value':vals, 'domain':domain}
    
currency_journal_config()

class currency_exchange_config(osv.osv):
    
    _name = 'currency.exchange.config'
    _columns = {
        'name': fields.char('Name', size=60),
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'cash_journal': fields.many2one('account.journal', 'Cash Journal', required=True, domain="[('type','=','cash'), ('currency_id','=',False)]"),
        'bank_journal': fields.many2one('account.journal', 'Bank Journal', required=True, domain="[('type','=','bank'), ('currency_id','=',False)]"),
        'tmp_ex_acc': fields.many2one('account.account', 'C.E. Temp. Account', required=True, domain="[('type','!=','view'), ('currency_id','=',False)]"),
        'cur_ex_journal': fields.many2one('account.journal', 'C.E. Journal', required=True, domain="[('type','=','cur_ex'), ('currency_id','=',False)]"),
        'cur_journal_ids': fields.one2many('currency.journal.config', 'config_id', 'Config ID'),
        'state': fields.selection([('draft','Draft'), ('created','Created')], 'State'),
    }
    _defaults = {
        'state': lambda *a: 'draft',
    }
    
    def onchange_company(self, cr, uid, ids, c_id):
        vals = {}
        vals['tmp_ex_acc'] = False
        vals['cash_journal'] = False
        vals['bank_journal'] = False
        vals['cur_ex_journal'] = False
        domain = {
            'tmp_ex_acc': [('company_id','=',c_id), ('type','!=','view'), ('currency_id','=',False)],
            'cash_journal': [('company_id','=',c_id), ('type','=','cash'), ('currency','=',False)],
            'bank_journal': [('company_id','=',c_id), ('type','=','bank'), ('currency','=',False)],
            'cur_ex_journal': [('company_id','=',c_id), ('type','=','cur_ex'), ('currency','=',False)]
        }
        return {'value':vals, 'domain':domain}
    
    def _check_journal(self, cr, uid, j_id, j_type, c_id, is_dom_cur, cur_id=None):
        acc_type_exist = False
        company = self.pool.get('res.company').browse(cr, uid, c_id)
        journal = self.pool.get('account.journal').browse(cr, uid, j_id)
        db = journal.default_debit_account_id
        ct = journal.default_credit_account_id
        acc_type_ids = self.pool.get('account.account.type').search(cr, uid, [('code','=',j_type)])
        if journal.type != j_type or journal.company_id.id != company.id:
            return False
        if db.company_id.id != company.id or ct.company_id.id != company.id:
            return False
        if is_dom_cur == True:
            if journal.currency.id != False:
                return False
            if db.currency_id.id != False or ct.currency_id.id != False:
                return False
        else:
            if journal.currency.id != cur_id:
                return False
            if db.currency_id.id != cur_id or ct.currency_id.id != cur_id:
                return False
        for ati in acc_type_ids:
            if ati == db.user_type.id and ati == ct.user_type.id:
                acc_type_exist = True
                break
        if acc_type_exist == False:
            return False
        return True
    
    def _vals_check(self, cr, uid, vals, ids=None):
        data = {
            'tmp_ex_acc':True,
            'cash_journal':True,
            'bank_journal':True,
            'cur_ex_journal':True,
            'cur_journal_ids':True}
        
        if 'tmp_ex_acc' in vals:
            tmp_ex_acc = self.pool.get('account.account').browse(cr, uid, vals['tmp_ex_acc'])
            if tmp_ex_acc.company_id.id != vals['company_id'] or tmp_ex_acc.currency_id.id != False:
                data['tmp_ex_acc'] = False
        
        if 'cash_journal' in vals:
            data['cash_journal'] = self._check_journal(cr, uid, vals['cash_journal'], 'cash', vals['company_id'], True)
        
        if 'bank_journal' in vals:
            data['bank_journal'] = self._check_journal(cr, uid, vals['bank_journal'], 'bank', vals['company_id'], True)
        
        if 'cur_ex_journal' in vals:
            cur_ex_journal = self.pool.get('account.journal').browse(cr, uid, vals['cur_ex_journal'])
            if cur_ex_journal.type != 'cur_ex' or cur_ex_journal.currency.id != False or cur_ex_journal.company_id.id != vals['company_id']:
                data['cur_ex_journal'] = False
        
        if 'cur_journal_ids' in vals:
            if vals['cur_journal_ids'] != []:
                obj = self.pool.get('currency.exchange.config').browse(cr, uid, ids[0])
                cur_data = {
                    'bank_journal':True,
                    'cash_journal':True}
                for item in vals['cur_journal_ids']:
                    if item[0] == 0 or item[0] == 1:
                        cur_data['bank_journal'] = self._check_journal(cr, uid, item[2]['cur_bank_journal'], 'bank', obj.company_id.id, False, item[2]['currency_id'])
                        if cur_data['bank_journal'] == False:
                            data['cur_journal_ids'] = False
                            break
                        cur_data['cash_journal'] = self._check_journal(cr, uid, item[2]['cur_cash_journal'], 'cash', obj.company_id.id, False, item[2]['currency_id'])
                        if cur_data['cash_journal'] == False:
                            data['cur_journal_ids'] = False
                            break
        return data
    
    def create(self, cr, uid, vals, context=None):
        c_id = vals['company_id']
        obj_ids = self.pool.get('currency.exchange.config').search(cr, uid, [('company_id','=',vals['company_id'])])
        if obj_ids != []:
            raise osv.except_osv(_('Error'), _("'%s' configuration already created.") % (self.pool.get('res.company').browse(cr, uid, c_id).name))
        checker = self._vals_check(cr, uid, vals)
        if checker['tmp_ex_acc'] == False:
            raise osv.except_osv(_('Error'), _("'C.E. Temp. Account' configuration is wrong."))
        if checker['cash_journal'] == False:
            raise osv.except_osv(_('Error'), _("'Cash Journal' configuration is wrong."))
        if checker['bank_journal'] == False:
            raise osv.except_osv(_('Error'), _("'Bank Journal' configuration is wrong."))
        if checker['cur_ex_journal'] == False:
            raise osv.except_osv(_('Error'), _("'C.E. Journal' configuration is wrong."))
        vals['state'] = 'created'
        vals['name'] = u'Валют арилжааны журналуудын тохиргоо (' + self.pool.get('res.company').browse(cr, uid, c_id).name + ')'
        return super(currency_exchange_config, self).create(cr, uid, vals, context)
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids[0])
        vals['company_id'] = obj.company_id.id
        checker = self._vals_check(cr, uid, vals, ids)
        if checker['tmp_ex_acc'] == False:
            raise osv.except_osv(_('Error'), _("'C.E. Temp. Account' configuration is wrong."))
        if checker['cash_journal'] == False:
            raise osv.except_osv(_('Error'), _("'Cash Journal' configuration is wrong."))
        if checker['bank_journal'] == False:
            raise osv.except_osv(_('Error'), _("'Bank Journal' configuration is wrong."))
        if checker['cur_ex_journal'] == False:
            raise osv.except_osv(_('Error'), _("'C.E. Journal' configuration is wrong."))
        if checker['cur_journal_ids'] == False:
            raise osv.except_osv(_('Error'), _('One of the Currency configuration is wrong.'))
        return super(currency_exchange_config, self).write(cr, uid, ids, vals, context)
    
    def configure(self, cr, uid, ids, vals, context=None):
        return True
    
currency_exchange_config()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
