# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import date

class currency_exchange_purchase(osv.osv):
    
    def _get_move_state(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids):
            if obj.move1_id.state == 'draft' or obj.move2_id.state == 'draft' or obj.move3_id.state == 'draft':
                res[obj.id] = 'draft'
            else:
                res[obj.id] = 'posted'
        return res
    
    def _get_company(self, cr, uid, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid)
        return user.company_id.id
    
    _name = "currency.exchange.purchase"
    _columns = {
        'name': fields.char('Name', size=50, readonly=True, store=True),
        'amount_cur': fields.float('Amount (Currency)', digits=(12,2), required=True, states={'done':[('readonly',True)]}),
        'amount_tug': fields.float('Amount (MNT)', digits=(12,2), readonly=True, store=True),
        'company_id': fields.many2one('res.company', 'Company'),
        'currency_id': fields.many2one('res.currency', 'Currency (purchase)', required=True, domain="[('name','!=','MNT')]", states={'done':[('readonly',True)]}),
        'custom_rate': fields.float('Currency Rate (purchase)', digits=(12,2), required=True, states={'done':[('readonly',True)]}),
        'is_cash_cur': fields.boolean('By Cash? (currency)', states={'done':[('readonly',True)]}),
        'is_cash_mnt': fields.boolean('By Cash? (MNT)', states={'done':[('readonly',True)]}),
        'is_enterprise': fields.boolean('Is Enterprise', states={'done':[('readonly',True)]}),
        'partner_id': fields.many2one('res.partner', 'Customer', states={'required':[('required',True)],'done':[('readonly',True)]}),
        'state': fields.selection([('draft','Draft'),
                                   ('done','Done'),
                                   ('required','Required')], 'State', readonly=True),
        
        'date': fields.date('Date', readonly=True, store=True),
        'currency_rate': fields.float('Rate', digits=(12,2), readonly=True, store=True),
        'amount_currency': fields.float('Amount Currency', digits=(12,2), readonly=True, store=True),
        'amount_mnt': fields.float('Amount MNT', digits=(12,2), readonly=True, store=True),
        'is_purchased': fields.selection([('purchase','Purchased'),('sale','Sold')], 'Is Purchased?', readonly=True),
        'cash_account': fields.many2one('account.account', 'Cash Account'),
        'currency_account': fields.many2one('account.account', 'Currency Account'),
        'user': fields.many2one('res.users', 'User'),
        'move_state': fields.function(_get_move_state, type="char", method="true", string="Move State"),
        'move1_id': fields.many2one('account.move', 'Move one'),
        'move2_id': fields.many2one('account.move', 'Move two'),
        'move3_id': fields.many2one('account.move', 'Move three'),
    }
    _defaults = {
        'name': lambda self, cr, uid, context: self.pool.get('ir.sequence').get(cr, uid, 'currency.exchange.purchase'),
        'company_id': _get_company,
        'amount_cur': 0,
        'amount_tug': 0,
        'custom_rate': 0,
        'date' : lambda *a: date.today().strftime('%Y-%m-%d'),
        'state' : lambda *a: 'draft',
        'is_purchased': lambda *a: False,
        'is_cash_cur': lambda *a: True,
        'is_cash_mnt': lambda *a: True,
        'is_enterprise': lambda *a: False,
        'amount_currency': 0,
        'amount_mnt': 0,
    }
    
    def onchange_currency(self, cr, uid, ids, currency_id, this_date):
        pre_rate = 0
        pur_ids = self.pool.get('currency.exchange.purchase').search(cr, uid, [('currency_id','=',currency_id), ('state','=','done'), ('date','=',this_date)], order='id')
        if pur_ids != []:
            pur_obj = self.pool.get('currency.exchange.purchase').browse(cr, uid, pur_ids[len(pur_ids)-1])
            pre_rate = pur_obj.custom_rate
        val = {}
        val['amount_cur'] = 0
        val['custom_rate'] = pre_rate
        return {'value':val}
    
    def onchange_amount(self, cr, uid, ids, custom_rate, amount_cur, currency_id):
        val = {}
        war = {}
        if currency_id != False:
            currency = self.pool.get('res.currency').browse(cr, uid, currency_id)
            amount = currency.rate * amount_cur
            if amount > 20000000:
                val['state'] = 'required'
            else:
                val['state'] = 'draft'
            val['amount_tug'] = custom_rate * amount_cur
            return {'value':val, 'warning':war}
        else:
            war['title'] = 'Warning'
            war['message'] = 'Choose Currency before entering the amount.'
            val['amount_cur'] = False
            return {'value':val, 'warning':war}
    
    def onchange_rate(self, cr, uid, ids, custom_rate, amount_cur, currency_id):
        val = {}
        val['amount_tug'] = custom_rate * amount_cur
        return {'value':val}
    
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0])
        if obj.move1_id.id != False:
            if obj.move1_id.state == 'posted':
                raise osv.except_osv(_('Error'), _("Posted entry can not be deleted."))
        if obj.move2_id.id != False:
            if obj.move2_id.state == 'posted':
                raise osv.except_osv(_('Error'), _("Posted entry can not be deleted."))
        if obj.move3_id.id != False:
            if obj.move3_id.state == 'posted':
                raise osv.except_osv(_('Error'), _("Posted entry can not be deleted."))
        if obj.move1_id.id != False:
            self.pool.get('account.move').unlink(cr, uid, [obj.move1_id.id], context)
        if obj.move2_id.id != False:
            self.pool.get('account.move').unlink(cr, uid, [obj.move2_id.id], context)
        if obj.move3_id.id != False:
            self.pool.get('account.move').unlink(cr, uid, [obj.move3_id.id], context)
        return super(currency_exchange_purchase, self).unlink(cr, uid, ids, context)
    
    def calc_btn(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        data = self.browse(cr, uid , ids)[0]
        
        name_pat = data.name
        company = data.company_id
        amount_cur = data.amount_cur
        currency = data.currency_id
        custom_rate = data.custom_rate
        partner = data.partner_id
        is_cash_cur = data.is_cash_cur
        is_cash_mnt = data.is_cash_mnt
        
        margin = 0
        cur_journal_configured = False
        
        if amount_cur <= 0:
            raise osv.except_osv(_('Error'), _("No input or wrong input (Amount Currency)."))
        if custom_rate <= 0:
            raise osv.except_osv(_('Error'), _("No input or wrong input (Currency Rate)."))
        
        cec_ids = self.pool.get('currency.exchange.config').search(cr, uid, [('company_id','=',company.id)])
        if cec_ids == []:
            raise osv.except_osv(_('Error'), _("Can not find currency exchange configuration of '%s'.") % (company.name))
        cec_obj = self.pool.get('currency.exchange.config').browse(cr, uid, cec_ids[0])
        
        losses = company.currency_exchange_losses
        gains = company.currency_exchange_gains
        temp_acc = cec_obj.tmp_ex_acc
        
        if gains.id == False:
            raise osv.except_osv(_('Error'), _("'Currency Exchange Gains' account of '%s' is not configured.") % (company.name))
        if losses.id == False:
            raise osv.except_osv(_('Error'), _("'Currency Exchange Losses' account of '%s' is not configured.") % (company.name))
        
        if is_cash_mnt == True:
            mnt_journal = cec_obj.cash_journal
            mnt_db = mnt_journal.default_debit_account_id
            mnt_ct = mnt_journal.default_credit_account_id
        else:
            mnt_journal = cec_obj.bank_journal
            mnt_db = mnt_journal.default_debit_account_id
            mnt_ct = mnt_journal.default_credit_account_id
        
        for cj in cec_obj.cur_journal_ids:
            if cj.currency_id == currency:
                cur_journal_configured = True
                if is_cash_cur == True:
                    cur_journal = cj.cur_cash_journal
                    cur_db = cur_journal.default_debit_account_id
                    cur_ct = cur_journal.default_credit_account_id
                else:
                    cur_journal = cj.cur_bank_journal
                    cur_db = cur_journal.default_debit_account_id
                    cur_ct = cur_journal.default_credit_account_id
        if cur_journal_configured == False:
            raise osv.except_osv(_('Error'), _("'%s' bank/cash journal is not configured.") % (currency.name))
        
        cej = cec_obj.cur_ex_journal
        
        amount_tug = custom_rate * amount_cur
        amount_mnt = currency.rate * amount_cur
        
        am_one_name = 'currency_exchange /purchase/ - 1 (%s)' % (name_pat)
        am_one_id = self.pool.get('account.move').create(cr, uid, {'name':am_one_name,'journal_id':mnt_journal.id,'ref':name_pat})
        self.pool.get('account.move.line').create(cr, uid, {
            'name':'%s purchased' % (currency.name),
            'account_id':mnt_ct.id,
            'credit':amount_tug,
            'move_id':am_one_id,
            'partner_id':partner.id
        })
        self.pool.get('account.move.line').create(cr, uid, {
            'name':'%s purchased' % (currency.name),
            'account_id':temp_acc.id,
            'debit':amount_tug,
            'move_id':am_one_id,
            'partner_id':partner.id
        })
        
        am_two_name = 'currency_exchange /purchase/ - 2 (%s)' % (name_pat)
        am_two_id = self.pool.get('account.move').create(cr, uid, {'name':am_two_name,'journal_id':cur_journal.id,'ref':name_pat})
        self.pool.get('account.move.line').create(cr, uid, {
            'name':'%s purchased' % (currency.name),
            'account_id':cur_db.id,
            'debit':amount_mnt,
            'currency_id':currency.id,
            'amount_currency':amount_cur,
            'move_id':am_two_id,
            'partner_id':partner.id
        })
        self.pool.get('account.move.line').create(cr, uid, {
            'name':'%s purchased' % (currency.name),
            'account_id':temp_acc.id,
            'credit':amount_mnt,
            'move_id':am_two_id,
            'partner_id':partner.id
        })
        
        if amount_tug < amount_mnt:
            margin = amount_mnt - amount_tug
            am_three_name = 'currency_exchange /purchase/ - 3 (%s)' % (name_pat)
            am_three_id = self.pool.get('account.move').create(cr, uid, {'name':am_three_name,'journal_id':cej.id,'ref':name_pat})
            self.pool.get('account.move.line').create(cr, uid, {
                'name':'%s purchased' % (currency.name),
                'account_id':gains.id,
                'credit':margin,
                'move_id':am_three_id,
                'partner_id':partner.id
            })
            self.pool.get('account.move.line').create(cr, uid, {
                'name':'%s purchased' % (currency.name),
                'account_id':temp_acc.id,
                'debit':margin,
                'move_id':am_three_id,
                'partner_id':partner.id
            })
        else:
            margin = amount_tug - amount_mnt
            am_three_name = 'currency_exchange /purchase/ - 3 (%s)' % (name_pat)
            am_three_id = self.pool.get('account.move').create(cr, uid, {'name':am_three_name,'journal_id':cej.id,'ref':name_pat})
            self.pool.get('account.move.line').create(cr, uid, {
                'name':currency.name + ' purchased',
                'account_id':losses.id,
                'debit':margin,
                'move_id':am_three_id,
                'partner_id':partner.id
            })
            self.pool.get('account.move.line').create(cr, uid, {
                'name':currency.name + ' purchased',
                'account_id':temp_acc.id,
                'credit':margin,
                'move_id':am_three_id,
                'partner_id':partner.id
            })
        
        self.write(cr, uid, ids, {
            'amount_tug': amount_tug,
            'name': 'currency_exchange /purchase/ (%s)' % (name_pat),
            'currency_rate': currency.rate,
            'amount_currency': amount_cur,
            'amount_mnt': amount_mnt,
            'is_purchased': 'purchase',
            'cash_account': mnt_ct.id,
            'currency_account': cur_db.id,
            'user': uid,
            'move1_id':am_one_id,
            'move2_id':am_two_id,
            'move3_id':am_three_id,
            'state':'done'})
        
        return True
    
currency_exchange_purchase()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
