# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging
import time
import datetime
from datetime import datetime
from datetime import time
from openerp import SUPERUSER_ID
from openerp import tools
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools.translate import _
from bsddb.dbtables import _columns_key, _columns

class seat_order(osv.Model):
    _name = 'seat.order'
    _description = 'Seat order'
    _inherit = ['mail.thread', 'ir.needaction_mixin']

    
    _columns = {
        
        #'fon': fields.char('Fon',readonly=True,copy=False),
        'name': fields.text('Name',required=True, track_visibility='onchange'),#Хийж гүйцэтгэх ажлын нэр
        'project_id': fields.many2one('project.project','Project', track_visibility='onchange'),#Төсөл
        'customer_name': fields.many2one('res.users','Customer Name', track_visibility='onchange'),#Захиалагчийн нэр
        'customer_phone': fields.char('Customer Phone', track_visibility='onchange'),#Захиалагчийн утас
        'date_start': fields.datetime('Date start', track_visibility='onchange'),#Эхлэх огноо
        'date_end': fields.datetime('Date end', track_visibility='onchange'),#Дуусах огноо
        
        'fleet_vehicle_id': fields.many2one('fleet.vehicle','Fleet vehicle', track_visibility='onchange'),#Тээврийн хэрэгслийн нэр
        'driver_name': fields.many2one('res.users', 'Driver name', track_visibility='onchange'),#Жолоочийн нэр
        'driver_phone': fields.char('Driver phone', track_visibility='onchange'),#Жолоочийн утас
        
        'comment': fields.text('Comment', track_visibility='onchange'),#Тайлбар
        'state': fields.selection([
                    ('sent', 'Sent'),
                    ('done', 'Done'),
                    ('cancel','Cancel')
                    ], 'State', track_visibility='onchange'),
                }
    
    _defaults = {
                #'fon': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'fleet.order'),
                'state': 'sent',
                'customer_name':lambda self, cr, uid, context: uid,
                }
    
    def action_to_done(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)
        if_group_fleet_manager = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_fleet_manager')
        
        if not if_group_fleet_manager:
            if uid != obj.create_uid.id:
                if uid != obj.customer_name.id:
                    if uid != obj.driver_name.id:
                        raise osv.except_osv(_('Анхааруулга'),_("Та өөрт хамааралгүй захиалгыг дуусгах эрхгүй!!!"))
        
        self.write(cr, uid, ids, {'state': 'done'})
        return True
    def action_to_cancel(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        if_group_fleet_manager = self.pool['res.users'].has_group(cr, uid, 'ierp_contract.group_fleet_manager')
        
        if not if_group_fleet_manager:
            if uid != obj.create_uid.id:
                if uid != obj.customer_name.id:
                    if uid != obj.driver_name.id:
                        raise osv.except_osv(_('Анхааруулга'),_("Та өөрт хамааралгүй захиалгыг цуцлах эрхгүй!!!"))
        
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True
    
    def onchange_customer_name(self, cr, uid, ids, customer_name, context=None):
        res = {}
        if customer_name:
            employee_obj = self.pool.get('hr.employee')
            emp_id = employee_obj.search(cr, uid, [('user_id','=',customer_name)])
            emp = employee_obj.browse(cr, uid, emp_id)
            if emp and emp.work_phone:
                res['customer_phone'] = emp.work_phone
            else:
                user_obj = self.pool.get('res.users')
                u_id = user_obj.search(cr, uid, [('id','=',customer_name)])
                user_real = user_obj.browse(cr, uid, u_id)
                res['customer_phone'] = user_real.phone or user_real.mobile
        else:
            res['customer_phone'] = False
            
        return {'value':res}
    
    def onchange_fleet_vehicle_id(self, cr, uid, ids, fleet_vehicle_id, context=None):
        res = {}
        if fleet_vehicle_id:
            veh_obj = self.pool.get('fleet.vehicle')
            veh_id = veh_obj.search(cr, uid, [('id','=',fleet_vehicle_id)])
            veh = veh_obj.browse(cr, uid, veh_id)
            
            user_obj = self.pool.get('res.users')
            u_id = user_obj.search(cr, uid, [('partner_id','=',veh.driver_id.id)])
            user_real = user_obj.browse(cr, uid, u_id)
            
            if user_real:
                res['driver_name'] = user_real.id
            else:
                res['driver_name'] = False
        else:
            res['driver_name'] = False
            
        return {'value':res}
    
    def onchange_driver_name(self, cr, uid, ids, driver_name, context=None):
        res = {}
        if driver_name:
            employee_obj = self.pool.get('hr.employee')
            emp_id = employee_obj.search(cr, uid, [('user_id','=',driver_name)])
            emp = employee_obj.browse(cr, uid, emp_id)
            if emp and emp.work_phone:
                res['driver_phone'] = emp.work_phone
            else:
                user_obj = self.pool.get('res.users')
                u_id = user_obj.search(cr, uid, [('id','=',driver_name)])
                user_real = user_obj.browse(cr, uid, u_id)
                res['driver_phone'] = user_real.phone or user_real.mobile
        else:
            res['driver_phone'] = False
            
        return {'value':res}
    
    def create(self, cr, uid, vals, context=None):
        seat_obj = self.pool.get('seat.order')
        
        obj_date_dund_vals_start = seat_obj.search(cr, uid, [('state','=','sent'),
                                             ('fleet_vehicle_id','=',vals['fleet_vehicle_id']),
                                             ('date_start','<=',vals['date_start']),
                                             ('date_end','>',vals['date_start'])])
        print 'obj_date_dund_vals_start#############', obj_date_dund_vals_start, len(obj_date_dund_vals_start)
        obj_date_dund_vals_end = seat_obj.search(cr, uid, [('state','=','sent'),
                                             ('fleet_vehicle_id','=',vals['fleet_vehicle_id']),
                                             ('date_start','<',vals['date_end']),
                                             ('date_end','>=',vals['date_end'])])
        print 'obj_date_dund_vals_end###############', obj_date_dund_vals_end, len(obj_date_dund_vals_end)
        vals_date_dund_obj_date = seat_obj.search(cr, uid, [('state','=','sent'),
                                             ('fleet_vehicle_id','=',vals['fleet_vehicle_id']),
                                             ('date_start','>=',vals['date_start']),
                                             ('date_end','<=',vals['date_end'])])
        print 'vals_date_dund_obj_date###############', vals_date_dund_obj_date, len(vals_date_dund_obj_date)
        
        if len(obj_date_dund_vals_start) > 0:
            raise osv.except_osv(_('Анхааруулга'),_("Эхлэх огнооны заагт хамаарах захиалга тухайн автомашинд захиалсан байна. Та өөр тээврийн хэрэгсэл эсвэл өөр цаг сонгоно уу!!!"))
        if len(obj_date_dund_vals_end) > 0:
            raise osv.except_osv(_('Анхааруулга'),_("Дуусах огнооны заагт хамаарах захиалга тухайн автомашинд захиалсан байна. Та өөр тээврийн хэрэгсэл эсвэл өөр цаг сонгоно уу!!!"))
        if len(vals_date_dund_obj_date) > 0:
            raise osv.except_osv(_('Анхааруулга'),_("Сонгосон огнооны заагтай давхцах захиалга тухайн автомашинд захиалсан байна. Та өөр тээврийн хэрэгсэл эсвэл өөр цаг сонгоно уу!!!"))
        
        if vals['date_start'] > vals['date_end']:
            raise osv.except_osv(_('Анхааруулга'),_("Эхлэх огноо нь дуусах огнооноос хойш байж болохгүй!!!"))
        
        seat_id = super(seat_order, self).create(cr, uid, vals, context=context)
        
        return seat_id
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)
        seat_obj = self.pool.get('seat.order')
        
        fleet_vehicle_id = 0
        date_start = False
        date_end = False
        if 'fleet_vehicle_id' in vals or 'date_start' in vals or 'date_end' in vals:
            if 'fleet_vehicle_id' in vals:
                fleet_vehicle_id = vals['fleet_vehicle_id']
            else:
                fleet_vehicle_id = obj.fleet_vehicle_id.id
            if 'date_start' in vals:
                date_start = vals['date_start']
            else:
                date_start = obj.date_start
            if 'date_end' in vals:
                date_end = vals['date_end']
            else:
                date_end = obj.date_end
            
            
            obj_date_dund_vals_start = seat_obj.search(cr, uid, [('state','=','sent'),
                                                 ('fleet_vehicle_id','=',fleet_vehicle_id),
                                                 ('date_start','<=',date_start),
                                                 ('date_end','>',date_start)])
            obj_date_dund_vals_end = seat_obj.search(cr, uid, [('state','=','sent'),
                                                 ('fleet_vehicle_id','=',fleet_vehicle_id),
                                                 ('date_start','<',date_end),
                                                 ('date_end','>=',date_end)])
            vals_date_dund_obj_date = seat_obj.search(cr, uid, [('state','=','sent'),
                                                 ('fleet_vehicle_id','=',fleet_vehicle_id),
                                                 ('date_start','>=',date_start),
                                                 ('date_end','<=',date_end)])
            if len(obj_date_dund_vals_start) > 0:
                raise osv.except_osv(_('Анхааруулга'),_("Эхлэх огнооны заагт хамаарах захиалга тухайн автомашинд захиалсан байна. Та өөр тээврийн хэрэгсэл эсвэл өөр цаг сонгоно уу!!!"))
            if len(obj_date_dund_vals_end) > 0:
                raise osv.except_osv(_('Анхааруулга'),_("Дуусах огнооны заагт хамаарах захиалга тухайн автомашинд захиалсан байна. Та өөр тээврийн хэрэгсэл эсвэл өөр цаг сонгоно уу!!!"))
            if len(vals_date_dund_obj_date) > 0:
                raise osv.except_osv(_('Анхааруулга'),_("Сонгосон огнооны заагтай давхцах захиалга тухайн автомашинд захиалсан байна. Та өөр тээврийн хэрэгсэл эсвэл өөр цаг сонгоно уу!!!"))
            
            if date_start > date_end:
                raise osv.except_osv(_('Анхааруулга'),_("Эхлэх огноо нь дуусах огнооноос хойш байж болохгүй!!!"))
        
        if_fleet_manager = self.pool['res.users'].has_group(cr, uid, 'fleet.group_fleet_manager')
        if vals:
            if not if_fleet_manager:
                if obj.create_uid.id != uid:
                    if obj.customer_name.id != uid:
                        if obj.driver_name.id != uid:
                            raise osv.except_osv(_('Анхааруулга'),_("Та бусдын захиалсан захиалгыг засч болохгүй!!!"))
                if obj.customer_name.id != uid:
                    if obj.create_uid.id != uid:
                        if obj.driver_name.id != uid:
                            raise osv.except_osv(_('Анхааруулга'),_("Та бусдын үүсгэсэн захиалгыг засч болохгүй!!!"))
                    
        return super(seat_order, self).write(cr, uid, ids, vals, context=context)
    
    
