# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging

from openerp import SUPERUSER_ID
from openerp import tools
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools.translate import _
import time
import datetime


class hr_applicant(osv.osv):
    _inherit = 'hr.applicant'
    
    _columns = {
                'name': fields.char('Subject / Application Name', required=True),
                'surname':fields.char('Surname'),
                'last_name':fields.char('Last name'),
                'register_number':fields.char('Register number', size=10),
                'country_id': fields.many2one('res.country', 'Nationality'),
                'job_idea': fields.many2one('hr.job', 'Job idea', required=True),
                'pretend_second_job':fields.char('Pretend second job'),
                'hire_out':fields.date('Hire out'),
                'app_others':fields.char('App others'),
                'nationality':fields.char('Nationality', size=50),
                'foreign':fields.char('Foreign', size=10),
                'birthday':fields.date('Birthday', required=True),
                'place_of_birth':fields.char('Place of birth'),
                'age':fields.integer('Age', size=2),
                'gender': fields.selection([('male', 'Male'), ('female', 'Female')], 'Gender'),
                'driver_type_id': fields.many2many('driver.type','app_driver_type_hr_rel','driver_type_id','hr_applicant_id','Driver type'),
                'driver_date':fields.date('Driver date'),
                'pass_address':fields.char('Pass address'),
                'pass_address_other':fields.char('Pass address other'),
                'apartment_type': fields.selection([('state_apartment', 'State apartment'), ('organization', 'Organization'), ('rental', 'Rental'), ('own_house', 'Own house')], 'Apartment type'),
                'education_id': fields.one2many('education','hr_applicant_id','Education'),
                'training_course_id': fields.one2many('training.course','hr_applicant_id','Training course'),
                'professionalize_id': fields.one2many('professionalize','hr_applicant_id','Professionalize course'),
                'employment_id': fields.one2many('employment','hr_applicant_id','Employment'),
                'employment_description': fields.text('Employment description'),
                'hr_language_id': fields.one2many('hr.language','hr_applicant_id','Hr Language'),
                'hr_language_exam_id': fields.one2many('hr.language.exam','hr_applicant_id','Hr language exam'),
                'computer_skill_id': fields.one2many('computer.skill','hr_applicant_id','Programm name'),
                'other_proramm':fields.text('Other proramm'),
                'other_use_skill': fields.selection([('bad', 'Bad'), 
                                          ('middle', 'Middle'),
                                          ('good', 'Good'), 
                                          ('best', 'Best')], 'Other use skill'),
                'live_people_id': fields.one2many('live.people','hr_applicant_id','Live'),
                'hr_relationship_id': fields.one2many('hr.relationship','hr_applicant_id','Hr relationship'),
                'your_health': fields.text('Your health'),
                'hobby': fields.text('Hobby'),
                'notify_people_id': fields.one2many('notify.people','hr_applicant_id','Relation'),
                'hr_visit_id': fields.one2many('hr.visit','hr_applicant_id','Country'),
                'more_info': fields.text('More info'),
                'offspring': fields.text('Offspring'),
                'how_year': fields.selection([('to_one', '1'),('one_to_three','1-3'),('three_to','3++')],'How year'),
                'interface': fields.text('Interface'),
                'your_more_info': fields.text('Your more info'),
                'government_award_id': fields.one2many('government.award','hr_applicant_id','Government award'),
                'hr_your_relation_id': fields.one2many('hr.your.relation','hr_applicant_id','Your relation'),
                'punishment': fields.selection([('yes', 'Yes'), ('no', 'No')], 'Punishment'),
                'claim_job': fields.text('Claim job'),
                'claim_own': fields.text('Claim own'),
                'advantage': fields.text('Advantage'),
                'vice': fields.text('Vice'),
                'five_year': fields.text('Five year'),
                'hr_practice_id': fields.one2many('hr.practice','hr_applicant_id','Practice organization'),
                'hr_capability_id': fields.one2many('hr.capability','hr_applicant_id','Hr capability'),
                'interview_human': fields.char('Interview human'),
                'interview_position': fields.char('Interview position'),
                'interview_date': fields.date('Interview date'),
                'introduction': fields.text('Introduction'),
                'one_score': fields.selection([('1', '1'), 
                                          ('2', '2'),
                                          ('3', '3'), 
                                          ('4', '4'),
                                          ('5', '5')], 'Score'),
                'ethics': fields.text('Ethics'),
                'two_score': fields.selection([('1', '1'), 
                                          ('2', '2'),
                                          ('3', '3'), 
                                          ('4', '4'),
                                          ('5', '5')], 'Score'),
                'aspect': fields.text('Aspect'),
                'three_score': fields.selection([('1', '1'), 
                                          ('2', '2'),
                                          ('3', '3'), 
                                          ('4', '4'),
                                          ('5', '5')], 'Score'),
                'approach': fields.text('Approach'),
                'four_score': fields.selection([('1', '1'), 
                                          ('2', '2'),
                                          ('3', '3'), 
                                          ('4', '4'),
                                          ('5', '5')], 'Score'),
                'footnote': fields.text('Footnote'),
                'worker_date': fields.date('Worker date', required=True),
                'somebody':fields.boolean('Somebody'),
                'salary_expected_max':fields.float('Salary expected max'),
                'salary_proposed_max':fields.float('Salary proposed max'),
                #'salary_exp_max': fields.float('Salary expected max'),
                }
    
    
    def onchange_applicant_age(self, cr, uid, ids, birthday, context=None):
        app_age = 0
        if birthday != False:
            y_now = time.strftime('%Y-%m-%d')[0:4]
            y_born = birthday[0:4]
            app_age = int(y_now) - int(y_born)
            return {'value': {'age': app_age,}}
    
    def create_employee_from_applicant(self, cr, uid, ids, vals, context=None):
        """ Create an hr.employee from the hr.applicants """
        if context is None:
            context = {}
        hr_employee = self.pool.get('hr.employee')
        model_data = self.pool.get('ir.model.data')
        act_window = self.pool.get('ir.actions.act_window')
        emp_id = False
        for applicant in self.browse(cr, uid, ids, context=context):
            address_id = contact_name = False
            if applicant.partner_id:
                address_id = self.pool.get('res.partner').address_get(cr, uid, [applicant.partner_id.id], ['contact'])['contact']
                contact_name = self.pool.get('res.partner').name_get(cr, uid, [applicant.partner_id.id])[0][1]
            if applicant.job_id and (applicant.partner_name or contact_name):
                applicant.job_id.write({'no_of_hired_employee': applicant.job_id.no_of_hired_employee + 1}, context=context)
                create_ctx = dict(context, mail_broadcast=True)
                #Getting 'Employee code'
                #===============================================================
                # obj = self.browse(cr, uid, ids)
                # i = ''
                # create_year = 0
                # create_year = obj.worker_date[2:4]
                # cr.execute("""select employee_code from hr_employee 
                #     where substr(employee_code,1,2) = '%s'
                #     order by employee_code desc limit 1"""%create_year)
                # employee_code_info  = cr.dictfetchall()
                # if employee_code_info:
                #     for employee in employee_code_info:
                #         empii = employee['employee_code']
                #         if empii == None:
                #             i = 1
                #         else:
                #             i = int(empii[3:5]) + 1
                # else:
                #     i = 1
                # code = u'%s'%create_year+u'%s'%str(i).zfill(3)
                #===============================================================
                
                emp_id = hr_employee.create(cr, uid, {'surname': applicant.surname,
                                                      'last_name': applicant.last_name,
                                                      'name': applicant.partner_name or contact_name,
                                                      'register_number':applicant.register_number,
                                                      'own_mail':applicant.email_from,
                                                      'mobile':applicant.partner_mobile,
                                                      'home_phone':applicant.partner_phone,
                                                      'type_id':applicant.type_id.id,
                                                      'country_id':applicant.country_id.id,
                                                      'nationality':applicant.nationality,
                                                      'foreign':applicant.foreign,
                                                      'gender':applicant.gender,
                                                      'birthday':applicant.birthday,
                                                      'place_of_birth':applicant.place_of_birth,
                                                      'age':applicant.age,
                                                      'apartment_type':applicant.apartment_type,
                                                      'employment_description':applicant.employment_description,
                                                      'other_proramm':applicant.other_proramm,
                                                      'other_use_skill':applicant.other_use_skill,
                                                      'your_health':applicant.your_health,
                                                      'hobby':applicant.hobby,
                                                      'more_info':applicant.more_info,
                                                      'offspring':applicant.offspring,
                                                      'how_year':applicant.how_year,
                                                      'interface':applicant.interface,
                                                      'your_more_info':applicant.your_more_info,
                                                      'punishment':applicant.punishment,
                                                      'claim_job':applicant.claim_job,
                                                      'claim_own':applicant.claim_own,
                                                      'advantage':applicant.advantage,
                                                      'vice':applicant.vice,
                                                      'five_year':applicant.five_year,
                                                      'driver_type_id':applicant.driver_type_id,
                                                      'driver_date':applicant.driver_date,
                                                      'accession_date':applicant.worker_date,
                                                      #'education_id':applicant.education_id.ids,
                                                      'job_id': applicant.job_id.id,
                                                      'job_code':applicant.job_id.job_code,
                                                      #'job_code': applicant.job_code,
                                                      'address_home_id': address_id,
                                                      'dependence_address': applicant.pass_address,
                                                      'home_address': applicant.pass_address_other,
                                                      'department_id': applicant.department_id.id or False,
                                                      'parent_id': applicant.department_id.manager_id.id or False,
                                                      'address_id': applicant.company_id and applicant.company_id.partner_id and applicant.company_id.partner_id.id or False,
                                                      'work_email': applicant.department_id and applicant.department_id.company_id and applicant.department_id.company_id.email or False,
                                                      'work_phone': applicant.department_id and applicant.department_id.company_id and applicant.department_id.company_id.phone or False,
                                                      'somebody':applicant.somebody,
                                                      #=========================
                                                      # 'employee_code' : code,
                                                      # 'emp_pre_code': i,
                                                      #=========================
                                                      'forum': 'noncontract',
                                                     }, context=create_ctx)
                self.write(cr, uid, [applicant.id], {'emp_id': emp_id}, context=context)
                for edu in applicant.government_award_id:
                    self.pool.get('government.award').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.education_id:
                    self.pool.get('education').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.driver_type_id:
                    self.pool.get('driver.type').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.training_course_id:
                    self.pool.get('training.course').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.professionalize_id:
                    self.pool.get('professionalize').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_practice_id:
                    self.pool.get('hr.practice').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_capability_id:
                    self.pool.get('hr.capability').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.employment_id:
                    self.pool.get('employment').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_language_id:
                    self.pool.get('hr.language').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_language_exam_id:
                    self.pool.get('hr.language.exam').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.computer_skill_id:
                    self.pool.get('computer.skill').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.live_people_id:
                    self.pool.get('live.people').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_relationship_id:
                    self.pool.get('hr.relationship').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.notify_people_id:
                    self.pool.get('notify.people').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_visit_id:
                    self.pool.get('hr.visit').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                for edu in applicant.hr_your_relation_id:
                    self.pool.get('hr.your.relation').write(cr,uid, edu.id,{'hr_employee_id':emp_id})
                self.pool['hr.job'].message_post(
                    cr, uid, [applicant.job_id.id],
                    body=_('New Employee %s Hired') % applicant.partner_name if applicant.partner_name else applicant.name,
                    subtype="hr_recruitment.mt_job_applicant_hired", context=context)
            else:
                raise osv.except_osv(_('Warning!'), _('You must define an Applied Job and a Contact Name for this applicant.'))

        action_model, action_id = model_data.get_object_reference(cr, uid, 'hr', 'open_view_employee_list')
        dict_act_window = act_window.read(cr, uid, [action_id], [])[0]
        if emp_id:
            dict_act_window['res_id'] = emp_id
        dict_act_window['view_mode'] = 'form,tree'
        return dict_act_window
    
    def onchange_availability(self, cr, uid, ids, hire_out, context=None):
        
        variance_day = 0.0
        now_date=time.strftime('%Y-%m-%d')
        day = '%Y-%m-%d'
        last_date = hire_out 
        
        if hire_out != False:
            a = datetime.datetime.strptime(now_date, day)
            
            b = datetime.datetime.strptime(last_date, day)
            
            timedelta = b - a
            
        
            variance_day=timedelta.days
        
            return {'value': {'availability': variance_day,}}
    


    #Gorilogchiin medeeleltei holbootoi unlink-vvd
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        for line in obj.education_id:
            line_obj = self.pool.get('education')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.training_course_id:
            line_obj = self.pool.get('training.course')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.professionalize_id:
            line_obj = self.pool.get('professionalize')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_practice_id:
            line_obj = self.pool.get('hr.practice')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.employment_id:
            line_obj = self.pool.get('employment')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_language_id:
            line_obj = self.pool.get('hr.language')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_language_exam_id:
            line_obj = self.pool.get('hr.language.exam')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.computer_skill_id:
            line_obj = self.pool.get('computer.skill')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.live_people_id:
            line_obj = self.pool.get('live.people')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_relationship_id:
            line_obj = self.pool.get('hr.relationship')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.notify_people_id:
            line_obj = self.pool.get('notify.people')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_capability_id:
            line_obj = self.pool.get('hr.capability')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_visit_id:
            line_obj = self.pool.get('hr.visit')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.government_award_id:
            line_obj = self.pool.get('government.award')
            line_obj.unlink(cr, uid, line.id)
        
        for line in obj.hr_your_relation_id:
            line_obj = self.pool.get('hr.your.relation')
            line_obj.unlink(cr, uid, line.id)
            
        return super(hr_applicant, self).unlink(cr, uid, ids, context=context)
    
    
    
class hr_practice(osv.osv):
    _name = 'hr.practice'
    _columns = {
               'name':fields.char('Practice organization'),
               'practice_department':fields.char('Practice department'),
               'practice_start':fields.date('Practice start'),
               'practice_end':fields.date('Practice end'),
               'practice_date':fields.char('Practice date'),
               'administrator_name':fields.char('Administrator name'),
               'administrator_phone':fields.char('Administrator phone'),
               'course_name':fields.char('Course name'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_capability(osv.osv):
    _name = 'hr.capability'
    _columns = {
               'name':fields.char('Capability'),
               'cap_type':fields.char('Cap type'),
               'studied_time':fields.char('Studied time'),
               'rank':fields.char('Rank'),
               'achievement':fields.char('Achievement'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    