# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging

from openerp import SUPERUSER_ID
from openerp import tools, api
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import date, timedelta, datetime
import time
from calendar import month, day_abbr, day_name
from dateutil import rrule
import math
from operator import countOf
import pyperclip
from openerp import models

class resource_resource_inherit(osv.osv):
    _inherit = 'resource.resource'
    _columns = {
        'company_id' : fields.many2one('res.company', 'Company', track_visibility='onchange'),
    }

class resource_calendar_inherit(osv.osv):
    _inherit = 'resource.calendar'
    _columns = {
        'must_saturday_overtime' : fields.boolean('Бямбад илүү цаг байх ёстой'),
    }

class hr_employee(osv.osv):
    _inherit = 'hr.employee'
    
    def name_get(self, cr, uid, ids, context=None):
        if isinstance(ids, (list, tuple)) and not len(ids):
            return []
        if isinstance(ids, (long, int)):
            ids = [ids]
        reads = self.read(cr, uid, ids, ['name','employee_code'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['employee_code']:
                name = name +' / '+ record['employee_code']
            res.append((record['id'], name))
        return res

    
    
    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=100):
        if args is None:
            args = []
        if context is None:
            context = {}
        ids = []
        if name:
            ids = self.search(cr, uid, [('name', 'ilike', name)] + args, limit=limit)
        if not ids:
            ids = self.search(cr, uid, [('employee_code', 'ilike', name)] + args, limit=limit)
        return self.name_get(cr, uid, ids, context=context)
    
    def _get_current_report_type(self, cr, uid, ids, name, arg, context=None):
        res = {}
        financial_report_ref = self._get_financial_report_ref(cr, uid, context=context)
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = 'none'
            for key, financial_report in financial_report_ref.items():
                list_ids = [x.id for x in financial_report.account_type_ids]
                if record.id in list_ids:
                    res[record.id] = key
        return res
    
    #===========================================================================
    # def _working_year_others(self, cr, uid, ids, field_name, args, context=None ):
    #     res = {}
    #     #return res
    #     line_month = 0.0
    #     line_year = 0.0
    #     working_year_others = 0.0
    #     for record in self.browse(cr, uid, ids, context=context):
    #         if record.employment_id:
    #             for emp in record.employment_id:
    #                 print emp
    #                 if emp.is_ndd == True:
    #                     a = int(emp.org_finish.name[3:7])
    #                     b = int(emp.org_start.name[3:7])
    #                     c = int(emp.org_finish.name[0:2])
    #                     d = int(emp.org_start.name[0:2])
    #                     if a==b and c>d:
    #                         line_month = c-d
    #                     elif a>b and c>d:
    #                         line_year = (a - b)*12
    #                         line_month = line_year + (c-d)
    #                     elif a>b and c<d:
    #                         line_year = (a - b -1)*12
    #                         line_month = line_year + (c + 12 - d)
    #                     elif a==b and c==d:
    #                         line_month = 1
    #                     elif a>b and c==d:
    #                         line_month = (a - b)*12
    #                     else:
    #                         line_month = 0
    #                 working_year_others += line_month
    #         else:
    #             working_year_others = 0.0
    #             
    #         res[record.id] = working_year_others
    #     print 'workingyearothers', res
    #     return res
    #===========================================================================
    
    def compute_all_wyo(self, cr, uid, context=None):
        #    wyo = working_year_other2
        if context is None:
            context = {}
        res = {}
        emp_obj = self.pool.get('hr.employee')
        emp_ids = emp_obj.search(cr, uid, [('active','=',True)])
        for pr_emps in emp_ids:
            emp_ids = emp_obj.working_year_other2(cr, uid, pr_emps)
        return True
    
    def working_year_other2(self, cr, uid,ids, context=None ):
        res = {}
        line_month = 0.0
        line_year = 0.0
        working_year_others = 0.0
        
        wyo = 0
        wyo_other = 0
        
        for record in self.browse(cr, uid,ids, context=None):
            if record.employment_id:
                for emp in record.employment_id:
                    if emp.is_ndd == True:
                        a = int(emp.org_finish.name[3:7])
                        b = int(emp.org_start.name[3:7])
                        c = int(emp.org_finish.name[0:2])
                        d = int(emp.org_start.name[0:2])
                        if a==b and c>d:
                            line_month = c-d + 1
                        elif a>b and c>d:
                            line_year = (a - b)*12
                            line_month = line_year + (c-d) + 1
                        elif a>b and c<d:
                            line_year = (a - b -1)*12
                            line_month = line_year + (c + 12 - d) + 1
                        elif a==b and c==d:
                            line_month = 1
                        elif a>b and c==d:
                            line_month = (a - b)*12 + 1
                        else:
                            line_month = 0
                    else:
                        line_month = 0.0
                    
                    working_year_others += line_month
        if int(working_year_others/12) > 0:
            wyo = int(working_year_others/12)
            wyo_other = working_year_others - int(working_year_others/12)*12
        elif int(working_year_others/12) == 0:
            wyo = 0
            wyo_other = working_year_others
        self.write(cr, uid,ids, {'working_year_other1': wyo,
                                 'working_year_other1_month': wyo_other}, context=None)
        return True
    
    
    #===========================================================================
    # def _get_seniority(self, cr, uid, ids, name, arg, context=None ):
    #     res = {}
    #     #return res
    #     line_month = 0.0
    #     line_year = 0.0
    #     ndd_month = 0.0
    #     for record in self.browse(cr, uid, ids, context=context)[0]:
    #         for emp in record.employment_id:
    #             if emp.is_ndd == True:
    #                 line_year = (int(emp.org_finish.name[3:7]) - int(emp.org_start.name[3:7]))*12
    #                 line_month += int(emp.org_finish.name[0:2]) - int(emp.org_start.name[0:2]) + line_year
    #                 
    #         if line_month == 0.0:
    #             line_month = 1.0
    #             ndd_month = line_month+record.working_year
    #             print 'poooooooooooo', ndd_month
    #         else:
    #             ndd_month = line_month+record.working_year+1
    #             print 'peeeeeeeeeee', ndd_month
    #         res[record.id] = ndd_month
    #         self.write(cr, uid, ids, {'forum': 'employee'})
    #     print 'tooooooooob', res
    #     return res
    #===========================================================================
    
    def compute_all_gs(self, cr, uid, context=None):
        #    gs = get_seniority2
        if context is None:
            context = {}
        res = {}
        emp_obj = self.pool.get('hr.employee')
        emp_ids = emp_obj.search(cr, uid, [('active','=',True)])
        for pr_emps in emp_ids:
            emp_ids = emp_obj.get_seniority2(cr, uid, pr_emps)
        return True
    
    def get_seniority2(self, cr, uid, ids, context=None ):
        res = {}
        obj = self.browse(cr, uid,ids)[0]
        #return res
        line_month = 0.0
        line_year = 0.0
        employment_month = 0
        our_month = 0
        ndd_month = 0.0
        
        sy = 0
        sy_other = 0
        
        #=======================================================================
        # for record in self.browse(cr, uid, ids, context=context)[0]:
        #     for emp in record.employment_id:
        #         if emp.is_ndd == True:
        #             line_year = (int(emp.org_finish.name[3:7]) - int(emp.org_start.name[3:7]))*12
        #             line_month += int(emp.org_finish.name[0:2]) - int(emp.org_start.name[0:2]) + line_year + 1
        #=======================================================================
                
                #line_month += line_month
            #employment_month = line_month+record.working_year
        employment_month = obj.working_year_other1 * 12 + obj.working_year_other1_month
        our_month = obj.working_year * 12 + obj.working_year_month
        ndd_month = employment_month + our_month
            
        if int(ndd_month/12) > 0:
            sy = int(ndd_month/12)
            sy_other = ndd_month - int(ndd_month/12)*12
        elif int(ndd_month/12) == 0:
            sy = 0
            sy_other = ndd_month
        self.write(cr, uid,ids, {'seniority': sy,
                                 'seniority_month': sy_other})
            #===================================================================
            # if line_month == 0.0:
            #     line_month = 1.0
            #     ndd_month = line_month+record.working_year
            # else:
            #     ndd_month = line_month+record.working_year
            #===================================================================
            #res[record.id] = ndd_month
        #self.write(cr, uid, ids, {'seniority': ndd_month})
        return True
    
    
    
    def _other_contract(self, cr, uid, ids, field_name, arg, context=None):
        res={}
        task_bom = self.pool.get('other.contract')
        for boms in ids:
            nb = task_bom.search_count(cr, uid, [('hr_employee_id', '=', boms)], context=context)
            res[boms] = nb
        print '_other_contract'
        return res
    
    def _labour_movement(self, cr, uid, ids, field_name, arg, context=None):
        res={}
        task_bom = self.pool.get('labour.movement')
        for boms in ids:
            nb = task_bom.search_count(cr, uid, [('hr_employee_id', '=', boms)], context=context)
            res[boms] = nb
        print '_labour_movement'
        return res
    
    def _award_list(self, cr, uid, ids, field_name, arg, context=None):
        res={}
        task_bom = self.pool.get('award.list')
        for boms in ids:
            nb = task_bom.search_count(cr, uid, [('hr_employee_id', '=', boms)], context=context)
            res[boms] = nb
        print '_award_list'
        return res
    
    #===========================================================================
    # def _autochange_age(self, cr, uid, ids,name, args, context = None):
    #     obj = self.browse(cr, uid,ids)[0]
    #     now_age = 0
    #     year = 0
    #     month = 0
    #     day = 0
    #     year1 = 0
    #     month1 = 0
    #     day1 = 0
    #     diff_year = 0
    #     diff_month = 0
    #     diff_day = 0
    #     #return {}
    #     if  obj.birthday:
    #         year = datetime.now().strftime('%Y-%m-%d')[0:4]
    #         year1 = obj.birthday[0:4]
    #         month = datetime.now().strftime('%Y-%m-%d')[5:7]
    #         month1 = obj.birthday[5:7]
    #         day = datetime.now().strftime('%Y-%m-%d')[8:10]
    #         day1 = obj.birthday[8:10]
    #         diff_year = int(year) - int(year1)
    #         diff_month = int(month) - int(month1)
    #         diff_day = int(day) - int(day1)
    #         if diff_month > 0:
    #             now_age = diff_year
    #         elif diff_month == 0:
    #             if  diff_day >= 0:
    #                 now_age = diff_year
    #             else:
    #                 now_age = diff_year -1
    #         else:
    #             now_age = diff_year -1
    #     print 'autochangeage', now_age
    #     return {ids[0]:now_age}
    #===========================================================================
    
    def compute_all_aa(self, cr, uid, context=None):
        #    aa = autochange_age
        if context is None:
            context = {}
        res = {}
        emp_obj = self.pool.get('hr.employee')
        emp_ids = emp_obj.search(cr, uid, [('active','=',True)])
        for pr_emps in emp_ids:
            emp_ids = emp_obj.autochange_age(cr, uid, pr_emps)
        return True
    
    def autochange_age(self, cr, uid, ids=None,context={}):
        res = {}
        obj = self.browse(cr, uid,ids)
        now_age = 0
        year = 0
        month = 0
        day = 0
        year1 = 0
        month1 = 0
        day1 = 0
        diff_year = 0
        diff_month = 0
        diff_day = 0
        #return {}
        if  obj.birthday:
            year = datetime.now().strftime('%Y-%m-%d')[0:4]
            year1 = obj.birthday[0:4]
            month = datetime.now().strftime('%Y-%m-%d')[5:7]
            month1 = obj.birthday[5:7]
            day = datetime.now().strftime('%Y-%m-%d')[8:10]
            day1 = obj.birthday[8:10]
            diff_year = int(year) - int(year1)
            diff_month = int(month) - int(month1)
            diff_day = int(day) - int(day1)
            if diff_month > 0:
                now_age = diff_year
            elif diff_month == 0:
                if  diff_day >= 0:
                    now_age = diff_year
                else:
                    now_age = diff_year -1
            else:
                now_age = diff_year -1
        else:
            pass
        self.write(cr, uid,obj.id, {'employee_age': now_age})
        return True
    
    
    def _get_latest_contract(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        #obj_contract = self.pool.get('hr.contract')
        for emp in self.browse(cr, uid, ids, context=context):
            #contract_ids = obj_contract.search(cr, uid, [('employee_id','=',emp.id) and ('state','=','approve')], context=context)
            #contract_ids = obj_contract.search(cr, uid, [('employee_id','=',emp.id),], order='date_start', context=context)
            cr.execute("""
                    select
                            hct.contract_type_code, hct.name, hc.id as hcid
                    from    hr_contract as hc,
                            hr_contract_type as hct
                    where   hc.employee_id = %s and
                            hct.id = hc.type_id and
                            hc.state = 'approve'"""%emp.id)
            type_code  = cr.dictfetchall()
            if type_code:
                for line in type_code:
                    if line:
                        res[emp.id] = line['hcid']
                        #res[emp.id] = contract_ids[-1:][0]
                        if line['contract_type_code'] == '10':
                            #res[emp.forum] = 'employee'
                            self.write(cr, uid, ids, {'forum': 'employee'})
                        elif line['contract_type_code'] == '20':
                            #res[emp.forum] = 'worker'
                            self.write(cr, uid, ids, {'forum': 'worker'})
                        elif line['contract_type_code'] == '30':
                            #res[emp.forum] = 'subcontractor'
                            self.write(cr, uid, ids, {'forum': 'subcontractor'})
                        else:
                            self.write(cr, uid, ids, {'forum': 'others'})
                    else:
                        self.write(cr, uid, ids, {'forum': 'noncontract'})
                        res[emp.id] = False
            else:
                self.write(cr, uid, ids, {'forum': 'noncontract'})
                res[emp.id] = False
                
        return res
    #===========================================================================
    # def _check_permissions(self, cr, uid, ids, field_name, arg, context):
    #     res = {}
    # 
    #     for i in ids:
    #         if not i:
    #             continue
    # 
    #     # Get the HR Officer/Manager id's
    #     group_obj = self.pool.get('res.groups')
    #     manager_ids = group_obj.search(cr, uid, ['|',('name','=', 'Human Resources / HR Officer'),('name','=', 'Human Resources / HR Manager')])
    # 
    #     # Get the user and see what groups he/she is in
    #     user_obj = self.pool.get('res.users')
    #     user = user_obj.browse(cr, uid, uid, context=context)
    # 
    #     group_ids = []
    #     for grp in user.groups_id:
    #         group_ids.append(grp.id)
    # 
    #     if (manager_ids[0] in group_ids) or (manager_ids[1] in group_ids):
    #         res[i] ='Manager'
    #     else:
    #         res[i] = 'User'
    # 
    #     return res
    #===========================================================================
    
    _columns = {
                'surname':fields.char('Surname'),
                'last_name':fields.char('Last name'),
                'register_number':fields.char('Register number', size=10, write = False),
                'foreign':fields.char('Foreign', size=10),
                'nationality':fields.char('Nationality', size=50),
                #'age':fields.function(_autochange_age, store=True, type='integer',string="Autochange age", update=False, recompute=False),
                'employee_age':fields.integer('Age'),
                'employee_code':fields.char('Employee code', size=7, 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'home_address':fields.char('Home address'),
                'dependence_address':fields.char('Dependence address'),
                'home_phone':fields.char('Home phone'),
                'mobile':fields.char('Mobile', size=8),
                'own_mail':fields.char('Own mail'),
                'several_name':fields.char('Several name'),
                'several_phone':fields.char('Several phone'),
                'work_address':fields.char('Work address'),
                'ndd':fields.char('NDD'),
                'emdd':fields.char('EMDD'),
                'driver_type_id': fields.many2many('driver.type','emp_driver_type_hr_rel','driver_type_id','hr_employee_id','Driver type'),
                #'sub_manager_id': fields.many2many('hr.department','department_hr_rel','sub_manager_id','sub_manager_ids','Sub managers'),
                'driver_date':fields.date('Driver date'),
                'education_id': fields.one2many('education','hr_employee_id','Education'),
                'training_course_id': fields.one2many('training.course','hr_employee_id','Training course'),
                'professionalize_id': fields.one2many('professionalize','hr_employee_id','Professionalize course'),
                'employment_id': fields.one2many('employment','hr_employee_id','Employment'),
                'hr_practice_id': fields.one2many('hr.practice','hr_employee_id','Practice organization'),
                'hr_capability_id': fields.one2many('hr.capability','hr_employee_id','Hr capability'),
                'employment_description': fields.text('Employment description'),
                'hr_language_id': fields.one2many('hr.language','hr_employee_id','Hr Language'),
                'hr_language_exam_id': fields.one2many('hr.language.exam','hr_employee_id','Hr language exam'),
                'computer_skill_id': fields.one2many('computer.skill','hr_employee_id','Programm name'),
                'other_proramm':fields.text('Other proramm'),
                'other_use_skill': fields.selection([('bad', 'Bad'), 
                                          ('middle', 'Middle'),
                                          ('good', 'Good'), 
                                          ('best', 'Best')], 'Other use skill'),
                'hr_relationship_id': fields.one2many('hr.relationship','hr_employee_id','Hr relationship'),
                'live_people_id': fields.one2many('live.people','hr_employee_id','Live'),
                'type_id': fields.many2one('hr.recruitment.degree', 'Degree'),
                'profession_id': fields.many2one('basis.profession', 'Profession name'),
                'att_code': fields.integer('Attendance code', eval="[(6, 0, [ref('base.group_user') ])]",),
                #'in_worked_year': fields.period('In worked year'),
                'apartment_type': fields.selection([('state_apartment', 'Relatives'), ('organization', 'Fatherly'), ('rental', 'Rental'), ('own_house', 'Own house')], 'Apartment type'),
                'job_grade_id': fields.many2one('job.grade', 'Job grade'),
                'salary_grade_id': fields.many2one('salary.grade', 'Salary grade'),
                'family_size':fields.integer('Family size', size=10),
                'notify_people_id': fields.one2many('notify.people','hr_employee_id','Relation'),
                'hr_visit_id': fields.one2many('hr.visit','hr_employee_id','Country'),
                'your_health': fields.text('Your health'),
                'hobby': fields.text('Hobby'),
                'more_info': fields.text('More info'),
                'offspring': fields.text('Offspring'),
                'how_year': fields.selection([('to_one', '1'),('one_to_three','1-3'),('three_to','3++')],'How year'),
                'interface': fields.text('Interface'),
                'your_more_info': fields.text('Your more info'),
                'government_award_id': fields.one2many('government.award','hr_employee_id','Government award'),
                'hr_your_relation_id': fields.one2many('hr.your.relation','hr_employee_id','Your relation'),
                'punishment': fields.selection([('yes', 'Yes'), ('no', 'No')], 'Punishment'),
                'claim_job': fields.text('Claim job'),
                'claim_own': fields.text('Claim own'),
                'advantage': fields.text('Advantage'),
                'vice': fields.text('Vice'),
                'five_year': fields.text('Five year'),
                'accession_date': fields.date('Accession date', required=True, 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'working_year': fields.integer('Working year'),
                'working_year_month': fields.integer('Working year month'),
                'out_reason_id': fields.many2one('out.reason', 'Out reason', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'decree_id': fields.many2one('ir.attachment', 'Decree id', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'decree_number': fields.integer('Decree number', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'decree_date': fields.date('Decree date', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'left_date': fields.date('Left date', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                #'working_year': fields.function(_compute_working_year, string='Working year',
                                    #multi='working_year', type="integer",
                                    #store={'hr.employee': (lambda self, cr, uid, ids, c={}: ids, ['working_year'], 10)}),
                #'working_year_others': fields.function(_working_year_others,type = 'integer',string='Working year others'),
                'working_year_other1':fields.integer('Working year other'),
                'working_year_other1_month': fields.integer('Working year other1 month'),
                #'seniority': fields.function(_get_seniority,type = 'integer',string='Seniority'),
                'seniority': fields.integer(string='Seniority'),
                'seniority_month': fields.integer(string='Seniority month'),
                'mobil_limit': fields.float('Mobil limit', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'drive_limit': fields.float('Drive limit', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'other_contract': fields.function(_other_contract, type='integer',string="Other contract"),
                'other_contract_ids':fields.one2many('other.contract', 'hr_employee_id', type='integer',string="Other contract"),
                'labour_movement': fields.function(_labour_movement, type='integer',string="Labour movement",),
                'labour_movement_ids':fields.one2many('labour.movement', 'hr_employee_id', string="Labour movement", type='integer'),
                'award_list': fields.function(_award_list, type='integer',string="Award list",),
                'award_list_ids':fields.one2many('award.list', 'hr_employee_id', string="Award list", type='integer'),
                'is_giving':fields.boolean('Is giving', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'giving':fields.text('Giving', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'birthday':fields.date('Birthday'),
                'job_id':fields.many2one('hr.job','Job id', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'job_code':fields.integer('Job code', 
                                          read=['base.group_user'], 
                                          write=['base.group_hr_user']),
                'somebody':fields.boolean('Somebody'),
                'emp_pre_code':fields.integer('PreCode'),
                'resource_calendar_id':fields.many2one('resource.calendar', 'Resource calendar'),
                'contract_id': fields.function(_get_latest_contract, string='Contract', type='many2one', relation="hr.contract", help='Latest contract of the employee'),
                #'autochange_age': fields.function(_autochange_age, type='integer',string="Autochange age",),
                #'permissions': fields.function(_check_permissions, type='char', method=True, string="Permissions"),
                'forum': fields.selection([
                    ('noncontract', 'Noncontract'),
                    ('others', 'Others'),
                    ('subcontractor', 'Subcontractor'),
                    ('worker', 'Worker'),
                    ('employee', 'Employee'),
                    ], 'Forum', 
                      read=['base.group_user'], 
                      write=['base.group_hr_user'])
                }
    _defaults = {
                'forum': 'noncontract',
                }
    #===========================================================================
    # _sql_constraints = [
    #     ('att_code', 'unique (att_code)', u' Төхөөрөмжийн код давхцаж байна!')
    # ]
    #===========================================================================
    
    #===========================================================================
    # @api.model
    # def update_ages(self):
    #     """Updates age field for all partners once a day"""
    #     for rec in self.env['hr.employee'].search([]):
    #         if rec.birthday:
    #             d1 = datetime.strptime(rec.birthday, "%Y-%m-%d").date()
    #             d2 = date.today()
    #             rec.age = relativedelta(d2, d1).years
    #===========================================================================
    
    def onchange_decree_id(self, cr, uid, ids, decree_id, context=None):
        res = {}
        if decree_id != None:
            decr_obj = self.pool.get('ir.attachment')
            decr_id = decr_obj.search(cr, uid, [('id','=',decree_id)])
            decr = decr_obj.browse(cr, uid, decr_id)
            res['decree_number']=decr.doc_number
            res['decree_date']=decr.doc_date
            return {'value':res}
        
    #===========================================================================
    # def onchange_employee_age(self, cr, uid, ids, birthday, context=None):
    #     emp_age = 0
    #     if birthday != False:
    #         y_now = time.strftime('%Y-%m-%d')[0:4]
    #         y_born = birthday[0:4]
    #         emp_age = int(y_now) - int(y_born)
    #         return {'value': {'age': emp_age,}}
    #===========================================================================
    
    def _get_tasks(self, cr, uid, ids, context=None):
        tasks = []
        for task in self.browse(cr, uid, ids, context=None):
            tasks += [x.id for x in task.other_contract_ids]
        return tasks
    def _get_movements(self, cr, uid, ids, context=None):
        movements = []
        for movement in self.browse(cr, uid, ids, context=None):
            movements += [x.id for x in movement.labour_movement_ids]
        return movements
    def _get_awards(self, cr, uid, ids, context=None):
        awards = []
        for award in self.browse(cr, uid, ids, context=None):
            awards += [x.id for x in award.award_list_ids]
        return awards
    def _get_act_window_dict(self, cr, uid, name, context=None):
        mod_obj = self.pool.get('ir.model.data')
        act_obj = self.pool.get('ir.actions.act_window')
        result = mod_obj.xmlid_to_res_id(cr, uid, name, raise_if_not_found=True)
        result = act_obj.read(cr, uid, [result], context=context)[0]
        return result
    
    def action_other_contract(self, cr, uid, ids, context=None):
        boms = self._get_tasks(cr, uid, ids, context=context)
        result = self._get_act_window_dict(cr, uid, 'ierp_hr.action_other_contract', context=context)
        result['domain'] = "[('id','in',[" + ','.join(map(str, boms)) + "])]"
        return result
    
    def action_labour_movement(self, cr, uid, ids, context=None):
        boms = self._get_movements(cr, uid, ids, context=context)
        result = self._get_act_window_dict(cr, uid, 'ierp_hr.action_labour_movement', context=context)
        result['domain'] = "[('id','in',[" + ','.join(map(str, boms)) + "])]"
        return result
    
    def action_award_list(self, cr, uid, ids, context=None):
        boms = self._get_awards(cr, uid, ids, context=context)
        result = self._get_act_window_dict(cr, uid, 'ierp_hr.action_award_list', context=context)
        result['domain'] = "[('id','in',[" + ','.join(map(str, boms)) + "])]"
        return result
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)
        month = 0
        create_date = 0
        now_date = 0
        year = 0
        yr = 0
        create_day = 0
        create_year = 0
        now_day = 0
        diff_day = 0
        
        #Ajiltan zasch bolohgvi talbariig zassan ved shalgah nohtsolvvd
        if_group_hr_user = self.pool['res.users'].has_group(cr, uid, 'base.group_hr_user')
        if_group_hr_manager = self.pool['res.users'].has_group(cr, uid, 'base.group_hr_manager')
        if_group_hr_headman = self.pool['res.users'].has_group(cr, uid, 'base.group_hr_headman')
            
        if 'remaining_leaves' in vals:
	    if if_group_hr_manager or if_group_hr_headman:
	    	vals['remaining_leaves'] = vals['remaining_leaves']
	    else:
            	raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Ээлжийн амралтын үлдэгдэл хоног>> -талбарыг сольж болохгүй!"))
            
        if 'accession_date' in vals:
	    if if_group_hr_manager or if_group_hr_headman:
		    create_date = vals['accession_date'][5:7]
		    create_year = vals['accession_date'][0:4]
		    create_day = vals['accession_date'][8:10]
	    else:
            	raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Ажилд орсон огноо>> -талбарыг сольж болохгүй!"))
        else:
            if obj.accession_date:
                create_day = obj.accession_date[8:10]
                create_date = obj.accession_date[5:7]
                create_year = obj.accession_date[0:4]
        now_day = datetime.now().strftime('%Y-%m-%d')[8:10]
        now_date = datetime.now().strftime('%Y-%m-%d')[5:7]
        now_year = datetime.now().strftime('%Y-%m-%d')[0:4]
        month = int(now_date)-int(create_date)
        year = int(now_year)-int(create_year)
        diff_day = int(now_day)-int(create_day)
        wy = 0
        wy_other = 0
        if year == 0:
            if month == 0:
                month = 1
            else:
                month = month
            if int(month/12) > 0:
                wy = int(month/12)
                wy_other = month - int(month/12)*12
            elif int(month/12) == 0:
                wy = 0
                wy_other = month
            vals.update({'working_year':wy,
                         'working_year_month':wy_other})
        else:
            yr = year*12
            if month == 0 and year==1:
                month = yr+1
            elif month == 0 and year ==0:
                month = 1
            else:
                if diff_day > 0:
                    month = month + yr+1
                else:
                    month = month + yr
            if int(month/12) > 0:
                wy = int(month/12)
                wy_other = month - int(month/12)*12
            elif int(month/12) == 0:
                wy = 0
                wy_other = month
            vals.update({'working_year':wy,
                         'working_year_month':wy_other})
        line_year = 0
        line_month = 0
        ndd_month = 0
        for emp in obj.employment_id:
            if emp != []:
                if emp.is_ndd == True:
                    line_year = (int(emp.org_finish.name[3:7]) - int(emp.org_start.name[3:7]))*12
                    line_month += int(emp.org_finish.name[0:2]) - int(emp.org_start.name[0:2]) + line_year
                    
        if line_month == 0:
            line_month = 1
            if 'working_year' in vals:
                ndd_month = line_month+vals['working_year']
        else:
            line_month = line_month
            if 'working_year' in vals:
                ndd_month = line_month+vals['working_year']
        
            #Bvh medeelel heseg(Public Information page)
        if 'company_id' in vals:
            if vals['company_id'] != obj.company_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['company_id'] = vals['company_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Компани>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        if 'user_id' in vals:
            if vals['user_id'] != obj.user_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['user_id'] = vals['user_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Холбоотой хэрэглэгч>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        if 'department_id' in vals:
            if vals['department_id'] != obj.department_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['department_id'] = vals['department_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Хэлтэс>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        if 'job_id' in vals:
            if vals['job_id'] != obj.job_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['job_id'] = vals['job_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Ажлын нэр>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
	else:
	    vals['job_id'] = obj.job_id.id
        if 'job_grade_id' in vals:
            if vals['job_grade_id'] != obj.job_grade_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['job_grade_id'] = vals['job_grade_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Албан тушаалын зэрэглэл>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        
        if 'job_code' in vals:
            if vals['job_code'] != obj.job_code:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['job_code'] = vals['job_code']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Ажлын байрны код>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        if 'parent_id' in vals:
            if vals['parent_id'] != obj.parent_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['parent_id'] = vals['parent_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Менежер>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        if 'coach_id' in vals:
            if vals['coach_id'] != obj.coach_id.id:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['coach_id'] = vals['coach_id']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Дадлагажуулагч>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        if 'manager' in vals:
            if vals['manager'] != obj.manager:
                if if_group_hr_user or if_group_hr_manager or if_group_hr_headman:
                    vals['manager'] = vals['manager']
                else:
                    raise osv.except_osv(_("Буруу үйлдэл!"), _("Та <<Менежер эсэх>> -талбарыг сольж болохгүй, та Хүний нөөцийн албанд хандан солиулах боломжтой!"))
        
        #vals.update({'seniority':ndd_month})
         
        return super(hr_employee, self).write(cr, uid, ids, vals, context=context)
    
    def create(self, cr, uid, data, context=None):
        res ={}
        context = dict(context or {})
        if context.get("mail_broadcast"):
            context['mail_create_nolog'] = True
    #Getting 'Employee code'
        i = ''
        create_year = 0
        create_year = data['accession_date'][0:4]
        cr.execute("""select employee_code from hr_employee 
                    where substr(employee_code,1,4) = '%s'
                    order by employee_code desc limit 1"""%create_year)
        employee_code_info  = cr.dictfetchall()
        if employee_code_info:
            for employee in employee_code_info:
                empii = employee['employee_code']
                if empii == None:
                    i = 1
                else:
                    i = int(empii[4:7]) + 1
                    #print 'dddddddddd', empii[5:7], empii[4:7]
        else:
            i = 1
        code = u'%s'%create_year+u'%s'%str(i).zfill(3)
        employee_id = super(hr_employee, self).create(cr, uid, data, context=context)
        self.write(cr, uid, employee_id, {'employee_code': code})

        if context.get("mail_broadcast"):
            self._broadcast_welcome(cr, uid, employee_id, context=context)
        return employee_id

    
    def action_to_probationer(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'probationer'})
        return True

    def action_to_worker(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'worker'})
        return True
    
    def action_to_tenderer(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'tenderer'})
        return True
    
    def change_discharge(self, cr, uid, ids, *args):
        obj = self.browse(cr, uid, ids)[0]
        asset_obj = self.pool.get('account.asset.asset')
        partner_obj = self.pool.get('res.partner')
	print 'ffffffffff', obj.user_id
        if obj.user_id:
            if obj.out_reason_id:
                 pass
            else:
                 raise osv.except_osv(_('Warning!'),_("Ажлаас гарсан шалтгаанаа оруулна уу!"))
            #if obj.decree_list_id.id == False:
             #   raise osv.except_osv(_('Warning!'),_("Ажлаас гарсан тушаалыг оруулна уу!"))
            if obj.left_date:
                 pass
            else:
                 raise osv.except_osv(_('Warning!'),_("Ажлаас гарсан өдрийг оруулна уу!"))
		
            asset_id = asset_obj.search(cr, uid, [('owner_id','=',obj.user_id.partner_id.id)])
            
            if asset_id:
                raise osv.except_osv(_('Error!'),_("You must an accumulation"))
            self.write(cr, uid, ids,{'active':False})
        else:
            self.write(cr, uid, ids,{'active':False})
        #self.write(cr, uid, ids,{'job_id':obj.job_id.id})
        #self.pool.get('hr.job')._get_nbr_employees(self, cr, uid, ids, obj.job_id.name, args)
        return True
    
    def onchange_job_id(self, cr, uid, ids, job_id, context=None):
        value = {}
        if job_id !=False:
            job = self.pool.get('hr.job').browse(cr, uid, job_id)
            value['job_grade_id'] = job.job_grade_id.id
            value['job_code'] = job.job_code
        return {'value': value}
    
    #Ajiltnii medeeleltei holbootoi unlink-vvd
    #===========================================================================
    # def unlink(self, cr, uid, ids, context=None):
    #     resource_ids = []
    #     for employee in self.browse(cr, uid, ids, context=context):
    #         resource_ids.append(employee.resource_id.id)
    #         
    #         for line in employee.education_id:
    #             line_obj = self.pool.get('education')
    #             line_obj.unlink(cr, uid, line.id)
    #             
    #         for line in employee.training_course_id:
    #             line_obj = self.pool.get('training.course')
    #             line_obj.unlink(cr, uid, line.id)
    #         
    #         for line in employee.professionalize_id:    
    #             line_obj = self.pool.get('professionalize')
    #             line_obj.unlink(cr, uid, line.id)
    #         
    #         for line in employee.hr_practice_id:
    #             line_obj = self.pool.get('hr.practice')
    #             line_obj.unlink(cr, uid, line.id)
    #         
    #         for line in employee.employment_id:
    #             line_obj = self.pool.get('employment')
    #             line_obj.unlink(cr, uid, line.id)
    #             
    #         for line in employee.hr_language_id:
    #             line_obj = self.pool.get('hr.language')
    #             line_obj.unlink(cr, uid, line.id)
    #         
    #         for line in employee.hr_language_exam_id:
    #             line_obj = self.pool.get('hr.language.exam')
    #             line_obj.unlink(cr, uid, line.id)
    #         
    #         for line in employee.computer_skill_id:
    #             line_obj = self.pool.get('computer.skill')
    #             line_obj.unlink(cr, uid, line.id)
    #         
    #         for line in employee.live_people_id:
    #             line_obj = self.pool.get('live.people')
    #             line_obj.unlink(cr, uid, line.id)    
    #         
    #         for line in employee.hr_relationship_id:
    #             line_obj = self.pool.get('hr.relationship')
    #             line_obj.unlink(cr, uid, line.id)
    #     
    #         for line in employee.notify_people_id:
    #             line_obj = self.pool.get('notify.people')
    #             line_obj.unlink(cr, uid, line.id)
    #     
    #         for line in employee.hr_capability_id:
    #             line_obj = self.pool.get('hr.capability')
    #             line_obj.unlink(cr, uid, line.id)
    #     
    #         for line in employee.hr_visit_id:
    #             line_obj = self.pool.get('hr.visit')
    #             line_obj.unlink(cr, uid, line.id)
    #     
    #         for line in employee.government_award_id:
    #             line_obj = self.pool.get('government.award')
    #             line_obj.unlink(cr, uid, line.id)
    #     
    #         for line in employee.hr_your_relation_id:
    #             line_obj = self.pool.get('hr.your.relation')
    #             line_obj.unlink(cr, uid, line.id)
    #     
    #     super(hr_employee, self).unlink(cr, uid, ids, context=context)
    #     return self.pool.get('resource.resource').unlink(cr, uid, resource_ids, context=context)
    #===========================================================================
    
    
    
class hr_job(osv.osv):
    _inherit = 'hr.job'
    
    def _count_all(self, cr, uid, ids, field_name, arg, context=None):
        Applicant = self.pool['hr.applicant']
        #employee = self.pool['hr.applicant']
        return {
            job_id: {
                'application_count': Applicant.search_count(cr,uid, [('job_id', '=', job_id)], context=context),
                'documents_count': len(self._get_attached_docs(cr, uid, [job_id], field_name, arg, context=context)[job_id]),
                #'employee_count': employee.search_count(cr,uid, [('job_id', '=', job_id)], context=context)
                }
            for job_id in ids
        }
    
    _columns = {
               'job_code':fields.integer('Job code'),
               'job_grade_id':fields.many2one('job.grade','Job grade'),
               #'job_grade_id2':fields.many2one('job.grade','Job grade2'),
               'actives_id':fields.boolean('Actives'),
               #'employee_count': fields.function(_count_all, type='integer', string='Employees', multi=True),
        }

    _defaults = {
        'company_id': False,
        'address_id': False,
        'actives_id': True
        #'company_id': lambda self, cr, uid, ctx=None: self.pool.get('res.company')._company_default_get(cr, uid, 'hr.job', context=ctx),
    
    }
    
    def create(self, cr, uid, vals, context=None):
        alias_context = dict(context, alias_model_name='hr.applicant', alias_parent_model_name=self._name)
        job_id = super(hr_job, self).create(cr, uid, vals, context=alias_context)
        job = self.browse(cr, uid, job_id, context=context)
        self.pool.get('mail.alias').write(cr, uid, [job.alias_id.id], {'alias_parent_thread_id': job_id, "alias_defaults": {'job_id': job_id}}, context)
        return job_id
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)[0]
        if 'actives_id' in vals:
            if vals['actives_id'] == False:
                if obj.application_count > 0 and obj.no_of_employee > 0:
                    raise osv.except_osv(_('Ажиллагааны алдаа!'),_("Энэ албан тушаал нь <Ажилд орох өргөдлүүд> болон <Ажилчдын одоогийн тоо> -д бүртгэгдсэн байна! Ажилтан болон Горилогчоос холбоог салгаснаар идэвхгүй болгох боломжтой"))
                elif obj.application_count > 0 or obj.no_of_employee > 0:
                    raise osv.except_osv(_('Ажиллагааны алдаа!'),_("Энэ албан тушаал нь <Ажилд орох өргөдлүүд> эсвэл <Ажилчдын одоогийн тоо> -д бүртгэгдсэн байна! Ажилтан эсвэл Горилогчоос холбоог салгаснаар идэвхгүй болгох боломжтой"))
        if 'job_code' in vals:
            emp_obj = self.pool.get('hr.employee').search(cr,uid,[('job_id','=',obj.id)])
            emp_ids = self.pool.get('hr.employee').browse(cr,uid,emp_obj)
            for emps_line in emp_ids:
                self.pool.get('hr.employee').write(cr, uid, emps_line.id, {'job_code': vals['job_code']})
        if 'job_grade_id' in vals:
            emp_obj = self.pool.get('hr.employee').search(cr,uid,[('job_id','=',obj.id)])
            emp_ids = self.pool.get('hr.employee').browse(cr,uid,emp_obj)
            for emps_line in emp_ids:
                self.pool.get('hr.employee').write(cr, uid, emps_line.id, {'job_grade_id': vals['job_grade_id']})
        
        return super(hr_job, self).write(cr, uid, ids, vals, context=context)
    
    
class job_grade(osv.osv):
    _name = 'job.grade'
    _columns = {
               'name':fields.char('Job grade'),
               #'hr_job_ids':fields.one2many('hr.job','job_grade_id2','Hr job'),
               'hr_jobs_ids': fields.many2many('hr.job','job_job_grade_id','job_grade_job_rel','job_grade_id','Job grade Job'),
               
               }
    _sql_constraints = [
        ('name', 'unique (name)', u' Албан тушаалын зэрэг давхцаж болохгүй!')
    ]
    
    def create(self, cr, uid, vals, context=None):
        #alias_context = dict(context, alias_model_name='hr.applicant', alias_parent_model_name=self._name)
        job_grade_id = super(job_grade, self).create(cr, uid, vals, context=context)
        if 'hr_jobs_ids' in vals:
            for line in vals['hr_jobs_ids']:
                for line2 in line[2]:
                    #ajliin bairnii job_grade_id solih
                    job_obj = self.pool.get('hr.job').search(cr,uid,[('id','=',line2)])
                    job_id = self.pool.get('hr.job').browse(cr,uid,job_obj)
                    self.pool.get('hr.job').write(cr, uid, line2, {'job_grade_id': job_grade_id})
                    #ajiltnii job_grade_id solih
                    emp_obj = self.pool.get('hr.employee').search(cr,uid,[('active','=',True),('job_id','=',line2)])
                    emp_ids = self.pool.get('hr.employee').browse(cr,uid,emp_obj)
                    for emps_line in emp_ids:
                        self.pool.get('hr.employee').write(cr, uid, emps_line.id, {'job_grade_id': job_grade_id})
        #job = self.browse(cr, uid, job_grade_id, context=context)
        #self.pool.get('mail.alias').write(cr, uid, [job.alias_id.id], {'alias_parent_thread_id': job_id, "alias_defaults": {'job_id': job_id}}, context)
        return job_grade_id
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)[0]
        if 'hr_jobs_ids' in vals:
            for line in vals['hr_jobs_ids']:
                for line2 in line[2]:
                    #ajliin bairnii job_grade_id solih
                    job_obj = self.pool.get('hr.job').search(cr,uid,[('id','=',line2)])
                    job_id = self.pool.get('hr.job').browse(cr,uid,job_obj)
                    self.pool.get('hr.job').write(cr, uid, line2, {'job_grade_id': obj.id})
                    #ajiltnii job_grade_id solih
                    emp_obj = self.pool.get('hr.employee').search(cr,uid,[('active','=',True),('job_id','=',line2)])
                    emp_ids = self.pool.get('hr.employee').browse(cr,uid,emp_obj)
                    for emps_line in emp_ids:
                        self.pool.get('hr.employee').write(cr, uid, emps_line.id, {'job_grade_id': obj.id})
        
        return super(job_grade, self).write(cr, uid, ids, vals, context=context)
    
class driver_type(osv.osv):
    _name = 'driver.type'
    _columns = {
               'name':fields.char('Driver type', size=1),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class education(osv.osv):
    _name = 'education'
    _columns = {
               'name':fields.char('School'),
               'edu_where':fields.char('Edu where'),
               'edu_start':fields.date('Edu start'),
               'edu_finish':fields.date('Edu finish'),
               'edu_profession':fields.char('Edu profession'),
               'edu_graduation':fields.char('Edu graduation'),
               'edu_rate':fields.float('Edu rate', size=3),
               'diploma':fields.char('Diploma', size=10),
               'diploma_subject':fields.char('Diploma subject'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }

class professionalize(osv.osv):
    _name = 'professionalize'
    _columns = {
               'name':fields.char('Professionalize'),
               'pro_start':fields.date('Pro start'),
               'pro_finish':fields.date('Pro finish'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class out_reason(osv.osv):
    _name = 'out.reason'
    _columns = {
               'name':fields.char('Left reason'),
               'left_type': fields.selection([('company_decision', 'Company decision'), ('own_decision', 'Own decision')], 'Left type'),
               'left_reason_code':fields.integer('Left reason code'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee')
               }
    
class training_course(osv.osv):
    _name = 'training.course'
    _columns = {
               'name':fields.char('College'),
               'where':fields.char('Where'),
               'start_date':fields.date('Start date'),
               'finish_date':fields.date('Finish date'),
               'training_profession':fields.char('Training profession'),
               'training_grade':fields.char('Training grade'),
               'rate':fields.float('Rate', size=3),
               'license':fields.char('License', size=10),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class employment(osv.osv):
    _name = 'employment'
    _columns = {
               'name':fields.char('Organization'),
               'fiscalyear_id_in': fields.many2one('hr.fiscalyear', 'HR Fiscal Year In'),
               'fiscalyear_id_out': fields.many2one('hr.fiscalyear', 'HR Fiscal Year Out'),
               'org_start':fields.many2one('hr.period','Org start', required=True),
               'org_finish':fields.many2one('hr.period','Org finish', required=True), 
               #'org_start':fields.date('Org start', required=True),
               #'org_finish':fields.date('Org finish', required=True),
               'salary_size':fields.float('Salary size'),
               'org_position':fields.char('Org position'),
               'org_reason':fields.char('Org reason'),
               'is_ndd':fields.boolean('Is ndd'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant'),               
               'director_name':fields.char('Director name'),               
               'org_phone':fields.char('Org phone')
               }
    
    def onchange_org_start(self, cr, uid, ids, org_start, context=None):
        res = {}
        if org_start:
            fy_obj = self.pool.get('hr.period')
            fy_ids = fy_obj.search(cr, uid, [('id','=',org_start)])
            fy = fy_obj.browse(cr, uid, fy_ids)
            res['fiscalyear_id_in'] = fy.fiscalyear_id.id
        return {'value': res}
    
    def onchange_org_finish(self, cr, uid, ids, org_finish, context=None):
        res = {}
        if org_finish:
            fy_obj = self.pool.get('hr.period')
            fy_ids = fy_obj.search(cr, uid, [('id','=',org_finish)])
            fy = fy_obj.browse(cr, uid, fy_ids)
            res['fiscalyear_id_out'] = fy.fiscalyear_id.id
        return {'value': res}
        
        

class language_type(osv.osv):
    _name = 'language.type'
    _columns = {
               'name':fields.char('Language type'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_language(osv.osv):
    _name = 'hr.language'
    _columns = {
               'name':fields.many2one('language.type','Hr Language'),
               'lang_speak': fields.selection([('bad', 'Bad'), 
                                          ('middle', 'Middle'), 
                                          ('good', 'Good')], 'Speak'),
               'lang_listen': fields.selection([('bad', 'Bad'), 
                                           ('middle', 'Middle'), 
                                           ('good', 'Good')], 'Listen'),
               'lang_read': fields.selection([('bad', 'Bad'), 
                                         ('middle', 'Middle'), 
                                         ('good', 'Good')], 'Read'),
               'lang_write': fields.selection([('bad', 'Bad'), 
                                          ('middle', 'Middle'), 
                                          ('good', 'Good')], 'Write'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_language_exam(osv.osv):
    _name = 'hr.language.exam'
    _columns = {
               'name':fields.char('Hr language exam'),
               'exam_date':fields.date('Exam date'),
               'exam_score':fields.float('Exam score', size=4),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_programm(osv.osv):
    _name = 'hr.programm'
    _columns = {
               'name':fields.char('Programm type'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class computer_skill(osv.osv):
    _name = 'computer.skill'
    _columns = {
               'name':fields.many2one('hr.programm','Programm name'),
               'use_skill': fields.selection([('bad', 'Bad'), 
                                          ('middle', 'Middle'), 
                                          ('good', 'Good')], 'Use skill'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_relationship(osv.osv):
    _name = 'hr.relationship'
    _columns = {
               'name':fields.char('Hr relationship'),
               'relationship_last_name':fields.char('Relationship last name'),
               'relationship_first_name':fields.char('Relationship first name'),
               'birth':fields.date('Birth'),
               'age':fields.integer('Age', size=3),
               'work_organization':fields.char('Work organization'),
               'relationship_position':fields.char('Relationship position'),
               'relationship_contact':fields.char('Relationship contact'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
    def onchange_age(self, cr, uid, ids, birth, context=None):
        rel_age = 0
        if birth != False:
            now = time.strftime('%Y-%m-%d')[0:4]
            born = birth[0:4]
            rel_age = int(now) - int(born)
            return {'value': {'age': rel_age,}}
        
        
       
class live_people(osv.osv):
    _name = 'live.people'
    
    
    def compute_age_live_people(self, cr, uid, context=None):
        #obj = self.browse(cr, uid,id)
        #    aa = autochange_age
        if context is None:
            context = {}
        res = {}
        obj = self.pool.get('live.people')
        emp_obj = self.pool.get('hr.employee')
        emp_ids = emp_obj.search(cr, uid, [('active','=',True)])
        obj_ids = obj.search(cr, uid, [('hr_employee_id', 'in', emp_ids)])
        for pr_emps in obj_ids:
            obj_ids = obj.autochange_age_live_people(cr, uid, pr_emps)
        return True
    
    def autochange_age_live_people(self, cr, uid, ids=None,context={}):
        res = {}
        obj = self.browse(cr, uid,ids)
        now_age = 0
        year = 0
        month = 0
        day = 0
        year1 = 0
        month1 = 0
        day1 = 0
        diff_year = 0
        diff_month = 0
        diff_day = 0
        #return {}
        if  obj.live_birth:
            year = datetime.now().strftime('%Y-%m-%d')[0:4]
            year1 = obj.live_birth[0:4]
            month = datetime.now().strftime('%Y-%m-%d')[5:7]
            month1 = obj.live_birth[5:7]
            day = datetime.now().strftime('%Y-%m-%d')[8:10]
            day1 = obj.live_birth[8:10]
            diff_year = int(year) - int(year1)
            diff_month = int(month) - int(month1)
            diff_day = int(day) - int(day1)
            if diff_month > 0:
                now_age = diff_year
            elif diff_month == 0:
                if  diff_day >= 0:
                    now_age = diff_year
                else:
                    now_age = diff_year -1
            else:
                now_age = diff_year -1
        else:
            pass
        self.write(cr, uid,obj.id, {'live_age': now_age})
        return True
    
    _columns = {
               'name':fields.char('Live'),
               'live_last_name':fields.char('Live last name'),
               'live_first_name':fields.char('Live first name'),
               'live_birth':fields.date('Live birth'),
               'live_age':fields.integer('Live age', size=3),
               'gender': fields.selection([('male', 'Male'), ('female', 'Female')], 'Gender'),
               'is_child':fields.boolean('Is your child'),
               'live_work_organization':fields.char('Live work organization'),
               'live_position':fields.char('Live position'),
               'live_contact':fields.char('Live contact'),
               'explanation':fields.char('Explanation'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
    def onchange_live_age(self, cr, uid, ids, live_birth, context=None):
        age = 0
        if live_birth != False:
            now_year = time.strftime('%Y-%m-%d')[0:4]
            born_year = live_birth[0:4]
            age = int(now_year) - int(born_year)
            return {'value': {'live_age': age,}}
    
class basis_profession(osv.osv):
    _name = 'basis.profession'
    _columns = {
               'name':fields.char('Profession name'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee')
               }
    
class salary_grade(osv.osv):
    _name = 'salary.grade'
    _columns = {
               'name':fields.char('Salary grade'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee')
               }
    
class notify_people(osv.osv):
    _name = 'notify.people'
    _columns = {
               'name':fields.char('Relation'),
               'relation_last_name':fields.char('Relation last name'),
               'relation_first_name':fields.char('Relation first name'),
               'relation_organization':fields.char('Relation organization'),
               'relation_position':fields.char('Relation position'),
               'relation_phone':fields.char('Relation phone', size=10),
               'relation_when':fields.char('Relation when'),
               'e_mail':fields.char('E mail'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_visit(osv.osv):
    _name = 'hr.visit'
    _columns = {
               'name':fields.char('Country'),
               'went_date':fields.date('Went date'),
               'come_date':fields.date('Come date'),
               'went_vision':fields.char('Went vision'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
class hr_your_relation(osv.osv):
    _name = 'hr.your.relation'
    _columns = {
               'name':fields.char('Your relation'),
               #'somebody':fields.boolean('Somebody'),
               #'your_relation_name':fields.char('Your relation name'),
               'your_relation_organization':fields.char('Organization'),
               'your_relation_position':fields.char('Position'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }
    
    def onchange_relation(self, cr, uid, ids, hr_employee_id, context=None):
        res = {}
        if hr_employee_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, hr_employee_id)[0]
            res['your_relation_organization']=empl.company_id.name
            res['your_relation_position']=empl.job_id.name
            return {'value':res}
    
    
class internal_training(osv.osv):
    _name = 'internal.training'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Training subject', required=True, track_visibility='onchange'),
               'training_type':fields.many2one('training.type', 'Training type', track_visibility='onchange'),
               'inst_name':fields.many2one('res.partner', 'Training organization', track_visibility='onchange'),
               'employee_number':fields.integer('Employee number'),
               'inst_date':fields.date('Start date', track_visibility='onchange'),
               'end_date':fields.date('End date', track_visibility='onchange'),
               'inst_budget':fields.float('Training budget', track_visibility='onchange'),
               'is_plan': fields.selection([('no', 'No'), ('yeas', 'Yeas')], 'Is plan', track_visibility='onchange'),
               'authority_sign':fields.many2one('res.users','Authority sign', readonly=True),
               'training_execution_id':fields.many2one('training.execution','Training execution'),
               #'employee_line_id': fields.one2many('employee.line','internal_training_id','Hr employee'),
               'employee_line_ids': fields.many2many('hr.employee','hr_inter_tr_id','inter_tr_hr_rel','internal_training_id','EmpInterTr', track_visibility='onchange'),
               #'employee_line_id': fields.one2many('employee.line','internal_training_id','Hr employee'),
               'state': fields.selection([
                    ('draft', 'Draft'),
                    ('cancel', 'Cancel'),
                    ('approve', 'Approve'),
                    ], 'State', track_visibility='onchange'),
               'company_id':fields.many2one('res.company','Company', required=True, track_visibility='onchange'),
               }
    _defaults = {
                 'state': 'draft',
                 'authority_sign':lambda self, cr, uid, context: uid,
                 'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'internal.training', context=c),
                 }
    def action_to_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'draft'})
        #------------- for record in self.browse(cr, uid, ids, context=context):
            #-------------------------------------- if record.employee_line_ids:
                # self.message_subscribe_users(cr, uid, [record.id], user_ids=[record.employee_line_ids.user_id.id], context=context)
        #----------------- return self.write(cr, uid, ids, {'state': 'confirm'})
        return True

    def action_to_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True
    
    def action_to_approve(self, cr, uid, ids, context=None):
        #mail yawuulah - send mail
        
#===============================================================================
#         obj = self.browse(cr, uid, ids)[0]
#         empl_obj = self.pool.get('hr.employee')
# 
#         template_obj = self.pool.get('email.template')
#         group_model_id = self.pool.get('ir.model').search(cr, uid, [('model', '=', 'internal.training')])[0]
#         body_html = '''Та дотоод сургалтанд хамрагдлаа'''
#         if obj.employee_line_ids:
#             for eee in obj.employee_line_ids:
#                 empl = empl_obj.browse(cr, uid, eee.id)
#                 print 'geeeeeeh', empl.user_id.login
# 
#                 template_data = {
#                     'model_id': group_model_id,
#                     'name': 'Сургалтанд хамрагдсан мэдэгдэл!',
#                     'subject' : obj.name,
#                     'body_html': body_html,
#                     #'email_from' : '${object.user_id.email}',
#                     'type' : 'email',
#                     'email_from' : 'torbatj79@gmail.com',
#                     'email_to' : empl.user_id.login,
#                 }
# 
#                 template_id = template_obj.create(cr, uid, template_data, context=context)
#                 template_obj.send_mail(cr, uid, template_id, ids[0], force_send=True, context=context)
#         
#         #zurwas vvsgeh - create message
#         for record in self.browse(cr, uid, ids, context=context):
#             if record.employee_line_ids:
#                 for lili in record.employee_line_ids:
#                      self.message_subscribe_users(cr, uid, [record.id], user_ids=[lili.user_id.id], context=context)
#         return self.write(cr, uid, ids, {'state': 'approve'})
#===============================================================================
        self.write(cr, uid, ids, {'state': 'approve'})
        
        return True
    #===========================================================================
    # def unlink(self, cr, uid, ids, context=None):
    #     obj = self.browse(cr, uid, ids)[0]
    #     line_obj = self.pool.get('employee.line')
    #     for line in obj.employee_line_id:
    #         line_obj.unlink(cr, uid, line.id)
    #     return super(internal_training, self).unlink(cr, uid, ids, context=context)
    #===========================================================================
    #@api.multi
    def create(self, cr, uid, vals, context=None):
        j=0
        nn = 0
        res = {}
        if len(vals['employee_line_ids']):
            for line in vals['employee_line_ids']:
                for myyy in line[2]:
                    if myyy:
                        j=j+1
        vals['employee_number'] = j
        my_id = super(internal_training, self).create(cr, uid, vals, context=context)
        obj = self.browse(cr, uid, my_id, context=context)
        return my_id
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)[0]
        
        j=0
        if 'employee_line_ids' in vals:
            for line in vals['employee_line_ids']:
                for myyy in line[2]:
                    if myyy:
                        j=j+1
            vals['employee_number'] = j
        else:
            for line in obj.employee_line_ids:
                if line:
                    j=j+1
            vals['employee_number'] = j
        return super(internal_training, self).write(cr, uid, ids, vals, context=context)
    
    
    
    #===========================================================================
    # def onchange_employee_line_ids(self, cr, uid, ids, employee_line_ids, context=None):
    #     #obj = self.browse(cr, uid, ids)
    #     j=0
    #     nn = 0
    #     res = {}
    #     if len(employee_line_ids):
    #         for line in employee_line_ids:
    #             for myyy in line[2]:
    #                 if myyy:
    #                     j=j+1
    #     res['employee_number']=j
    #     #self.write(cr,uid,id,{'employee_number':j})
    #     return {'value':res}
    #===========================================================================
    
    
#===============================================================================
# class employee_line(osv.osv):
#     _name = 'employee.line'
#     _columns = {
#                'name':fields.char('Last name'),
#                'department_name':fields.many2one('hr.department','Department'),
#                'position_name':fields.many2one('hr.job','Position'),
#                'internal_training_id':fields.many2one('internal.training','Internal training'),
#                'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
#                }
#     
#     def onchange_employee_line(self, cr, uid, ids, empl_id, context=None):
#         res = {}
#         if empl_id:
#             empl_obj = self.pool.get('hr.employee')
#             empl = empl_obj.browse(cr, uid, empl_id)[0]
#             res['name']=u'%s'%empl.last_name
#             res['department_name']=empl.department_id.id
#             res['position_name']=empl.job_id.id
#             return {'value':res}
#===============================================================================
        
class training_execution(osv.osv):
    _name = 'training.execution'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.many2one('res.partner', 'Training organization', required=True, track_visibility='onchange'),
               'training_type':fields.char('Training type'),
               #'inst_name':fields.char('Training subject'),
               'employee_number':fields.integer('Employee number'),
               'involved_number':fields.integer('Involved number'),
               'inst_date':fields.date('Start date', track_visibility='onchange'),
               'end_date':fields.date('End date', track_visibility='onchange'),
               'is_plan': fields.selection([('no', 'No'), ('yeas', 'Yeas')], 'Is plan', track_visibility='onchange'),
               'inst_budget':fields.float('Training budget', track_visibility='onchange'),
               'base_expense':fields.float('Base expense', track_visibility='onchange'),
               'other_expense':fields.float('Other expense', track_visibility='onchange'),
               'present_expense':fields.float('Present expense', track_visibility='onchange'),
               'variation_cost':fields.float('Variation cost', track_visibility='onchange'),
               'internal_training_id':fields.many2one('internal.training','Training subject', required=True, track_visibility='onchange'),
               'invoice_id': fields.many2one('account.invoice','Invoice'),
               'company_id': fields.many2one('res.company','Company', track_visibility='onchange'),
               'execution_line_id': fields.many2many('hr.employee','hr_exec_id','exec_line_hr_rel','training_execution_id','EmpExecLine', track_visibility='onchange'),
               #'driver_type_id': fields.many2many('driver.type','emp_driver_type_hr_rel','driver_type_id','hr_employee_id','Driver type'),
                'state': fields.selection([
                    ('draft', 'Draft'),
                    ('invoiced', 'Invoiced'),
                    ], 'State', track_visibility='onchange'),
               }
    
    _defaults = {
                 'state': 'draft',
                 #'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'training.execution', context=c),
                 }
    
    
    
    def create(self, cr, uid, vals, context=None):
        res ={}
        context = dict(context or {})
        if context.get("mail_broadcast"):
            context['mail_create_nolog'] = True
        j=0
        if 'execution_line_id' in vals:
            for line in vals['execution_line_id']:
                for myyy in line[2]:
                    if myyy:
                        j=j+1
            vals['involved_number'] = j
        employees_id = super(training_execution, self).create(cr, uid, vals, context=context)
        obj = self.browse(cr, uid, employees_id, context=context)
        #execution_line_id = super(training_execution, self).create(cr, uid, data, context=context)
        #=======================================================================
        # cr.execute("""select el.name, el.id, el.hr_employee_id as emp_id
        #                 from internal_training as i, employee_line as el 
        #                 where i.id = el.internal_training_id and i.id = %s"""%vals['internal_training_id'])
        # employee_list_info  = cr.dictfetchall()
        # print 'qqqqqqaaaaaaaaa', employee_list_info
        # if employee_list_info:
        #     for employee in employee_list_info:
        #         print 'adadadadad', employee['emp_id'], vals['execution_line_id']
        #         self.write(cr, uid, employees_id, {'execution_line_id': (4, 0, [employee['id']])})
        # else:
        #     self.write(cr, uid, employees_id, {'execution_line_id': None})
        #=======================================================================
        

        if context.get("mail_broadcast"):
            self._broadcast_welcome(cr, uid, employees_id, context=context)
        return employees_id
    
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)[0]
        j=0
        if 'execution_line_id' in vals:
            for line in vals['execution_line_id']:
                for myyy in line[2]:
                    if myyy:
                        j=j+1
            vals['involved_number'] = j
        else:
            for line in obj.execution_line_id:
                if line:
                    j=j+1
            vals['involved_number'] = j
        return super(training_execution, self).write(cr, uid, ids, vals, context=context)
    
    
    
    
    def onchange_name(self, cr, uid, ids, training_id, context=None):
        obj = self.browse(cr, uid, ids)
        res = {}
        if training_id:
            training_obj = self.pool.get('internal.training')
            training = training_obj.browse(cr, uid, training_id)[0]
            #res['internal_training_id']=u'%s'%training.name
            res['training_type']=training.training_type.name
            res['name']=training.inst_name
            res['employee_number']=training.employee_number
            res['inst_date']=training.inst_date
            res['end_date']=training.end_date
            res['inst_budget']=training.inst_budget
            res['is_plan']=training.is_plan
            res['execution_line_id']=training.employee_line_ids
            
            
            return {'value':res}
    
    def onchange_sum(self, cr, uid, ids, other_expense, base_expense,  context=None):
       if other_expense >= 0 and base_expense >= 0:
           sum = base_expense+other_expense
           return {'value': {'present_expense': sum,}}
       
    def onchange_odd(self, cr, uid, ids, inst_budget, present_expense,  context=None):
       if inst_budget >= 0 and present_expense >= 0:
           odd = inst_budget-present_expense
           return {'value': {'variation_cost': odd,}}
       
    def action_invoice(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        invoice = self.pool.get('account.invoice')
        journal_obj = self.pool.get('account.journal')
        #par_obj = self.pool.get('fleet.vehicle')
       
        currency_id = obj.company_id.currency_id.id
        journal_ids = journal_obj.search(cr, uid, [('type', '=','purchase'),('company_id', '=', obj.company_id.id or False)], limit=1)
        #par_ids = par_obj.browse(cr, uid, par_obj.search(cr, uid, [('id','=',obj.fleet_vehicle_id.id)]))
        #fpos = par_ids.driver_id.property_account_position or False
        invoice_lines = []
             
        invoice_line = []
        invoice_line.append((0, 0, {
                                        'name': obj.internal_training_id.name,
                                        'account_id': obj.name.property_account_payable.id,
                                        #'unit_type': obj.unit_type,
                                        'price_unit': obj.present_expense,
                                        #'quantity': obj.continuity_time,
                                        #'price_subtotal': obj.total_price,
                                        
        }))
            
        sch_invoice = invoice.create(cr, uid, {
                                    'account_id': obj.name.property_account_payable.id,
                                    'type': 'in_invoice',
                                    'partner_id': obj.name.id,
                                    #'currency_id': currency_id,
                                    #'journal_id': len(journal_ids) and journal_ids[0] or False,
                                    #'date_invoice': obj.executed_time,
                                    #'fiscal_position': fpos and fpos.id,
                                    'company_id': obj.name.property_account_payable.company_id.id,
                                    'invoice_line': invoice_line,
                                    #'date_due': obj.executed_time,
                                    #'comment': obj.description,
                                     })
        self.write(cr, uid, ids, {'state': 'invoiced',
                                  'invoice_id': sch_invoice})
        return True   
    
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        line_obj = self.pool.get('execution.line')
        for line in obj.execution_line_id:
            line_obj.unlink(cr, uid, line.id)
        return super(training_execution, self).unlink(cr, uid, ids, context=context)
    
    
class execution_line(osv.osv):
    _name = 'execution.line'
    _columns = {
               'name':fields.char('Last name'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'training_execution_id':fields.many2one('training.execution','Training execution'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               }
    
    def onchange_execution_line(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=u'%s'%empl.last_name
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}
        
class training_type(osv.osv):
    _name = 'training.type'
    _columns = {
               'name':fields.char('Training type'),
               'training_code':fields.char('Training code'),
               'level': fields.selection([('basic', 'Basic'), ('medium', 'Medium'), ('best', 'Best')], 'Level'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee')
               }  
        
class government_award(osv.osv):
    _name = 'government.award'
    _columns = {
               'name':fields.char('Government award'),
               'gov_award_date':fields.date('Gov award date'),
               'gov_award_where':fields.char('Gov award where'),
               'gov_award_description':fields.char('Gov award description'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               'hr_applicant_id':fields.many2one('hr.applicant','Hr applicant')
               }

class award_list(osv.osv):
    _name = 'award.list'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Award name', required=True, track_visibility='onchange'),
               'award_type':fields.many2one('award.type','Award type'),
               'partner_id': fields.many2one('res.partner','Invoicer', track_visibility='onchange'),
               'award_last_name':fields.char('Last name'),
               'award_department':fields.many2one('hr.department','Department', ),
               'award_position':fields.many2one('hr.job','Position', ),
               'received_date':fields.date('Received date', track_visibility='onchange'),
               'is_amount':fields.boolean('Is amount'),
               'amount':fields.float('Amount', track_visibility='onchange'),
               'definition':fields.text('Definition'),
               'company_id': fields.many2one('res.company','Company'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee', track_visibility='onchange'),
               'invoice_id': fields.many2one('account.invoice','Invoice', track_visibility='onchange'),
               'state': fields.selection([
                    ('draft', 'Draft'),
                    ('invoiced', 'Invoiced'),
                    ], 'State', track_visibility='onchange'),
               }
    
    _defaults = {
                 'state': 'draft',
                 #'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'award.list', context=c),
                 }
    
    def onchange_award(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['award_last_name']=u'%s'%empl.last_name
            res['company_id']=empl.company_id.id
            res['award_department']=empl.department_id.id
            res['award_position']=empl.job_id.id
            return {'value':res}   
        
    def action_award_invoice(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        invoice = self.pool.get('account.invoice')
        journal_obj = self.pool.get('account.journal')
        #par_obj = self.pool.get('fleet.vehicle')
       
        currency_id = obj.company_id.currency_id.id
        journal_ids = journal_obj.search(cr, uid, [('type', '=','purchase'),('company_id', '=', obj.company_id.id or False)], limit=1)
        invoice_lines = []
             
        invoice_line = []
        invoice_line.append((0, 0, {
                                        'name': obj.name,
                                        'account_id': obj.partner_id.property_account_payable.id,
                                        #'unit_type': obj.unit_type,
                                        'price_unit': obj.amount,
                                        
        }))
            
        sch_invoice = invoice.create(cr, uid, {
                                    'account_id': obj.partner_id.property_account_payable.id,
                                    'type': 'in_invoice',
                                    'partner_id': obj.partner_id.id,
                                    'currency_id': currency_id,
                                    'journal_id': len(journal_ids) and journal_ids[0] or False,
                                    'company_id': obj.company_id.id,
                                    'invoice_line': invoice_line,
                                    #'date_due': obj.executed_time,
                                     })
        self.write(cr, uid, ids, {'state': 'invoiced',
                                  'invoice_id': sch_invoice})
        return True   
        
class award_type(osv.osv):
    _name = 'award.type'
    _columns = {
               'name':fields.char('Award name'),
               'award_code':fields.char('Award code'),
               'level': fields.selection([('government', 'Government'), ('international', 'International'), ('organization', 'Organization'), ('nonpolitical', 'Nonpolitical')], 'Level'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee')
               }
        
     
class hr_department(osv.osv):
    _inherit = 'hr.department'
    _order = 'sequence'
    _columns = {
                'code':fields.char('Department code'),
                'sequence': fields.integer('Sequence', help="Gives the sequence order when displaying a list of stages."),
                'sub_manager_id':fields.many2one('hr.employee','Sub manager'),
                'sub_manager_ids': fields.many2many('hr.employee','department_hr_rel','sub_manager_ids','sub_manager_id','Sub managers'),
                
               }
    _defaults = {
        'sequence': 1,
    }
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)[0]
        true_manager = 0
        emp_obj = self.pool.get('hr.employee')
        emps_id = emp_obj.search(cr, uid, [('department_id','=',obj.id)])
        emp_id = emp_obj.browse(cr, uid, emps_id)
        if 'manager_id' in vals:
            if vals['manager_id'] != False:
                for managers_id in emp_id:
                    #dep_obj = self.pool.get('hr.employee')
                    #deps_id = dep_obj.search(cr, uid, [('id','=',vals['manager_id'])])
                    #dep_id = dep_obj.browse(cr, uid, deps_id)
                    #songoson menejeriin heltes taarch bwal ajiltan bvrtgeliin medeeleld parent_id, manager -g shinechleh
                    #if dep_id.department_id.id == obj.id:
                    if managers_id.id == vals['manager_id']:
                        true_manager = 1
                    else:
                        true_manager = 0
                    emp_obj.write(cr, uid, managers_id.id, {'parent_id':vals['manager_id'],
                                                            'manager':true_manager,})
                        #emp_obj.onchange_department_id(self, cr, uid, ids, managers_id.department_id, context=None)
                    #songoson menejeriin heltes zorj bwal hiine
                    #else:
                    #    raise osv.except_osv(_('Өгөгдөл бөглөлтийн алдаа!'),_("Менежер хэсэгт зөвхөн энэ хэлтсийн ажилтнуудаас сонгох боломжтой!!!"))
            else:
                for managers_id in emp_id:
                    emp_obj.write(cr, uid, managers_id.id, {'parent_id':None,
                                                            'manager':False,})
        return super(hr_department, self).write(cr, uid, ids, vals, context=context)
    
class hr_separatee(osv.osv):
    _inherit = 'hr.employee'
    _columns = {
                'out_reason_id':fields.many2one('out.reason','Out reason'),
                'is_giving':fields.boolean('Is giving')
                }

class hr_contract_type_inherit(osv.osv):
    _inherit = 'hr.contract.type'
    _description = 'Contract Type Inherit'
    _columns = {
                'contract_type_code': fields.char('Contract type code', required = True, size = 2),
    }

class hr_contract(osv.osv):
    _inherit = 'hr.contract'
    
    
    #===========================================================================
    # def _contract_expiration(self, cr, uid, ids,name, args, context = None):
    #     res = {}
    #     obj = self.browse(cr, uid,ids)[0]
    #     delta_day = 0
    #     day = '%Y-%m-%d'
    #     today = datetime.today().strftime('%Y-%m-%d')
    #     a = datetime.strptime(obj.date_end, day)
    #     b = datetime.strptime(today, day)
    #     timedelta = a - b
    #     delta_day=timedelta.days
    #     print 'daydaydaydaydaydaydayday', delta_day
    #     if obj.state == 'approve':
    #         if delta_day > 30:
    #             self.write(cr, uid, ids, {'expiration': 'normal_exp'})
    #             res[contract_expiration] = delta_day
    #         elif 0 <= delta_day <=30:
    #             self.write(cr, uid, ids, {'expiration': 'near_exp'})
    #             res[contract_expiration] = delta_day
    #         elif delta_day < 0:
    #             self.write(cr, uid, ids, {'expiration': 'overdue_exp'})
    #             res[contract_expiration] = delta_day
    #     return res
    #===========================================================================
    
    _columns = {
                'contract_number': fields.char('Contract number', readonly = True),
                'food_amount': fields.float('Food amount'),
                'contract_percent': fields.integer('Contract percent'),
                'income_percent': fields.integer('Income percent'),
                'square_percent': fields.integer('Square percent'),
                #'mill_fund_id': fields.integer('Millionaire percent'),
                'contract_definition': fields.text('Contract definition', required = True),
                #'date_start': fields.many2one('hr.contract','Date start'),
                #'date_end': fields.many2one('hr.contract','Date end'),
                'inc_spf_acc': fields.integer('CPA'),
                'cont_num_code': fields.integer('Cont num code'),
                'company_id': fields.integer('Company id', readonly = True),
                #'contract_expiration': fields.function(_contract_expiration, method = True, type = 'integer',string='Contract Expiration', store=True),
                'state': fields.selection([
                    ('cancel','Cancel'),
                    ('approve','Approve'),
                    ],'state'),
                'expiration': fields.selection([
                    ('normal_exp','Normal Exp'),
                    ('near_exp','Near Exp'),
                    ('overdue_exp','Overdue Exp'),
                    ],'Expiration'),
                }
    _defaults = {
                 'state': 'cancel',
                 'expiration': 'normal_exp',
                 }
    
    def action_contract_cancel(self, cr, uid, ids, *args):
        obj_employee = self.pool.get('hr.employee')
        obj = self.browse(cr, uid, ids)[0]
        state = ''
        cr.execute("""select * from other_contract as o
                        where o.hr_employee_id = %s
                            and o.state = 'approve'"""%obj.employee_id.id)
        have_other  = cr.dictfetchall()
        if have_other:
            state = 'others'
        else:
            state = 'noncontract'
        obj_employee.write(cr, uid, obj.employee_id.id, {'forum':state})
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True

    def action_contract_approve(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        emp_obj = self.pool.get('hr.employee')
        #tuhain ajiltnii niit batlagdsan gereeg yalgaj awah
        #choose the approved contract from employee
        cr.execute("""select c.*, t.name as tname, t.contract_type_code, t.id as tid 
                        from 
                            hr_contract as c, 
                            hr_employee as e, 
                            hr_contract_type as t
                        where 
                            e.id = %s and 
                            e.id = c.employee_id and 
                            c.state = 'approve' and 
                            c.type_id = t.id"""%obj.employee_id.id)
        conts_state  = cr.dictfetchall()
        #zowhon neg batlagdsan hodolmoriin geree baih estoi
        #Only one approved labour contract
        if conts_state:
            raise osv.except_osv(_('Анхааруулга'),_("Нэг ажилтанд нэгээс илүү батлагдсан гэрээ байж болохгүй. Та өмнөх хүчинтэй гэрээг цуцласнаар шинэ гэрээ баталж болно!!!"))
        
        #herew batlagdsan geree baihgvi buyu "i" too 0 bol shuud batlah buyu vndsen gereeg batlah vildel hiih
        #if not approved contract, will approve
        else:
            cr.execute("""select company_code, c.id as compa_id
                        from 
                            public.res_company as c,
                            public.hr_employee as e,
                            public.resource_resource as r
                        where
                            e.id = %s and
                            r.id = e.resource_id AND
                            c.id = r.company_id"""%obj.employee_id.id)
            conts  = cr.dictfetchall()
            val = 0
            val1 = 0
            if conts:
                for cont in conts:
                    val1 = str(cont['company_code'])
                    
                    if len(val1) != 4:
                        #number size must be 4 the company_code in company of employee
                        raise osv.except_osv(_('Анхааруулга'),_("Гэрээлэгч ажилтны компанийн код буруу байна. Компанийн код 4-н оронтой байх ёстой!!!"))
                    else:
                        val2 = obj.type_id.contract_type_code
                        
                        vall = str(val1).zfill(4) + str(val2).zfill(2)
                        
                        cr.execute("select contract_number from hr_contract where contract_number like '"+vall+"%' order by contract_number desc limit 1")
                        cont_code  = cr.dictfetchall()
                        
                        if cont_code:
                            cont_code = cont_code[0]
                            val = cont_code['contract_number']
                            val = val[6:]
            #employee haven't company
            else:
                raise osv.except_osv(_('Анхааруулга!'),_("Ажилтны компани талбар хоосон байна!!!"))
            
            
            val = int(val) + 1
            
            contract_number1 = u'%s'%cont['company_code'] + u'%s'%obj.type_id.contract_type_code + u'%s'%str(val).zfill(3)
            cont_num_code1 = val
            self.write(cr, uid, ids, {'employee_id.' : contract_number1,
                                      'state': 'approve',
                                      'contract_number' : contract_number1,
                                      'cont_num_code' : cont_num_code1})
            #change the forum of employee
            state1 = ''
            if val2 == '10':
                state1 = 'employee'
            elif val2 == '20':
                state1 = 'worker'
            elif val2 == '30':
                state1 = 'subcontractor'
            else:
                state1 = 'others'
            emp_obj.write(cr, uid, obj.employee_id.id, {'forum':state1})

        return True
    
    def action_contract_ending(self, cr, uid, ids, date_end, context=None):
        self.write(cr, uid, ids, {'state': 'ending'})
        delta_day = 0.0
        day = '%Y-%m-%d'
        today = date.today()
        if state == approve:
                a = datetime.strptime(date_end, day)
                b = datetime.strptime(today, day)
                timedelta = b - a
                delta_day=timedelta.days
                if 0 <= delta_day <=30:
                    is_short = True
        return True
    

class other_contract(osv.osv):
    _name = 'other.contract'
    _columns = {
               'name':fields.char('Contract name', required=True),
               'contract_number':fields.char('Contract number', readonly=True),
               'contract_type':fields.selection([('secrets_contract','Secrets contract'),('property_responsibility_contract','Property responsibility contract')], 'Contract type', required=True),
               'agent_name':fields.many2one('hr.employee','Agent name', required=True),
               'last_name':fields.char('Last name'),
               'hr_employee_id':fields.many2one('hr.employee','First name', required=True),
               'position':fields.char('Position'),
               'department_name':fields.char('Department'),
               'date_start':fields.datetime('Date start'),
               'date_end':fields.datetime( 'Date end'),
               'definition':fields.text('Definition', required=True),
               'notes':fields.text('Notes'),
               'state': fields.selection([
                    ('draft', 'Draft'),
                    ('cancel', 'Cancel'),
                    ('approve', 'Approve'),
                    ], 'State'),
               } 
    _defaults = {
                 'state': 'draft',
                 } 
    def action_other_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'draft'})
        return True

    def action_other_cancel(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        emp_obj = self.pool.get('hr.employee')
        state = ''
        i = 0
        cr.execute("""select * from other_contract as o
                        where o.hr_employee_id = %s
                            and o.state = 'approve'"""%obj.hr_employee_id.id)
        have_other  = cr.dictfetchall()
        if have_other:
            for quant in have_other:
                i = i + 1
        cr.execute("""select * from hr_contract as h
                        where h.employee_id = %s
                            and h.state = 'approve'"""%obj.hr_employee_id.id)
        have_contract  = cr.dictfetchall()
        if have_contract:
            state = 'employee'
        else:
            if i > 1:
                state = 'others'
            else:
                state = 'noncontract'
        emp_obj.write(cr, uid, obj.hr_employee_id.id, {'forum':state})
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True
    
    def action_other_approve(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        emp_obj = self.pool.get('hr.employee')
        cr.execute("""select company_code
                    from 
                        public.res_company as c,
                        public.hr_employee as e,
                        public.resource_resource as r
                    where
                        e.id = %s and
                        r.id = e.resource_id AND
                        c.id = r.company_id"""%obj.hr_employee_id.id)
        conts  = cr.dictfetchall()
        val1 = 0
        val = 0
        if conts:
            for cont in conts:
                val1 = str(cont['company_code'])
                if len(val1) != 4:
                    raise osv.except_osv(_('Анхааруулга'),_("Гэрээлэгч ажилтны компанийн код буруу байна. Компанийн код 4-н оронтой байх ёстой!!! Та системийн администраторт хандана уу."))
                else:
                    val2 = ''
                    if obj.contract_type == 'secrets_contract':
                        val2 = 30
                    elif obj.contract_type == 'property_responsibility_contract':
                        val2 = 40
                    else:
                        raise osv.except_osv(_('Error!'),_("You must select the contract_type"))
                    
                    vall = str(val1) + str(val2)
                    cr.execute("select contract_number from other_contract where contract_number like '"+vall+"%' order by contract_number desc limit 1")
                    cont_code  = cr.dictfetchall()
                    if cont_code:
                        cont_code = cont_code[0]
                        valar = cont_code['contract_number']
                        val = int(valar[6:])
                    
        else:
            raise osv.except_osv(_('Анхааруулга!'),_("Ажилтны компани талбар хоосон байна!!!"))
        
        val = val + 1
        contract_number1 = u'%s'%cont['company_code'] + u'%s'%val2 + u'%s'%str(val).zfill(3)
        #busad geree batlahad ajiltnii hr_employee medeelliin tolow solih
        state = ''
        cr.execute("""select * from hr_contract as h
                        where h.employee_id = %s
                            and h.state = 'approve'"""%obj.hr_employee_id.id)
        have_contract  = cr.dictfetchall()
        if have_contract:
            state = 'employee'
        else:
            state = 'others'
        emp_obj.write(cr, uid, obj.hr_employee_id.id, {'forum':state})
        #busad geree buyu oor bdre tolow shiljvvlj, contract_number bodoh
        self.write(cr, uid, ids, {'state': 'approve',
                                  'contract_number' : contract_number1})
        return True
        
    def onchange_other_contract(self, cr, uid, ids, hr_employee_id, context=None):
        res = {}
        if hr_employee_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, hr_employee_id)[0]
            res['last_name']=empl.last_name
            res['department_name']=empl.department_id.name
            res['position']=empl.job_id.name
            return {'value':res}
    
#class decree_list(osv.osv):
#    _name = 'decree.list'
 #   _columns = {
  #             'name':fields.char('Decree comment'),
   #            'dec_date':fields.date('Decree date'),
    #           'start_date':fields.date('Start date'),
     #          'end_date':fields.date('End date'),
      #         'trial_duration':fields.date('Trial duration'),
       #        'hired_date':fields.date('Hired date'),
        #       'imp_duration':fields.date('Imp duration'),
         #      'position_name':fields.char('Position'),
          #     'last_name':fields.char('Last name'),
           #    'first_name':fields.char('First name'),
            #   'basic_salary':fields.float('Basic salary'),
             #  'left_dec':fields.char('left deduction'),
              # 'remark':fields.text('Remark'),
               #'other':fields.text('Other'),
              # 'hr_employee_id':fields.many2one('hr.employee','Hr employee')
               #}
    
#===============================================================================
# class millionaire_fund(osv.osv):
#     _name = 'millionaire.fund'
#     _columns = {
#                'name':fields.char('Last name'),
#                'company_name':fields.many2one('res.company','Company name'),
#                'department_name':fields.many2one('hr.department','Department name'),
#                'position_name':fields.many2one('hr.job','Position'),
#                'enter_date':fields.date('Enter date'),
#                'working_years':fields.integer('Working years'),
#                'verify_doc':fields.char('Verify document', required=True),
#                'doc_num':fields.char('Document number'),
#                'loan_contract_num':fields.char('loan contract number'),
#                'released_amount':fields.float('Released amount', required=True),
#                'released_date':fields.date('Released date', required=True),
#                'hr_employee_id':fields.many2one('hr.employee','First name', required=True),
#                'company_id': fields.many2one('res.company','Company'),
#                'mill_invoice':fields.many2one('res.partner','Partner', required=True),
#                'invoice_id': fields.many2one('account.invoice','Invoice'),
#                'state': fields.selection([
#                     ('draft','Draft'),
#                     ('invoiced','Invoiced'),
#                     ],'state'),
#                 }
#     _defaults = {
#                  'state': 'draft',
#                 'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'millionaire.fund', context=c),
#                 }
#     
#     def onchange_millionaire(self, cr, uid, ids, empl_id, context=None):
#         res = {}
#         if empl_id:
#             empl_obj = self.pool.get('hr.employee')
#             empl = empl_obj.browse(cr, uid, empl_id)[0]
#             res['name']=empl.last_name
#             res['company_name']=empl.company_id.id
#             res['enter_date']=empl.accession_date
#             res['working_years']=empl.working_year
#             res['department_name']=empl.department_id.id
#             res['position_name']=empl.job_id.id
#             return {'value':res}  
#         
#     def action_mill_invoice(self, cr, uid, ids, context=None):
#         obj = self.browse(cr, uid, ids)[0]
#         invoice = self.pool.get('account.invoice')
#         journal_obj = self.pool.get('account.journal')
#         #par_obj = self.pool.get('fleet.vehicle')
#        
#         currency_id = obj.company_id.currency_id.id
#         journal_ids = journal_obj.search(cr, uid, [('type', '=','purchase'),('company_id', '=', obj.company_id.id or False)], limit=1)
#         invoice_lines = []
#              
#         invoice_line = []
#         invoice_line.append((0, 0, {
#                                         'name': obj.verify_doc,
#                                         'account_id': obj.mill_invoice.property_account_payable.id,
#                                         #'unit_type': obj.unit_type,
#                                         'price_unit': obj.released_amount,
#                                         
#         }))
#             
#         sch_invoice = invoice.create(cr, uid, {
#                                     'account_id': obj.mill_invoice.property_account_payable.id,
#                                     'type': 'in_invoice',
#                                     'partner_id': obj.mill_invoice.id,
#                                     'currency_id': currency_id,
#                                     'journal_id': len(journal_ids) and journal_ids[0] or False,
#                                     'date_invoice': obj.released_date,
#                                     #'fiscal_position': fpos and fpos.id,
#                                     'company_id': obj.company_name.id,
#                                     'invoice_line': invoice_line,
#                                      })
#         self.write(cr, uid, ids, {'state': 'invoiced',
#                                   'invoice_id': sch_invoice})
#         return True    
#===============================================================================
    
class hr_penalty(osv.osv):
    _name = 'hr.penalty'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Last name', track_visibility='onchange'),
               'penalty_name':fields.many2one('hr.penalty.type','Penalty name', track_visibility='onchange'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'date':fields.date('Date', required=True, track_visibility='onchange'),
               'penalty_select':fields.many2one('hr.imposing.penalty','Penalty select', required=True, track_visibility='onchange'),
               'percent':fields.float('Percent', track_visibility='onchange'),
               'amount':fields.float('Amount', track_visibility='onchange'),
               'deg_department':fields.many2one('hr.department','Deg department', track_visibility='onchange'),
               'deg_position':fields.many2one('hr.job','Deg position', track_visibility='onchange'),
               'deg_wage':fields.float('Deg wage', track_visibility='onchange'),
               'note':fields.text('Note'),
               'department_id':fields.many2one('hr.department','Department'),
               'hr_employee_id':fields.many2one('hr.employee','First name', required=True, track_visibility='onchange'),
               'hr_contract_id':fields.many2one('hr.contract','Hr contract'),
               'is_degredation':fields.boolean('Is degredation', track_visibility='onchange'),
               'state': fields.selection([
                    ('nonapprove','Nonapprove'),
                    ('approve','Approve'),
                    ],'state', track_visibility='onchange'),
                }
    _defaults = {
                 'state': 'nonapprove',
                 }
    def action_to_nonapprove(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'nonapprove'})
        return True
    
    def action_to_approve(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'approve'})
        emp_obj = self.pool.get('hr.employee')
        contract_obj = self.pool.get('hr.contract')
        job_obj = self.pool.get('hr.job')
        obj = self.browse(cr, uid, ids)
        asset_obj = self.pool.get('account.asset.asset')
        partner_obj = self.pool.get('res.partner')
        partner_id = partner_obj.search(cr, uid, [('user_id','=',obj.hr_employee_id.user_id.id)])
        if partner_id !=[]:
            asset_id = asset_obj.search(cr, uid, [('owner_id','=',partner_id[0])])
            if asset_id:
                raise osv.except_osv(_('Error!'),_("You must an accumulation"))
        if obj.hr_employee_id.contract_id:
            if obj.is_degredation == True:
                contract_obj.write(cr, uid, obj.hr_employee_id.contract_id.id, {'wage':obj.deg_wage,
                                                                        'job_id':obj.deg_position.id,}),
                emp_obj.write(cr, uid, obj.hr_employee_id.id, {'department_id':obj.deg_department.id,
                                                                'job_id':obj.deg_position.id,
                                                                'job_grade_id':obj.deg_position.job_grade_id.id,
                                                                'job_code':obj.deg_position.job_code,})
            
        else:
            raise osv.except_osv(_('Error!'),_("Тухайн ажилтан хөдөлмөрийн гэрээ байгуулаагүй байна! Хөдөлмөрийн гэрээ нь хүчин төгөлдөр(батлагдсан) ажилтанд сахилга шийтгэлийг оноох боломжтой"))
                
        return True
    #===========================================================================
    # def write(self, cr, uid, ids,vals,context=None):
    #     print "assssssssssssssss",vals.get('amount')
    #     if vals.get('percent'):
    #         print "logg"
    #         vals.update({'amount':vals['amount']})
    #     return super(hr_penalty,self).write(cr, uid, ids,vals, context=context)
    # 
    #===========================================================================
    def onchange_calculate(self, cr, uid, ids, empl_id, percent, context=None):
       obj = self.browse(cr, uid, ids) 
       emp_obj = self.pool.get('hr.employee')
       contract_wage = 0
       if percent >= 0:
           for line in emp_obj.browse(cr, uid, empl_id):
                if line.contract_id.id!=False:
                    contract_wage = line.contract_id.wage
                else:
                    contract_wage = 0
           sum = (percent*contract_wage)/100
           return {'value':{'amount':sum}}
         
    def onchange_penalty(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=empl.last_name
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}
        
class hr_penalty_type(osv.osv):
    _name = 'hr.penalty.type'
    _columns = {
               'name':fields.char('Penalty name'),
               'penalty_code':fields.char('Penalty code'),
               'hr_employee_id':fields.many2one('hr.employee','First name')
               }
class hr_imposing_penalty(osv.osv):
    _name = 'hr.imposing.penalty'
    _columns = {
               'name':fields.char('Imposing penalty'),
               'hr_employee_id':fields.many2one('hr.employee','First name')
               }
    
class out_work(osv.osv):
    _name = 'out.work'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Purpose', required=True, track_visibility='onchange'),
               'site_name':fields.char('Site name'),
               'approval_date':fields.date('Approval date', track_visibility='onchange'),
               'out_time':fields.date('Out time', track_visibility='onchange'),
               'in_time':fields.date('In time', track_visibility='onchange'),
               'guideline':fields.char('Guideline', required=True, track_visibility='onchange'),
               'day':fields.integer('Day', track_visibility='onchange'),
               #'end_date':fields.date('End date'),
               #'timeout':fields.integer('Timeout'),
               'country':fields.many2one('res.country','Country', track_visibility='onchange'),
               'city':fields.many2one('res.country.state','City', track_visibility='onchange'),
               #'authority_sign':fields.many2one('res.users','Authority sign', readonly=True),
               'authority_sign':fields.many2one('res.partner','Authority sign', track_visibility='onchange'),
               'budget':fields.float('Budget', track_visibility='onchange'),
               'is_plan': fields.selection([('no', 'No'), ('yeas', 'Yeas')], 'Is plan', track_visibility='onchange'),
               'work_line_id': fields.one2many('work.line','out_work_id','Work line', track_visibility='onchange'),
               'state': fields.selection([
                    ('draft', 'Draft'),
                    ('cancel', 'Cancel'),
                    ('approve', 'Approve'),
                    ], 'State', track_visibility='onchange'),
               'company_id':fields.many2one('res.company','Company', required=True, track_visibility='onchange'),
               }
    _defaults = {
                 'state': 'draft',
                 'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'out.work', context=c),
                 #'authority_sign':lambda self, cr, uid, context: uid,
                 }
    def action_work_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'draft'})
        return True

    def action_work_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True
    
    def action_work_approve(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'approve'})
       
    def onchange_in_time(self, cr, uid, ids, out_time, in_time, context=None):
        delta_day = 0.0
        day = '%Y-%m-%d'
        if in_time != False :
            if out_time ==False:
                raise osv.except_osv(_('Error!'),_("You must fill the out_time"))
            else:
                a = datetime.strptime(out_time, day)
                b = datetime.strptime(in_time, day)
                timedelta = b - a
                delta_day=timedelta.days
                return {'value': {'day': delta_day,}}
        
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        line_obj = self.pool.get('work.line')
        for line in obj.work_line_id:
            line_obj.unlink(cr, uid, line.id)
        return super(out_work, self).unlink(cr, uid, ids, context=context)
    #def onchange_time(self, cr, uid, ids, out_time, in_time,  context=None):
       #if int(emp.org_finish.name[0:2]) - int(emp.org_start.name[0:2]):
           #sum = base_expense+other_expense
           #return {'value': {'present_expense': sum,}}        
    
class work_line(osv.osv):
    _name = 'work.line'
    _columns = {
               'name':fields.char('Last name'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'out_work_id':fields.many2one('out.work','Out work'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               }       
    def onchange_out_work(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=empl.last_name
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}
class out_execution(osv.osv):
    _name = 'out.execution'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Guideline', required=True),
               'site_name':fields.char('Site name'),
               'approval_date':fields.date('Approval date', track_visibility='onchange'),
               'out_time':fields.date('Out time', track_visibility='onchange'),
               'in_time':fields.date('In time', track_visibility='onchange'),
               #'day':fields.integer('Day'),
               #'start_date':fields.date('Start date'),
               #'end_date':fields.date('End date'),
               'timeout':fields.integer('Timeout', track_visibility='onchange'),
               'is_plan': fields.selection([('no', 'No'), ('yeas', 'Yeas')], 'Is plan', track_visibility='onchange'),
               'country':fields.many2one('res.country','Country'),
               'city':fields.many2one('res.country.state','City'),
               #'authority_sign':fields.many2one('res.users','Authority sign', readonly=True),
               'authority_sign':fields.many2one('res.partner','Authority sign', required=True),
               'budget':fields.float('Budget', track_visibility='onchange'),
               'core_cost':fields.float('Core cost', track_visibility='onchange'),
               'other_cost':fields.float('Other cost', track_visibility='onchange'),
               'real_cost':fields.float('Real cost', track_visibility='onchange'),
               'variation_cost':fields.float('Variation cost', track_visibility='onchange'),
               'out_work_id':fields.many2one('out.work','Purpose', required=True, track_visibility='onchange'),
               'invoice_id': fields.many2one('account.invoice','Invoice'),
               'company_id': fields.many2one('res.company','Company', track_visibility='onchange'),
               'planned_day': fields.integer('Planned day'),
               'out_execution_line_id': fields.one2many('out.execution.line','out_execution_id','Out execution', track_visibility='onchange'),
               'state': fields.selection([
                    ('draft', 'Draft'),
                    ('invoiced', 'Invoiced'),
                    ], 'State', track_visibility='onchange'),
                }
    
                
    _defaults = {
                 'state': 'draft',
                 #'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'out.execution', context=c),
                 }
    
    def onchange_work(self, cr, uid, ids, out_work_id, context=None):
        res = {}
        if out_work_id:
            work_obj = self.pool.get('out.work')
            work = work_obj.browse(cr, uid, out_work_id)[0]
            res['name']=work.guideline
            res['site_name']=work.site_name
            res['is_plan']=work.is_plan
            res['approval_date']=work.approval_date
            res['planned_day']=work.day
            #res['in_time']=work.in_time
            #res['day']=work.day
            #res['end_date']=work.end_date
            #res['timeout']=work.timeout
            res['country']=work.country
            res['city']=work.city
            res['authority_sign']=work.authority_sign
            res['budget']=work.budget
            res['company_id']=work.company_id
            return {'value':res}
        
    def onchange_cost(self, cr, uid, ids, core_cost, other_cost,  context=None):
       if core_cost >= 0 and other_cost >= 0:
           sum = core_cost+other_cost
           return {'value': {'real_cost': sum,}}
       
    def onchange_variation(self, cr, uid, ids, budget, real_cost,  context=None):
       if budget >= 0 and real_cost >= 0:
           sum = budget-real_cost
           return {'value': {'variation_cost': sum,}}
       
    def onchange_timeout(self, cr, uid, ids, out_time, in_time, planned_day, context=None):
        delta = 0.0
        day = '%Y-%m-%d'
        if in_time != False :
            if out_time ==False:
                raise osv.except_osv(_('Error!'),_("You must fill the out_time"))
            else:
                a = datetime.strptime(out_time, day)
                b = datetime.strptime(in_time, day)
                timedelta = b - a
                delta=planned_day - timedelta.days
                return {'value': {'timeout': delta,}}
    #def onchange_timeout(self, cr, uid, ids, out_time, in_time,  context=None):
     #  obj = self.browse(cr, uid, ids)[0]
      # day_obj = self.pool.get('out.work')
#       
 #      in_day = obj.in_time.strftime('%Y-%m-%d')[8:10]
  #     in_month = obj.in_time.strftime('%Y-%m-%d')[5:7]
   #    in_year = obj.in_time.strftime('%Y-%m-%d')[0:4]
    #   
     #  out_day = obj.out_time.strftime('%Y-%m-%d')[8:10]
      # out_month = obj.out_time.strftime('%Y-%m-%d')[5:7]
       #out_year = obj.out_time.strftime('%Y-%m-%d')[0:4]
       
#       in_number_of_day =int(in_year)*360+ int(in_month)*30+int(in_day) 
 #      out_number_of_day =int(out_year)*360+ int(out_month)*30+int(out_day)
  #     
   #    number_day=int(in_number_of_day) - int(out_number_of_day)
    #   
     #  if out_time and in_time:
      #     day_obj.write(cr, uid, obj.out_work_id.id, {'day':obj.day})
       #    cal = day - number_day
        #   return {'value': {'timeout': cal,}}   
       
    def action_create_invoice(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        invoice = self.pool.get('account.invoice')
        journal_obj = self.pool.get('account.journal')
        #par_obj = self.pool.get('fleet.vehicle')
       
        currency_id = obj.company_id.currency_id.id
        journal_ids = journal_obj.search(cr, uid, [('type', '=','purchase'),('company_id', '=', obj.company_id.id or False)], limit=1)
        #par_ids = par_obj.browse(cr, uid, par_obj.search(cr, uid, [('id','=',obj.fleet_vehicle_id.id)]))
        #fpos = par_ids.driver_id.property_account_position or False
        invoice_lines = []
             
        invoice_line = []
        invoice_line.append((0, 0, {
                                        'name': obj.out_work_id.name,
                                        'account_id': obj.authority_sign.property_account_payable.id,
                                        #'unit_type': obj.unit_type,
                                        'price_unit': obj.real_cost,
                                        #'quantity': obj.continuity_time,
                                        #'price_subtotal': obj.total_price,
                                        
        }))
            
        sch_invoice = invoice.create(cr, uid, {
                                    'account_id': obj.authority_sign.property_account_payable.id,
                                    'type': 'in_invoice',
                                    'partner_id': obj.authority_sign.id,
                                    'currency_id': currency_id,
                                    'journal_id': len(journal_ids) and journal_ids[0] or False,
                                    #'date_invoice': obj.executed_time,
                                    #'fiscal_position': fpos and fpos.id,
                                    'company_id': obj.company_id.id,
                                    'invoice_line': invoice_line,
                                    #'date_due': obj.executed_time,
                                    #'comment': obj.description,
                                     })
        self.write(cr, uid, ids, {'state': 'invoiced',
                                  'invoice_id': sch_invoice})
        return True   
        
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        line_obj = self.pool.get('out.execution.line')
        for line in obj.out_execution_line_id:
            line_obj.unlink(cr, uid, line.id)
        return super(out_execution, self).unlink(cr, uid, ids, context=context)
        
class out_execution_line(osv.osv):
    _name = 'out.execution.line'
    _columns = {
               'name':fields.char('Last name'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'out_execution_id':fields.many2one('out.execution','Out execution'),
               'hr_employee_id':fields.many2one('hr.employee','Hr employee'),
               }       
    def onchange_out_execution(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=empl.last_name
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}                          

class supply_bonus(osv.osv):
    _name = 'supply.bonus'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Last name'),
               'company_name':fields.many2one('res.company','Company'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'type': fields.selection([('discount', 'Discount'), ('supply', 'Supply'), ('allowance', 'Allowance'), ('bonus', 'Bonus'), ('payback', 'Payback')], 'Type', track_visibility='onchange'),
               'amount':fields.float('Amount', required=True, track_visibility='onchange'),
               'giving_date':fields.date('Giving date', required=True, track_visibility='onchange'),
               'description':fields.text('Description', required=True),
               'hr_employee_id':fields.many2one('hr.employee','First name', required=True, track_visibility='onchange'),
               'supply_invoice':fields.many2one('res.partner','Partner', required=True, track_visibility='onchange'),
               'company_id':fields.many2one('res.company','Company'),
               'invoice_id': fields.many2one('account.invoice','Invoice'),
               #================================================================
               # 'state': fields.selection([
               #      ('draft', 'Draft'),
               #      ('approved', 'Approved'),
               #      ], 'State'),
               #================================================================
               }
    
    _defaults = {
                 #'state': 'draft',
                 'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'supply.bonus', context=c),
                 }
                
    def onchange_supply_bonus(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=empl.last_name
            res['company_name']=empl.company_id.id
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}
        
        
    #===========================================================================
    # def action_supply_invoice(self, cr, uid, ids, context=None):
    #     obj = self.browse(cr, uid, ids)[0]
    #     invoice = self.pool.get('account.invoice')
    #     journal_obj = self.pool.get('account.journal')
    #     #par_obj = self.pool.get('fleet.vehicle')
    #    
    #     currency_id = obj.company_id.currency_id.id
    #     journal_ids = journal_obj.search(cr, uid, [('type', '=','purchase'),('company_id', '=', obj.company_id.id or False)], limit=1)
    #     invoice_lines = []
    #          
    #     invoice_line = []
    #     invoice_line.append((0, 0, {
    #                                     'name': obj.description,
    #                                     'account_id': obj.supply_invoice.property_account_payable.id,
    #                                     #'unit_type': obj.unit_type,
    #                                     'price_unit': obj.amount,
    #                                     
    #     }))
    #         
    #     sch_invoice = invoice.create(cr, uid, {
    #                                 'account_id': obj.supply_invoice.property_account_payable.id,
    #                                 'type': 'in_invoice',
    #                                 'partner_id': obj.supply_invoice.id,
    #                                 'currency_id': currency_id,
    #                                 'journal_id': len(journal_ids) and journal_ids[0] or False,
    #                                 'date_invoice': obj.giving_date,
    #                                 #'fiscal_position': fpos and fpos.id,
    #                                 'company_id': obj.company_id.id,
    #                                 'invoice_line': invoice_line,
    #                                  })
    #     self.write(cr, uid, ids, {'state': 'invoiced',
    #                               'invoice_id': sch_invoice})
    #     return True
    #===========================================================================
        
class application_complaint(osv.osv):
    _name = 'application.complaint'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Last name'),
               'company_name':fields.many2one('res.company','Company'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'date':fields.date('Date', required=True, track_visibility='onchange'),
               'decision':fields.text('Decision', track_visibility='onchange'),
               'deliver_date':fields.date('Deliver date', track_visibility='onchange'),
               'comment':fields.text('Comment', required=True, track_visibility='onchange'),
               'hr_employee_id':fields.many2one('hr.employee','First name', required=True, track_visibility='onchange'),
               'complaint_type':fields.many2one('complaint.type','Complaint type', required=True, track_visibility='onchange'),
               #'mill_amount':fields.float('Mill amount'),
               'supply_amount':fields.float('Supply amount', track_visibility='onchange'),
               'partner_id':fields.many2one('res.partner','Partner', required=True, track_visibility='onchange'),
               'state': fields.selection([
                    ('create', 'Create'),
                    ('delivery', 'Delivery'),
                    ('cancel', 'Cancel'),
                    ('approve', 'Approve'),
                    ], 'State', track_visibility='onchange'),
               }
    _defaults = {
                 'state': 'create',
                 #'hr_employee_id': lambda obj, cr, uid, context: uid,
                 #'partner_id': lambda self, cr, uid, context: context.get('partner_id', False) and self.pool.get('res.partner').browse(cr, uid, context['partner_id']).partner_id.id,
                 }
    #===========================================================================
    # def action_delivery(self, cr, uid, ids, *args):
    #     obj = self.browse(cr, uid, ids)[0]
    #     mill_amount = obj.mill_amount
    #     supply_amount = obj.supply_amount
    #     if mill_amount != 0:
    #         if supply_amount != 0:
    #             raise osv.except_osv(_('Өгөгдөл бөглөлтийн алдаа!'),_("Саятан сангаас мөнгө авах болон Хөнгөлөлт эдэлж мөнгө авах гэсэн талбаруудын аль нэгээр нь л нэхэмжлэл үүсгэх боломжтой тул аль нэг нь хоосон байх ёстой!"))
    #             
    #             return
    #         else: self.write(cr, uid, ids, {'state': 'delivery'})
    #         
    #     else: self.write(cr, uid, ids, {'state': 'delivery'})
    #     
    #     return True
    #===========================================================================
    def action_delivery(self, cr, uid, ids, *args):
        #=======================================================================
        # obj = self.browse(cr, uid, ids)[0]
        # #mill_amount = obj.mill_amount
        # supply_amount = obj.supply_amount
        # if supply_amount == 0:
        #     raise osv.except_osv(_('Өгөгдөл бөглөлтийн алдаа!'),_("Хөнгөлөлт эдэлж мөнгө авах бол мөнгөн дүн 0 байж болохгүй!"))
        #     
        #     return
        # else: self.write(cr, uid, ids, {'state': 'delivery'})
        #=======================================================================
        self.write(cr, uid, ids, {'state': 'delivery'})
        return True

    def action_draft(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'create'})
        return True

    def action_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True
      
    def onchange_application(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=empl.last_name
            res['company_name']=empl.company_id.id
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}
        
    #===========================================================================
    # def action_approve(self, cr, uid, ids, context=None):
    #     obj = self.browse(cr, uid, ids)[0]
    #     mill_amount = obj.mill_amount
    #     supply_amount = obj.supply_amount
    #     if mill_amount != 0:
    #         if supply_amount == 0:
    #             obj = self.browse(cr, uid, ids)[0]
    #             mill_draft = self.pool.get('millionaire.fund')
    #             sch_mill_draft = mill_draft.create(cr, uid, {
    #                                 'hr_employee_id': obj.hr_employee_id.id,
    #                                 'name': obj.name,
    #                                 'company_name': obj.company_name.id,
    #                                 'department_name': obj.department_name.id,
    #                                 'position_name': obj.position_name.id,
    #                                 'released_amount': obj.mill_amount,
    #                                 'released_date': obj.date,
    #                                 'enter_date': obj.hr_employee_id.accession_date,
    #                                 'working_years': obj.hr_employee_id.working_year,
    #                                 'mill_invoice':obj.partner_id.id,
    #                                 'verify_doc':obj.complaint_type.name,
    #                                 #'journal_id': len(journal_ids) and journal_ids[0] or False,
    #                                  })
    #             self.write(cr, uid, ids, {'state': 'approve'})
    #             return
    #         else: raise osv.except_osv(_('Өгөгдөл бөглөлтийн!'),_("Саятан сангаас мөнгө авах болон Хөнгөлөлт эдэлж мөнгө авах гэсэн талбаруудын аль нэгээр нь л нэхэмжлэл үүсгэх боломжтой тул аль нэг нь хоосон байх ёстой!"))
    #     
    #     else: 
    #         if supply_amount != 0:
    #             supply_draft = self.pool.get('supply.bonus')
    #     
    #             sch_supply_draft = supply_draft.create(cr, uid, {
    #                                 'hr_employee_id': obj.hr_employee_id.id,
    #                                 'name': obj.name,
    #                                 'company_name': obj.company_name.id,
    #                                 'department_name': obj.department_name.id,
    #                                 'position_name': obj.position_name.id,
    #                                 'amount': obj.supply_amount,
    #                                 'giving_date': obj.date,
    #                                 'supply_invoice':obj.partner_id.id,
    #                                 'description':obj.complaint_type.name,
    #                                 #'journal_id': len(journal_ids) and journal_ids[0] or False,
    #                                  })
    #     
    #         self.write(cr, uid, ids, {'state': 'approve'})
    #         
    #     return True
    #===========================================================================
    def action_approve(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        supply_amount = obj.supply_amount
        
        #if supply_amount != 0:
        supply_draft = self.pool.get('supply.bonus')

        sch_supply_draft = supply_draft.create(cr, uid, {
                            'hr_employee_id': obj.hr_employee_id.id,
                            'name': obj.name,
                            'company_name': obj.company_name.id,
                            'department_name': obj.department_name.id,
                            'position_name': obj.position_name.id,
                            'amount': obj.supply_amount,
                            'giving_date': obj.date,
                            'supply_invoice':obj.partner_id.id,
                            'description':obj.complaint_type.name,
                            #'journal_id': len(journal_ids) and journal_ids[0] or False,
                             })
        self.write(cr, uid, ids, {'state': 'approve'})
        #else: raise osv.except_osv(_('Өгөгдөл бөглөлтийн!'),_("Хөнгөлөлт эдэлж мөнгө авах бол мөнгөн дүн 0 байж болохгүй!"))
    
            
        return True
    
    
class complaint_type(osv.osv):
    _name = 'complaint.type'
    _columns = {
               'name':fields.char('Complaint type'),
               'hr_employee_id':fields.many2one('hr.employee','First name'),
               }
class labour_movement(osv.osv):
    _name = 'labour.movement'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
               'name':fields.char('Last name'),
               'company_name':fields.many2one('res.company','Company'),
               'department_name':fields.many2one('hr.department','Department'),
               'position_name':fields.many2one('hr.job','Position'),
               'to_company':fields.many2one('res.company','To company', track_visibility='onchange'),
               'to_department':fields.many2one('hr.department','To department', track_visibility='onchange'),
               'to_position':fields.many2one('hr.job','To position', track_visibility='onchange'),
               #'possesed_asset':fields.char('Possesed asset'),
               'date':fields.date('Date', track_visibility='onchange'),
               'shift_reason':fields.text('Shift reason', track_visibility='onchange'),
               'hr_employee_id':fields.many2one('hr.employee','First name', track_visibility='onchange'),
               'state': fields.selection([
                    ('draft', 'Draft'),
                    ('cancel', 'Cancel'),
                    ('approve', 'Approve'),
                    ], 'State', track_visibility='onchange'),
               } 
    _defaults = {
                 'state': 'draft',
                 } 
    def action_to_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'draft'})
        return True

    def action_to_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True
    
    def action_to_approve(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'approve'})
        emp_obj = self.pool.get('hr.employee')
        obj = self.browse(cr, uid, ids)
        asset_obj = self.pool.get('account.asset.asset')
        partner_obj = self.pool.get('res.partner')
        
        if obj.hr_employee_id.user_id.partner_id.id !=[]:
            asset_id = asset_obj.search(cr, uid, [('owner_id','=',obj.hr_employee_id.user_id.partner_id.id)])
            if asset_id !=[]:
                raise osv.except_osv(_('Error!'),_("You must an accumulation"))
            else:
                emp_obj.write(cr, uid, obj.hr_employee_id.id, {'company_id':obj.to_company.id,
                                                                'department_id':obj.to_department.id,
                                                                'job_id':obj.to_position.id,
                                                                'job_grade_id':obj.to_position.job_grade_id.id,
                                                                'job_code':obj.to_position.job_code})
        return True
    
    def onchange_movement(self, cr, uid, ids, empl_id, context=None):
        res = {}
        if empl_id:
            empl_obj = self.pool.get('hr.employee')
            empl = empl_obj.browse(cr, uid, empl_id)[0]
            res['name']=empl.last_name
            res['company_name']=empl.company_id.id
            res['department_name']=empl.department_id.id
            res['position_name']=empl.job_id.id
            return {'value':res}
        
    #def change_movement(self, cr, uid, ids, *args):
        #obj = self.browse(cr, uid, ids)
        #asset_obj = self.pool.get('account.asset.asset')
        #asset_id = asset_obj.search(cr, uid, [('owner_id','=',obj.hr_employee_id.id)])
        
        #if asset_id:
            #raise osv.except_osv(_('Error!'),_("You must an accumulation"))

        #else:
            #self.write(cr, uid, ids,{'active':False})
        #return True
                  
        
    
class hr_holidays(osv.osv):
    _inherit = 'hr.holidays'
    
    _columns = {
               'company_name':fields.many2one('res.company','Company name'),
               'position_name':fields.many2one('hr.job','Job name'),
               'employee_id':fields.many2one('hr.employee','Hr employee'),
               'date_to':fields.datetime('Date to'),
               'date_from':fields.datetime('Date from'),
               'sick_type_id':fields.many2one('hr.holidays.sick.type','Sick type'),
               'is_sick':fields.boolean('Is sick'),
               'is_out_work':fields.boolean('Is out work'),
               'minutes':fields.integer('Minutes'),
               }

    def _get_number_of_days(self, date_from, date_to):
        """Returns a float equals to the timedelta between two dates given as string."""

        DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
        #print 'xxxxxxxx', date_from, date_to
        from_dt = datetime.strptime(date_from, DATETIME_FORMAT)
        to_dt = datetime.strptime(date_to, DATETIME_FORMAT)
        timedelta = to_dt - from_dt
        #diff_day = timedelta.days + float(timedelta.seconds) / 86400
        diff_business_days = len(list(rrule.rrule(rrule.DAILY,
                                          dtstart=from_dt,
                                          until=to_dt,
                                          byweekday=(rrule.MO, rrule.TU, rrule.WE, rrule.TH, rrule.FR))))
        diff_day = diff_business_days + float(timedelta.seconds) / 86400
        #=======================================================================
        # aaaa = diff_business_days*8 + int((float(timedelta.seconds) / 60 - 60)/60)
        # print 'hhhhhh', timedelta,'gggggg', diff_business_days*8 + int((float(timedelta.seconds) / 60 - 60)/60), 'ssssssssss', (float(timedelta.seconds) / 60 - 60)/60 - aaaa
        # 
        # print 'odornnnnn', diff_day, int(diff_business_days)*8, 'tsagnnnnnnnnnn', float(timedelta.seconds) / 3600, 'fffffffff', (timedelta(diff_day).seconds)/60
        # print 't-----------b', diff_business_days, diff_day, 'ooooooooo', int(timedelta.seconds)/60
        #=======================================================================
        return diff_day
    
    def onchange_date_to(self, cr, uid, ids, date_to, date_from,employee_id,holiday_status_id):
        """
        Update the number_of_days.
        """
        holiday_status_obj = self.pool.get('hr.holidays.status')
        holiday_status_id_id = holiday_status_obj.search(cr, uid, [('id','=',holiday_status_id)])
        hol_id = holiday_status_obj.browse(cr, uid, holiday_status_id_id)
        seniority=0
        now_working_year = 0
        seniority_by_year = 0
        hol_days_add = 0
        hol_days_base = 0
        hol_days_total_this_year = 0
        #herew is_vacations -g checklesen buyu zowhon eeljiin amralt honogiig tootsoh
        if hol_id.is_vacations == True:
            # Compute and update the number of days
            if (date_from and date_to) and (date_from > date_to):
                raise osv.except_osv(_('Warning!'),_('The start date must be anterior to the end date.'))
            result = {'value': {}}
            if (date_to and date_from) and (date_from <= date_to):
                diff_day = self._get_number_of_days(date_from, date_to)
                #diff_hours = diff_day * 24
            #compute the seniority
            hr_employee_obj = self.pool.get('hr.employee')
            hr_employee_obj_id = hr_employee_obj.search(cr, uid, [('id','=',employee_id)])
            emp_id = hr_employee_obj.browse(cr, uid, hr_employee_obj_id)
            if emp_id.working_year < 1:
                #jil ajillaagvi bol eeljiin amralt awah erhgvi
                now_working_year = emp_id.working_year_month
                raise osv.except_osv(_('Warning!'),_("Та компанид нэг жилээс дээш хугацаагаар ажилласан бол 'Ээлжийн амралт' авах боломжтой болно, та одоогоор " + str(now_working_year) + "-сар ажилласан байна!"))
            else:
                #jilees deesh bol hed honog amrahiig bodoh
                if employee_id and date_from and date_to:
                    seniority = emp_id.seniority
                    #ulsad ajillasan jil n saraar irj bga ba jil rvv shiljvvleh
                    #sen = int(seniority)
                    #seniority_by_year = int(sen/12)
                    #eeljiin amraltiin huuliar tohiruulj ogson tohirgoonoos vndsen bolon nemelt honoguudiig awah
                    hr_holidays_obj = self.pool.get('hr.holiday.days')
                    holiday_id = hr_holidays_obj.search(cr, uid, [('min_year','<=',seniority),('max_year','>=',seniority)])
                    holiday_id_id = hr_holidays_obj.browse(cr, uid, holiday_id)
                    if holiday_id_id:
                        hol_days_add = holiday_id_id.get_days
                        hol_days_base = holiday_id_id.base_days
                        #print 'hr holiday days info',holiday_id_id.get_days,holiday_id_id.min_year,holiday_id_id.max_year
                    else:
                        raise osv.except_osv(_('Warning!'),_("Уучлаарай, ээлжийн амралт олгоход бодогдох нэмэлт хоногийн тохиргоо хийгдээгүй байна. Та системийн админдаа хандана уу!"))
                    #awch boloh eeljiin amraltiin honogoos ilvv honogiin choloo oruulsan bol anhaaruulah
                    hol_days_total_this_year = hol_days_base + hol_days_add
                    if hol_days_total_this_year < round(math.floor(diff_day)):
                        raise osv.except_osv(_('Warning!'),_("Таны оруулсан хоног боломжит ээлжийн амралтын хоногоос их байна!" + " Таны ээлжийн амралтын боломжит хоног "+ str(hol_days_total_this_year)+". Та шалгаад, дахин оруулна уу!"))
                        result['value']['number_of_days_temp'] = 0
                        #result['value']['hours'] = 0
                    #awch boloh eeljiin amraltiin honogoos ilvvgvi honogiin choloo oruulsan bol hvssen honogiin toog haruulah
                    else:
                        result['value']['number_of_days_temp'] = round(math.floor(diff_day))
                        #result['value']['hours'] = diff_hours
        #eeljiin amraltaas busad amraltiin odriig bodoh
        else:
            # date_to has to be greater than date_from
            if (date_from and date_to) and (date_from > date_to):
                raise osv.except_osv(_('Warning!'),_('The start date must be anterior to the end date.'))
       
            result = {'value': {}}
       
            # Compute and update the number of days
            if (date_to and date_from) and (date_from <= date_to):
                diff_day = self._get_number_of_days(date_from, date_to)
                
                #------------------------- DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
                #------- from_dt = datetime.strptime(date_from, DATETIME_FORMAT)
                #----------- to_dt = datetime.strptime(date_to, DATETIME_FORMAT)
                #----------------------------------- timedelta = to_dt - from_dt
                # print 'bbbbbbbbbbbbb', timedelta, 'ddddd',  float(timedelta.seconds) / 60
                
                #diff_hours = diff_day * 24
                result['value']['number_of_days_temp'] = round(math.floor(diff_day))
                #result['value']['minutes'] = int(timedelta.seconds) / 60
            else:
                result['value']['number_of_days_temp'] = 0
                #result['value']['minutes'] = 0
        return result
    
    def onchange_employee(self, cr, uid, ids, employee_id):
        result = {'value': {'department_id': False}}
        if employee_id:
            employee = self.pool.get('hr.employee').browse(cr, uid, employee_id)
            job = self.pool.get('hr.job').browse(cr, uid, employee_id)
            result['value'] = {'department_id': employee.department_id.id,
                               'company_name':employee.company_id.id,
                               'position_name':employee.job_id.id}
        return result
    
    def onchange_holiday_status_id(self, cr, uid, ids,type, holiday_status_id):
        result = {'value': {}}
        if holiday_status_id and type == 'remove':
            hol_stat = self.pool.get('hr.holidays.status').browse(cr, uid, holiday_status_id)
            result['value']['is_sick'] = hol_stat.is_sick
            result['value']['sick_type_id'] = False
            result['value']['is_out_work'] = hol_stat.is_out_work
            
        return result
    
    #===========================================================================
    # def create(self, cr, uid, values, context=None):
    #     """ Override to avoid automatic logging of creation """
    #     if context is None:
    #         context = {}
    #         
    #     
    #         
    #     employee_id = values.get('employee_id', False)
    #     context = dict(context, mail_create_nolog=True, mail_create_nosubscribe=True)
    #     if values.get('state') and values['state'] not in ['draft', 'confirm', 'cancel'] and not self.pool['res.users'].has_group(cr, uid, 'base.group_hr_user'):
    #         raise osv.except_osv(_('Warning!'), _('You cannot set a leave request as \'%s\'. Contact a human resource manager.') % values.get('state'))
    #     hr_holiday_id = super(hr_holidays, self).create(cr, uid, values, context=context)
    #     self.add_follower(cr, uid, [hr_holiday_id], employee_id, context=context)
    #     return hr_holiday_id
    #===========================================================================
    

    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid,ids)[0]
        
        
        date_from = ''
        date_to = ''
        if 'date_from' in vals:
            date_from = vals['date_from']
        else:
            date_from = obj.date_from
        if 'date_to' in vals:
            date_to = vals['date_to']
        else:
            date_to = obj.date_to
        
        
        if 'number_of_days_temp' in vals:
            if obj.type == 'remove':
                if vals['number_of_days_temp'] != int(self._get_number_of_days(date_from, date_to)):
                    #print 'kkkkkkkk', vals['number_of_days_temp'], int(self._get_number_of_days(date_from, date_to))
                    raise osv.except_osv(_('Буруу үйлдэл!'), _('Та өдрийг гараар засч болохгүй. Огнооны хоорондох өдөр нийт өдрийн тоотой тэнцүү байх ёстой !'))
                else:
                    vals['number_of_days_temp'] = vals['number_of_days_temp']
            else:
                vals['number_of_days_temp'] = vals['number_of_days_temp']
        
        #=======================================================================
        # if obj.type == 'remove':
        #     if 'is_sick' in vals and vals['is_sick'] == True:
        #         print '11111111', vals, vals['is_sick'], obj.sick_type_id
        #         if 'sick_type_id' in vals and vals['sick_type_id'] == False:
        #             print 'aaaaas', vals['sick_type_id']
        #             raise osv.except_osv(_('Анхааруулга!'), _('<<Өвчний шалтгаан>> талбарыг заавал бөглөнө үү !'))
        #=======================================================================
                
        if vals.get('state') and vals['state'] not in ['draft', 'confirm', 'cancel'] and not self.pool['res.users'].has_group(cr, uid, 'base.group_hr_user'):
            raise osv.except_osv(_('Warning!'), _('You cannot set a leave request as \'%s\'. Contact a human resource manager.') % vals.get('state'))
        
        return super(hr_holidays, self).write(cr, uid, ids, vals, context=context)
    
    def holidays_confirm(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if record.holiday_type == 'employee':
                #if_headmans = self.pool['res.users'].has_group(cr, uid, 'ierp_hr.group_headmans')
                if_hrmanagers = self.pool['res.users'].has_group(cr, uid, 'base.group_hr_manager')
                if_GED = self.pool['res.users'].has_group(cr, uid, 'ierp_hr.group_executive_director')
                
                if if_hrmanagers or if_GED:
                    self.message_subscribe_users(cr, uid, [record.id], user_ids=[record.user_id.id], context=context)
                    print 'yyyyyyy'
                elif record.employee_id and record.employee_id.parent_id and record.employee_id.parent_id.user_id:
                    self.message_subscribe_users(cr, uid, [record.id], user_ids=[record.employee_id.parent_id.user_id.id], context=context)
                    print 'ммммммммммммм'
            #===================================================================
            # elif record.holiday_type == 'category':
            #     obj_employee = self.pool.get('hr.employee')
            #     emp_ids = obj_employee.search(cr, uid, [('active','=',True),('category_ids','in',record.category_id.id)])
            #     print 'mmmmmm', emp_ids
            #     emps = obj_employee.browse(cr, uid, emp_ids)
            #     print 'gggggggggg', emps
            #     i = 0
            #     for emp in emps:
            #         i=i+1
            #         self.message_subscribe_users(cr, uid, [record.id], user_ids=[emp.parent_id.user_id.id], context=context)
            #     print 'llllllli', i
            #===================================================================
        return self.write(cr, uid, ids, {'state': 'confirm'})
    
    def holidays_validate(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid,ids)[0]
        obj_employee = self.pool.get('hr.employee')
        decr_id = obj_employee.search(cr, uid, [('id','=',obj.employee_id.id)])
        decr = obj_employee.browse(cr, uid, decr_id)
        if decr.user_id.id == uid:
            raise osv.except_osv(_('Warning!'), _('Та өөрийн амралтыг батлах эрхгүй, таныг хариуцсан менежер батална, та хариуцсан менежертээ хандана уу!!!'))
        
        for record1 in self.browse(cr, uid, ids, context=context):
            if_headmans_helper = self.pool['res.users'].has_group(cr, uid, 'ierp_hr.group_headmans_helper')
            if_headmans = self.pool['res.users'].has_group(cr, uid, 'ierp_hr.group_headmans')
            if_hrmanagers = self.pool['res.users'].has_group(cr, uid, 'base.group_hr_manager')
            #GED = Group Executive Director
            if_GED = self.pool['res.users'].has_group(cr, uid, 'ierp_hr.group_executive_director')
            confirm_ok = ''
            if if_headmans or if_headmans_helper:
                #Albanii darga mon eseh
                if if_GED == False:
                    if record1.number_of_days_temp < 2:
                        confirm_ok = 'Yes'
                    elif 1 < record1.number_of_days_temp < 7:
                        confirm_ok = 'Do hrmanagers'
                    else:
                        confirm_ok = 'Do GED'
                        
                #Gvitsetgeh zahiral mon eseh
                else:
                    confirm_ok = 'Yes'
                    
            elif if_hrmanagers:
                if record1.number_of_days_temp < 7:
                    confirm_ok = 'Yes'
                else:
                    confirm_ok = 'Do GED'
            
            
            obj_emp = self.pool.get('hr.employee')
            ids2 = obj_emp.search(cr, uid, [('user_id', '=', uid)])
            manager = ids2 and ids2[0] or False
            self.write(cr, uid, ids, {'state':'validate'})
            data_holiday = self.browse(cr, uid, ids)
            for record in data_holiday:
                if record.double_validation:
                    self.write(cr, uid, [record.id], {'manager_id2': manager})
                else:
                    self.write(cr, uid, [record.id], {'manager_id': manager})
                if record.holiday_type == 'employee' and record.type == 'remove':
                    if confirm_ok == 'Yes':
                        meeting_obj = self.pool.get('calendar.event')
                        meeting_vals = {
                            'name': record.name or _('Leave Request'),
                            'categ_ids': record.holiday_status_id.categ_id and [(6,0,[record.holiday_status_id.categ_id.id])] or [],
                            'duration': record.number_of_days_temp * 8,
                            'description': record.notes,
                            'user_id': record.user_id.id,
                            'start': record.date_from,
                            'stop': record.date_to,
                            'allday': False,
                            'state': 'open',            # to block that meeting date in the calendar
                            'class': 'confidential'
                        }   
                        #Add the partner_id (if exist) as an attendee             
                        if record.user_id and record.user_id.partner_id:
                            meeting_vals['partner_ids'] = [(4,record.user_id.partner_id.id)]
                            
                        ctx_no_email = dict(context or {}, no_email=True)
                        meeting_id = meeting_obj.create(cr, uid, meeting_vals, context=ctx_no_email)
                        self._create_resource_leave(cr, uid, [record], context=context)
                        self.write(cr, uid, ids, {'meeting_id': meeting_id})
                        
                        #GADUUR AJIL baiwal irts rvv auto vvsgelt
                        if obj.is_out_work == True:
                            attendance_obj = self.pool.get('hr.attendance')
                            attendance_date_from = {
                                'name': record.date_from,
                                'employee_id': obj.employee_id.id,
                                'action': 'sign_in',
                            }
                            attendance_date_to = {
                                'name': record.date_to,
                                'employee_id': obj.employee_id.id,
                                'action': 'sign_out',
                            }
                            attendance_obj.create(cr, uid, attendance_date_from)
                            attendance_obj.create(cr, uid, attendance_date_to)
                            
                    elif confirm_ok == 'Do hrmanagers':
                        raise osv.except_osv(_('Warning!'), _('Амралтын хоног 2-оос 6-н хооронд байгаа тул хүний нөөцийн алба батална!!!'))
                    elif confirm_ok == 'Do GED':
                        raise osv.except_osv(_('Warning!'), _('Амралтын хоног 6-с их байгаа тул Гүйцэтгэх захирал батална!!!'))
                elif record.holiday_type == 'category':
                    emp_ids = obj_emp.search(cr, uid, [('active', '=', True),('category_ids', 'child_of', [record.category_id.id])])
                    leave_ids = []
                    for emp in obj_emp.browse(cr, uid, emp_ids):
                        vals = {
                            'name': record.name,
                            'type': record.type,
                            'holiday_type': 'employee',
                            'holiday_status_id': record.holiday_status_id.id,
                            'date_from': record.date_from,
                            'date_to': record.date_to,
                            'notes': record.notes,
                            'number_of_days_temp': record.number_of_days_temp,
                            'parent_id': record.id,
                            'employee_id': emp.id
                        }
                        leave_ids.append(self.create(cr, uid, vals, context=None))
                    for leave_id in leave_ids:
                        # TODO is it necessary to interleave the calls?
                        for sig in ('confirm', 'validate', 'second_validate'):
                            self.signal_workflow(cr, uid, [leave_id], sig)
            
        return True
    
    #------------------- def holidays_confirm(self, cr, uid, ids, context=None):
        #------------- for record in self.browse(cr, uid, ids, context=context):
             # if record.employee_id and record.employee_id.parent_id and record.employee_id.parent_id.user_id:
                 # self.message_subscribe_users(cr, uid, [record.id], user_ids=[record.employee_id.parent_id.user_id.id], context=context)
        #----------------- return self.write(cr, uid, ids, {'state': 'confirm'})
    
    
    
class hr_holiday_days(osv.osv):
    _name = 'hr.holiday.days'
    _columns = {
               'name':fields.char('Name'),
               'min_year':fields.integer('Min year', required = True),
               'max_year':fields.integer('Max year', required = True),
               'get_days':fields.integer('Get days', required = True),
               'base_days':fields.integer('Base days', required = True),
               }
    
class hr_attendance_machines(osv.osv):
    _name = 'hr.attendance.machines'
    _columns = {
               'name':fields.char('Name'),
               'ip_add':fields.char('IP address'),
               'user_name':fields.char('User name'),
               'user_password':fields.char('User password'),
               }

class hr_holidays_sick_type(osv.osv):
    _name = 'hr.holidays.sick.type'
    _columns = {
               'name':fields.char('Name', help="Зөвхөн Хүний нөөцийн албаны менежер эрх олгогдсон хүмүүс л засах боломжтой"),
               }
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid,ids)[0]
        if_hrmanagers = self.pool['res.users'].has_group(cr, uid, 'base.group_hr_manager')
        if obj.name != vals['name']:
            if if_hrmanagers:
                vals['name'] = vals['name']
            else:
                raise osv.except_osv(_('Анхааруулга!'), _('Та Өвчний шалтгааныг засч болохгүй. Зөвхөн үүсгэх эрхтэй тул зөв үүсгэж хэвшинэ үү!!! Заавал засах шаардлагатай бол системийн Администратор эсвэл Хүний нөөцийн менежертээ хандаж засуулна уу.'))
        
        return super(hr_holidays_sick_type, self).write(cr, uid, ids, vals, context=context)

class hr_holidays_status_inherit(osv.osv):
    _inherit = 'hr.holidays.status'
    _columns = {
               'name':fields.char('Name'),
               'is_vacations':fields.boolean('Is vacations'),
               'is_sick':fields.boolean('Is sick', help="Энэ амралтын төрөл нь зөвхөн <<Өвчний чөлөө>> бол сонгоно уу! "),
               'is_out_work':fields.boolean('Is out work', help="Энэ амралтын төрөл нь зөвхөн <<Гадуур ажил>> бол сонгоно уу! "),
               'payroll_holiday':fields.boolean('Payroll holiday', help="Энэ амралтын төрөл нь зөвхөн <<Цалинтай чөлөө>> бол сонгоно уу! "),
               'unpayroll_holiday':fields.boolean('Unpayroll holiday', help="Энэ амралтын төрөл нь зөвхөн <<Цалингүй чөлөө>> бол сонгоно уу! "),
               'official_work':fields.boolean('Official work', help="Энэ амралтын төрөл нь зөвхөн <<Тэмдэглэлт баярын өдрүүдэд>> бол сонгоно уу! "),
               }
    
    
    def onchange_holiday_type_limit(self, cr, uid, ids,limit, is_sick, is_out_work):
        result = {'value': {}}
        if limit == False:
            result['value']['is_sick'] = False
            result['value']['is_out_work'] = False
        return result
    
    def onchange_holiday_type_sick(self, cr, uid, ids,limit, is_sick, is_out_work):
        result = {'value': {}}
        if is_sick == True:
            result['value']['limit'] = True
            result['value']['is_out_work'] = False
        return result
    
    def onchange_holiday_type_out_work(self, cr, uid, ids,limit, is_sick, is_out_work):
        result = {'value': {}}
        if is_out_work == True:
            result['value']['limit'] = True
            result['value']['is_sick'] = False
        return result
    
    def onchange_holiday_type_payroll_holiday(self, cr, uid, ids,payroll_holiday, unpayroll_holiday):
        result = {'value': {}}
        if payroll_holiday == True:
            result['value']['unpayroll_holiday'] = False
        return result
    
    def onchange_holiday_type_unpayroll_holiday(self, cr, uid, ids,payroll_holiday, unpayroll_holiday):
        result = {'value': {}}
        if unpayroll_holiday == True:
            result['value']['payroll_holiday'] = False
        return result
  


class event_cost(osv.osv):
    _inherit = 'event.event'
    
    def _write_event_var_cost(self, cr, uid, ids,name, args, context = None):
        
        obj = self.browse(cr, uid,ids)[0]
        event_line_obj = self.pool.get('event.line')
        line_id = event_line_obj.search(cr, uid, [('event_line_id','=',obj.id)])
        sum_price = 0.0
        for eline in line_id:
            event_line_id = event_line_obj.browse(cr, uid, eline)[0]
            sum_price += event_line_id.unit_price * event_line_id.unit_number
        return {ids[0]:obj.event_budges - sum_price}
    
    _columns = {
               'name':fields.char('Ename'),
               'event_budges':fields.float('Event budges', required=True),
               'event_real_cost':fields.float('Event real cost'),
               'event_var_cost':fields.function(_write_event_var_cost,type="float",string='Event var cost'),
               'line_id':fields.one2many('event.line','event_line_id','Line'),
               }
       
class event_line(osv.osv):
    _name = 'event.line'
    
    def _write_total_price(self, cr, uid, ids,name, args, context = None):
        obj = self.browse(cr, uid,ids)
        res = {}
        sum_price = 0.0
        for line in obj:
             sum_price = line.unit_price * line.unit_number
             res[line.id] = sum_price
        return res
    
    _columns = {
               'name':fields.char('Lname'), 
               'cost_name':fields.char('Cost name', required=True),
               'measure':fields.char('Measure'),
               'unit_price':fields.float('Unit price', required=True),
               'unit_number':fields.float('Unit number', required=True),
               'total_price':fields.function(_write_total_price,type="float",string='Total price'),
               'event_line_id':fields.many2one('event.event','Event line'),
               }

class res_company_inherit(osv.osv):
    _inherit = 'res.company'
    _columns = {
               'company_code':fields.integer('Company code', size = 4),
               'is_swiss':fields.boolean('Is swiss'),
               #'contract_cipher':fields.char('Contract cipher'),
               }
    _defaults = {
                'is_swiss': False,
                }
    


class common_procedure(osv.osv):
    _name = 'common.procedure'
    _columns = {
               'code':fields.char('Code', help="Журмын дугаарлалт / кодлолт..."),
               'name':fields.char('Name', help="Журмын нэр"),
               'procedure':fields.html('Procedure', help="Журмын агуулга testtest", copy=True, paste=False),
               }
    def action_to_upgrade_self(self, cr, uid, ids, context=None):
        #print 'yyyyyy', self
        mod_id = self.pool.get('ir.module.module').search(cr, uid, [('name','=','ierp_hr')], context=context)
        modobj = self.pool.get('ir.module.module').browse(cr,uid,mod_id)
        #print 'uuuuuuuuuuuuu', mod_id, 'fffffffff', modobj.description_html
        """
        Upgrade the selected module(s) immediately and fully,
        return the next res.config action to execute
        """
        modobj.button_immediate_upgrade()
        return True
	

class routing_slip(osv.osv):
    _name = 'routing.slip'
    _description = 'Routing slip'
    _columns = {
               'employee_id':fields.many2one('hr.employee','Employee', copy=False),
               'last_name':fields.char('Last name'),
               'job_id':fields.many2one('hr.job','Job'),
               'accession_date':fields.date('Accession date'),
               'left_date':fields.date('Left date'),
               'out_reason_id':fields.many2one('out.reason','Out reason'),
               'routing_date':fields.date('Routing date'),
               'slip_line_id': fields.one2many('routing.slip.line','routing_slip_id','Line'),
               'state': fields.selection([('routing_print','Routing print'),#Тойрох хэвлэж өгсөн
                                    ('routing_ended','Routing ended'),#Тойрох зуруулж дууссан
                                    ('leaved_job','Leaved job'),#Ажлаас гарсан
                                    ],'State'),#Төлөв
        }
    _sql_constraints = [
        ('employee_id', 'unique("employee_id")', 'Field employee_id must be unique.'),
        ]
    _defaults = {
                'state' : 'routing_print',
                'routing_date' : date.today(),
        }
    
    def to_ended(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'routing_ended'})
        return True
    
    def to_leaved(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state': 'leaved_job'})
        return True
    
    #@api.one
    def onchange_employee_id(self, cr, uid, ids, employee_id, context=None):
        obj = self.browse(cr, uid, ids)
        res = {}
        list = []
        if employee_id:
            employee_obj = self.pool.get('hr.employee')
            emp_id = employee_obj.search(cr, uid, [('id','=',employee_id)])
            emp = employee_obj.browse(cr, uid, emp_id)
            if emp:
                res['last_name'] = emp.last_name
                res['job_id'] = emp.job_id.id
                res['accession_date'] = emp.accession_date
            if emp.job_id:
                rm_obj = self.pool.get('routing.master')
                rm_id = rm_obj.search(cr, uid, [('job_id','=',emp.job_id.id)])
                rm = rm_obj.browse(cr, uid, rm_id)
                rsl_obj = self.pool.get('routing.slip.line')
                if rm:
                    for line in rm.master_line_id:
                        line_values = {
                            'code': line.code,
                            'job_id': line.job_id.id,
                            'employee_id': line.employee_id.id,
                            'date': line.date,
                            'note': line.note,
                            'signature': line.signature
                            }
                        list.append(line_values)
                        res['slip_line_id'] = list
                else:
                    res['slip_line_id'] = False
        else:
            res['last_name'] = False
            res['job_id'] = False
            res['accession_date'] = False
            res['slip_line_id'] = False
        #self.update({'slip_line_id': list })
        return {'value':res}
        
    @api.multi
    def refresh_page(self):
        return { 'type': 'ir.actions.client', 'tag': 'reload'}
    
    #===========================================================================
    # def create(self, cr, uid, vals, context=None):
    #     res ={}
    #     context = dict(context or {})
    #     
    #     routing_id = super(routing_slip, self).create(cr, uid, vals, context=context)
    #     return routing_id
    #     self.refresh_page()
    #===========================================================================
    
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        line_obj = self.pool.get('routing.slip.line')
        for line in obj.slip_line_id:
            line_obj.unlink(cr, uid, line.id)
        return super(routing_slip, self).unlink(cr, uid, ids, context=context)

class routing_slip_line(osv.osv):
    _name = 'routing.slip.line'
    _description = 'Routing slip line'
    _columns = {
                'routing_slip_id':fields.many2one('routing.slip','Routing slip'),
                'code':fields.integer('Code'),
                'job_id':fields.many2one('hr.job','Job'),
                'employee_id':fields.many2one('hr.employee','Employee'),
                'date':fields.date('Date'),
                'note':fields.char('Note'),
                'signature':fields.char('Signature'),
        }
    #===========================================================================
    # _sql_constraints = [
    #     ('job_id', 'unique("job_id")', 'Line field job_id must be unique.'),
    #     ]
    #===========================================================================
    
    def onchange_job_id(self, cr, uid, ids, job_id, context=None):
        obj = self.browse(cr, uid, ids)
        res = {}
        if job_id:
            job_obj = self.pool.get('hr.job')
            job_id = job_obj.search(cr, uid, [('id','=',job_id)])
            job = job_obj.browse(cr, uid, job_id)
            if job:
                res['code'] = job.job_code
                #print 'nnnnnnnnnn', job_id
                employee_obj = self.pool.get('hr.employee')
                emp_id = employee_obj.search(cr, uid, [('job_id','in',job_id)])
                emp = employee_obj.browse(cr, uid, emp_id)
                if len(emp_id) == 1:
                    res['employee_id'] = emp.id
                    
                #emp = employee_obj.browse(cr, uid, emp_id)
            
        return {'value':res}

class routing_master(osv.osv):
    _name = 'routing.master'
    _description = 'Routing master'
    _columns = {
               'job_id':fields.many2one('hr.job','Job', copy=False),
               'master_line_id': fields.one2many('routing.master.line','routing_master_id','Line', copy=False ),
        }
    _sql_constraints = [
        ('job_id', 'unique("job_id")', 'Field job_id must be unique.'),
        ]
    
    def create(self, cr, uid, vals, context=None):
        res ={}
        context = dict(context or {})
        
        routing_id = super(routing_master, self).create(cr, uid, vals, context=context)
        return routing_id
    
    def unlink(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)[0]
        line_obj = self.pool.get('routing.master.line')
        for line in obj.master_line_id:
            line_obj.unlink(cr, uid, line.id)
        return super(routing_master, self).unlink(cr, uid, ids, context=context)

class routing_master_line(osv.osv):
    _name = 'routing.master.line'
    _description = 'Routing master line'
    _columns = {
                'routing_master_id':fields.many2one('routing.master','Routing master'),
                'code':fields.integer('Code'),
                'job_id':fields.many2one('hr.job','Job', required=True),
                'employee_id':fields.many2one('hr.employee','Employee'),
                'date':fields.date('Date'),
                'note':fields.char('Note'),
                'signature':fields.char('Signature'),
        }
    #===========================================================================
    # _sql_constraints = [
    #     ('job_id', 'unique("job_id")', 'Line field job_id must be unique.'),
    #     ]
    #===========================================================================
    
    def create(self, cr, uid, vals, context=None):
        res ={}
        context = dict(context or {})
        
        master_id = super(routing_master_line, self).create(cr, uid, vals, context=context)
        return master_id
    
    def onchange_job_id(self, cr, uid, ids, job_id, context=None):
        obj = self.browse(cr, uid, ids)
        res = {}
        if job_id:
            job_obj = self.pool.get('hr.job')
            job_id = job_obj.search(cr, uid, [('id','=',job_id)])
            job = job_obj.browse(cr, uid, job_id)
            if job:
                res['code'] = job.job_code
                #print 'nnnnnnnnnn', job_id
                employee_obj = self.pool.get('hr.employee')
                emp_id = employee_obj.search(cr, uid, [('job_id','in',job_id)])
                emp = employee_obj.browse(cr, uid, emp_id)
                if len(emp_id) == 1:
                    res['employee_id'] = emp.id
                    
                #emp = employee_obj.browse(cr, uid, emp_id)
            
        return {'value':res}
    
    
    
    
    
    
    
    
    
    
    
#===============================================================================
#     
#     
# 
# class test_slip(models.Model):
#     _name = 'test.slip'
#     _description = 'Routing slip'
#     _columns = {
#                'employee_id':fields.many2one('hr.employee','Employee', copy=False),
#                'last_name':fields.char('Last name'),
#                'job_id':fields.many2one('hr.job','Job'),
#                'accession_date':fields.date('Accession date'),
#                'left_date':fields.date('Left date'),
#                'out_reason_id':fields.many2one('out.reason','Out reason'),
#                'routing_date':fields.date('Routing date'),
#                'slip_line_id': fields.one2many('test.slip.line','routing_slip_id','Line'),
#                'state': fields.selection([('routing_print','Routing print'),#Тойрох хэвлэж өгсөн
#                                     ('routing_ended','Routing ended'),#Тойрох зуруулж дууссан
#                                     ('leaved_job','Leaved job'),#Ажлаас гарсан
#                                     ],'State'),#Төлөв
#         }
#     
#     _defaults = {
#                 'state' : 'routing_print',
#                 'routing_date' : date.today(),
#         }
#     
#     def to_ended(self, cr, uid, ids, *args):
#         self.write(cr, uid, ids, {'state': 'routing_ended'})
#         return True
#     
#     def to_leaved(self, cr, uid, ids, *args):
#         self.write(cr, uid, ids, {'state': 'leaved_job'})
#         return True
#     
#     def onchange_employee_id(self, cr, uid, ids, employee_id, context=None):
#         obj = self.browse(cr, uid, ids)
#         res = {}
#         list = []
#         
#         res1 = False
#         res2 = False
#         res3 = False
#         res4 = False
#         
#         rsl2_obj = self.pool.get('test.slip.line')
#         
#         if employee_id:
#             employee_obj = self.pool.get('hr.employee')
#             emp_id = employee_obj.search(cr, uid, [('id','=',employee_id)])
#             emp = employee_obj.browse(cr, uid, emp_id)
#             if emp:
#                 res1 = emp.last_name
#                 res2 = emp.job_id.id
#                 res3 = emp.accession_date
#                 #===============================================================
#                 # res['last_name'] = emp.last_name
#                 # res['job_id'] = emp.job_id.id
#                 # res['accession_date'] = emp.accession_date
#                 #===============================================================
#             if emp.job_id:
#                 rm_obj = self.pool.get('routing.master')
#                 rm_id = rm_obj.search(cr, uid, [('job_id','=',emp.job_id.id)])
#                 rm = rm_obj.browse(cr, uid, rm_id)
#                 rsl_obj = self.pool.get('test.slip.line')
#                 if rm:
#                     for line in rm.master_line_id:
#                         line_values = {
#                             'code': line.code,
#                             'job_id': line.job_id.id,
#                             'employee_id': line.employee_id.id,
#                             'date': line.date,
#                             'note': line.note,
#                             'signature': line.signature
#                             }
#                         #list.append(self.create(cr, uid, line_values, context=None))
#                         list.append([0, 0, line_values])
#                         #res['slip_line_id'] = list
#                         #slip_line_id.append(self.create(cr, uid, list, context=None))
#                         #self.create = (cr, uid, res4, context=None)
#                         res4 = list
#                 else:
#                     #res['slip_line_id'] = False
#                     res4 = False
#         else:
#             res1 = False
#             res2 = False
#             res3 = False
#             res4 = False
#         
#         res['last_name'] = res1
#         res['job_id'] = res2
#         res['accession_date'] = res3
#         res['slip_line_id'] = res4
#         #self.write(cr,uid,id, {'slip_line_id': list})
#         
#         #list.append([0, 0, move_line])
#         #print list
#         #self.update({'slip_line_id': list })
#         return {'value':res}
#     
#     def unlink(self, cr, uid, ids, context=None):
#         obj = self.browse(cr, uid, ids)[0]
#         line_obj = self.pool.get('test.slip.line')
#         for line in obj.slip_line_id:
#             line_obj.unlink(cr, uid, line.id)
#         return super(test_slip, self).unlink(cr, uid, ids, context=context)
# 
# class test_slip_line(models.Model):
#     _name = 'test.slip.line'
#     _description = 'Routing slip line'
#     _columns = {
#                 'routing_slip_id':fields.many2one('test.slip','Routing slip'),
#                 'code':fields.integer('Code'),
#                 'job_id':fields.many2one('hr.job','Job'),
#                 'employee_id':fields.many2one('hr.employee','Employee'),
#                 'date':fields.date('Date'),
#                 'note':fields.char('Note'),
#                 'signature':fields.char('Signature'),
#         }
#     #===========================================================================
#     # _sql_constraints = [
#     #     ('job_id', 'unique("job_id")', 'Line field job_id must be unique.'),
#     #     ]
#     #===========================================================================
#     
#     def onchange_job_id(self, cr, uid, ids, job_id, context=None):
#         obj = self.browse(cr, uid, ids)
#         res = {}
#         if job_id:
#             job_obj = self.pool.get('hr.job')
#             job_id = job_obj.search(cr, uid, [('id','=',job_id)])
#             job = job_obj.browse(cr, uid, job_id)
#             if job:
#                 res['code'] = job.job_code
#                 print 'nnnnnnnnnn', job_id
#                 employee_obj = self.pool.get('hr.employee')
#                 emp_id = employee_obj.search(cr, uid, [('job_id','in',job_id)])
#                 emp = employee_obj.browse(cr, uid, emp_id)
#                 if len(emp_id) == 1:
#                     res['employee_id'] = emp.id
#                     
#                 #emp = employee_obj.browse(cr, uid, emp_id)
#             
#         return {'value':res}
#===============================================================================
