# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging

from openerp import SUPERUSER_ID
from openerp import tools
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools.translate import _
import time
from openerp.tools import float_compare, float_is_zero


class hr_salary_rule(osv.osv):
    '''Payroll inherit'''

    _inherit = 'hr.salary.rule'
    _description = 'Payroll inherit'

    _columns = {
        #'name':fields.char('Name', required=True, readonly=False),    
        'line_id':fields.many2one('hr.payroll.end.line', 'Hr payroll end line'),
        'payroll_end_id':fields.many2one('hr.payroll.end','Hr payroll end'),
        'incentive_id':fields.many2one('incentive.line', 'Incentive line'),
        'salary_incentive_id':fields.many2one('salary.incentive','Salary incentive'),
        'deduction_id':fields.many2one('deduction.line','Deduction line'),
        'salary_deduction_id':fields.many2one('salary.deduction','Salary deduction'),
        'hr_advance_salary_line_id': fields.many2one('hr.advance.salary.line', 'Employee'),
        'advance_id':fields.many2one('hr.advance.salary','Employee'),
        'employee_info_id':fields.many2one('employee.info','Employee info'),
        'hr_hour_id':fields.many2one('hr.hour.record','Hr hour'),
        #'sick_benefit_id':fields.many2one('sick.benefit','Sick benefit'),
        #'sick_line_id': fields.many2one('sick.line', 'Sick line'),
    }
    
    _defaults = {
        'amount_python_compute': '''
# Available variables:
#----------------------
# payslip: object containing the payslips
# employee: hr.employee object
# contract: hr.contract object
# line: hr.payroll.end.line object
# end: hr.payroll.end object
# incentive: incentive.line object
# deduction: deduction.line object
# advance: hr.advance.salary.line object
# hour: employee.info object
# rules: object containing the rules code (previously computed)
# categories: object containing the computed salary rule categories (sum of amount of all rules belonging to that category).
# worked_days: object containing the computed worked days.
# inputs: object containing the computed inputs.

# Note: returned value have to be set in the variable 'result'

result = contract.wage * 0.10''',
        'condition_python':
'''
# Available variables:
#----------------------
# payslip: object containing the payslips
# employee: hr.employee object
# contract: hr.contract object
# line: hr.payroll.end.line object
# end: hr.payroll.end object
# incentive: incentive.line object
# deduction: deduction.line object
# advance: hr.advance.salary.line object
# hour: employee.info object
# rules: object containing the rules code (previously computed)
# categories: object containing the computed salary rule categories (sum of amount of all rules belonging to that category).
# worked_days: object containing the computed worked days
# inputs: object containing the computed inputs

# Note: returned value have to be set in the variable 'result'

result = rules.NET > categories.NET * 0.10''',
        'condition_range': 'contract.wage',
        'sequence': 5,
        'appears_on_payslip': True,
        'active': True,
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id,
        'condition_select': 'none',
        'amount_select': 'fix',
        'amount_fix': 0.0,
        'amount_percentage': 0.0,
        'quantity': '1.0',
     }
    
#class hr_payslip_line(osv.osv):
    #'''
    #Payslip Line inherit
    #'''

    #_inherit = 'hr.payslip.line'
    #_description = 'Payslip Line inherit'
    #_order = 'contract_id, sequence'

    #_columns = {
        #'line_id':fields.many2one('hr.payroll.end.line', 'Hr payroll end line'),
        #'payroll_end_id':fields.many2one('hr.payroll.end','Hr payroll end'),
    #}
    
class hr_payslip(osv.osv):
    '''
    Pay Slip
    '''

    _inherit = 'hr.payslip'
    _description = 'Pay Slip' 
    
    _columns = {
        'line_id':fields.many2one('hr.payroll.end.line', 'Hr payroll end line'),
        'payroll_end_id':fields.many2one('hr.payroll.end','Hr payroll end'),
        'incentive_id':fields.many2one('incentive.line', 'Incentive line'),
        'salary_incentive_id':fields.many2one('salary.incentive','Salary incentive'),
        'deduction_id':fields.many2one('deduction.line','Deduction line'),
        'salary_deduction_id':fields.many2one('salary.deduction','Salary deduction'),
        'hr_advance_salary_line_id': fields.many2one('hr.advance.salary.line', 'Employee'),
        'advance_id':fields.many2one('hr.advance.salary','Employee'),
        'employee_info_id':fields.many2one('employee.info','Employee info'),
        'hr_hour_id':fields.many2one('hr.hour.record','Hr hour'),
        #'sick_benefit_id':fields.many2one('sick.benefit','Sick benefit'),
        #'sick_line_id': fields.many2one('sick.line', 'Sick line'),
    }
    
    def get_payslip_lines(self, cr, uid, contract_ids, payslip_id, context):
        def _sum_salary_rule_category(localdict, category, amount):
            if category.parent_id:
                localdict = _sum_salary_rule_category(localdict, category.parent_id, amount)
            localdict['categories'].dict[category.code] = category.code in localdict['categories'].dict and localdict['categories'].dict[category.code] + amount or amount
            return localdict

        class BrowsableObject(object):
            def __init__(self, pool, cr, uid, employee_id, dict):
                self.pool = pool
                self.cr = cr
                self.uid = uid
                self.employee_id = employee_id
                self.dict = dict

            def __getattr__(self, attr):
                return attr in self.dict and self.dict.__getitem__(attr) or 0.0

        class InputLine(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(amount) as sum\
                            FROM hr_payslip as hp, hr_payslip_input as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()[0]
                return res or 0.0

        class WorkedDays(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def _sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(number_of_days) as number_of_days, sum(number_of_hours) as number_of_hours\
                            FROM hr_payslip as hp, hr_payslip_worked_days as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done'\
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                return self.cr.fetchone()

            def sum(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[0] or 0.0

            def sum_hours(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[1] or 0.0

        class Payslips(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""

            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                self.cr.execute("SELECT sum(case when hp.credit_note = False then (pl.total) else (-pl.total) end)\
                            FROM hr_payslip as hp, hr_payslip_line as pl \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pl.slip_id AND pl.code = %s",
                            (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()
                return res and res[0] or 0.0

        #we keep a dict with the result because a value can be overwritten by another rule with the same code
        result_dict = {}
        rules = {}
        categories_dict = {}
        blacklist = []
        salary_inc_obj = self.pool.get('salary.incentive')
        salary_ded_obj = self.pool.get('salary.deduction')
        salary_adv_obj = self.pool.get('hr.advance.salary')
        hour_rec_obj = self.pool.get('hr.hour.record')
        #sick_obj = self.pool.get('sick.benefit')
        payslip_obj = self.pool.get('hr.payslip')
        inputs_obj = self.pool.get('hr.payslip.worked_days')
        obj_rule = self.pool.get('hr.salary.rule')
        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)
        worked_days = {}
        for worked_days_line in payslip.worked_days_line_ids:
            worked_days[worked_days_line.code] = worked_days_line
        inputs = {}
        for input_line in payslip.input_line_ids:
            inputs[input_line.code] = input_line

        categories_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, categories_dict)
        input_obj = InputLine(self.pool, cr, uid, payslip.employee_id.id, inputs)
        worked_days_obj = WorkedDays(self.pool, cr, uid, payslip.employee_id.id, worked_days)
        payslip_obj = Payslips(self.pool, cr, uid, payslip.employee_id.id, payslip)
        rules_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, rules)

        baselocaldict = {'categories': categories_obj, 'rules': rules_obj, 'payslip': payslip_obj, 'worked_days': worked_days_obj, 'inputs': input_obj}
        #get the ids of the structures on the contracts and their parent id as well
        structure_ids = self.pool.get('hr.contract').get_all_structures(cr, uid, contract_ids, context=context)
        #get the rules of the structure and thier children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, context=context)
        #run the rules by sequence
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        for contract in self.pool.get('hr.contract').browse(cr, uid, contract_ids, context=context):
            employee = contract.employee_id
            #line_id = self.pool.get('hr.payroll.end.line').search(cr, uid, [('employee_id','=',contract.employee_id.id)])
            #if line_id != []:
                #lin_id = self.pool.get('hr.payroll.end.line').browse(cr, uid, line_id)[0]
            sal_inc_id = salary_inc_obj.search(cr, uid, [('name','=',payslip.date_from),('end_date','=',payslip.date_to)])
            #print "aaldaaa",sal_inc_id
            incentive_id = self.pool.get('incentive.line').search(cr, uid, [('hr_employee_id','=',contract.employee_id.id),('salary_incentive_id','=',sal_inc_id)])
            inc_id = ''
            if incentive_id != []:
                inc_id = self.pool.get('incentive.line').browse(cr, uid, incentive_id)[0]
            
            sal_ded_id = salary_ded_obj.search(cr, uid, [('name','=',payslip.date_from),('end_date','=',payslip.date_to)])    
            #print "laadddd",sal_ded_id
            deduction_id = self.pool.get('deduction.line').search(cr, uid, [('hr_employee_id','=',contract.employee_id.id),('salary_deduction_id','=',sal_ded_id)])
            ded_id = ''
            if deduction_id != []:
                ded_id = self.pool.get('deduction.line').browse(cr, uid, deduction_id)[0]
            
            sal_adv_id = salary_adv_obj.search(cr, uid, [('start_date','=',payslip.date_from)])  
            #print "addd",    sal_adv_id
            hr_advance_salary_line_id = self.pool.get('hr.advance.salary.line').search(cr, uid, [('employee_id','=',contract.employee_id.id),('advance_id','=',sal_adv_id)])
            if hr_advance_salary_line_id != []:
                ad_id = self.pool.get('hr.advance.salary.line').browse(cr, uid, hr_advance_salary_line_id)[0]
                
            #sick_id = sick_obj.search(cr, uid, [('date','=',payslip.date_from)])  
            #print "addd",    sick_id
            #sick_benefit_id = self.pool.get('sick.benefit').search(cr, uid, [('hr_employee_id','=',contract.employee_id.id),('id','=',sick_id)])
            #if sick_benefit_id != []:
                #si_id = self.pool.get('sick.benefit').browse(cr, uid, sick_benefit_id)[0]   
            
            hour_rec_id = hour_rec_obj.search(cr, uid, [('start_date','=',payslip.date_from),('end_date','=',payslip.date_to)])  
            #print "lllllllllllllllll", hour_rec_id 
            employee_info_id = self.pool.get('employee.info').search(cr, uid, [('employee_id','=',contract.employee_id.id),('hr_hour_id','=',hour_rec_id)])
            #print "addddddddddddddddddddddddd",employee_info_id
            if employee_info_id != []:
                ho_id = self.pool.get('employee.info').browse(cr, uid, employee_info_id)[0]
                localdict = dict(baselocaldict, employee=employee, contract=contract, hour=ho_id,advance=ad_id,deduction=ded_id, incentive=inc_id)            
                    
            else:
                localdict = dict(baselocaldict, employee=employee, contract=contract)
            #===================================================================
            # line_id = self.pool.get('hr.payroll.end.line')
            # line = line_id.browse(cr, uid, payroll_end_id, context=context)
            #===================================================================
            
            #localdict = dict(baselocaldict, employee=employee, contract=contract, line=lin_id)
            for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
                key = rule.code + '-' + str(contract.id)
                localdict['result'] = None
                localdict['result_qty'] = 1.0
                localdict['result_rate'] = 100
                #check if the rule can be applied
                if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                    #compute the amount of the rule
                    amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)
                    #sal_inc_id = salary_inc_obj.search(cr, uid, [('name','=',payslip.date_from),('end_date','=',payslip.date_to)])
                    #if sal_inc_id != []:
                        #inc_id = salary_inc_obj.browse(cr, uid, sal_inc_id)
                        #for line in inc_id.incentive_id:
                            #if line.hr_employee_id.id == payslip.employee_id.id:
                                #amount = line.result_percent
                    #check if there is already a rule computed with that code
                    previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                    #set/overwrite the amount computed for this rule in the localdict
                    tot_rule = amount * qty * rate / 100.0
                    localdict[rule.code] = tot_rule
                    rules[rule.code] = rule
                    #sum the amount for its salary category
                    localdict = _sum_salary_rule_category(localdict, rule.category_id, tot_rule - previous_amount)
                    #create/overwrite the rule in the temporary results
                    result_dict[key] = {
                        'salary_rule_id': rule.id,
                        'contract_id': contract.id,
                        'name': rule.name,
                        'code': rule.code,
                        'category_id': rule.category_id.id,
                        'sequence': rule.sequence,
                        'appears_on_payslip': rule.appears_on_payslip,
                        'condition_select': rule.condition_select,
                        'condition_python': rule.condition_python,
                        'condition_range': rule.condition_range,
                        'condition_range_min': rule.condition_range_min,
                        'condition_range_max': rule.condition_range_max,
                        'amount_select': rule.amount_select,
                        'amount_fix': rule.amount_fix,
                        'amount_python_compute': rule.amount_python_compute,
                        'amount_percentage': rule.amount_percentage,
                        'amount_percentage_base': rule.amount_percentage_base,
                        'register_id': rule.register_id.id,
                        'amount': amount,
                        'employee_id': contract.employee_id.id,
                        'quantity': qty,
                        'rate': rate,
                    }
                else:
                    #blacklist this rule and its children
                    blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result   
    
    
class hr_payroll_structure(osv.osv):
    """
#    Salary structure used to defined
#    - Basic
#    - Allowances
#    - Deductions
#    """

    _inherit = 'hr.payroll.structure'
    _description = 'Salary Structure'
    _columns = {
        'insured_type':fields.char('Insured type'),
    }   
    
    
class hr_payslip_run(osv.osv):

    _inherit = 'hr.payslip.run'
    _description = 'Payslip Batches'
    _columns = {
    }
    
    def compute_salary(self, cr, uid, ids, context=None):
        self.compute_sheet(cr, uid, ids, context)
        hr_payslip_pool = self.pool.get('hr.payslip')
        obj = self.browse(cr, uid,ids)
        payslip_id = hr_payslip_pool.search(cr, uid, [('payslip_run_id', '=', obj.id)], context=context)
     
        
        
        for payslip in hr_payslip_pool.browse(cr, uid, payslip_id, context=context):
             
                                                             
            hr_payslip_pool.write(cr, uid, payslip.id, {'state': 'done'}, context=context) 
            
            
            move_pool = self.pool.get('account.move')
            period_pool = self.pool.get('account.period')
            precision = self.pool.get('decimal.precision').precision_get(cr, uid, 'Payroll')
            timenow = time.strftime('%Y-%m-%d')
            
            line_ids = []
            debit_sum = 0.0
            credit_sum = 0.0
            if not payslip.period_id:
                search_periods = period_pool.find(cr, uid, payslip.date_to, context=context)
                period_id = search_periods[0]
            else:
                period_id = payslip.period_id.id

            default_partner_id = payslip.employee_id.address_home_id.id
            name = _('Payslip of %s') % (payslip.employee_id.name)
            move = {
                'narration': name,
                'date': timenow,
                'ref': payslip.number,
                'journal_id': payslip.journal_id.id,
                'period_id': period_id,
            }
            for line in payslip.details_by_salary_rule_category:
                amt = payslip.credit_note and -line.total or line.total
                if float_is_zero(amt, precision_digits=precision):
                    continue
                partner_id = line.salary_rule_id.register_id.partner_id and line.salary_rule_id.register_id.partner_id.id or default_partner_id
                debit_account_id = line.salary_rule_id.account_debit.id
                credit_account_id = line.salary_rule_id.account_credit.id

                if debit_account_id:

                    debit_line = (0, 0, {
                    'name': line.name,
                    'date': timenow,
                    'partner_id': (line.salary_rule_id.register_id.partner_id or line.salary_rule_id.account_debit.type in ('receivable', 'payable')) and partner_id or False,
                    'account_id': debit_account_id,
                    'journal_id': payslip.journal_id.id,
                    'period_id': period_id,
                    'debit': amt > 0.0 and amt or 0.0,
                    'credit': amt < 0.0 and -amt or 0.0,
                    'analytic_account_id': line.salary_rule_id.analytic_account_id and line.salary_rule_id.analytic_account_id.id or False,
                    'tax_code_id': line.salary_rule_id.account_tax_id and line.salary_rule_id.account_tax_id.id or False,
                    'tax_amount': line.salary_rule_id.account_tax_id and amt or 0.0,
                })
                    line_ids.append(debit_line)
                    debit_sum += debit_line[2]['debit'] - debit_line[2]['credit']

                if credit_account_id:

                    credit_line = (0, 0, {
                    'name': line.name,
                    'date': timenow,
                    'partner_id': (line.salary_rule_id.register_id.partner_id or line.salary_rule_id.account_credit.type in ('receivable', 'payable')) and partner_id or False,
                    'account_id': credit_account_id,
                    'journal_id': payslip.journal_id.id,
                    'period_id': period_id,
                    'debit': amt < 0.0 and -amt or 0.0,
                    'credit': amt > 0.0 and amt or 0.0,
                    'analytic_account_id': line.salary_rule_id.analytic_account_id and line.salary_rule_id.analytic_account_id.id or False,
                    'tax_code_id': line.salary_rule_id.account_tax_id and line.salary_rule_id.account_tax_id.id or False,
                    'tax_amount': line.salary_rule_id.account_tax_id and amt or 0.0,
                })
                    line_ids.append(credit_line)
                    credit_sum += credit_line[2]['credit'] - credit_line[2]['debit']

            if float_compare(credit_sum, debit_sum, precision_digits=precision) == -1:
                acc_id = payslip.journal_id.default_credit_account_id.id
                if not acc_id:
                    raise osv.except_osv(_('Configuration Error!'),_('The Expense Journal "%s" has not properly configured the Credit Account!')%(slip.journal_id.name))
                adjust_credit = (0, 0, {
                    'name': _('Adjustment Entry'),
                    'date': timenow,
                    'partner_id': False,
                    'account_id': acc_id,
                    'journal_id': payslip.journal_id.id,
                    'period_id': period_id,
                    'debit': 0.0,
                    'credit': debit_sum - credit_sum,
                })
                line_ids.append(adjust_credit)

            elif float_compare(debit_sum, credit_sum, precision_digits=precision) == -1:
                acc_id = payslip.journal_id.default_debit_account_id.id
                if not acc_id:
                    raise osv.except_osv(_('Configuration Error!'),_('The Expense Journal "%s" has not properly configured the Debit Account!')%(slip.journal_id.name))
                adjust_debit = (0, 0, {
                    'name': _('Adjustment Entry'),
                    'date': timenow,
                    'partner_id': False,
                    'account_id': acc_id,
                    'journal_id': payslip.journal_id.id,
                    'period_id': period_id,
                    'debit': credit_sum - debit_sum,
                    'credit': 0.0,
                })
                line_ids.append(adjust_debit)

            move.update({'line_id': line_ids})
            move_id = move_pool.create(cr, uid, move, context=context)
            hr_payslip_pool.write(cr, uid, [payslip.id], {'move_id': move_id, 'period_id' : period_id}, context=context)
            if payslip.journal_id.entry_posted:
                move_pool.post(cr, uid, [move_id], context=context)
            self.write(cr, uid, obj.id, {'state': 'close'}, context=context) 
         
        return True
    
    def compute_sheet(self, cr, uid, ids, context=None):
        slip_line_pool = self.pool.get('hr.payslip.line')
        sequence_obj = self.pool.get('ir.sequence')
        hr_payslip_pool = self.pool.get('hr.payslip')
        obj = self.browse(cr, uid,ids)
        payslip_id = hr_payslip_pool.search(cr, uid, [('payslip_run_id', '=', obj.id)], context=context)
        
        
        for payslip in hr_payslip_pool.browse(cr, uid, payslip_id, context=context):
            number = payslip.number or sequence_obj.get(cr, uid, 'salary.slip')
            #delete old payslip lines
            old_slipline_ids = slip_line_pool.search(cr, uid, [('slip_id', '=', payslip.id)], context=context)
            #old_slipline_ids
            if old_slipline_ids:
                slip_line_pool.unlink(cr, uid, old_slipline_ids, context=context)
            if payslip.contract_id:
                #set the list of contract for which the rules have to be applied
                contract_ids = [payslip.contract_id.id]
            else:
                #if we don't give the contract, then the rules to apply should be for all current contracts of the employee
                contract_ids = hr_payslip_pool.get_contract(cr, uid, payslip.employee_id, payslip.date_from, payslip.date_to, context=context)
            lines = [(0,0,line) for line in self.pool.get('hr.payslip').get_payslip_lines(cr, uid, contract_ids, payslip.id, context=context)]
            hr_payslip_pool.write(cr, uid, [payslip.id], {'line_ids': lines, 'number': number,}, context=context)
        return True
    
    
    
    
    
    