# -*- coding: utf-8 -*-
import time
from datetime import datetime
from datetime import date
from dateutil.relativedelta import relativedelta

from openerp.osv import osv, fields
import openerp.netsvc
import openerp.pooler
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT

#
# Model definition
#
class close_loan_from_rf(osv.osv):
      
    STATE_SELECTION = [
        ('draft', 'Draft'),
        ('approved', 'Approved'),
        ('done', 'Done'),
        ('cancel', 'Cancelled')
    ]
    
    _columns = {
            'company_id':fields.many2one('res.company','Company'),
        'name': fields.char(u'Close risk number', size=64, select=True,readonly=True),
        'loan_order': fields.many2one('loan.order',u'Loan contract', domain=[('state','=','paid'),('loan_state','!=','d00')],states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'date_order':fields.date('Order Date', required=True, states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},help="Date on which this document has been created."),
        'date_approve':fields.date('Date Approved', readonly=1),
        'state': fields.selection(STATE_SELECTION, 'State', readonly=True, help="Help..."),        
        'partner_id':fields.many2one('res.partner', 'Debtor', required=True,states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]}),
        'notes': fields.text('Other',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]}),        
        'loan_balance':fields.float(u'Loan balance',digits = (16,2),states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'total_risk_amount':fields.float(u'Amount of risk',digits = (16,2),states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'risk_percent': fields.float(u'Percent of risk',digits = (16,2),states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'risk_account': fields.many2one('account.account',u'Account of risk foundation',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        
        'journal': fields.many2one('account.journal',u'Journal',states={'done':[('readonly',True)],'approved':[('readonly',True)]},required=True),
        
		'loan_current_state': fields.selection([
        				('d00','Normal'),
                        ('d90','Past due loans'),
                        ('d180','Abnormal loans'),
                        ('d360','Doubted loans'),
                        ('d361g','Bad loan')], u'Loan state(current)',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
		'approve_accountant': fields.many2one('hr.employee',u'Approved accountant',readonly=True),
    }
    _defaults = {
        'company_id':lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'date_order': fields.date.context_today,
        'state': 'draft',
    }
    _sql_constraints = [
        ('name_uniq', 'unique(name)', 'Close risk number must be unique!'),
    ]
    _name = "close.loan.from.rf"
    _description = "Close loan from risk foundation"
    _order = "name desc"

#   ---------------------------------MY----------------------------------------

    def onchange_loan_order(self, cr, uid, ids, lid):
        val = {}
        lorder = self.pool.get('loan.order').browse(cr,uid,lid)
        risk_ids = self.pool.get('loan.risk.foundation').search(cr,uid,[
		                            ('state','=','done'), 
		                            ('loan_order','=',lorder.id)          
		                         ],order="id")
        if risk_ids == []:
            raise osv.except_osv(_('Warning'), _("No risk foundation.This loan is normal."))
            return True
            
        tr = 0.0
        per = 0.0
        for risk in self.pool.get('loan.risk.foundation').browse(cr,uid,risk_ids):
            tr += risk.risk_amount
            per += risk.risk_percent
        
        val['partner_id'] = lorder.partner_id.id
        val['total_risk_amount'] = tr
        val['risk_percent'] = per
        val['loan_current_state'] = lorder.loan_state
        val['loan_balance'] = lorder.loan_balance
        
        return {'value':val}

    def _get_days(self,now,last,m):
		last_year = int(last[:4])
		last_mounth = int(last[5:7])+m
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		delta = d1 - d0
		return delta.days

#   ---------------------------------ERP----------------------------------------
    def create(self, cr, uid, vals, context=None):
        if vals['total_risk_amount'] <= 0:
            raise osv.except_osv(_('Invalid risk amount!'), _('Insert risk amount!'))
        if vals['risk_percent'] <= 0:
            raise osv.except_osv(_('Invalid risk percent!'), _('Insert risk period!'))
        return super(close_loan_from_rf, self).create(cr, uid, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        orders = self.read(cr, uid, ids, ['state'], context=context)
        unlink_ids = []
        for s in orders:
            if s['state'] in ['draft','cancel']:
                unlink_ids.append(s['id'])
            else:
                raise osv.except_osv(_('Invalid action !'), _('In order to delete a loan order, it must be cancelled first!'))
        return super(close_loan_from_rf, self).unlink(cr, uid, unlink_ids, context=context)

    def action_cancel_close_loan(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'draft'})
        print 'Cancel-------------'
        return True

    def action_approve_close_loan(self, cr, uid, ids, context=None):
        hrid = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
        if hrid !=[]:
            self.write(cr, uid, ids, {'state': 'approved', 
                                  'approve_accountant' : hrid[0], 
                                  'name':self.pool.get('ir.sequence').get(cr, uid, 'close.loan.from.rf'),
                                  'date_approve': fields.date.context_today(self,cr,uid,context=context),
                                  })
        else:
            raise osv.except_osv(_('Warning'), _('Хүний нөөц дээр энэ ажилтанг бүртгээгүй эсвэл холбоотой хэрэглэгчийг тохируулаагүй байна!'))
        

        print 'Approved-------------'
        return True

    def action_done_close_loan(self, cr, uid, ids, context=None):
        order = self.browse(cr,uid,ids)[0]
        cur_id = order.loan_order.product_id.currency.id 
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')
        
        if order.total_risk_amount != order.loan_order.loan_balance:
            raise osv.except_osv(_('Warning'), _('Total risk amount is incorrect(loan balance)!'))

        company_currency_id = res_users_obj.browse(cr, uid, uid,context=context).company_id.currency_id.id
        currency_id = order.loan_order.product_id.currency.id
        
        old_st = order.loan_current_state
        
        condition = ''
        old_desc = ''
        
        if old_st == 'd00':
            old_desc = u'Хэвийн'
            condition = 'product_id'
        elif old_st == 'd90':
            old_desc = u'Хугацаа хэтэрсэн'
            condition = 'xxz_product'
        elif old_st == 'd180':
            old_desc = u'Хэвийн бус'
            condition = 'xb_product'
        elif old_st == 'd360':
            old_desc = u'Эргэлзээтэй зээл'
            condition = 'ez_product'
        elif old_st == 'd361g':
            old_desc = u'Муу зээл'
            condition = 'mz_product'
        
        cid = self.pool.get('configure.loan.product').search(cr,uid,[(condition,'=',order.loan_order.product_id.id)])
        if cid == []:
            raise osv.except_osv(_('Warning'), _('Not found configure!'))
             
        config = self.pool.get('configure.loan.product').browse(cr,uid,cid[0])
        
        pro_id = 0
        if old_st == 'd00':
            pro_id = config.product_id.id
        elif old_st == 'd90':
            pro_id = config.xxz_product.id
        elif old_st == 'd180':
            pro_id = config.xb_product.id
        elif old_st == 'd360':
            pro_id = config.ez_product.id
        elif old_st == 'd361g':
            pro_id = config.mz_product.id
        
        pro = self.pool.get('product.product').browse(cr,uid,pro_id)
        amount_currency = res_currency_obj.compute(cr,uid, company_currency_id,currency_id,order.loan_order.loan_balance, context=context)
        amount = 0.0
        if amount_currency == order.loan_order.loan_balance:
            amount_currency = 0.0
            amount = order.loan_order.loan_balance
        else:
            t = amount_currency
            amount_currency = order.loan_order.loan_balance
            amount = t
        if company_currency_id != currency_id:
            am_id2 = self.pool.get('account.move').create(cr,uid, {
                                        'name':order.name,
                                        'journal_id':order.journal.id,
                                        'ref':order.loan_order.name})
            aml_one_one2 = self.pool.get('account.move.line').create(cr,uid,{
    									'name':order.loan_order.name + u'гэрээг эрсдэлийн сангаас хаав.',
    									'partner_id':order.partner_id.id,
    									'account_id':pro.property_account_income.id,
    									'credit':order.loan_order.loan_balance,
    									'currency_id':currency_id,
    									'amount_currency':amount_currency*(-1),
    									'move_id':am_id2 })
    									
            aml_one_two2 = self.pool.get('account.move.line').create(cr,uid,{
    									'name':order.loan_order.name + u'гэрээг эрсдэлийн сангаас хаав.',
    									'partner_id':order.partner_id.id,
    									'account_id':order.risk_account.id,
    									'debit':order.loan_order.loan_balance,
    									'currency_id':currency_id,
    									'amount_currency':amount_currency,
        								'move_id':am_id2 })
        else:
            am_id2 = self.pool.get('account.move').create(cr,uid, {
                                        'name':order.name,
                                        'journal_id':order.journal.id,
                                        'ref':order.loan_order.name})
            aml_one_one2 = self.pool.get('account.move.line').create(cr,uid,{
                                        'name':order.loan_order.name + u'гэрээг эрсдэлийн сангаас хаав.',
                                        'partner_id':order.partner_id.id,
                                        'account_id':pro.property_account_income.id,
                                        'credit':order.loan_order.loan_balance,
#                                        'currency_id':currency_id,
                                        'amount_currency':amount_currency*(-1),
                                        'move_id':am_id2 })
                                        
            aml_one_two2 = self.pool.get('account.move.line').create(cr,uid,{
                                        'name':order.loan_order.name + u'гэрээг эрсдэлийн сангаас хаав.',
                                        'partner_id':order.partner_id.id,
                                        'account_id':order.risk_account.id,
                                        'debit':order.loan_order.loan_balance,
#                                        'currency_id':currency_id,
                                        'amount_currency':amount_currency,
                                        'move_id':am_id2 })
        amove2 = self.pool.get('account.move').browse(cr,uid,am_id2)
        amove2.button_validate()
               
        print 'update lorder-',self.pool.get('loan.order').write(cr,uid,order.loan_order.id,
                                        {'product_id':pro_id,
                                         'state':'done',
                                         'loan_balance':0.0,
                                        })
        
        sid = self.pool.get('loan.account.statement').create(cr,uid, {
                                    'contract_id':order.loan_order.id,
                                    'name':u"Зээлийг эрсдэлийн сангаас хаав.",
                                    'write_date':order.date_order,
                                    'risk':(-1)*order.loan_order.loan_balance})
        hrid = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
        if hrid !=[]:
            self.write(cr, uid, ids, {'approve_accountant' : hrid[0],
                                      'state': 'done'})
        else:
            raise osv.except_osv(_('Warning'), _('Хүний нөөц дээр энэ ажилтанг бүртгээгүй эсвэл холбоотой хэрэглэгчийг тохируулаагүй байна!'))
        
        
        return am_id2

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'state':'draft',
            'name': self.pool.get('ir.sequence').get(cr, uid, 'close.loan.from.rf'),
        })
        return super(close_loan_from_rf, self).copy(cr, uid, id, default, context)

close_loan_from_rf()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
