# -*- coding: utf-8 -*-
import time
from datetime import datetime
from datetime import date
from datetime import timedelta
from dateutil.relativedelta import relativedelta
from openerp.osv import osv, fields
import openerp.netsvc
import openerp.pooler
import calendar
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT
#
# Model definition
#
class loan_append(osv.osv):
      
    STATE_SELECTION = [
        ('draft', 'Request for Quotation'),
        ('confirmed', 'Waiting Approval'),
        ('approved', 'Approved'),
        ('paid', 'Paid'),
        ('cancel', 'Cancelled')
    ]

    _columns = {
        'company_id':fields.many2one('res.company','Company'),
        'name': fields.char(u'Appending number', size=64, select=True,readonly=True),
        'contract_id': fields.many2one('loan.order',u'Loan Contract', domain=[('state','=','paid')],states={'confirmed':[('readonly',True)],'paid':[('readonly',True)], 'approved':[('readonly',True)]},required=True),
        'product_id': fields.many2one('loan.product',u'Loan product', domain=[('loan_product_type','=','loan_product')],states={'payment':[('readonly',True)],'confirmed':[('readonly',True)],'paid':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'period':fields.integer(u'Period',states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]},required=True),
		'installment_date': fields.date(u'Installment date',required=True,states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]}),
		'close_date': fields.date(u'Close date',readonly=True),
		
		'payment_day':fields.integer('Date of montly repayment',states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]},required=True),
		'end_of_month': fields.boolean(u'End of month',states={'confirmed':[('readonly',True)], 'approved':[('readonly',True)],'payment':[('readonly',True)],'done':[('readonly',True)],'paid':[('readonly',True)]}),
		
		'fixed_payment':fields.float('Fixed repayment amount',states={'confirmed':[('readonly',True)],'payment':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)],'paid':[('readonly',True)]},store=True),
		
		'xxt':fields.float(u'Accumlate interest',required=True,digits = (16,2),store=True,states={'payment':[('readonly',True)],'confirmed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)],'paid':[('readonly',True)]}),
		
		'balance':fields.float(u'Loan balance',required=True,digits = (16,2),store=True,states={'payment':[('readonly',True)],'confirmed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)],'paid':[('readonly',True)]}),
		
		'amount':fields.float(u'Appending amount',digits = (16,2),states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]},required=True),
		'interest':fields.float(u'Loan interest',digits = (16,2),states={'confirmed':[('readonly',True)], 'approved':[('readonly',True)],'paid':[('readonly',True)]},required=True),
		
		'schedules': fields.one2many('loan.schedule','append_id',u'Graphics',states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]}),
		'schedule_type': fields.selection([
		                ('type1','Fixed repayment PMT'),
                        ('type3','Fixed loan repayment'),
                        ('type5','Loan reapyment on last')], u'Graphic type',states={'confirmed':[('readonly',True)], 'approved':[('readonly',True)],'paid':[('readonly',True)]}),
        
        'cash_journal': fields.many2one('account.journal',u'Cash journal',domain=[('type','=','cash')],states={'paid':[('readonly',True)]}),
        'bank_journal': fields.many2one('account.journal',u'Bank journal',domain=[('type','=','bank')],states={'paid':[('readonly',True)]}),
        
        'current_economist': fields.many2one('hr.employee',u'Current economist',states={'confirmed':[('readonly',True)], 'approved':[('readonly',True)],'paid':[('readonly',True)]},required=True),
        
        'payment_type': fields.selection([
        				('cash','Cash'),
                        ('bank','Bank')], u'Payment type'),
        
        'date_order':fields.date('Order Date', required=True, states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]}),
        'date_approve':fields.date('Date Approved', readonly=1, select=True),
        'partner_id':fields.many2one('res.partner', 'Debtor', required=True, states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]}, change_default=True),
        'state': fields.selection(STATE_SELECTION, 'State', readonly=True, help="Help...", select=True),
        'validator' : fields.many2one('res.users', 'Validated by', readonly=True),
        'notes': fields.text('Other',states={'confirmed':[('readonly',True)],'approved':[('readonly',True)],'paid':[('readonly',True)]}),
    }
    _defaults = {
        'company_id':lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'date_order': fields.date.context_today,
        'state': 'draft',
        'name': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'loan.append'),
    }
    _sql_constraints = [
        ('name_uniq', 'unique(name)', 'Append number must be unique!'),
    ]
    _name = "loan.append"
    _description = "Append loan amount"
    _order = "name desc"

#   ---------------------------------MY----------------------------------------
    def onchange_schedule_type(self, cr, uid, ids, schtype,interest,amount,period,balance):
        val = {}
        amount = amount + balance
        val['fixed_payment'] = 0
        if schtype=='type1':
            if interest <= 0:
                raise osv.except_osv(_('Warning'), _('Insert interest!'))    
            if amount <= 0:
                raise osv.except_osv(_('Warning'), _('Insert amount!'))    
            if period <= 0:
                raise osv.except_osv(_('Warning'), _('Insert period!'))    
            c = 0.0
            c = float(interest)/100.0
            a = 1+c
            b = 1/pow(a,period)
            d = (1-b)/c
            if int(period) > 4 and int(period) <8:
                pmt = (amount/(period-1))
            elif int(period) > 8  and int(period) < 11:
                pmt = (amount/(period-2))
            elif int(period) > 8  and int(period) < 18:
                pmt = (amount/(period-3))
            elif int(period)> 18:
                pmt = (amount/(period-6))
            else:
                pmt = (amount/period)
            val['fixed_payment'] = pmt
            
        return {'value':val}

    def _get_days0(self,now,last):
		last_year = int(last[:4])
		last_mounth = int(last[5:7])
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		delta = d1 - d0
		return delta.days

    def _date_inc_dec(self,dddd,d):
        print "lllllllllllllk",dddd
        day = int(dddd[8:])
        month = int(dddd[5:7])
        year = int(dddd[:4])
        dd = date(year,month,day)+timedelta(days=d)
        return dd
		
    def _get_last_date(self,cr,uid,lid):
        loan = self.pool.get('loan.order').browse(cr,uid,lid)
        xtid = self.pool.get('loan.payment').search(cr,uid,[('state','=','paid'),('loan_order','=',lid)],order='id')
        xxtid = self.pool.get('loan.xxt.payment').search(cr, uid, [('loan_order', '=',lid),('state','!=','draft')],order='id')
        print '--',xtid,xxtid
        last_date = ''
        if xtid == [] and xxtid == []: 
            last_date = loan.date_approve
            last_date = str(self._date_inc_dec(last_date,-1))
            print 'First XXT--',last_date
        elif xtid != [] and xxtid == []:
            xt = self.pool.get('loan.payment').browse(cr,uid,xtid[len(xtid)-1])    
            last_date = xt.test_date
            print 'tulult xiisen XXT--',last_date
        elif xtid == [] and xxtid != []:
            xxt = self.pool.get('loan.xxt.payment').browse(cr,uid,xxtid[len(xxtid)-1])    
            last_date = xxt.test_date
            print 'xurimtluulsan XXT--',last_date
        elif xtid != [] and xxtid != []:
            xt = self.pool.get('loan.payment').browse(cr,uid,xtid[len(xtid)-1])    
            xxt = self.pool.get('loan.xxt.payment').browse(cr,uid,xxtid[len(xxtid)-1])    
            print '--------',xt.test_date,xxt.test_date
            if xt.test_date <= xxt.test_date:
                last_date = xxt.test_date
            else:
                last_date = xt.test_date
            print 'xt<xxt XXT--',xt.test_date,xxt.test_date
        return last_date
        
    def onchange_contract_id(self, cr, uid, ids, con_id,test_date):
        val = {}
        lorder = self.pool.get('loan.order').browse(cr, uid, con_id)
        if lorder:
            last_date = self._get_last_date(cr,uid,lorder.id)
        else:
            last_date = test_date
        test_date = str(self._date_inc_dec(test_date,-1))
        xxt = 0        
        if test_date > last_date:
            sd = self._get_days0(test_date,last_date)
            balance = lorder.loan_balance
            rate = lorder.interest
            c = rate/100
            xxt = balance*c/30*sd
            xxt = round(xxt, 2)
            print 'dates ',test_date,last_date,sd,xxt,balance
        print "log",lorder.loan_balance
        hrid = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
        val['period'] = lorder.period
        val['interest'] = lorder.interest 
        val['installment_date'] = lorder.installment_date
        val['payment_day'] = lorder.payment_day
        val['end_of_month'] = lorder.end_of_month
        val['partner_id'] = lorder.partner_id.id 
        val['balance'] = lorder.loan_balance
        val['schedule_type'] = lorder.schedule_type  
        val['schedules'] = lorder.schedules
        val['current_economist'] = hrid  
        val['xxt'] = xxt  
        gids = []
        for gra in lorder.schedules:
            gids.append(gra.id)
        val['schedules'] = gids
        val['product_id'] = lorder.product_id.id
        print '-con',con_id,val['schedules']
        return {'value':val}

    def onchange_journal(self, cr, uid, ids,pt,jid,pid):
		war = {}
		val = {}
		journal = self.pool.get('account.journal').browse(cr,uid,jid)
		product = self.pool.get('loan.product').browse(cr,uid,pid)
		if journal.default_credit_account_id.currency_id.id != product.currency.id and journal.default_credit_account_id.currency_id.id != False :
		    war['message'] = product.currency.name +unicode(' -тай ')+pt+unicode(' сонго.')
		    val['cash_journal'] = False
		    val['bank_journal'] = False	
		else:
		    val['account_balance'] = journal.default_credit_account_id.balance	    
		return {'value':val,'warning':war}

    def _get_mounth_days(self,x):
		return {
        	1:31, 2:28, 3:31, 4:30, 5:31, 6:30, 7:31, 8:31, 9:30, 10:31, 11:30, 12:31
        }.get(x)
    def _get_month_days(self,x):
        return {
            1:31, 2:28, 3:31, 4:30, 5:31, 6:30, 7:31, 8:31, 9:30, 10:31, 11:30, 12:31
        }.get(x)
        
    def _get_month_days(self,year,month):
        return calendar.monthrange(year, month)[1]
    def calc_graphics(self,cr,uid,ids,vals,context={}):  
        read_obj = self.read(cr,uid,ids)[0]
        aid = read_obj['id']
        cid = read_obj['contract_id'][0]
              
        product = self.browse(cr,uid,ids)[0].product_id
        period_type = product.period_type
                
        gtype = read_obj['schedule_type']
        t = read_obj['period']
        installment_date = read_obj['installment_date']
        xvv = read_obj['interest']
        pay_day = read_obj['payment_day']
        eom = read_obj['end_of_month']
        custom = read_obj['fixed_payment']
        today = read_obj['date_order']
        
        lorder = self.pool.get('loan.order').browse(cr,uid,cid)
        vzeel = read_obj['amount'] + lorder.loan_balance
		
        dddd = ''
        if lorder.schedules != []:
            for grap in lorder.schedules:
                    if grap.is_paid != 'paid':
                        raise osv.except_osv(_('Warning'), _('Reset graphic!'))
                    
        start_year = int(installment_date[:4])
        start_mounth = int(installment_date[5:7])
        start_day = int(installment_date[8:])
        
        gid = 0
        sd = 0
        mxvv = 0.0
        vld = 0.0
        if gtype == 'type1':
            if period_type == 'mounth':
                c = xvv/100
                a = 1+c
                b = 1/pow(a,t)
                d = (1-b)/c
                niit = vzeel/d
                niit = custom
                if eom == True:
                    pay_day = self._get_mounth_days(start_mounth)
            
                if pay_day > start_day:
                    sd = pay_day-start_day
                    mxvv = vzeel*c/30*sd
                    vld = vzeel
                    print start_year,start_mounth,start_day,sd,'---','0',' xvv=',mxvv,'niit',mxvv,'balance=',vzeel
                    dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,mxvv,vzeel,0,dddd,0)
                    start_day = pay_day
                    t -= 1
                else:
                    vld = vzeel
                    
        
                start_mounth += 1
                for i in range(0,t):
                    if start_mounth > 12:
                        start_year += 1
                        start_mounth = 1
                        
                    prem = 0
                    prey = 0
                    if start_mounth == 1:
                        prem = 12
                        prey = start_year - 1
                    else:
                        prem = start_mounth-1
                        prey = start_year
                    
                    print 'pre',prey,prem        
                    if eom == True:
                        pay_day = self._get_mounth_days(prem)
                    d0 = date(prey, prem, start_day)
                    
                    if eom == True:
                        pay_day = self._get_mounth_days(start_mounth)
                        start_day = pay_day
                    else:
                        start_day = pay_day
                    d1 = date(start_year, start_mounth, pay_day)
                    
                    delta = d1 - d0
                    sd = delta.days - (start_day-pay_day)
                    
                    
        #                start_day = pay_day            
                    mxvv = vld*c/30*sd    
                    vz = niit-mxvv
                    vld -= vz
                    
                    if i == t-1:
                        vz += vld
                        niit = mxvv + vz
                        print start_year,start_mounth,pay_day,sd,'---',vz,' xvv=',mxvv,'niit',niit,'balance=',0
                        dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                        gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,0,vz,dddd,gid)
                        break
                    
                    print start_year,start_mounth,pay_day,sd,'---',vz,' xvv=',mxvv,'niit',niit,'balance=',vld
                    dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vld,vz,dddd,gid)
                    start_mounth += 1    
            else:
                c = xvv/100
                a = 1+c
                b = 1/pow(a,t)
                d = (1-b)/c
                niit = vzeel/d
                niit = custom
                mxvv = 0.0
                pday = 0
                vzt = 0
                vld = vzeel

                if eom == True:
                    pay_day = self._get_month_days(start_year,start_mounth)
                if pay_day <= start_day:
                    pay_day = start_day
                else:
                    pay_day = pay_day
                for days in range(1,t+1):
                    mxvv = vzeel*c/30
                    if start_month:
                        end_day = str(self._get_month_days(start_month))
                    if pay_day == end_day:
                        pay_day = 0
                        start_month +=1
                    else:
                        dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                        ddddd = str(start_year)+'-'+str(start_month)+'-'+str(pay_day)
                        pay_day +=1
                        if dddd == ddddd:
                                dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                                pay_day = 1
                                start_month +=1
                        else:
                            if int(pay_day) > int(end_day):
                                dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                                pay_day = 1
                                start_month +=1
                        dddd = str(start_year)+'-'+str(start_month)+'-'+str(pay_day)
                    if start_mounth == 12:
                        start_year +=1
                        start_mounth = 1
                    if start_year/4==0:
                        if start_month==2 and pay_day==29:
                            pay_day=29
                        elif start_month==2 and pay_day==30:
                            pay_day=29
                        else:
                            pay_day=read_obj['payment_day']
                    else:
                        if start_mounth==2 and pay_day==29:
                            pay_day=28
                        elif start_mounth==2 and pay_day==30:
                            pay_day=28
                            

                    sd = 1
                    mxvv = vld*c/30*sd    
                    vz = niit-mxvv
                    vld0 = vld
                    if vld < vz:
                        vz = vld
                        niit = vz+mxvv
                        #break
                    vld -=vz
                    t -= 1

                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vld,vz,dddd,gid)
                    
        if gtype == 'type3':
            if period_type == 'mounth':
                #===============================================================
                # if custom == 0:
                #     raise osv.except_osv(_('Warning!'), _('Insert pay amount!'))
                # return
                #===============================================================
                vz = custom
                c = xvv/100
                one = 0
                
                if eom == True:
                    pay_day = self._get_mounth_days(start_mounth)
                if pay_day > start_day:
                    sd = pay_day-start_day
                    mxvv = vzeel*c/30*sd
                    niit = 0 + mxvv
                    dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vzeel,0,dddd,0)
                    start_day = pay_day
                    t -= 1
    
                start_mounth += 1
                for i in range(0,t):
                    if start_mounth > 12:
                        start_year += 1
                        start_mounth = 1
                    
                    prem = 0
                    prey = 0
                    if start_mounth == 1:
                        prem = 12
                        prey = start_year - 1
                    else:
                        prem = start_mounth-1
                        prey = start_year
                    
                    print 'pre',prey,prem        
                    if eom == True:
                        pay_day = self._get_mounth_days(prem)
                    d0 = date(prey, prem, pay_day)
                    if eom == True:
                        pay_day = self._get_mounth_days(start_mounth)
                        start_day = pay_day
                    d1 = date(start_year, start_mounth, pay_day)
                    delta = d1 - d0
                    sd = delta.days - (start_day-pay_day) + one
                    one = 0
                    
                    start_day = pay_day            
                    mxvv = vzeel*c/30*sd
                    niit = vz + mxvv
                    
                    print i,'=i'
                    if vzeel-vz <= 0:
                        if eom == True:
                            pay_day = self._get_mounth_days(start_mounth)
                            print sd,start_mounth,'------AAA-' 
                        else:
                            sd = self._get_mounth_days(start_mounth)-(start_day-pay_day)
                        mxvv = vzeel*c/30*sd
                        niit = vzeel + mxvv
                        dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                        gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,0,vzeel,dddd,gid)
                        break;
                    
                    if i == t-1:
                        niit = mxvv + vzeel
                        print start_year,start_mounth,pay_day,sd,'---',vzeel,' xvv=',mxvv,'niit=',niit,'balance=',vzeel
                        dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                        gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,0,vzeel,dddd,gid)    
                        break
                        
                    vzeel -= vz
                    print start_year,start_mounth,pay_day,sd,'---',vz,' xvv=',mxvv,'niit=',niit,'balance=',vzeel
                    dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vzeel,vz,dddd,gid)
                    start_mounth += 1
            else:
                vz = custom
                c = xvv/100
                niit = 0.0
                vld = vzeel
                if pay_day <= start_day:
                    pay_day = start_day
                else:
                    pay_day = pay_day                   
                for days in range(0,t-1):
                    mxvv = vzeel*c/30
                    if start_month:
                        end_day = str(self._get_month_days(start_month))
                    if pay_day == end_day:
                        pay_day = 0
                        start_month +=1
                    else:
                        dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                        ddddd = str(start_year)+'-'+str(start_month)+'-'+str(pay_day)
                        pay_day +=1
                        if dddd == ddddd:
                                dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                                pay_day = 1
                                start_month +=1
                        else:
                            if int(pay_day) > int(end_day):
                                dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                                pay_day = 1
                                start_month +=1
                        dddd = str(start_year)+'-'+str(start_month)+'-'+str(pay_day)
                    if start_mounth == 12:
                        start_year +=1
                        start_mounth = 1
                    if start_year/4==0:
                        if start_mounth==2 and pay_day==29:
                            pay_day=29
                        elif start_mounth==2 and pay_day==30:
                            pay_day=29
                        else:
                            pay_day=read_obj['payment_day']
                    else:
                        if start_mounth==2 and pay_day==29:
                            pay_day=28
                        elif start_mounth==2 and pay_day==30:
                            pay_day=28
                            

                    sd = 1
                    mxvv = vld*c/30*sd
                    niit = vz + mxvv

                    vld -=vz
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vld,vz,dddd,gid)
                    
                if vld > vz:
                    pay_day +=1
                    vz = vld
                    mxvv = vld*c/30*sd
                    niit = vz + mxvv
                    vld -=vz
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vld,vz,dddd,gid)
                    
                    
                    
                        
        if gtype == 'type5':
            if period_type == 'mounth':
                c = xvv/100
                if eom == True:
                    pay_day = self._get_mounth_days(start_mounth)
                    print 'eom',pay_day
                one = 0
                if pay_day > start_day:
                    sd = pay_day-start_day
                    mxvv = vzeel*c/30*sd
                    print start_year,start_mounth,pay_day,sd,'---','0',' xvv=',mxvv,'balance=',vzeel
                    dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,mxvv,vzeel,0,dddd,0)
                    start_day = pay_day
                    t -= 1
                            
                start_mounth += 1
                for i in range(1,t):
                    if start_mounth > 12:
                        start_year += 1
                        start_mounth = 1
                    
                    prem = 0
                    prey = 0
                    if start_mounth == 1:
                        prem = 12
                        prey = start_year - 1
                    else:
                        prem = start_mounth-1
                        prey = start_year
                    
                    print 'pre',prey,prem  
                    if eom == True:
                        pay_day = self._get_mounth_days(prem)      
                    d0 = date(prey, prem, pay_day)
                    if eom == True:
                        pay_day = self._get_mounth_days(start_mounth)
                        start_day = pay_day
                    d1 = date(start_year, start_mounth, pay_day)
                    delta = d1 - d0
                    sd = delta.days - (start_day-pay_day)
                    one = 0
                    
                    start_day = pay_day
                    
                    mxvv = vzeel*c/30*sd
                    print start_year,start_mounth,pay_day,sd,'---  0 xvv=',mxvv,'balance=',vzeel
                    dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,mxvv,vzeel,0,dddd,gid)
                    start_mounth += 1    
                
                if start_mounth > 12:
                    start_year += 1
                    start_mounth = 1
                prem = 0
                if start_mounth == 1:
                    prem = 12
                else:
                    prem = start_mounth - 1
                
                if eom == True:
                    pay_day = self._get_mounth_days(start_mounth)    
                sd = self._get_mounth_days(prem)    
                print start_year,start_mounth,pay_day,sd,'--- ',vzeel,'xvv=',mxvv,'balance=','0'
                dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day)
                mxvv = vzeel*c/30*sd
                niit = mxvv + vzeel
                gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,0,vzeel,dddd,gid)
            else:
                c = xvv/100
                niit = 0.0
                vld = vzeel
                if pay_day <= start_day:
                    pay_day = start_day
                else:
                    pay_day = start_day
                for days in range(1,t):                    
                    mxvv = vzeel*c/30
                    if start_month:
                        end_day = str(self._get_month_days(start_month))
                    if pay_day == end_day:
                        pay_day = 0
                        start_month +=1
                    else:
                        dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                        ddddd = str(start_year)+'-'+str(start_month)+'-'+str(pay_day)
                        pay_day +=1
                        if dddd == ddddd:
                                dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                                pay_day = 1
                                start_month +=1
                        else:
                            if int(pay_day) > int(end_day):
                                dddd = str(start_year)+'-'+str(start_month)+'-'+str(end_day)
                                pay_day = 1
                                start_month +=1
                        dddd = str(start_year)+'-'+str(start_month)+'-'+str(pay_day)
                    if start_mounth == 12:
                        start_year +=1
                        start_mounth = 1
                    if start_year/4==0:
                        if start_mounth==2 and pay_day==29:
                            pay_day=29
                        elif start_mounth==2 and pay_day==30:
                            pay_day=29
                        else:
                            pay_day=read_obj['payment_day']
                    else:
                        if start_mounth==2 and pay_day==29:
                            pay_day=28
                        elif start_mounth==2 and pay_day==30:
                            pay_day=28
                            

                    sd = 1
                    mxvv = vzeel*c/30*sd
                    niit = mxvv
                    gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vzeel,0,dddd,gid)                       
                    t -= 1
                    
                mxvv = vzeel*c/30*sd
                niit = mxvv + vzeel
                vzeel -=vld
                dddd = str(start_year)+'-'+str(start_mounth)+'-'+str(pay_day+1)
                gid = self._create_graph(cr,uid,aid,cid,mxvv,sd,niit,vzeel,vld,dddd,gid)
        print 'Close date',self.write(cr,uid,ids,{'close_date':dddd})
			
    def _create_graph(self,cr,uid,aid,cg_id,xvv,md,to_pa,balance,ma_lo,pa_da,pr_id):
		gid = self.pool.get('loan.schedule').create(cr, uid, {
					'contract_id':cg_id,
					'append_id':aid,
					'interest_payment':xvv,
					'days':md,
					'total_payment':to_pa,
					'balance':balance,
					'loan_payment':ma_lo,
					'payment_date':pa_da,
					'prior_id':pr_id,
					})
		return gid				
			
    def reset_graphics(self,cr,uid,ids,vals,context={}): 
        read_obj = self.read(cr,uid,ids)[0]
        gids = self.pool.get('loan.schedule').search(cr, uid,[('contract_id','=',read_obj['contract_id'][0]),('is_paid','=','un_paid')])
        self.pool.get('loan.schedule').unlink(cr,uid,gids)
		
    def print_graphics(self,cr,uid,ids,vals,context={}):  
		read_obj = self.read(cr,uid,ids)
                context['app_active_id'] = context.get('active_ids') and context.get('active_ids')[0] or False
                return {
                'type': 'ir.actions.report.xml',
                'context': context,
                'report_name': 'loan.append.schedule',
                'datas': {
                'model': 'loan.order',
                'report_type': 'pdf',
                'form': vals,
                },
                'nodestroy': False
            }
    def _get_days(self,now,last,m):
		last_year = int(last[:4])
		last_mounth = int(last[5:7])+m
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		delta = d1 - d0
		return delta.days

#   ---------------------------------ERP----------------------------------------
    def create(self, cr, uid, vals, context=None):
        if vals['amount'] < 0:
            raise osv.except_osv(_('Invalid period!'), _('Insert loan period!'))
        if vals['period'] <= 0:
            raise osv.except_osv(_('Invalid period!'), _('Insert loan period!'))
        if vals['interest'] <= 0:
            raise osv.except_osv(_('Invalid interest!'), _('Insert loan interest!'))
        return super(loan_append, self).create(cr, uid, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        loan_appends = self.read(cr, uid, ids, ['state'], context=context)
        unlink_ids = []
        for s in loan_appends:
            if s['state'] in ['draft','cancel']:
                unlink_ids.append(s['id'])
            else:
                raise osv.except_osv(_('Invalid action !'), _('In order to delete a loan order, it must be cancelled first!'))
        return super(loan_append, self).unlink(cr, uid, unlink_ids, context=context)

    def action_append_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'draft'})
        print 'Cancel-------------'
        return True

    def action_confirm_append(self, cr, uid, ids, context=None):
        for id in ids:
            if self.browse(cr, uid, [id])[0].amount < 0:
                raise osv.except_osv(_('Warning!'), _('Insert appending amount!'))
            self.write(cr, uid, [id], {'state' : 'confirmed'})
            print 'Confirmed-------------'
        return True

    def action_approve_append(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'approved', 
                                  'validator' : uid, 
                                  'date_approve': fields.date.context_today(self,cr,uid,context=context),
                                  })
        print 'Approved-------------'
        return True

    def action_payment_append(self, cr, uid, ids, context=None):
        obj = self.browse(cr,uid,ids)[0]
        journal = 0
        
        if obj.payment_type == 'cash':
            print '-------------Cashaar'
            journal = obj.cash_journal
        else:
            print '-------------Bankaar'
            journal = obj.bank_journal
                    
        print 'Payment-------------'
        if journal == [] or journal.id == False:
            raise osv.except_osv(_('Warning!'), _('Choose journal!'))
            
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')

        company_currency_id = res_users_obj.browse(cr, uid, uid,context=context).company_id.currency_id.id
        currency_id = obj.contract_id.product_id.currency.id
        
        amount_currency = res_currency_obj.compute(cr,uid, company_currency_id,currency_id,obj.amount, context=context)
        print 'amount currency-----',amount_currency
        amount = 0.0
        if amount_currency == obj.amount:
            amount_currency = 0.0
            amount = obj.amount
        else:
            t = amount_currency
            amount_currency = obj.amount
            amount = t
        if obj.amount == 0:
            sid = self.pool.get('loan.account.statement').create(cr,uid, {
                                    'contract_id':obj.contract_id.id,
                                    'write_date':obj.date_order,
                                    'name':obj.partner_id.name+u'-т Зээлийн хугацаа сунгав.',})
            
            t_balance = obj.contract_id.loan_balance + amount
            print 'update contract',self.pool.get('loan.order').write(cr,uid,obj.contract_id.id,
                                                                     {
                                                                      'period':obj.period,
                                                                      'installment_date':obj.installment_date,
                                                                      'close_date':obj.close_date,
                                                                      'end_of_month':obj.end_of_month,
                                                                      'payment_day':obj.payment_day,
                                                                      'current_economist':obj.current_economist.id,
                                                                      'loan_balance':t_balance,
                                                                      'schedule_type':obj.schedule_type,
                                                                      'fixed_payment':obj.fixed_payment,
                                                                      'interest':obj.interest,})
            
            self.write(cr, uid, ids, {'state': 'paid'})
        else:
            
            am_id = self.pool.get('account.move').create(cr,uid, {
                                        'name':obj.name,
                                        'journal_id':journal.id,
                                        'ref':obj.contract_id.name})
            

            
            if company_currency_id == currency_id:
                aml_one_one = self.pool.get('account.move.line').create(cr,uid,{
        									'name':obj.partner_id.name + u'-т нэмж зээл олгов.',
        									'partner_id':obj.partner_id.id,
        									'account_id':journal.default_credit_account_id.id,
        									'credit':amount,
    #    									'currency_id':currency_id,
    #    									'amount_currency':amount_currency*(-1),
        									'move_id':am_id })
        									
                aml_one_two = self.pool.get('account.move.line').create(cr,uid,{
        									'name':obj.partner_id.name + u'-т нэмж зээл олгов.',
        									'partner_id':obj.partner_id.id,
        									'account_id':obj.contract_id.product_id.property_account_income.id,
        									'debit':amount,
    #    									'currency_id':currency_id,
    #    									'amount_currency':amount_currency,
            								'move_id':am_id })
            else:
                aml_one_one = self.pool.get('account.move.line').create(cr,uid,{
                                            'name':obj.partner_id.name + u'-т нэмж зээл олгов.',
                                            'partner_id':obj.partner_id.id,
                                            'account_id':journal.default_credit_account_id.id,
                                            'credit':amount,
                                            'currency_id':currency_id,
                                            'amount_currency':amount_currency*(-1),
                                            'move_id':am_id })
                                            
                aml_one_two = self.pool.get('account.move.line').create(cr,uid,{
                                            'name':obj.partner_id.name + u'-т нэмж зээл олгов.',
                                            'partner_id':obj.partner_id.id,
                                            'account_id':obj.contract_id.product_id.property_account_income.id,
                                            'debit':amount,
                                            'currency_id':currency_id,
                                            'amount_currency':amount_currency,
                                            'move_id':am_id })    
        								
            print 'Return AM',am_id
            amove = self.pool.get('account.move').browse(cr,uid,am_id)
            amove.button_validate()
            self.write(cr, uid, ids, {'state': 'paid'})
            
            sid = self.pool.get('loan.account.statement').create(cr,uid, {
                                        'contract_id':obj.contract_id.id,
                                        'write_date':obj.date_order,
                                        'name':obj.partner_id.name+u'-т Нэмж зээл олгов',
                                        'amount':obj.amount})
            
            testdate = obj.date_order
            day = int(testdate[8:])
            month = int(testdate[5:7])
            year = int(testdate[:4])
            d = date(year,month,day)-timedelta(days=1)
            
            xxtid = self.pool.get('loan.xxt.payment').create(cr,uid,
    		                                          {'parent_id':False,
    		                                           'loan_order':obj.contract_id.id,
    		                                           'test_date':d,
    		                                           'partner_id':obj.partner_id.id,
                                                       'name':obj.partner_id.name+u' харилцагчийн хуримтлагдсан хүүгийн ',
    		                                           'xxt_payment':obj.xxt,
    		                                          })
            
            xxtobj = self.pool.get('loan.xxt.payment').browse(cr,uid,xxtid)
            xxtobj.action_to_confirm()
    #        xxtobj.action_to_payment()
            
            t_amount = obj.contract_id.amount + obj.amount
            t_balance = obj.contract_id.loan_balance + amount
            print 'update contract',self.pool.get('loan.order').write(cr,uid,obj.contract_id.id,
                                                                     {'amount':t_amount,
                                                                      'period':obj.period,
                                                                      'installment_date':obj.installment_date,
                                                                      'close_date':obj.close_date,
                                                                      'end_of_month':obj.end_of_month,
                                                                      'payment_day':obj.payment_day,
                                                                      'current_economist':obj.current_economist.id,
                                                                      'loan_balance':t_balance,
                                                                      'schedule_type':obj.schedule_type,
                                                                      'fixed_payment':obj.fixed_payment,
                                                                      'interest':obj.interest,})
            
            self.write(cr, uid, ids, {'state': 'paid'})
            return am_id

    def _get_days(self,now,last,m):
		last_year = int(last[:4])
		last_mounth = int(last[5:7])+m
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		print 'last',last_year, last_mounth, last_day,'now',now_year, now_mounth, now_day
		delta = d1 - d0
		return delta.days

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'state':'draft',
            'name': self.pool.get('ir.sequence').get(cr, uid, 'loan.append'),
        })
        return super(loan_append, self).copy(cr, uid, id, default, context)

loan_append()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
