# -*- coding: utf-8 -*-
import time
from datetime import datetime
from datetime import date
from dateutil.relativedelta import relativedelta

from openerp.osv import osv, fields
import openerp.netsvc
import openerp.pooler
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT

#
# Model definition
#
class loan_refund_risk_foundation(osv.osv):
      
    STATE_SELECTION = [
        ('draft', 'Draft'),
        ('approved', 'Approved'),
        ('cancel', 'Cancelled')
    ]
    
    _columns = {
                'company_id':fields.many2one('res.company','Company'),
        'name': fields.char(u'Refund number', size=64, select=True,readonly=True),
        'risk_lines': fields.one2many('loan.risk.foundation','refund_id','Loan risk foundation',
#                states={'draft':[('readonly',True)]},
                readonly=True,
                required=True),
        'loan_order': fields.many2one('loan.order',u'Loan contract', domain=[('loan_state','!=','d00')],states={'approved':[('readonly',True)]},required=True),
        'date_order':fields.date('Order Date', required=True, states={'approved':[('readonly',True)]}, help="Date on which this document has been created."),
        'date_approve':fields.date('Date Approved', readonly=True),
        'state': fields.selection(STATE_SELECTION, 'State', readonly=True, help="Help...", select=True),        
        'partner_id':fields.many2one('res.partner', 'Debtor', required=True, states={'approved':[('readonly',True)]}),
        'notes': fields.text('Other',states={'approved':[('readonly',True)]}),        
        'total_risk_amount':fields.float(u'Total amount of risk',digits = (16,2),states={'approved':[('readonly',True)]}),
        'risk_percent': fields.float(u'Percent of risk',digits = (16,2),states={'approved':[('readonly',True)]}),
		'loan_current_state': fields.selection([
        				('d00','Normal'),
                        ('d90','Past due loans'),
                        ('d180','Abnormal loans'),
                        ('d360','Doubted loans'),
                        ('d361g','Bad loan')], u'Loan state(current)',required=True,states={'approved':[('readonly',True)]}),
		'approve_accountant': fields.many2one('hr.employee',u'Approved accountant',readonly=True),
    }
    _defaults = {
        'company_id':lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'date_order': fields.date.context_today,
        'state': 'draft',
    }
    _sql_constraints = [
        ('name_uniq', 'unique(name)', 'Refund risk number must be unique!'),
    ]
    _name = "loan.refund.risk.foundation"
    _description = "Loan Refund Risk Foundation"
    _order = "name desc"

#   ---------------------------------MY----------------------------------------
    def onchange_loan_order(self, cr, uid, ids, lid):
        val = {}
        lorder = self.pool.get('loan.order').browse(cr,uid,lid)
        val['loan_current_state'] = lorder.loan_state
        val['partner_id'] = lorder.partner_id.id
        return {'value':val}

    def _get_month_days(self,x):
		return {
        	1:31, 2:28, 3:31, 4:30, 5:31, 6:30, 7:31, 8:31, 9:30, 10:31, 11:30, 12:31
        }.get(x)

    def import_risk_lines(self,cr,uid,ids,context=None):  
		order = self.browse(cr,uid,ids)[0]
		risk_ids = self.pool.get('loan.risk.foundation').search(cr,uid,[
		                            ('state','=','done'), 
		                            ('loan_order','=',order.loan_order.id)          
		                         ],order="id")
		if risk_ids == []:
		    raise osv.except_osv(_('Warning'), _("No risks to import."))
		    return True
		tr = 0.0
		per = self.pool.get('loan.risk.foundation').browse(cr,uid,risk_ids[len(risk_ids)-1]).risk_percent
		for risk in self.pool.get('loan.risk.foundation').browse(cr,uid,risk_ids):
		    tr += risk.risk_amount
		
		print 'update-',self.write(cr,uid,ids,{'total_risk_amount':tr,'risk_percent':per})
		print 'update lines',self.pool.get('loan.risk.foundation').write(cr,uid,risk_ids,{'refund_id':order.id})
		return True
				
    def _get_days(self,now,last,m):
		last_year = int(last[:4])
		last_mounth = int(last[5:7])+m
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		delta = d1 - d0
		return delta.days

#   ---------------------------------ERP----------------------------------------
    def create(self, cr, uid, vals, context=None):
#        if vals['amount'] <= 0:
#            raise osv.except_osv(_('Invalid amount!'), _('Insert loan amount!'))
#        if vals['period'] <= 0:
#            raise osv.except_osv(_('Invalid period!'), _('Insert loan period!'))
#        if vals['interest'] < 0:
#            raise osv.except_osv(_('Invalid interest!'), _('Insert loan interest!'))
#        if vals['loss_percent'] <= 0:
#            raise osv.except_osv(_('Invalid loss percent!'), _('Insert loss percent!'))    
        return super(loan_refund_risk_foundation, self).create(cr, uid, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        loan_orders = self.read(cr, uid, ids, ['state'], context=context)
        unlink_ids = []
        for s in loan_orders:
            if s['state'] in ['draft','cancel']:
                unlink_ids.append(s['id'])
            else:
                raise osv.except_osv(_('Invalid action !'), _('In order to delete a loan order, it must be cancelled first!'))
        return super(loan_refund_risk_foundation, self).unlink(cr, uid, unlink_ids, context=context)
 
    def action_refund_risk_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'draft'})
        print 'Cancel-------------'
        return True

    def action_approve_refund_risk(self, cr, uid, ids, context=None):
        hrid = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
        if hrid !=[]:
            self.write(cr, uid, ids, {'state': 'approved', 
                                  'approve_accountant' : hrid[0], 
                                  'name': self.pool.get('ir.sequence').get(cr, uid, 'loan.refund.risk.foundation'),
                                  'date_approve': fields.date.context_today(self,cr,uid,context=context),
                                  })
        
            
        else:
            raise osv.except_osv(_('Warning'), _('Хүний нөөц дээр энэ ажилтанг бүртгээгүй эсвэл холбоотой хэрэглэгчийг тохируулаагүй байна!'))
        return True

    def _get_days(self,now,last,m):
		last_year = int(last[:4])
		last_mounth = int(last[5:7]) + m
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		print 'last',last_year, last_mounth, last_day,'now',now_year, now_mounth, now_day
		delta = d1 - d0
		return delta.days

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'state':'draft',
            'name': self.pool.get('ir.sequence').get(cr, uid, 'loan.refund.risk.foundation'),
        })
        return super(loan_refund_risk_foundation, self).copy(cr, uid, id, default, context)

loan_refund_risk_foundation()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
