# -*- coding: utf-8 -*-
import time
from datetime import datetime
from datetime import date
from dateutil.relativedelta import relativedelta

from openerp.osv import osv, fields
import openerp.netsvc
import openerp.pooler
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT
from mx.DateTime.mxDateTime.mxDateTime_Python import DateTime
#
# Model definition
#
class loan_risk_foundation(osv.osv):
    _name = "loan.risk.foundation"
    _description = "Loan Risk Foundation"
    _order = "name desc"
    STATE_SELECTION = [
        ('draft', 'Draft'),
        ('approved', 'Approved'),
        ('done', 'Done'),
        ('closed', 'Closed'),
        ('cancel', 'Cancelled')
    ]
    
    _columns = {
        'company_id':fields.many2one('res.company','Company'),
        'name': fields.char(u'Risk number', size=64, select=True,readonly=True),
        'refund_id': fields.many2one('loan.refund.risk.foundation',u'Refund risk foundation',readonly=True),
        'loan_order': fields.many2one('loan.order',u'Loan contract', domain=[('state','=','paid')],states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'date_order':fields.date('Order Date', required=True, states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},help="Date on which this document has been created."),
        'date_approve':fields.date('Date Approved', readonly=1),
        'state': fields.selection(STATE_SELECTION, 'State', readonly=True, help="Help..."),        
        'partner_id':fields.many2one('res.partner', 'Debtor', required=True,states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]}),
        'notes': fields.text('Other',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]}),        
        'loan_balance':fields.float(u'Loan balance',digits = (16,2),states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'beyond_days':fields.integer(u'Beyond days',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'risk_amount':fields.float(u'Amount of risk',digits = (16,2),states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'risk_percent': fields.float(u'Percent of risk',digits = (16,2),states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'risk_account': fields.many2one('account.account',u'Account of risk foundation',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'risk_expense_account': fields.many2one('account.account',u'Account of risk expense',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'journal': fields.many2one('account.journal',u'Journal',states={'done':[('readonly',True)],'approved':[('readonly',True)]},required=True),

        'loan_current_state': fields.selection([
        				('d00','Normal'),
                        ('d90','Past due loans'),
                        ('d180','Abnormal loans'),
                        ('d360','Doubted loans'),
                        ('d361g','Bad loan')], u'Loan state(current)',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
        'loan_next_state': fields.selection([
        				('d00','Normal'),
                        ('d90','Past due loans'),
                        ('d180','Abnormal loans'),
                        ('d360','Doubted loans'),
                        ('d361g','Bad loan')], u'Loan state(changed)',states={'closed':[('readonly',True)], 'approved':[('readonly',True)],'done':[('readonly',True)]},required=True),
		'approve_economist': fields.many2one('hr.employee',u'Approved economist',readonly=True),
		'approve_accountant': fields.many2one('hr.employee',u'Approved accountant',readonly=True),
    }
    _defaults = {
        'company_id':lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'date_order': fields.date.context_today,
        'state': 'draft',
    }
    _sql_constraints = [
        ('name_uniq', 'unique(name)', 'Risk number must be unique!'),
    ]


#   ---------------------------------MY----------------------------------------
    def onchange_risk_percent(self, cr, uid, ids, percent,balance):
        val = {}
        if percent <= 0:
            return {'warning':{'message':"Wrong percent",'title':'Warning!'}}
        ra = 0.0
        ra = percent*balance/100.0
        val['risk_amount'] = ra
        return {'value':val}

    def onchange_loan_order(self, cr, uid, ids, lid,today):
        val = {}
        lorder = self.pool.get('loan.order').browse(cr,uid,lid)
        lps = self.pool.get('loan.payment').search(cr,uid,[('state','=','paid'),('loan_order','=',lorder.id)],order='id')
        last_date = ''
        one = 0
        if lps != []:
            last_date = self.pool.get('loan.payment').browse(cr,uid,lps[len(lps)-1]).graphic.payment_date
            one = 1
            print 'lp',last_date
        else:
            last_date = lorder.schedules[0].payment_date
            print 'no lp',last_date
        print 'LPS',lps
        print '-----days',self._get_days(today,last_date,one),today,last_date        
        days = 0
        days = self._get_days(today,last_date,one)
        val['beyond_days'] = days
        if days <= 0:
            return {'value':{'loan_order':False},'warning':{'message':"This loan is normal. Don't create risk foundation!",'title':'Warning!'}}
        
        val['partner_id'] = lorder.partner_id.id
        val['loan_current_state'] = lorder.loan_state
        val['loan_balance'] = lorder.loan_balance
        
        return {'value':val}

    def _get_month_days(self,x):
		return {
        	1:31, 2:28, 3:31, 4:30, 5:31, 6:30, 7:31, 8:31, 9:30, 10:31, 11:30, 12:31
        }.get(x)

    def _get_days(self,now,last,m):
		last_year = int(last[:4])
		last_mounth = int(last[5:7])+m
		last_day = int(last[8:])	
		
		if last_mounth > 12:
		    last_mounth = 1
		    last_year += 1
		
		now_year = int(now[:4])
		now_mounth = int(now[5:7])
		now_day = int(now[8:])	
		
		d0 = date(last_year, last_mounth, last_day)
		d1 = date(now_year, now_mounth, now_day)
		delta = d1 - d0
		return delta.days

#   ---------------------------------ERP----------------------------------------
    def create(self, cr, uid, vals, context=None):
        if vals['risk_amount'] <= 0:
            raise osv.except_osv(_('Invalid risk amount!'), _('Insert risk amount!'))
        if vals['risk_percent'] <= 0:
            raise osv.except_osv(_('Invalid risk percent!'), _('Insert risk period!'))
        return super(loan_risk_foundation, self).create(cr, uid, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        loan_orders = self.read(cr, uid, ids, ['state'], context=context)
        unlink_ids = []
        for s in loan_orders:
            if s['state'] in ['draft','cancel']:
                unlink_ids.append(s['id'])
            else:
                raise osv.except_osv(_('Invalid action !'), _('In order to delete a loan order, it must be cancelled first!'))
        return super(loan_risk_foundation, self).unlink(cr, uid, unlink_ids, context=context)

    def refund_risk_validate(self, cr, uid, ids, context=None):
        order = self.browse(cr, uid, ids)[0]
        print '---',cr,uid,ids
        print 'refund action',order.name
        
        cur_id = order.loan_order.product_id.currency.id 
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')

        company_currency_id = res_users_obj.browse(cr, uid, uid,context=context).company_id.currency_id.id
        currency_id = order.loan_order.product_id.currency.id
        
        amount_currency = res_currency_obj.compute(cr,uid, company_currency_id,currency_id,order.risk_amount, context=context)
        print 'amount currency-----',amount_currency
        amount = 0.0
        if amount_currency == order.risk_amount:
            amount_currency = 0.0
            amount = order.risk_amount
        else:
            t = amount_currency
            amount_currency = order.risk_amount
            amount = t
        
        am_id = self.pool.get('account.move').create(cr,uid, {
                                    'name':order.name,
                                    'journal_id':order.journal.id,
                                    'ref':order.loan_order.name})
                                    
        aml_one_one = self.pool.get('account.move.line').create(cr,uid,{
									'name':order.loan_order.name + u'гэрээний эрсдэлийн санг буцаав.',
									'partner_id':order.partner_id.id,
									'account_id':order.risk_expense_account.id,
									'credit':amount,
									#'currency_id':currency_id,
									#'amount_currency':amount_currency*(-1),
									'move_id':am_id })
									
        aml_one_two = self.pool.get('account.move.line').create(cr,uid,{
									'name':order.loan_order.name + u'гэрээний эрсдэлийн санг буцаав.',
									'partner_id':order.partner_id.id,
									'account_id':order.risk_account.id,
									'debit':amount,
									#'currency_id':currency_id,
									#'amount_currency':amount_currency,
    								'move_id':am_id })
        amove = self.pool.get('account.move').browse(cr,uid,am_id)
        amove.button_validate()
        
        new_st = order.loan_current_state
        old_st = order.loan_next_state
        
        condition = ''
        old_desc = ''
        
        if old_st == 'd00':
            old_desc = u'Хэвийн'
            condition = 'product_id'
        elif old_st == 'd90':
            old_desc = u'Хугацаа хэтэрсэн'
            condition = 'xxz_product'
        elif old_st == 'd180':
            old_desc = u'Хэвийн бус'
            condition = 'xb_product'
        elif old_st == 'd360':
            old_desc = u'Эргэлзээтэй зээл'
            condition = 'ez_product'
        elif old_st == 'd361g':
            old_desc = u'Муу зээл'
            condition = 'mz_product'
        
        cid = self.pool.get('configure.loan.product').search(cr,uid,[(condition,'=',order.loan_order.product_id.id)])
        if cid == []:
            raise osv.except_osv(_('Warning'), _('Not found configure!'))
             
        config = self.pool.get('configure.loan.product').browse(cr,uid,cid[0])
        print '--------------c id',cid
        
        new_desc = ''
        pro_id = 0
        if new_st == 'd00':
            new_desc = u'Хэвийн'
            pro_id = config.product_id.id
        elif new_st == 'd90':
            new_desc = u'Хугацаа хэтэрсэн'
            pro_id = config.xxz_product.id
        elif new_st == 'd180':
            new_desc = u'Хэвийн бус'
            pro_id = config.xb_product.id
        elif new_st == 'd360':
            new_desc = u'Эргэлзээтэй зээл'
            pro_id = config.ez_product.id
        elif new_st == 'd361g':
            new_desc = u'Муу зээл'
            pro_id = config.mz_product.idfields.date.context_today(self,cr,uid,context=context),
               
        pro = self.pool.get('product.product').browse(cr,uid,pro_id)
        amount_currency = res_currency_obj.compute(cr,uid, company_currency_id,currency_id,order.loan_order.loan_balance, context=context)
        amount = 0.0
        if amount_currency == order.loan_order.loan_balance:
            amount_currency = 0.0
            amount = order.loan_order.loan_balance
        else:
            t = amount_currency
            amount_currency = order.loan_order.loan_balance
            amount = t
        am_id2 = self.pool.get('account.move').create(cr,uid, {
                                    'name':order.name,
                                    'journal_id':order.journal.id,
                                    'ref':order.loan_order.name})
        aml_one_one2 = self.pool.get('account.move.line').create(cr,uid,{
									'name':order.loan_order.name + u'гэрээ ангилал шилжив.',
									'partner_id':order.partner_id.id,
									'account_id':order.loan_order.product_id.property_account_expense.id,
									'credit':order.loan_order.loan_balance,
									#'currency_id':currency_id,
									#'amount_currency':amount_currency*(-1),
									'move_id':am_id2 })
									
        aml_one_two2 = self.pool.get('account.move.line').create(cr,uid,{
									'name':order.loan_order.name + u'гэрээ ангилалrisk_orders шилжив.',
									'partner_id':order.partner_id.id,
									'account_id':pro.property_account_income.id,
									'debit':order.loan_order.loan_balance,
									#'currency_id':currency_id,
									#'amount_currency':amount_currency,
    								'move_id':am_id2 })
        amove2 = self.pool.get('account.move').browse(cr,uid,am_id2)
        amove2.button_validate()
        
        print 'new state-',new_st,'pro',pro_id
        print 'update lorder-',self.pool.get('loan.order').write(cr,uid,order.loan_order.id,
                                        {'loan_state':new_st,
                                         'product_id':pro_id,
                                        })
        print 'update refund-',self.pool.get('loan.refund.risk.foundation').write(cr,uid,order.refund_id.id,{'loan_current_state':new_st})
        
        sid = self.pool.get('loan.account.statement').create(cr,uid, {
                                    'contract_id':order.loan_order.id,
                                    'name':u"Эрсдэлийн сан буцаав, ангилал шилжив('"+unicode(old_desc)+"'->'"+unicode(new_desc)+"')",
                                    'write_date':order.date_order,
                                    'risk':(-1)*order.risk_amount})

        self.write(cr, uid, ids, {'state': 'closed'})
        return am_id

    def action_risk_cancel(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'draft'})
        print 'Cancel-------------'
        return True
    
        
    def action_approve_risk(self, cr, uid, ids, context=None):
#         print "tessss",self.pool.get('ir.sequence').get(cr, uid, 'loan.risk.foundation')
        hrid = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
        if hrid !=[]:
            self.write(cr, uid, ids, {'state': 'approved', 
                                      'approve_economist' : hrid[0], 
                                      'name': self.pool.get('ir.sequence').get(cr, uid, 'loan.risk.foundation'),
                                      'date_approve': fields.date.context_today(self,cr,uid,context=context),
                                      })
        else:
            raise osv.except_osv(_('Warning'), _('Хүний нөөц дээр энэ ажилтанг бүртгээгүй эсвэл холбоотой хэрэглэгчийг тохируулаагүй байна!'))

        return True

    def action_done_risk(self, cr, uid, ids, context=None):
        order = self.browse(cr,uid,ids)[0]
        cur_id = order.loan_order.product_id.currency.id 
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')

        company_currency_id = res_users_obj.browse(cr, uid, uid,context=context).company_id.currency_id.id
        currency_id = order.loan_order.product_id.currency.id
        
        amount_currency = res_currency_obj.compute(cr,uid, company_currency_id,currency_id,order.risk_amount, context=context)
        amount = 0.0
        if amount_currency == order.risk_amount:
            amount_currency = 0.0
            amount = order.risk_amount
        else:
            t = amount_currency
            amount_currency = order.risk_amount
            amount = t
        
        am_id = self.pool.get('account.move').create(cr,uid, {
                                    'name':order.name,
                                    'journal_id':order.journal.id,
                                    'ref':order.loan_order.name})
        if company_currency_id == currency_id:
                                   
            aml_one_one = self.pool.get('account.move.line').create(cr,uid,{
    									'name':order.loan_order.name + u'гэрээнд эрсдэлийн сан байгуулав.',
    									'partner_id':order.partner_id.id,
    									'account_id':order.risk_account.id,
    									'credit':amount,
#     									'currency_id':currency_id,
#     									'amount_currency':amount_currency*(-1),
                                        'date':datetime.now(),
    									'move_id':am_id })
    									
            aml_one_two = self.pool.get('account.move.line').create(cr,uid,{
    									'name':order.loan_order.name + u'гэрээнд эрсдэлийн сан байгуулав.',
    									'partner_id':order.partner_id.id,
    									'account_id':order.risk_expense_account.id,
    									'debit':amount,
                                        'date': datetime.now(),
#     									'currency_id':currency_id,
#     									'amount_currency':amount_currency,
        								'move_id':am_id })
    								
        else:
            aml_one_one = self.pool.get('account.move.line').create(cr,uid,{
                                        'name':order.loan_order.name + u'гэрээнд эрсдэлийн сан байгуулав.',
                                        'partner_id':order.partner_id.id,
                                        'account_id':order.risk_account.id,
                                        'credit':amount,
                                         'currency_id':currency_id,
                                         'amount_currency':amount_currency*(-1),
                                        'date':datetime.now(),
                                        'move_id':am_id })
                                        
            aml_one_two = self.pool.get('account.move.line').create(cr,uid,{
                                        'name':order.loan_order.name + u'гэрээнд эрсдэлийн сан байгуулав.',
                                        'partner_id':order.partner_id.id,
                                        'account_id':order.risk_expense_account.id,
                                        'debit':amount,
                                        'date': datetime.now(),
                                         'currency_id':currency_id,
                                         'amount_currency':amount_currency,
                                        'move_id':am_id })
        print 'Return AM',am_id
        amove = self.pool.get('account.move').browse(cr,uid,am_id)
        amove.button_validate()
        
        old_st = order.loan_current_state
        new_st = order.loan_next_state
        
        condition = ''
        old_desc = ''
        
        if old_st == 'd00':
            old_desc = u'Хэвийн'
            condition = 'product_id'
        elif old_st == 'd90':
            old_desc = u'Хугацаа хэтэрсэн'
            condition = 'xxz_product'
        elif old_st == 'd180':
            old_desc = u'Хэвийн бус'
            condition = 'xb_product'
        elif old_st == 'd360':
            old_desc = u'Эргэлзээтэй зээл'
            condition = 'ez_product'
        elif old_st == 'd361g':
            old_desc = u'Муу зээл'
            condition = 'mz_product'
        
        cid = self.pool.get('configure.loan.product').search(cr,uid,[(condition,'=',order.loan_order.product_id.id)])
        if cid == []:
            raise osv.except_osv(_('Warning'), _('Not found configure!'))
             
        config = self.pool.get('configure.loan.product').browse(cr,uid,cid[0])
        print '--------------c id',cid
        
        new_desc = ''
        pro_id = 0
        am_id2 = 0
        days = order.beyond_days
        if new_st == 'd00':
            new_desc = u'Хэвийн'
            pro_id = config.product_id.id
        elif new_st == 'd90':
            new_desc = u'Хугацаа хэтэрсэн'
            pro_id = config.xxz_product.id
        elif new_st == 'd180':
            new_desc = u'Хэвийн бус'
            pro_id = config.xb_product.id
        elif new_st == 'd360':
            new_desc = u'Эргэлзээтэй зээл'
            pro_id = config.ez_product.id
        elif new_st == 'd361g':
            new_desc = u'Муу зээл'
            pro_id = config.mz_product.id
        
        pro = self.pool.get('product.product').browse(cr,uid,pro_id)
        
        amount_currency = res_currency_obj.compute(cr,uid, company_currency_id,currency_id,order.loan_order.loan_balance, context=context)
        amount = 0.0
        if amount_currency == order.loan_order.loan_balance:
            amount_currency = 0.0
            amount = order.loan_order.loan_balance
        else:
            t = amount_currency
            amount_currency = order.loan_order.loan_balance
            amount = t
        print "account",order.loan_order
        am_id2 = self.pool.get('account.move').create(cr,uid, {
                                    'name':order.name,
                                    'journal_id':order.journal.id,
                                    'ref':order.loan_order.name})
        if company_currency_id == currency_id:

            aml_one_one2 = self.pool.get('account.move.line').create(cr,uid,{
    									'name':order.loan_order.name + u'гэрээ ангилал шилжив.',
    									'partner_id':order.partner_id.id,
    									'account_id':order.loan_order.product_id.property_account_expense.id,
    									'credit':order.loan_balance,
                                        'date': datetime.now(),
#      									'currency_id':currency_id,
#      									'amount_currency':amount_currency*(-1),
    									'move_id':am_id2 })
    									
            aml_one_two2 = self.pool.get('account.move.line').create(cr,uid,{
    									'name':order.loan_order.name + u'гэрээ ангилал шилжив.',
    									'partner_id':order.partner_id.id,
    									'account_id':pro.property_account_income.id,
                                        'date': datetime.now(),
    									'debit':order.loan_order.loan_balance,
#     									'currency_id':currency_id,
#     									'amount_currency':amount_currency,
        								'move_id':am_id2 })
        else:
            aml_one_one2 = self.pool.get('account.move.line').create(cr,uid,{
                                        'name':order.loan_order.name + u'гэрээ ангилал шилжив.',
                                        'partner_id':order.partner_id.id,
                                        'account_id':order.loan_order.product_id.property_account_expense.id,
                                        'credit':order.loan_balance,
                                        'date': datetime.now(),
                                         'currency_id':currency_id,
                                         'amount_currency':amount_currency*(-1),
                                        'move_id':am_id2 })
                                        
            aml_one_two2 = self.pool.get('account.move.line').create(cr,uid,{
                                        'name':order.loan_order.name + u'гэрээ ангилал шилжив.',
                                        'partner_id':order.partner_id.id,
                                        'account_id':pro.property_account_income.id,
                                        'date': datetime.now(),
                                        'debit':order.loan_order.loan_balance,
                                        'currency_id':currency_id,
                                        'amount_currency':amount_currency,
                                        'move_id':am_id2 })
        amove2 = self.pool.get('account.move').browse(cr,uid,am_id2)
        amove2.button_validate()
        
        print 'new state-',new_st,'pro',pro_id
        print 'update lorder-',self.pool.get('loan.order').write(cr,uid,order.loan_order.id,
                                        {'loan_state':new_st,
                                         'product_id':pro_id,
                                         'late_day':order.beyond_days,
                                        })
        
        sid = self.pool.get('loan.account.statement').create(cr,uid, {
                                    'contract_id':order.loan_order.id,
                                    'name':u"Эрсдэлийн санг "+str(order.risk_percent)+u"%-р байгуулав, ангилал шилжив('"+unicode(old_desc)+"'->'"+unicode(new_desc)+"')",
                                    'write_date':order.date_order,
                                    'risk':order.risk_amount})
        hrid = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
        if hrid !=[]:
            self.write(cr, uid, ids, {'approve_accountant':hrid[0],
                                  'state': 'done'})
        else:
            raise osv.except_osv(_('Warning'), _('Хүний нөөц дээр энэ ажилтанг бүртгээгүй эсвэл холбоотой хэрэглэгчийг тохируулаагүй байна!'))


        return am_id

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'state':'draft',
            'name': self.pool.get('ir.sequence').get(cr, uid, 'loan.risk.foundation'),
        })
        return super(loan_risk_foundation, self).copy(cr, uid, id, default, context)

loan_risk_foundation()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
