# -*- coding: utf-8 -*-
##############################################################################
#
#                       tbab - ERP
#
##############################################################################

from openerp import tools
from openerp.osv import fields, osv

from openerp.tools.sql import drop_view_if_exists

class performance_material_report(osv.osv):
    _name = "performance.material.report"
    _description = "Performance material report analysis"
    _auto = False
    #_rec_name = 'date'
    
    _columns = {
        'per_product_id':fields.many2one('product.product', 'Product name'),
        'product_id':fields.many2one('product.product', 'Product name'),
        'pro_uom_id':fields.many2one('product.uom', 'Unit'),
        'product_bom':fields.float('Product bom'),
        'project_id':fields.many2one('project.project', 'Project', required=True),        
        'project_phase_id':fields.many2one('project.task.phase', 'Task stage', required=True),
        'task_id': fields.many2one('project.project.task','Task name'),
        'res_partner_id': fields.many2one('res.partner','Executive'),
        'sub_phase_id':fields.many2one('sub.phase', 'Sub phase'),  
        'per_date':fields.date('Date'), 
        'company_id':fields.many2one('res.company','Company'),
        'user_id':fields.many2one('res.users', 'User'),
        'pr_material_id':fields.many2one('performance.material','Material list'), 
    }
    _order = 'per_date desc'

    def init(self, cr):
        drop_view_if_exists(cr, 'performance_material_report')
        cr.execute("""
            create or replace view performance_material_report as (
                SELECT
                    ml.project_id as project_id,
                    ml.project_phase_id as project_phase_id,
                    ml.task_id as task_id,
                    ml.sub_phase_id as sub_phase_id,
                    ml.user_id as user_id,
                    ml.res_partner_id as res_partner_id,
                    ml.product_id as product_id,
                    ml.pro_uom_id as pro_uom_id,
                    ml.per_date as per_date,
                    sum(ml.product_bom)
                
                FROM 
                    material_line ml
                    LEFT JOIN project_project pp ON (pp.id = ml.project_id)
                    LEFT JOIN project_task_phase ptp ON (ptp.id = ml.project_phase_id)
                    LEFT JOIN project_project_task ppt ON (ppt.id = ml.task_id)
                    LEFT JOIN sub_phase sp ON (sp.id = ml.sub_phase_id)
                    LEFT JOIN res_users ru ON (ru.id = ml.user_id)
                    LEFT JOIN res_partner rp ON (rp.id = ml.res_partner_id)
                    LEFT JOIN product_product prpr ON (prpr.id = ml.product_id)
                    LEFT JOIN product_uom puom ON (puom.id = ml.pro_uom_id)
                    
                
                GROUP BY
                    ml.project_id,
                    ml.project_phase_id,
                    ml.task_id,
                    ml.sub_phase_id,
                    ml.user_id,
                    ml.res_partner_id,
                    ml.product_id,
                    ml.pro_uom_id,
                    ml.per_date
            )""")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: