# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging

from openerp import SUPERUSER_ID
from openerp import tools
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import datetime
from datetime import date, timedelta, datetime

class purchase_order(osv.osv):
    _inherit = 'purchase.order'
    
    def _prepare_invoice(self, cr, uid, order, line_ids, context=None):
        """Prepare the dict of values to create the new invoice for a
           purchase order. This method may be overridden to implement custom
           invoice generation (making sure to call super() to establish
           a clean extension chain).

           :param browse_record order: purchase.order record to invoice
           :param list(int) line_ids: list of invoice line IDs that must be
                                      attached to the invoice
           :return: dict of value to create() the invoice
        """
        journal_ids = self.pool['account.journal'].search(
                            cr, uid, [('type', '=', 'purchase'),
                                      ('company_id', '=', order.company_id.id)],
                            limit=1)
        if not journal_ids:
            raise osv.except_osv(
                _('Error!'),
                _('Define purchase journal for this company: "%s" (id:%d).') % \
                    (order.company_id.name, order.company_id.id))
        return {
            'name': order.partner_ref or order.name,
            'reference': order.partner_ref or order.name,
            'account_id': order.partner_id.property_account_payable.id,
            'type': 'in_invoice',
            'partner_id': order.partner_id.id,
            'currency_id': order.currency_id.id,
            'journal_id': len(journal_ids) and journal_ids[0] or False,
            'invoice_line': [(6, 0, line_ids)],
            'contract_id':order.contract_id.id,
            'origin': order.contract_id.code,
            'fiscal_position': order.fiscal_position.id or False,
            'payment_term': order.payment_term_id.id or False,
            'company_id': order.company_id.id,
        }
    
    def _prepare_order_line_move(self, cr, uid, order, order_line, picking_id, group_id, context=None):
        ''' prepare the stock move data from the PO line. This function returns a list of dictionary ready to be used in stock.move's create()'''
        product_uom = self.pool.get('product.uom')
        price_unit = order_line.price_unit
        if order_line.product_uom.id != order_line.product_id.uom_id.id:
            price_unit *= order_line.product_uom.factor / order_line.product_id.uom_id.factor
        if order.currency_id.id != order.company_id.currency_id.id:
            #we don't round the price_unit, as we may want to store the standard price with more digits than allowed by the currency
            price_unit = self.pool.get('res.currency').compute(cr, uid, order.currency_id.id, order.company_id.currency_id.id, price_unit, round=False, context=context)
        res = []
        move_template = {
            'name': order_line.name or '',
            'project_id':order_line.project_id.id,
            'collection_id':order_line.collection_id.id,
            'product_id': order_line.product_id.id,
            'product_code':order_line.product_id.new_code,
            'cost_price':order_line.product_id.standard_price,
            'total_cost':order_line.product_id.standard_price * order_line.product_qty,
            'sale_price':order_line.price_unit,
            'total_sale_cost':order_line.price_unit * order_line.product_qty,
            'product_uom': order_line.product_uom.id,
            'product_uos': order_line.product_uom.id,
            'date': order.date_order,
            'date_expected': fields.date.date_to_datetime(self, cr, uid, order_line.date_planned, context),
            'location_id': order.partner_id.property_stock_supplier.id,
            'location_dest_id': order.location_id.id,
            'picking_id': picking_id,
            'partner_id': order.dest_address_id.id or order.partner_id.id,
            'move_dest_id': False,
            'state': 'draft',
            'purchase_line_id': order_line.id,
            'company_id': order.company_id.id,
            'price_unit': price_unit,
            'picking_type_id': order.picking_type_id.id,
            'group_id': group_id,
            'procurement_id': False,
            'origin': order.name,
            'route_ids': order.picking_type_id.warehouse_id and [(6, 0, [x.id for x in order.picking_type_id.warehouse_id.route_ids])] or [],
            'warehouse_id':order.picking_type_id.warehouse_id.id,
            'invoice_state': order.invoice_method == 'picking' and '2binvoiced' or 'none',
        }

        diff_quantity = order_line.product_qty
        for procurement in order_line.procurement_ids:
            procurement_qty = product_uom._compute_qty(cr, uid, procurement.product_uom.id, procurement.product_qty, to_uom_id=order_line.product_uom.id)
            tmp = move_template.copy()
            tmp.update({
                'product_uom_qty': min(procurement_qty, diff_quantity),
                'product_uos_qty': min(procurement_qty, diff_quantity),
                'move_dest_id': procurement.move_dest_id.id,  #move destination is same as procurement destination
                'group_id': procurement.group_id.id or group_id,  #move group is same as group of procurements if it exists, otherwise take another group
                'procurement_id': procurement.id,
                'invoice_state': procurement.rule_id.invoice_state or (procurement.location_id and procurement.location_id.usage == 'customer' and procurement.invoice_state=='picking' and '2binvoiced') or (order.invoice_method == 'picking' and '2binvoiced') or 'none', #dropship case takes from sale
                'propagate': procurement.rule_id.propagate,
            })
            diff_quantity -= min(procurement_qty, diff_quantity)
            res.append(tmp)
        #if the order line has a bigger quantity than the procurement it was for (manually changed or minimal quantity), then
        #split the future stock move in two because the route followed may be differentcont_id.
        if diff_quantity > 0:
            move_template['product_uom_qty'] = diff_quantity
            move_template['product_uos_qty'] = diff_quantity
            res.append(move_template)
        return res
    _columns = {
                'contract_id':fields.many2one('account.analytic.account','Contract'),
                'mrp_day':fields.integer('Manufacture day'),
                'shipment_day':fields.integer('Shipment day'),
                'weight':fields.float('Weight'),
                'length':fields.char('Length, width, height'),
                'tech_line':fields.one2many('tech.indicator','order_id','Tech indicator'),
                }
    
    def action_code_on_change(self, cr, uid, ids, contract_id):
        res = {}
        contract_obj = self.pool.get('account.analytic.account')
        cont_id = contract_obj.browse(cr, uid, contract_id)
        if cont_id:
            res['partner_ref']=cont_id.code
        return {'value':res}
    
    def action_picking_create(self, cr, uid, ids, context=None):
        for order in self.browse(cr, uid, ids):
            picking_vals = {
                'contract_number':max([l.project_id.id for l in order.order_line]),
                'picking_type_id': order.picking_type_id.id,
                'partner_id': order.dest_address_id.id or order.partner_id.id,
                'date': max([l.date_planned for l in order.order_line]),
                'origin': order.name
            }
            picking_id = self.pool.get('stock.picking').create(cr, uid, picking_vals, context=context)
            self._create_stock_moves(cr, uid, order, order.order_line, picking_id, context=context)

class tech_indicator(osv.osv):
    _name = 'tech.indicator'
    
    _columns = {
                'name':fields.char('Name'),
                'desc':fields.char('Description'),
                'order_id':fields.many2one('purchase.order','Purchase order')
                }

class purchase_order_line(osv.osv):
    _inherit = 'purchase.order.line'
    
    _columns = {
                'project_id':fields.many2one('project.project','Project'),
                'project_task_id':fields.many2one('project.task.type','Project task'),
                'door_id':fields.many2one('product.template','Door'),
                'collection_id':fields.many2one('collection.integrate','Collection'),
                }
    

class purchase_requisition(osv.osv):
    _inherit = 'purchase.requisition'
    
    _columns = {
                'requisition_type': fields.selection([
                                                    ('plumbing', 'Plumbing'),#Сантехник
                                                    ('electricity', 'Electricity'),#Цахилгаан
                                                    ('it', 'IT'),#Мэдээллийн технологи
                                                    ('other', 'Other'),#Бусад
                                                    ], 'Requisition type', track_visibility='onchange'),#Захиалгын төрөл
                'import_manager_id':fields.many2one('res.users','Import manager'),
               	'res_users_id':fields.many2many('res.users','pr_users_id','pr_users_rel','users_pr_id','Import managers'),#Импортын менежерүүд
                'import_manager_mail_created':fields.boolean('IMMC'),
                'user_mail_created':fields.boolean('UMC'),
                'partner_mail_created':fields.boolean('PMC'),
                'state_mail_created':fields.boolean('SMC'),
                'project_id':fields.many2one('project.project','Project'),
                'state_id_before': fields.selection([('draft', 'Ноорог'),
                                                  ('in_progress', 'Батлагдсан'),
                                                  ('open', 'Үнийн саналын сонголт'),
                                                  ('done', 'Худалдан авах захиалга үүссэн'),
                                                  ('cancel', 'Цуцлагдсан')], 'Өмнөх төлөв', track_visibility='onchange'),
                
                }
    _defaults = {
                'date_end': fields.datetime.now,
                'message_unread': True,
                'stage_id_before': 'draft',
                }
        
    def create(self, cr, uid, vals, context=None):
        config_day_obj = self.pool.get('purchase.date.config')
        group_obj = self.pool.get('res.groups') 
        user_obj = self.pool.get('res.users')
        manager_id = group_obj.search(cr, uid, [('name','=',u'Менежер')])
        cr.execute("select gid from res_groups_users_rel where gid=%s and uid =%s"%(manager_id[0],uid))
        manager = cr.fetchone()
        config_id  = config_day_obj.search(cr, uid, [('company_id','=',vals.get('company_id'))])
        if manager == None or manager == []:
            if config_id !=[]:
                today = datetime.now().isoweekday()
                config = config_day_obj.browse(cr, uid, config_id)
                if config.line_id:
                    for line in config.line_id:
                        if today != line.day:
                            print 'Амжилттай хийгдлээ'
                            #raise osv.except_osv(_('Invalid Action!'), (u'Худалдан авалтын захиалга зөвхөн лхагва гарагт хийж болно.'))
                else:
                    raise osv.except_osv(_('Invalid Action!'),_(u'Та өнөөдөр худалдан авалтын захиалга үүсгэх боломжгүй тул менежерээр үүсгэнэ үү!'))
            else:
                raise osv.except_osv(_('Invalid Action!'), _(u'Та худалдан авалтын захиалгын тохиргоог үүсгээгүй байгаа тул үүсгэх боломжгүй'))
    	must_date = date.today() + timedelta(days=2)
    	check_date = datetime.strptime(str(vals['schedule_date']), '%Y-%m-%d')
    	print 'yyyyyyyyyyyyy', check_date.date(), must_date
        if must_date >= check_date.date():
        	raise osv.except_osv(_('Буруу үйлдэл!'), (u'Товлогдсон огноо нь Үнийн саналын огнооноос дор хаяж 3 өдрийн дараа байх ёстой!. Та товлогдсон огноогоо засна уу'))
        list = []
        if vals['requisition_type'] == 'plumbing':
            #print 'nnnnn', user.groups_id['group_plumbing_senior_engineer']
            #user_gr_id = user_obj.search(cr, uid, [('groups_id.group_plumbing_senior_engineer','=','True')])
            #user_gr_id = user_obj.search([('group_ids','in', [g.id for g in self.pool.get('res.users').groups_id]), ('group_plumbing_senior_engineer','=',True)])
            us_gr_id = group_obj.search(cr, uid, [('name','=','Сантехникийн ахлах инженер')])
            gr_id = group_obj.browse(cr, uid, us_gr_id)
            #print 'wwwww', gr_id, gr_id.name, gr_id.users, gr_id.users.name, gr_id.users.id
            if gr_id:
                for line in gr_id.users:
                    list.append(line.id)
                print 'wmmmmmmmmm', list
        elif vals['requisition_type'] == 'electricity':
            us_gr_id = group_obj.search(cr, uid, [('name','=','Цахилгааны ахлах инженер')])
            gr_id = group_obj.browse(cr, uid, us_gr_id)
            #print 'wwwww', gr_id, gr_id.name, gr_id.users, gr_id.users.name, gr_id.users.id
            if gr_id:
                for line in gr_id.users:
                    list.append(line.id)
                print 'wmmmmmmmmm', list
        #rel_users = vals['res_users_id'][0][2] + list
        #vals['res_users_id'] = rel_users
        vals['res_users_id'] = [[6,0, [vals['res_users_id'][0][2] + list]]]
        print 'ssssss', vals
        context = dict(context or {}, mail_create_nolog=True)
        order =  super(purchase_requisition, self).create(cr, uid, vals, context=context)
        self.message_post(cr, uid, [order], body=_("RFQ created"), context=context)
        return order
    
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)
        group_obj = self.pool.get('res.groups') 
        if 'project_id' in vals:
            for lines in obj.line_ids:
                lines.write({'project_id':vals['project_id']})
        if 'state' in vals:
            vals['write_date'] = fields.datetime.now()
            self.write(cr, uid, ids, {'state_id_before': obj.state,
                                      'state_mail_created': False}, context=context)
        if 'user_id' in vals:
            vals['user_id'] = vals['user_id']
            self.write(cr, uid, ids, {'user_mail_created': False}, context=context)
        
        #if 'import_manager_id' in vals:
        #    vals['import_manager_id'] = vals['import_manager_id']
        #    self.write(cr, uid, ids, {'import_manager_mail_created': False}, context=context)
        
        if 'res_users_id' in vals:
            #vals['res_users_id'] = vals['res_users_id']
            self.write(cr, uid, ids, {'import_manager_mail_created': False}, context=context)
        if 'message_follower_ids' in vals:
            print 'tyyyyyyyyyyy', vals['message_follower_ids'], obj.message_follower_ids
            if obj.message_follower_ids != vals['message_follower_ids']:
            #vals['message_follower_ids'] = vals['message_follower_ids']
                self.write(cr, uid, ids, {'partner_mail_created': False}, context=context)
                
        if 'requisition_type' in vals:
            list=[]
            if vals['requisition_type'] == 'plumbing':
                us_gr_id = group_obj.search(cr, uid, [('name','=','Сантехникийн ахлах инженер')])
                gr_id = group_obj.browse(cr, uid, us_gr_id)
                #print 'wwwww', gr_id, gr_id.name, gr_id.users, gr_id.users.name, gr_id.users.id
                if gr_id:
                    for line in gr_id.users:
                        list.append(line.id)
            elif vals['requisition_type'] == 'electricity':
                us_gr_id = group_obj.search(cr, uid, [('name','=','Цахилгааны ахлах инженер')])
                gr_id = group_obj.browse(cr, uid, us_gr_id)
                #print 'wwwww', gr_id, gr_id.name, gr_id.users, gr_id.users.name, gr_id.users.id
                if gr_id:
                    for line in gr_id.users:
                        list.append(line.id)
            list3 = []
            if 'res_users_id' in vals:
                if obj.res_users_id:
                    for line23 in obj.res_users_id:
                        list3.append(line23.id)
                vals['res_users_id'] = [[6,0, [list3 + vals['res_users_id'][0][2] + list]]]
            else:
                if obj.res_users_id:
                    for line23 in obj.res_users_id:
                        list3.append(line23.id)
                vals['res_users_id'] = [[6,0, [list3 + list]]]
	#cr_date  = ''
	#if 'schedule_date' in vals:
		#cr_date = vals['date_end']
	#else:
		#cr_date = obj.date_end
	#must_date = datetime.strptime(str(obj.date_end), '%Y-%m-%d %H:%M:%S').date() + timedelta(days=2)
	if obj.date_end:
		must_date = datetime.strptime(str(obj.date_end), '%Y-%m-%d %H:%M:%S').date() + timedelta(days=2)
	else:
		must_date = date.today()
	schedule_date  = ''
	if 'schedule_date' in vals:
		schedule_date = vals['schedule_date']
	else:
		schedule_date = obj.schedule_date
	#check_date = datetime.strptime(str(vals['schedule_date']), '%Y-%m-%d')
	if obj.schedule_date:
		check_date = datetime.strptime(str(schedule_date), '%Y-%m-%d')
	else:
		check_date = datetime.now() + timedelta(days=2)
		
	#print 'yyyyyyyyyyyyy', check_date.date(), must_date
        if must_date >= check_date.date():
        	raise osv.except_osv(_('Буруу үйлдэл!'), (u'Товлогдсон огноо нь Үнийн саналын огнооноос дор хаяж 3 өдрийн дараа байх ёстой!. Та товлогдсон огноогоо засна уу'))

        return super(purchase_requisition, self).write(cr, uid, ids, vals, context=context)
    
    
    #===========================================================================
    # def onchange_message_follower_ids(self, cr, uid, ids, message_follower_ids):
    #     print 'qqqqqqqqq', self, message_follower_ids
    #     if message_follower_ids:
    #         self.write(cr, uid, ids, {'partner_mail_created': False})
    #     return True
    #===========================================================================
    
    
    #===========================================================================
    # def action_picking_create(self, cr, uid, ids, context=None):
    #     build_id = None
    #     task_id = None
    #     project_id = None
    #     for order in self.browse(cr, uid, ids):
    #         if order.project_id:
    #             project_id = order.project_id.id
    #         if order.build_id:
    #             build_id = order.build_id.id
    #         if order.project_task_id:
    #             task_id = order.project_task_id.id
    #         picking_vals = {
    #             'contract_number':project_id,
    #             'buil_id':build_id,
    #             'project_task_id':task_id,
    #             'picking_type_id': order.picking_type_id.id,
    #             'partner_id': order.dest_address_id.id or order.partner_id.id,
    #             'date': max([l.date_planned for l in order.order_line]),
    #             'origin': order.name
    #         }
    #         picking_id = self.pool.get('stock.picking').create(cr, uid, picking_vals, context=context)
    #         self._create_stock_moves(cr, uid, order, order.order_line, picking_id, context=context)
    #===========================================================================
