# -*- coding: utf-8 -*-
##############################################################################
#
#    Asterisk Technologies LLC, Enterprise Management Solution    
#    Copyright (C) 2013-2013 Asterisk Technologies LLC (<http://www.erp.mn>). All Rights Reserved
#
#    Email : unuruu25@gmail.com
#    Phone : 976 + 88005462
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
import openerp.pooler
from openerp import SUPERUSER_ID
import time
from datetime import datetime
from openerp.http import request
from pychart.tick_mark import Null
from lxml import etree

class purchase_comparison(osv.osv):
    _name = "purchase.comparison"
    _rec_name = "name"
    _description = "Purchase Comparison"

    
    STATE_SELECTION = [('draft','New'),
                       ('confirm','Confirm'),
                       ('done','Done By Director'),
                       ('cancelled','Cancelled'),
                    ]
    
    def _set_department(self, cr, uid, ids, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid)
        employee_ids = self.pool.get('hr.employee').search(cr, uid,[('user_id','=',user.id)])
        if employee_ids:
            employee = self.pool.get('hr.employee').browse(cr, uid, employee_ids)[0]
            return employee.department_id.id
        else:
            raise osv.except_osv(_('Warning!'), _('You don\'t have related employee. Please contact administrator.'))
        return None
    
    _columns = {
        'name': fields.char('Comparison Reference', size=32, required=True,readonly=True),
        'date': fields.date('Comparison Date',required=True),
        'user_id': fields.many2one('res.users','Purchase Employee'),
        'department_id': fields.many2one('hr.department','Department'),
        'assessment': fields.text('Assessment'),
        'state': fields.selection(STATE_SELECTION,'State'),
        'type': fields.selection([
        	('save_of_planned','Save of Planned'),
        	('save_of_last_purchase','Save of Last Purchase'),
        	('save_of_deal','Save of Deal'),
        	('other','Other'),
        ],'Save Type', states={'sent_to_chief': [('readonly', True)]}),
        'save_amount': fields.float('Save Amount',states={'sent_to_chief': [('readonly', True)],
                                                          'sent_to_financial_analyst': [('readonly', True)]
                                                        }),
        'description': fields.text('Description',states={'sent_to_chief': [('readonly', True)],
                                                          'sent_to_financial_analyst': [('readonly', True)]
                                                        }),
        'is_accepted': fields.boolean('Is Accepted',states={'sent_to_financial_analyst': [('readonly', True)]}),
        'buyer_confirmed_date': fields.date('Buyer Confirmed Date'),
        'chief_confirmed_date': fields.date('Chief Confirmed Date'),
        'suspend_date':fields.date('Suspended date',readonly=True),
        'project_id':fields.many2one('project.project','Project',required=True),
       
    }
    
    _defaults = {
                 'user_id':lambda obj, cr, uid, context:uid,
                 'date':fields.date.context_today,
                 'department_id': _set_department,
                 'state':'draft',
                 'name': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'purchase.comparison'),
                
    }
    
    
    def action_confirm(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        purchase_obj = self.pool.get('purchase.order')
        purchase_id = purchase_obj.search(cr, uid, [('comparison_id','=',obj.id),('is_selected_by_employee','=',True)])
        if purchase_id !=[]:
            self.write(cr, uid, ids, {
                                      'state':'confirm',
                                      })
            purchase_obj.write(cr,uid,purchase_id,{
                                           'state':'sent',
                                           })
        else:
            raise osv.except_osv(_('Warning!'), _(u'Худалдан авалтын менежерийн санал гэсэн сонголтоос аль нэгийг заавал сонгоно уу!'))
        return True
    def action_done(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        purchase_obj = self.pool.get('purchase.order')
        purchase_id = purchase_obj.search(cr, uid, [('comparison_id','=',obj.id),('is_selected_by_proc_director','=',True)])
        if purchase_id != []:
            self.write(cr, uid, ids, {
                                      'state':'done',
                                      })
            purchase_obj.write(cr,uid,purchase_id,{
                                                   'state':'approved',
                                                   })
        else:
            raise osv.except_osv(_('Warning!'), _(u'Та хангамжийн захирлын шийдвэр гэсэн сонголтоос аль нэгийг заавал сонгоно уу!'))
                
        return True
    def get_groups(self, cr, uid, context=None):
        # def has_role(self, cr, uid, group_name):#create(self, cr, uid,values,context=None):
        # user_group_ids = self.pool.get('ir.model.data').get_object_reference(self.cr, self.uid, 'l10n_mn_mcsi_proc', None)


        # self.print_log(user_group_ids, "GROUPD IDS:")

        # if user_group_ids :
        #      = self.pool.get('res.users').search(self.cr,self.uid,[('groups_id','in',[user_group_ids[1]])])
            
        #     return self.uid in sel_user_ids

        # return False
        s = self
        query = "\
            SELECT \
                rgur.gid as id,\
                imd.name as name\
            FROM ir_model_data imd\
            INNER JOIN res_groups_users_rel rgur ON rgur.gid = imd.res_id \
            WHERE \
                rgur.uid = " + str(uid) + " AND \
                imd.name like 'group_mno%'"
        cr.execute(query)
        return cr.dictfetchall()

purchase_comparison()


class purchase_order(osv.osv):
    _inherit = "purchase.order"
    
    def _calc_price(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for order in self.browse(cr, uid, ids):
            res[order.id] = order.amount_total
        return res
    
    def _calc_total_eval(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for order in self.browse(cr, uid, ids):
            res[order.id] = order.calculated_price
        return res
    def onchange_comparison(self, cr, uid, ids, comparison_id, context=None):
        if ids !=False:
            order = self.browse(cr, uid, ids, context=context)
            value = {}
            warning = ''
            if comparison_id:
                comparison = self.pool.get('purchase.comparison').browse(cr, uid, comparison_id, context=context)
                another_order_id = self.pool.get('purchase.order').search(cr, uid,[('comparison_id','=',comparison.id)])
                if another_order_id !=[]:
                    another_order = self.pool.get('purchase.order').browse(cr, uid, another_order_id[0], context=context)
                    services = order.po_line_compare(po_ids=[another_order_id[0]],context=context)
                    if services:
                        value = {'comparison_id': comparison.id}
                    else:
                        warning += (u'Харьцуулалт үүсгэх үнийн санал дахь бараатай адил бараа байх хэрэгтэй!')
        if warning:
            value['comparison_id']= None
            return {'value': value,'warning':{'title':_('Warning!'),'message':warning}}
        else:
            return {'value': value}
    def po_line_compare(self,cr, uid, ids, po_ids,context=None):
        if not isinstance(po_ids,list): 
            po_ids = [po_ids]
        for id in ids:
            if id not in po_ids:
                po_ids.append(id)
        pos = self.browse(cr, uid, po_ids)
        product_ids = pos[0].get_product_ids_by_type()
        services = []
        for po in pos:
            pids = po.get_product_ids_by_type()
            for pid in product_ids['consu']:
                if pid not in pids['consu']:
                    return False
            services.extend(pids['service'])
        return services if services else True   
            
    def get_product_ids_by_type(self,cr, uid, ids, context=None):
        this = self.browse(cr, uid, ids)
        product_ids = {'consu': [], 'service':[]}
        for po in this:
            for line in po.order_line:
                if line.product_id.product_tmpl_id.type != 'service':
                    product_ids['consu'].append(line.product_id.id)
                else:
                    product_ids['service'].append(line.product_id.id)
        return product_ids
    _columns = {
        'comparison_id': fields.many2one('purchase.comparison','Purchase Comparison', domain=[('state','=','draft')]),
        'is_selected_by_employee': fields.boolean('Is Selected By Employee'),
        'is_selected_by_director': fields.boolean('Is Selected Director'),
        'is_selected_by_proc_director':fields.boolean('Is Selected by Procurement'),
        'calculated_price':fields.function(_calc_price, method=True, store=True,string='Calculated Price'),
        'experience_amount': fields.text('Experience Evaluation'),
        'guarantee_amount': fields.text('Guarantee Evaluation'),
        'supply_dur_amount': fields.text('Supply Duration Evaluation'),
        
        'operational_cost': fields.float('Operational Cost'), 
        'maintenance_cost': fields.float('Maintenance Cost'),
        'spare_parts_cost': fields.float('Spare Parts Cost'),
        'quality_cost': fields.float('Quality Cost'),
        'total_evaluation':fields.function(_calc_total_eval, method=True, string='Total Evaluation'),
    }
    
    def download_attachment(self, cr, uid, id_message, attachment_id, context=None):
        """ Return the content of linked attachments. """
        attachment = self.pool.get('ir.attachment').browse(cr, SUPERUSER_ID, attachment_id, context=context)
        if attachment.datas and attachment.datas_fname:
            return {
                'base64': attachment.datas,
                'filename': attachment.datas_fname,
            }
        return False
    
purchase_order()

class purchase_comparison1(osv.osv):
    _inherit = "purchase.comparison"
    _columns = {
        'order_ids': fields.one2many('purchase.order','comparison_id','Purchase Orders'),
        #'state_history_idss': fields.one2many('purchase.comparison.state.history', 'comparison_id', 'Purchase comparison State History'),
        'product_id': fields.related('order_ids','product_id', type='many2one', relation='product.product', string='Product'),
        'partner_id': fields.related('order_ids','partner_id', type='many2one', relation='res.partner', string='Product'),
    }

    
    def check_vote(self, cr, uid, ids, role):
        comparison = self.pool.get('purchase.comparison').browse(cr, uid, ids, context = None)
        selected = []
        not_selected = []
        for order in comparison[0].order_ids:
            if (role == 'employee' and order.is_selected_by_employee) or (role == 'director' and order.is_selected_by_director) or (role=='proc_director' and order.is_selected_by_proc_director) :
                selected.append(order.id)
            else:
                not_selected.append(order.id)
        if not selected:
            raise osv.except_osv(_('Warning!'), _(u'Та аль нэг нийлүүлэгчид саналаа өгнө үү!'))
        if len(selected) >= 2:
            raise osv.except_osv(_('Warning!'), _(u'Та зөвхөн нэг нийлүүлэгчид саналаа өгнө үү!'))
        return selected, not_selected
    
    def set_didnt_win_state(self, cr, uid, not_selected, context=None):
        return self.pool.get('purchase.order').write(cr, uid, not_selected, {'state':'did_not_win'})
    
    def add_follower(self, cr, uid, ids):
        user = self.pool.get('res.users').browse(cr, uid, uid)
        mail_obj = self.pool.get('mail.followers')
        is_follower = mail_obj.search(cr, uid, [('partner_id','=',user.partner_id.id),('res_model','=','purchase.comparison'),('res_id','=',ids[0])])
     
        if not is_follower:
           
            return self.pool.get('mail.followers').create(cr, uid, {'res_model':'purchase.comparison','res_id':ids[0], 'partner_id':user.partner_id.id})
        return True
    
    def send_notif_to_followers(self, cr, uid, ids, signal):
        states = {'draft': u'Ноорог',
                  'sent_to_dep_director': u'Газрын захиралд илгээгдсэн',
                  'sent_to_proc_director': u'Хангамжийн захиралд илгээгдсэн',
                  'sent_to_ceo': u'Захирлуудын зөвлөлд илгээгдсэн',
                'sent_to_financial_director': u'Санхүүгийн захиралд илгээгдсэн',
                  'cancel': u'Цуцлагдсан',
                    'done': u'Шалгарсан',
                  'suspend':u'Хойшлогдсон'
                  }
        user = self.pool.get('res.users').browse(cr, uid, uid)
        mail_obj = self.pool.get('mail.followers')
        template_id = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'l10n_mn_purchase_comparison', 'follower_notif_email_template')[1]
        
        
        follower_ids = mail_obj.search(cr, SUPERUSER_ID, [('res_model','=','purchase.comparison'),('res_id','in',ids)])
     
        
        if follower_ids:
            followers = self.pool.get('mail.followers').browse(cr, SUPERUSER_ID, follower_ids)
            
            user_ids = []
            if followers:
                user_ids = self.pool.get('res.users').search(cr, uid, [('partner_id','in',[follower.partner_id.id for follower in followers])])
            
            comparison = self.browse(cr, uid, ids)
            comp=""
            if comparison[0].department_id.parent_id:
                    comp=comparison[0].department_id.parent_id.name+"/"+comparison[0].department_id.name
                
            data = {
                    'subject': 'Purchase Comparison Follower Notification',
                    'model': 'purchase.comparison',
                    'department': comp,
                    'name': comparison[0].name,
                    'base_url': self.pool.get('ir.config_parameter').get_param(cr, uid, 'web.base.url'),
                    'action_id': self.pool.get('ir.model.data').get_object_reference(cr, uid, 'l10n_mn_purchase_comparison', 'action_purchase_comparison')[1],
                    'id': comparison[0].id,
                    'db_name': request.session.db,
                    'state': states[signal],
                    'sender': self.pool.get('res.users').browse(cr, uid, uid).name,
                    'menu_path': u'Худалдан авалт / Худалдан авалт / Худалдан авалтын харьцуулалт',
                    }
            
            for user_id in user_ids:
                if user_id != uid:
                   
                    self.pool.get('email.template').send_mail(cr, uid, template_id, user_id, force_send=True, context=data)
        return True
    
    def send_notification(self, cr, uid, ids, signal):
        model_obj = openerp.pooler.get_pool(cr.dbname).get('ir.model.data')
        groups = {'send_notif_to_dep_director':'group_mno_dep_director',
                  'send_notif_to_proc_director':'group_mno_proc_director',
                  'send_notif_to_ceo':'group_mno_ceo',
                  'send_notif_to_financial_director':'group_mno_financial_director'}
        
        states = {'send_notif_to_dep_director': u'Газрын захиралд илгээгдсэн',
                  'send_notif_to_proc_director': u'Хангамжийн захиралд  илгээгдсэн',
                  'send_notif_to_ceo': u'Захирлуудын зөвлөлд илгээгдсэн',
                  'send_notif_to_financial_director':u'Санхүү хариуцсан захиралд илгээгдсэн'
                }
        print 'signal: ', signal
        notif_groups = model_obj.get_object_reference(cr, uid, 'l10n_mn_purchase', groups[signal])
        group_user_ids = None
        comparison = self.browse(cr, uid, ids)
        if comparison:
            sel_user_ids = self.pool.get('res.users').search(cr,uid,[('groups_id','in',[notif_groups[1]])])
            #Нэвтэрсэн хэрэглэгчийн хэлтсийн, chief группын ажилчдын жагсаалт
#
       
            if signal in ['send_notif_to_dep_director']:
#               
                comp=comparison[0].department_id
    
                while comp.parent_id:
                    
                    notif_groups = model_obj.get_object_reference(cr, uid, 'l10n_mn_purchase', groups['send_notif_to_dep_director'])
                    sel_user_ids1 = self.pool.get('res.users').search(cr,uid,[('groups_id','in',[notif_groups[1]])])
                    print 'sel_user_ids1: ', sel_user_ids1
                    if sel_user_ids1:
                        group_user_ids = self.pool.get('res.users').search(cr,uid,[('id','in',sel_user_ids),('department_id','=',comp.id)])
                    comp=comp.parent_id    
            else:
                    group_user_ids = self.pool.get('res.users').search(cr,uid,[('id','in',sel_user_ids),])
         
#             group_user_ids = self.pool.get('res.users').search(cr,uid,[('id','in',sel_user_ids),('department_id','=',comparison[0].department_id.id)])
          
            
            data = {
                    'department': comparison[0].department_id.name,
                    'name': comparison[0].name,
                    'base_url': self.pool.get('ir.config_parameter').get_param(cr, uid, 'web.base.url'),
                    'action_id': self.pool.get('ir.model.data').get_object_reference(cr, uid, 'l10n_mn_purchase_comparison', 'action_purchase_comparison')[1],
                    'id': comparison[0].id,
                    'db_name': request.session.db,
                    'state': states[signal],
                    'buyer': comparison[0].user_id.name,
                    'sender': self.pool.get('res.users').browse(cr, uid, uid).name,
                    }
            
            
            template_id = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'l10n_mn_purchase_comparison', 'comparison_notif_cron_email_template')[1]
            if group_user_ids:
                users = self.pool.get('res.users').browse(cr, uid, group_user_ids)
                user_emails = []
                for user in users:
                    
                    user_emails.append(user.login)
                    self.pool.get('email.template').send_mail(cr, uid, template_id, user.id, force_send=True, context=data)
                email = u'Төлөв: → ' + states[signal]
                self.pool.get('purchase.comparison').message_post(cr, uid, ids, body= email + u'.\n Дараах хэрэглэгчид рүү имэйл илгээгдэв: ' + ', '.join(user_emails), context=None)
            else:
                raise osv.except_osv(_('Warning!'), _(u'Хүлээн авагч олдсонгүй. Та админтай холбогдоно уу!'))
        return True
    
    def send_to_directors(self, cr, uid, ids, context=None):
        selected, not_selected = self.check_vote(cr, uid, ids, 'employee')
        comparison = self.browse(cr, uid, ids)
        exigent=False
        in_budget=False
        max_amount=0
        selected_order = self.pool.get('purchase.comparison').browse(cr, uid, ids)
        
        for order in comparison[0].order_ids:
                if order.requisition_id.is_exigent:
                    exigent=True
                if order.amount_total>max_amount:
                    max_amount=order.amount_total
        if selected_order[0].assessment==False:
            raise osv.except_osv(_('Warning!'), _(u'Харьцуулалтын дүгнэлтээ бичнэ үү'))            
#         if exigent or (max_amount <= 100000000 and max_amount > 50000000) :
#             self.send_notification(cr, uid, ids, 'send_notif_to_dep_director')
#             self.send_notif_to_followers(cr, uid, ids, 'sent_to_dep_director')
#             self.add_follower(cr, uid, ids)
#             self.add_state_history(cr, uid, ids, 'comp_sent_to_dep_director')
#             self.add_comparison_state_history(cr, uid, ids, 'sent_to_dep_director')
#             return self.write(cr, uid, ids, {'state': 'sent_to_dep_director'})
#         elif in_budget or max_amount <= 50000000:
#                 self.send_notification(cr, uid, ids, 'send_notif_to_proc_director')
#                 self.send_notif_to_followers(cr, uid, ids, 'sent_to_proc_director')
#                 self.add_state_history(cr, uid, ids, 'comp_sent_to_proc_director')
#                 self.add_comparison_state_history(cr, uid, ids, 'sent_to_proc_director')
#                 self.add_follower(cr, uid, ids)
#                 return self.write(cr, uid, ids, {'state': 'sent_to_proc_director'})
#         else:
#             self.send_notification(cr, uid, ids, 'send_notif_to_ceo')
#             self.send_notif_to_followers(cr, uid, ids, 'sent_to_ceo')
#             self.add_state_history(cr, uid, ids, 'comp_sent_to_ceo')
#             self.add_comparison_state_history(cr, uid, ids, 'sent_to_ceo')
#            
#             self.add_follower(cr, uid, ids)
#             return self.write(cr, uid, ids, {'state': 'sent_to_ceo'})
        if exigent or max_amount <= 100000000 :
            self.send_notification(cr, uid, ids, 'send_notif_to_dep_director')
            self.send_notif_to_followers(cr, uid, ids, 'sent_to_dep_director')
            self.add_follower(cr, uid, ids)
            self.add_state_history(cr, uid, ids, 'comp_sent_to_dep_director')
            #self.add_comparison_state_history(cr, uid, ids, 'sent_to_dep_director')
            self.wishlist_add_state_history(cr, uid, ids, 'comp_sent_to_dep_director')
                    
            return self.write(cr, uid, ids, {'state': 'sent_to_dep_director'})
       
        else: 
            self.send_notification(cr, uid, ids, 'send_notif_to_ceo')
            self.send_notif_to_followers(cr, uid, ids, 'sent_to_ceo')
            self.add_state_history(cr, uid, ids, 'comp_sent_to_ceo')
           # self.add_comparison_state_history(cr, uid, ids, 'sent_to_ceo')
            self.wishlist_add_state_history(cr, uid, ids, 'comp_sent_to_ceo')
            self.add_follower(cr, uid, ids)
            return self.write(cr, uid, ids, {'state': 'sent_to_ceo'})
    def send_to_proc_directors(self,cr,uid,ids,context=None):
            comparison = self.browse(cr, uid, ids)
            selected, not_selected = self.check_vote(cr, uid, ids, 'director')
            is_selected_by_employee = self.pool.get('purchase.order').search(cr, uid, [('id','in',[order.id for order in comparison[0].order_ids]),('is_selected_by_employee','=',True)])
            is_selected_by_director = self.pool.get('purchase.order').search(cr, uid, [('id','in',[order.id for order in comparison[0].order_ids]),('is_selected_by_director','=',True)])
            if is_selected_by_employee[0] != is_selected_by_director[0]:
                mod_obj = self.pool.get('ir.model.data')
                res = mod_obj.get_object_reference(cr, uid, 'l10n_mn_purchase_comparison', 'action_different_choice_note')
                context['approve'] = True
                context['button'] = 'done_by_director'
                return {
                'name': 'Comparison note',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'purchase.comparison.different.choice.note',
                'context': context,
                'type': 'ir.actions.act_window',
                'nodestroy': True,
                'target': 'new',
                }
            else:
                self.change_state_to_proc_director(cr, uid, ids, context)
              
          
            
    def send_to_financial_directors(self, cr, uid, ids, context=None):
        comparison = self.browse(cr, uid, ids)
        
        selected, not_selected = self.check_vote(cr, uid, ids, 'proc_director')
        is_selected_by_employee = self.pool.get('purchase.order').search(cr, uid, [('id','in',[order.id for order in comparison[0].order_ids]),('is_selected_by_employee','=',True)])
        is_selected_by_director = self.pool.get('purchase.order').search(cr, uid, [('id','in',[order.id for order in comparison[0].order_ids]),('is_selected_by_director','=',True)])
        is_selected_by_proc_director = self.pool.get('purchase.order').search(cr, uid, [('id','in',[order.id for order in comparison[0].order_ids]),('is_selected_by_proc_director','=',True)])
        

        if is_selected_by_employee[0] != is_selected_by_director[0] or is_selected_by_employee[0] != is_selected_by_proc_director[0] :
            mod_obj = self.pool.get('ir.model.data')
            res = mod_obj.get_object_reference(cr, uid, 'l10n_mn_purchase_comparison', 'action_different_choice_note')
            context['approve'] = True
            context['button'] = 'done_by_proc_director'
            return {
                'name': 'Comparison note',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'purchase.comparison.different.choice.note',
                'context': context,
                'type': 'ir.actions.act_window',
                'nodestroy': True,
                'target': 'new',
            }
        else:
            self.change_state_to_financial_director(cr, uid, ids, context)
    def change_state_to_proc_director(self, cr, uid, ids, context=None):      
        
        selected_order = self.pool.get('purchase.comparison').browse(cr, uid, ids)
        self.send_notification(cr, uid, ids, 'send_notif_to_proc_director')
        self.send_notif_to_followers(cr, uid, ids, 'sent_to_proc_director')
        self.add_state_history(cr, uid, ids, 'comp_sent_to_proc_director')
        #self.add_comparison_state_history(cr, uid, ids, 'sent_to_proc_director')
        comparison = self.browse(cr, uid, ids)
        self.wishlist_add_state_history(cr, uid, ids, 'comp_sent_to_proc_director')
        self.add_follower(cr, uid, ids)
        return self.write(cr, uid, ids, {'state': 'sent_to_proc_director'})
    def change_state_to_financial_director(self, cr, uid, ids, context=None):
        self.send_notification(cr, uid, ids, 'send_notif_to_financial_director')
        self.send_notif_to_followers(cr, uid, ids, 'sent_to_financial_director')
        self.add_state_history(cr, uid, ids, 'comp_sent_to_financial_director')
        comparison = self.browse(cr, uid, ids)
        self.wishlist_add_state_history(cr, uid, ids, 'comp_sent_to_financial_director')
      #  self.add_comparison_state_history(cr, uid, ids, 'sent_to_financial_director')

        self.add_follower(cr, uid, ids)
        
        return self.write(cr, uid, ids, {'state': 'sent_to_financial_director'})

    def suspend_financial_director(self,cr,uid,ids,date, context=None):
        states = {'Suspended': u'Хойшлогдсон',
                }
        self.add_state_history(cr, uid, ids, 'comp_suspended')
        comparison = self.browse(cr, uid, ids)
        self.wishlist_add_state_history(cr, uid, ids, 'comp_suspended')
        self.send_notif_to_followers(cr, uid, ids, 'suspend')
        email = u'Төлөв: → ' + states['Suspended']
        self.pool.get('purchase.comparison').message_post(cr, uid, ids, body= email, context=None)
        return self.write(cr, uid, ids, {'state': 'suspended',
                                         'suspend_date':date
                                         })
    def done_by_financial_director(self, cr, uid, ids, context=None):
        selected, not_selected = self.check_vote(cr, uid, ids, 'director')
     
        selected_order = self.pool.get('purchase.order').browse(cr, uid, selected)
        self.add_follower(cr, uid, ids)
        if not_selected:
            self.set_didnt_win_state(cr, uid, not_selected)
        self.pool.get('purchase.order').wkf_approve_order(cr, uid, selected)
        self.send_notif_to_followers(cr, uid, ids, 'done')
        self.add_state_history(cr, uid, ids, 'comp_done')
        #self.add_comparison_state_history(cr, uid, ids, 'done')
        comparison = self.browse(cr, uid, ids)
        self.wishlist_add_state_history(cr, uid, ids, 'comp_done')
        self.update_req_line_state(cr, uid, ids, selected_order[0], 'purchased')
        return self.write(cr, uid, ids, {'state': 'done'})
    def wishlist_add_state_history(self,cr,uid,ids,state):
        comparison = self.browse(cr, uid, ids)[0]
        for poline in comparison.order_ids:
                    for rline in poline.requisition_id.line_ids:
                        for wline in rline.wishlist_line_ids:
                            wish_line_hist_id=self.pool.get('purchase.wishlist.line.state.history').create(cr,SUPERUSER_ID,{
                                                                                                        'state': state,
                                                                                                        'user_id': uid,
                                                                                                        'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                                                                                                        'wishlist_line_id': wline.id                      
                                                                                                                            
                                                                                                                            })
#                     print "dfssdfds"
                    break
                                        
                
    def add_state_history(self, cr, uid, ids, state):
        comparison = self.browse(cr, uid, ids)[0]
        for order in comparison.order_ids:
            for line in order.order_line:
                req_line_hist_id = self.pool.get('purchase.requisition.line.state.history').create(cr, SUPERUSER_ID, {
                                                            'state': state,
                                                            'user_id': uid,
                                                            'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                                                            'requisition_line_id': line.requisition_line_id.id
                                                            })
            break
    def add_purchase_state_history(self, cr, uid, ids, state):
        order_ids = self.pool.get('purchase.order').browse(cr, uid, ids)
        for order in order_ids:
            order_state_hist_id = self.pool.get('purchase.order.state.history').create(cr, SUPERUSER_ID, {
                                                    'state': state,
                                                    'user_id': uid,
                                                    'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                                                    'order_id': order.id
                                                    })
    #===========================================================================
    # def add_comparison_state_history(self, cr, uid, ids, state):
    #     comparison = self.browse(cr, uid, ids)[0]
    #     comp_state_hist_id = self.pool.get('purchase.comparison.state.history').create(cr, SUPERUSER_ID, {
    #                                                 'state': state,
    #                                                 'user_id': uid,
    #                                                 'date': time.strftime('%Y-%m-%d %H:%M:%S'),
    #                                                 'comparison_id': comparison.id
    #                                                 })
    #===========================================================================
    def update_req_line_state(self, cr, uid, ids, selected_order, state):
        for line in selected_order.order_line:
            self.pool.get('purchase.requisition.line').write(cr, uid, line.requisition_line_id.id, {'state': state, 'done': True})
        
    
    def reject(self, cr, uid, ids, context=None):
        states = {'Draft': u'Шинэ',
                }
        self.add_state_history(cr, uid, ids, 'comp_rejected')
#         self.add_comparison_state_history(cr, uid, ids, 'rejected')
        email = u'Төлөв: → ' + states['Draft']
        self.pool.get('purchase.comparison').message_post(cr, uid, ids, body= email, context=None)
            
        self.send_notif_to_followers(cr, uid, ids, 'draft')
        
        return self.write(cr, uid, ids, {'state': 'draft'})
        
    def cancel(self, cr, uid, ids, context=None):
        
        self.add_state_history(cr, uid, ids, 'comp_cancelled')
        #self.add_comparison_state_history(cr, uid, ids, 'cancelled')
        self.send_notif_to_followers(cr, uid, ids, 'cancel')
        return self.write(cr, uid, ids, {'state': 'cancelled'})
  
    def unlink(self, cr, uid, ids, context=None):
        state_history_obj = self.pool.get('purchase.order.state.history')
        comparison = self.browse(cr, uid, ids)[0]
        
        if comparison.user_id.id != uid:
            raise osv.except_osv(_('Warning!'), _(u'Та энэхүү харьцуулалтыг үүсгэсэн хүн биш тул устгах эрхгүй'))
        else:
            if comparison.state != 'draft':
                raise osv.except_osv(_('Warning!'), _(u'Энэхүү харьцуулалт нь ноорог төлөвтэй биш тул та устгах боломжгүй'))
            
        for order in comparison.order_ids:
            last_state_id = state_history_obj.search(cr, uid, [('order_id','=',order.id), ('state','<>','comparison_created')], order='date desc', limit=1)
            
            if last_state_id:
                last_state = state_history_obj.browse(cr, uid, last_state_id)[0]
                self.pool.get('purchase.order').write(cr, uid, order.id, {'state': last_state.state})
            else:
                self.pool.get('purchase.order').write(cr, uid, order.id, {'state': ('master_draft')})
                
            state_history_obj.create(cr, uid, {
                                            'state':'draft',
                                            'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                                            'user_id': uid,
                                            'order_id': order.id
                                            })
        
        return super(purchase_comparison, self).unlink(cr, uid, ids, context=context)
  
purchase_comparison1()
