# -*- coding: utf-8 -*-

import math
import re
import time

from openerp import SUPERUSER_ID
from openerp import tools
from openerp.osv import osv, fields, expression
from openerp.tools.translate import _
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
from openerp.modules.module import get_module_resource
import psycopg2

from openerp.tools.safe_eval import safe_eval as eval
import openerp.addons.decimal_precision as dp
from openerp.tools.float_utils import float_round

class product_template(osv.osv):
    _inherit = "product.template"
    _description = "Product Template"
    _order = "name"
    
    _columns = {
                'is_asset': fields.boolean('Is asset'),
                'building_id':fields.many2one('building.building','Building'),
                'asset_type': fields.selection([
                    ('apartment', 'Apartment'),
                    ('garage', 'Garage'),
                    ('object', 'Object')
                    ], 'Asset type'),
                'project_id':fields.many2one('project.project','Project'),
                'room_size':fields.integer('Room size'),
                'windows_location':fields.many2many('windows.location', 'windows_location_rel','door_id','windows_location', 'Windows location'),
                'total_squaring':fields.float('Total squaring'),
                'houseroom':fields.float('Houseroom'),
                'entrance':fields.char('Entrance'),
                'floor':fields.char('Floor'),
                'atso':fields.integer('ATsO number'),
                'door_type':fields.many2one('door.type','Door type'),
                #'default_code': fields.related('product_variant_ids', 'default_code', type='char', string='Internal Reference'),
                'new_code':fields.related('product_variant_ids', 'new_code', type='char', string='Product code'),
                #'new_code':fields.char('Product code', size=8, required=True),
                'retail_price':fields.integer('Retail price'),
                'disbursing_price':fields.integer('Disbursing price'),
                'collect_income_day':fields.integer('Collection income day'),
                'size':fields.char('Size'),
                'color_shape':fields.char('Color, Shape'),
                'others':fields.char('Others'),
                }
    
    _defaults = {
                #'company_id': lambda s,cr,uid,c: s.pool.get('res.company')._company_default_get(cr, uid, 'product.template', context=c),
                'company_id': False,
                'type' : 'product',
                #'cost_method' : 'average',
                #'valuation' : 'Real Time (automated)',
                'is_asset': False,
                 }
    #===========================================================================
    # _defaults = {
    #     'company_id': lambda s,cr,uid,c: s.pool.get('res.company')._company_default_get(cr, uid, 'product.template', context=c),
    #     'list_price': 1,
    #     'standard_price': 0.0,
    #     'sale_ok': 1,        
    #     'uom_id': _get_uom_id,
    #     'uom_po_id': _get_uom_id,
    #     'uos_coeff': 1.0,
    #     'mes_type': 'fixed',
    #     'categ_id' : _default_category,
    #     'type' : 'consu',
    #     'active': True,
    # }
    #===========================================================================
    
    _sql_constraints = [
        ('new_code', 'unique (new_code)', u' Барааны код давхцаж болохгүй тул та өөр код өгнө үү!')
    ]
    
    def create_variant_ids(self, cr, uid, ids, context=None):
        product_obj = self.pool.get("product.product")
        ctx = context and context.copy() or {}

        if ctx.get("create_product_variant"):
            return None

        ctx.update(active_test=False, create_product_variant=True)

        tmpl_ids = self.browse(cr, uid, ids, context=ctx)
        for tmpl_id in tmpl_ids:
            # list of values combination
            all_variants = [[]]
            #Pr huwilbariin mor
            for variant_id in tmpl_id.attribute_line_ids:
                if len(variant_id.value_ids) > 1:
                    temp_variants = []
                    for value_id in variant_id.value_ids:
                        for variant in all_variants:
                            temp_variants.append(variant + [int(value_id)])
                    all_variants = temp_variants
                    

            # check product
            variant_ids_to_active = []
            variants_active_ids = []
            variants_inactive = []
            for product_id in tmpl_id.product_variant_ids:
                variants = map(int,product_id.attribute_value_ids)
                if variants in all_variants:
                    variants_active_ids.append(product_id.id)
                    all_variants.pop(all_variants.index(variants))
                    if not product_id.active:
                        variant_ids_to_active.append(product_id.id)
                else:
                    variants_inactive.append(product_id)
            if variant_ids_to_active:
                product_obj.write(cr, uid, variant_ids_to_active, {'active': True,}, context=ctx)
            # create new product
            for variant_ids in all_variants:
                values = {
                        'collect_income_day':tmpl_id.collect_income_day,
                        'retail_price':tmpl_id.retail_price,
                        'disbursing_price':tmpl_id.disbursing_price,
                        'new_code':tmpl_id.new_code,
                        'product_tmpl_id': tmpl_id.id,
                        'attribute_value_ids': [(6, 0, variant_ids)],
                        'is_asset': tmpl_id.is_asset,
                        'building_id': tmpl_id.building_id.id,
                        'asset_type': tmpl_id.asset_type,
                        'room_size': tmpl_id.room_size,
                        'total_squaring': tmpl_id.total_squaring,
                        'houseroom': tmpl_id.houseroom,
                        'entrance': tmpl_id.entrance,
                        'floor': tmpl_id.floor,
                        #'windows_location': [(6, 0 ,windows_location)],
                        'atso': tmpl_id.atso,
                        'door_type': tmpl_id.door_type.id,
                        'size':tmpl_id.size,
                        'color_shape':tmpl_id.color_shape,
                        'others':tmpl_id.others,
                }
                id = product_obj.create(cr, uid, values, context=ctx)
                variants_active_ids.append(id)

            # unlink or inactive product
            for variant_id in map(int,variants_inactive):
                try:
                    with cr.savepoint():
                        product_obj.unlink(cr, uid, [variant_id], context=ctx)
                except (psycopg2.Error, osv.except_osv):
                    product_obj.write(cr, uid, [variant_id], {'active': False}, context=ctx)
                    pass
        return True
    
    def create(self, cr, uid, vals, context=None):
        ''' Store the initial standard price in order to be able to retrieve the cost of a product template for a given date'''
        product_template_id = super(product_template, self).create(cr, uid, vals, context=context)
        if not context or "create_product_product" not in context:
            self.create_variant_ids(cr, uid, [product_template_id], context=context)
        self._set_standard_price(cr, uid, product_template_id, vals.get('standard_price', 0.0), context=context)
        # TODO: this is needed to set given values to first variant after creation
        # these fields should be moved to product as lead to confusion
        #=======================================================================
        # if vals.get('new_code'):
        #     if len(vals.get('new_code'))<8:
        #         raise osv.except_osv(_(u'Анхааруулга!'), _(u'Барааны код 8 оронтой байх ёстой.') )
        #=======================================================================
        
        #=======================================================================
        # new_code_new = ''
        # code_new = ''
        # val = 0
        # categ_obj = self.pool.get('product.category')
        # cat_id = categ_obj.browse(cr, uid, categ_obj.search(cr, uid, [('id','=',vals.get('categ_id'))]))
        # if cat_id.category_code:
        #     new_code_new = cat_id.category_code
        #     cr.execute("select new_code from product_product where new_code like '"+new_code_new+"%' order by new_code desc limit 1")
        #     cont_code = cr.dictfetchall()
        #     if cont_code:
        #         cont_code = cont_code[0]
        #         val = cont_code['new_code'][len(new_code_new):]
        #         val = int(val) + 1
        #     else:
        #         val = 1
        # else:
        #     raise osv.except_osv(_(u'Анхааруулга!'), _(u'Барааны ангилал нь кодгүй байна. Та барааны ангиллын кодыг эхэлж өгнө үү') )
        # code_new = u'%s'%new_code_new + u'%s'%str(val).zfill(3)
        # print 'ccccccccccccc', code_new
        #=======================================================================
        
        related_vals = {}
        if vals.get('collect_income_day'):
            related_vals['collect_income_day'] = vals['collect_income_day']
        if vals.get('ean13'):
            related_vals['ean13'] = vals['ean13']
        if vals.get('default_code'):
            related_vals['default_code'] = vals['default_code']
        if vals.get('new_code'):
            related_vals['new_code'] = vals.get('new_code')
        if vals.get('retail_price'):
             related_vals['retail_price'] = vals['retail_price']
        if vals.get('disbursing_price'):
             related_vals['disbursing_price'] = vals['disbursing_price']
        if related_vals:
            self.write(cr, uid, product_template_id, related_vals, context=context)
        if vals.get('is_asset'):
            if vals['is_asset'] == True:
                windows = vals['windows_location'][0]
                information = self.pool.get('sales.information')
                building = self.pool.get('building.building')
                bui_obj = building.browse(cr, uid, building.search(cr, uid, [('id','=',vals['building_id'])]))
                product = self.pool.get('product.product')
                pro_obj = product.browse(cr, uid, product.search(cr, uid, [('product_tmpl_id','=',product_template_id)]))
                information.create(cr, uid,{
                                            'product_id': product_template_id,
                                            'to_commission_season': bui_obj.to_commission_season.id,
                                            'windows_location': [(6, 0 ,vals['windows_location'][0][2])],
                                            'floor': vals['floor'] or False,
                                            'entrance': vals['entrance'] or False,
                                            'building_id': vals['building_id'],
                                            'total_squar': vals['total_squaring'] or False,
                                            'project_id': bui_obj.project_id.id,
                                            'state': 'unsold',
                                            })
        
        return product_template_id
         
    def write(self, cr, uid, ids, vals, context=None):
        ''' Store the standard price change in order to be able to retrieve the cost of a product template for a given date'''
        obj = self.browse(cr, uid, ids)
        #=======================================================================
        # if 'categ_id' in vals:
        #     new_code_new = ''
        #     code_new = ''
        #     val = 0
        #     categ_obj = self.pool.get('product.category')
        #     cat_id = categ_obj.browse(cr, uid, categ_obj.search(cr, uid, [('id','=',vals.get('categ_id'))]))
        #     if cat_id.category_code:
        #         new_code_new = cat_id.category_code
        #         cr.execute("select new_code from product_product where new_code like '"+new_code_new+"%' order by new_code desc limit 1")
        #         cont_code = cr.dictfetchall()
        #         if cont_code:
        #             cont_code = cont_code[0]
        #             val = cont_code['new_code'][len(new_code_new):]
        #             val = int(val) + 1
        #         else:
        #             val = 1
        #     else:
        #         raise osv.except_osv(_(u'Анхааруулга!'), _(u'Барааны ангилал нь кодгүй байна. Та барааны ангиллын кодыг эхэлж өгнө үү') )
        #     code_new = u'%s'%new_code_new + u'%s'%str(val).zfill(3)
        #     if obj.categ_id.id == vals['categ_id']:
        #         pass
        #     else:
        #         vals['new_code'] = code_new
        #=======================================================================
        if isinstance(ids, (int, long)):
            ids = [ids]
        if 'uom_po_id' in vals:
            new_uom = self.pool.get('product.uom').browse(cr, uid, vals['uom_po_id'], context=context)
            for product in self.browse(cr, uid, ids, context=context):
                old_uom = product.uom_po_id
                if old_uom.category_id.id != new_uom.category_id.id:
                    raise osv.except_osv(_('Unit of Measure categories Mismatch!'), _("New Unit of Measure '%s' must belong to same Unit of Measure category '%s' as of old Unit of Measure '%s'. If you need to change the unit of measure, you may deactivate this product from the 'Procurements' tab and create a new one.") % (new_uom.name, old_uom.category_id.name, old_uom.name,))
        if 'standard_price' in vals:
            for prod_template_id in ids:
                self._set_standard_price(cr, uid, prod_template_id, vals['standard_price'], context=context)
        if 'attribute_line_ids' in vals or vals.get('active'):
            self.create_variant_ids(cr, uid, ids, context=context)
        if 'active' in vals and not vals.get('active'):
            ctx = context and context.copy() or {}
            ctx.update(active_test=False)
            product_ids = []
            for product in self.browse(cr, uid, ids, context=ctx):
                product_ids = map(int,product.product_variant_ids)
            self.pool.get("product.product").write(cr, uid, product_ids, {'active': vals.get('active')}, context=ctx)
        
        if obj.is_asset == True:
            product = self.pool.get('product.product')
            pro_obj = product.browse(cr, uid, product.search(cr, uid, [('product_tmpl_id','=',obj.id)]))
            info=self.pool.get('sales.information')
            information = info.browse(cr, uid, info.search(cr, uid, [('product_id','=', pro_obj.id)]))
            building = self.pool.get('building.building')
            bui_obj = building.browse(cr, uid, building.search(cr, uid, [('id','=',obj.building_id.id)]))
            #------------------- print 'reeeeet2', bui_obj, information, pro_obj
            #--------------------------------------- information.write(cr, uid,{
                                        #------------- 'product_id': pro_obj.id,
                                        # 'to_commission_season': bui_obj.to_commission_season.id,
                                        # 'windows_location': [(6, 0, 1)] or False,
                                        #---------- 'floor': obj.floor or False,
                                        #---- 'entrance': obj.entrance or False,
                                        #---- 'building_id': obj.building_id.id,
                                        #-- 'project_id': bui_obj.project_id.id,
                                        # 'total_squaring': obj.total_squaring or False,
                                        #------------------------------------ })
        res = super(product_template, self).write(cr, uid, ids, vals, context=context)
        return res


class windows_location(osv.osv):
    _name = "windows.location"
    _description = "Windows location"
    
    _columns = {
                'name': fields.char('Location name'),
                'door_id':fields.many2one('product.template','Door', invisible=True, readonly = True),
                'producWt_door_id':fields.many2one('product.template','Door', invisible=True, readonly = True),
                'information_id':fields.many2one('product.template','Door', invisible=True, readonly = True)      
                }  
    
class door_type(osv.osv):
    _name="door.type"
    _description = "Door type ABC"
    
    _columns = {
                'name': fields.char('Door type')
                }
    
class sale_account_invoice_inherit(osv.Model):
    _inherit="account.invoice"
    _columns = {
                'sale_door_id': fields.many2one('product.template','Door number')
                }

class product_product(osv.osv):
    _inherit = "product.product"
    _description = "Product product"
    
    _columns = {
                'is_asset': fields.boolean('Is asset'),
                'building_id':fields.many2one('building.building','Building'),
                'asset_type': fields.selection([
                    ('apartment', 'Apartment'),
                    ('garage', 'Garage'),
                    ('object', 'Object')
                    ], 'Asset type'),
                'room_size':fields.integer('Room size'),
                'total_squaring':fields.float('Total squaring'),
                'houseroom':fields.float('Houseroom'),
                'entrance':fields.char('Entrance'),
                'floor':fields.char('Floor'),
                'atso':fields.integer('ATsO number'),
                'door_type':fields.many2one('door.type','Door type'),
                'windows_location':fields.many2many('windows.location', 'product_windows_location_rel','product_door_id','windows_location', 'Windows location'),
                'new_code':fields.char( 'Product code', select=True),
                #'new_code':fields.char('Product code', size=8,required=True),
                'retail_price':fields.integer('Retail price'),
                'disbursing_price':fields.integer('Disbursing price'),
                'size':fields.char('Size'),
                'color_shape':fields.char('Color, Shape'),
                'others':fields.char('Others'),
                'collect_income_day':fields.integer('Collection income day'),
                }
    #
    def name_get(self, cr, uid, ids, context=None):
        if isinstance(ids, (list, tuple)) and not len(ids):
            return []
        if isinstance(ids, (long, int)):
            ids = [ids]
        reads = self.read(cr, uid, ids, ['name','new_code'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['new_code']:
                name = name +' / '+ record['new_code']
            res.append((record['id'], name))
        return res
    
    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=100):
        if args is None:
            args = []
        if context is None:
            context = {}
        ids = []
        if name:
            ids = self.search(cr, uid, [('name', 'ilike', name)] + args, limit=limit)
        if not ids:
            ids = self.search(cr, uid, [('new_code', 'ilike', name)] + args, limit=limit)
            #ids = self.search(cr, uid, [('new_code', 'like', name), ('name', operator, name)]+args, limit=limit)
                
        return self.name_get(cr, uid, ids, context=context)