# -*- coding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2012 Agile Business Group sagl (<http://www.agilebg.com>)
#    Copyright (C) 2012 Domsense srl (<http://www.domsense.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
import openerp.addons.decimal_precision as dp
from openerp.tools.translate import _
from dateutil.relativedelta import relativedelta
import time
from datetime import timedelta, datetime
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT, DEFAULT_SERVER_DATE_FORMAT


class product_research(osv.osv):
    _name = 'product.research'
    _columns = {
                'analyse_type':fields.selection([('old','Old'),
                                                 ('new','New')],'Analyse type',required=True),
                'name':fields.char('Name'),
                'description':fields.char('Description'),
                #'building_id':fields.many2one('building.building','Building'),
                'project_id':fields.many2one('project.project','Project',required=True),
                'when_use':fields.date('When use',required=True),
                'order_date':fields.date('Order date',readonly=True,required=True),
                'employee_id':fields.many2one('hr.employee','Hr employee',required=True),
                'job_id':fields.many2one('hr.job','Job'),
                
                'product_video':fields.binary('Product video'),
                'company_id':fields.many2one('res.company','Company'),
                
                'state':fields.selection([('new','New'),
                                          ('learning','Learning'),
                                          ('done','Done')],'State'),
                'product_id':fields.many2one('product.product','Product name'),
                'line_id':fields.one2many('product.research.line','parent_id','Product analyse line'),
                'new_line_id':fields.one2many('product.research.new.line','parent_id','Product analyse line'),
                }
    _defaults = { 
                 'analyse_type':'new',
                 'order_date':fields.date.context_today,
                 'when_use':fields.date.context_today,
                'state':'new',
                'company_id': lambda s,cr,uid,c: s.pool.get('res.company')._company_default_get(cr, uid, 'product.analyse', context=c),
                 }
    
    def action_learning(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state':'learning'}, context)
        return True
    def action_done(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state':'done'}, context)
        return True
    def onchange_employee_id(self, cr, uid, ids, employee_id, context=None):
        res = {}
        if employee_id != None:
            emp_obj = self.pool.get('hr.employee')
            emp_id = emp_obj.search(cr, uid, [('id','=',employee_id)])
            emp = emp_obj.browse(cr, uid, emp_id)
            res['job_id']=emp.job_id
            return {'value':res}

class product_research_line(osv.osv):
    _name = 'product.research.line'
    
    
    def _get_total_price(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = record.unit_price+record.landed_cost_price
        return res
        
    _columns = {
                'parent_id':fields.many2one('product.research','Product analyse'),
                'product_index':fields.char('Product index'),
                'product_pic':fields.binary('Product picture'),
                'partner_id':fields.many2one('res.partner','Partner'),
                'unit_price':fields.float('Unit price'),
                'product_id':fields.many2one('product.product','Product name',required=True),
                'total_price':fields.function(_get_total_price, type='float',string='Total price',store=True,readonly=True),
                'landed_cost_price':fields.float('Landed cost price'),
                'uom_id':fields.many2one('product.uom','Product uom'),
                'is_selected':fields.boolean('Is selected'),
                'address':fields.char('Address'),
                'state':fields.selection([('draft','Draft'),
                                          ('pur_order','Purchase order'),
                                          ('done','Done')],'State',readonly=True),
                'company_id':fields.many2one('res.company','Company'),
                }
    _defaults = {

                 'state':'draft',
                 }
    
    def purchase_create(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        purchase_obj = self.pool.get('purchase.order')
        purchase_line_obj = self.pool.get('purchase.order.line')
        product_obj = self.pool.get('product.product')
        type_obj = self.pool.get('stock.picking.type')
        user_obj = self.pool.get('res.users')
        location_obj = self.pool.get('stock.location')
        company_id = user_obj.browse(cr, uid, uid, context=context).company_id.id
        location = location_obj.search(cr, uid, [('company_id','=',company_id),('usage','=','internal')])

        types = type_obj.search(cr, uid, [('code', '=', 'incoming'), ('warehouse_id.company_id', '=', company_id)], context=context)
        if not types:
            types = type_obj.search(cr, uid, [('code', '=', 'incoming'), ('warehouse_id', '=', False)], context=context)
        
        po_vals = {
                        'name':'/',
                        'partner_id': obj.partner_id.id,
                        'location_id': location[0],
                        'picking_type_id': types[0],
                        'pricelist_id': obj.partner_id.property_product_pricelist_purchase.id,
                        'currency_id': obj.partner_id.property_product_pricelist_purchase and obj.partner_id.property_product_pricelist_purchase.currency_id.id or obj.company_id.currency_id.id,
                        'company_id': company_id,
                        'fiscal_position': obj.partner_id.property_account_position and obj.partner_id.property_account_position.id or False,
                        'payment_term_id': obj.partner_id.property_supplier_payment_term.id or False,
                        'dest_address_id': obj.partner_id.id,
                    }
        if obj.is_selected ==True:
            order_id = purchase_obj.create(cr, uid, po_vals,context=None)
            purchase_line_obj.create(cr, uid, {
                                               'order_id':order_id,
                                               'product_id':obj.product_id.id,
                                               'price_unit':obj.unit_price,
                                               #''obj.,
                                               'name':'',
                                               'date_planned':time.strftime(DEFAULT_SERVER_DATETIME_FORMAT),
                                               })
        else:
            raise osv.except_osv(_('Error!'),_("Та энэ барааг сонгоогүй байгаа тул худалдан авалтын захиалга үүсэхгүй!"))
        self.write(cr, uid, ids,{'state':'pur_order',
                                 })
        
        return True
    
    
    
class product_research_new_line(osv.osv):
    _name = 'product.research.new.line'
    
    
    def _get_total_price(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = record.unit_price+record.landed_cost_price
        return res
        
    _columns = {
                'parent_id':fields.many2one('product.research','Product analyse'),
                'product_index':fields.char('Product index'),
                'name':fields.char('Product name',required=True),
                'product_pic':fields.binary('Product picture'),
                'partner_id':fields.many2one('res.partner','Partner'),
                'unit_price':fields.float('Unit price'),
                'total_price':fields.function(_get_total_price, type='float',string='Total price',store=True,readonly=True),
                'landed_cost_price':fields.float('Landed cost price'),
                'uom_id':fields.many2one('product.uom','Product uom'),
                'is_selected':fields.boolean('Is selected'),
                'address':fields.char('Address'),
                'state':fields.selection([('draft','Draft'),
                                          ('pur_order','Purchase order'),
                                          ('done','Done')],'State',readonly=True),
                'company_id':fields.many2one('res.company','Company'),
                }
    _defaults = {

                 'state':'draft',
                 }
    
    def purchase_create(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        purchase_obj = self.pool.get('purchase.order')
        purchase_line_obj = self.pool.get('purchase.order.line')
        product_obj = self.pool.get('product.product')
        type_obj = self.pool.get('stock.picking.type')
        user_obj = self.pool.get('res.users')
        location_obj = self.pool.get('stock.location')
        company_id = user_obj.browse(cr, uid, uid, context=context).company_id.id
        location = location_obj.search(cr, uid, [('company_id','=',company_id),('usage','=','internal')])

        types = type_obj.search(cr, uid, [('code', '=', 'incoming'), ('warehouse_id.company_id', '=', company_id)], context=context)
        if not types:
            types = type_obj.search(cr, uid, [('code', '=', 'incoming'), ('warehouse_id', '=', False)], context=context)
        pro_id = product_obj.create(cr, uid, {
                                     'name':obj.name,
                                     })
        
        po_vals = {
                        'name':'/',
                        'partner_id': obj.partner_id.id,
                        'location_id': location[0],
                        'picking_type_id': types[0],
                        'pricelist_id': obj.partner_id.property_product_pricelist_purchase.id,
                        'currency_id': obj.partner_id.property_product_pricelist_purchase and obj.partner_id.property_product_pricelist_purchase.currency_id.id or obj.company_id.currency_id.id,
                        'company_id': company_id,
                        'fiscal_position': obj.partner_id.property_account_position and obj.partner_id.property_account_position.id or False,
                        'payment_term_id': obj.partner_id.property_supplier_payment_term.id or False,
                        'dest_address_id': obj.partner_id.id,
                    }
        if obj.is_selected ==True:
            order_id = purchase_obj.create(cr, uid, po_vals,context=None)
            purchase_line_obj.create(cr, uid, {
                                               'order_id':order_id,
                                               'product_id':pro_id,
                                               'price_unit':obj.unit_price,
                                               #''obj.,
                                               'name':'',
                                               'date_planned':time.strftime(DEFAULT_SERVER_DATETIME_FORMAT),
                                               })
        else:
            raise osv.except_osv(_('Error!'),_("Та энэ барааг сонгоогүй байгаа тул худалдан авалтын захиалга үүсэхгүй!"))
        self.write(cr, uid, ids,{'state':'pur_order',
                                 })
        
        return True