# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2014 iERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : iERP@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

from datetime import datetime
from openerp.osv import osv
from openerp.osv import fields
from openerp.tools import translate
from openerp import netsvc
import time
import datetime
from datetime import date, timedelta
from lxml import etree
from operator import itemgetter
import openerp.addons.decimal_precision as dp
from openerp.fields import Many2one
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, DATETIME_FORMATS_MAP


class stock_picking_inherit(osv.osv):
    _inherit = 'stock.picking'
    
    _columns = {
                'track_id':fields.one2many('stock.tracking','picking_id','Stock tracking'),

                'contract_number':fields.many2one('project.project','Contract number'),
                'project_task_id':fields.many2one('project.task.type','Project task type'),
                'build_id':fields.many2one('building.building','Build number'),
                }
    
    def _prepare_pack_ops(self, cr, uid, picking, quants, forced_qties, context=None):
        """ returns a list of dict, ready to be used in create() of stock.pack.operation.

        :param picking: browse record (stock.picking)
        :param quants: browse record list (stock.quant). List of quants associated to the picking
        :param forced_qties: dictionary showing for each product (keys) its corresponding quantity (value) that is not covered by the quants associated to the picking
        """
        def _picking_putaway_apply(product):
            location = False
            # Search putaway strategy
            if product_putaway_strats.get(product.id):
                location = product_putaway_strats[product.id]
            else:
                location = self.pool.get('stock.location').get_putaway_strategy(cr, uid, picking.location_dest_id, product, context=context)
                product_putaway_strats[product.id] = location
            return location or picking.location_dest_id.id

        pack_obj = self.pool.get("stock.quant.package")
        quant_obj = self.pool.get("stock.quant")
        vals = []
        qtys_grouped = {}
        #for each quant of the picking, find the suggested location
        quants_suggested_locations = {}
        product_putaway_strats = {}
        for quant in quants:
            if quant.qty <= 0:
                continue
            suggested_location_id = _picking_putaway_apply(quant.product_id)
            quants_suggested_locations[quant] = suggested_location_id

        #find the packages we can movei as a whole
        top_lvl_packages = self._get_top_level_packages(cr, uid, quants_suggested_locations, context=context)
        # and then create pack operations for the top-level packages found
        for pack in top_lvl_packages:
            pack_quant_ids = pack_obj.get_content(cr, uid, [pack.id], context=context)
            pack_quants = quant_obj.browse(cr, uid, pack_quant_ids, context=context)
            vals.append({
                    'picking_id': picking.id,
                    'package_id': pack.id,
                    'product_qty': 1.0,
                    'location_id': pack.location_id.id,
                    'location_dest_id': quants_suggested_locations[pack_quants[0]],
                })
            #remove the quants inside the package so that they are excluded from the rest of the computation
            for quant in pack_quants:
                del quants_suggested_locations[quant]

        # Go through all remaining reserved quants and group by product, package, lot, owner, source location and dest location
        for quant, dest_location_id in quants_suggested_locations.items():
            key = (quant.product_id.id, quant.package_id.id, quant.lot_id.id, quant.owner_id.id, quant.location_id.id, dest_location_id)
            if qtys_grouped.get(key):
                qtys_grouped[key] += quant.qty
            else:
                qtys_grouped[key] = quant.qty

        # Do the same for the forced quantities (in cases of force_assign or incomming shipment for example)
        for product, qty in forced_qties.items():
            if qty <= 0:
                continue
            suggested_location_id = _picking_putaway_apply(product)
            key = (product.id, False, False, False, picking.location_id.id, suggested_location_id)
            if qtys_grouped.get(key):
                qtys_grouped[key] += qty
            else:
                qtys_grouped[key] = qty

        # Create the necessary operations for the grouped quants and remaining qtys
        for key, qty in qtys_grouped.items():
            vals.append({
                'picking_id': picking.id,
                'contract_number':picking.contract_number.id,
                'product_qty': qty,
                'product_id': key[0],
                'package_id': key[1],
                'lot_id': key[2],
                'owner_id': key[3],
                'location_id': key[4],
                'location_dest_id': key[5],
                'product_uom_id': self.pool.get("product.product").browse(cr, uid, key[0], context=context).uom_id.id,
            })
        return vals
    
class stock_move(osv.osv):
    _inherit = 'stock.move'
    
    _columns = {
                'project_id':fields.many2one('project.project','Project'),
                'collection_id':fields.many2one('collection.integrate','Collection'),
                'product_code':fields.char('Product code',size=8),
                'cost_price': fields.float('Cost Price'),
                'total_cost': fields.float('Total cost'),
                'sale_price': fields.float('Sale price'),
                'total_sale_cost': fields.float('Total sale cost'),
                }
    
    def action_done(self, cr, uid, ids, context=None):
        """ Process completely the moves given as ids and if all moves are done, it will finish the picking.
        """
        context = context or {}
        picking_obj = self.pool.get("stock.picking")
        quant_obj = self.pool.get("stock.quant")
        todo = [move.id for move in self.browse(cr, uid, ids, context=context) if move.state == "draft"]
        if todo:
            ids = self.action_confirm(cr, uid, todo, context=context)
        pickings = set()
        procurement_ids = []
        #Search operations that are linked to the moves
        operations = set()
        move_qty = {}
        for move in self.browse(cr, uid, ids, context=context):
            move_qty[move.id] = move.product_qty
            for link in move.linked_move_operation_ids:
                operations.add(link.operation_id)

        #Sort operations according to entire packages first, then package + lot, package only, lot only
        operations = list(operations)
        operations.sort(key=lambda x: ((x.package_id and not x.product_id) and -4 or 0) + (x.package_id and -2 or 0) + (x.lot_id and -1 or 0))

        for ops in operations:
            if ops.picking_id:
                pickings.add(ops.picking_id.id)
            main_domain = [('qty', '>', 0)]
            for record in ops.linked_move_operation_ids:
                move = record.move_id
                self.check_tracking(cr, uid, move, not ops.product_id and ops.package_id.id or ops.lot_id.id, context=context)
                prefered_domain = [('reservation_id', '=', move.id)]
                fallback_domain = [('reservation_id', '=', False)]
                fallback_domain2 = ['&', ('reservation_id', '!=', move.id), ('reservation_id', '!=', False)]
                prefered_domain_list = [prefered_domain] + [fallback_domain] + [fallback_domain2]
                dom = main_domain + self.pool.get('stock.move.operation.link').get_specific_domain(cr, uid, record, context=context)
                quants = quant_obj.quants_get_prefered_domain(cr, uid, ops.location_id, move.product_id, record.qty, domain=dom, prefered_domain_list=prefered_domain_list,
                                                          restrict_lot_id=move.restrict_lot_id.id, restrict_partner_id=move.restrict_partner_id.id, context=context)
                if ops.result_package_id.id:
                    #if a result package is given, all quants go there
                    quant_dest_package_id = ops.result_package_id.id
                elif ops.product_id and ops.package_id:
                    #if a package and a product is given, we will remove quants from the pack.
                    quant_dest_package_id = False
                else:
                    #otherwise we keep the current pack of the quant, which may mean None
                    quant_dest_package_id = ops.package_id.id
                quant_obj.quants_move(cr, uid, quants, move, ops.location_dest_id, location_from=ops.location_id, lot_id=ops.lot_id.id, owner_id=ops.owner_id.id, src_package_id=ops.package_id.id, dest_package_id=quant_dest_package_id, context=context)
                # Handle pack in pack
                if not ops.product_id and ops.package_id and ops.result_package_id.id != ops.package_id.parent_id.id:
                    self.pool.get('stock.quant.package').write(cr, SUPERUSER_ID, [ops.package_id.id], {'parent_id': ops.result_package_id.id}, context=context)
                #===============================================================
                # print 'zaaaaaaaaaaaa', move_qty
                # 
                # hr_employee = self.pool.get('stock.move')
                # employee = hr_employee.search(cr, uid, [('id','=',move.id)])
                # emp = hr_employee.browse(cr, uid, employee)
                # 
                # #move_tbtb = self.pool.get('stock.move').browse(cr, uid, [('id', '=', move.id)])
                # print 'dddddddddddddd',emp, emp.name, move_qty[move.id],
                # print 'ttttttttttttttt', record.qty
                #===============================================================
                move_qty[move.id] -= record.qty
        #Check for remaining qtys and unreserve/check move_dest_id in
        move_dest_ids = set()
        for move in self.browse(cr, uid, ids, context=context):
            if move_qty[move.id] > 0:  # (=In case no pack operations in picking)
                main_domain = [('qty', '>', 0)]
                prefered_domain = [('reservation_id', '=', move.id)]
                fallback_domain = [('reservation_id', '=', False)]
                fallback_domain2 = ['&', ('reservation_id', '!=', move.id), ('reservation_id', '!=', False)]
                prefered_domain_list = [prefered_domain] + [fallback_domain] + [fallback_domain2]
                self.check_tracking(cr, uid, move, move.restrict_lot_id.id, context=context)
                qty = move_qty[move.id]
                quants = quant_obj.quants_get_prefered_domain(cr, uid, move.location_id, move.product_id, qty, domain=main_domain, prefered_domain_list=prefered_domain_list, restrict_lot_id=move.restrict_lot_id.id, restrict_partner_id=move.restrict_partner_id.id, context=context)
                quant_obj.quants_move(cr, uid, quants, move, move.location_dest_id, lot_id=move.restrict_lot_id.id, owner_id=move.restrict_partner_id.id, context=context)

            # If the move has a destination, add it to the list to reserve
            if move.move_dest_id and move.move_dest_id.state in ('waiting', 'confirmed'):
                move_dest_ids.add(move.move_dest_id.id)

            if move.procurement_id:
                procurement_ids.append(move.procurement_id.id)

            #unreserve the quants and make them available for other operations/moves
            quant_obj.quants_unreserve(cr, uid, move, context=context)
        # Check the packages have been placed in the correct locations
        self._check_package_from_moves(cr, uid, ids, context=context)
        #set the move as done
        self.write(cr, uid, ids, {'state': 'done', 'date': time.strftime(DEFAULT_SERVER_DATETIME_FORMAT)}, context=context)
        self.pool.get('procurement.order').check(cr, uid, procurement_ids, context=context)
        #assign destination moves
        if move_dest_ids:
            self.action_assign(cr, uid, list(move_dest_ids), context=context)
        #check picking state to set the date_done is needed
        done_picking = []
        for picking in picking_obj.browse(cr, uid, list(pickings), context=context):
            if picking.state == 'done' and not picking.date_done:
                done_picking.append(picking.id)
        if done_picking:
            picking_obj.write(cr, uid, done_picking, {'date_done': time.strftime(DEFAULT_SERVER_DATETIME_FORMAT)}, context=context)
        return True
    
class stock_picking_wave(osv.osv):
    _inherit = 'stock.picking.wave'
    
    _columns = {
                'description':fields.char('Clearance limits'),
                'track_id':fields.one2many('stock.tracking','picking_wave_id','Stock tracking'),
                }
    
    def confirm_picking(self, cr, uid, ids, context=None):
        picking_todo = self.pool.get('stock.picking').search(cr, uid, [('wave_id', 'in', ids)], context=context)
        self.write(cr, uid, ids, {'state': 'in_progress'}, context=context)
        track_id = self.pool.get('stock.tracking').search(cr, uid, [('picking_wave_id','in',ids)])
        self.pool.get('stock.tracking').write(cr, uid, track_id,{
                                                                    'picking_id':picking_todo,
                                                                    })
        return self.pool.get('stock.picking').action_assign(cr, uid, picking_todo, context=context)
        
class stock_tracking(osv.osv):
    _name = 'stock.tracking'
    
    _columns = {
                'picking_id':fields.many2one('stock.picking','Stock picking'),
                'price':fields.integer('Price'),
                'out_location_id':fields.many2one('transport.location','Out of location'),
                'in_location_id':fields.many2one('transport.location','In of location'),
                'out_date':fields.date('Out date'),
                'in_date':fields.date('In date'),
                'transport_type':fields.selection([('rail_transport','Rail transport'),
                                                   ('carrier','Carrier'),
                                                   ('air_freight','Air freight'),
                                                   ('sea_freight','Sea freight')],'Transport type'),
                'state':fields.selection([('in','In'),
                                          ('out','Out')],'State'),
                'picking_wave_id':fields.many2one('stock.picking.wave','Stock picking'),

                }
class stock_pack_operation(osv.osv): 
    _inherit = "stock.pack.operation"
    
    _columns = {
                'contract_number':fields.many2one('project.project','Contract number'),
                'project_task_id':fields.many2one('project.task.type','Project task type'),
                'build_id':fields.many2one('building.building','Build number'),
                }
class product_loss_config(osv.osv):
    _name = 'product.loss.config'
    
    _columns = {
                'name':fields.char('Name'),
                'date':fields.date('Date'),
                'percent':fields.float('Percent'),
                'company_id':fields.many2one('res.company','Company'),
                'create_user':fields.many2one('res.users','Create user'),
                'line_id':fields.one2many('product.loss.config.line','config_id','Line id')
                }
    

class product_loss_config_line(osv.osv):
    _name = 'product.loss.config.line'
    
    _columns = {
                'name':fields.date('Start date'),
                'end_date':fields.date('End date'),
                'company_id':fields.many2one('res.company','Company'),
                'employee_id':fields.many2one('hr.employee','Employee'),
                'product_id':fields.many2one('product.product','Product'),
                'location_id':fields.many2one('stock.location','Location'),
                'config_id':fields.many2one('product.loss.config','Product config'),
                }

    