# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

from c2c_reporting_tools.reports.standard_report import *
from c2c_reporting_tools.flowables.simple_row_table import *
from c2c_reporting_tools.c2c_helper import *
from c2c_reporting_tools.translation import _
from reportlab.platypus import *
from reportlab.lib.colors import red, black, navy, white, green
from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY

from datetime import datetime
from dateutil.relativedelta import relativedelta
import time
from operator import itemgetter
from xlwt import Workbook, Formula


class account_profit_report(StandardReport):
    ''' Орлогын тайлан
    '''
    def get_template_title(self, cr, context):
        """ return the title of the report """
        return u"ОРЛОГО ҮР ДҮНГИЙН ТАЙЛАН"
    
    def get_report_barcode(self, cr, context=None):
        ''' Баримтын кодыг зураасан код болгож зурах '''
        if context is None:
            context = {}
        uid = 1
        if not self.datas['form']['save']:
            self.code = '-----'
            return False
        
        tmp, sequence_id = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'l10n_mn_base','seq_account_report')
        #self.directory_name = 'B10101'
        sequence = self.pool.get('ir.sequence').get_id(cr, uid, sequence_id, code_or_id='id', context=context)
        if not sequence :
            self.code = '-----'
            return False
        self.code = sequence
        self.attache_name = sequence
        self.force_save = True
        return sequence
    
    def create_report_xls_data(self, cr, uid, data, context=None):
        ''' Орлогын тайлангийн мэдээллийг боловсруулж
            тайлангийн форматад тохируулан python [{},{},{}...]
            загвараар хүснэгтийн мөр багануудын өгөгдлийг боловсруулна.
            
            @return: income - Орлого талын дансны өгөгдлүүд 
                     expense - Үндсэн үйл ажиллагааны зардал талын дансны өгөгдлүүд
                     income2 - Үндсэн бус үйл ажиллагааны зардал талын дансны өгөгдлүүд
        '''
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        currency_obj = self.pool.get('res.currency')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        type_obj = self.pool.get('account.account.type')
        if not context:
            context = {}
        
        fy = fiscalyear_obj.browse(cr, uid, data['fiscalyear_id'], context=context)
        ctx = context.copy()
        ctx['fiscalyear'] = data.get('fiscalyear_id', False)
        ctx['date_from'] = fy.date_start
        ctx['date_to'] =  data.get('date_to', time.strftime('%Y-%m-%d'))
        ctx['state'] = data.get('target_move', 'all')
        ctx['return_initial_bal_journal'] = True
        
        def _get_amount_by_type(account_report_type):
            ''' Тухайн санхүүгийн байдлын тайлангийн дансны төрөлд хийгдсэн тайлант хугацааны
             болон өнгөрсөн жилийн мөн үеийн гүйлгээний дүнг харуулна.
            '''
            bef_init_balance=0
            type_ids = conf_obj.search(cr, uid, [('name','=','income')])
            conf_id  = account_config_report_obj.search(cr, uid, [('report_id','=',type_ids),('type','=',account_report_type)])
            res = {}
            start_amount = 0
            end_amount = 0
            if type_ids:
                for x in account_config_report_obj.browse(cr, uid, conf_id, context=context):
                    for acc in x.account_id:
                        config_end_line = account_obj.browse(cr, uid,[acc.id],ctx)
                        for e_line in config_end_line:
                            start_amount = e_line['balance']
                    res[x.id] = {
                        'id': x.id,
                        'name': x.name,
                        'start_amount':start_amount,
                        'sequence': x.sequence or 1
                    }
                        
            return sorted(res.values(), key=itemgetter("sequence"))
        
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        account_rel = self.pool.get('account.config.rel')
        currency_obj = self.pool.get('res.currency')
        company_obj = self.pool.get('res.company')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        account_move_line_obj = self.pool.get('account.move.line')
        account_config_report_obj = self.pool.get('account.config')
        
        conf_line_obj = self.pool.get('account.cashflow.report.configure.line')
        conf_obj = self.pool.get('account.report.configure')
        
        
        move_state = 'all'
        if data['target_move'] == 'posted':
            move_state = 'posted'
                
        grid = []
        sublines = []
        sale_income = 0
        for line in _get_amount_by_type('sale_income') :
            sublines.append([line['sequence'], line['name'], line['start_amount'], '<font color="white">_______</font>'])
        grid.extend(sublines)
        
        sale_cost = 0
        gain_and_loss = 0
        sublines = []
        for line in _get_amount_by_type('sale_cost'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
        grid.extend(sublines)
        grid.append(['3', u'Нийт ашиг ( алдагдал)', Formula('C8-C9'), '<font color="white">___</font>'])

      
        income = 0
        sublines = []
        for line in _get_amount_by_type('income'):
            sublines.append([line['sequence'], line['name'],line['start_amount'], '<font color="white">_______</font>'])
            income += line['start_amount']
        grid.extend(sublines)
            
        cost = 0
        tax_gain_loss = 0
        sublines = []
        for line in _get_amount_by_type('cost'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
            cost += line['start_amount']
        grid.extend(sublines)
        tax_gain_loss = gain_and_loss+income+cost
        grid.append(['18', u'<b>Татвар төлөхийн өмнөх ашиг ( алдагдал)</b>', Formula('SUM(C11:C15)-SUM(C16:C24)'),'<font color="white">___</font>'])
        
        owner_property_total = 0
        total = 0
        tax_to_next = 0
        income_tax_loss = 0
        sublines = []
        for line in _get_amount_by_type('tax_cost'):
            sublines.append([line['sequence'], line['name'],Formula('C25/10'),  '<font color="white">_______</font>'])
            owner_property_total += line['start_amount']
        grid.extend(sublines)
        tax_to_next = tax_gain_loss-income_tax_loss
        grid.append(['20', u'<b>Татварын дараах ашиг (алдагдал)</b>', Formula('C25-C26'), ''])
        grid.append(['21', u'<b>Зогсоосон үйл ажиллагааны татварын дараах ашиг (алдагдал)</b>', tax_to_next, '<font color="white">___</font>'])
        grid.append(['22', u'<b>Тайлант үеийн цэвэр ашиг ( алдагдал)</b>', Formula('C27-C28'), '<font color="white">___</font>'])
        grid.append(['23', u'<b>Бусад дэлгэрэнгүй орлого</b>','', '', ''])
        
        other_income = 0
        total = 0
        sublines = []
        for line in _get_amount_by_type('other_income'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
        grid.extend(sublines)
        grid.append(['24', u'<b>Орлогын нийт дүн</b>',Formula('SUM(C31:C33)'), ''])
        grid.append(['25', u'<b>Нэгж хувьцаанд ногдох суурь ашиг (алдагдал)</b>','', ''])
        
        return grid
    

    def create_report_pdf_data(self, cr, uid, data, context=None):
        ''' Орлогын тайлангийн мэдээллийг боловсруулж
            тайлангийн форматад тохируулан python [{},{},{}...]
            загвараар хүснэгтийн мөр багануудын өгөгдлийг боловсруулна.
            
            @return: income - Орлого талын дансны өгөгдлүүд 
                     expense - Үндсэн үйл ажиллагааны зардал талын дансны өгөгдлүүд
                     income2 - Үндсэн бус үйл ажиллагааны зардал талын дансны өгөгдлүүд
        '''
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        currency_obj = self.pool.get('res.currency')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        type_obj = self.pool.get('account.account.type')
        if not context:
            context = {}
        
        fy = fiscalyear_obj.browse(cr, uid, data['fiscalyear_id'], context=context)
        ctx = context.copy()
        ctx['fiscalyear'] = data.get('fiscalyear_id', False)
        ctx['date_from'] = fy.date_start
        ctx['date_to'] =  data.get('date_to', time.strftime('%Y-%m-%d'))
        ctx['state'] = data.get('target_move', 'all')
        ctx['return_initial_bal_journal'] = True
        
        def _get_amount_by_type(account_report_type):
            ''' Тухайн санхүүгийн байдлын тайлангийн дансны төрөлд хийгдсэн тайлант хугацааны
             болон өнгөрсөн жилийн мөн үеийн гүйлгээний дүнг харуулна.
            '''
            bef_init_balance=0
            type_ids = conf_obj.search(cr, uid, [('name','=','income')])
            conf_id  = account_config_report_obj.search(cr, uid, [('report_id','=',type_ids),('type','=',account_report_type)])
            res = {}
            start_amount = 0
            end_amount = 0
            if type_ids:
                for x in account_config_report_obj.browse(cr, uid, conf_id, context=context):
                    for acc in x.account_id:
                        config_end_line = account_obj.browse(cr, uid,[acc.id],ctx)
                        for e_line in config_end_line:
                            start_amount = e_line['balance']
                    res[x.id] = {
                        'id': x.id,
                        'name': x.name,
                        'start_amount':start_amount,
                        'sequence': x.sequence or 1
                    }
                        
            return sorted(res.values(), key=itemgetter("sequence"))
        
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        account_rel = self.pool.get('account.config.rel')
        currency_obj = self.pool.get('res.currency')
        company_obj = self.pool.get('res.company')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        account_move_line_obj = self.pool.get('account.move.line')
        account_config_report_obj = self.pool.get('account.config')
        
        conf_line_obj = self.pool.get('account.cashflow.report.configure.line')
        conf_obj = self.pool.get('account.report.configure')
        
        
        move_state = 'all'
        if data['target_move'] == 'posted':
            move_state = 'posted'
                
        grid = []
        sublines = []
        sale_income = 0
        for line in _get_amount_by_type('sale_income') :
            sublines.append([line['sequence'], line['name'], line['start_amount'], '<font color="white">_______</font>'])
            sale_income += line['start_amount']
        grid.extend(sublines)
        
        sale_cost = 0
        gain_and_loss = 0
        sublines = []
        for line in _get_amount_by_type('sale_cost'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
            sale_cost += line['start_amount']
        grid.extend(sublines)
        gain_and_loss = sale_income-sale_cost
        grid.append(['3', u'Нийт ашиг ( алдагдал)', gain_and_loss, '<font color="white">___</font>'])

      
        income = 0
        sublines = []
        for line in _get_amount_by_type('income'):
            sublines.append([line['sequence'], line['name'],line['start_amount'], '<font color="white">_______</font>'])
            income += line['start_amount']
        grid.extend(sublines)
            
        cost = 0
        tax_gain_loss = 0
        sublines = []
        for line in _get_amount_by_type('cost'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
            cost += line['start_amount']
        grid.extend(sublines)
        tax_gain_loss = gain_and_loss+income+cost
        grid.append(['18', u'<b>Татвар төлөхийн өмнөх ашиг ( алдагдал)</b>', tax_gain_loss,'<font color="white">___</font>'])
        
        owner_property_total = 0
        total = 0
        tax_to_next = 0
        income_tax_loss = 0
        sublines = []
        income_tax_loss = tax_gain_loss/10
        for line in _get_amount_by_type('tax_cost'):
            sublines.append([line['sequence'], line['name'],income_tax_loss,  '<font color="white">_______</font>'])
            owner_property_total += line['start_amount']
        grid.extend(sublines)
        tax_to_next = tax_gain_loss-income_tax_loss
        grid.append(['20', u'<b>Татварын дараах ашиг (алдагдал)</b>', '', ''])
        grid.append(['21', u'<b>Зогсоосон үйл ажиллагааны татварын дараах ашиг (алдагдал)</b>', tax_to_next, '<font color="white">___</font>'])
        grid.append(['22', u'<b>Тайлант үеийн цэвэр ашиг ( алдагдал)</b>', tax_to_next, '<font color="white">___</font>'])
        grid.append(['23', u'<b>Бусад дэлгэрэнгүй орлого</b>','', '', ''])
        
        other_income = 0
        total = 0
        sublines = []
        for line in _get_amount_by_type('other_income'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
            other_income += line['start_amount']
        grid.extend(sublines)
        grid.append(['24', u'<b>Орлогын нийт дүн</b>',other_income, ''])
        grid.append(['25', u'<b>Нэгж хувьцаанд ногдох суурь ашиг (алдагдал)</b>','', ''])
        
        return grid

    
    def get_story(self):
        ''' Тайлангийн өгөгдлүүдийг боловсруулж, PDF -г зурна.
            A4 цаасний стандарт хэмжээ 210x297 үүний тайлангийн бие нь
            Хэвтээ хэлбэрээр 280x200 байхаар тогтоолоо.
            
        '''
        story = []
        cr = self.cr
        uid = self.uid 
        context = self.context
        
        data = self.datas['form']
        period_obj = self.pool.get('account.period')
        account_obj = self.pool.get('account.account')
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        )
        
        styNormal = ParagraphStyle('normal', fontName='Helvetica', fontSize=8, leading=13,
                   textColor=navy, alignment=TA_LEFT, leftIndent=8, spaceAfter=10)
        styNormalR = ParagraphStyle('normal', fontName='Helvetica', fontSize=8, leading=13,
                   textColor=navy, alignment=TA_RIGHT, leftIndent=8, spaceAfter=10)
        title_style = ParagraphStyle('tabledataleft', fontName='Helvetica-Bold', fontSize=12, leading=17, 
                    alignment=TA_CENTER, leftIndent=1, rightIndent=1, spaceAfter=10, spaceBefore=10)
        nullpara = Paragraph('', styNormal)
        
        grid = [[
            Paragraph(u"", styNormal),
            Paragraph(u"Байгууллагын нэр: %s" % company.name, styNormalR)
        ],[
            Paragraph(u"САНХҮҮГИЙН БАЙДЛЫН ТАЙЛАН", title_style),
            nullpara
        ],[
            Paragraph(u"Дугаар: %s" % self.code, styNormal),
            Paragraph(u"Тайлант хугацаа: %s" % date_str, styNormalR),
        ]]
        
        tsGrid = TableStyle([
                ('GRID', (0,0), (-1,-1), 0.25, colors.white),
                ('SPAN', (0,1), (1,1))
        ])
        
        story.append(Table(grid, style=tsGrid, colWidths=[90*mm,90*mm]))
        story.append(Spacer(20,20))
        
        table = SimpleRowsTableBuilder(u'Санхүүгийн байдлын тайлан')
        
        table.add_text_column(u'Мөрийн дугаар', 15 * mm, extra={'font_size':7,'header_font_size':8})
        table.add_text_column(u'Үзүүлэлт', 100 * mm, extra={'font_size':7,'header_font_size':8})
        table.add_num_column(u'%s сарын %s'%(datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%m'),datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%d')), 25*mm, 2, extra={'font_size':7,'header_font_size':8})
        
        grid = self.create_report_pdf_data(cr, uid, data, context)
        
        for row in grid:
            table.add_text_cell(0)
            table.add_text_cell(row[1])
            table.add_num_cell(row[2])
        
        # Ингээд л болоо
        story.append(table.get_table())
        
        # Тайлангийн доод хэсэгт гарын үсэг зурах жижиг хүснэгт бэлдэнэ.
        tsGrid = TableStyle([
                ('GRID', (0,0), (-1,-1), 0.25, colors.white),
                ('FONT', (0,0),(-1,-1),'Helvetica', 8)
        ])
        #spacer
        story.append(Spacer(30,30))
        
        td_right_style = ParagraphStyle('tabledataright', fontName='Helvetica', fontSize=6, leading=11, 
                    alignment=TA_RIGHT, leftIndent=3, rightIndent=1, spaceAfter=1, spaceBefore=2)
        td_left_style = ParagraphStyle('tabledataleft', fontName='Helvetica', fontSize=6, leading=11, 
                    alignment=TA_LEFT, leftIndent=1, rightIndent=3, spaceAfter=1, spaceBefore=2)
        nullpara = Paragraph('', td_right_style)
        grid = [
            [Paragraph(u'', td_right_style), Paragraph(u'Захирал..................................................../<font color="white">____________________________________</font>/', td_left_style)],
            [nullpara,Paragraph(u'Ерөнхий нягтлан бодогч...................................................../<font color="white">____________________________________</font>/', td_left_style)]
        ]
        story.append(Table(grid, style=tsGrid, colWidths=[80*mm,160*mm]))
        
        return story

account_profit_report('report.account.profit.report', "Account Profit Report", 'account.profit.report', StandardReport.A4_PORTRAIT)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: