# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

import time
from lxml import etree

from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import datetime
from openerp import netsvc
import xlwt
from xlwt import Workbook, Formula
from operator import itemgetter


class account_balance_sheet_report(osv.osv_memory):
    """
        Санхүүгийн байдлын тайлан
    """
    _inherit = "abstract.report.excel"
    _name = "account.balance.sheet.report"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'date_from': fields.date("Start Date", required=True),
        'date_to': fields.date("End Date", required=True),
        'target_move': fields.selection([('posted', 'All Posted Entries'),
                                         ('all', 'All Entries'),
                                        ], 'Target Moves', required=True),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True),
    }
    
    
    _defaults = {
        'company_id': lambda obj,cr,uid,c=None:obj.pool.get('res.company')._company_default_get(cr,uid,'account.balance.sheet',context=c),
        'fiscalyear_id': lambda obj,cr,uid,c=None:obj.pool.get('account.fiscalyear').find(cr, uid, context=c),
        'target_move': 'posted',
        'date_from':time.strftime('%Y-%m-01'),
        'date_to':time.strftime('%Y-%m-%d'),
    }
    
    
    def prepare_data(self, cr, uid, ids, context=None):
        fy_obj = self.pool.get('account.fiscalyear')
        period_obj = self.pool.get('account.period')
        obj = self.browse(cr, uid, ids[0], context=context)
        
        data = {
            'ids': ids,
            'model': 'account.balance.sheet.report',
            'form': {
                'company_id': obj.company_id.id,
                'date_from': obj.date_from,
                'date_to': obj.date_to,
                'target_move': obj.target_move,
                'fiscalyear_id': obj.fiscalyear_id.id,
                'save': obj.save
            },
            'report_name': 'account.balance.sheet.report'
        }
        
        fy = fy_obj.read(cr, uid, data['form']['fiscalyear_id'], ['date_start','date_stop'])
        if data['form']['date_from'] < fy['date_start'] or data['form']['date_to'] > fy['date_stop'] \
            or data['form']['date_from'] > data['form']['date_to']:
            raise osv.except_osv(_('Error'),_('Select a valid date for fiscalyear.'))

        return data
    
    
    def get_export_data(self, cr, uid, ids, context=None):
        ''' Тайлангийн загварыг боловсруулж өгөгдлүүдийг
            тооцоолж байрлуулна.
        '''
        def name_padding(param):
            l = param['level'] - 2
            return ('  ' * l) + param['name']
        
        if context is None:
            context = {}
        datas = self.prepare_data(cr, uid, ids, context=context)
        #=======================================================================
        # report_service = netsvc.LocalService('report.account.balance.sheet.report')
        # report_service.cr = cr
        # report_service.uid = uid
        # report_service.datas = datas
        # report_service.context = context
        # report_service.pool = self.pool
        # report_service.lang = context.get('lang', 'mn_MN')
        #=======================================================================
        data = datas['form']
        
        period_obj = self.pool.get('account.period')
        account_line_obj = self.pool.get('account.move.line')
        account_obj = self.pool.get('account.account')
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        )
        obj = self.browse(cr, uid, ids[0], context=context)
        year = obj.fiscalyear_id.name
        document_code = ''
        directory_name = False
        #=======================================================================
        # if data['save']:
        #     generate_barcode = report_service.get_report_barcode(cr, context=context)
        #     document_code = report_service.code
        #=======================================================================
        
        ezxf = xlwt.easyxf
        styles = self.get_easyxf_styles()
        
        
        self.context = context
        ctx = self.context.copy()
        ctx1 = self.context.copy()
        ctx['company_id'] = data['company_id']
        ctx['fiscalyear'] = data['fiscalyear_id']
        ctx1['date_from'] = data['date_from']
        ctx1['date_to'] =  data['date_from']
        ctx['date_from'] = data['date_from']
        ctx['date_to'] =  data['date_to']
        ctx['state'] = data['target_move']
        
        book = xlwt.Workbook(encoding='utf8')
        sheet = book.add_sheet('Balance')
        style = ezxf('font:bold on;align:wrap off,vert centre,horiz right;font: height 200;borders: left thin, right thin, top thin, bottom medium;','#,##0.00_);(#,##0.00) ')
        styless = ezxf('font:bold on;align:wrap off,vert centre,horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylesss = ezxf('font:bold on;align:wrap off,vert centre,horiz centre;font: height 220;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        header = ezxf('font:bold on;align:wrap on,vert centre,horiz left;font: height 220;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylessss = ezxf('font:bold off;align:wrap on,vert centre,horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylesssss = ezxf('font:bold off;align:wrap on,vert centre,horiz right;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        under_style = ezxf('font:bold on;align:wrap off,vert centre,horiz right;font: height 200;borders: left thin, right thin, top thin, bottom thick;','#,##0.00_);(#,##0.00) ')
        sheet.write_merge(1, 1, 0, 3, u'САНХҮҮГИЙН БАЙДЛЫН ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 240'))
        sheet.write(3, 0, u' %s' %(company.name,), ezxf('font:bold on;align:wrap off,vert centre,horiz left;font: height 250 '))
        sheet.write(4, 0, u'(Аж ахуй нэгжийн нэр)', ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet.write(4, 3, u'%s оны %s сарын %s өдөр' %(year,datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%m'),datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%d')), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        sheet.write(5, 3, u'(төгрөгөөр)' , ezxf('font:bold off;align:wrap off,vert centre,horiz centre;'))
        sheet.write(6, 0, u'Мөрийн дугаар', header)
        sheet.write(6, 1, u'Үзүүлэлт', stylesss)
        sheet.write(6, 2, u'01-р сарын 01', stylesss)
        sheet.write(6, 3, u'%s-р сарын %s'%(datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%m'),datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%d')), stylesss)

        
        rowx = 7
        
        def _get_amount_by_type(account_report_type):
            ''' Тухайн санхүүгийн байдлын тайлангийн дансны төрөлд хийгдсэн тайлант хугацааны
             болон өнгөрсөн жилийн мөн үеийн гүйлгээний дүнг харуулна.
            '''
            bef_init_balance=0
            type_ids = conf_obj.search(cr, uid, [('name','=','balance'),('company_id','=',data['company_id'])])
            conf_id  = account_config_report_obj.search(cr, uid, [('report_id','=',type_ids),('type','=',account_report_type)])
            res = {}
            start_amount = 0
            end_amount = 0
            self.context = context
            ctx = self.context.copy()
            ctx1 = self.context.copy()
            ctx['fiscalyear'] = data['fiscalyear_id']
            ctx['company_id'] = data['company_id']
            ctx1['date_from'] = time.strftime("%Y-01-01")
            ctx1['date_to'] = data['date_from']
            ctx['date_from'] = data['date_from']
            ctx['date_to'] =  data['date_to']
            ctx['state'] = data['target_move']
            if type_ids:
                for x in account_config_report_obj.browse(cr, uid, conf_id, context=context):
                    sum = 0
                    sum1 = 0
                    for acc in x.account_id:
                        
                        config_start_line = account_obj.browse(cr, uid,[acc.id],ctx1)
                        if config_start_line:
                            for s_line in config_start_line:
                                start_amount = s_line['balance']
                                sum += start_amount
                        else:
                             sum = 0
                        config_end_line = account_obj.browse(cr, uid,[acc.id],ctx)
                        if config_end_line:
                            for e_line in config_end_line:
                                end_amount = e_line['balance']
                                sum1 +=end_amount
                        else:
                            sum1 = 0
                    res[x.id] = {
                        'id': x.id,
                        'name': x.name,
                        'start_amount':sum,
                        'end_amount':sum1,
                        'sequence': x.sequence or 1
                    }
                        
            return sorted(res.values(), key=itemgetter("sequence"))
        
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        account_rel = self.pool.get('account.config.rel')
        currency_obj = self.pool.get('res.currency')
        company_obj = self.pool.get('res.company')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        account_move_line_obj = self.pool.get('account.move.line')
        account_config_report_obj = self.pool.get('account.config')
        
        conf_line_obj = self.pool.get('account.cashflow.report.configure.line')
        conf_obj = self.pool.get('account.report.configure')
        
        
        move_state = 'all'
        if data['target_move'] == 'posted':
            move_state = 'posted'
                
        s_amount = 0
        e_amount = 0
        sheet.write(rowx, 0, '1',styless)
        sheet.write(rowx, 1, u'ХӨРӨНГӨ',styless)
        sheet.write(rowx, 2, '', styless)
        sheet.write(rowx, 3, '', styless)
        sheet.write(rowx+1, 0, '1.1',styless)
        sheet.write(rowx+1, 1, u'Эргэлтийн хөрөнгө',styless)
        sheet.write(rowx+1, 2, '', styless)
        sheet.write(rowx+1, 3, '', styless)

        rowx = 9
        for line in _get_amount_by_type('current_asset'):
            if line['start_amount'] < 0:
                s_amount = -line['start_amount']
            else:
                s_amount = line['start_amount']
            if line['end_amount'] < 0:
                e_amount = -line['end_amount']
            else:
                e_amount = line['end_amount']
            sheet.write(rowx, 0, line['sequence'],stylessss)
            sheet.write(rowx, 1, line['name'],stylessss)
            sheet.write(rowx, 2, s_amount,stylesssss)
            sheet.write(rowx, 3, e_amount ,stylesssss)
            rowx +=1
        rowx = 17
        sheet.write(rowx+1, 0, '1.1.10', stylessss)
        sheet.write(rowx+1, 1, '',stylessss)
        sheet.write(rowx+2, 0, '1.1.11',styless)
        sheet.write(rowx+2, 1,  'Эргэлтийн хөрөнгийн дүн',styless)
        sheet.write(rowx+2, 2, Formula('sum(C10:C19)') or '',under_style)
        sheet.write(rowx+2, 3, Formula('sum(D10:D19)') or '',under_style)
        sheet.write(rowx+3, 0, '1.2',styless)
        sheet.write(rowx+3, 1, u'Эргэлтийн бус хөрөнгө',styless)
        sheet.write(rowx+3, 2, '', styless)
        sheet.write(rowx+3, 3, '',styless)
         

        rowx = 21
        for line in _get_amount_by_type('noncurrent_asset'):
            if line['start_amount'] < 0:
                s_amount = -line['start_amount']
            else:
                s_amount = line['start_amount']
            if line['end_amount'] < 0:
                e_amount = -line['end_amount']
            else:
                e_amount = line['end_amount']
            sheet.write(rowx, 0, line['sequence'],stylessss)
            sheet.write(rowx, 1, line['name'],stylessss)
            sheet.write(rowx, 2, s_amount or '',stylessss)
            sheet.write(rowx, 3, e_amount or '',stylessss)
            rowx +=1
        rowx = 29
        sheet.write(rowx, 0, '1.2.9',stylessss)
        sheet.write(rowx, 1,  u'',stylessss)
        sheet.write(rowx, 2, '', stylessss)
        sheet.write(rowx, 3, '',stylessss)
        
        sheet.write(rowx+1, 0, '1.2.10', styless)
        sheet.write(rowx+1, 1, u'Эргэлтийн бус хөрөнгийн дүн',styless)
        sheet.write(rowx+1, 2, Formula('SUM(C22:C30)') or '',under_style)
        sheet.write(rowx+1, 3, Formula('SUM(D22:D30)') or '',under_style)
         
        sheet.write(rowx+2, 0, '1.3',styless)
        sheet.write(rowx+2, 1, u'НИЙТ ХӨРӨНГИЙН ДҮН',styless)
        sheet.write(rowx+2, 2, Formula('C20+C31') or '',style)
        sheet.write(rowx+2, 3, Formula('D20+D31') or '',style)
        sheet.write(rowx+3, 0, '2',styless)
        sheet.write(rowx+3, 1, u'ӨР ТӨЛБӨР БА ЭЗДИЙН ӨМЧ',styless)
        sheet.write(rowx+3, 2, '', styless)
        sheet.write(rowx+3, 3, '', styless)
        sheet.write(rowx+4, 0, '2.1', styless)
        sheet.write(rowx+4, 1, u'Өр төлбөр',styless)
        sheet.write(rowx+4, 2, '', styless)
        sheet.write(rowx+4, 3, '', styless)
        sheet.write(rowx+5, 0, '2.1.1',styless)
        sheet.write(rowx+5, 1, u'Богино хугацаат өр төлбөр',styless)
        sheet.write(rowx+5, 2, '', styless)
        sheet.write(rowx+5, 3, '', styless)
        rowx = 35
        for line in _get_amount_by_type('short_term_liability'):
            if line['start_amount'] < 0:
                s_amount = -line['start_amount']
            else:
                s_amount = line['start_amount']
            if line['end_amount'] < 0:
                e_amount = -line['end_amount']
            else:
                e_amount = line['end_amount']
            sheet.write(rowx, 0, line['sequence'],stylessss)
            sheet.write(rowx, 1, line['name'], stylessss)
            sheet.write(rowx, 2, s_amount or '',stylessss)
            sheet.write(rowx, 3, e_amount or '',stylessss)
            rowx +=1
        sheet.write(rowx, 0, '2.1.1.12',stylessss)
        sheet.write(rowx, 1, '', stylessss)
        sheet.write(rowx, 2, '', stylessss)
        sheet.write(rowx, 3, '', stylessss)
        sheet.write(rowx+1, 0, '2.1.1.13', styless)
        sheet.write(rowx+1, 1, u'Богино хугацаат өр төлбөрийн дүн', styless)
        sheet.write(rowx+1, 2, Formula('SUM(C36:C47)') or '',under_style)
        sheet.write(rowx+1, 3, Formula('SUM(D36:D47)') or '',under_style)
        
        sheet.write_merge(rowx+9, rowx+9, 0, 3, u'САНХҮҮГИЙН БАЙДЛЫН ТАЙЛАН (үргэлжлэл)',ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 240'))
        sheet.write(rowx+11, 0, u'Мөрийн дугаар', header)
        sheet.write(rowx+11, 1, u'Үзүүлэлт', stylesss)
        sheet.write(rowx+11, 2, u'01-р сарын 01', stylesss)
        sheet.write(rowx+11, 3, u'%s-р сарын %s'%(datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%m'),datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%d')), stylesss)
        sheet.write(rowx+12, 0, '2.1.2', styless)
        sheet.write(rowx+12, 1, u'Урт хугацаат өр төлбөр',styless)
        sheet.write(rowx+12, 2, '',styless)
        sheet.write(rowx+12, 3, '',styless)
        rowx = 59
        for line in _get_amount_by_type('long_term_liability'):
            if line['start_amount'] < 0:
                s_amount = -line['start_amount']
            else:
                s_amount = line['start_amount']
            if line['end_amount'] < 0:
                e_amount = -line['end_amount']
            else:
                e_amount = line['end_amount']
            sheet.write(rowx, 0, line['sequence'],stylessss)
            sheet.write(rowx, 1, line['name'], stylessss)
            sheet.write(rowx, 2, s_amount or '',stylesssss)
            sheet.write(rowx, 3, e_amount or '',stylesssss)
            rowx +=1        
        sheet.write(rowx, 0, '2.1.2.5', stylessss)
        sheet.write(rowx, 1, '', stylessss)
        sheet.write(rowx, 2, '', stylessss)
        sheet.write(rowx, 3, '', stylessss)
        sheet.write(rowx+1, 0, '2.1.2.6', styless)
        sheet.write(rowx+1, 1, u'Урт хугацаат өр төлбөрийн дүн',styless)
        sheet.write(rowx+1, 2, Formula('SUM(C50:C54)') or '',style)
        sheet.write(rowx+1, 3, Formula('SUM(D50:D54)') or '',style)
        sheet.write(rowx+2, 0, '2.2',styless)
        sheet.write(rowx+2, 1, u'Өр төлбөрийн нийт дүн', styless)
        sheet.write(rowx+2, 2, (Formula('C48+C65') or ''),under_style)
        sheet.write(rowx+2, 3, Formula('D48+D65') or '',under_style)
        sheet.write(rowx+3, 0, '2.3', styless)
        sheet.write(rowx+3, 1, u'Эздийн өмч',styless)
        sheet.write(rowx+3, 2, '', styless)
        sheet.write(rowx+3, 3, '', styless)
        rowx = 67
        for line in _get_amount_by_type('owner_property'):
            if line['start_amount'] < 0:
                s_amount = -line['start_amount']
            else:
                s_amount = line['start_amount']
            if line['end_amount'] < 0:
                e_amount = -line['end_amount']
            else:
                e_amount = line['end_amount']
            sheet.write(rowx, 0, line['sequence'], stylessss)
            sheet.write(rowx, 1, line['name'], stylessss)
            sheet.write(rowx, 2, (s_amount or ''),stylessss)
            sheet.write(rowx, 3, (e_amount or ''),stylessss) 
            rowx +=1
        sheet.write(rowx, 0, '2.3.10',stylessss)
        sheet.write(rowx, 1, '', stylessss)
        sheet.write(rowx, 2, '', stylessss)
        sheet.write(rowx, 3, '', stylessss)
        sheet.write(rowx+1, 0, '2.3.11',styless)
        sheet.write(rowx+1, 1, u'Эздийн өмчийн дүн',styless)
        sheet.write(rowx+1, 2, Formula('SUM(C68:C77)') or '',style)
        sheet.write(rowx+1, 3, Formula('SUM(D68:D77)') or '',style)
        sheet.write(rowx+2, 0, '2.4',styless)
        sheet.write(rowx+2, 1, u'ӨР ТӨЛБӨР БА ЭЗДИЙН ӨМЧИЙН ДҮН',styless)
        sheet.write(rowx+2, 2, Formula('C66+C78') or '',under_style)
        sheet.write(rowx+2, 3, Formula('D66+D78') or '',under_style)                
        rowx = 80
        inch = 1000
        sheet.col(0).width = 3*inch
        sheet.col(1).width = 13*inch
        sheet.col(2).width = 4*inch
        sheet.col(3).width = 4*inch
        sheet.row(37).height = 600
        sheet.row(58).height = 600
        sheet.row(17).height = 600
        sheet.row(6).height = 600
        
                
        sheet.write_merge(rowx+2, rowx+2, 1, 3, u'Захирал                           _____________________________(.................................)', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet.write_merge(rowx+4, rowx+4, 1, 3, u'Ерөнхий нягтлан бодогч    ____________________________(...............................)', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))

        return {'data':book}

account_balance_sheet_report()