# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2014 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

from datetime import datetime
from openerp.osv import fields, osv
from openerp.tools.translate import _
from operator import itemgetter
import time
from xlwt import Workbook, Formula
from openerp import netsvc
import xlwt
class account_cashflow_statement_report(osv.osv_memory):
    """
        МӨНГӨН ГҮЙЛГЭЭНИЙ ТАЙЛАН
    """
    _inherit = "abstract.report.excel"
    _name = "account.cashflow.statement.report"
    _description = "Cashflows statement"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'filter': fields.selection([('filter_date', 'Date'), ('filter_period', 'Periods')], "Filter by", required=True),
        'period_from': fields.many2one('account.period', 'Start Period'),
        'period_to': fields.many2one('account.period', 'End Period'),
        'date_from': fields.date("Start Date"),
        'date_to': fields.date("End Date"),
        'target_move': fields.selection([('posted', 'All Posted Entries'),
                                         ('all', 'All Entries'),
                                        ], 'Target Moves', required=True),
        'chart_account_id': fields.many2one('account.account', 'Chart of Account', help='Select Charts of Accounts', required=True, domain=[('parent_id','=',False)]),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True)
    }
    
    def default_chart_account(self, cr, uid, context=None):
        account_ids = self.pool.get('account.account').search(cr, uid, [('parent_id','=',False)], context=context)
        return (account_ids and account_ids[0]) or False
    
    _defaults = {
        'company_id': lambda obj,cr,uid,c=None:obj.pool.get('res.company')._company_default_get(cr,uid,'account.balance.sheet',context=c),
        'filter': 'filter_date',
        'fiscalyear_id': lambda obj,cr,uid,c=None:obj.pool.get('account.fiscalyear').find(cr, uid, context=c),
        'target_move': 'posted',
        'chart_account_id': default_chart_account
    }
    
    def onchange_filter(self, cr, uid, ids, filter='filter_date', fiscalyear_id=False, context=None):
        res = {'value': {}}
        if filter == 'filter_date':
            res['value'] = {'period_from': False, 'period_to': False, 'date_from': time.strftime('%Y-01-01'), 'date_to': time.strftime('%Y-%m-%d')}
        elif filter == 'filter_period':
            start_period = end_period = False
            cr.execute('''
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_start ASC, p.special ASC
                               LIMIT 1) AS period_start
                UNION ALL
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_stop DESC
                               LIMIT 1) AS period_stop''', (fiscalyear_id, fiscalyear_id))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
            res['value'] = {'period_from': start_period, 'period_to': end_period, 'date_from': False, 'date_to': False}
        return res
    
    def onchange_fiscalyear(self, cr, uid, ids, fiscalyear, filter, context=None):
        start_period = end_period = False
        if filter == 'filter_period':
            cr.execute('''
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_start ASC, p.special ASC
                               LIMIT 1) AS period_start
                UNION ALL
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_stop DESC
                               LIMIT 1) AS period_stop''', (fiscalyear, fiscalyear))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
        return {'value':{'period_from':start_period,'period_to':end_period}}
    
    def prepare_data(self, cr, uid, ids, context=None):
        fy_obj = self.pool.get('account.fiscalyear')
        obj = self.browse(cr, uid, ids[0], context=context)
        
        data = {
            'ids': ids,
            'model': 'account.cashflow.statement.report',
            'form': {
                'company_id': obj.company_id.id,
                'filter': obj.filter,
                'period_from': obj.period_from.id,
                'period_to': obj.period_to.id,
                'date_from': obj.date_from,
                'date_to': obj.date_to,
                'target_move': obj.target_move,
                'chart_account_id': obj.chart_account_id.id,
                'fiscalyear_id': obj.fiscalyear_id.id,
                'save': obj.save
            },
            'report_name': 'account.cashflow.statement.report'
        }
        
        if data['form']['filter'] == 'filter_date':
            fy = fy_obj.read(cr, uid, data['form']['fiscalyear_id'], ['date_start','date_stop'])
            if data['form']['date_from'] < fy['date_start'] or data['form']['date_to'] > fy['date_stop'] \
                or data['form']['date_from'] > data['form']['date_to']:
                raise osv.except_osv(_('Error'),_('Select a valid date for fiscalyear.'))
        elif data['form']['filter'] == 'filter_period':
            if not data['form']['period_from'] or not data['form']['period_to']:
                raise osv.except_osv(_('Error'),_('Select a starting and an ending period'))
            data['form']['date_from'] = period_obj.read(cr, uid, data['form']['period_from'], ['date_start'])['date_start']
            data['form']['date_to'] = period_obj.read(cr, uid, data['form']['period_to'], ['date_stop'])['date_stop']
        return data
    
    def get_log_message(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        
        date_from = (obj.period_from and obj.period_from.date_start) or obj.date_from
        date_to = (obj.period_to and obj.period_to.date_stop) or obj.date_to
        body = (u"Мөнгөн гүйлгээний тайлан (Эхлэх='%s', Дуусах='%s', Мод='%s')") % \
            (date_from, date_to, obj.chart_account_id.name)
        return body
    
    
    def create_report_xls_data(self, cr, uid, data, context=None):
        ''' Гүйлгээ баланс тайлангийн мэдээллийг боловсруулж
            тайлангийн форматад тохируулан python [{},{},{}...]
            загвараар хүснэгтийн мөр багануудын өгөгдлийг боловсруулна.
            
        '''
        def get_initial_journal(fy):
            initial_bal_journal = False
            prev_fy_ids = fiscalyear_obj.search(cr, uid, [('date_stop', '<', fy.date_start)], order="date_start desc")
            if prev_fy_ids :
                prev_fy = fiscalyear_obj.browse(cr, uid, prev_fy_ids[0])
                if prev_fy.end_journal_period_id :
                    initial_bal_journal = prev_fy.end_journal_period_id.journal_id.id
            return initial_bal_journal, (prev_fy_ids and prev_fy_ids[0]) or False
        
        def _get_amount_by_type(move_type_group):
            ''' Тухайн мөнгөн гүйлгээний төрөлд хийгдсэн тайлант хугацааны
             болон өнгөрсөн жилийн мөн үеийн гүйлгээний дүнг харуулна.
            '''
            bef_init_balance=0
            type_ids = cash_move_type_obj.search(cr, uid, [('group','=',move_type_group)], order='sequence')
            res = {}
            if type_ids:
                for x in cash_move_type_obj.browse(cr, uid, type_ids, context=context):
                    config_year_ids = conf_obj.search(cr,uid,[('filter','=','year'),('year_id', '=', data['fiscalyear_id'])], context=None)
                    year_amount = 0
                    period_amount = 0
                    if config_year_ids !=[]:
                        config_year_line_ids = conf_line_obj.search(cr,uid,[('line','=',config_year_ids[0])], context=None)
                        config_year_line = conf_line_obj.browse(cr, uid,config_year_line_ids,context=None)
                        
                        
                        if config_year_line:
                            for line in config_year_line:
                                if line['cash_move_type'].group == move_type_group and line['cash_move_type'].sequence ==x.sequence:
                                    if line:
                                        year_amount = line['amount']
                                    else:
                                        year_amount = 0
                                config_period_ids = conf_obj.search(cr,uid,[('filter','=','period')], context=None)
                                if config_period_ids !=[]:
                                    config_period = conf_obj.browse(cr, uid,config_period_ids,context=None)
                                    config_period_line_ids = conf_line_obj.search(cr,uid,[('line','=',config_period_ids[0])], context=None)
                                    config_period_line = conf_line_obj.browse(cr, uid,config_period_line_ids,context=None)
                                    
                                    if config_period[0]['period_id'].fiscalyear_id.id== data['fiscalyear_id']:
                                        for line1 in config_period_line:
                                            if line1['cash_move_type'].group == move_type_group and line1['cash_move_type'].sequence ==x.sequence:
                                                if line1:
                                                    period_amount = line1['amount']
                                                else:
                                                    period_amount = 0
                            
                    res[x.id] = {
                        'id': x.id,
                        'name': x.name,
                        'initail_amount':year_amount,
                        'amount':period_amount,
                        'sequence': x.sequence or 1
                    }
                        
                cr.execute("select l.cash_move_type, coalesce(SUM(l.amount),0) "
                           "from account_bank_statement_line l "
                           "where " + query + " "
                           "and l.account_id in %s "
                           "and l.cash_move_type is not null "
                           "and l.cash_move_type in %s "
                           "group by l.cash_move_type ",
                           (account_tuple,tuple(type_ids)))
                fetched = cr.fetchall()
                period_amount = 0
                
                if fetched:
                    for type_id, amount in fetched:
                        res[type_id]['amount'] +=  amount
            return sorted(res.values(), key=itemgetter("sequence"))
        
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        journal_obj = self.pool.get('account.journal')
        currency_obj = self.pool.get('res.currency')
        company_obj = self.pool.get('res.company')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        account_move_line_obj = self.pool.get('account.move.line')
        cash_move_type_obj = self.pool.get('account.cash.move.type')
        
        conf_line_obj = self.pool.get('account.cashflow.report.configure.line')
        conf_obj = self.pool.get('account.cashflow.report.configure')
        
        # Тухайн санхүүгийн жилд эхний үлдэгдэл бичсэн санхүүгийн журналыг тодорхойлно.
        fiscalyear = fiscalyear_obj.browse(cr, uid, data['fiscalyear_id'], context=context)
        current_initial_journal, previous_year_id = get_initial_journal(fiscalyear)
        
        move_state = 'all'
        if data['target_move'] == 'posted':
            move_state = 'posted'
            
        query = "date >= \'" +data['date_from']+ "\' AND date <= \'"+data['date_to']+"\' "
        account_ids = account_obj.search(cr, uid, [('type','<>','liquidity'),('parent_id','child_of',[data['chart_account_id']])], context=context)
        account_tuple = tuple(account_ids)
        
        grid = []
        total = grand = 0.0
        total1 = grand1 = 0.0
        grid.append(['1', u'<b>Үндсэн үйл ажиллагааны мөнгөн гүйлгээ</b>', '', '', ''])
        sublines = []
        for line in _get_amount_by_type('activities_income') :
            sublines.append(['', line['name'], line['initail_amount'], line['amount'], '<font color="white">_______</font>'])
        grid.append(['1.1', u'Мөнгөн орлогын дүн', Formula('SUM(C8:C13)'),Formula('SUM(D8:D13)'), '<font color="white">___</font>'])
        grid.extend(sublines)
        
        sublines = []
        subtotal = 0
        subtotal1 = 0
        for line in _get_amount_by_type('activities_expense') :
            sublines.append(['', line['name'],-line['initail_amount'], -line['amount'], '<font color="white">_______</font>'])
            subtotal += line['amount']
            subtotal1 += line['initail_amount']
            
        grid.append(['', u'', '', '', '<font color="white">___</font>'])
        grid.append(['1.2', u'Мөнгөн зарлагын дүн',Formula('SUM(C16:C24)'), Formula('SUM(D16:D24)'), '<font color="white">___</font>'])
        grid.extend(sublines)
         
        grid.append(['', u'', '', '', '<font color="white">___</font>'])
        grid.append(['1.3', u'<b>Үндсэн үйл ажиллагааны цэвэр мөнгөн гүйлгээний дүн</b>', Formula('C7-C15'),Formula('D7-D15'), ''])
         
        total = 0
        subtotal = 0
        subtotal1 = 0
        sublines = []
        grid.append(['2', u'<b>Хөрөнгө оруулалтын үйл ажиллагааны мөнгөн гүйлгээ</b>', '', '',''])
        for line in _get_amount_by_type('investing_income') :
            sublines.append(['', line['name'], line['initail_amount'],line['amount'], '<font color="white">_______</font>'])
        grid.append(['2.1', u'Мөнгөн орлогын дүн', Formula('SUM(C29:C35)'), Formula('SUM(D29:D35)'), '<font color="white">___</font>'])
        grid.extend(sublines)
         
        subtotal = 0
        subtotal1 = 0
        total = 0
        total1 = 0
        sublines = []
        for line in _get_amount_by_type('investing_expense'):
            sublines.append(['', line['name'],-line['initail_amount'], -line['amount'], '<font color="white">_______</font>'])
            subtotal += line['amount']
            subtotal1 += line['initail_amount']
        grid.append(['', u'', '', '', '<font color="white">___</font>'])
        grid.append(['2.2', u'Мөнгөн зарлагын дүн', Formula('SUM(C38:C42)'),Formula('SUM(C38:C42)'), '<font color="white">___</font>'])
        grid.extend(sublines)
        total += subtotal
        total1 += subtotal1
        
        grid.append(['', u'', '', '', '<font color="white">___</font>'])
        grid.append(['2.3', u'<b>Хөрөнгө оруулалтын үйл ажиллагааны цэвэр мөнгөн гүйлгээний дүн</b>', Formula('C28-C37'),Formula('D28-D37'), ''])
        grand += total
        grand1 += total1
        total = 0
        total1 = 0
         
        sublines = []
        subtotal = 0
        subtotal1 = 0
        grid.append(['3', u'<b>Санхүүгийн үйл ажиллагааны мөнгөн гүйлгээ</b>', '', '',''])
        for line in _get_amount_by_type('financing_income'):
            sublines.append(['', line['name'],line['initail_amount'], line['amount'], '<font color="white">_______</font>'])
        grid.append(['3.1', u'Мөнгөн орлогын дүн', Formula('SUM(C47:C49)'), Formula('SUM(D47:D49)'), '<font color="white">___</font>'])
        grid.extend(sublines)
         
        subtotal = 0
        subtotal1 = 0
        sublines = []
        for line in _get_amount_by_type('financing_expense'):
            sublines.append(['', line['name'],-line['initail_amount'], -line['amount'], '<font color="white">_______</font>'])
        grid.append(['', u'', '', '', '<font color="white">___</font>'])
        grid.append(['3.2', u'Мөнгөн зарлагын дүн', Formula('SUM(C52:C55)'), Formula('SUM(D52:D55)'), '<font color="white">___</font>'])
        grid.extend(sublines)
         
        grid.append(['', u'', '', '', '<font color="white">___</font>'])
        grid.append(['3.3', u'<b>Санхүүгийн үйл ажиллагааны цэвэр мөнгөн гүйлгээний дүн</b>', Formula('C46-C51'), Formula('D46-D51'), ''])
        grid.append(['4', u'<b>Бүх цэвэр мөнгөн гүйлгээ</b>', Formula('C26+C44+C57'), Formula('D26+D44+D57'), ''])
        
#        Эхний эцсийн үлдэгдлийг харин санхүүгийн бичилтээс тооцоно.
        account_ids = account_obj.search(cr, uid, [('type','=','liquidity'),('parent_id','child_of',[data['chart_account_id']])], context=context)
        account_tuple = tuple(account_ids)
 
        initial_bal = account_obj.get_initial_balance(cr, uid, account_ids, context=context)
        context.update({'date_from': fiscalyear.date_start})
        bal = dict((a['id'], a) for a in account_obj.read(cr, uid, account_ids, ['debit','credit'], context=context))
        
        bef_init_balance=0
        for line in _get_amount_by_type('dummy'):
            bef_init_balance = line['initail_amount']
        
        bef_balance = grand1+bef_init_balance
        now_balance = grand + bef_balance
        grid.append(['5', u'<b>Мөнгө, түүнтэй адилтгах хөрөнгийн эхний үлдэгдэл</b>', bef_init_balance, bef_balance, ''])
        grid.append(['6', u'<b>Мөнгө, түүнтэй адилтгах хөрөнгийн эцсийн үлдэгдэл</b>',Formula('C58+C59'), Formula('D58+D59'), ''])
        
        return grid
    
    def get_export_data(self, cr, uid, ids, context=None):
        ''' Тайлангийн загварыг боловсруулж өгөгдлүүдийг
            тооцоолж байрлуулна.
        '''
        def name_padding(param):
            l = param['level'] - 2
            return ('  ' * l) + param['name']
        
        if context is None:
            context = {}
        
        datas = self.prepare_data(cr, uid, ids, context=context)
        
        data = datas['form']
        
        period_obj = self.pool.get('account.period')
        account_obj = self.pool.get('account.account')
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        
        )
        date_str1 = data['date_to']
        
        
        ezxf = xlwt.easyxf
        styles = self.get_easyxf_styles()
        
        book = xlwt.Workbook(encoding='utf8')
        sheet = book.add_sheet(u'Мөнгөн гүйлгээний тайлан')
        
        style = ezxf('font:bold on;align:wrap on,vert centre,horiz centre;font: height 200;borders: left thin, right thin, top thin, bottom medium;','#,##0.00_);(#,##0.00) ')
        styless = ezxf('font:bold on;align:wrap on,vert centre,horiz left;font: height 200;borders: left thin, right thin, top thin, bottom medium;','#,##0.00_);(#,##0.00) ')

        stylesssss = ezxf('font:bold off;align:wrap on,vert centre,horiz right;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylessss = ezxf('font:bold off;align:wrap on,vert centre,horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylesss = ezxf('font:bold off;align:wrap on,vert centre,horiz centre;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')


        sheet.write_merge(0, 0, 0, 3, u'МӨНГӨН ГҮЙЛГЭЭНИЙ ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 220'))
        
        sheet.write(2, 0, u'________%s______' %(company.name,), ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet.write(3, 0, u' ( Аж ахуйн нэгжийн нэр )')
        sheet.write(2, 3, u'%s өдөр' %(date_str1,), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        
        rowx = 4
        sheet.write(rowx, 0, u'Мөрийн дугаар', stylesss)
        sheet.write(rowx, 1, u'Үзүүлэлт', stylesss)
        sheet.write(rowx, 2, u'%s оны 12 сарын 31 өдөр', stylesss)
        sheet.write(rowx, 3, u'%s оны 12 сарын 31 өдөр', stylesss)
        rowx += 1
        
        grid = self.create_report_xls_data(cr, uid, data, context)
         
        for row in grid:
            name = row[4]+row[1]
            name_style = styles['text_xf']
            if '<b>' in name:
                name_style = styles['text_bold_xf']
            name = name.replace('<font color="white">','').replace('</font>','')
            name = name.replace('<b>','').replace('</b>','')
            name = name.replace('_',' ')
            sheet.write(rowx, 0, row[0], stylessss)
            sheet.write(rowx, 1, name, stylessss)
            sheet.write(rowx, 2, row[2], stylesssss)
            sheet.write(rowx, 3, row[3], stylesssss)
            rowx += 1
            
        inch = 1000
        sheet.col(0).width = 2*inch
        sheet.col(1).width = 15*inch
        sheet.col(2).width = 4*inch
        sheet.col(3).width = 4*inch
        sheet.row(4).height = 500
        
        sheet.write(rowx+1, 1, u'      Захирал             _____________________________(.................................)', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet.write(rowx+3, 1, u'      Ерөнхий нягтлан бодогч    ______________________________(.................................)', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        
        return {'data':book}

account_cashflow_statement_report()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: