# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

import time
from lxml import etree

from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import datetime
from openerp import netsvc
import xlwt
from xlwt import Workbook, Formula
from operator import itemgetter

class account_profit_report(osv.osv_memory):
    """
        Орлогын дэлгэрэнгүй тайлан
    """
    _inherit = "abstract.report.excel"
    _name = "account.profit.report"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'date_from': fields.date("Start Date",required=True),
        'date_to': fields.date("End Date",required=True),
        'target_move': fields.selection([('posted', 'All Posted Entries'),
                                         ('all', 'All Entries'),
                                        ], 'Target Moves', required=True),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True),
    }
    
    
    _defaults = {
        'company_id': lambda obj,cr,uid,c=None:obj.pool.get('res.company')._company_default_get(cr,uid,'account.balance.sheet',context=c),
        'fiscalyear_id': lambda obj,cr,uid,c=None:obj.pool.get('account.fiscalyear').find(cr, uid, context=c),
        'target_move': 'posted',
        'date_from':time.strftime('%Y-%m-01'),
        'date_to':time.strftime('%Y-%m-%d'),
    }
    
    
    def prepare_data(self, cr, uid, ids, context=None):
        fy_obj = self.pool.get('account.fiscalyear')
        period_obj = self.pool.get('account.period')
        obj = self.browse(cr, uid, ids[0], context=context)
        
        data = {
            'ids': ids,
            'model': 'account.profit.report',
            'form': {
                'company_id': obj.company_id.id,
                'date_from': obj.date_from,
                'date_to': obj.date_to,
                'target_move': obj.target_move,
                'fiscalyear_id': obj.fiscalyear_id.id,
                'save': obj.save
            },
            'report_name': 'account.profit.report'
        }
        
        fy = fy_obj.read(cr, uid, data['form']['fiscalyear_id'], ['date_start','date_stop'])
        if data['form']['date_from'] < fy['date_start'] or data['form']['date_to'] > fy['date_stop'] \
            or data['form']['date_from'] > data['form']['date_to']:
            raise osv.except_osv(_('Error'),_('Select a valid date for fiscalyear.'))
        return data
    
    def create_report_xls_data(self, cr, uid, data, context=None):
        ''' Орлогын тайлангийн мэдээллийг боловсруулж
            тайлангийн форматад тохируулан python [{},{},{}...]
            загвараар хүснэгтийн мөр багануудын өгөгдлийг боловсруулна.
            
            @return: income - Орлого талын дансны өгөгдлүүд 
                     expense - Үндсэн үйл ажиллагааны зардал талын дансны өгөгдлүүд
                     income2 - Үндсэн бус үйл ажиллагааны зардал талын дансны өгөгдлүүд
        '''
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        currency_obj = self.pool.get('res.currency')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        type_obj = self.pool.get('account.account.type')
        if not context:
            context = {}
        
        fy = fiscalyear_obj.browse(cr, uid, data['fiscalyear_id'], context=context)
        ctx = context.copy()
        ctx['fiscalyear'] = data.get('fiscalyear_id', False)
        ctx['date_from'] = fy.date_start
        ctx['date_to'] =  data.get('date_to', time.strftime('%Y-%m-%d'))
        ctx['state'] = data.get('target_move', 'all')
        ctx['return_initial_bal_journal'] = True
        
        def _get_amount_by_type(account_report_type):
            ''' Тухайн санхүүгийн байдлын тайлангийн дансны төрөлд хийгдсэн тайлант хугацааны
             болон өнгөрсөн жилийн мөн үеийн гүйлгээний дүнг харуулна.
            '''
            bef_init_balance=0
            type_ids = conf_obj.search(cr, uid, [('name','=','income')])#tailangiin nernii torol(income bolon balance)
            conf_id  = account_config_report_obj.search(cr, uid, [('report_id','=',type_ids),('type','=',account_report_type)])
            res = {}
            start_amount = 0
            end_amount = 0
            if type_ids:
                for x in account_config_report_obj.browse(cr, uid, conf_id, context=context):
                    for acc in x.account_id:
                        config_end_line = account_obj.browse(cr, uid,acc.id,ctx)
                        for e_line in config_end_line:
                            start_amount = start_amount + e_line['balance']
                        
                    res[x.id] = {
                        'id': x.id,
                        'name': x.name,
                        'start_amount':start_amount,
                        'sequence': x.sequence or 1
                    }
                        
            return sorted(res.values(), key=itemgetter("sequence"))
        
        if not context:
            context = {}
        account_rel = self.pool.get('account.config.rel')
        currency_obj = self.pool.get('res.currency')
        company_obj = self.pool.get('res.company')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        account_move_line_obj = self.pool.get('account.move.line')
        conf_line_obj = self.pool.get('account.cashflow.report.configure.line')
        
        account_obj = self.pool.get('account.account')
        account_config_report_obj = self.pool.get('account.config')
        conf_obj = self.pool.get('account.report.configure')
        
        
        move_state = 'all'
        if data['target_move'] == 'posted':
            move_state = 'posted'
                
        grid = []
        sublines = []
        sale_income = 0
        for line in _get_amount_by_type('sale_income') :
            sublines.append([line['sequence'], line['name'], line['start_amount'], '<font color="white">_______</font>'])
            sale_income += line['start_amount']
            #===================================================================
            # if sale_income =='':
            #    sale_income +=  line['start_amount']
            # else:
            #     sale_income = sale_income + ',' + sale_income
            #===================================================================
        grid.extend(sublines)
        
        sale_cost = 0
        gain_and_loss = 0
        sublines = []
        for line in _get_amount_by_type('sale_cost'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
        grid.extend(sublines)
        grid.append(['3', u'Нийт ашиг ( алдагдал)', Formula('C8-C9'), '<font color="white">___</font>'])

      
        income = 0
        sublines = []
        for line in _get_amount_by_type('income'):
            sublines.append([line['sequence'], line['name'],line['start_amount'], '<font color="white">_______</font>'])
            income += line['start_amount']
        grid.extend(sublines)
            
        cost = 0
        tax_gain_loss = 0
        sublines = []
        for line in _get_amount_by_type('cost'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
            cost += line['start_amount']
        grid.extend(sublines)
        tax_gain_loss = gain_and_loss+income+cost
        grid.append(['18', u'<b>Татвар төлөхийн өмнөх ашиг ( алдагдал)</b>', Formula('SUM(C11:C15)-SUM(C16:C24)'),'<font color="white">___</font>'])
        
        owner_property_total = 0
        total = 0
        tax_to_next = 0
        income_tax_loss = 0
        sublines = []
        for line in _get_amount_by_type('tax_cost'):
            sublines.append([line['sequence'], line['name'],Formula('C25/10'),  '<font color="white">_______</font>'])
            owner_property_total += line['start_amount']
        grid.extend(sublines)
        tax_to_next = tax_gain_loss-income_tax_loss
        grid.append(['20', u'<b>Татварын дараах ашиг (алдагдал)</b>', Formula('C25-C26'), ''])
        grid.append(['21', u'<b>Зогсоосон үйл ажиллагааны татварын дараах ашиг (алдагдал)</b>', tax_to_next, '<font color="white">___</font>'])
        grid.append(['22', u'<b>Тайлант үеийн цэвэр ашиг ( алдагдал)</b>', Formula('C27-C28'), '<font color="white">___</font>'])
        grid.append(['23', u'<b>Бусад дэлгэрэнгүй орлого</b>','', '', ''])
        
        other_income = 0
        total = 0
        sublines = []
        for line in _get_amount_by_type('other_income'):
            sublines.append([line['sequence'], line['name'],line['start_amount'],  '<font color="white">_______</font>'])
        grid.extend(sublines)
        grid.append(['24', u'<b>Орлогын нийт дүн</b>',Formula('SUM(C31:C33)'), ''])
        grid.append(['25', u'<b>Нэгж хувьцаанд ногдох суурь ашиг (алдагдал)</b>','', ''])
        
        return grid
    
    def get_export_data(self, cr, uid, ids, context=None):
        ''' Тайлангийн загварыг боловсруулж өгөгдлүүдийг
            тооцоолж байрлуулна.
        '''
        def name_padding(param):
            l = param['level'] - 2
            return ('  ' * l) + param['name']
        
        if context is None:
            context = {}
        datas = self.prepare_data(cr, uid, ids, context=context)
       
        data = datas['form']
        
        period_obj = self.pool.get('account.period')
        account_line_obj = self.pool.get('account.move.line')
        account_obj = self.pool.get('account.account')
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        )
        obj = self.browse(cr, uid, ids[0], context=context)
        year = obj.fiscalyear_id.name
        document_code = ''
        directory_name = False
    
        ezxf = xlwt.easyxf
        styles = self.get_easyxf_styles()
        
        
        self.context = context
        ctx = self.context.copy()
        ctx['fiscalyear'] = data['fiscalyear_id']
        ctx['date_from'] = data['date_from']
        ctx['date_to'] =  data['date_to']
        ctx['state'] = data['target_move']

        book = xlwt.Workbook(encoding='utf8')
        sheet = book.add_sheet('Profit')
        
        style = ezxf('font:bold on;align:wrap off,vert centre,horiz right;font: height 200;borders: left thin, right thin, top thin, bottom medium;','#,##0.00_);(#,##0.00) ')
        styless = ezxf('font:bold on;align:wrap off,vert centre,horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylesss = ezxf('font:bold on;align:wrap off,vert centre,horiz centre;font: height 220;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        header = ezxf('font:bold on;align:wrap on,vert centre,horiz centre;font: height 220;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylessss = ezxf('font:bold off;align:wrap on,vert centre,horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')
        stylesssss = ezxf('font:bold off;align:wrap on,vert centre,horiz right;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00) ')



        sheet.write_merge(1, 1, 0, 1, u'ОРЛОГЫН ДЭЛГЭРЭНГҮЙ ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 220'))
        sheet.write_merge(2, 2, 0, 2, u' %s оны %s сарын %s өдөр' %(year,datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%m'),datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%d')), ezxf('font:bold off;align:wrap off,vert centre,horiz right;font: height 200 '))

        sheet.write(3, 0, u'%s'%(company.name), ezxf('font:bold on;align:wrap off,vert centre,horiz left;'))
        sheet.write(4, 0, u'(Аж ахуй нэгжийн нэр)', ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet.write(6, 0, u'Мөрийн дугаар', header)
        sheet.write(6, 1, u'Үзүүлэлт', header)
        sheet.write(6, 2, u'Тайлант үеийн дүн', header)
        
        rowx = 7
        grid = self.create_report_xls_data(cr, uid, data, context)
         
        for row in grid:
            name = row[3]+row[1]
            name_style = styles['text_xf']
            if '<b>' in name:
                name_style = styles['text_bold_xf']
            name = name.replace('<font color="white">','').replace('</font>','')
            name = name.replace('<b>','').replace('</b>','')
            name = name.replace('_',' ')
            sheet.write(rowx, 0, row[0], styles['text_xf'])
            sheet.write(rowx, 1, name, name_style)
            sheet.write(rowx, 2, row[2], styles['number_xf'])
            rowx += 1

        rowx = 35
        inch = 1000
        sheet.col(0).width = 3*inch
        sheet.col(1).width = 16*inch
        sheet.col(2).width = 4*inch
        sheet.col(3).width = 4*inch
        sheet.row(0).height = 500
        sheet.row(1).height = 500
        sheet.row(6).height = 300
        sheet.row(7).height = 300
        sheet.row(6).height = 600
                
        sheet.write_merge(rowx+2, rowx+2, 1, 2, u'Захирал_______________________________________/                                           /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet.write_merge(rowx+4, rowx+4, 1, 2, u'Ерөнхий нягтлан бодогч ____________________________________/                                              /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        
        return {'data':book}

account_profit_report()