# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

import time
from lxml import etree
from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import datetime
from openerp import netsvc
import xlwt
from xlwt import Workbook, Formula

class account_property_report(osv.osv_memory):
    """
        Санхүүгийн тодруулга
    """
    _inherit = "abstract.report.excel"
    _name = "account.property.report"
    _description = "Account property report"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'filter': fields.selection([('filter_date', 'Date'), ('filter_period', 'Periods')], "Filter by", required=True),
        'period_from': fields.many2one('account.period', 'Start Period'),
        'period_to': fields.many2one('account.period', 'End Period'),
        'date_from': fields.date("Start Date"),
        'date_to': fields.date("End Date"),
        'target_move': fields.selection([('posted', 'All Posted Entries'),
                                         ('all', 'All Entries'),
                                        ], 'Target Moves', required=True),
        'level': fields.selection([('3','3'),('4','4'),('5','5'),('6','6')], 'Account Level', required=True),
        'chart_account_id': fields.many2one('account.account', 'Chart of Account', help='Select Charts of Accounts', required=True, domain=[('parent_id','=',False)]),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True),
    }
    
    def default_chart_account(self, cr, uid, context=None):
        account_ids = self.pool.get('account.account').search(cr, uid, [('parent_id','=',False)], context=context)
        return (account_ids and account_ids[0]) or False
    
    _defaults = {
        'company_id': lambda obj,cr,uid,c=None:obj.pool.get('res.company')._company_default_get(cr,uid,'account.balance.sheet',context=c),
        'filter': 'filter_date',
        'level': '4',
        'fiscalyear_id': lambda obj,cr,uid,c=None:obj.pool.get('account.fiscalyear').find(cr, uid, context=c),
        'target_move': 'posted',
        'chart_account_id': default_chart_account
    }
    
    def onchange_filter(self, cr, uid, ids, filter='filter_date', fiscalyear_id=False, context=None):
        res = {'value': {}}
        if filter == 'filter_date':
            res['value'] = {'period_from': False, 'period_to': False, 'date_from': time.strftime('%Y-%m-01'), 'date_to': time.strftime('%Y-%m-%d')}
        elif filter == 'filter_period':
            start_period = end_period = False
            cr.execute('''
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_start ASC, p.special ASC
                               LIMIT 1) AS period_start
                UNION ALL
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_stop DESC
                               LIMIT 1) AS period_stop''', (fiscalyear_id, fiscalyear_id))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
            res['value'] = {'period_from': start_period, 'period_to': end_period, 'date_from': False, 'date_to': False}
        return res
    
    def onchange_fiscalyear(self, cr, uid, ids, fiscalyear, filter, context=None):
        start_period = end_period = False
        if filter == 'filter_period':
            cr.execute('''
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_start ASC, p.special ASC
                               LIMIT 1) AS period_start
                UNION ALL
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_stop DESC
                               LIMIT 1) AS period_stop''', (fiscalyear, fiscalyear))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
        return {'value':{'period_from':start_period,'period_to':end_period}}
    
    def prepare_data(self, cr, uid, ids, context=None):
        fy_obj = self.pool.get('account.fiscalyear')
        period_obj = self.pool.get('account.period')
        obj = self.browse(cr, uid, ids[0], context=context)
        
        data = {
            'ids': ids,
            'model': 'account.balance.sheet',
            'form': {
                'company_id': obj.company_id.id,
                'filter': obj.filter,
                'period_from': obj.period_from.id,
                'period_to': obj.period_to.id,
                'date_from': obj.date_from,
                'date_to': obj.date_to,
                'target_move': obj.target_move,
                'level': obj.level,
                'chart_account_id': obj.chart_account_id.id,
                'fiscalyear_id': obj.fiscalyear_id.id,
                'save': obj.save
            },
            'report_name': 'account.balance.sheet'
        }
        
        if data['form']['filter'] == 'filter_date':
            fy = fy_obj.read(cr, uid, data['form']['fiscalyear_id'], ['date_start','date_stop'])
            if data['form']['date_from'] < fy['date_start'] or data['form']['date_to'] > fy['date_stop'] \
                or data['form']['date_from'] > data['form']['date_to']:
                raise osv.except_osv(_('Error'),_('Select a valid date for fiscalyear.'))
        elif data['form']['filter'] == 'filter_period':
            if not data['form']['period_from'] or not data['form']['period_to']:
                raise osv.except_osv(_('Error'),_('Select a starting and an ending period'))
            data['form']['date_from'] = period_obj.read(cr, uid, data['form']['period_from'], ['date_start'])['date_start']
            data['form']['date_to'] = period_obj.read(cr, uid, data['form']['period_to'], ['date_stop'])['date_stop']
        return data
    
    def get_log_message(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        
        date_from = (obj.period_from and obj.period_from.date_start) or obj.date_from
        date_to = (obj.period_to and obj.period_to.date_stop) or obj.date_to
        body = (u"Балансын тайлан (Эхлэх='%s', Дуусах='%s', Мод='%s')") % \
            (date_from, date_to, obj.chart_account_id.name)
        return body
    
    def get_export_data(self, cr, uid, ids, context=None):
        ''' Тайлангийн загварыг боловсруулж өгөгдлүүдийг
            тооцоолж байрлуулна.
        '''
        def name_padding(param):
            l = param['level'] - 2
            return ('  ' * l) + param['name']
        
        if context is None:
            context = {}
        datas = self.prepare_data(cr, uid, ids, context=context)
        
        #report_service = netsvc.LocalService('report.bbsb.account.balance.sheet')
       # report_service.cr = cr
        #report_service.uid = uid
        #report_service.datas = datas
      #  report_service.context = context
       # report_service.pool = self.pool
        #report_service.lang = context.get('lang', 'mn_MN')
        data = datas['form']
        
        period_obj = self.pool.get('account.period')
        account_line_obj = self.pool.get('account.move.line')
        account_obj = self.pool.get('account.account')
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        )
        obj = self.browse(cr, uid, ids[0], context=context)
        year = obj.fiscalyear_id.name
        document_code = ''
        directory_name = False
      #  if data['save']:
       #     generate_barcode = report_service.get_report_barcode(cr, context=context)
        #    document_code = report_service.code
        
        ezxf = xlwt.easyxf
        styles = self.get_easyxf_styles()
        
        
        self.context = context
        ctx = self.context.copy()
        ctx1 = self.context.copy()
        ctx['fiscalyear'] = data['fiscalyear_id']
        if data['filter'] == 'filter_period':
            ctx1['period_from'] = data['period_from']
            ctx1['period_to'] = data['period_from']
            ctx['period_from'] = data['period_from']
            ctx['period_to'] = data['period_to']
        elif data['filter'] == 'filter_date':
            ctx1['date_from'] = data['date_from']
            ctx1['date_to'] = data['date_from']
            ctx['date_from'] = data['date_from']
            ctx['date_to'] =  data['date_to']
        ctx['state'] = data['target_move']
        
        ctx2 = self.context.copy()
        ctx3 = self.context.copy()
        ctx2['date_from'] = time.strftime("%Y-01-01")
        ctx2['date_to'] = time.strftime("%Y-01-01")
        
        ctx3['date_from'] = time.strftime("%Y-01-01")
        ctx3['date_to'] = data['date_to']
        
        header = ezxf('font: bold on, height 180; align: wrap on, vert centre, horiz center; borders: top thin, left thin, bottom thin, right thin; pattern: pattern solid, fore_colour light_yellow')
        styled = ezxf('font: bold off; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        styless = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        style = ezxf('font: bold off; align: wrap on, vert centre, horiz center;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        data_styless = ezxf('font: bold off; align: wrap on, vert centre, horiz right;font: height 150;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        stylex = ezxf('font:bold off;align:wrap off,vert centre,horiz left;','#,##0.00_);(#,##0.00)')
        sheet_style = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet_style_min = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 160;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet_style1 = ezxf('font: bold on; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        
        sheet_style3 = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        
        sheet4_style = ezxf('font: bold on; align: wrap on, vert centre, horiz centre;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        
        sheet4_style1 = ezxf('font: bold on; align: wrap on, vert centre, horiz centre;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        
        sheet4_style2 = ezxf('font: bold on; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        
        sheet4_style3 = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        
        sheet4_style4 = ezxf('font: bold on; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')        

        datas_styles = ezxf('font: bold off; align: wrap on, vert centre, horiz centre;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        datas_style = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')



        book = xlwt.Workbook(encoding='utf8')
        sheet1 = book.add_sheet(u'page1')
        sheet2 = book.add_sheet(u'page2')
        sheet3 = book.add_sheet(u'page3')
        sheet4 = book.add_sheet(u'page4')
        sheet4.portrait=False
        sheet5 = book.add_sheet(u'page5')
        sheet5.portrait=False
        sheet6 = book.add_sheet(u'page6')
        sheet7 = book.add_sheet(u'page7')
        sheet8 = book.add_sheet(u'page8')
        sheet9 = book.add_sheet(u'page9')
        sheet10 = book.add_sheet(u'page10')
        sheet10.portrait=False
        


        date_to_obj = datetime.strptime(data['date_to'],'%Y-%m-%d')
         
        
        title_style = ezxf('font:bold on;align:wrap off,vert centre,horiz centre;font: height 220')
        title_style2 = ezxf('font:bold on;align:wrap on,vert centre,horiz centre;font: height 220')
        simple_style = ezxf('font:bold off;align:wrap off,vert centre,horiz left;font: height 200')
        simple_style2 = ezxf('font:bold off;align:wrap on,vert centre,horiz left;font: height 200')
        
        '''----------------------------- Санхүүгийн тайлангийн тодруулга ---------------------'''
        
        sheet1.write_merge(0, 0, 2, 6, u'САНХҮҮГИЙН ТАЙЛАНГИЙН ТОДРУУЛГА', title_style)
        sheet1.write_merge(1, 1, 0, 3, u'%s'%company.name, simple_style)
        sheet1.write_merge(2, 2, 0, 3, u'(Аж ахуй нэгжийн нэр)', simple_style)
        sheet1.write(1, 6, u'%s оны'%time.strftime("%Y"), simple_style)
        sheet1.write(1, 7, u'%s сарын'%time.strftime("%m"), simple_style)
        sheet1.write(1, 8, u'%s өдөр'%time.strftime("%d"), simple_style)
        sheet1.write_merge(5, 5, 0, 4, u'Үндсэн үйл ажиллагааны чиглэл/төрөл/:', simple_style)
        sheet1.write_merge(6, 6, 0, 7, u'(a)______________________________________________________________________________________', simple_style)
        sheet1.write_merge(7, 7, 0, 7, u'(б)______________________________________________________________________________________', simple_style)
        sheet1.write_merge(8, 8, 0, 7, u'(в)______________________________________________________________________________________', simple_style)
        sheet1.write_merge(10, 10, 0, 4, u'Туслах үйл ажиллагааны чиглэл/төрөл/:', simple_style)
        sheet1.write_merge(11, 11, 0, 4, u'(a)__________________________________________', simple_style)
        sheet1.write_merge(12, 12, 0, 4, u'(в)__________________________________________', simple_style)
        sheet1.write_merge(11, 11, 5, 6, u'(б)__________________________________________', simple_style)
        sheet1.write_merge(14, 14, 0, 4, u'Салбар, төлөөлөгчийн газрын нэр, байршил', simple_style)
        sheet1.write_merge(15, 15, 0, 7, u'(a)______________________________________________________________________________________', simple_style)
        sheet1.write_merge(16, 16, 0, 7, u'(б)______________________________________________________________________________________', simple_style)
        sheet1.write_merge(17, 17, 0, 7, u'(в)______________________________________________________________________________________', simple_style)
        
        '''----------------------------- Тайлан бэлтгэх үндэслэл ---------------------'''
        
        sheet1.write_merge(19, 19, 2, 6, u'1.ТАЙЛАН БЭЛТГЭХ ҮНДЭСЛЭЛ', title_style)
        row = 20
        for i in range(1,13):
            sheet1.write_merge(row, row, 0, 7, u'..............................................................................................................................................................................................', simple_style)
            row += 1

        '''----------------------------- НЯГТЛАН БОДОХ БҮРТГЭЛИЙН БОДЛОГЫН ӨӨРЧЛӨЛТ ---------------------'''
        
        sheet1.write_merge(row+2, row+2, 2, 6, u'2.НЯГТЛАН БОДОХ БҮРТГЭЛИЙН БОДЛОГЫН ӨӨРЧЛӨЛТ', title_style)
        row += 3
        for i in range(1,13):
            sheet1.write_merge(row, row, 0, 7, u'..............................................................................................................................................................................................', simple_style)
            row += 1
        
        '''----------------------------- МӨНГӨ, ТҮҮНТЭЙ АДИЛТГАХ ХӨРӨНГӨ ---------------------'''
        sum_balance = 0
        sum_balance2 = 0
        sum_cass_balance1 = 0
        sum_bank_balance1 = 0
        
        account_id = account_obj.search(cr, uid, [('code','=','10011')])#kassd bga belen mongo togrog
        account_data = account_obj.browse(cr, uid, account_id, ctx2)[0]
        sum_balance += account_data.balance
        account_id = account_obj.search(cr, uid, [('code','=','10021')])#kassd bga belen mongo walut
        account_data = account_obj.browse(cr, uid, account_id, ctx2)[0]
        sum_balance += account_data.balance
        
        bnk_id = account_obj.search(cr, uid, [('code','=','11011')])#bankind bga belen mongo togrog
        bnk_data = account_obj.browse(cr, uid, bnk_id, ctx2)[0]
        sum_balance2 += bnk_data.balance
        bnk_id = account_obj.search(cr, uid, [('code','=','11021')])#bankind bga belen mongo walut
        bnk_data = account_obj.browse(cr, uid, bnk_id, ctx2)[0]
        sum_balance2 += bnk_data.balance
        
        
        account_id = account_obj.search(cr, uid, [('code','=','10011')])
        account_data = account_obj.browse(cr, uid, account_id, ctx3)[0]
        sum_cass_balance1 += account_data.balance
        account_id = account_obj.search(cr, uid, [('code','=','10021')])
        account_data = account_obj.browse(cr, uid, account_id, ctx3)[0]
        sum_cass_balance1 += account_data.balance
        
        bnk_id = account_obj.search(cr, uid, [('code','=','11011')])
        bnk_data = account_obj.browse(cr, uid, bnk_id, ctx3)[0]
        sum_bank_balance1 += bnk_data.balance
        bnk_id = account_obj.search(cr, uid, [('code','=','11021')])
        bnk_data = account_obj.browse(cr, uid, bnk_id, ctx3)[0]
        sum_bank_balance1 += bnk_data.balance
        
        sheet2.write_merge(0, 0, 2, 4, u'3.МӨНГӨ, ТҮҮНТЭЙ АДИЛТГАХ ХӨРӨНГӨ', title_style)
        sheet2.write_merge(2, 2, 0, 2, u'Мөнгөн хөрөнгийн зүйлс', sheet_style)
        sheet2.write_merge(3, 3, 0, 2, u'Касс дахь мөнгө', sheet_style)
        sheet2.write_merge(4, 4, 0, 2, u'Банкин дахь мөнгө', sheet_style)
        sheet2.write_merge(5, 5, 0, 2, u'Мөнгөтэй адилтгах хөрөнгө', sheet_style)
        sheet2.write_merge(6, 6, 0, 2, u'Нийт дүн', sheet_style)
        sheet2.write_merge(2, 2, 3, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet2.write_merge(3, 3, 3, 3, u'%s'%sum_balance, sheet_style)
        sheet2.write_merge(4, 4, 3, 3, u'%s'%sum_balance2, sheet_style)
        sheet2.write_merge(5, 5, 3, 3, u'', sheet_style)
        sheet2.write_merge(6, 6, 3, 3, u'%s'%(sum_balance+sum_balance2), sheet_style)
        sheet2.write_merge(2, 2, 4, 4, u'Эцсийн үлдэгдэл', sheet_style)
        sheet2.write_merge(3, 3, 4, 4, u'%s'%sum_cass_balance1, sheet_style)
        sheet2.write_merge(4, 4, 4, 4, u'%s'%sum_bank_balance1, sheet_style)
        sheet2.write_merge(5, 5, 4, 4, u'', sheet_style)
        sheet2.write_merge(6, 6, 4, 4, u'%s'%(sum_cass_balance1+sum_bank_balance1), sheet_style)
        sheet2.write_merge(7, 7, 0, 5, u'Тэмдэглэл.(Мөнгө, түүнтэй адилтгах хөрөнгөтэй холбоотой тайлбар, тэмдэглэлийг хийнэ).', simple_style2)
        sheet2.write_merge(8, 8, 0, 5, u'__________________________________________________________________________________________', simple_style)
        sheet2.write_merge(9, 9, 0, 5, u'__________________________________________________________________________________________', simple_style)
        sheet2.write_merge(10, 10, 0, 5, u'__________________________________________________________________________________________', simple_style)
        
        '''----------------------------- ДАНСНЫ БОЛОН БУСАД АВЛАГА ---------------------'''
        
        sheet2.write_merge(11, 11, 2, 4, u'4.ДАНСНЫ БОЛОН БУСАД АВЛАГА', title_style)
        sheet2.write(12, 0, u'4.1 Дансны авлага', simple_style)
        sheet2.write(13, 0, u'№', sheet_style)
        sheet2.write(14, 0, u'1', sheet_style)
        sheet2.write(15, 0, u'2', sheet_style)
        sheet2.write(16, 0, u'3', sheet_style)
        sheet2.write_merge(17, 18, 0, 0, u'', sheet_style)
        sheet2.write(19, 0, u'4', sheet_style)
        sheet2.write(13, 1, u'Үзүүлэлт', sheet_style)
        sheet2.write(14, 1, u'Эхний үлдэгдэл', sheet_style)
        sheet2.write(15, 1, u'Нэмэгдсэн', sheet_style)
        sheet2.write(16, 1, u'Хасагдсан(-):', sheet_style)
        sheet2.write(17, 1, u'-Төлөгдсөн', sheet_style)
        sheet2.write(18, 1, u'-Найдваргүй болсон', sheet_style)
        sheet2.write(19, 1, u'Эцсийн үлдэгдэл', sheet_style)
        sheet2.write(13, 2, u'Дансны авлага', sheet_style)
        sheet2.write(14, 2, u'', sheet_style)
        sheet2.write(15, 2, u'', sheet_style)
        sheet2.write(16, 2, u'', sheet_style)
        sheet2.write(17, 2, u'', sheet_style)
        sheet2.write(18, 2, u'', sheet_style)
        sheet2.write(19, 2, u'', sheet_style)
        sheet2.write_merge(13, 13, 3, 4, u'Найдваргүй авлагын хасагдуулга', sheet_style)
        sheet2.write_merge(14, 14, 3, 4, u'', sheet_style)
        sheet2.write_merge(15, 15, 3, 4, u'', sheet_style)
        sheet2.write_merge(16, 16, 3, 4, u'', sheet_style)
        sheet2.write_merge(17, 17, 3, 4, u'', sheet_style)
        sheet2.write_merge(18, 18, 3, 4, u'', sheet_style)
        sheet2.write_merge(19, 19, 3, 4, u'', sheet_style)
        sheet2.write_merge(13, 13, 5, 6, u'Дансны авлага (цэвэр дүнгээр)', sheet_style)
        sheet2.write_merge(14, 14, 5, 6, u'', sheet_style)
        sheet2.write_merge(15, 15, 5, 6, u'', sheet_style)
        sheet2.write_merge(16, 16, 5, 6, u'', sheet_style)
        sheet2.write_merge(17, 17, 5, 6, u'', sheet_style)
        sheet2.write_merge(18, 18, 5, 6, u'', sheet_style)
        sheet2.write_merge(19, 19, 5, 6, u'', sheet_style)
        
        sum_balance = 0
        account_id = account_obj.search(cr, uid, [('parent_id','=','13')])
        
        sheet2.write_merge(21, 21, 0, 4, u'4.2 Татвар, нийгмийн даатгалын шимтгэл(НДШ)-ийн авлага', simple_style)
        sheet2.write(22, 0, u'№', sheet_style)
        sheet2.write_merge(22, 22, 1, 2, u'Төрөл', sheet_style)
        row = 23
        num = 1
        for line in account_obj.browse(cr, uid, account_id, ctx2):
            sheet2.write(row, 0, u'%d'%num, sheet_style)
            sheet2.write_merge(row, row, 1, 2, u'%s'%line.name, sheet_style)
            sheet2.write(row, 3, u'%s'%line.balance, sheet_style)
            sum_balance += line.balance
            row+=1
            num+=1
        sheet2.write(row, 3, u'%s'%sum_balance, sheet_style)
        sheet2.write(22, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet2.write(22, 4, u'Эцсийн үлдэгдэл', sheet_style)
        row = 23
        sum_balance = 0
        for line in account_obj.browse(cr, uid, account_id, ctx3):
            sheet2.write(row, 4, u'%d'%line.balance, sheet_style)
            sum_balance += line.balance
            row+=1
        sheet2.write(row, 0, u'%d'%num, sheet_style)
        sheet2.write_merge(row, row, 1, 2, u'Нийт дүн', sheet_style)
        sheet2.write(row, 4, u'%s'%sum_balance, sheet_style)    
        
        
        sheet2.write_merge(29, 29, 0, 3, u'4.3 Бусад богино хугацаат авлага(төрлөөр нь ангилна)', simple_style)
        sheet2.write(30, 0, u'№', sheet_style)
        sheet2.write(31, 0, u'1', sheet_style)
        sheet2.write(32, 0, u'2', sheet_style)
        sheet2.write(33, 0, u'3', sheet_style)
        sheet2.write(34, 0, u'4', sheet_style)
        sheet2.write(35, 0, u'5', sheet_style)
        sheet2.write(36, 0, u'6', sheet_style)
        sheet2.write(37, 0, u'7', sheet_style)
        sheet2.write(38, 0, u'8', sheet_style)
        sheet2.write_merge(30, 30, 1, 3, u'Төрөл', sheet_style)
        sheet2.write_merge(31, 31, 1, 3, u'Холбоотой талаас авах авлага(эргэлтийн хөрөнгөнд хамаарах дүн)', sheet_style)
        sheet2.write_merge(32, 32, 1, 3, u'Ажилчдаас авах авлага', sheet_style)
        sheet2.write_merge(33, 33, 1, 3, u'Ноогдол ашгийн авлага', sheet_style)
        sheet2.write_merge(34, 34, 1, 3, u'Хүүний авлага', sheet_style)
        sheet2.write_merge(35, 35, 1, 3, u'Богино хугацаат авлагын бичиг', sheet_style)
        sheet2.write_merge(36, 36, 1, 3, u'Бусад талуудаас авах авлага', sheet_style)
        sheet2.write_merge(37, 37, 1, 3, u'', sheet_style)
        sheet2.write_merge(38, 38, 1, 3, u'Нийт дүн', sheet_style)
        sheet2.write(30, 4, u'Эхний үлдэгдэл', sheet_style)
        sheet2.write(31, 4, u'', sheet_style)
        sheet2.write(32, 4, u'', sheet_style)
        sheet2.write(33, 4, u'', sheet_style)
        sheet2.write(34, 4, u'', sheet_style)
        sheet2.write(35, 4, u'', sheet_style)
        sheet2.write(36, 4, u'', sheet_style)
        sheet2.write(37, 4, u'', sheet_style)
        sheet2.write(38, 4, u'', sheet_style)
        sheet2.write(30, 5, u'Эцсийн үлдэгдэл', sheet_style)
        sheet2.write(31, 5, u'', sheet_style)
        sheet2.write(32, 5, u'', sheet_style)
        sheet2.write(33, 5, u'', sheet_style)
        sheet2.write(34, 5, u'', sheet_style)
        sheet2.write(35, 5, u'', sheet_style)
        sheet2.write(36, 5, u'', sheet_style)
        sheet2.write(37, 5, u'', sheet_style)
        sheet2.write(38, 5, u'', sheet_style)
        
        sheet2.write_merge(40, 40, 0, 7, u'Тэмдэглэл. (Дансны авлагыг төлөгдөх хугацаандаа байгаа, хугацаа хэтэрсэн, төлөгдөх найдваргүй гэж ангилна. Найдваргүй авлагын хасагдуулга байгуулсан арга, гадаад валютаар илэрхийлэгдсэн авлагын талаар болон бусапд тайлбар, тэмдэглэлийг хийнэ.)', simple_style2)  
        sheet2.write_merge(41, 41, 0, 6, u'__________________________________________________________________________________________', simple_style)
        sheet2.write_merge(42, 42, 0, 6, u'__________________________________________________________________________________________', simple_style)
        
        inch = 2000
        
        sheet2.col(0).width = 1 * inch
        sheet2.col(1).width = 2 * inch
        sheet2.col(2).width = 2 * inch
        sheet2.col(3).width = 2 * inch 
        
        row_inch = 200
        sheet2.row(40).height = 5 * row_inch
        sheet2.row(30).height = 3 * row_inch
        sheet2.row(31).height = 3 * row_inch
        sheet2.row(13).height = 3 * row_inch
        sheet2.row(17).height = 2 * row_inch
        sheet2.row(7).height = 3 * row_inch
        sheet2.row(18).height = 2 * row_inch
        sheet2.row(22).height = 2 * row_inch
        sheet2.row(2).height = 2 * row_inch
        
        
        '''----------------------------- БУСАД САНХҮҮГИЙН ХӨРӨНГӨ ---------------------'''
        sum_balance = 0
        account_id = account_obj.search(cr, uid, [('parent_id','=','15')])
        sheet3.write_merge(0, 0, 2, 4, u'5. БУСАД САНХҮҮГИЙН ХӨРӨНГӨ', title_style)
        sheet3.write(1, 0, u'№', sheet_style)
        sheet3.write_merge(1, 1, 1, 2, u'Төрөл', sheet_style)
        sheet3.write(1, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet3.write(1, 4, u'Эцсийн үлдэгдэл', sheet_style)
        if account_id != []:
            num = 1
            row = 2 
            for line in account_obj.browse(cr, uid, account_id, ctx2):
                sheet3.write(row, 0, u'%d'%num, sheet_style)
                sheet3.write_merge(row, row, 1, 2, u'%s'%line.name, sheet_style)
                sheet3.write(row, 3, u'%s'%line.balance, sheet_style)
                sum_balance += line.balance
                row +=1
                num +=1
            sheet3.write(row, 3, u'%s'%sum_balance, sheet_style)
            row = 2
            for line in account_obj.browse(cr, uid, account_id, ctx3):
                sheet3.write(row, 4, u'%s'%line.balance, sheet_style)
            sheet3.write_merge(row, row, 1, 2, u'Нийт дүн', sheet_style)
            sheet3.write(row, 5, u'%s'%sum_balance, sheet_style)
        
        '''----------------------------- БАРАА МАТЕРИАЛ -account.cashflow.statement.report--------------------'''
        
        sum_balance = 0
        account_id = account_obj.search(cr, uid, [('parent_id','=','16'),('level','=','5')])
        
        sheet3.write_merge(6, 6, 2, 4, u'6. БАРАА МАТЕРИАЛ', title_style)
        sheet3.write_merge(8, 9, 0, 0, u'№', sheet_style)
        sheet3.write(10, 0, u'1', sheet_style)
        sheet3.write(11, 0, u'2', sheet_style)
        sheet3.write(12, 0, u'3', sheet_style)
        sheet3.write(13, 0, u'4', sheet_style)
        sheet3.write(14, 0, u'5', sheet_style)
        sheet3.write(15, 0, u'6', sheet_style)
        sheet3.write(16, 0, u'7', sheet_style)
        sheet3.write(17, 0, u'7.1', sheet_style)
        sheet3.write(18, 0, u'7.2', sheet_style)
        sheet3.write_merge(8, 9, 1, 1, u'Үзүүлэлт', sheet_style)
        sheet3.write(10, 1, u'Эхний үлдэгдэл(өртгөөр)', sheet_style)
        sheet3.write(11, 1, u'Нэмэгдсэн дүн', sheet_style)
        sheet3.write(12, 1, u'Хасагдсан дүн (-)', sheet_style)
        sheet3.write(13, 1, u'Эцсийн үлдэгдэл(өртгөөр)', sheet_style)
        sheet3.write(14, 1, u'Үнийн бууралтын гарз(-)', sheet_style)
        sheet3.write(15, 1, u'Үнийн бууралтын буцаалт', sheet_style)
        sheet3.write(16, 1, u'Дансны цэвэр дүн', sheet_style)
        sheet3.write(17, 1, u'Эхний үлдэгдэл', sheet_style)
        sheet3.write(18, 1, u'Эцсийн үлдэгдэл', sheet_style)
        column = 2
        for line in account_obj.browse(cr, uid, account_id, ctx2):
            sheet3.write(9, column, u'%s'%line.name, sheet_style_min)
            column += 1
        sheet3.write_merge(8, 8, 2, column-1, u'Бараа материалын төрөл', sheet_style)
        sheet3.write_merge(8, 9, column, column, u'Нийт дүн', sheet_style_min)
        
        
        sheet3.write_merge(20, 20, 0, 6, u'*Дансны цэвэр дүнгийн эхний, эцсийн үлдэгдлийн нийт дүн нь санхүүгийн байдлын тайлан дахь бараа материалын дансны эхний, эцсийн үлдэгдлийн дүнтэй тэнцүү байна.', simple_style2)
        sheet3.write_merge(22, 22, 0, 6, u'Тэмдэглэл.(Бараа материалын өртгийг тодорхойлоход ашигласан арга, бараа материалын бүртгэлийн систем, өртөг болон цэвэр боломжит үнийн аль багыг сонгох аргын талаар тайлбар, тэмдэглэл хийнэ.)', simple_style2)
        sheet3.write_merge(23, 23, 0, 7, u'__________________________________________________________________________________________', simple_style)
        sheet3.write_merge(24, 24, 0, 7, u'__________________________________________________________________________________________', simple_style)
        sheet3.write_merge(25, 25, 0, 7, u'__________________________________________________________________________________________', simple_style)
        
        '''----------------------------- БОРЛУУЛАХ ЗОРИЛГООР ЭЗЭМШИЖ БУЙ ЭРГЭЛТИЙН БУС ХӨРӨНГӨ ---------------------'''
        
        sheet3.write_merge(27, 27, 2, 5, u'7. БОРЛУУЛАХ ЗОРИЛГООР ЭЗЭМШИЖ БУЙ ЭРГЭЛТИЙН БУС ХӨРӨНГӨ (ЭСВЭЛ БОРЛУУЛАХ БҮЛЭГ ХӨРӨНГӨ) БОЛОН ӨР ТӨЛБӨР', title_style2)
        sheet3.write_merge(28, 28, 0, 6, u'Тэмдэгдэл.(Борлуулах зорилгоор эзэмшиж буй эргэлтийн бус хөрөнгө (эсвэл борлуулах бүлэг хөрөнгө) болон өр төлбөрийн тодорхойлолт, хэмжилтийн суурь, борлуулалт хийгдсэн аль эсвэл хийгдэхэд хүргэсэн нөхцөл байдал, борлуулах арга, хугацаа, хүлээн зөвшөөрсөн олз ба гарз болон бусад тайлбар, тэмдэглэлийг хийнэ)', simple_style2)
        row = 29
        for i in range(1, 5):
            sheet3.write_merge(row, row, 0, 7, u'_____________________________________________________________________________________________________________________',simple_style)
            row += 1
        
        '''----------------------------- УРЬДЧИЛЖ ТӨЛСӨН ЗАРДАЛ/ТООЦОО ---------------------'''
        sum_balance = 0
        account_id = account_obj.search(cr, uid, [('parent_id','=','17')])
        
        sheet3.write_merge(34, 34, 2, 4, u'8. УРЬДЧИЛЖ ТӨЛСӨН ЗАРДАЛ/ТООЦОО', title_style2)
        sheet3.write(35, 0, u'№', sheet_style)
        sheet3.write_merge(35, 35, 1, 2, u'Төрөл', sheet_style)
        row = 36
        num = 1
        for line in account_obj.browse(cr, uid, account_id, ctx2):
            sheet3.write(row, 0, u'%d'%num, sheet_style)
            sheet3.write_merge(row, row, 1, 2, u'%s'%line.name, sheet_style)
            sheet3.write(row, 3, u'%s'%line.balance, sheet_style)
            sum_balance += line.balance
            row +=1
            num +=1
        sheet3.write(row, 0, u'%d'%num, sheet_style)
        sheet3.write_merge(row, row, 1, 2, u'Нийт дүн', sheet_style)
        sheet3.write(row, 3, u'%s'%sum_balance, sheet_style)
        sheet3.write(35, 3, u'Эхний үлдэгдэл', sheet_style)
        row = 36
        sum_balance = 0
        for line in account_obj.browse(cr, uid, account_id, ctx3):
            sheet3.write(row, 4, u'%s'%line.balance, sheet_style)
            sum_balance += line.balance
            row +=1
        sheet3.write(35, 4, u'Эцсийн үлдэгдэл', sheet_style)
        sheet3.write(row, 4, u'%s'%sum_balance, sheet_style)
        
        inch = 2000
        
        sheet3.col(0).width = 1 * inch
        sheet3.col(1).width = 2 * inch
        sheet3.col(2).width = 2 * inch
        sheet3.col(3).width = 2 * inch 
        sheet3.col(4).width = 2 * inch
        sheet3.col(5).width = 2 * inch
        sheet3.col(6).width = 1 * inch
        sheet3.col(7).width = 1 * inch
        
        row_inch = 200
        sheet3.row(1).height = 3 * row_inch
        sheet3.row(9).height = 6 * row_inch
        sheet3.row(10).height = 3 * row_inch
        sheet3.row(11).height = 3 * row_inch
        sheet3.row(12).height = 3 * row_inch
        sheet3.row(13).height = 3 * row_inch
        sheet3.row(14).height = 3 * row_inch
        sheet3.row(15).height = 3 * row_inch
        sheet3.row(20).height = 5 * row_inch
        sheet3.row(22).height = 5 * row_inch
        sheet3.row(27).height = 5 * row_inch
        sheet3.row(28).height = 5 * row_inch
        sheet3.row(38).height = 3 * row_inch
        sheet3.row(35).height = 3 * row_inch
        sheet3.row(34).height = 3 * row_inch
        
        '''----------------------------- ҮНДСЭН ХӨРӨНГӨ ---------------------'''
        sheet4.write_merge(0, 0, 2, 4, u'9. ҮНДСЭН ХӨРӨНГӨ', title_style)
        sheet4.write(1, 0, u'№', sheet_style)
        sheet4.write(2, 0, u'1', sheet_style)
        sheet4.write(3, 0, u'1.1', sheet_style)
        sheet4.write(4, 0, u'1.2', sheet_style)
        sheet4.write_merge(5, 8, 0, 0, u'', sheet_style)
        sheet4.write(9, 0, u'1.3', sheet_style)
        sheet4.write_merge(10, 13, 0, 0, u'', sheet_style)
        sheet4.write(14, 0, u'1.4', sheet_style)
        sheet4.write(15, 0, u'1.5', sheet_style)
        sheet4.write(16, 0, u'1.6', sheet_style)
        sheet4.write(17, 0, u'2', sheet_style)
        sheet4.write(18, 0, u'2.1', sheet_style)
        sheet4.write(19, 0, u'2.2', sheet_style)
        sheet4.write_merge(20, 22, 0, 0, u'', sheet_style)
        sheet4.write(23, 0, u'2.3', sheet_style)
        sheet4.write_merge(24, 26, 0, 0, u'', sheet_style)
        sheet4.write(27, 0, u'2.4', sheet_style)
        sheet4.write(28, 0, u'3', sheet_style)
        sheet4.write(29, 0, u'3.1', sheet_style)
        sheet4.write(30, 0, u'3.2', sheet_style)
        sheet4.write_merge(1, 1, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet4.write_merge(2, 2, 1, 2, u'ҮНДСЭН ХӨРӨНГӨ /ӨРТӨГ/', sheet_style1)
        sheet4.write_merge(3, 3, 1, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet4.write_merge(4, 4, 1, 2, u'Нэмэгдсэн дүн', sheet_style)
        sheet4.write_merge(5, 5, 1, 2, u'Өөрөө үйлдвэрлэсэн', sheet_style)
        sheet4.write_merge(6, 6, 1, 2, u'Худалдаж авсан', sheet_style)
        sheet4.write_merge(7, 7, 1, 2, u'Үнэ төлбөргүй авсан', sheet_style)
        sheet4.write_merge(8, 8, 1, 2, u'Дахин үнэлгээний нэмэгдэл', sheet_style)
        sheet4.write_merge(9, 9, 1, 2, u'Хасагдсан дүн(-)', sheet_style)
        sheet4.write_merge(10, 10, 1, 2, u'Худалдсан', sheet_style)
        sheet4.write_merge(11, 11, 1, 2, u'Үнэгүй шилжүүлсэн', sheet_style)
        sheet4.write_merge(12, 12, 1, 2, u'Акталсан', sheet_style)
        sheet4.write_merge(13, 13, 1, 2, u'', sheet_style)
        sheet4.write_merge(14, 14, 1, 2, u'Үндсэн хөрөнгө дахин ангилсан', sheet_style)
        sheet4.write_merge(15, 15, 1, 2, u'Үндсэн хөрөнгө, ХОЗҮХХ хооронд дахин ангилсан', sheet_style)
        sheet4.write_merge(16, 16, 1, 2, u'Эцсийн үлдэгдэл', sheet_style)
        sheet4.write_merge(17, 17, 1, 2, u'ХУРИМТЛАГДСАН ЭЛЭГДЭЛ', sheet_style1)
        sheet4.write_merge(18, 18, 1, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet4.write_merge(19, 19, 1, 2, u'Нэмэгдсэн дүн', sheet_style)
        sheet4.write_merge(20, 20, 1, 2, u'Байгуулсан элэгдэл', sheet_style)
        sheet4.write_merge(21, 21, 1, 2, u'Дахин үнэлгээгээр нэмэгдсэн', sheet_style)
        sheet4.write_merge(22, 22, 1, 2, u'Үнэ цэнийн бууралтын буцаалт', sheet_style)
        sheet4.write_merge(23, 23, 1, 2, u'Хасагдсан дүн', sheet_style)
        sheet4.write_merge(24, 24, 1, 2, u'Данснаас хассан хөрөнгийн элэгдэл', sheet_style)
        sheet4.write_merge(25, 25, 1, 2, u'Дахин үнэлгээгээр хасагдсан', sheet_style)
        sheet4.write_merge(26, 26, 1, 2, u'Үнэ цэнийн бууралт', sheet_style)
        sheet4.write_merge(27, 27, 1, 2, u'Эцсийн үлдэгдэл', sheet_style)
        sheet4.write_merge(28, 28, 1, 2, u'ДАНСНЫ ЦЭВЭР ДҮН', sheet_style1)
        sheet4.write_merge(29, 29, 1, 2, u'Эхний үлдэгдэл (1.1-2.1)', sheet_style)
        sheet4.write_merge(30, 30, 1, 2, u'Эцсийн үлдэгдэл (1.6-2.4)', sheet_style)
        
        sheet4.write(1, 3, u'Газрын сайжруулалт', sheet_style_min)
        sheet4.write(1, 4, u'Барилга, байгууламж', sheet_style_min)
        sheet4.write(1, 5, u'Машин, тоног төхөөрөмж', sheet_style_min)
        sheet4.write(1, 6, u'Тээврийн хэрэгсэл', sheet_style_min)
        sheet4.write(1, 7, u'Тавилга эд хогшил', sheet_style_min)
        sheet4.write(1, 8, u'Комьютер, бусад хэрэгсэл', sheet_style_min)
        sheet4.write(1, 9, u'Бусад үндсэн хөрөнгө', sheet_style_min)
        sheet4.write(1, 10, u'Нийт дүн', sheet_style_min)
        sheet4.write_merge(31, 31, 0, 10, u'Тэмдэглэл.(Үндсэн хөрөнгийн анги бүрийн хувьд ашигласан хэмжилтийн суурь; элэгдэл тооцох арга; ашиглалтын хугацаа; дахин үнэлсэн бол дахин үнэлгээ хүчинтэй болсон хугацаа, хараат бус үнэлгээчин үнэлсэн эсэх талаар; үндсэн хөрөнгийн дахин ангилал, түүний шалтгаан; бусад тайлбар тэмдэглэлийг хийнэ.)', simple_style2)
        row = 32
        for i in range(1, 4):
            sheet4.write_merge(row, row, 0, 10, u'_____________________________________________________________________________________________________________________',simple_style)
            row += 1
        
        
        inch = 2000
        
        sheet4.col(0).width = 1 * inch
        sheet4.col(1).width = 2 * inch
        sheet4.col(2).width = 2 * inch
        sheet4.col(3).width = 2 * inch 
        sheet4.col(4).width = 1 * inch
        sheet4.col(5).width = 2 * inch
        sheet4.col(6).width = 1 * inch
        sheet4.col(7).width = 2 * inch
        sheet4.col(8).width = 2 * inch
        sheet4.col(9).width = 2 * inch
        sheet4.col(10).width = 1 * inch
        
        row_inch = 250
        sheet4.row(1).height = 2 * row_inch
        sheet4.row(15).height = 2 * row_inch
        sheet4.row(31).height = 4 * row_inch
        sheet4.row(24).height = 2 * row_inch
        
        '''----------------------------- БИЕТ БУС ХӨРӨНГӨ ---------------------'''
        sheet5.write_merge(0, 0, 2, 4, u'10. БИЕТ БУС ХӨРӨНГӨ', title_style)
        sheet5.write(1, 0, u'№', sheet_style)
        sheet5.write(2, 0, u'1', sheet_style)
        sheet5.write(3, 0, u'1.1', sheet_style)
        sheet5.write(4, 0, u'1.2', sheet_style)
        sheet5.write_merge(5, 8, 0, 0, u'', sheet_style)
        sheet5.write(9, 0, u'1.3', sheet_style)
        sheet5.write_merge(10, 13, 0, 0, u'', sheet_style)
        sheet5.write(14, 0, u'1.4', sheet_style)
        sheet5.write(15, 0, u'2', sheet_style)
        sheet5.write(16, 0, u'2.1', sheet_style)
        sheet5.write(17, 0, u'2.2', sheet_style)
        sheet5.write_merge(18, 20, 0, 0, u'', sheet_style)
        sheet5.write(21, 0, u'2.3', sheet_style)
        sheet5.write_merge(22, 24, 0, 0, u'', sheet_style)
        sheet5.write(25, 0, u'2.4', sheet_style)
        sheet5.write(26, 0, u'3', sheet_style)
        sheet5.write(27, 0, u'3.1', sheet_style)
        sheet5.write(28, 0, u'3.2', sheet_style)
        sheet5.write_merge(1, 1, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet5.write_merge(2, 2, 1, 2, u'БИЕТ БУС ХӨРӨНГӨ /ӨРТӨГ/', sheet_style1)
        sheet5.write_merge(3, 3, 1, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet5.write_merge(4, 4, 1, 2, u'Нэмэгдсэн дүн', sheet_style)
        sheet5.write_merge(5, 5, 1, 2, u'Өөрөө үйлдвэрлэсэн', sheet_style)
        sheet5.write_merge(6, 6, 1, 2, u'Худалдаж авсан', sheet_style)
        sheet5.write_merge(7, 7, 1, 2, u'Үнэ төлбөргүй авсан', sheet_style)
        sheet5.write_merge(8, 8, 1, 2, u'Дахин үнэлгээний нэмэгдэл', sheet_style)
        sheet5.write_merge(9, 9, 1, 2, u'Хасагдсан дүн(-)', sheet_style)
        sheet5.write_merge(10, 10, 1, 2, u'Худалдсан', sheet_style)
        sheet5.write_merge(11, 11, 1, 2, u'Үнэгүй шилжүүлсэн', sheet_style)
        sheet5.write_merge(12, 12, 1, 2, u'Акталсан', sheet_style)
        sheet5.write_merge(13, 13, 1, 2, u'', sheet_style)
        sheet5.write_merge(14, 14, 1, 2, u'Эцсийн үлдэгдэл', sheet_style)
        sheet5.write_merge(15, 15, 1, 2, u'ХУРИМТЛАГДСАН ХОРОГДОЛ', sheet_style1)
        sheet5.write_merge(16, 16, 1, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet5.write_merge(17, 17, 1, 2, u'Нэмэгдсэн дүн', sheet_style)
        sheet5.write_merge(18, 18, 1, 2, u'Байгуулсан хорогдол', sheet_style)
        sheet5.write_merge(19, 19, 1, 2, u'Дахин үнэлгээгээр нэмэгдсэн', sheet_style)
        sheet5.write_merge(20, 20, 1, 2, u'Үнэ цэнийн бууралтын буцаалт', sheet_style)
        sheet5.write_merge(21, 21, 1, 2, u'Хасагдсан:', sheet_style)
        sheet5.write_merge(22, 22, 1, 2, u'Данснаас хассан хөрөнгийн хорогдол', sheet_style)
        sheet5.write_merge(23, 23, 1, 2, u'Дахин үнэлгээгээр хасагдсан', sheet_style)
        sheet5.write_merge(24, 24, 1, 2, u'Үнэ цэнийн бууралт', sheet_style)
        sheet5.write_merge(25, 25, 1, 2, u'Эцсийн үлдэгдэл', sheet_style)
        sheet5.write_merge(26, 26, 1, 2, u'ДАНСНЫ ЦЭВЭР ДҮН', sheet_style1)
        sheet5.write_merge(27, 27, 1, 2, u'Эхний үлдэгдэл (1.1-2.1)', sheet_style)
        sheet5.write_merge(28, 28, 1, 2, u'Эцсийн үлдэгдэл (1.4-2.4)', sheet_style)
        sheet5.write(1, 3, u'Зохиогчийн эрх', sheet_style)
        sheet5.write(1, 4, u'Компьютерийн програм хангамж', sheet_style)
        sheet5.write(1, 5, u'Патент', sheet_style)
        sheet5.write(1, 6, u'Барааны тэмдэг', sheet_style)
        sheet5.write(1, 7, u'Тусгай зөвшөөрөл', sheet_style)
        sheet5.write(1, 8, u'Газар эзэмших эрх', sheet_style)
        sheet5.write(1, 9, u'Бусад биет бус хөрөнгө', sheet_style)
        sheet5.write(1, 10, u'Нийт дүн', sheet_style)
        sheet5.write_merge(29, 29, 0, 10, u'Тэмдэглэл.(Биет бус хөрөнгийн анги бүрийн хувьд ашигласан хэмжилтийн суурь, хорогдол тооцох арга, ашиглалтын хугацаа дахин үнэлсэн бол дахин үнэлгээ хүчинтэй болсон хугацаа, хараат бус үнэлгээчин үнэлсэн эсэх, бусад биет бус хөрөнгийн бүрэлдэхүүн болон бусад тайлбар тэмдэглэлийг хийнэ.)', simple_style2)
        row = 30
        for i in range(1, 4):
            sheet5.write_merge(row, row, 0, 10, u'_____________________________________________________________________________________________________________________',simple_style)
            row += 1
        
        
        inch = 2000
        
        sheet5.col(0).width = 1 * inch
        sheet5.col(1).width = 2 * inch
        sheet5.col(2).width = 2 * inch
        sheet5.col(3).width = 2 * inch 
        sheet5.col(4).width = 2 * inch
        sheet5.col(5).width = 2 * inch
        sheet5.col(6).width = 1 * inch
        sheet5.col(7).width = 2 * inch
        sheet5.col(8).width = 2 * inch
        sheet5.col(9).width = 2 * inch
        sheet5.col(10).width = 1 * inch
        
        row_inch = 250
        sheet5.row(1).height = 2 * row_inch
        sheet5.row(15).height = 2 * row_inch
        sheet5.row(29).height = 4 * row_inch
        sheet5.row(22).height = 2 * row_inch
        
        '''----------------------------- ДУУСААГҮЙ БАРИЛГА ---------------------'''
        sheet6.write_merge(0, 0, 2, 4, u'11. ДУУСААГҮЙ БАРИЛГА', title_style)
        sheet6.write(1, 0, u'№', sheet_style)
        sheet6.write(2, 0, u'1', sheet_style)
        sheet6.write(3, 0, u'2', sheet_style)
        sheet6.write(4, 0, u'3', sheet_style)
        sheet6.write_merge(1, 1, 1, 2, u'Дуусаагүй барилгын нэр', sheet_style)
        sheet6.write(1, 3, u'Эхэлсэн он', sheet_style)
        sheet6.write(1, 4, u'Дуусгалтын хувь', sheet_style)
        sheet6.write(1, 5, u'Нийт төсөвт өртөг', sheet_style)
        sheet6.write_merge(1, 1, 6, 7, u'Ашиглалтанд орох эцсийн хугацаа',sheet_style)
        sheet6.write_merge(4, 4, 1, 2, u'Нийт дүн', sheet_style)
        
        '''----------------------------- БИОЛОГИЙН ХӨРӨНГӨ ---------------------'''
        sheet6.write_merge(6, 6, 2, 4, u'12. БИОЛОГИЙН ХӨРӨНГӨ', title_style)
        sheet6.write_merge(7, 8, 0, 0, u'№', sheet_style)
        sheet6.write(9, 0, u'1', sheet_style)
        sheet6.write(10, 0, u'2', sheet_style)
        sheet6.write_merge(7, 8, 1, 2, u'Биологийн хөрөнгийн төрөл', sheet_style)
        sheet6.write_merge(10, 10, 1, 2, u'Нийт дүн', sheet_style)
        sheet6.write_merge(7, 7, 3, 4, u'Эхний үлдэгдэл', sheet_style)
        sheet6.write(8, 3, u'тоо', sheet_style)
        sheet6.write(8, 4, u'дансны үнэ', sheet_style)
        sheet6.write_merge(7, 7, 5, 6, u'Эцсийн үлдэгдэл', sheet_style)
        sheet6.write(8, 5, u'тоо', sheet_style)
        sheet6.write(8, 6, u'дансны үнэ', sheet_style)
        sheet6.write_merge(11, 11, 0, 5, u'Тэмдэглэл.(Биологийн хөрөнгийн хэмжилтийн суурь болон бусад тайлбар, тэмдэглэлийг хийнэ.)', simple_style2)
        row = 12
        for i in range(1, 3):
            sheet6.write_merge(row, row, 0, 6, u'_____________________________________________________________________________________________________________________',simple_style)
            row += 1
            
        '''----------------------------- УРТ ХУГАЦААТ ХӨРӨНГӨ ОРУУЛАЛТ ---------------------'''
        sheet6.write_merge(15, 15, 2, 4, u'13. УРТ ХУГАЦААТ ХӨРӨНГӨ ОРУУЛАЛТ', title_style)
        sheet6.write_merge(16, 17, 0, 0,  u'№', sheet_style)
        sheet6.write(18, 0, u'1', sheet_style)
        sheet6.write(19, 0, u'2', sheet_style)
        sheet6.write(20, 0, u'3', sheet_style)
        sheet6.write(21, 0, u'4', sheet_style)
        sheet6.write_merge(16, 17, 1, 2, u'Хөрөнгө оруулалтын төрөл', sheet_style)
        sheet6.write_merge(21, 21, 1, 2, u'Нийт дүн', sheet_style)
        sheet6.write_merge(16, 16, 3, 4, u'Эхний үлдэгдэл', sheet_style)
        sheet6.write_merge(16, 16, 5, 6, u'Эцсийн үлдэгдэл', sheet_style)
        sheet6.write(17, 3, u'Хөрөнгө оруулалтын хувь', sheet_style)
        sheet6.write(17, 4, u'Хөрөнгө оруулалтын дүн', sheet_style)
        sheet6.write(17, 5, u'Хөрөнгө оруулалтын хувь', sheet_style)
        sheet6.write(17, 6, u'Хөрөнгө оруулалтын дүн', sheet_style)
        
        sheet6.write_merge(22, 22, 0, 5, u'Тэмдэглэл.(Урт хугацаат хөрөнгө оруулалттай холбоотой бий болсон олз, газрын дүн, бүртгэсэн аргыг тодруулна. Охин компани, хамтын хяналттай аж ахуйн нэгж, хараат компанид оруулсан хөрөнгө оруулалтыг НББОУС 27 Нэгтгэсэн болон тусдаа санхүүгийн тайлан-ийн дагуу тодруулна.)', simple_style2)
        row = 23
        for i in range(1, 4):
            sheet6.write_merge(row, row, 0, 6, u'_____________________________________________________________________________________________________________________',simple_style)
            row += 1
        
        '''----------------------------- ХӨРӨНГӨ ОРУУЛАЛТЫН ЗОРИУЛАЛТТАЙ ҮЛ ХӨДЛӨХ ХӨРӨНГӨ ---------------------'''
        sheet6.write_merge(27, 27, 2, 5, u'14. ХӨРӨНГӨ ОРУУЛАЛТЫН ЗОРИУЛАЛТТАЙ ҮЛ ХӨДЛӨХ ХӨРӨНГӨ', title_style2)
        sheet6.write_merge(28, 28, 0, 7, u'Тэмдэглэл.(Хөрөнгө оруулалтын зориулалттай үл хөдлөх хөрөнгийн хувьд ашигласан хэмжилтийн суурь; бодит үнэ цэнийн загвар ашигладаг бол бодит үнэ цэнийг тодорхойлоход ашигласан арга, бодит үнэ цэнийн тохируулгаас үүссэн олз, гарз; хэрэв түрээслэдэг бол түрээсийн орлого, түрээслэсэн хөрөнгөтэй холбоотой гарсан зардлууд. Хэрэв өртгийн загвар ашигладаг бол хөрөнгийн ашиглалтын хугацаа, элэгдэл тооцох араг болон НББОУС 40 Хөрөнгө оруулалтын зориулалттай үл хөдлөх хөрөнгө - д заасны дагуу бусад тодруулгыг хийнэ.)', simple_style2)
        row = 29
        for i in range(1, 7):
            sheet6.write_merge(row, row, 0, 6, u'_____________________________________________________________________________________________________________________',simple_style)
            row += 1
            
        inch = 2000
        
        sheet6.col(0).width = 1 * inch
        sheet6.col(1).width = 2 * inch
        sheet6.col(2).width = 2 * inch
        sheet6.col(3).width = 2 * inch 
        sheet6.col(4).width = 2 * inch
        sheet6.col(5).width = 2 * inch
        
        row_inch = 250
        sheet6.row(1).height = 3 * row_inch
        sheet6.row(11).height = 3 * row_inch
        sheet6.row(22).height = 4 * row_inch
        sheet6.row(16).height = 2 * row_inch
        sheet6.row(17).height = 3 * row_inch
        sheet6.row(27).height = 2 * row_inch
        sheet6.row(28).height = 6 * row_inch
        
        '''----------------------------- БУСАД ЭРГЭЛТИЙН БУС ХӨРӨНГӨ ---------------------'''
        sheet7.write_merge(0, 0, 2, 4, u'15. БУСАД ЭРГЭЛТИЙН БУС ХӨРӨНГӨ', title_style)
        sheet7.write(1, 0, u'№', sheet_style)
        sheet7.write(2, 0, u'1', sheet_style)
        sheet7.write(3, 0, u'2', sheet_style)
        sheet7.write(4, 0, u'3', sheet_style)
        sheet7.write_merge(1, 1, 1, 2, u'Төрөл', sheet_style)
        sheet7.write_merge(4, 4, 1, 2, u'Нийт дүн', sheet_style)
        sheet7.write(1, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(1, 4, u'Эцсийн үлдэгдэл', sheet_style)
        sheet7.write_merge(5, 5, 0, 6, u'Тэмдэглэл.(Бусад эргэлтийн бус хөрөнгийн төрөл тус бүрээр тайлбар, тэмдэглэлийг хийнэ. Урт хугацаат авлагыг тодруулна.)', simple_style2)
        row  = 6
        for i in range(1, 4):
            sheet7.write_merge(row, row, 0, 6, u'____________________________________________________________________________________________', simple_style)
            row += 1
        
        '''----------------------------- ӨР ТӨЛБӨР ---------------------'''
        sheet7.write_merge(10, 10, 2, 4, '16. ӨР ТӨЛБӨР', title_style)
        sheet7.write(11, 0, u'16.1 Дансны өглөг', simple_style)
        sheet7.write(12, 0, u'№', sheet_style)
        sheet7.write(13, 0, u'1', sheet_style)
        sheet7.write(14, 0, u'2', sheet_style)
        sheet7.write(15, 0, u'3', sheet_style)
        sheet7.write(16, 0, u'4', sheet_style)
        sheet7.write_merge(12, 12, 1, 2, u'Ангилал', sheet_style)
        sheet7.write_merge(13, 13, 1, 2, u'- Төлөгдөх хугацаандаа байгаа', sheet_style)
        sheet7.write_merge(14, 14, 1, 2, u'- Хугацаа хэтэрсэн', sheet_style)
        sheet7.write(12, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(12, 4, u'Эцсийн үлдэгдэл', sheet_style)
        
        sheet7.write(17, 0, u'16.2 Татварын өр', simple_style)
        sheet7.write(18, 0, u'№', sheet_style)
        sheet7.write(19, 0, u'1', sheet_style)
        sheet7.write(20, 0, u'2', sheet_style)
        sheet7.write(21, 0, u'3', sheet_style)
        sheet7.write(22, 0, u'4', sheet_style)
        sheet7.write_merge(18, 18, 1, 2, u'Татварын өрийн төрөл', sheet_style)
        sheet7.write_merge(19, 19, 1, 2, u'Орлогын албан татварын өглөг', sheet_style)
        sheet7.write_merge(20, 20, 1, 2, u'ХАОАТ-ын өглөг', sheet_style)
        sheet7.write_merge(21, 21, 1, 2, u'НӨАТ-ын өглөг', sheet_style)
        sheet7.write_merge(22, 22, 1, 2, u'Нийт дүн', sheet_style)
        sheet7.write(18, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(18, 4, u'Эцсийн үлдэгдэл', sheet_style)
        
        sheet7.write(23, 0, u'16.3 Богино хугацаат зээл', simple_style)
        sheet7.write_merge(24, 25, 0, 0, u'№', sheet_style)
        sheet7.write(26, 0, u'1', sheet_style)
        sheet7.write(27, 0, u'2', sheet_style)
        sheet7.write(28, 0, u'3', sheet_style)
        sheet7.write(29, 0, u'4', sheet_style)
        sheet7.write_merge(24, 25, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet7.write_merge(26, 26, 1, 2, u'- Төлөгдөх хугацаандаа байгаа', sheet_style)
        sheet7.write_merge(27, 27, 1, 2, u'- Хугацаа хэтэрсэн', sheet_style)
        sheet7.write_merge(29, 29, 1, 2, u'Нийт дүн', sheet_style)
        sheet7.write_merge(24, 24, 3, 4, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(25, 3, u'төгрөгөөр', sheet_style)
        sheet7.write(25, 4, u'вальютаар', sheet_style)
        sheet7.write_merge(24, 24, 5, 6, u'Эцсийн үлдэгдэл', sheet_style)
        sheet7.write(25, 5, u'төгрөгөөр', sheet_style)
        sheet7.write(25, 6, u'вальютаар', sheet_style)
        
        sheet7.write(30, 0, u'16.4 Богино хугацаат нөөц(өр төлбөр)', simple_style)
        sheet7.write(31, 0, u'№', sheet_style)
        sheet7.write(32, 0, u'1', sheet_style)
        sheet7.write(33, 0, u'2', sheet_style)
        sheet7.write(34, 0, u'3', sheet_style)
        sheet7.write(35, 0, u'4', sheet_style)
        sheet7.write(31, 1, u'Нөөцийн төрөл', sheet_style)
        sheet7.write(32, 1, u'Баталгаат засварын', sheet_style)
        sheet7.write(33, 1, u'Нөхөн сэргээлтийн', sheet_style)
        sheet7.write(35, 1, u'Нийт дүн', sheet_style)
        sheet7.write(31, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(31, 3, u'Нэмэгдсэн', sheet_style)
        sheet7.write(31, 4, u'Хасагдсан (ашигласан нөөц)(-)', sheet_style)
        sheet7.write(31, 5, u'Ашиглаагүй буцаан бичсэн дүн', sheet_style)
        sheet7.write(31, 6, u'Эцсийн үлдэгдэл', sheet_style)
        sheet7.write_merge(36, 36, 0, 6, u'Тэмдэглэл.(Урт хугацаат нөөцийн дүнг тодруулна. Нөөцийн төрлөөр тайлбар, тэмдэглэл хийнэ.)', simple_style)
        sheet7.write_merge(37, 37, 0, 6, u'____________________________________________________________________________________________', simple_style)
        sheet7.write_merge(38, 38, 0, 6, u'____________________________________________________________________________________________', simple_style)
        sheet7.write_merge(39, 39, 0, 2, u'16.5 Бусад богино хугацаат өр төлбөр', simple_style)
        sheet7.write(40, 0, u'№', sheet_style)
        sheet7.write(41, 0, u'1', sheet_style)
        sheet7.write(42, 0, u'2', sheet_style)
        sheet7.write_merge(40, 40, 1, 2, u'Төрөл', sheet_style)
        sheet7.write(40, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(40, 4, u'Эцсийн үлдэгдэл', sheet_style)
        sheet7.write_merge(42, 42, 1, 2, u'Нийт дүн', sheet_style)
        sheet7.write_merge(43, 43, 0, 6, u'Тэмдэглэл.(Гадаад валютаар илэрхийлэгдсэн богино хугацаат өр төлбөрийн дүнг тусад нь тодруулна.)', simple_style)
        sheet7.write_merge(44, 44, 0, 6, u'____________________________________________________________________________________________', simple_style)
        sheet7.write_merge(45, 45, 0, 6, u'____________________________________________________________________________________________', simple_style)
        sheet7.write_merge(46, 46, 0, 2, u'16.6 Урт хугацаат зээл болон бусад урт хугацаат өр төлбөр', simple_style)
        sheet7.write_merge(47, 48, 0, 0, u'№', sheet_style)
        sheet7.write(49, 0, u'1', sheet_style)
        sheet7.write_merge(50, 52, 0, 0, u'', sheet_style)
        sheet7.write(53, 0, u'2', sheet_style)
        sheet7.write(54, 0, u'3', sheet_style)
        sheet7.write_merge(47, 48, 1, 2, u'Төрөл', sheet_style)
        sheet7.write_merge(49, 49, 1, 2, u'Урт хугацаат зээлийн дүн', sheet_style)
        sheet7.write_merge(50, 50, 1, 2, u'Гадаадын байгууллагаас шууд авсан зээл', sheet_style)
        sheet7.write_merge(51, 51, 1, 2, u'Гадаадын байгууллагаас дамжуулан авсан зээл', sheet_style)
        sheet7.write_merge(52, 52, 1, 2, u'Дотоодын эх үүсвэрээс авсан зээл', sheet_style)
        sheet7.write_merge(53, 53, 1, 2, u'Бусад урт хугацаат өр төлбөрийн дүн гадаад дотоодын зах зээлд гаргасан бонд, өрийн бичиг', sheet_style)
        sheet7.write_merge(54, 54, 1, 2, u'', sheet_style)
        sheet7.write_merge(47, 47, 3, 4, u'Эхний үлдэгдэл', sheet_style)
        sheet7.write(48, 3, u'төгрөгөөр', sheet_style)
        sheet7.write(48, 4, u'валютаар', sheet_style)
        sheet7.write_merge(47, 47, 5, 6, u'Эцсийн үлдэгдэл', sheet_style)
        sheet7.write(48, 5, u'төгрөгөөр', sheet_style)
        sheet7.write(48, 6, u'валютаар', sheet_style)
        sheet7.write_merge(55, 55, 0, 6, u'Тэмдэглэл.(Урт хугацаат зээл болон бусад урт хугацаат өр төлбөрийн төрлөөр тайлбар, тэмдэглэл хийнэ).', simple_style2)
        sheet7.write_merge(56, 56, 0, 6, u'____________________________________________________________________________________________', simple_style)
        sheet7.write_merge(57, 57, 0, 6, u'____________________________________________________________________________________________', simple_style)
        sheet7.write_merge(58, 58, 0, 6, u'____________________________________________________________________________________________', simple_style)
        
        inch = 2000
        
        sheet7.col(0).width = 1 * inch
        sheet7.col(1).width = 3 * inch
        sheet7.col(2).width = 2 * inch
        sheet7.col(3).width = 2 * inch 
        sheet7.col(4).width = 2 * inch
        sheet7.col(5).width = 2 * inch
        sheet7.col(6).width = 2 * inch
        sheet7.col(7).width = 2 * inch
        sheet7.col(8).width = 2 * inch
        sheet7.col(9).width = 2 * inch
        sheet7.col(10).width = 2 * inch
        
        row_inch = 250
        sheet7.row(1).height = 2 * row_inch
        sheet7.row(12).height = 2 * row_inch
        sheet7.row(18).height = 2 * row_inch
        sheet7.row(31).height = 3 * row_inch
        sheet7.row(40).height = 2 * row_inch
        sheet7.row(46).height = 3 * row_inch
        sheet7.row(5).height = 2 * row_inch
        sheet7.row(50).height = 2 * row_inch
        sheet7.row(46).height = 3 * row_inch
        sheet7.row(51).height = 2 * row_inch
        sheet7.row(52).height = 2 * row_inch
        sheet7.row(53).height = 2 * row_inch
        sheet7.row(54).height = 2 * row_inch
        sheet7.row(55).height = 3 * row_inch
        
        '''----------------------------- ЭЗДИЙН ӨМЧ ---------------------'''
        
        sheet8.write_merge(0, 0, 2, 4, u'17. ЭЗДИЙН ӨМЧ', title_style)
        sheet8.write(1, 0, u'17.1 Өмч', simple_style)
        sheet8.write(2, 0, u'№', sheet_style)
        sheet8.write(3, 0, u'', sheet_style)
        sheet8.write(4, 0, u'1', sheet_style)
        sheet8.write(5, 0, u'2', sheet_style)
        sheet8.write(6, 0, u'3', sheet_style)
        sheet8.write(7, 0, u'4', sheet_style)
        sheet8.write_merge(2, 2, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet8.write_merge(2, 2, 3, 4, u'Эргэлтэнд байгаа бүрэн төлөгдсөн энгийн хувьцаа', sheet_style)
        sheet8.write_merge(2, 2, 5, 6, u'Давуу эрхтэй хувьцаа', sheet_style)
        sheet8.write(2, 7, u'Өмчийн дүн(төгрөгөөр)', sheet_style)
        sheet8.write(3, 3, u'Тоо ширхэг', sheet_style)
        sheet8.write(3, 4, u'Дүн (төгрөгөөр)', sheet_style)
        sheet8.write(3, 5, u'Тоо ширхэг', sheet_style)
        sheet8.write(3, 6, u'Дүн (төгрөгөөр)', sheet_style)
        sheet8.write(3, 7, u'', sheet_style)
        sheet8.write_merge(3, 3, 1, 2, u'', sheet_style)
        sheet8.write_merge(4, 4, 1, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet8.write_merge(5, 5, 1, 2, u'Нэмэгдсэн', sheet_style)
        sheet8.write_merge(6, 6, 1, 2, u'Хасагдсан(-)', sheet_style)
        sheet8.write_merge(7, 7, 1, 2, u'Эцсийн үлдэгдэл', sheet_style)
        sheet8.write_merge(8, 8, 0, 2, u'17.2 Хөрөнгийн дахин үнэлгээний нэмэгдэл', simple_style)
        sheet8.write(9, 0, u'№', sheet_style)
        sheet8.write(10, 0, u'1', sheet_style)
        sheet8.write(11, 0, u'2', sheet_style)
        sheet8.write_merge(12, 13, 0, 0, u'', sheet_style)
        sheet8.write(14, 0, u'3', sheet_style)
        sheet8.write_merge(15, 17, 0, 0, u'', sheet_style)
        sheet8.write(18, 0, u'4', sheet_style)
        sheet8.write_merge(9, 9, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet8.write_merge(10, 10, 1, 2, u'Эхний үлдэгдэл', sheet_style)
        sheet8.write_merge(11, 11, 1, 2, u'Нэмэгдсэн дүн', sheet_style)
        sheet8.write_merge(12, 12, 1, 2, u'Дахин үнэлгээний нэмэгдлийн зөрүү', sheet_style)
        sheet8.write_merge(13, 13, 1, 2, u'Дахин үнэлсэн хөрөнгийн үнэ цэнийн бууралтын газрын буцаалт', sheet_style)
        sheet8.write_merge(14, 14, 1, 2, u'Хасагдсан дүн(-)', sheet_style)
        sheet8.write_merge(15, 15, 1, 2, u'Дахин үнэлгээний нэмэгдлийн зөрүү', sheet_style)
        sheet8.write_merge(16, 16, 1, 2, u'Дахин үнэлгээний нэмэгдлийн хэрэгжсэн дүн', sheet_style)
        sheet8.write_merge(17, 17, 1, 2, u'Дахин үнэлсэн хөрөнгийн үнэ цэнийн бууралтын гарз', sheet_style)
        sheet8.write_merge(18, 18, 1, 2, u'Эцсийн үлдэгдэл', sheet_style)
        sheet8.write_merge(9, 9, 3, 4, u'Үндсэн хөрөнгийн дахин үнэлгээний нэмэгдэл', sheet_style)
        sheet8.write_merge(9, 9, 5, 6, u'Биет бус хөрөнгийн дахин үнэлгээний нэмэгдэл', sheet_style)
        sheet8.write(9, 7, u'Нийт дүн', sheet_style)
        sheet8.write_merge(19, 19, 0, 2, u'17.3 Гадаад валютын хөрвүүлэлтийн нөөц', simple_style)
        sheet8.write(20, 0, u'№', sheet_style)
        sheet8.write(21, 0, u'1', sheet_style)
        sheet8.write(22, 0, u'2', sheet_style)
        sheet8.write(23, 0, u'3', sheet_style)
        sheet8.write(24, 0, u'4', sheet_style)
        sheet8.write_merge(20, 20, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet8.write_merge(21, 21, 1, 2, u'Гадаад үйл ажиллагааны хөрвүүлэлтээс үүссэн зөрүү', sheet_style)
        sheet8.write_merge(22, 22, 1, 2, u'Бүртгэлийн валютыг толилуулгын валют руу хөрвүүлснээс үүссэн зөрүү', sheet_style)
        sheet8.write_merge(23, 23, 1, 2, u'Бусад', sheet_style)
        sheet8.write_merge(24, 24, 1, 2, u'Нийт дүн', sheet_style)
        sheet8.write(20, 3, u'Эхний үлдэгдэл', sheet_style)
        sheet8.write(20, 4, u'Нэмэгдсэн', sheet_style)
        sheet8.write(20, 5, u'Хасагдсан(-)', sheet_style)
        sheet8.write(20, 6, u'Эцсийн үлдэгдэл', sheet_style)
        sheet8.write_merge(25, 25, 0, 2, u'17.4 Эздийн өмчийн бусад хэсэг', simple_style)
        sheet8.write_merge(26, 26, 0, 6, u'Тэмдэглэл.(Эздийн өмчийн бусад хэсгийн бүрэлдэхүүн тус бүрээр тодруулж тайлбар, тэмдэглэл хийнэ.)', simple_style2)
        row  = 27
        for i in range(1, 5):
            sheet8.write_merge(row, row, 0, 6, u'____________________________________________________________________________________________', simple_style)
            row += 1
        
        '''----------------------------- БОРЛУУЛАЛТЫН ОРЛОГО БОЛОН БОРЛУУЛАЛТЫН ӨРТӨГ ---------------------'''
        
        sheet8.write_merge(31, 31, 2, 5, u'18. БОРЛУУЛАЛТЫН ОРЛОГО БОЛОН БОРЛУУЛАЛТЫН ӨРТӨГ', title_style2)
        sheet8.write(32, 0, u'№', sheet_style1)
        sheet8.write(33, 0, u'1', sheet_style)
        sheet8.write_merge(34, 35, 0, 0, u'1.1', sheet_style)
        sheet8.write_merge(36, 37, 0, 0, u'1.2', sheet_style)
        sheet8.write(38, 0, u'1.3', sheet_style)
        sheet8.write(39, 0, u'2', sheet_style1)
        sheet8.write(40, 0, u'3', sheet_style1)
        sheet8.write(41, 0, u'4', sheet_style1)
        sheet8.write_merge(42, 43, 0, 0, u'4.1', sheet_style)
        sheet8.write_merge(44, 45, 0, 0, u'4.2', sheet_style)
        sheet8.write(46, 0, u'4.3', sheet_style1)
        sheet8.write_merge(32, 32, 1, 3, u'Үзүүлэлт', sheet_style)
        sheet8.write_merge(33, 33, 1, 3, u'Борлуулалтын орлого:', sheet_style1)
        sheet8.write_merge(34, 34, 1, 3, u'Бараа, бүтээгдэхүүн борлуулсны орлого', sheet_style)
        sheet8.write_merge(35, 35, 1, 3, u'', sheet_style)
        sheet8.write_merge(36, 36, 1, 3, u'Ажил, үйлчилгээ борлуулсны орлого', sheet_style)
        sheet8.write_merge(37, 37, 1, 3, u'', sheet_style)
        sheet8.write_merge(38, 38, 1, 3, u'Нийт борлуулалтын орлого', sheet_style)
        sheet8.write_merge(39, 39, 1, 3, u'Борлуулалтын буцаалт, хөнгөлөлт, үнийн бууралт(-)', sheet_style1)
        sheet8.write_merge(40, 40, 1, 3, u'Цэвэр борлуулалт', sheet_style1)
        sheet8.write_merge(41, 41, 1, 3, u'Борлуулалтын өртөг:', sheet_style1)
        sheet8.write_merge(42, 42, 1, 3, u'Борлуулсан бараа, бүтээгдэхүүний өртөг', sheet_style)
        sheet8.write_merge(43, 43, 1, 3, u'', sheet_style)
        sheet8.write_merge(44, 44, 1, 3, u'Борлуулсан ажил, үйлчилгээний өртөг', sheet_style)
        sheet8.write_merge(45, 45, 1, 3, u'', sheet_style)
        sheet8.write_merge(46, 46, 1, 3, u'Нийт борлуулалтын өртөг', sheet_style)
        sheet8.write(32, 4, u'Өмнөх оны дүн', sheet_style)
        sheet8.write(32, 5, u'Тайлант оны дүн', sheet_style)
        
        '''----------------------------- БУСАД ОРЛОГО, ОЛЗ(ГАРЗ), АШИГ(АЛДАГДАЛ) ---------------------'''
        
        sheet8.write_merge(47, 47, 2, 5, u'19. БУСАД ОРЛОГО, ОЛЗ(ГАРЗ), АШИГ(АЛДАГДАЛ)', title_style2)
        sheet8.write(48, 0, u'19.1 Бусад орлого', simple_style)
        sheet8.write(49, 0, u'№', sheet_style)
        sheet8.write(50, 0, u'1', sheet_style)
        sheet8.write(51, 0, u'2', sheet_style)
        sheet8.write(52, 0, u'3', sheet_style)
        sheet8.write_merge(49, 49, 1, 2, u'Орлогын төрөл', sheet_style)
        sheet8.write_merge(52, 52, 1, 2, u'Нийт дүн', sheet_style)
        sheet8.write(49, 3, u'Өмнөх оны дүн', sheet_style)
        sheet8.write(49, 4, u'Тайлант оны дүн', sheet_style)
        sheet8.write_merge(53, 53, 0, 2, u'19.2 Гадаад валютын ханшийн зөрүүний олз, гарз', sheet_style)
        sheet8.write(54, 0, u'№', sheet_style)
        sheet8.write(55, 0, u'1', sheet_style)
        sheet8.write(56, 0, u'2', sheet_style)
        sheet8.write(57, 0, u'3', sheet_style)
        sheet8.write(58, 0, u'4', sheet_style)
        sheet8.write(59, 0, u'5', sheet_style)
        sheet8.write_merge(54, 54, 1, 3, u'Төрөл', sheet_style)
        sheet8.write_merge(55, 55, 1, 3, u'Мөнгөн хөрөнгийн үлдэгдэл хийсэн ханшийн тэгшитгэлийн ханшийн зөрүү', sheet_style)
        sheet8.write_merge(56, 56, 1, 3, u'Эргэлтийн авлага, өр төлбөртэй холбоотой үүссэн ханшийн зөрүү', sheet_style)
        sheet8.write_merge(57, 57, 1, 3, u'Эргэлтийн бус авлага, өр төлбөртэй холбоотой үүссэн ханшийн зөрүү', sheet_style)
        sheet8.write_merge(58, 58, 1, 3, u'Валютын арилжаанаас үүссэн олз/гарз', sheet_style)
        sheet8.write_merge(59, 59, 1, 3, u'Нийт дүн', sheet_style)
        sheet8.write(54, 4, u'Өмнөх оны дүн', sheet_style)
        sheet8.write(54, 5, u'Тайлант оны дүн', sheet_style)
        sheet8.write_merge(60, 60, 0, 1, u'19.3 Бусад ашиг (алдагдал)', simple_style)
        sheet8.write(61, 0, u'№', sheet_style)
        sheet8.write(62, 0, u'1', sheet_style)
        sheet8.write(63, 0, u'2', sheet_style)
        sheet8.write(64, 0, u'3', sheet_style)
        sheet8.write(65, 0, u'4', sheet_style)
        sheet8.write(66, 0, u'5', sheet_style)
        sheet8.write(67, 0, u'6', sheet_style)
        sheet8.write(68, 0, u'7', sheet_style)
        sheet8.write_merge(61, 61, 1, 3, u'Ашиг(алдагдал)', sheet_style)
        sheet8.write(61, 4, u'Өмнөх оны дүн', sheet_style)
        sheet8.write(61, 5, u'Тайлант оны дүн', sheet_style)
        sheet8.write_merge(62, 62, 1, 3, u'Хөрөнгийн үнэ цэнийн бууралтын гарз', sheet_style)
        sheet8.write_merge(63, 63, 1, 3, u'ХОЗҮХХ-ийн бодит үнэ цэнийн өөрчлөлтийн олз, гарз', sheet_style)
        sheet8.write_merge(64, 64, 1, 3, u'ХОЗҮХХ данснаас хассаны олз, гарз', sheet_style)
        sheet8.write_merge(65, 65, 1, 3, u'Хөрөнгийн дахин үнэлгээний олз, гарз', sheet_style)
        sheet8.write_merge(66, 66, 1, 3, u'Хөрөнгийн үнэ цэнийн бууралтын гарз(гарзын буцаалт)', sheet_style)
        sheet8.write_merge(67, 67, 1, 3, u'', sheet_style)
        sheet8.write_merge(68, 68, 1, 3, u'Нийт дүн', sheet_style)
        
        inch = 2000
        
        sheet8.col(0).width = 1 * inch
        sheet8.col(1).width = 3 * inch
        sheet8.col(2).width = 2 * inch
        sheet8.col(3).width = 1 * inch 
        sheet8.col(4).width = 1 * inch
        sheet8.col(5).width = 1 * inch
        sheet8.col(6).width = 1 * inch
        sheet8.col(7).width = 1 * inch
        
        row_inch = 250
        sheet8.row(2).height = 4 * row_inch
        sheet8.row(3).height = 2 * row_inch
        sheet8.row(9).height = 3 * row_inch
        sheet8.row(12).height = 2 * row_inch
        sheet8.row(15).height = 2 * row_inch
        sheet8.row(16).height = 2 * row_inch
        sheet8.row(17).height = 2 * row_inch
        sheet8.row(13).height = 2 * row_inch
        sheet8.row(22).height = 3 * row_inch
        sheet8.row(27).height = 2 * row_inch
        sheet8.row(20).height = 2 * row_inch
        sheet8.row(21).height = 2 * row_inch
        sheet8.row(26).height = 2 * row_inch
        sheet8.row(31).height = 2 * row_inch
        sheet8.row(32).height = 2 * row_inch
        sheet8.row(47).height = 3 * row_inch
        sheet8.row(55).height = 2 * row_inch
        sheet8.row(56).height = 2 * row_inch
        sheet8.row(57).height = 2 * row_inch
        sheet8.row(61).height = 2 * row_inch
        sheet8.row(66).height = 2 * row_inch
        sheet8.row(54).height = 2 * row_inch
        sheet8.row(49).height = 2 * row_inch
        '''----------------------------- ЗАРДАЛ ---------------------'''
        
        sheet9.write_merge(0, 0, 2, 4, u'20. ЗАРДАЛ', title_style)
        sheet9.write_merge(1, 1, 0, 3, u'20.1 Борлуулалт маркетингийн болон ерөнхий ба удирдлагын зардлууд', simple_style2)
        sheet9.write_merge(3, 4, 0, 0, u'№', sheet_style)
        sheet9.write(5, 0, u'1', sheet_style)
        sheet9.write(6, 0, u'2', sheet_style)
        sheet9.write(7, 0, u'3', sheet_style)
        sheet9.write(8, 0, u'4', sheet_style)
        sheet9.write(9, 0, u'5', sheet_style)
        sheet9.write(10, 0, u'6', sheet_style)
        sheet9.write(11, 0, u'7', sheet_style)
        sheet9.write(12, 0, u'8', sheet_style)
        sheet9.write(13, 0, u'9', sheet_style)
        sheet9.write(14, 0, u'10', sheet_style)
        sheet9.write(15, 0, u'11', sheet_style)
        sheet9.write(16, 0, u'12', sheet_style)
        sheet9.write(17, 0, u'13', sheet_style)
        sheet9.write(18, 0, u'14', sheet_style)
        sheet9.write(19, 0, u'15', sheet_style)
        sheet9.write(20, 0, u'16', sheet_style)
        sheet9.write(21, 0, u'17', sheet_style)
        sheet9.write(22, 0, u'18', sheet_style)
        sheet9.write(23, 0, u'19', sheet_style)
        sheet9.write(24, 0, u'20', sheet_style)
        sheet9.write(25, 0, u'21', sheet_style)
        sheet9.write(26, 0, u'22', sheet_style)
        sheet9.write(27, 0, u'23', sheet_style)
        sheet9.write_merge(3, 4, 1, 2, u'Зардлын төрөл', sheet_style)
        sheet9.write_merge(5, 5, 1, 2, u'Ажиллагчдын цалингийн зардал', sheet_style)
        sheet9.write_merge(6, 6, 1, 2, u'Аж ахуйн нэгжээс төлсөн НДШ-ийн зардал', sheet_style)
        sheet9.write_merge(7, 7, 1, 2, u'Албан татвар, төлбөр, хураамжийн зардал', sheet_style)
        sheet9.write_merge(8, 8, 1, 2, u'Томилолтын зардал', sheet_style)
        sheet9.write_merge(9, 9, 1, 2, u'Бичиг хэргийн зардал', sheet_style)
        sheet9.write_merge(10, 10, 1, 2, u'Шуудан холбооны зардал', sheet_style)
        sheet9.write_merge(11, 11, 1, 2, u'Мэргэжлийн үйлчилгээний зардал', sheet_style)
        sheet9.write_merge(12, 12, 1, 2, u'Сургалтын зардал', sheet_style)
        sheet9.write_merge(13, 13, 1, 2, u'Сонин сэтгүүл захиалгын зардал', sheet_style)
        sheet9.write_merge(14, 14, 1, 2, u'Даатгалын зардал', sheet_style)
        sheet9.write_merge(15, 15, 1, 2, u'Ашиглалтын зардал', sheet_style)
        sheet9.write_merge(16, 16, 1, 2, u'Засварын зардал', sheet_style)
        sheet9.write_merge(17, 17, 1, 2, u'Элэгдэл, хорогдлын зардал', sheet_style)
        sheet9.write_merge(18, 18, 1, 2, u'Түрээсийн зардал', sheet_style)
        sheet9.write_merge(19, 19, 1, 2, u'Харуул хамгаалалтын зардал', sheet_style)
        sheet9.write_merge(20, 20, 1, 2, u'Цэвэрлэгээ үйлчилгээний зардал', sheet_style)
        sheet9.write_merge(21, 21, 1, 2, u'Тээврийн зардал', sheet_style)
        sheet9.write_merge(22, 22, 1, 2, u'Шатахууны зардал', sheet_style)
        sheet9.write_merge(23, 23, 1, 2, u'Хүлээн авалтын зардал', sheet_style)
        sheet9.write_merge(24, 24, 1, 2, u'Зар сурталчилгааны зардал', sheet_style)
        sheet9.write_merge(25, 25, 1, 2, u'', sheet_style)
        sheet9.write_merge(26, 26, 1, 2, u'', sheet_style)
        sheet9.write_merge(27, 27, 1, 2, u'Нийт дүн', sheet_style)
        sheet9.write_merge(3, 3, 3, 4, u'Өмнөх оны дүн', sheet_style)
        sheet9.write_merge(3, 3, 5, 6, u'Тайлант оны дүн', sheet_style)
        sheet9.write(4, 3, u'БорМар', sheet_style)
        sheet9.write(4, 4, u'ЕрУд', sheet_style)
        sheet9.write(4, 5, u'БорМар', sheet_style)
        sheet9.write(4, 6, u'ЕрУд', sheet_style)
        
        sheet9.write_merge(28, 28, 0, 1, u'20.2 Бусад зардал', simple_style)
        sheet9.write(29, 0, u'№', sheet_style)
        sheet9.write(30, 0, u'1', sheet_style)
        sheet9.write(31, 0, u'2', sheet_style)
        sheet9.write(32, 0, u'3', sheet_style)
        sheet9.write(33, 0, u'4', sheet_style)
        sheet9.write(34, 0, u'5', sheet_style)
        sheet9.write_merge(29, 29, 1, 2, u'Зардлын төрөл', sheet_style)
        sheet9.write_merge(30, 30, 1, 2, u'Алданги, торгуулийн зардал', sheet_style)
        sheet9.write_merge(31, 31, 1, 2, u'Хандивийн зардал', sheet_style)
        sheet9.write_merge(32, 32, 1, 2, u'Найдваргүй авлагын зардал', sheet_style)
        sheet9.write_merge(33, 33, 1, 2, u'', sheet_style)
        sheet9.write_merge(34, 34, 1, 2, u'Нийт дүн', sheet_style)
        sheet9.write(29, 3, u'Өмнөх оны дүн', sheet_style)
        sheet9.write(29, 4, u'Тайлант оны дүн', sheet_style)
        sheet9.write_merge(35, 35, 0, 1, u'20.3 Цалингийн зардал', simple_style)
        sheet9.write_merge(36, 37, 0, 2, u'Ангилал', sheet_style)
        sheet9.write_merge(38, 38, 0, 2, u'Үйлдвэрлэл, үйлчилгээний', sheet_style)
        sheet9.write_merge(39, 39, 0, 2, u'', sheet_style)
        sheet9.write_merge(40, 40, 0, 2, u'Борлуулалт маркетингийн', sheet_style)
        sheet9.write_merge(41, 41, 0, 2, u'', sheet_style)
        sheet9.write_merge(42, 42, 0, 2, u'Ерөнхий ба удирдлагын', sheet_style)
        sheet9.write_merge(43, 43, 0, 2, u'', sheet_style)
        sheet9.write_merge(44, 44, 0, 2, u'Нийт дүн', sheet_style)
        sheet9.write_merge(36, 37, 3, 4, u'Ажиллагчдын дундаж тоо', sheet_style)
        sheet9.write_merge(36, 36, 5, 6, u'Цалингийн зардлын дүн', sheet_style)
        sheet9.write(37, 5, u'Өмнөх оны дүн', sheet_style)
        sheet9.write(37, 6, u'Тайлант оны дүн', sheet_style)
        
        '''----------------------------- ОРЛОГЫН ТАТВАРЫН ЗАРДАЛ ---------------------'''
        
        sheet9.write_merge(45, 45, 2, 5, u'21. ОРЛОГЫН ТАТВАРЫН ЗАРДАЛ', title_style)
        sheet9.write(46, 0, u'№', sheet_style)
        sheet9.write(47, 0, u'1', sheet_style)
        sheet9.write(48, 0, u'2', sheet_style)
        sheet9.write(49, 0, u'3', sheet_style)
        sheet9.write_merge(46, 46, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet9.write_merge(47, 47, 1, 2, u'Тайлант үеийн орлогын татварын зардал', sheet_style)
        sheet9.write_merge(48, 48, 1, 2, u'Хойшлогдсон татварын зардал(орлого)', sheet_style)
        sheet9.write_merge(49, 49, 1, 2, u'Орлогын татварын зардал(орлого)-ын нийт дүн', sheet_style)
        sheet9.write(46, 3, u'Өмнөх оны дүн', sheet_style)
        sheet9.write(46, 4, u'Тайлант оны дүн', sheet_style)
        sheet9.write_merge(50, 50, 0, 6, u'Тэмдэглэл.(Орлогын татварын зардал(орлого)-ын бүрэлдэхүүн тус бүрээр тайлбар, тэмдэглэл хийнэ.)', simple_style2)
        row  = 51
        for i in range(1, 7):
            sheet9.write_merge(row, row, 0, 6, u'____________________________________________________________________________________________', simple_style)
            row += 1
        
        '''----------------------------- ХОЛБООТОЙ ТАЛУУДЫН ТОДРУУЛГА ---------------------'''
        sheet9.write_merge(57, 57, 2, 5, u'22. ХОЛБООТОЙ ТАЛУУДЫН ТОДРУУЛГА', title_style)
        sheet9.write_merge(58, 58, 0, 6, u'22.1 Толгой компани, хамгийн дээд хяналт тавигч компани, хувь хүний ', simple_style2)
        sheet9.write_merge(59, 59, 0, 1, u'Үзүүлэлт', sheet_style)
        sheet9.write_merge(60, 60, 0, 1, u'Нэр', sheet_style)
        sheet9.write_merge(61, 61, 0, 1, u'Бүртгэгдсэн(оршин суугаа)улс', sheet_style)
        sheet9.write_merge(62, 62, 0, 1, u'Эзэмшлийн хувь', sheet_style)        
        sheet9.write_merge(59, 59, 2, 3, u'Толгой компани', sheet_style)
        sheet9.write_merge(59, 59, 4, 5, u'Хамгийн дээд хяналт тавигч толгой компани', sheet_style)
        sheet9.write_merge(59, 59, 6, 7, u'Хамгийн дээд хяналт тавигч хувь хүн', sheet_style)
        sheet9.write_merge(59, 59, 8, 9, u'Тайлбар', sheet_style)
        sheet9.write_merge(63, 63, 0, 6, u'22.2 Тэргүүлэх удирдлагын бүрэлдэхүүнд олгосон нөхөн олговрын тухай мэдээлэл', simple_style2)
        sheet9.write_merge(64, 64, 0, 6, u'Тэргүүлэх удирдлага гэдэгт....................................................................... бүрэлдэхүүнийг хамруулна.', simple_style2)
        sheet9.write(65, 0, u'№', sheet_style)
        sheet9.write(66, 0, u'1', sheet_style)
        sheet9.write(67, 0, u'2', sheet_style)
        sheet9.write(68, 0, u'3', sheet_style)
        sheet9.write(69, 0, u'4', sheet_style)
        sheet9.write_merge(65, 65, 1, 2, u'Нөхөн олговрын нэр', sheet_style)
        sheet9.write_merge(66, 66, 1, 2, u'Богино болон урт хугацааны тэтгэмж', sheet_style)
        sheet9.write_merge(67, 67, 1, 2, u'Ажил эрхлэлтийн дараах, ажлаас халагдсаны тэтгэмж', sheet_style)
        sheet9.write_merge(68, 68, 1, 2, u'Хувьцаанд суурилсан төлбөр', sheet_style)
        sheet9.write_merge(69, 69, 1, 2, u'Нийт дүн', sheet_style)
        sheet9.write(65, 3, u'Өмнөх оны дүн', sheet_style)
        sheet9.write(65, 4, u'Тайлант оны дүн', sheet_style)
        sheet9.write_merge(70, 70, 0, 2, u'22.3 Холбоотой талуудтай хийсэн ажил гүйлгээ', simple_style2)
        sheet9.write(71, 0, u'№', sheet_style)
        sheet9.write(72, 0, u'1', sheet_style)
        sheet9.write(73, 0, u'2', sheet_style)
        sheet9.write_merge(71, 71, 1, 2, u'Холбоотой талын нэр', sheet_style)
        sheet9.write_merge(71, 71, 3, 4, u'Ажил гүйлгээний утга', sheet_style)
        sheet9.write_merge(71, 71, 5, 6, u'Дүн', sheet_style)
        sheet9.write(71, 7, u'Тайлбар', sheet_style)
        
        '''----------------------------- БОЛЗОШГҮЙ ХӨРӨНГӨ БӨ ӨР ТӨЛБӨР ---------------------'''
        sheet9.write_merge(74, 74, 2, 5, u'23. БОЛЗОШГҮЙ ХӨРӨНГӨ БӨ ӨР ТӨЛБӨР', title_style2)
        sheet9.write_merge(75, 75, 0, 6, u'Тэмдэглэл.(Болзошгүй хөрөнгө ба өр төлбөрийн мөн чанар, хэрэв практик боломжттой бол тэдгээрийн санхүүгийн нөлөөний тооцооллыг тодруулна.)', simple_style2)
        row  = 76
        for i in range(1, 5):
            sheet9.write_merge(row, row, 0, 6, u'____________________________________________________________________________________________', simple_style)
            row += 1
        
        '''----------------------------- ТАЙЛАГНАЛЫН ҮЕИЙН ДАРААХ ҮЙЛ ЯВДАЛ ---------------------'''
        sheet9.write_merge(80, 80, 0, 6, u'24. ТАЙЛАГНАЛЫН ҮЕИЙН ДАРААХ ҮЙЛ ЯВДАЛ', title_style)
        sheet9.write_merge(81, 81, 0, 6, u'Тэмдэглэл.(Тайлагналын өдрийн дараах үл залруулагдах үйл явдлын материаллаг ангилал тус бүрийн хувьд мөн чанар, санхүүгийн нөлөөллийн тооцоолол зэргийг тодруулж бусад тайлбар, тэмдэглэл хийнэ.)', simple_style2)
        row  = 82
        for i in range(1, 5):
            sheet9.write_merge(row, row, 0, 6, u'____________________________________________________________________________________________', simple_style)
            row += 1
        
        inch = 2000
        
        sheet9.col(0).width = 1 * inch
        sheet9.col(1).width = 2 * inch
        sheet9.col(2).width = 1 * inch
        sheet9.col(3).width = 1 * inch 
        sheet9.col(4).width = 1 * inch
        sheet9.col(5).width = 1 * inch
        sheet9.col(6).width = 1 * inch
        sheet9.col(7).width = 1 * inch
        sheet9.col(8).width = 1 * inch
        sheet9.col(9).width = 1 * inch
        sheet9.col(10).width = 1 * inch
        
        row_inch = 250
        sheet9.row(1).height = 2 * row_inch
        sheet9.row(5).height = 2 * row_inch
        sheet9.row(6).height = 2 * row_inch
        sheet9.row(7).height = 2 * row_inch
        sheet9.row(8).height = 2 * row_inch
        sheet9.row(9).height = 2 * row_inch
        sheet9.row(10).height = 2 * row_inch
        sheet9.row(11).height = 2 * row_inch
        sheet9.row(12).height = 2 * row_inch
        sheet9.row(13).height = 2 * row_inch
        sheet9.row(14).height = 2 * row_inch
        sheet9.row(15).height = 2 * row_inch
        sheet9.row(16).height = 2 * row_inch
        sheet9.row(17).height = 2 * row_inch
        sheet9.row(18).height = 2 * row_inch
        sheet9.row(19).height = 2 * row_inch
        sheet9.row(20).height = 2 * row_inch
        sheet9.row(29).height = 2 * row_inch
        sheet9.row(36).height = 2 * row_inch
        sheet9.row(37).height = 2 * row_inch
        sheet9.row(46).height = 2 * row_inch
        sheet9.row(47).height = 2 * row_inch
        sheet9.row(48).height = 2 * row_inch
        sheet9.row(49).height = 2 * row_inch
        sheet9.row(50).height = 3 * row_inch
        sheet9.row(59).height = 2 * row_inch
        sheet9.row(61).height = 2 * row_inch
        sheet9.row(63).height = 2 * row_inch
        sheet9.row(64).height = 2 * row_inch
        sheet9.row(65).height = 2 * row_inch
        sheet9.row(66).height = 2 * row_inch
        sheet9.row(67).height = 2 * row_inch
        sheet9.row(70).height = 2 * row_inch
        sheet9.row(74).height = 2 * row_inch
        sheet9.row(75).height = 2 * row_inch
        sheet9.row(81).height = 4 * row_inch
        
        '''----------------------------- ХӨРӨНГӨ ОРУУЛАЛТ ---------------------'''
        sheet10.write_merge(0, 0, 2, 4, u'25. ХӨРӨНГӨ ОРУУЛАЛТ', title_style)
        sheet10.write_merge(1, 2, 0, 0, u'№', sheet_style)
        sheet10.write(3, 0, u'1', sheet_style1)
        sheet10.write(4, 0, u'1.1', sheet_style)
        sheet10.write(5, 0, u'1.2', sheet_style)
        sheet10.write(6, 0, u'1.2.1', sheet_style)
        sheet10.write(7, 0, u'1.2.2', sheet_style)
        sheet10.write(8, 0, u'1.3', sheet_style)
        sheet10.write(9, 0, u'1.4', sheet_style)
        sheet10.write(10, 0, u'1.5', sheet_style)
        sheet10.write(11, 0, u'1.6', sheet_style)
        sheet10.write(12, 0, u'1.7', sheet_style)
        sheet10.write(13, 0, u'1.8', sheet_style)
        sheet10.write(14, 0, u'1.9', sheet_style)
        sheet10.write(15, 0, u'1.10', sheet_style1)
        sheet10.write(16, 0, u'2', sheet_style1)
        sheet10.write(17, 0, u'2.1', sheet_style)
        sheet10.write(18, 0, u'2.2', sheet_style)
        sheet10.write(19, 0, u'2.2.1', sheet_style)
        sheet10.write(20, 0, u'2.2.2', sheet_style)
        sheet10.write(21, 0, u'2.3', sheet_style)
        sheet10.write(22, 0, u'2.4', sheet_style)
        sheet10.write(23, 0, u'2.5', sheet_style)
        sheet10.write(24, 0, u'2.6', sheet_style)
        sheet10.write(25, 0, u'2.7', sheet_style)
        sheet10.write(26, 0, u'2.7.1', sheet_style)
        sheet10.write(27, 0, u'2.8', sheet_style1)
        sheet10.write(28, 0, u'3', sheet_style1)
        sheet10.write(29, 0, u'3.1', sheet_style)
        sheet10.write(30, 0, u'3.2', sheet_style)
        sheet10.write(31, 0, u'4', sheet_style1)
        sheet10.write_merge(1, 2, 1, 2, u'Үзүүлэлт', sheet_style)
        sheet10.write_merge(3, 3, 1, 2, u'Биет хөрөнгө:', sheet_style1)
        sheet10.write_merge(4, 4, 1, 2, u'Газрын сайжруулалт', sheet_style)
        sheet10.write_merge(5, 5, 1, 2, u'Барилга байгууламж', sheet_style)
        sheet10.write_merge(6, 6, 1, 2, u'Үүнээс:Орон сууцны барилга', sheet_style)
        sheet10.write_merge(7, 7, 1, 2, u'Авто зам', sheet_style)
        sheet10.write_merge(8, 8, 1, 2, u'Машин тоног, төхөөрөмж', sheet_style)
        sheet10.write_merge(9, 9, 1, 2, u'Тээврийн хэрэгсэл', sheet_style)
        sheet10.write_merge(10, 10, 1, 2, u'Тавилга эд хогшил', sheet_style)
        sheet10.write_merge(11, 11, 1, 2, u'Компютер, бусад хэрэгсэл', sheet_style)
        sheet10.write_merge(12, 12, 1, 2, u'Биологийн хөрөнгө', sheet_style)
        sheet10.write_merge(13, 13, 1, 2, u'Бусад биет хөрөнгө:', sheet_style)
        sheet10.write_merge(14, 14, 1, 2, u'Үүнээс: ХОЗҮХХ', sheet_style)
        sheet10.write_merge(15, 15, 1, 2, u'Биет хөрөнгийн дүн', sheet_style1)
        sheet10.write_merge(16, 16, 1, 2, u'Биет бус хөрөнгө:', sheet_style1)
        sheet10.write_merge(17, 17, 1, 2, u'Зохиогчийн эрх', sheet_style)
        sheet10.write_merge(18, 18, 1, 2, u'Компьютерийн програм хангамж', sheet_style)
        sheet10.write_merge(19, 19, 1, 2, u'Үүнээс: Програм хангамж', sheet_style)
        sheet10.write_merge(20, 20, 1, 2, u'Мэдээллийн сан', sheet_style)
        sheet10.write_merge(21, 21, 1, 2, u'Патент', sheet_style)
        sheet10.write_merge(22, 22, 1, 2, u'Барааны тэмдэг', sheet_style)
        sheet10.write_merge(23, 23, 1, 2, u'Тусгай зөвшөөрөл', sheet_style)
        sheet10.write_merge(24, 24, 1, 2, u'Газар эзэмших эрх', sheet_style)
        sheet10.write_merge(25, 25, 1, 2, u'Бусад биет бус хөрөнгө', sheet_style)
        sheet10.write_merge(26, 26, 1, 2, u'Үүнээс: Зураг төслийн ажил, ТЭЗҮ боловсруулах, туршилт судалгаа', sheet_style)
        sheet10.write_merge(27, 27, 1, 2, u'Биет бус хөрөнгийн дүн', sheet_style1)
        sheet10.write_merge(28, 28, 1, 2, u'Хайгуул үнэлгээний хөрөнгө', sheet_style1)
        sheet10.write_merge(29, 29, 1, 2, u'Үүнээс: Биет хөрөнгө', sheet_style)
        sheet10.write_merge(30, 30, 1, 2, u'Биет бус хөрөнгө', sheet_style)
        sheet10.write_merge(31, 31, 1, 2, u'Нийт дүн', sheet_style1)
        
        
        sheet10.write_merge(1, 2, 3, 3, u'Эхний үлдэгдэл', sheet_style_min)
        sheet10.write_merge(1, 1, 4, 13, u'Тайлант хугацаанд хийгдсэн хөрөнгө оруулалт(төгрөгөөр)', sheet_style_min)
        sheet10.write(2, 4, u'Аж ахуйн нэгжийн өөрийн хөрөнгөөр', sheet_style_min)
        sheet10.write(2, 5, u'Улсын төсвийн хөрөнгөөр', sheet_style_min)
        sheet10.write(2, 6, u'Орон нутгийн төсвийн хөрөнгөөр', sheet_style_min)
        sheet10.write(2, 7, u'Банкны зээл', sheet_style_min)
        sheet10.write(2, 8, u'Гадаадын шууд хөрөнгө оруулалт', sheet_style_min)
        sheet10.write(2, 9, u'Гадаадын зээл', sheet_style_min)
        sheet10.write(2, 10, u'Гадаадын буцалтгүй тусламж', sheet_style_min)
        sheet10.write(2, 11, u'Төсөл, хөтөлбөр, хандив', sheet_style_min)
        sheet10.write(2, 12, u'Бусад эх үүсвэр', sheet_style_min)
        sheet10.write(2, 13, u'Дүн', sheet_style_min)
        sheet10.write_merge(1, 2, 14, 14, u'Эцсийн үлдэгдэл', sheet_style_min)
        
        inch = 2000
        
        sheet10.col(0).width = 1 * inch
        sheet10.col(1).width = 3 * inch
        sheet10.col(2).width = 2 * inch
        sheet10.col(3).width = 1 * inch 
        sheet10.col(4).width = 1 * inch
        sheet10.col(5).width = 1 * inch
        sheet10.col(6).width = 1 * inch
        sheet10.col(7).width = 1 * inch
        sheet10.col(8).width = 1 * inch
        sheet10.col(9).width = 1 * inch
        sheet10.col(10).width = 1 * inch
        sheet10.col(11).width = 1 * inch
        sheet10.col(12).width = 1 * inch
        sheet10.col(13).width = 1 * inch
        
        row_inch = 250
        sheet10.row(1).height = 2 * row_inch
        sheet10.row(2).height = 5 * row_inch
        sheet10.row(8).height = 2 * row_inch
        sheet10.row(18).height = 2 * row_inch
        sheet10.row(26).height = 5 * row_inch
        sheet10.row(27).height = 2 * row_inch
        sheet10.row(28).height = 2 * row_inch
        sheet10.row(11).height = 5 * row_inch
        sheet10.row(26).height = 3 * row_inch
        sheet10.row(6).height = 2 * row_inch
        
        return {'data':book,}

account_property_report()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: