# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2014 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################
from datetime import timedelta, datetime
from lxml import etree
from openerp.osv import fields, osv
from openerp.tools.translate import _
import time
import xlwt
import openerp.netsvc
#from compiler.ast import For
#logger = netsvc.Logger()

class account_stockholders_equity_report(osv.osv_memory):
    """
        ӨМЧИЙН ӨӨРЧЛӨЛТИЙН ТАЙЛАН
        Statement of stockholder's equity
    """
    _inherit = "abstract.report.excel"
    _name = "account.stockholders.equity.report"
    _description = "Statement of stockholder's equity"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True),
        'date_from': fields.date('Start Date', required=True),
        'date_to': fields.date('End Date', required=True),
    }
    
    def _get_account(self, cr, uid, context=None):
        accounts = self.pool.get('account.account').search(cr, uid, [('parent_id', '=', False)], limit=1)
        return accounts and accounts[0] or False
    
    _defaults = {
        'company_id': lambda obj,cr,uid,c=None:obj.pool.get('res.company')._company_default_get(cr,uid,'account.stockholders.equity.report',context=c),
        'date_from': lambda *a: time.strftime('%Y-01-01'),
        'date_to': lambda *a: time.strftime("%Y-%m-%d"),
        'fiscalyear_id': lambda obj,cr,uid,c=None:obj.pool.get('account.fiscalyear').find(cr, uid, context=c)
    }
    
    def onchange_fiscalyear(self, cr, uid, ids, fiscalyear, context=None):
        fy = self.pool.get('account.fiscalyear').browse(cr, uid, fiscalyear, context=context)
        date_from = fy.date_start
        date_to = fy.date_stop
        now_day = time.strftime('%Y-%m-%d')
        if now_day > date_from and now_day < date_to:
            date_to = now_day
        return {'value':{'date_from':date_from,'date_to':date_to}}
    
    def prepare_data(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        
        data = {
            'ids': ids,
            'model': 'account.stockholders.equity.report',
            'form': {
                'company_id': obj.company_id.id,
                'date_from': obj.date_from,
                'date_to': obj.date_to,
                'fiscalyear_id': obj.fiscalyear_id.id,
                'save': obj.save
            },
            'report_name':'account.stockholders.equity.report'
        }
        
        if obj.date_from < obj.fiscalyear_id.date_start or \
           obj.date_from > obj.fiscalyear_id.date_stop or \
           obj.date_to < obj.fiscalyear_id.date_start or \
           obj.date_to > obj.fiscalyear_id.date_stop:
            raise osv.except_osv(_('Integrity Error!'), _('The report duration does not correct for this fiscalyear: %s') % form.fiscalyear_id.name)
        config_obj = self.pool.get('account.stockholders.equity.report.config') 
        config_id = config_obj.search(cr, uid, [('company_id', '=', obj.company_id.id),('year_id','=',obj.fiscalyear_id.id)]) 
        config = config_obj.browse(cr, uid, config_id, context=context)
        
        if not config[0].property_adjustment_journal :
            raise osv.except_osv(_('Integrity Error!'), _('There is no completely necessary data in your company configuration for Statement of stockholders equity report.'))       
        return data
    
    def get_log_message(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        body = u"Өмчийн өөрчлөлтийн тайлан (Эхлэх='%s', Дуусах='%s')" % (obj.date_from, obj.date_to)
        return body
    def create_report_data(self, cr, uid, data, context=None):
        ''' Өмчийн өөрчлөлтийн тайлангийн мэдээллийг боловсруулж
            тайлангийн форматад тохируулан python [{},{},{}...]
            загвараар хүснэгтийн мөр багануудын өгөгдлийг боловсруулна.
            
            @return: cols - Хүснэгтийн толгойн өгөгдлүүд 
                     rows - Хүснэгтийн биеийн өгөгдлүүд
        '''
        def _get_initial_balance(dicte, a):
            if a.user_type.report_type not in ('expense','asset') :
                balance = dicte[a.id]['credit'] - dicte[a.id]['debit']
            else:
                balance = dicte[a.id]['debit'] - dicte[a.id]['credit']
            return balance
        
        if not context:
            context = {}
        account_obj = self.pool.get('account.account')
        journal_obj = self.pool.get('account.journal')
        currency_obj = self.pool.get('res.currency')
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        if not context:
            context = {}
        fiscalyear = fiscalyear_obj.browse(cr, uid, data['form']['fiscalyear_id'], context=context)
        
        config_obj = self.pool.get('account.stockholders.equity.report.config') 
        config_id = config_obj.search(cr, uid, [('company_id', '=', data['form']['fiscalyear_id']),('year_id','=',data['form']['fiscalyear_id'])]) 
        config = config_obj.browse(cr, uid, config_id, context=context)
        config_line_obj = self.pool.get('account.stockholders.equity.config')
        conf_line_id = config_line_obj.search(cr, uid, [('report_id','=',config[0].id)])
        initial_account_ids = []
        res = []
        ress = []
        resss = []
        for res1 in config[0].report_id:
            initial_account_ids.append(res1['account_id'].id)
            res.append(res1['account_id'])
            ress.append(res1['init_balance1'])
            resss.append(res1['init_balance2'])
        number = 1
        rows = []
        context.update({
            'fiscalyear': fiscalyear.id,
            'date_from': data['form']['date_from'],
            'date_to':  data['form']['date_to'],
            'company_id': data['form']['company_id']
        })
        
        ### Эхний үлдэгдэл
        date_from_obj = datetime.strptime(data['form']['date_from'],'%Y-%m-%d')
        row = [str(number), u'<b>%s оны 12 сарын 31-ний үлдэгдэл</b>' % (int(date_from_obj.strftime('%Y'))-2)]
        initial_bal = account_obj.get_initial_balance(cr, uid, initial_account_ids, context=context)
        row.append(ress[0])
        row.append(ress[1])
        row.append(ress[2])
        row.append(ress[3])
        row.append(ress[4])
        row.append(ress[5])
        row.append(ress[6])
        row.append(Formula('sum(C8:I8)'))
        rows.append(row)
        
        ### Бүртгэлийн бодлогын өөрчлөлт
        number += 1 # line number
        ctx = context.copy()
        start_years = ''
        start_end_years = ''
        start = time.strftime("%Y-%m-%d")
        start_year = int(start[:4])
        start_years = str(start_year-2)+'-'+str(01)+'-'+str(01)
        start_end_years = str(start_year-2)+'-'+str(12)+'-'+str(31)
        ctx['date_from'] = start_years
        ctx['date_to'] = start_end_years
        
        acc_move = self.pool.get('account.move')
        acc_mv_line = self.pool.get('account.move.line')
        total = 0
        move_id = acc_move.search(cr, uid, [('id','=',config[0].property_adjustment_journal.id),('date','<=',start_years),('date','>=',start_end_years)])
        if move_id !=[]:
            line_id = acc_mv_line.search(cr, uid, [('move_id','=',move_id),('account_id','=',initial_account_ids)])
            if line_id !=[]:
                for lines in acc_mv_line.browse(cr, uid, line_id):
                 total = lines.debit - lines.credit   
        row = [str(number), u'Нягтлан бодох бүртгэлийн бодлогын өөрчлөлтийн нөлөө, алдааны залруулга','0', '0', '0', '0', '0', '0']
        row.append(total)
        row.append(Formula('SUM(C9:I9)'))
        rows.append(row)
        
        ### Залруулсан үлдэгдэл
        number += 1
        row = [str(number), u'Залруулсан үлдэгдэл']
        row.append(Formula('SUM(C8:C9)'))
        row.append(Formula('SUM(D8:D9)'))
        row.append(Formula('SUM(E8:E9)'))
        row.append(Formula('SUM(F8:F9)'))
        row.append(Formula('SUM(G8:G9)'))
        row.append(Formula('SUM(H8:H9)'))
        row.append(Formula('SUM(I8:I9)'))
        row.append(Formula('SUM(C10:I10)'))
        rows.append(row)
        
        ### Тайлант үеийн цэвэр ашиг
        number += 1
        row = [str(number), u'Тайлант үеийн цэвэр ашиг', '0', '0', '0', '0', '0', '0']
        a = account_obj.browse(cr, uid, res[0].id, ctx)
        row.append(a.debit-a.credit)
        row.append(Formula('SUM(C11:I11)'))
        rows.append(row)
        
        ### Бусад дэлгэрэнгүй орлого
        number += 1
        row = [str(number), u'Бусад дэлгэрэнгүй орлого','0','0','0',]
        revaluation_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','revaluation_addition_account')]))[0]
        foreign_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','foreign_currency_conversion_account')]))[0]
        remainder_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','stockholders_equity_remainder_account')]))[0]
        aobjs = account_obj.browse(cr, uid, [revaluation_account_id.account_id.id,foreign_account_id.account_id.id,remainder_account_id.account_id.id],
                                   ctx)
        row.append(aobjs[0].debit - aobjs[0].credit)
        row.append(aobjs[1].debit - aobjs[1].credit)
        row.append(aobjs[2].debit - aobjs[2].credit)
        row.extend(['0'])
        row.append(Formula('SUM(C12:I12)'))
        rows.append(row)
        
        ### Өмчид гарсан өөрчлөлт
        number += 1
        row = [str(number), u'Өмчид гарсан өөрчлөлт']
        context5 = context.copy()
        journal_id = config[0].property_adjustment_journal.id
        expense_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','statutory_fund_account')]))[0]
        context5.update({'journal_ids':[journal_id]})
        a = account_obj.browse(cr, uid, expense_account_id.account_id.id, context=context5)
        row.append(a.debit - a.credit)
        row.extend(['0','0', '0','0','0','0'])
        row.append(Formula('SUM(C13:I13)'))
        rows.append(row)
        
        ### Зарласан ногдол ашиг
        number += 1
        row = [str(number), u'Зарласан ногдол ашиг', '0','0','0','0','0','0']
        a = account_obj.browse(cr, uid, expense_account_id.account_id.id, ctx)
        row.append(a.debit - a.credit)
        row.append(Formula('SUM(C14:I14)'))
        rows.append(row)
        
        ### Дахин үнэлгээний нэмэгдлийн хэрэгжсэн дүн
        number += 1
        row = [str(number), u'Дахин үнэлгээний нэмэгдлийн хэрэгжсэн дүн', '0', '0', '0']
        a = account_obj.browse(cr, uid, expense_account_id.account_id.id, ctx)
        row.append(a.debit - a.credit)
        row.extend(['0','0','0'])
        row.append(Formula('SUM(C15:I15)'))
        rows.append(row)
        
        # Хөлийн дүн
        number += 1
        row = [str(number), u'<b>%s оны 12 сарын 31-ний үлдэгдэл</b>' % (int(date_from_obj.strftime('%Y'))-1)]
        initial_bal = account_obj.get_initial_balance(cr, uid, initial_account_ids, context=context)
        row.append(resss[0])
        row.append(resss[1])
        row.append(resss[2])
        row.append(resss[3])
        row.append(resss[4])
        row.append(resss[5])
        row.append(resss[6])
        row.append(Formula('SUM(C16:I16)'))
        rows.append(row)
        
        ### Бүртгэлийн бодлогын өөрчлөлт
        number += 1 # line number
        ctx2 = context.copy()
        date = time.strftime("%Y-%m-%d")
        year = int(date[:4])
        new_year = year-1
        month1 = 1
        day1 = 1
        month2 = 12
        day2 = 31
        new_date1 = ''
        new_date2 = ''
        new_date1 = str(new_year)+'-'+str(month1)+'-'+str(day1)
        new_date2 = str(new_year)+'-'+str(month2)+'-'+str(day2)
        ctx2['date_from'] = new_date1
        ctx2['date_to'] = new_date2
        total2 = 0
        move_ids = acc_move.search(cr, uid, [('id','=',config[0].property_adjustment_journal.id),('date','<=',new_date1),('date','>=',new_date2)])
        if move_ids !=[]:
            line_ids = acc_mv_line.search(cr, uid, [('move_id','=',move_ids),('account_id','=',initial_account_ids)])
            if line_ids !=[]:
                for liness in acc_mv_line.browse(cr, uid, line_ids):
                 total2 = liness.debit - liness.credit
        row = [str(number), u'Нягтлан бодох бүртгэлийн бодлогын өөрчлөлтийн нөлөө, алдааны залруулга']
        row.extend(['0','0','0','0','0','0'])
        row.append(total2)
        row.append(Formula('SUM(C17:I17)'))
        rows.append(row)
        
        ### Залруулсан үлдэгдэл
        number += 1
        row = [str(number), u'Залруулсан үлдэгдэл']
        row.append(Formula('SUM(C16:C17)'))
        row.append(Formula('SUM(D16:D17)'))
        row.append(Formula('SUM(E16:E17)'))
        row.append(Formula('SUM(F16:F17)'))
        row.append(Formula('SUM(G16:G17)'))
        row.append(Formula('SUM(H16:H17)'))
        row.append(Formula('SUM(I16:I17)'))
        row.append(Formula('SUM(C18:I18)'))
        rows.append(row)
        
        ### Тайлант үеийн цэвэр ашиг
        number += 1
        row = [str(number), u'Тайлант үеийн цэвэр ашиг', '0', '0', '0', '0', '0', '0']
        a = account_obj.browse(cr, uid, res[6].id, ctx2)
        row.append(a.debit-a.credit)
        row.append(a.debit-a.credit)
        rows.append(row)
        
        ### Бусад дэлгэрэнгүй орлого
        number += 1
        row = [str(number), u'Бусад дэлгэрэнгүй орлого', '0', '0', '0',]
        revaluation_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','revaluation_addition_account')]))[0]
        foreign_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','foreign_currency_conversion_account')]))[0]
        remainder_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','stockholders_equity_remainder_account')]))[0]
        aobjs = account_obj.browse(cr, uid, [revaluation_account_id.account_id.id,foreign_account_id.account_id.id,remainder_account_id.account_id.id],
                                   ctx2)
        row.append(aobjs[0].debit - aobjs[0].credit)
        row.append(aobjs[1].debit - aobjs[1].credit)
        row.append(aobjs[2].debit - aobjs[2].credit)
        row.extend(['0'])
        row.append(Formula('SUM(C20:I20)'))
        rows.append(row)
        
        ### Өмчид гарсан өөрчлөлт
        number += 1
        row = [str(number), u'Өмчид гарсан өөрчлөлт']
        expense_account_id = config_line_obj.browse(cr, uid, config_line_obj.search(cr, uid, [('type','=','statutory_fund_account')]))[0]
        a = account_obj.browse(cr, uid, expense_account_id.account_id.id, ctx2)
        row.append(a.debit - a.credit)
        row.extend(['0','0','0','0','0','0'])
        row.append(Formula('SUM(C21:I21)'))
        rows.append(row)
        
        ### Зарласан ногдол ашиг
        number += 1
        row = [str(number), u'Зарласан ногдол ашиг', '0']
        a = account_obj.browse(cr, uid, expense_account_id.account_id.id, ctx2)
        row.append(a.debit - a.credit)
        row.extend(['0', '0', '0', '0', '0'])
        row.append(Formula('SUM(C22:I22)'))
        rows.append(row)
        
        ### Дахин үнэлгээний нэмэгдлийн хэрэгжсэн дүн
        number += 1
        row = [str(number), u'Дахин үнэлгээний нэмэгдлийн хэрэгжсэн дүн', '0', '0', '0']
        a = account_obj.browse(cr, uid, expense_account_id.account_id.id, ctx2)
        row.append(a.debit - a.credit)
        row.extend(['0','0','0'])
        row.append(Formula('SUM(C23:I23)'))
        rows.append(row)
        
        # Хөлийн дүн
        number += 1
        date_to_obj = datetime.strptime(data['form']['date_to'],'%Y-%m-%d')
        row = [str(number), u'<b>%s оны %s сарын %s-ний үлдэгдэл</b>' % (date_to_obj.strftime('%Y'),
                                                                         date_to_obj.strftime('%m'),
                                                                         date_to_obj.strftime('%d'))]
        row.append(Formula('SUM(C17:C23)'))
        row.append(Formula('SUM(D17:D23)'))
        row.append(Formula('SUM(E17:E23)'))
        row.append(Formula('SUM(F17:F23)'))
        row.append(Formula('SUM(G17:H23)'))
        row.append(Formula('SUM(H17:H23)'))
        row.append(Formula('SUM(I17:I23)'))
        row.append(Formula('SUM(C24:I24)'))
        rows.append(row)
        return rows
    
    def get_export_data(self, cr, uid, ids, context=None):
        ''' Тайлангийн загварыг боловсруулж өгөгдлүүдийг
            тооцоолж байрлуулна.
        '''
        if context is None:
            context = {}
        datas = self.prepare_data(cr, uid, ids, context=context)
     
        data = datas['form']
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        )
        document_code = ''
        directory_name = False
        
        ezxf = xlwt.easyxf
        styles = self.get_easyxf_styles()
        book = xlwt.Workbook(encoding='utf8')
        sheet = book.add_sheet(u'Өмчийн өөрчлөлтийн тайлан')
        sheet.write(0, 0, u'Маягт CT-3', ezxf('font:bold on;align:wrap off,vert centre,horiz left;'))
        sheet.write(0, 3, u'Байгууллагын нэр: %s' %(company.name,), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        sheet.write(2, 1, u'ӨМЧИЙН ӨӨРЧЛӨЛТИЙН ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz left;font: height 250'))
        sheet.row(2).height = 400
        sheet.write(4, 0, u'Дугаар: %s' %(document_code,), ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet.write(4, 3, u'Тайлант хугацаа: %s' %(date_str,), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        
        rowx = 6
        sheet.write(rowx, 0, u'Д/д', styles['heading_xf'])
        sheet.write(rowx, 1, u'Үзүүлэлт', styles['heading_xf'])
        sheet.write(rowx, 2, u'Өмч', styles['heading_xf'])
        sheet.write(rowx, 3, u'Халаасны хувьцаа', styles['heading_xf'])
        sheet.write(rowx, 4, u'Нэмж төлөгдсөн капитал', styles['heading_xf'])
        sheet.write(rowx, 5, u'Дахин үнэлгээний нэмэгдэл', styles['heading_xf'])
        sheet.write(rowx, 6, u'Гадаад валютын хөрвүүлэлтийн нөөц', styles['heading_xf'])
        sheet.write(rowx, 7, u'Эздийн өмчийн бусад хэсэг', styles['heading_xf'])
        sheet.write(rowx, 8, u'Хуримтлагдсан ашиг', styles['heading_xf'])
        sheet.write(rowx, 9, u'Нийт дүн', styles['heading_xf'])
        rowx += 1
        rows = self.create_report_data(cr, uid, datas, context=context)
        for row in rows :
            sheet.write(rowx, 0, row[0], styles['text_xf'])
            if '<b>' in row[1]:
                sheet.write(rowx, 1, row[1].replace('<b>','').replace('</b>',''), styles['text_bold_xf'])
            else :
                sheet.write(rowx, 1, row[1], styles['text_xf'])
            colx = 2
            for td in row[2:]:
                sheet.write(rowx, colx, td, styles['number_xf'])
                colx += 1
            rowx += 1
         
        inch = 1000
        sheet.col(0).width = 2*inch
        sheet.col(1).width = 10*inch
        sheet.col(2).width = 4*inch
        sheet.col(3).width = 4*inch
        sheet.col(4).width = 4*inch
        sheet.col(5).width = 4*inch
        sheet.col(6).width = 4*inch
        sheet.col(7).width = 4*inch
        sheet.col(8).width = 4*inch
        sheet.col(9).width = 4*inch
        sheet.row(6).height = 700
        sheet.row(8).height = 600
        sheet.row(16).height = 600
        rowx += 2
        sheet.write(rowx, 1, u'Боловсруулсан :',ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet.write(rowx, 2, u'Гүйцэтгэх захирал....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet.write(rowx+2, 2, u'Ерөнхий нягтлан бодогч....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        return {'data':book}
account_stockholders_equity_report()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: