# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

from openerp.osv import osv, fields
from openerp import SUPERUSER_ID
import calendar
import time
import threading
import psycopg2
import logging
from datetime import datetime

import openerp
import openerp.tools
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
from openerp.tools.translate import _

_logger = logging.getLogger(__name__)

def detect_ip_addr2(ifname):
    import socket
    import fcntl
    import struct
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(s.fileno(), 0x8915, struct.pack('256s', ifname[:15]))[20:24])

class ir_attachment(osv.osv):
    _inherit = 'ir.attachment'
    
    def check(self, cr, uid, ids, mode, context=None, values=None):
        """Restricts the access to an ir.attachment, according to referred model
        In the 'document' module, it is overriden to relax this hard rule, since
        more complex ones apply there.
        """
        if not ids:
            return
        if mode == 'read':
            return
        return super(ir_attachment, self).check(cr, uid, ids, mode, context=context, values=values)
        
ir_attachment()


class ir_translation(osv.osv):
    _inherit = 'ir.translation'

    def create(self, cr, uid, vals, context=None):
        if uid <> SUPERUSER_ID and vals.get('type', 'field') <> 'model':
            self.pool.get('ir.model.access').check(cr, uid, 'ir.model.fields', 'write', True, context)
        return super(ir_translation, self).create(cr, uid, vals, context=context)

    def write(self, cursor, user, ids, vals, context=None):
        if user <> SUPERUSER_ID:
            for rd in self.read(cursor, user, ids, ['type'], context=context):
                print type(rd)
                if type(rd) is dict:
                    if vals.get('type', rd['type']) <> 'model':
                        self.pool.get('ir.model.access').check(cursor, user, 'ir.model.fields', 'write', True, context)
        return super(ir_translation, self).write(cursor, user, ids, vals, context=context)

ir_translation()


class ir_cron(osv.osv):
    _inherit = 'ir.cron'
    
    _columns = {
        'server': fields.char('Server Name', size=20, required=False)
    }
    
    _defaults = {
        'server': 'localhost'
    }
    
    def _run_jobs_multithread(self):
        # TODO remove 'check' argument from addons/base_action_rule/base_action_rule.py
        """ Process the cron jobs by spawning worker threads.

        This selects in database all the jobs that should be processed. It then
        tries to lock each of them and, if it succeeds, spawns a thread to run
        the cron job (if it doesn't succeed, it means the job was already
        locked to be taken care of by another thread).

        The cursor used to lock the job in database is given to the worker
        thread (which has to close it itself).

        """
        server_addr = detect_ip_addr2('eth0')
        server_where = ' and (server = \'%s\' or server = \'localhost\' or server is null) ' % server_addr
        db = self.pool.db
        cr = db.cursor()
        db_name = db.dbname
        try:
            jobs = {} # mapping job ids to jobs for all jobs being processed.
            now = datetime.now() 
            # Careful to compare timestamps with 'UTC' - everything is UTC as of v6.1.
            cr.execute("""SELECT * FROM ir_cron
                          WHERE numbercall != 0
                              AND active AND nextcall <= (now() at time zone 'UTC')
                        """ +server_where+ """
                          ORDER BY priority""")
            for job in cr.dictfetchall():
                if not openerp.cron.get_thread_slots():
                    break
                jobs[job['id']] = job

                task_cr = db.cursor()
                try:
                    # Try to grab an exclusive lock on the job row from within the task transaction
                    acquired_lock = False
                    task_cr.execute("""SELECT *
                                       FROM ir_cron
                                       WHERE id=%s
                                       FOR UPDATE NOWAIT""",
                                   (job['id'],), log_exceptions=False)
                    acquired_lock = True
                except psycopg2.OperationalError, e:
                    if e.pgcode == '55P03':
                        # Class 55: Object not in prerequisite state; 55P03: lock_not_available
                        _logger.debug('Another process/thread is already busy executing job `%s`, skipping it.', job['name'])
                        continue
                    else:
                        # Unexpected OperationalError
                        raise
                finally:
                    if not acquired_lock:
                        # we're exiting due to an exception while acquiring the lot
                        task_cr.close()

                # Got the lock on the job row, now spawn a thread to execute it in the transaction with the lock
                task_thread = threading.Thread(target=self._run_job, name=job['name'], args=(task_cr, job, now))
                # force non-daemon task threads (the runner thread must be daemon, and this property is inherited by default)
                task_thread.setDaemon(False)
                openerp.cron.take_thread_slot()
                task_thread.start()
                _logger.debug('Cron execution thread for job `%s` spawned', job['name'])

            # Find next earliest job ignoring currently processed jobs (by this and other cron threads)
            find_next_time_query = """SELECT min(nextcall) AS min_next_call
                                      FROM ir_cron WHERE numbercall != 0 AND active """+ server_where 
            if jobs:
                cr.execute(find_next_time_query + " AND id NOT IN %s", (tuple(jobs.keys()),))
            else:
                cr.execute(find_next_time_query)
            next_call = cr.dictfetchone()['min_next_call']

            if next_call:
                next_call = calendar.timegm(time.strptime(next_call, DEFAULT_SERVER_DATETIME_FORMAT))
            else:
                # no matching cron job found in database, re-schedule arbitrarily in 1 day,
                # this delay will likely be modified when running jobs complete their tasks
                next_call = time.time() + (24*3600)

            openerp.cron.schedule_wakeup(next_call, db_name)

        except Exception, ex:
            _logger.warning('Exception in cron:', exc_info=True)

        finally:
            cr.commit()
            cr.close()

ir_cron()

class ir_sequence(osv.osv):
    _inherit = 'ir.sequence'
    
    _columns = {
        'reset_monthly': fields.boolean('Reset sequence by monthly'),
    }
    
    _defaults = {
        'reset_monthly': False
    }
    
    def reset_next_value(self, cr, uid, context=None):
        ''' Тоолуурын дугаарлалтыг эхнээс нь эхлүүлнэ.
        '''
        if context is None:
            context = {}
        ids = self.search(cr, uid, [('reset_monthly','=',True)], context)
        self.write(cr, uid, ids, {'number_next':1}, context)
        message = u"Тайлан анхан шатны баримтын %s дугаарлалын тоолуурыг 1-ээс эхлүүллээ." % (len(ids),)
        _logger.debug("[BACKGROUND PROCESS] %s " % message)
        context.update({'scheduler_log':True})
        self.log(cr, uid, [], u"[BACKGROUND PROCESS] "+message, context=context)
        return True
    
ir_sequence()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: