# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    
#
##############################################################################

import logging
import time
from datetime import date, timedelta, datetime
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv, orm
from calendar import month, day_abbr, day_name
from openerp.exceptions import except_orm
from openerp.exceptions import Warning
import math
import sys
import urllib2, json
import requests
from openerp.tools.translate import _

class feedback_hubspot(osv.osv):
    _name = 'feedback.hubspot'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    
    _track = {
        'state': {
            'state': lambda self, cr, uid, obj, ctx=None: obj.state and obj.state.id,
        },
    }
    
    _columns = {
                'project_id': fields.many2one('project.project', u'Төсөл', track_visibility='onchange',required=True),#Төсөл
                'project_building_id': fields.many2one('project.building', u'Байр', track_visibility='onchange',required=True),#Байрны дугаар
                'project_door_id': fields.many2one('project.door', u'Тоот', track_visibility='onchange', copy=False,required=True),#Хаалганы тоот
                'is_erp_edit':fields.boolean(u'Сүүлийн ERP өөрчлөлт'),
                'states':fields.related('state', type='many2one',relation='feedback.state'),
                'attachment':fields.binary(u'Хавсралт'),#Хавсралт
                'ticket_name':fields.char(u'Гарсан санал хүсэлт',required=True),#Гарсан санал хүсэлт
                'name': fields.char(u'Дугаарлалт'),#Гарсан санал хүсэлт
                'date_to_solve': fields.date(u'Гүйцэтгэх хугацаа'),#Гүйцэтгэх хугацаа
                'date_solved': fields.date('Date solved'),#Гүйцэтгэсэн хугацаа
                'is_done': fields.boolean('Is done'),#Хийгдсэн эсэх
                'note': fields.text(u'Тайлбар'),#Тайлбар
                'phone_number': fields.char(u'Утасны дугаар'),#Утасны дугаар
                'user_id': fields.many2one('res.users',u'Гүйцэтгэгч ажилтан'),#Гүйцэтгэгч ажилтан
                'reviewer_id': fields.many2one('res.users',u'Хариуцагч'),#Хариуцагч
                'owner_name': fields.char(u'Эзэмшигч'),#Эзэмшигч
                'created_date':fields.datetime(u'Хүлээн авсан огноо'),
                'accepted_date': fields.date('Accepted date'),#Түлхүүр хүлээн авсан хугацаа
                'hubspot_id':fields.integer('Hubspot ID', help=u'Hubspot-ийн id',readonly=True), #hubspot id
                'ticket_type':fields.selection([(u'Санал хүсэлт',u'Санал хүсэлт'),
                                                (u'Гомдол',u'Гомдол'),
                                                (u'Өөрчлөлт',u'Өөрчлөлт'),
                                                (u'Байр хүлээлцэх актын дутуу ажил',u'Байр хүлээлцэх актын дутуу ажил')],string='Төрөл',required=True),
                #===============================================================
                # 'sub_type_new':fields.selection([(u'Агуулах, зогсоол',u'Агуулах, зогсоол'),
                #                                 (u'Актны дутуу ажил',u'Актны дутуу ажил'),
                #                                 (u'Борооны ус',u'Борооны ус'),
                #                                 (u'Бусад',u'Бусад'),
                #                                 (u'Гадна зам талбай',u'Гадна зам талбай'),
                #                                 (u'Засал',u'Засал'),
                #                                 (u'Сантехник',u'Сантехник'),
                #                                 (u'Тавилга',u'Тавилга'),
                #                                 (u'Тоноглол',u'Тоноглол'),
                #                                 (u'Холбоо дохиолол',u'Холбоо дохиолол'),
                #                                 (u'Цахилгаан',u'Цахилгаан'),
                #                                 (u'Цонх',u'Цонх'),
                #                                 (u'Цэвэрлэгээ',u'Цэвэрлэгээ')],string='Дэд төрөл',required=True),
                #===============================================================

                'source_type':fields.selection([('FORM',u'Form'),
                                                ('FROM',u'Дамжуулж'),
                                                ('PHONE',u'Утсаар'),
                                                ('LETTERS',u'Бичиг'),
                                                ('EMAIL',u'И-мэйл'),
                                                ('MEETING',u'Уулзалт'),
                                                ('SOCIAL',u'Сошиал'),
                                                ],string='Эх сурвалж',required=True),
                
                'ticket_priority':fields.selection([('LOW','LOW'),
                                                    ('MEDIUM','MEDIUM'),
                                                    ('HIGH','HIGH'),
                                                    ],'Урьтамж',required=True),
                'progress':fields.char(u'Ажлын явц'),
                'state':fields.many2one('feedback.state',u'Төлөв',required=True,select=True),
                'receiver':fields.many2one('feedback.receiver',u'Хүлээн авагч',required=True),
                'last_update':fields.date('Last update'),
                'ticket_owner':fields.many2one('res.users','Ticket owner'),
                'sub_type_new':fields.many2one('sub.type.new','Дэд төрөл шинэ',required=False),
                'ticket_sub_type':fields.many2many('feedback.sub.type','ticket_hub_sub_type_rel','sol_id','sub_type_id','Дэд төрөл',required=False),
                'hs_pipeline':fields.selection([('128294',u'Явц')
                                                ],string='HS pipeline',required=True,readonly=True),
                'is_created':fields.boolean('Is create ticket'),
                'user_mail_created': fields.boolean('User mail created'),#Гүйцэтгэгч ажилтан мэйл илгээсэн эсэх
                
        }
    
    
    
    
    _defaults = {
                'name':lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'feedback.hubspot'),
                'hs_pipeline':'128294',
                'is_created':False,
                'is_erp_edit':False,
                'user_mail_created': False,
                'hubspot_id':0,
        }
    
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid, ids)        
        #=======================================================================
        # if vals:
        #     vals.update({'is_erp_edit':True})
        #=======================================================================
            
        edit = False
        if 'project_id' in vals:
            edit = True
        if 'project_building_id' in vals:
            edit = True
        if 'project_door_id' in vals:
            edit = True
        if 'ticket_name' in vals:
            edit = True
        if 'note' in vals:
            edit = True
        if 'ticket_sub_type' in vals:
            edit = True
        if 'sub_type_new' in vals:
            edit = True
        if 'source_type' in vals:
            edit = True
        if 'ticket_priority' in vals:
            edit = True
        if 'receiver' in vals:
            edit = True
            
        if 'user_id' in vals:
            edit = True
        if 'reviewer_id' in vals:
            edit = True
        if 'phone_number' in vals:
            edit = True
        if 'owner_name' in vals:
            edit = True
        if 'attachment' in vals:
            edit = True
            
        vals.update({'is_erp_edit':edit})
            
        print "aaaaa",vals
        return super(feedback_hubspot, self).write(cr, uid, ids, vals, context=context)
    
    
    
    
    