# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    
#
##############################################################################

import logging
import time
from datetime import date, timedelta, datetime
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv, orm
from calendar import month, day_abbr, day_name
from openerp.exceptions import except_orm
from openerp.exceptions import Warning
import math
import sys
import urllib2, json
import requests
from openerp.tools.translate import _

class hubspot_conf(osv.osv):
    _name = 'hubspot.conf'
    _description = 'Hubspot configuration'
    
    _inherit = ['mail.thread', 'ir.needaction_mixin']



    _columns = {
                'name':fields.char('Configuration Name',required=True, help=u'Тохиргооны нэр'), #Тохиргооны нэр
                'hapi_key': fields.char('HAPI Key', required=True, help=u'Hubspot API key оруулна'),#HAPI key 
                'arguments':fields.char('Arguments',required=True),
                'company_id':fields.many2one('res.company','Company',required=True),
                'url':fields.char('Ticket all data get url',required=True),
                'create_ticket_url':fields.char('Ticket create url',required=True),
                'update_ticket_url':fields.char('Ticket update url',required=True),
                'update_date':fields.date('Hubspot update date'),
                'is_ticket_create': fields.boolean('Odoo ERP Create'),
                'is_ticket_update': fields.boolean('Odoo ERP Update'),
                'is_ticket_delete': fields.boolean('Odoo ERP Delete'),
                'is_ticket_create_hubspot': fields.boolean('Hubspot Create'),
                'is_ticket_update_hubspot': fields.boolean('Hubspot Update'),
                'is_ticket_delete_hubspot': fields.boolean('Hubspot Delete'),
                'log_ids':fields.one2many('hubspot.log','config_id','Log'),
                'ticket_field_ids':fields.one2many('ticket.field','config_id','Ticket'),
    }
    
    _defaults = {
                'company_id':lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'hubspot.conf',context=c),
                'update_date':lambda obj, cr, uid, context: time.strftime('%Y-%m-%d'),
                
        }


    def action_get_property_all_ticket(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        get_url = 'https://api.hubapi.com/properties/v2/tickets/properties?hapikey=%s'%obj.hapi_key
        headers = {}
        r = requests.get(url= get_url, headers = headers)
        get_data = json.loads(r.text)
        ticket_field_obj = self.pool.get('ticket.field')
        for line in get_data:
            ticket_ids = ticket_field_obj.search(cr, uid, [('name','=',u'%s'%line['name'])])
            if ticket_ids==[]:
                ticket_field_obj.create(cr, uid, {
                                                'company_id':obj.company_id.id,
                                                'name':u'%s'%line['name'],
                                                'desc':u'%s'%line['label'],
                                                'config_id':obj.id,
                                                })
        
        return True
    
    
    def log_clear(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        log_obj = self.pool.get('hubspot.log')
        if obj.log_ids:
            for log_lines in obj.log_ids:
                log_obj.unlink(cr, uid, log_lines.id)
        return True
    
    def action_create_ticket(self,cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids)
        feed_back_obj = self.pool.get('feedback.act')
        feed_solution_obj = self.pool.get('feedback.hubspot')
        log_obj = self.pool.get('hubspot.log')
        conf_obj = self.pool.get('hubspot.conf')
        #feed_ids = feed_solution_obj.search(cr, uid, [])
        feed_ids = feed_solution_obj.search(cr, uid, [('is_created','=',False),('hubspot_id','=',0)],limit=1)
        parameter = []
        headers = {}
        if obj.is_ticket_create_hubspot == True:
            if feed_ids!=[]:
                for sol_data in feed_solution_obj.browse(cr, uid, feed_ids):
                    send_data = {}
                    project = {}
                    content = {}
                    subject = {}
                    engineer = {}
                    pipeline = {}
                    build = {}
                    toot = {}
                    pipeline_stage = {}
                    owner = {}
                    priority = {}
                    day = {}
                    sub_type = {}
                    progress = {}
                    rec = {}
                    source = {}
                    sub_type_new = {}
                    if sol_data.is_created == False and sol_data.hubspot_id==0:
                        if sol_data.project_id:
                            project['name']='project'
                            project['value']=sol_data.project_id.code
                            parameter.append(project)
                        
                        if sol_data.accepted_date:
                            rec_date =  int(time.mktime(datetime.strptime(sol_data.accepted_date, "%Y-%m-%d").timetuple())*1000.0)
                            day['name']='received_day'
                            day['value']=rec_date
                            parameter.append(day)
                            
                        if sol_data.progress:
                            progress['name']='progress'
                            progress['value']=sol_data.progress
                            parameter.append(progress)
                            
                        if sol_data.project_door_id:
                            toot['name'] = 'toot'
                            toot['value'] = sol_data.project_door_id.name
                            parameter.append(toot)
                        
                        if sol_data.project_building_id:
                            build['name'] = 'barilga'
                            build['value'] = sol_data.project_building_id.name
                            parameter.append(build)
                        if sol_data.ticket_type:
                            send_data['name']='type'
                            send_data['value']=sol_data.ticket_type
                            parameter.append(send_data)
                            
                        if sol_data.receiver:
                            rec['name']='receiver'
                            rec['value']=u'%s'%sol_data.receiver.name
                            parameter.append(rec)
                            
                        if sol_data.sub_type_new:
                            print 'HHHHHHHHHH', sub_type_new
                            sub_type_new['name']='sub_type_new'
                            sub_type_new['value']=sol_data.sub_type_new.name
                            parameter.append(sub_type_new)
                            print 'dddddddddddd', parameter
                            
                        if sol_data.ticket_sub_type:
                            #sol_data.ticket_sub_type
                            sub_type_data = ''
                            if sol_data.ticket_sub_type:
                                for li in sol_data.ticket_sub_type:
                                    if len(sol_data.ticket_sub_type)==1:
                                        sub_type_datas = li.name
                                    else:
                                        sub_type_data += li.name+';'
                                        sub_type_datas = sub_type_data[:-1]
                            sub_type['name']='sub_type'
                            sub_type['value']=sub_type_datas
                            parameter.append(sub_type)
                        if sol_data.source_type:
                            source['name']='source_type'
                            source['value']=sol_data.source_type
                            parameter.append(source)
                            
                        if sol_data.note:
                            content['name']='content'
                            content['value']=sol_data.note
                            parameter.append(content)
                            
                        if sol_data.ticket_name:
                            subject['name']='subject'
                            subject['value']=sol_data.ticket_name
                            parameter.append(subject)
                        if sol_data.ticket_owner:
                            owner['name']='hubspot_owner_id'
                            owner['value'] = sol_data.ticket_owner
                            parameter.append(owner)
                            
                        if sol_data.hs_pipeline:  
                            pipeline['name'] = 'hs_pipeline'
                            pipeline['value'] = sol_data.hs_pipeline
                            parameter.append(pipeline)
                            
                        state = ''
                        if sol_data.state:
                            state = sol_data.state.code
                            pipeline_stage['name'] = 'hs_pipeline_stage'
                            pipeline_stage['value'] = state
                            parameter.append(pipeline_stage)
                        
                        if sol_data.ticket_priority:  
                            priority['name'] = 'hs_ticket_priority'
                            priority['value'] = sol_data.ticket_priority
                            parameter.append(priority)
                            
                        if sol_data.user_id.hubspot_id:
                            engineer['name'] = 'engineer'
                            engineer['value'] = sol_data.user_id.hubspot_id
                            parameter.append(engineer)
                        
                
                        APIKEY = "hapikey=" + obj.hapi_key
                        HS_API_URL = obj.create_ticket_url
                        header  = {
                                    'Connection': 'keep-alive',
                                    'Content-Type':'application/json; charset=utf-8',
                                     }
                        # First, we build the correct url
                        get_url = HS_API_URL + APIKEY
                        
                        if sol_data.project_id and sol_data.project_door_id and sol_data.project_building_id:
                           json_datas = json.dumps(parameter)
                           r = requests.post(url= get_url, data = json_datas,headers = header)
                           if r.status_code == 200:
                               get_data = json.loads(r.text)
                               hubspot_id = ''
                               try:
                                    hubspot_id = get_data['properties']['hs_ticket_id']['value']
                                    #print 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA', get_data
                               except Exception, e:
                                   print e
                               feed_solution_obj.write(cr, uid, sol_data.id,{'hubspot_id':hubspot_id,
                                                                              'is_created':True,
                                                                              'is_erp_edit':False,
                                                                     })
                           else:
                                last_date = time.strftime('%Y-%m-%d %H:%M:%S')
                                conf_id = conf_obj.search(cr, uid, [('company_id','=',obj.company_id.id)])
                                log_obj.create(cr, uid, {'company_id':obj.company_id.id,
                                                        'date':last_date,
                                                        'name':u'%s'%r.text,
                                                         'config_id':conf_id[0],
                                                       #  'ticket_id':lines.hubspot_id,
                                                        })
                
        return True
    

    def action_get_all_ticket(self, cr, uid, ids, *args):
        # Change the APIKEY_VALUE by your API key
        feedback_act_obj = self.pool.get('feedback.act')
        feedback_solution_obj = self.pool.get('feedback.hubspot')
        project_obj = self.pool.get('project.project')
        build_obj = self.pool.get('project.building')
        state_obj = self.pool.get('feedback.state')
        door_obj = self.pool.get('project.door')
        sub_type_new_obj = self.pool.get('sub.type.new')
        user_obj = self.pool.get('res.users')
        receive_obj = self.pool.get('feedback.receiver')
        obj = self.browse(cr, uid, ids)
        APIKEY = "hapikey=" + obj.hapi_key
        HS_API_URL = obj.url
        url_argument = obj.arguments
        headers = {}
        # First, we build the correct url
        get_url = HS_API_URL + APIKEY + url_argument
        # Now we use urllib2 to open the url and read it
        r = requests.get(url= get_url, headers = headers)
        
        # Transform the response, a JSON object, into a Python dictionary object
        
        # Finally, return the number of contacts
        if r.status_code == 200:
            get_data = json.loads(r.text)
            
            for datas in get_data['objects']:
                if datas['properties']['hs_ticket_id']['value'] == '47277137':
                    print '--------------------------------------------------------------------------------------------------------', datas['properties']['hs_ticket_id']['value']
                partner_data = ''
                ticket_desc = ''
                ticket_id = ''
                priority = ''
                barilga = ''
                toot = ''
                project = ''
                sub_type = ''
                sub_type_new = ''
                hubspot_owner_id = ''
                source_type = ''
                type = ''
                phone = ''
                ticket_name = ''
                hs_pipeline_stage =''
                hs_pipeline = ''
                new_date = ''
                status = ''
                state = False
                engineer = ''
                contact = ''
                dates = ''
                hs_file_upload_data = ''
                progress = ''
                received_day = False
                receiver = False
                receivers = ''
                try:
                    hs_file_upload_data = datas['properties']['hs_file_upload']['sourceId']
                except Exception, e:
                    print e
                    
                try:
                    received_day = datas['properties']['received_day']['value']
                except Exception, e:
                    print e
                    
                try:
                    progress = datas['properties']['progress']['value']
                except Exception, e:
                    print e
                    
                try:
                    receivers = datas['properties']['receiver']['value']
                except Exception, e:
                    print e
                
                try:
                    partner_data = datas['properties']['content']['sourceId']
                except Exception, e:
                    print e

                try:
                    status = datas['properties']['hs_pipeline_stage']['versions']
                except Exception, e:
                    print e
                try:
                    ticket_name = datas['properties']['subject']['value']
                except Exception, e:
                    print e
                    
                    
                try:
                    phone = datas['properties']['phone_number']['value']
                except Exception, e:
                    print e
                try:
                    project = datas['properties']['project']['value']
                except Exception, e:
                    print e
                    
                try:
                    barilga = datas['properties']['barilga']['value']
                except Exception, e:
                    print e
                    
                try:
                    toot = datas['properties']['toot']['value']
                except Exception, e:
                    print e   
                    
                try:
                    contact = datas['properties']['contacts']['value']
                except Exception, e:
                    print e 
                try:
                    ticket_desc = datas['properties']['content']['value']
                except Exception, e:
                    print e
                    
                try:
                    type = datas['properties']['type']['value']
                    
                except Exception, e:
                    print e
                    
                try:
                    priority = datas['properties']['hs_ticket_priority']['value']
                    
                except Exception, e:
                    print e
                    
                try:
                    ticket_id = datas['properties']['hs_ticket_id']['value']
                    print 'OOOOOOOOOOOOOOOOOOOO', ticket_id
                    if ticket_id == 47277137:
                        'WWWWWWWWWWWWWWWWWWWWWWWWW', ticket_id
                except Exception, e:
                    print e
                    
                try:
                    sub_type_new = datas['properties']['sub_type_new']['value']
                except Exception, e:
                    print e
                try:
                    sub_type = datas['properties']['sub_type']['value']
                except Exception, e:
                    print e
                    
                try:
                    hubspot_owner_id = datas['properties']['hubspot_owner_id']['value']
                except Exception, e:
                    print e
                    
                try:
                    source_type = datas['properties']['source_type']['value']
                except Exception, e:
                    print e
                    
                try:
                    hs_pipeline = datas['properties']['hs_pipeline']['value']
                except Exception, e:
                    print e
                    
                try:
                    new_date = datas['properties']['createdate']['value']
                except Exception, e:
                    print e
                    
                try:
                    engineer = datas['properties']['engineer']['value']
                except Exception, e:
                    print e
                dates = datetime.fromtimestamp(int(new_date)/1000).strftime("%Y-%m-%d %H:%M:%S")
                project_id = project_obj.search(cr, uid, [('code','=',str(project))])
                if project_id !=[]:
                    project_id = project_id[0]
                else:
                    project_id = False
                
                build_id = build_obj.search(cr, uid, [('name','=',str(barilga)),('project_id','=',int(project_id))])
                if build_id !=[]:
                    build_id = build_id[0]
                    
                
                #door_id = door_obj.search(cr, uid, [('name','=',str(toot)),('project_id','=',int(project_id)),('building_id','=',int(build_id))])
                door_id = door_obj.search(cr, uid, [('name','=',str(toot))])
                if door_id !=[]:
                    door_id = door_id[0]
                
                sub_type_new_obj
                sub_type_new_id = sub_type_new_obj.search(cr, uid, [('name','=',str(sub_type_new))])
                if sub_type_new_id !=[]:
                    sub_type_new_id = sub_type_new_id[0]
                
                feed_back_id = []
                if project_id != False:
                    print 'ttttttt', project_id,door_id,build_id,ticket_id,barilga
                    #feed_back_id = feedback_solution_obj.search(cr, uid, [('project_id','=',int(project_id)),('project_door_id','=',int(door_id)),('project_building_id','=',int(build_id)),('name','=',str(ticket_name))])
                    feed_back_id = feedback_solution_obj.search(cr, uid, [('project_id','=',int(project_id)),('project_door_id','=',int(door_id)),('project_building_id','=',int(build_id)),('hubspot_id','=',ticket_id)])
                    print 'tttttttEEEEEEEEEEEEEE', feed_back_id, '------------------------'
                    
                    if not feed_back_id:
                        'WWWWWWWWWWWWWWWWWWWWWWWWW', ticket_id
                
                state_ids = state_obj.search(cr, uid,[('code','=',status[0]['value'])])
                if state_ids:
                    states = state_obj.browse(cr, uid, state_ids[0])
                    state = states.id
                
                #sub_type_new_id = sub_type_new_obj.search(cr, uid, [('name','=',str(toot))])
                
                rec_id = receive_obj.search(cr, uid, [('name','=',str(receivers))])
                if rec_id:
                    receiver = rec_id[0]
                #print 'dddddd222222222dddddddddd', sub_type, sub_type_new
                type_data = sub_type.split(";")
                if type_data[0]!=' ' or type_data[0] !=[]:
                    if len(type_data)== 1:
                        type_datas = self.pool.get('feedback.sub.type').search(cr, uid, [('name','=',u'%s'%type_data[0])])
                    elif len(type_data) == 2:
                        type_datas = self.pool.get('feedback.sub.type').search(cr, uid, [('name','in',[u'%s'%type_data[0],u'%s'%type_data[1]])])
                    elif len(type_data) == 3:
                        type_datas = self.pool.get('feedback.sub.type').search(cr, uid, [('name','in',[u'%s'%type_data[0],u'%s'%type_data[1],u'%s'%type_data[2]])])
                
                type_data2 = sub_type_new.split(";")
                if type_data2[0]!=' ' or type_data2[0] !=[]:
                    if len(type_data2)== 1:
                        type_datas2 = self.pool.get('sub.type.new').search(cr, uid, [('name','=',u'%s'%type_data2[0])])
                    elif len(type_data2) == 2:
                        type_datas2 = self.pool.get('sub.type.new').search(cr, uid, [('name','in',[u'%s'%type_data2[0],u'%s'%type_data2[1]])])
                    elif len(type_data2) == 3:
                        type_datas2 = self.pool.get('sub.type.new').search(cr, uid, [('name','in',[u'%s'%type_data2[0],u'%s'%type_data2[1],u'%s'%type_data2[2]])])
                
                feed_sol_id = feedback_solution_obj.search(cr, uid, [('hubspot_id','=',ticket_id)])
                if type_datas==[]:
                    type_datas = []
                else:
                    type_datas = [type_datas]
                user_id = False
                if engineer:
                    users = user_obj.search(cr, uid, [('hubspot_id','=',engineer)])
                    if users:
                        user_id = user_obj.browse(cr, uid, users).id
                re_dates = False
                if received_day != False:
                    re_dates = datetime.fromtimestamp(int(received_day)/1000).strftime("%Y-%m-%d") 
                if feed_back_id == [] or feed_back_id[0] == False:
                    print '11111__________________________', feed_sol_id
                    if project_id and door_id and build_id:
                        print '222222__________________________', feed_sol_id
                        if feed_sol_id ==[] or feed_sol_id[0] == False:
                            print '333333__________________________', feed_sol_id
                            lines = {                               'project_door_id':door_id,
                                                                    'accepted_date':re_dates,
                                                                    'project_building_id':build_id,
                                                                    'project_id':project_id,
                                                                    'ticket_name':ticket_name,
                                                                    'user_id':user_id,
                                                                    'phone_number':phone,
                                                                    'created_date':dates,
                                                                    'source_type':source_type,
                                                                    'sub_type_new':sub_type_new_id,
                                                                    'ticket_sub_type':[(6,0,type_datas)],
                                                                    'ticket_type':u'%s'%type,
                                                                    'hubspot_id':ticket_id, 
                                                                    'is_created':True,
                                                                    'is_erp_edit':False,
                                                                    'receiver':receiver,
                                                                    
                                                                    'hs_pipeline':hs_pipeline,
                                                                    'progress':progress,
                                                                   # 'attachment':hs_file_upload_data,
                                                                    'state':state,
                                                                    'ticket_priority':priority, 
                                                                    'note':ticket_desc,
                                                                    
                                }
                            feedback_solution_obj.create(cr, uid, lines)
                else:
                    print '44444__________________________', feed_sol_id
                    if feed_back_id:
                        fee_id = feedback_solution_obj.browse(cr, uid, feed_sol_id[0])
                        if fee_id.is_erp_edit == False:
                            feedback_solution_obj.write(cr, uid, feed_sol_id[0],{
                                                                    'project_door_id':door_id,
                                                                    'accepted_date':re_dates,
                                                                    'project_building_id':build_id,
                                                                    'project_id':project_id,
                                                                    'ticket_name':ticket_name,
                                                                    'user_id':user_id,
                                                                    'created_date':dates,
                                                                    'source_type':source_type,
                                                                    'sub_type_new':sub_type_new_id,
                                                                    'progress':progress,
                                                                    'receiver':receiver,
                                                                 #   'attachment':hs_file_upload_data,
                                                                    'ticket_sub_type':[(6,0,type_datas)],
                                                                    'ticket_type':u'%s'%type,
                                                                    'phone_number':phone,
                                                                    'state':state,
                                                                    'is_erp_edit':False,
                                                                    'is_created':True,
                                                                    'hs_pipeline':hs_pipeline,
                                                                    'ticket_priority':priority, 
                                                                    'note':ticket_desc,
                                                                    })
                    

                        
        
        return True

    
    
    
    
    
    
    def action_update_ticket(self, cr, uid, ids, *args):
        # Change the APIKEY_VALUE by your API key
        obj = self.browse(cr, uid, ids)
        feedback_solution_obj = self.pool.get('feedback.hubspot')
        log_obj = self.pool.get('hubspot.log')
        conf_obj = self.pool.get('hubspot.conf')
        feedback_ids = feedback_solution_obj.search(cr, uid, [('is_erp_edit','=',True)])
        if feedback_ids:
            for lines in feedback_solution_obj.browse(cr, uid, feedback_ids):
                    if lines.hubspot_id !=0:
                        api = "?hapikey=" + obj.hapi_key
                        update_url = obj.update_ticket_url
                        ticket_id = lines.hubspot_id
                        # First, we build the correct url
                        get_urls = update_url + str(ticket_id) + api
                        para_data = []
                        #vals.update({'is_erp_edit':True})
                        toot_data = ''
                        sub_type_new_data = ''
                        headers = {
                                    'Connection': 'keep-alive',
                                    'Content-Type':'application/json; charset=utf-8',
                                }
                        state = ''
                        if lines.state:
                            state = lines.state.code
                            
                        states = {
                                "name":"hs_pipeline_stage",
                                'value':state,
                                }
                        if state !='':
                            para_data.append(states)
                            
                            
                        receiver = ''
                        if lines.receiver:
                            receiver = lines.receiver.name
                            
                        receivers = {
                                "name":"receiver",
                                'value':receiver,
                                }
                        if receiver !='':
                            para_data.append(receivers)
                            
                        progress = {
                                    "name":"progress",
                                    "value":u'%s'%lines.progress,
                            }
                        if lines.progress:
                            para_data.append(progress)
                        contents = {
                                    "name":"content",
                                    "value":u"%s"%lines.note
                            }
                        
                        if lines.note:
                            para_data.append(contents)
                            
                        subject_name = {
                                        "name":"subject",
                                        "value":u'%s'%lines.ticket_name
                            
                            }
                        if lines.ticket_name:
                            para_data.append(subject_name)
                        if lines.project_door_id:
                            toot_data = lines.project_door_id.name
                        
                        toot = {
                            
                                "name":"toot",
                                "value":u'%s'%toot_data,
                                }
                        if toot_data!='':
                            para_data.append(toot)
                            
                            
                        build_data = ''
                        if lines.project_building_id:
                            build_data = lines.project_building_id.name
                        barilga = {
                                    "name":"barilga",
                                    "value":u'%s'%build_data,
                            }
                        if build_data !='':
                            para_data.append(barilga)
                            
                        project_data = ''
                        if lines.project_id:
                            project_data = lines.project_id.code
                        project = {
                                    "name":"project",
                                    "value":u'%s'%project_data,
                            }
                        if project_data !='':
                            para_data.append(project)
                        type = {
                                "name":"type",
                                "value":u"%s"%lines.ticket_type,
                                }
                        if type !='':
                            para_data.append(type)
                            
                        if lines.sub_type_new:
                            sub_type_new_data = lines.sub_type_new.name
                        
                        sub_type_new = {
                                "name":"sub_type_new",
                                "value":u'%s'%sub_type_new_data,
                                }
                        
                        if sub_type_new_data!='':
                            para_data.append(sub_type_new)
                            
                        #=======================================================
                        # sub_type_new= {
                        #         "name":"sub_type_new",
                        #         "value":u"%s"%lines.sub_type_new,
                        #         }
                        # if sub_type_new !='':
                        #     para_data.append(sub_type_new)
                        # print 'PPPPPPPPPPPPP',sub_type_new, para_data
                        #=======================================================
                        sub_type_data = ''
                        sub_type_datas = ''
                        if lines.ticket_sub_type:
                            for li in lines.ticket_sub_type:
                                if len(lines.ticket_sub_type)==1:
                                        sub_type_datas = li.name
                                else:
                                    sub_type_data += li.name+';'
                                    sub_type_datas = sub_type_data[:-1]
                        sub_type = {
                                    "name":"sub_type",
                                    "value":u"%s"%sub_type_datas,
                            }
                        para_data.append(sub_type)
                        
                        priority = {
                                    "name":"hs_ticket_priority",
                                    "value":lines.ticket_priority,
                            }
                        para_data.append(priority)
                        
                        source_type = {
                                        "name":"source_type",
                                        "value":u"%s"%lines.source_type,
                            }
                        para_data.append(source_type)
                        
                        if lines.accepted_date:
                            rec_day = int(time.mktime(datetime.strptime(lines.accepted_date, "%Y-%m-%d").timetuple())*1000.0)
                            recevice_day = {
                                            "name":"received_day",
                                            "value":rec_day,
                                }
                            
                            para_data.append(recevice_day)

                        engineer = ''
                        if lines.user_id.hubspot_id:
                            engineer = lines.user_id.hubspot_id
                        #=======================================================
                        # elif lines.assigned_to.hubspot_id:vals.update({'is_erp_edit':True})
                        #     engineer = lines.assigned_to.hubspot_id
                        #=======================================================
                        engineer_data = {
                                        "name":"engineer",
                                        "value":u"%s"%engineer,
                                    }
                        
                        para_data.append(engineer_data)
                        jdata = json.dumps(para_data)
                        if lines.project_id and lines.project_building_id and lines.project_door_id:
                            r = requests.put(url= get_urls, data = jdata,headers = headers)
                            lasts_date = fields.datetime.now()
                            if r.status_code == 200:
                                last_date = time.strftime('%Y-%m-%d %H:%M:%S')
                                feedback_solution_obj.write(cr, uid, lines.id,{'is_erp_edit':False})
                            else:
                                conf = False
                                conf_id = conf_obj.search(cr, uid, [])
                                if conf_id == []:
                                    raise osv.except_osv(_(u'Анхааруулга!'),_(u'Hubspot тохиргоо хийгдсэн эсэхийг шалгана уу'))
                                else:
                                    conf = conf_id[0]

                                log_obj.create(cr, uid, {
                                                        #'company_id':lines.company_id.id,
                                                        'name':r.text,
                                                        'date':lasts_date,
                                                        'config_id':conf,
                                                        'ticket_id':lines.hubspot_id,
                                    })
                        else:
                            raise osv.except_osv(_(u'Анхааруулга!'),_(u'Төсөл, байр, тоот аль нэг нь хоосон байгаа тул та шалгана уу!'))
                        

        return True
    
       
        is_erp_edit
    
            
#===============================================================================
# class feedback_act(osv.osv):
#     _inherit='feedback.act'
#     
#     
#     _columns = {
#                 'is_created':fields.boolean('Is create ticket'),
#                 'company_id':fields.many2one('res.company','Company'),
#                 'last_update':fields.datetime(u'Шинэчлэгдсэн огноо'),
#                 'created_date':fields.date(u'Үүсгэсэн огноо'),
#                 'ticket_partner':fields.char('Ticket partner'),
#                 #===============================================================
#                 # 'state':fields.selection([('draft','Draft'),
#                 #                             ('process','Process'),
#                 #                             ('cancel','Cancel'),
#                 #                             ('transfer','Transfer'),
#                 #                             ('yaraltai',u'Яаралтай'),
#                 #                             ('no_answer',u'Хариу өгөөгүй'),
#                 #                              ('non_solution',u'Шийдэлгүй'),
#                 #                               ('solution',u'Шийдвэрлэсэн'),
#                 #                             ('stop','Stop'),
#                 #                             ('closed','Closed')],'State', track_visibility='onchange'),
#                 #===============================================================
#                 }    
# 
# 
#     _defaults = {
#                 'company_id':lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'feedback.act', context=c),
#         }
#     def unlink(self, cr, uid, ids, context=None):
#        obj = self.browse(cr, uid, ids)[0]
#        line_obj = self.pool.get('feedback.solution')
#        for line in obj.solution_ids:
#            line_obj.unlink(cr, uid, line.id)
#        return super(feedback_act, self).unlink(cr, uid, ids, context=context)
# 
# class feedback_solution(osv.osv):
#     _inherit = 'feedback.solution'
# 
#     _columns = {
#                 'created_date':fields.datetime(u'Үүссэн огноо'),
#                 'hubspot_id':fields.integer('Hubspot ID', help=u'Hubspot-ийн id',readonly=True), #hubspot id
#                 'ticket_type':fields.selection([(u'Санал хүсэлт',u'Санал хүсэлт'),
#                                                 (u'Гомдол',u'Гомдол'),
#                                                 (u'Өөрчлөлт',u'Өөрчлөлт')],string='Ticket type',required=True),
#                 'source_type':fields.selection([('FORM',u'Form'),
#                                                 ('FROM',u'Дамжуулж'),
#                                                 ('PHONE',u'Утсаар'),
#                                                 ('LETTERS',u'Бичиг'),
#                                                 ('EMAIL',u'И-мэйл'),
#                                                 ('MEETING',u'Уулзалт'),
#                                                 ('SOCIAL',u'Сошиал'),
#                                                 ],string='Source',required=True),
#                 
#                 'ticket_priority':fields.selection([('LOW','LOW'),
#                                                     ('MEDIUM','MEDIUM'),
#                                                     ('HIGH','HIGH'),
#                                                     ],'Ticket priority',required=True),
#                 'progress':fields.char(u'Ажлын явц'),
#                  'state':fields.many2one('feedback.state',u'Төлөв',required=True),
#                 'receiver':fields.many2one('feedback.receiver',u'Хүлээн авагч',required=True),
#                 'last_update':fields.date('Last update'),
#                 'ticket_owner':fields.many2one('res.users','Ticket owner'),
#                 'ticket_sub_type':fields.many2many('feedback.sub.type','ticket_sub_type_rel','sol_id','sub_type_id','Ticket sub type',required=True),
#                 'hs_pipeline':fields.selection([('128294',u'Явц')
#                                                 ],string='hs_pipeline',required=True),
#                 'is_created':fields.boolean('Is create ticket'),
# 
#         }
#     _defaults = {
#                 'hs_pipeline':'128294',
#                 'is_created':False,
#                 }
#===============================================================================
    
class ticket_field(osv.osv):
    _name = 'ticket.field'
    
    _columns = {
                'company_id':fields.many2one('res.company','Company'),
                'name':fields.char('Name'),
                'desc':fields.char('Field title'),
                'config_id':fields.many2one('hubspot.conf','Hub conf')
                
        }
    
class hubspot_log(osv.osv):
    _name = 'hubspot.log'
    _order = 'date desc'
    
    _columns = {
                'company_id':fields.many2one('res.company','Company'),
                'name':fields.text('Hubspot log history'),
                'date':fields.datetime('Log date'),
                'config_id':fields.many2one('hubspot.conf','Hub conf'),
                'ticket_id':fields.integer('Hubspot ID', help=u'Hubspot-ийн id',readonly=True), #hubspot id
        }
    
class feedback_state(osv.osv):
    _name = 'feedback.state'
    
    _columns = {
                'company_id':fields.many2one('res.company',u'Компани'),
                'name':fields.char(u'Төлөвийн нэр',required=True),
                'code':fields.char(u'Код',required=True),
        }
    
class feedback_receiver(osv.osv):
    _name = 'feedback.receiver'
    
    _columns = {
                'company_id':fields.many2one('res.company',u'Компани'),
                'name':fields.char(u'Хүлээн авагчийн нэр',required=True),
        }
    
    
class sub_type_new(osv.osv):
    _name = 'sub.type.new'
    
    _columns = {
                'name':fields.char('Feedback sub type name',required=True)
                
        }
    
    
class feedback_sub_type(osv.osv):
    _name = 'feedback.sub.type'
    
    _columns = {
                'company_id':fields.many2one('res.company','Company',required=True),
                'name':fields.char('Feedback sub type name /old/',required=True)
                
        }


class res_users(osv.osv):
    _inherit = 'res.users'
    
    _columns = {
                'hubspot_id':fields.integer('Hubspot user id'),
        
        }
