# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    
#
##############################################################################

import logging
import time
import datetime
from datetime import date, timedelta, datetime
from openerp import SUPERUSER_ID
from openerp import tools
from openerp.modules.module import get_module_resource
from openerp.osv import fields, osv
from openerp.tools import float_is_zero
from openerp.tools.translate import _
from calendar import month, day_abbr, day_name
from dateutil import rrule
import math
from dateutil.relativedelta import relativedelta

class tbab_overtime(osv.osv):
    _name = 'tbab.overtime'
    _description = 'Tbab Overtime'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    #_order = 'sequence'
    
    def _employee_get(self, cr, uid, context=None):        
        emp_id = context.get('default_employee_id', False)
        if emp_id:
            return emp_id
        ids = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)], context=context)
        if ids:
            return ids[0]
        return False
    
    _columns = {
        'name': fields.text('Илүү цагаар ажиллах шалтгаан', required=True),
        'employee_id': fields.many2one('hr.employee', 'Ажилтан', track_visibility='onchange'),
        'department_id': fields.many2one('hr.department', 'Харьяалагдах нэгж'),
        'job_id': fields.many2one('hr.job', 'Албан тушаал'),
        'date_start': fields.datetime('Эхлэх огноо', select=True, copy=False),
        'date_end': fields.datetime('Дуусах огноо', select=True, copy=False,),
        'hours': fields.float('Илүү ажилласан цаг'),
        'manager_id': fields.many2one('hr.employee', 'Зөвшөөрөл өгсөн удирдлагын нэр'),
        
        'in_time': fields.float('Ирсэн цаг'),
        'out_time': fields.float('Явсан цаг'),
        
        'state': fields.selection([('requested', 'Хүсэлт'),('approved', 'Батлагдсан'),('cancelled', 'Цуцлагдсан')], 'Төлөв', track_visibility='onchange'),
        
    }

    _defaults = {
        'state': 'requested',
        'employee_id': _employee_get,
        
    }
    
    def action_confirm(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid,ids)
        obj_employee = self.pool.get('hr.employee')
        emp_id = obj_employee.search(cr, uid, [('user_id','=',uid)])
        emp = obj_employee.browse(cr, uid, emp_id)
        
        self.write(cr, uid, ids, {'state': 'approved',
                                  'manager_id':emp.id})
        return True
    
    def action_cancel(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid,ids)
        obj_employee = self.pool.get('hr.employee')
        emp_id = obj_employee.search(cr, uid, [('user_id','=',uid)])
        emp = obj_employee.browse(cr, uid, emp_id)
        
        self.write(cr, uid, ids, {'state': 'cancelled',
                                  'manager_id':emp.id})
        return True
    
    def action_draft(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'requested',
                                  'manager_id':False})
        return True
    
    def onchange_employee_id(self, cr, uid, ids, employee_id):
        result = {'value': {'department_id': False, 'job_id': False}}
        if employee_id:
            employee = self.pool.get('hr.employee').browse(cr, uid, employee_id)
            result['value'] = {'department_id': employee.department_id.id,
                               'job_id': employee.job_id.id}
        return result
    
    #_order = 'sequence'
    def onchange_date_start(self, cr, uid, ids, date_start, date_end):
        DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
        TIME_FORMAT = "%H:%M:%S"
        diff_hours = 0
        
        # date_end has to be greater than date_start
        if (date_start and date_end) and (date_start > date_end):
            raise osv.except_osv(_('Анхааруулга!'),_('Эхлэх огноо нь Дуусах огнооноос хойно байж болохгүй.'))
        result = {'value': {}}
        # Compute and update the hours of day - Amraltiin odriin ilvv tsag oloh
        if (date_start and date_end) and (date_start <= date_end):
            timedelta = datetime.strptime(date_end, DATETIME_FORMAT) - datetime.strptime(date_start, DATETIME_FORMAT)
            #Odor damnasan ognoo songowol aldaa zaana
            if timedelta.days > 0:
                raise osv.except_osv(_('Анхааруулга!'),_('Та өдөр болгоноор илүү цагийг тус тусад нь оруулна уу!'))
            #Ognoonuud neg odort hamaarch bwal hiih vildlvvd
            else:
                #Herew amraltiin odor ilvv tsag ajillasniig hvsch bga bol shuud ilvv tsagiin zorvvg olj tsag talbart ogno
                if datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Saturday' or datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Sunday':
                    diff_hours = float(timedelta.seconds) / 3600
                    result['value']['hours'] = diff_hours
                #Herew 'Ajliin odor'-t ajillasan ilvv tsag hvsch bwal hiih vildlvvd
                else:
                    date_start_date = date_start[0:10]  #ehleh ognoonii zowhon ognoog salgaj awah
                    date_end_date = date_end[0:10]  #duusah ognoonii zowhon ognoog salgaj awah
                    ehleh_tsag = str(date_end_date) + str(' 09:00:00')  #ajliin ehleh tsag
                    duusah_tsag = str(date_end_date) + str(' 18:00:00') #ajliin duusah tsag
                    tttt = datetime.strptime(date_start, DATETIME_FORMAT) + relativedelta(hours=8)  #tsagiin zorvv arilgalt
                    if str(tttt) <= ehleh_tsag:
                        niit_odriin_tsag = datetime.strptime(ehleh_tsag, DATETIME_FORMAT) - datetime.strptime(str(tttt), DATETIME_FORMAT)
                        #niit_odriin_tsag = datetime.strptime(niit_odriin_tsag_str, DATETIME_FORMAT)
                        datetime_bolgoh = str(date_end_date) + ' ' + str(niit_odriin_tsag)
                        #tsagiin zorvv olood ilvv tsag
                        ehlel_ilvv = datetime.strptime(str(datetime_bolgoh), DATETIME_FORMAT)
                        
                    else:
                        ehlel_ilvv_tsag = '00:00:00'
                        ehlel_ilvv = str(date_start_date) + ' ' + str(ehlel_ilvv_tsag)
                    pppp = datetime.strptime(date_end, DATETIME_FORMAT) + relativedelta(hours=8)
                    if str(pppp) >= duusah_tsag:
                        duusah_ilvv_tsag = datetime.strptime(str(pppp), DATETIME_FORMAT) - datetime.strptime(duusah_tsag, DATETIME_FORMAT)
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    else:
                        duusah_ilvv_tsag = '00:00:00'
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    duusah_ilvv_dt = datetime.strptime(duusah_ilvv, DATETIME_FORMAT)
                    diff_hours = float(str(ehlel_ilvv)[11:13]) + float(str(ehlel_ilvv)[14:16]) / 60.0 + float(str(duusah_ilvv_dt)[11:13]) + float(str(duusah_ilvv_dt)[14:16]) / 60.0
                    result['value']['hours'] = diff_hours
        else:
            result['value']['hours'] = 0

        return result
    
    
    def onchange_date_end(self, cr, uid, ids, date_start, date_end):
        DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
        TIME_FORMAT = "%H:%M:%S"
        diff_hours = 0
        
        # date_end has to be greater than date_start
        if (date_start and date_end) and (date_start > date_end):
            raise osv.except_osv(_('Анхааруулга!'),_('Эхлэх огноо нь Дуусах огнооноос хойно байж болохгүй.'))
        result = {'value': {}}
        # Compute and update the hours of day - Amraltiin odriin ilvv tsag oloh
        if (date_start and date_end) and (date_start <= date_end):
            timedelta = datetime.strptime(date_end, DATETIME_FORMAT) - datetime.strptime(date_start, DATETIME_FORMAT)
            #Odor damnasan ognoo songowol aldaa zaana
            if timedelta.days > 0:
                raise osv.except_osv(_('Анхааруулга!'),_('Та өдөр болгоноор илүү цагийг тус тусад нь оруулна уу!'))
            #Ognoonuud neg odort hamaarch bwal hiih vildlvvd
            else:
                #Herew amraltiin odor ilvv tsag ajillasniig hvsch bga bol shuud ilvv tsagiin zorvvg olj tsag talbart ogno
                if datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Saturday' or datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Sunday':
                    diff_hours = float(timedelta.seconds) / 3600
                    result['value']['hours'] = diff_hours
                #Herew 'Ajliin odor'-t ajillasan ilvv tsag hvsch bwal hiih vildlvvd
                else:
                    date_start_date = date_start[0:10]  #ehleh ognoonii zowhon ognoog salgaj awah
                    date_end_date = date_end[0:10]  #duusah ognoonii zowhon ognoog salgaj awah
                    ehleh_tsag = str(date_end_date) + str(' 09:00:00')  #ajliin ehleh tsag
                    duusah_tsag = str(date_end_date) + str(' 18:00:00') #ajliin duusah tsag
                    tttt = datetime.strptime(date_start, DATETIME_FORMAT) + relativedelta(hours=8)  #tsagiin zorvv arilgalt
                    if str(tttt) <= ehleh_tsag:
                        niit_odriin_tsag = datetime.strptime(ehleh_tsag, DATETIME_FORMAT) - datetime.strptime(str(tttt), DATETIME_FORMAT)
                        datetime_bolgoh = str(date_end_date) + ' ' + str(niit_odriin_tsag)
                        #tsagiin zorvv olood ilvv tsag
                        ehlel_ilvv = datetime.strptime(str(datetime_bolgoh), DATETIME_FORMAT)
                    else:
                        ehlel_ilvv_tsag = '00:00:00'
                        ehlel_ilvv = str(date_start_date) + ' ' + str(ehlel_ilvv_tsag)
                    pppp = datetime.strptime(date_end, DATETIME_FORMAT) + relativedelta(hours=8)
                    if str(pppp) >= duusah_tsag:
                        duusah_ilvv_tsag = datetime.strptime(str(pppp), DATETIME_FORMAT) - datetime.strptime(duusah_tsag, DATETIME_FORMAT)
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    else:
                        duusah_ilvv_tsag = '00:00:00'
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    duusah_ilvv_dt = datetime.strptime(duusah_ilvv, DATETIME_FORMAT)
                    diff_hours = float(str(ehlel_ilvv)[11:13]) + float(str(ehlel_ilvv)[14:16]) / 60.0 + float(str(duusah_ilvv_dt)[11:13]) + float(str(duusah_ilvv_dt)[14:16]) / 60.0
                    result['value']['hours'] = diff_hours
        else:
            result['value']['hours'] = 0

        return result
    
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        
        #Onchange hiih ognoonuudaa todorhoiloh
        if 'date_start' in vals:
            date_start = vals['date_start']
        if 'date_end' in vals:
            date_end = vals['date_end']
        
        DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
        TIME_FORMAT = "%H:%M:%S"
        diff_hours = 0
        
        # date_end has to be greater than date_start
        if (date_start and date_end) and (date_start > date_end):
            raise osv.except_osv(_('Анхааруулга!'),_('Эхлэх огноо нь Дуусах огнооноос хойно байж болохгүй.'))
        result = {'value': {}}
        # Compute and update the hours of day - Amraltiin odriin ilvv tsag oloh
        if (date_start and date_end) and (date_start <= date_end):
            timedelta = datetime.strptime(date_end, DATETIME_FORMAT) - datetime.strptime(date_start, DATETIME_FORMAT)
            #Odor damnasan ognoo songowol aldaa zaana
            if timedelta.days > 0:
                raise osv.except_osv(_('Анхааруулга!'),_('Та өдөр болгоноор илүү цагийг тус тусад нь оруулна уу!'))
            #Ognoonuud neg odort hamaarch bwal hiih vildlvvd
            else:
                #Herew amraltiin odor ilvv tsag ajillasniig hvsch bga bol shuud ilvv tsagiin zorvvg olj tsag talbart ogno
                if datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Saturday' or datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Sunday':
                    diff_hours = float(timedelta.seconds) / 3600
                    #result['value']['hours'] = diff_hours
                #Herew 'Ajliin odor'-t ajillasan ilvv tsag hvsch bwal hiih vildlvvd
                else:
                    date_start_date = date_start[0:10]  #ehleh ognoonii zowhon ognoog salgaj awah
                    date_end_date = date_end[0:10]  #duusah ognoonii zowhon ognoog salgaj awah
                    ehleh_tsag = str(date_end_date) + str(' 09:00:00')  #ajliin ehleh tsag
                    duusah_tsag = str(date_end_date) + str(' 18:00:00') #ajliin duusah tsag
                    tttt = datetime.strptime(date_start, DATETIME_FORMAT) + relativedelta(hours=8)  #tsagiin zorvv arilgalt
                    if str(tttt) <= ehleh_tsag:
                        niit_odriin_tsag = datetime.strptime(ehleh_tsag, DATETIME_FORMAT) - datetime.strptime(str(tttt), DATETIME_FORMAT)
                        datetime_bolgoh = str(date_end_date) + ' ' + str(niit_odriin_tsag)
                        #tsagiin zorvv olood ilvv tsag
                        ehlel_ilvv = datetime.strptime(str(datetime_bolgoh), DATETIME_FORMAT)
                    else:
                        ehlel_ilvv_tsag = '00:00:00'
                        ehlel_ilvv = str(date_start_date) + ' ' + str(ehlel_ilvv_tsag)
                    pppp = datetime.strptime(date_end, DATETIME_FORMAT) + relativedelta(hours=8)
                    if str(pppp) >= duusah_tsag:
                        duusah_ilvv_tsag = datetime.strptime(str(pppp), DATETIME_FORMAT) - datetime.strptime(duusah_tsag, DATETIME_FORMAT)
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    else:
                        duusah_ilvv_tsag = '00:00:00'
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    duusah_ilvv_dt = datetime.strptime(duusah_ilvv, DATETIME_FORMAT)
                    diff_hours = float(str(ehlel_ilvv)[11:13]) + float(str(ehlel_ilvv)[14:16]) / 60.0 + float(str(duusah_ilvv_dt)[11:13]) + float(str(duusah_ilvv_dt)[14:16]) / 60.0
                    #result['value']['hours'] = diff_hours
        else:
            diff_hours = 0
        
	#print 'hhhhhhhhh', date_start[12:14], date_start[15:16]
	#vals['in_time'] = str(date_start[12:14]) + ',' + str(date_start[15:16])
	#print 'eeeeeeeeeee', vals['in_time']
        #=======================================================================
        # if 'hours' in vals:
        #     if vals['hours'] != diff_hours:
        #         raise osv.except_osv(_('Анхааруулга!'),_('Та цагийг гараар өөрчилж болохгүй, та огноог сонгож цагаа оруулна уу!'))
        #     else:
        #         vals['hours'] = vals['hours']
        #=======================================================================
        
        
        overtime_id = super(tbab_overtime, self).create(cr, uid, vals, context=context)
        return overtime_id
    
    def write(self, cr, uid, ids, vals, context=None):
        obj = self.browse(cr, uid,ids)
        if context is None:
            context = {}
            
        #Onchange hiih ognoonuudaa todorhoiloh
        if 'date_start' in vals:
            date_start = vals['date_start']
        else:
            date_start = obj.date_start
        if 'date_end' in vals:
            date_end = vals['date_end']
        else:
            date_end = obj.date_end
        
        
        DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
        TIME_FORMAT = "%H:%M:%S"
        diff_hours = 0
        
        # date_end has to be greater than date_start
        if (date_start and date_end) and (date_start > date_end):
            raise osv.except_osv(_('Анхааруулга!'),_('Эхлэх огноо нь Дуусах огнооноос хойно байж болохгүй.'))
        result = {'value': {}}
        # Compute and update the hours of day - Amraltiin odriin ilvv tsag oloh
        if (date_start and date_end) and (date_start <= date_end):
            timedelta = datetime.strptime(date_end, DATETIME_FORMAT) - datetime.strptime(date_start, DATETIME_FORMAT)
            #Odor damnasan ognoo songowol aldaa zaana
            if timedelta.days > 0:
                raise osv.except_osv(_('Анхааруулга!'),_('Та өдөр болгоноор илүү цагийг тус тусад нь оруулна уу!'))
            #Ognoonuud neg odort hamaarch bwal hiih vildlvvd
            else:
                #Herew amraltiin odor ilvv tsag ajillasniig hvsch bga bol shuud ilvv tsagiin zorvvg olj tsag talbart ogno
                if datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Saturday' or datetime.strptime(date_end, DATETIME_FORMAT).strftime("%A") == 'Sunday':
                    diff_hours = float(timedelta.seconds) / 3600
                    #result['value']['hours'] = diff_hours
                #Herew 'Ajliin odor'-t ajillasan ilvv tsag hvsch bwal hiih vildlvvd
                else:
                    date_start_date = date_start[0:10]  #ehleh ognoonii zowhon ognoog salgaj awah
                    date_end_date = date_end[0:10]  #duusah ognoonii zowhon ognoog salgaj awah
                    ehleh_tsag = str(date_end_date) + str(' 09:00:00')  #ajliin ehleh tsag
                    duusah_tsag = str(date_end_date) + str(' 18:00:00') #ajliin duusah tsag
                    tttt = datetime.strptime(date_start, DATETIME_FORMAT) + relativedelta(hours=8)  #tsagiin zorvv arilgalt
                    if str(tttt) <= ehleh_tsag:
                        niit_odriin_tsag = datetime.strptime(ehleh_tsag, DATETIME_FORMAT) - datetime.strptime(str(tttt), DATETIME_FORMAT)
                        datetime_bolgoh = str(date_end_date) + ' ' + str(niit_odriin_tsag)
                        #tsagiin zorvv olood ilvv tsag
                        ehlel_ilvv = datetime.strptime(str(datetime_bolgoh), DATETIME_FORMAT)
                    else:
                        ehlel_ilvv_tsag = '00:00:00'
                        ehlel_ilvv = str(date_start_date) + ' ' + str(ehlel_ilvv_tsag)
                    pppp = datetime.strptime(date_end, DATETIME_FORMAT) + relativedelta(hours=8)
                    if str(pppp) >= duusah_tsag:
                        duusah_ilvv_tsag = datetime.strptime(str(pppp), DATETIME_FORMAT) - datetime.strptime(duusah_tsag, DATETIME_FORMAT)
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    else:
                        duusah_ilvv_tsag = '00:00:00'
                        duusah_ilvv = str(date_end_date) + ' ' + str(duusah_ilvv_tsag)
                    duusah_ilvv_dt = datetime.strptime(duusah_ilvv, DATETIME_FORMAT)
                    diff_hours = float(str(ehlel_ilvv)[11:13]) + float(str(ehlel_ilvv)[14:16]) / 60.0 + float(str(duusah_ilvv_dt)[11:13]) + float(str(duusah_ilvv_dt)[14:16]) / 60.0
                    #result['value']['hours'] = diff_hours
        else:
            diff_hours = 0
        
        #=======================================================================
        # if 'hours' in vals:
        #     if vals['hours'] != diff_hours:
        #         raise osv.except_osv(_('Анхааруулга!'),_('Та цагийг гараар өөрчилж болохгүй, та огноог сонгож цагаа оруулна уу!'))
        #     else:
        #         vals['hours'] = vals['hours']
        #=======================================================================
        
        return super(tbab_overtime,self).write(cr, uid, ids, vals, context)
    
    
    
    
    
