# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-TODAY OpenERP S.A. <http://www.openerp.com>
#
#    This program is free software: you can redistribute it and / or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.tools.translate import _
from openerp import SUPERUSER_ID
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT as DF
from openerp.addons.website.models.website import slug
from urlparse import urljoin
from itertools import product
from collections import Counter
from collections import OrderedDict

import datetime
import logging
import re
import uuid

_logger = logging.getLogger(__name__)


class hr_applicant_inherit(osv.Model):
    '''Create CV to NCD company.
    '''
    _name = 'hr.applicant.inherit'
    _description = 'HR applicant inherit'
    #_rec_name = 'question'
    #_rec_name = 'title'
    _inherit = ['mail.thread', 'hr.applicant', 'ir.needaction_mixin']
    # Model fields #

    _columns = {
        'name': fields.char('Name'),
        #=======================================================================
        # 'title': fields.char('Title', required=1, translate=True),
        # 'res_model': fields.char('Category'),
        # 'page_ids': fields.one2many('survey.page', 'survey_id', 'Pages', copy=True),
        # 'stage_id': fields.many2one('survey.stage', string="Stage", ondelete="set null", copy=False),
        # 'auth_required': fields.boolean('Login required',
        #     help="Users with a public link will be requested to login before taking part to the survey",
        #     oldname="authenticate"),
        # 'users_can_go_back': fields.boolean('Users can go back',
        #     help="If checked, users can go back to previous pages."),
        # 'tot_sent_survey': fields.function(_get_tot_sent_survey,
        #     string="Number of sent surveys", type="integer"),
        # 'tot_start_survey': fields.function(_get_tot_start_survey,
        #     string="Number of started surveys", type="integer"),
        # 'tot_comp_survey': fields.function(_get_tot_comp_survey,
        #     string="Number of completed surveys", type="integer"),
        # 'description': fields.html('Description', translate=True,
        #     oldname="description", help="A long description of the purpose of the survey"),
        # 'color': fields.integer('Color Index'),
        # 'user_input_ids': fields.one2many('survey.user_input', 'survey_id',
        #     'User responses', readonly=1),
        # 'designed': fields.function(_is_designed, string="Is designed?",
        #     type="boolean"),
        # 'public_url': fields.function(_get_public_url,
        #     string="Public link", type="char"),
        # 'public_url_html': fields.function(_get_public_url_html,
        #     string="Public link (html version)", type="char"),
        # 'print_url': fields.function(_get_print_url,
        #     string="Print link", type="char"),
        # 'result_url': fields.function(_get_result_url,
        #     string="Results link", type="char"),
        # 'email_template_id': fields.many2one('email.template',
        #     'Email Template', ondelete='set null'),
        # 'thank_you_message': fields.html('Thank you message', translate=True,
        #     help="This message will be displayed when survey is completed"),
        # 'quizz_mode': fields.boolean(string='Quiz mode')
        #=======================================================================
    }

    
    # Public methods #

    #===========================================================================
    # def copy_data(self, cr, uid, id, default=None, context=None):
    #     current_rec = self.read(cr, uid, id, fields=['title'], context=context)
    #     title = _("%s (copy)") % (current_rec.get('title'))
    #     default = dict(default or {}, title=title)
    #     return super(survey_survey, self).copy_data(cr, uid, id, default,
    #         context=context)
    #===========================================================================

    def next_page(self, cr, uid, user_input, page_id, go_back=False, context=None):
        '''The next page to display to the user, knowing that page_id is the id
        of the last displayed page.

        If page_id == 0, it will always return the first page of the survey.

        If all the pages have been displayed and go_back == False, it will
        return None

        If go_back == True, it will return the *previous* page instead of the
        next page.

        .. note::
            It is assumed here that a careful user will not try to set go_back
            to True if she knows that the page to display is the first one!
            (doing this will probably cause a giant worm to eat her house)'''
        survey = user_input.survey_id
        pages = list(enumerate(survey.page_ids))

        # First page
        if page_id == 0:
            return (pages[0][1], 0, len(pages) == 1)

        current_page_index = pages.index((filter(lambda p: p[1].id == page_id, pages))[0])

        # All the pages have been displayed
        if current_page_index == len(pages) - 1 and not go_back:
            return (None, -1, False)
        # Let's get back, baby!
        elif go_back and survey.users_can_go_back:
            return (pages[current_page_index - 1][1], current_page_index - 1, False)
        else:
            # This will show the last page
            if current_page_index == len(pages) - 2:
                return (pages[current_page_index + 1][1], current_page_index + 1, True)
            # This will show a regular page
            else:
                return (pages[current_page_index + 1][1], current_page_index + 1, False)

    # Actions

    def action_start_survey(self, cr, uid, ids, context=None):
        ''' Open the website page with the survey form '''
        trail = ""
        context = dict(context or {}, relative_url=True)
        if 'survey_token' in context:
            trail = "/" + context['survey_token']
        return {
            'type': 'ir.actions.act_url',
            'name': "Start Survey",
            'target': 'self',
            'url': self.read(cr, uid, ids, ['public_url'], context=context)[0]['public_url'] + trail
        }

    def action_send_survey(self, cr, uid, ids, context=None):
        ''' Open a window to compose an email, pre-filled with the survey
        message '''
        if not self._has_questions(cr, uid, ids, context=None):
            raise osv.except_osv(_('Error!'), _('You cannot send an invitation for a survey that has no questions.'))

        survey_browse = self.pool.get('survey.survey').browse(cr, uid, ids,
            context=context)[0]
        if survey_browse.stage_id.closed:
            raise osv.except_osv(_('Warning!'),
                _("You cannot send invitations for closed surveys."))

        assert len(ids) == 1, 'This option should only be used for a single \
                                survey at a time.'
        ir_model_data = self.pool.get('ir.model.data')
        templates = ir_model_data.get_object_reference(cr, uid,
                                'survey', 'email_template_survey')
        template_id = templates[1] if len(templates) > 0 else False
        ctx = dict(context)

        ctx.update({'default_model': 'survey.survey',
                    'default_res_id': ids[0],
                    'default_survey_id': ids[0],
                    'default_use_template': bool(template_id),
                    'default_template_id': template_id,
                    'default_composition_mode': 'comment'}
                   )
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'survey.mail.compose.message',
            'target': 'new',
            'context': ctx,
        }



