# -*- coding: utf-8 -*-
##############################################################################
#
#    ShineERP, Enterprise Management Solution    
#    Copyright (C) 2007-2012 ShineERP Co.,ltd (<http://www.erp.mn>). All Rights Reserved
#
#    Address : Suhkbaatar District, National ITPark, ShineERP LLC
#    Email : info@erp.mn
#    Phone : 976 + 11-318043
#
##############################################################################

import time
from lxml import etree

from openerp.osv import fields, osv
from openerp.tools.translate import _
from datetime import datetime
from openerp import netsvc
import xlwt
from xlwt import Workbook, Formula

class account_balance_sheet(osv.osv_memory):
    """
        Санхүүгийн байдлын тайлан
    """
    _inherit = "abstract.report.excel"
    _name = "account.balance.sheet"
    _description = "Balance Sheet Report"
    
    _columns = {
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'filter': fields.selection([('filter_date', 'Date'), ('filter_period', 'Periods')], "Filter by", required=True),
        'period_from': fields.many2one('account.period', 'Start Period'),
        'period_to': fields.many2one('account.period', 'End Period'),
        'date_from': fields.date("Start Date"),
        'date_to': fields.date("End Date"),
        'target_move': fields.selection([('posted', 'All Posted Entries'),
                                         ('all', 'All Entries'),
                                        ], 'Target Moves', required=True),
        'level': fields.selection([('3','3'),('4','4'),('5','5'),('6','6')], 'Account Level', required=True),
        'chart_account_id': fields.many2one('account.account', 'Chart of Account', help='Select Charts of Accounts', required=True, domain=[('parent_id','=',False)]),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True),
    }
    
    def default_chart_account(self, cr, uid, context=None):
        account_ids = self.pool.get('account.account').search(cr, uid, [('parent_id','=',False)], context=context)
        return (account_ids and account_ids[0]) or False
    
    _defaults = {
        'company_id': lambda obj,cr,uid,c=None:obj.pool.get('res.company')._company_default_get(cr,uid,'account.balance.sheet',context=c),
        'filter': 'filter_date',
        'level': '4',
        'fiscalyear_id': lambda obj,cr,uid,c=None:obj.pool.get('account.fiscalyear').find(cr, uid, context=c),
        'target_move': 'posted',
        'chart_account_id': default_chart_account
    }
    
    def onchange_filter(self, cr, uid, ids, filter='filter_date', fiscalyear_id=False, context=None):
        res = {'value': {}}
        if filter == 'filter_date':
            res['value'] = {'period_from': False, 'period_to': False, 'date_from': time.strftime('%Y-%m-01'), 'date_to': time.strftime('%Y-%m-%d')}
        elif filter == 'filter_period':
            start_period = end_period = False
            cr.execute('''
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_start ASC, p.special ASC
                               LIMIT 1) AS period_start
                UNION ALL
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_stop DESC
                               LIMIT 1) AS period_stop''', (fiscalyear_id, fiscalyear_id))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
            res['value'] = {'period_from': start_period, 'period_to': end_period, 'date_from': False, 'date_to': False}
        return res
    
    def onchange_fiscalyear(self, cr, uid, ids, fiscalyear, filter, context=None):
        start_period = end_period = False
        if filter == 'filter_period':
            cr.execute('''
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_start ASC, p.special ASC
                               LIMIT 1) AS period_start
                UNION ALL
                SELECT * FROM (SELECT p.id
                               FROM account_period p
                               LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                               WHERE f.id = %s
                               AND p.special = false
                               ORDER BY p.date_stop DESC
                               LIMIT 1) AS period_stop''', (fiscalyear, fiscalyear))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
        return {'value':{'period_from':start_period,'period_to':end_period}}
    
    def prepare_data(self, cr, uid, ids, context=None):
        fy_obj = self.pool.get('account.fiscalyear')
        period_obj = self.pool.get('account.period')
        obj = self.browse(cr, uid, ids[0], context=context)
        
        data = {
            'ids': ids,
            'model': 'account.balance.sheet',
            'form': {
                'company_id': obj.company_id.id,
                'filter': obj.filter,
                'period_from': obj.period_from.id,
                'period_to': obj.period_to.id,
                'date_from': obj.date_from,
                'date_to': obj.date_to,
                'target_move': obj.target_move,
                'level': obj.level,
                'chart_account_id': obj.chart_account_id.id,
                'fiscalyear_id': obj.fiscalyear_id.id,
                'save': obj.save
            },
            'report_name': 'account.balance.sheet'
        }
        
        if data['form']['filter'] == 'filter_date':
            fy = fy_obj.read(cr, uid, data['form']['fiscalyear_id'], ['date_start','date_stop'])
            if data['form']['date_from'] < fy['date_start'] or data['form']['date_to'] > fy['date_stop'] \
                or data['form']['date_from'] > data['form']['date_to']:
                raise osv.except_osv(_('Error'),_('Select a valid date for fiscalyear.'))
        elif data['form']['filter'] == 'filter_period':
            if not data['form']['period_from'] or not data['form']['period_to']:
                raise osv.except_osv(_('Error'),_('Select a starting and an ending period'))
            data['form']['date_from'] = period_obj.read(cr, uid, data['form']['period_from'], ['date_start'])['date_start']
            data['form']['date_to'] = period_obj.read(cr, uid, data['form']['period_to'], ['date_stop'])['date_stop']
        return data
    
    def get_log_message(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        
        date_from = (obj.period_from and obj.period_from.date_start) or obj.date_from
        date_to = (obj.period_to and obj.period_to.date_stop) or obj.date_to
        body = (u"Балансын тайлан (Эхлэх='%s', Дуусах='%s', Мод='%s')") % \
            (date_from, date_to, obj.chart_account_id.name)
        return body
    
    def get_export_data(self, cr, uid, ids, context=None):
        ''' Тайлангийн загварыг боловсруулж өгөгдлүүдийг
            тооцоолж байрлуулна.
        '''
        def name_padding(param):
            l = param['level'] - 2
            return ('  ' * l) + param['name']
        
        if context is None:
            context = {}
        datas = self.prepare_data(cr, uid, ids, context=context)
        
#         report_service = netsvc.LocalService('report.account.balance.sheet')
#         report_service.cr = cr
#         report_service.uid = uid
#         report_service.datas = datas
#         report_service.context = context
#         report_service.pool = self.pool
#         report_service.lang = context.get('lang', 'mn_MN')
        data = datas['form']
        
        period_obj = self.pool.get('account.period')
        account_line_obj = self.pool.get('account.move.line')
        account_obj = self.pool.get('account.account')
        company = self.pool.get('res.company').browse(cr, uid, data['company_id'])
        
        date_str = '%s-%s' % (
            datetime.strptime(data['date_from'],'%Y-%m-%d').strftime('%Y.%m.%d'),
            datetime.strptime(data['date_to'],'%Y-%m-%d').strftime('%Y.%m.%d')
        )
        obj = self.browse(cr, uid, ids[0], context=context)
        year = obj.fiscalyear_id.name
        document_code = ''
        directory_name = False
        if data['save']:
            generate_barcode = report_service.get_report_barcode(cr, context=context)
            document_code = report_service.code
        
        ezxf = xlwt.easyxf
        styles = self.get_easyxf_styles()
        
        
        self.context = context
        ctx = self.context.copy()
        ctx1 = self.context.copy()
        ctx['fiscalyear'] = data['fiscalyear_id']
        if data['filter'] == 'filter_period':
            ctx1['period_from'] = data['period_from']
            ctx1['period_to'] = data['period_from']
            ctx['period_from'] = data['period_from']
            ctx['period_to'] = data['period_to']
        elif data['filter'] == 'filter_date':
            ctx1['period_from'] = data['period_from']
            ctx1['period_to'] = data['period_from']
            ctx['date_from'] = data['date_from']
            ctx['date_to'] =  data['date_to']
        ctx['state'] = data['target_move']
        
        styled = ezxf('font: bold on; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        styless = ezxf('font: bold on; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        style = ezxf('font: bold off; align: wrap on, vert centre, horiz center;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        data_styless = ezxf('font: bold off; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        stylex = ezxf('font:bold off;align:wrap off,vert centre,horiz left;','#,##0.00_);(#,##0.00)')
        datas_style = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        datas_styles = ezxf('font: bold off; align: wrap on, vert centre, horiz centre;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet1_style = ezxf('font: bold on; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet1_style1 = ezxf('font: bold on; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet1_style2 = ezxf('font: bold off; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet1_style3 = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet2_style = ezxf('font: bold on; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet2_style1 = ezxf('font: bold on; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet2_style2 = ezxf('font: bold off; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet2_style3 = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet3_style = ezxf('font: bold on; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet3_style1 = ezxf('font: bold on; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet3_style2 = ezxf('font: bold off; align: wrap on, vert centre, horiz right;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet3_style3 = ezxf('font: bold off; align: wrap on, vert centre, horiz left;font: height 180;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')
        sheet3_style4 = ezxf('font: bold on; align: wrap on, vert centre, horiz centre;font: height 200;borders: left thin, right thin, top thin, bottom thin;','#,##0.00_);(#,##0.00)')




        book = xlwt.Workbook(encoding='utf8')
        sheet = book.add_sheet('Balance')
        sheet2 = book.add_sheet('Orlogo')
        sheet3 = book.add_sheet(u'Мөнгөн гүйлгээний тайлан')
        sheet4 = book.add_sheet(u'Өмчийн өөрчлөлтийн тайлан')
        sheet5 = book.add_sheet(u'Санхүүгийн тайлан тодруулга')
        sheet6 = book.add_sheet('tod1')
        sheet7 = book.add_sheet('tod2')
        sheet8 = book.add_sheet('tod3')
        sheet9 = book.add_sheet('tod4')
        sheet10 = book.add_sheet('tod5')
        
        

        sheet.row(2).height = 500
        sheet.write_merge(2, 2, 0, 1, u'БАЛАНС', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 250'))
        sheet.write_merge(2, 2, 2, 2, u'CT-1', ezxf('font:bold on;align:wrap off,vert centre,horiz right;'))
        sheet.write_merge(3, 3, 0, 2, u' %s' %(company.name,), ezxf('font:bold on;align:wrap off,vert centre,horiz centre;font: height 250 '))
        sheet.row(3).height = 500
        sheet.row(5).height = 400
        sheet.write(5, 0, u'Тайлант хугацаа: %s он' %(year,), ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet.write(5, 2, u'/мян.төг/' , ezxf('font:bold off;align:wrap off,vert centre,horiz centre;'))
        sheet.write_merge(6, 8, 0, 0, u'Балансын зүйл', styles['heading_xf'])
        sheet.write_merge(6, 6, 1, 2, u'Үлдэгдэл', styles['heading_xf'])
        sheet.write_merge(7, 7, 1, 2, u'Эхний үлдэгдэл', styles['heading_xf'])
        sheet.write(8, 1, u'01-р сарын 01', styles['heading_xf'])
        sheet.write(8, 2, u'12-р сарын 31', styles['heading_xf'])
        
        start_date = year+'-01-01'
        end_date = year+'-12-31' 
        
        sheet.write(9, 0, u'1. ХӨРӨНГӨ', sheet1_style)
        
        sheet.write(9, 1, ' ', sheet1_style1)
        sheet.write(9, 2, ' ', sheet1_style1)
        
        sheet.write(10, 0, u'\t1.1. ЭРГЭЛТИЙН ХӨРӨНГӨ', sheet1_style)
        
        sheet.write(10, 1, ' ', sheet1_style1)
        sheet.write(10, 2, ' ', sheet1_style1)
        
        sheet.write(11, 0, u'\t1.1.1 Мөнгө,түүнтэй адилтгах хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '11')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '11')]),ctx)
        if not datas:
            sheet.write(11, 1, 0, sheet1_style2)
        else:
            sheet.write(11, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(11, 2, datas1[0]['balance'], sheet1_style2)

            
        sheet.write(12, 0, u'\t1.1.2. Дансны авлага', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '12')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '12')]),ctx)
        if not datas:
            sheet.write(12, 1, 0, sheet1_style2)
        else:
            sheet.write(12, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(12, 2, datas1[0]['balance'],sheet1_style2)
        
        sheet.write(13, 0, u'\t1.1.3. Татвар, НДШ-ийн авлага', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet.write(13, 1, 0, sheet1_style2)
        else:
            sheet.write(13, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(13, 2, datas1[0]['balance'], sheet1_style2)
           
        sheet.write(14, 0, u'\t1.1.4. Бусад авлага', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet.write(14, 1, 0, sheet1_style2)
        else:
            sheet.write(14, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(14, 2, datas1[0]['balance'], sheet1_style2)

        sheet.write(15, 0, u'\t1.1.5. Бусад санхүүгийн хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet.write(15, 1, 0, sheet1_style2)
        else:
            sheet.write(15, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(15, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(16, 0, u'\t1.1.6. Бараа материал', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '16')]),ctx)
        if not datas and not datas1:
            sheet.write(16, 1, 0, sheet1_style2)
        else:
            sheet.write(16, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(16, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(17, 0, u'\t1.1.7. Урьдчилж төлсөн зардал/тооцоо', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '17')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '17')]),ctx)
        if not datas:
            sheet.write(17, 1, 0, sheet1_style2)
        else:
            sheet.write(17, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(17, 2, datas1[0]['balance'],sheet1_style2)
        
        sheet.write(18, 0, u'\t1.1.8. Бусад эргэлтийн хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '18')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '18')]),ctx)
        if not datas:
            sheet.write(18, 1, 0, sheet1_style2)
        else:
            sheet.write(18, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(18, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(19, 0, u'\t1.1.9. Борлуулах зорилгоор эзэмшиж буй эргэлтийн бус хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '19')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '19')]),ctx)
        if not datas:
            sheet.write(19, 1, 0, sheet1_style2)
        else:
            sheet.write(19, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(19, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(20, 0, u'\t1.1.10', sheet1_style3)

        if not datas:
            sheet.write(20, 1, 0, sheet1_style2)
        else:
            sheet.write(20, 1, '', sheet1_style2)
        sheet.write(20, 2, '', sheet1_style2)
        
        sheet.write(21, 0, u'\t1.1.11. Эргэлтийн хөрөнгийн дүн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet.write(21, 1, 0, sheet1_style2)
        else:
            sheet.write(21, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(21, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(22, 0, u'1.2. Эргэлтийн бус хөрөнгө', sheet1_style)
       
        sheet.write(22, 1, ' ', sheet1_style1)
        sheet.write(22, 2, ' ',sheet1_style1)
        
        sheet.write(23, 0, u'\t1.2.1. Үндсэн хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        if not datas:
            sheet.write(23, 1, 0, sheet1_style2)
        else:
            sheet.write(23, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(23, 2, datas1[0]['balance'], sheet1_style2) 
        
        sheet.write(24, 0, u'\t1.2.2. Биет бус хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '22')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '22')]),ctx)
        if not datas:
            sheet.write(24, 1, 0, sheet1_style2)
        else:
            sheet.write(24, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(24, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(25, 0, u'\t1.2.3. Биологийн хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        if not datas:
            sheet.write(25, 1, 0, sheet1_style2)
        else:
            sheet.write(25, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(25, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(26, 0, u'\t1.2.4. Урт хугацаат хөрөнгө оруулалт', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '24')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '24')]),ctx)
        if not datas:
            sheet.write(26, 1, 0, sheet1_style2)
        else:
            sheet.write(26, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(26, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(27, 0, u'\t1.2.5. Хайгуул ба үнэлгээний хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '25')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '25')]),ctx)
        if not datas:
            sheet.write(27, 1, 0, sheet1_style2)
        else:
            sheet.write(27, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(27, 2, datas1[0]['balance'],sheet1_style2)  

        sheet.write(28, 0, u'\t1.2.6. Хойшлогдсон татварын хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        if not datas and not datas1:
            sheet.write(28, 1, 0, sheet1_style2)
        else:
            sheet.write(28, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(28, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(29, 0, u'\t1.2.7. Хөрөнгө оруулалтын зориулалттай үл хөдлөх хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '27')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '27')]),ctx)
        if not datas:
            sheet.write(29, 1, 0, sheet1_style2)
        else:
            sheet.write(29, 1, datas[0]['balance'], sheet1_style2)       
        sheet.write(29, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(30, 0, u'\t1.2.8. Бусад эргэлтийн бус хөрөнгө', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '28')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '28')]),ctx)
   
        if not datas:
            sheet.write(30, 1, 0, sheet1_style2)
        else:
            sheet.write(30, 1, datas[0]['balance'], sheet1_style2)       
        sheet.write(30, 2, datas1[0]['balance'], sheet1_style2)

        sheet.write(31, 0, u'\t1.2.9  ', sheet1_style3)
        sheet.write(31, 1, ' ', sheet1_style2)
        sheet.write(31, 2, ' ', sheet1_style2)
        
        sheet.write(32, 0, u'\t1.2.10. Эргэлтийн бус хөрөнгийн дүн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '2')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '2')]),ctx)
   
        if not datas:
            sheet.write(32, 1, 0, sheet1_style2)
        else:
            sheet.write(32, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(32, 2, datas1[0]['balance'], sheet1_style2)
        
        sheet.write(33, 0, u'1.3. НИЙТ ХӨРӨНГИЙН ДҮН', sheet1_style3)
        
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', 'ACTIVE')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', 'ACTIVE')]),ctx)
        
        if not datas:
            sheet.write(33, 1, 0, sheet1_style2)
        else:
            sheet.write(33, 1, (datas[0]['balance'] or 0), sheet1_style2)
        sheet.write(33, 2, (datas1[0]['balance'] or 0), sheet1_style2)

        
        sheet.write(34, 0, u'2. ӨР ТӨЛБӨР БА ЭЗДИЙН ӨМЧ', sheet1_style)

        sheet.write(34, 1, ' ', sheet1_style1)
        sheet.write(34, 2, ' ', sheet1_style1)
        
        sheet.write(35, 0, u'2.1. Өр төлбөр', sheet1_style)
        
        sheet.write(35, 1, ' ', sheet1_style2)
        sheet.write(35, 2, ' ', sheet1_style2)
        
        sheet.write(36, 0, u'\t2.1.1. Богино хугацаат өр төлбөр', sheet1_style3)
        
        sheet.write(36, 1, ' ', sheet1_style2)
        sheet.write(36, 2, ' ', sheet1_style2)

        # Нийт санхүүгийн түрээсийн тооцооны авлага/валютаар/
        sheet.write(37, 0, u'\t\t2.1.1.1. Дансны өглөг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3101')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3101')]),ctx)
   
        if not datas:
            sheet.write(37, 1, 0, sheet1_style2)
        else:
            sheet.write(37, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(37, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(38, 0, u'\t\t2.1.1.2. Цалингийн өглөг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3102')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3102')]),ctx)
        if not datas:
            sheet.write(38, 1, 0, sheet1_style2)
        else:
            sheet.write(38, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(38, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(39, 0, u'\t\t2.1.1.3 Татварын өр', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3103')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3103')]),ctx)
        if not datas:
            sheet.write(39, 1, 0, sheet1_style2)
        else:
            sheet.write(39, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(39, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(40, 0, u'\t\t2.1.1.4. НДШ - ийн өглөг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3104')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3104')]),ctx)
        if not datas:
            sheet.write(40, 1, 0, sheet1_style2)
        else:
            sheet.write(40, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(40, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(41, 0, u'\t\t2.1.1.5. Богино хугацаат зээл', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3105')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3105')]),ctx)
        if not datas:
            sheet.write(41, 1, 0, sheet1_style2)
        else:
            sheet.write(41, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(41, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(42, 0, u'\t\t2.1.1.6. Хүүний өглөг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3106')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3106')]),ctx)
        if not datas:
            sheet.write(42, 1, 0, sheet1_style2)
        else:
            sheet.write(42, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(42, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(43, 0, u'\t\t2.1.1.7. Ногдол ашгийн өглөг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3107')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3107')]),ctx)
        if not datas:
            sheet.write(43, 1, 0, sheet1_style2)
        else:
            sheet.write(43, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(43, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(44, 0, u'\t\t2.1.1.8. Урьдчилж орсон орлого', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3108')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3108')]),ctx)
        if not datas:
            sheet.write(44, 1, 0, sheet1_style2)
        else:
            sheet.write(44, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(44, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(45, 0, u'\t\t2.1.1.9. Нөөц /өр төлбөр/', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3109')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3109')]),ctx)
        if not datas:
            sheet.write(45, 1, 0, sheet1_style2)
        else:
            sheet.write(45, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(45, 2, datas1[0]['balance'], sheet1_style2)
         
                
        sheet.write(46, 0, u'\t\t2.1.1.10. Бусад богино хугацаат өр төлбөр', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3110')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3110')]),ctx)
        if not datas:
            sheet.write(46, 1, 0, sheet1_style2)
        else:
            sheet.write(46, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(46, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(47, 0, u'\t\t2.1.1.11. Борлуулах зорилгоор эзэмшиж буй эргэлтийн бус хөрөнгө (борлуулах бүлэг хөрөнгө) - нд хамаарах өр төлбөр', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3111')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3111')]),ctx)
        if not datas:
            sheet.write(47, 1, 0, sheet1_style2)
        else:
            sheet.write(47, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(47, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(48, 0, u'\t\t2.1.1.12. ', sheet1_style3)
        
        sheet.write(48, 1, ' ', sheet1_style2)
        sheet.write(48, 2, ' ', sheet1_style2)
         
        sheet.write(49, 0, u'\t\t2.1.1.13. Богино хугацаат өр төлбөрийн дүн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '31')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '31')]),ctx)
        if not datas:
            sheet.write(49, 1, 0, sheet1_style2)
        else:
            sheet.write(49, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(49, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(50, 0, u'\t2.1.2. Урт хугацаат өр төлбөр', sheet1_style)
            
        sheet.write(50, 1, ' ', sheet1_style1)
        sheet.write(50, 2, ' ', sheet1_style1)
         
        sheet.write(51, 0, u'\t\t2.1.2.1. Урт хугацаат зээл', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3201')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3201')]),ctx)
        if not datas:
            sheet.write(51, 1, 0, sheet1_style2)
        else:
            sheet.write(51, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(51, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(52, 0, u'\t\t2.1.2.2. Нөөц /өр төлбөр/', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3202')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3202')]),ctx)
        if not datas:
            sheet.write(52, 1, 0, sheet1_style2)
        else:
            sheet.write(52, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(52, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(53, 0, u'\t\t2.1.2.3. Хойшлогдсон татварын өр', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3203')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3203')]),ctx)
        if not datas:
            sheet.write(53, 1, 0, sheet1_style2)
        else:
            sheet.write(53, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(53, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(54, 0, u'\t\t2.1.2.4. Бусад урт хугацаат өр төлбөр', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3204')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3204')]),ctx)
        if not datas:
            sheet.write(54, 1, 0, sheet1_style2)
        else:
            sheet.write(54, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(54, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(55, 0, u'\t\t2.1.2.5. ', sheet1_style3)
        sheet.write(55, 1, ' ', sheet1_style2)
        sheet.write(55, 2, ' ', sheet1_style2)
         
        sheet.write(56, 0, u'\t\t2.1.2.6. Урт хугацаат өр төлбөрийн дүн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '32')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '32')]),ctx)
        if not datas:
            sheet.write(56, 1, 0, sheet1_style2)
        else:
            sheet.write(56, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(56, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(57, 0, u'2.2. Өр төлбөрийн нийт дүн', sheet1_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '3')]),ctx)
        if not datas:
            sheet.write(57, 1, 0, sheet1_style2)
        else:
            sheet.write(57, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(57, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(58, 0, u'2.3. Эздийн өмч', sheet1_style)
       
        sheet.write(58, 1, ' ', sheet1_style2)
        sheet.write(58, 2, ' ', sheet1_style2)
         
        sheet.write(59, 0, u'\t2.3.1. Өмч:\t- төрийн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41000001')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41000001')]),ctx)
        if not datas:
            sheet.write(59, 1, 0, sheet1_style2)
        else:
            sheet.write(59, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(59, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(60, 0, u'\t2.3.2. \t - хувийн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41000002')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41000002')]),ctx)
        if not datas:
            sheet.write(60, 1, 0, sheet1_style2)
        else:
            sheet.write(60, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(60, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(61, 0, u'\t2.3.3. \t- хувьцаат', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41000003')]),ctx1)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41000003')]),ctx)
        if not datas:
            sheet.write(61, 1, 0, sheet1_style2)
        else:
            sheet.write(61, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(61, 2, datas[0]['balance'], sheet1_style2)
         
        sheet.write(62, 0, u'\t2.3.4. Халаасны хувьцаа', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '42')]),ctx1)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '42')]),ctx)
        if not datas:
            sheet.write(62, 1, 0, sheet1_style2)
        else:
            sheet.write(62, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(62, 2, datas[0]['balance'], sheet1_style2)
         
        sheet.write(63, 0, u'\t2.3.5. Нэмж төлөгдсөн капитал', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '43')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '43')]),ctx)
        if not datas:
            sheet.write(63, 1, 0, sheet1_style2)
        else:
            sheet.write(63, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(63, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(64, 0, u'\t2.3.6. Хөрөнгийн дахин үнэлгээний нэмэгдэл', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '44')]),ctx1)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '44')]),ctx)
        if not datas:
            sheet.write(64, 1, 0, sheet1_style2)
        else:
            sheet.write(64, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(64, 2, datas[0]['balance'], sheet1_style2)
         
        sheet.write(65, 0, u'\t2.3.7. Гадаад валютын хөрвүүлэлтийн нөөц', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '45')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '45')]),ctx)
        if not datas:
            sheet.write(65, 1, 0, sheet1_style2)
        else:
            sheet.write(65, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(65, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(66, 0, u'\t2.3.8. Эздийн өмчийн бусад хэсэг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '46')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '46')]),ctx)
        if not datas:
            sheet.write(66, 1, 0, sheet1_style2)
        else:
            sheet.write(66, 1, datas[0]['balance'],sheet1_style2)
        sheet.write(66, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(67, 0, u'\t2.3.9. Хуримтлагдсан ашиг', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '47')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '47')]),ctx)
        if not datas:
            sheet.write(67, 1, 0, sheet1_style2)
        else:
            sheet.write(67, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(67, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(69, 0, u'\t2.3.10.', sheet1_style3)
        
        sheet.write(69, 1, ' ', sheet1_style2)
        sheet.write(69, 2, ' ', sheet1_style2)
         
        sheet.write(70, 0, u'\t2.3.11. Эздийн өмчийн дүн', sheet1_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '4')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '4')]),ctx)
        if not datas:
            sheet.write(70, 1, 0, sheet1_style2)
        else:
            sheet.write(70, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(70, 2, datas1[0]['balance'], sheet1_style2)
         
        sheet.write(71, 0, u'2.4. ӨР ТӨЛБӨР БА ЭЗДИЙН ӨМЧИЙН ДҮН', sheet1_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', 'PASSIVE')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', 'PASSIVE')]),ctx)
        if not datas:
            sheet.write(71, 1, 0, sheet1_style2)
        else:
            sheet.write(71, 1, datas[0]['balance'], sheet1_style2)
        sheet.write(71, 2, datas1[0]['balance'], sheet1_style2)

        
      
        rowx = 72
        inch = 1000
        sheet.col(0).width = 17*inch
        sheet.col(1).width = 4*inch
        sheet.col(2).width = 4*inch
        sheet.row(6).height = 300
        sheet.row(7).height = 300
        sheet.row(8).height = 300
        rowx += 2
        
        
        sheet.write_merge(rowx+2, rowx+2, 0, 2, u'Захирал....................../                     /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet.write_merge(rowx+4,rowx+4,0, 2, u'Ерөнхий нягтлан бодогч......................./                  /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        
        
        
        

        
        sheet2.write(3, 3, u'%s' %(company.name,), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        
        sheet2.write_merge(2, 2, 0, 3, u'ОРЛОГЫН ДЭЛГЭРЭНГҮЙ ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 250'))
        sheet2.row(2).height = 400
        
        sheet2.write(4, 0, u'20.... оны .... сарын ..... өдөр', ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet2.write(4, 3, u'/төгрөгөөр/', ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        
        rowx = 6
        
        sum_prev_balance = sum_balance = 0.0
        last_number = 0
        lines = account_obj.search(cr, uid, [('company_id','=',data['company_id']),('type','=','view')])
        sheet2.write_merge(rowx, rowx, 0, 1, u'Үзүүлэлт', style)
        sheet2.write(rowx, 2, u'Өмнөх оны дүн', style)
        sheet2.write(rowx, 3, u'Тайлант оны дүн', style)
        sheet2.write(rowx+1, 0, u'1', style)
        sheet2.write(rowx+1, 1, u'Борлуулалтын орлого (цэвэр)', styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','51011')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '51011')]),ctx)
        if not datas:
            sheet2.write(7, 2, 0, styled)
        else:
            sheet2.write(7, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(7, 3, datas1[0]['balance'], sheet2_style2)
        sheet2.write(rowx+2, 1, 'Борлуулалтын өртөг', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','61')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '61')]),ctx)
        if not datas:
            sheet2.write(8, 2, 0, styled)
        else:
            sheet2.write(8, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(8, 3, datas1[0]['balance'], sheet2_style2)
        sheet2.write(9, 0, u'3', styless)
        sheet2.write(9, 1, 'Нийт ашиг ( алдагдал)', sheet2_style3)
       
        sheet2.write(9, 2, Formula('C8-C9'), sheet2_style2)
        sheet2.write(9, 3, Formula('D8-D9'), sheet2_style2)
        
        sheet2.write(10, 0, u'4', styless)
        sheet2.write(10, 1, 'Түрээсийн орлого', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','5201')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '5201')]),ctx)
        if not datas:
            sheet2.write(10, 2, 0, styled)
        else:
            sheet2.write(10, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(10, 3, datas1[0]['balance'], sheet2_style2)
        sheet2.write(11, 0, u'5', styless)
        sheet2.write(11, 1, 'Хүүний орлого', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','5202')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '5202')]),ctx)
        if not datas:
            sheet2.write(11, 2, 0, sheet2_style2)
        else:
            sheet2.write(11, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(11, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(12, 0, u'6', styless)
        sheet2.write(12, 1, 'Ногдол ашгийн орлого', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','5203')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '5203')]),ctx)
        if not datas:
            sheet2.write(12, 2, 0, styled)
        else:
            sheet2.write(12, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(12, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(13, 0, u'7', styless)
        sheet2.write(13, 1, 'Эрхийн шимтгэлийн орлого', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','5204')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '5204')]),ctx)
        if not datas:
            sheet2.write(13, 2, 0, styled)
        else:
            sheet2.write(13, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(13, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(14, 0, u'8', styless)
        sheet2.write(14, 1, 'Бусад орлого', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','5206')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '5206')]),ctx)
        if not datas:
            sheet2.write(14, 2, 0, styled)
        else:
            sheet2.write(14, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(14, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(15, 0, u'9', styless)
        sheet2.write(15, 1, u'Борлуулалт, маркетингийн зардал', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','7101')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '7101')]),ctx)
        if not datas:
            sheet2.write(15, 2, 0, styled)
        else:
            sheet2.write(15, 2, -datas[0]['balance'], sheet2_style2)
        sheet2.write(15, 3, -datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(rowx+10, 0, u'10', styless)
        sheet2.write(rowx+10, 1, u'Ерөнхий ба удирдлагын зардал', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','7102')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '7102')]),ctx)
        if not datas:
            sheet2.write(16, 2, 0, sheet2_style2)
        else:
            sheet2.write(16, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(16, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(rowx+11, 0, u'11', styless)
        sheet2.write(rowx+11, 1, u'Санхүүгийн зардал', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','7103')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '7103')]),ctx)
        if not datas:
            sheet2.write(17, 2, 0, sheet2_style2)
        else:
            sheet2.write(17, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(17, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(18, 0, u'12', styless)
        sheet2.write(18, 1, u'Бусад зардал', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','7104')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '7104')]),ctx)
        if not datas:
            sheet2.write(18, 2, 0, sheet2_style2)
        else:
            sheet2.write(18, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(18, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(19, 0, u'13', styless)
        sheet2.write(19, 1, u'Гадаад валютын ханшийн зөрүүний олз (гарз)', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','84000001')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '84000001')]),ctx)
        if not datas:
            sheet2.write(19, 2, 0, sheet2_style2)
        else:
            sheet2.write(19, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(19, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(20, 0, u'14', styless)
        sheet2.write(20, 1, u'Үндсэн хөрөнгө данснаас хассаны олз (гарз)', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','84000002')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '84000002')]),ctx)
        if not datas:
            sheet2.write(20, 2, 0, sheet2_style2)
        else:
            sheet2.write(20, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(20, 3, datas1[0]['balance'], sheet2_style2)
        
        sheet2.write(21, 0, u'15', styless)
        sheet2.write(21, 1, u'Биет бус хөрөнгө данснаас хассаны олз (гарз)', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(21, 2, 0, sheet2_style2)
        else:
            sheet2.write(21, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(21, 3, datas1[0]['balance'], sheet2_style2)

        sheet2.write(22, 0, u'16', styless)
        sheet2.write(22, 1, u'Хөрөнгө оруулалт борлуулснаас үүссэн олз (гарз)', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(22, 2, 0, sheet2_style2)
        else:
            sheet2.write(22, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(22, 3, datas1[0]['balance'], sheet2_style2)

        sheet2.write(23, 0, u'17', styless)
        sheet2.write(23, 1, u'Бусад ашиг ( алдагдал)', sheet2_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(23, 2, 0, styled)
        else:
            sheet2.write(23, 2, datas[0]['balance'], sheet2_style2)
        sheet2.write(23, 3, datas1[0]['balance'], sheet2_style2)  
           
        sheet2.write(24, 0, u'18', style)
        sheet2.write(24, 1, u'Татвар төлөхийн өмнөх ашиг ( алдагдал)', sheet2_style3)
        sheet2.write(24, 2, u'', style)
        sheet2.write(24, 3, Formula('D8-D19'), sheet2_style2)

        
        sheet2.write(25, 0, u'19', style)
        sheet2.write(25, 1, u'Орлогын татварын зардал', sheet2_style3)
        sheet2.write(25, 2, u'', sheet2_style2)
        sheet2.write(25, 3, Formula('D27+D30+D33'), sheet2_style2)
        
        sheet2.write(26, 0, u'20', style)
        sheet2.write(26, 1, u'Татварын дараах ашиг (алдагдал)', sheet2_style3)
        sheet2.write(26, 2, u'', sheet2_style2)
        sheet2.write(26, 3, Formula('D28+D29'), sheet2_style2)

        sheet2.write(27, 0, u'21', style)
        sheet2.write(27, 1, u'Зогсоосон үйл ажиллагааны татварын дараах ашиг (алдагдал)', sheet2_style3)
        sheet2.write(27, 2, u'', style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(27, 3, 0, sheet2_style2)
        else:
            sheet2.write(27, 3, -datas[0]['balance'], sheet2_style2)

        sheet2.write(28, 0, u'22', style)
        sheet2.write(28, 1, u'Тайлант үеийн цэвэр ашиг ( алдагдал)', sheet2_style3)
        sheet2.write(28, 2, u'', sheet2_style2)
        sheet2.write(28, 3, Formula('D31+D32'), sheet2_style2)

        sheet2.write(29, 0, u'23', style)
        sheet2.write(29, 1, u'Бусад дэлгэрэнгүй орлого', sheet2_style3)
        sheet2.write(29, 2, u'', style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(29, 3, 0, sheet2_style2)
        else:
            sheet2.write(29, 3, -datas[0]['balance'], sheet2_style2)
 
        sheet2.write(30, 0, u'', style)
        sheet2.write(30, 1, u'Хөрөнгийн дахин үнэлгээний нэмэгдлийн зөрүү', sheet2_style3)
        sheet2.write(30, 2, u'', style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(30, 3, 0, sheet2_style2)
        else:
            sheet2.write(30, 3, -datas[0]['balance'], sheet2_style2)
        
        sheet2.write(31, 0, u'', style)
        sheet2.write(31, 1, u'Гадаад валютын хөрвүүлэлтийн зөрүү', sheet2_style3)
        sheet2.write(31, 2, u'', style)
        sheet2.write(31, 3, Formula('D34+D39'), sheet2_style2)
            
        sheet2.write(32, 0, u'', style)
        sheet2.write(32, 1, u'Бусад олз (гарз)', sheet2_style3)
        sheet2.write(32, 2, u'', style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(32, 3, 0, sheet2_style2)
        else:
            sheet2.write(32, 3, -datas[0]['balance'], sheet2_style2)

        sheet2.write(33, 0, u'24', style)
        sheet2.write(33, 1, u'Орлогын нийт дүн', sheet2_style3)
        sheet2.write(33, 2, u'', style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(33, 3, 0, sheet2_style2)
        else:
            sheet2.write(33, 3, -datas[0]['balance'], sheet2_style2)
        
        sheet2.write(34, 0, u'25', style)
        sheet2.write(34, 1, u'Нэгж хувьцаанд ногдох суурь ашиг (алдагдал)', sheet2_style3)
        sheet2.write(34, 2, u'', style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet2.write(34, 3, 0, styled)
        else:
            sheet2.write(34, 3, -datas[0]['balance'], sheet2_style2)
  
        rowx = 40
        inch = 1000
        sheet2.col(0).width = 1*inch
        sheet2.col(1).width = 17*inch
        sheet2.col(2).width = 4*inch
        sheet2.col(3).width = 4*inch
        sheet2.row(6).height = 500
        sheet2.row(57).height = 500
        sheet2.row(58).height = 500
        sheet2.row(59).height = 500
        sheet2.row(84).height = 500
        
        
        
        rowx += 2
        
        sheet2.write(rowx, 2, u'Захирал....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet2.write(rowx+2, 2, u'Ерөнхий нягтлан бодогч....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        
        
        '''------------------------------- МӨНГӨН ГҮЙЛГЭЭНИЙ ТАЙЛАН -------------------------------'''
        
        
        
        sheet3.write_merge(2, 2, 0, 2, u'МӨНГӨН ГҮЙЛГЭЭНИЙ ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 250'))
        sheet3.row(2).height = 400
        sheet3.write_merge(3, 3, 0, 0, u'%s' %(company.name,), ezxf('font:bold on;align:wrap off,vert centre,horiz centre;'))

        sheet3.write(5, 2, u'Тайлант хугацаа: %s' %(date_str,), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        sheet3.write(4, 0, u'(Аж ахуйн нэгжийн нэр)', ezxf('font:bold off;align:wrap off,vert centre,horiz centre;'))

        rowxxx = 6
        sheet3.write(rowxxx, 0, u'Мөрийн дугаар', styles['heading_xf'])
        sheet3.write(rowxxx, 1, u'Үзүүлэлт', styles['heading_xf'])
        sheet3.write(rowxxx, 2, u'Тайлант үеийн дүн', styles['heading_xf'])
        rowxxx += 1
         
        sheet3.write(7, 0, u'1. ҮЙЛ АЖИЛЛАГААНЫ МӨНГӨН ГҮЙЛГЭЭНИЙ ДҮН', sheet3_style)
        sheet3.write(7, 1, '', sheet3_style1)
        sheet3.write(7, 2, '', sheet3_style1)
         
 
        sheet3.write(8, 0, u'1.1 Мөнгөн орлогын дүн(+)', sheet3_style)
        sheet3.write(8, 1, Formula('SUM(B10+B15)'), sheet3_style1)
        sheet3.write(8, 2, Formula('SUM(C10+C15)'), sheet3_style1)
   
        sheet3.write(9, 0, u'\t Бараа борлуулсаны, үйлчилгээ үзүүлсний орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','51011')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '51011')]),ctx)
        if not datas:
            sheet3.write(9, 1, 0, sheet3_style2)
        else:
            sheet3.write(9, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(9, 2, -datas1[0]['balance'], sheet3_style2)  
 
        sheet3.write(10, 0, u'\t Эрхийн шимтгэл, хураамж, төлбөрийн орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(10, 1, 0, sheet3_style2)
        else:
            sheet3.write(10, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(10, 2, -datas1[0]['balance'], sheet3_style2)  
         
        sheet3.write(11, 0, u'\t Даатгалын нөхвөрөөс хүлээн авсан мөнгө', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx)
        if not datas:
            sheet3.write(11, 1, 0, sheet3_style2)
        else:
            sheet3.write(11, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(11, 2, -datas1[0]['balance'], sheet3_style2)  
   
        sheet3.write(12, 0, u'\t Буцаан авсан албан татвар', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(12, 1, 0, sheet3_style2)
        else:
            sheet3.write(12, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(12, 2, -datas1[0]['balance'], sheet3_style2)
  
        sheet3.write(13, 0, u'\t Татаас, санхүүжилтийн орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(13, 1, 0, sheet3_style2)
        else:
            sheet3.write(13, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(13, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(14, 0, u'\t Бусад мөнгөн орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(14, 1, 0, sheet3_style2)
        else:
            sheet3.write(14, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(14, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(15, 0, u'1.2 Мөнгөн зарлагын дүн(-)', sheet3_style)
        sheet3.write(15, 1, Formula('SUM(B17+B25)'), sheet3_style1)
        sheet3.write(15, 2, Formula('SUM(C17+C25)'), sheet3_style1)
    
        sheet3.write(16, 0, u'\t Ажиллагчдад төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(16, 1, 0, sheet3_style2)
        else:
            sheet3.write(16, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(16, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(17, 0, u'\t Нийгмийн даатгалын байгууллагад төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(17, 1, 0, sheet3_style2)
        else:
            sheet3.write(17, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(17, 2, -datas1[0]['balance'], sheet3_style2)
  
        sheet3.write(18, 0, u'\t Бараа материал худалдан авахад төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(18, 1, 0, sheet3_style2)
        else:
            sheet3.write(18, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(18, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(19, 0, u'\t Ашиглалтын зардалд төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(19, 1, 0, sheet3_style2)
        else:
            sheet3.write(19, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(19, 2, -datas1[0]['balance'], sheet3_style2)
  
        sheet3.write(20, 0, u'\t Түлш шатахуун, тээврийн хөлс, сэлбэг хэрэгсэлд төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(20, 1, 0, sheet3_style2)
        else:
            sheet3.write(20, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(20, 2, -datas1[0]['balance'], sheet3_style2) 
 
        sheet3.write(21, 0, u'\t Хүүний төлбөрт төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(21, 1, 0, sheet3_style2)
        else:
            sheet3.write(21, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(21, 2, -datas1[0]['balance'], sheet3_style2) 
 
        sheet3.write(22, 0, u'\t Татварын байгууллагад төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(22, 1, 0, sheet3_style2)
        else:
            sheet3.write(22, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(22, 2, -datas1[0]['balance'], sheet3_style2)  
 
        sheet3.write(23, 0, u'\t Даатгалын төлбөрт төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(23, 1, 0, sheet3_style2)
        else:
            sheet3.write(23, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(23, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(24, 0, u'\t Бусад мөнгөн зарлага', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(24, 1, 0, sheet3_style2)
        else:
            sheet3.write(24, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(24, 2, -datas1[0]['balance'], sheet3_style2)
  
        sheet3.write(25, 0, u'1.3. Үндсэн үйл ажиллагааны цэвэр мөнгөн гүйлгээний дүн', sheet3_style)
        sheet3.write(25, 1, Formula('B9+B16'), sheet3_style1)
        sheet3.write(25, 2, Formula('C9+C16'), sheet3_style1)
  
        sheet3.write(26, 0, u'2. Хөрөнгө оруулалтын ажиллагааны мөнгөн гүйлгээ', sheet3_style)
        sheet3.write(26, 1, '', sheet3_style2)
        sheet3.write(26, 2, '', sheet3_style2)
  
        sheet3.write(27, 0, u'2.1 Мөнгөн орлогын дүн(+)', sheet3_style)
        sheet3.write(27, 1, Formula('SUM(B29+B35)'), sheet3_style1)
        sheet3.write(27, 2, Formula('SUM(C29+C35)'), sheet3_style1)
 
        sheet3.write(28, 0, u'\t Үндсэн хөрөнгө борлуулсны орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(28, 1, 0, sheet3_style2)
        else:
            sheet3.write(28, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(28, 2, datas1[0]['balance'], sheet3_style2)
 
 
        sheet3.write(29, 0, u'\t Биет бус хөрөнгө борлуулсны орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(29, 1, 0, sheet3_style2)
        else:
            sheet3.write(29, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(29, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(30, 0, u'\t Хөрөнгө оруулалт борлуулсны орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(30, 1, 0, sheet3_style2)
        else:
            sheet3.write(30, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(30, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(31, 0, u'\t Бусад урт хугацаат хөрөнгө оруулалт борлуулсны орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(31, 1, 0, sheet3_style2)
        else:
            sheet3.write(31, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(31, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(32, 0, u'\t Бусдад олгосон зээл, мөнгөн урьдчилгааны буцаан төлөлт', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(32, 1, 0,sheet3_style2)
        else:
            sheet3.write(32, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(32, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(33, 0, u'\t Хүлээн авсан хүүний орлого', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(33, 1, 0, sheet3_style2)
        else:
            sheet3.write(33, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(33, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(34, 0, u'\t Хүлээн авсан ногдол ашиг', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(34, 1, 0, sheet3_style2)
        else:
            sheet3.write(34, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(34, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(35, 0, u'', sheet3_style3)
        sheet3.write(35, 1, '', sheet3_style2)
        sheet3.write(35, 2, '', sheet3_style2)
 
        sheet3.write(36, 0, u'1.2 Мөнгөн зарлагын дүн(-)', sheet3_style)
        sheet3.write(36, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(36, 2, Formula('SUM(C38+C42)'), sheet3_style1)
 
        sheet3.write(37, 0, u'\t Үндсэн хөрөнгө олж эзэмшихэд төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(37, 1, 0, sheet3_style2)
        else:
            sheet3.write(37, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(37, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(38, 0, u'\t Биет бус хөрөнгө олж эзэмшихэд төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(38, 1, 0, sheet3_style2)
        else:
            sheet3.write(38, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(38, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(39, 0, u'\t Хөрөнгө оруулалт олж эзэмшихэд төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(39, 1, 0, sheet3_style2)
        else:
            sheet3.write(39, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(39, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(40, 0, u'\t Бусад урт хугацаат хөрөнгө олж эзэмшихэд төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(40, 1, 0, sheet3_style2)
        else:
            sheet3.write(40, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(40, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(41, 0, u'\t Бусдад олгосон зээл болон урьдчилгаа', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(41, 1, 0, sheet3_style2)
        else:
            sheet3.write(41, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(41, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(42, 0, u'', sheet3_style3)
        sheet3.write(42, 1, '', sheet3_style2)
        sheet3.write(42, 2, '', sheet3_style2)
 
        sheet3.write(43, 0, u'2.3. Хөрөнгө оруулалтын үйл ажиллагааны цэвэр мөнгөн гүйлгээний дүн', sheet3_style)
        sheet3.write(43, 1, Formula('B38+B42'), sheet3_style1)
        sheet3.write(43, 2, Formula('C38+C42'), sheet3_style1)
 
        sheet3.write(44, 0, u'3. Санхүүгийн үйл ажиллагааны мөнгөн гүйлгээ', sheet3_style)
        sheet3.write(44, 1, '', sheet3_style2)
        sheet3.write(44, 2, '', sheet3_style2)
 
        sheet3.write(45, 0, u'3.1 Мөнгөн орлогын дүн (+)',sheet3_style3)
        sheet3.write(45, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(45, 2, Formula('SUM(C38+C42)'), sheet3_style1)
 
        sheet3.write(46, 0, u'\t Зээл авсан, өрийн үнэт цаас гаргаснаас хүлээн авсан',sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(46, 1, 0, sheet3_style2)
        else:
            sheet3.write(46, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(46, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(47, 0, u'\t Хувьцаа болон өмчийн бусад үнэт цаас гаргаснаас хүлээн авсан', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(47, 1, 0, sheet3_style2)
        else:
            sheet3.write(47, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(47, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(48, 0, u'\t Төрөл бүрийн хандив', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(48, 1, 0, sheet3_style2)
        else:
            sheet3.write(48, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(48, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(49, 0, u'', styles['text_bold_xf'])
        sheet3.write(49, 1, '', sheet3_style2)
        sheet3.write(49, 2, '', sheet3_style2)
 
        sheet3.write(50, 0, u'3.2 Мөнгөн зарлагын дүн(-)', sheet3_style)
        sheet3.write(50, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(50, 2, Formula('SUM(C38+C42)'), sheet3_style1)
 
        sheet3.write(51, 0, u'\t Зээл авсан, өрийн үнэт цаасны төлбөрт төлсөн мөнгө', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(51, 1, 0, sheet3_style2)
        else:
            sheet3.write(51, 1, datas[0]['balance'], sheet3_style2)
        sheet3.write(51, 2, datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(52, 0, u'\t Санхүүгийн түрээсийн өглөгт төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(52, 1, 0, sheet3_style2)
        else:
            sheet3.write(52, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(52, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(53, 0, u'\t Хувьцаа буцаан авахад төлсөн', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(53, 1, 0, sheet3_style2)
        else:
            sheet3.write(53, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(53, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(54, 0, u'\t Төлсөн ногдол ашиг', sheet3_style3)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1')]),ctx)
        if not datas:
            sheet3.write(54, 1, 0, sheet3_style2)
        else:
            sheet3.write(54, 1, -datas[0]['balance'], sheet3_style2)
        sheet3.write(54, 2, -datas1[0]['balance'], sheet3_style2)
 
        sheet3.write(55, 0, u'',sheet3_style)
        sheet3.write(55, 1, '', sheet3_style2)
        sheet3.write(55, 2, '', sheet3_style2)
 
        sheet3.write(56, 0, u'3.3. Санхүүгийн үйл ажиллагааны цэвэр мөнгөн гүйлгээний дүн', sheet3_style)
        sheet3.write(56, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(56, 2, Formula('SUM(C38+C42)'), sheet3_style1)
 
        sheet3.write(57, 0, u'4. Бүх мөнгөн гүйлгээ', sheet3_style3)
        sheet3.write(57, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(57, 2, Formula('SUM(C38+C42)'), sheet3_style1)
 
        sheet3.write(58, 0, u'5. Мөнгө түүнтэй адилтгах хөрөнгийн эхний үлдэгдэл',sheet3_style3)
        sheet3.write(58, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(58, 2, Formula('SUM(C38+C42)'), sheet3_style1)
   
        sheet3.write(59, 0, u'6. Мөнгө түүнтэй адилтгах хөрөнгийн эцсийн үлдэгдэл', sheet3_style3)
        sheet3.write(59, 1, Formula('SUM(B38+B42)'), sheet3_style1)
        sheet3.write(59, 2, Formula('SUM(C38+C42)'), sheet3_style1)
        
         
        rowxxx = 60
        inch = 1000
        sheet3.col(0).width = 15*inch
        sheet3.col(1).width = 4*inch
        sheet3.col(2).width = 4*inch
        sheet3.row(6).height = 500
        rowxxx += 2
         
        sheet3.write(rowxxx, 0, u'/Тамга тэмдэг, гарын үсэг /:',ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet3.write(rowxxx+1, 0, u'      ГҮЙЦЭТГЭХ ЗАХИРАЛ....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet3.write(rowxxx+3, 0, u'      НЯГТЛАН БОДОГЧ....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))       
        
        '''---------------------- Өмчийн өөрчлөлтийн тайлан ------------------------'''

        date_to_obj = datetime.strptime(data['date_to'],'%Y-%m-%d')
        
        sheet4.write_merge(2, 2, 0, 7, u'ӨМЧИЙН ӨӨРЧЛӨЛТИЙН ТАЙЛАН', ezxf('font: bold on; align: wrap off, vert centre, horiz centre;font: height 250'))
        sheet4.row(2).height = 400
        sheet4.write_merge(3, 3, 0,1, u'%s' %(company.name,), ezxf('font:bold on;align:wrap off,vert centre,horiz centre;'))
        sheet4.write_merge(4, 4, 0,1, u'(Аж ахуй нэгжийн нэр)', ezxf('font:bold off;align:wrap off,vert centre,horiz centre;'))
        sheet4.write(5, 3, u'Тайлант хугацаа: %s' %(date_str,), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        
        rowx = 6
        sheet4.write_merge(rowx, rowx,0,1, u'ҮЗҮҮЛЭЛТ', sheet3_style4)
        sheet4.write(rowx, 2, u'Өмч', sheet3_style4)
        sheet4.write(rowx, 3, u'Халаасны хувьцаа', sheet3_style4)
        sheet4.write(rowx, 4, u'Нэмж төлөгдсөн капитал', sheet3_style4)
        sheet4.write(rowx, 5, u'Хөрөнгийн дахин үнэлгээний нэмэгдэл', sheet3_style4)
        sheet4.write(rowx, 6, u'Гадаад валютын хөрвүүлэлтийн нөөц', sheet3_style4)
        sheet4.write(rowx, 7, u'Эздийн өмчийн бусад хэсэг', sheet3_style4)
        sheet4.write(rowx, 8, u'Хуримтлагдсан ашиг', sheet3_style4)
        sheet4.write(rowx, 9, u'Нийт дүн', sheet3_style4)
        rowx += 1
        num = 0
        for rowx in range(7,16):
            num += 1
            sheet4.write(rowx, 0, u'%s'%num, data_styless)
        rowx = 7
        sheet4.write(rowx, 1, u'%s оны 01 сарын 01-ний үлдэгдэл' % (int(date_to_obj.strftime('%Y'))), sheet3_style)
        sheet4.write(rowx, 2, Formula("'Balance'!B134"),sheet3_style1)
        sheet4.write(rowx, 3, u'',sheet3_style1)
        sheet4.write(rowx, 4, u'',sheet3_style1)
        sheet4.write(rowx, 5, u'',sheet3_style1)
        sheet4.write(rowx, 6, u'',sheet3_style1)
        sheet4.write(rowx, 7, u'',sheet3_style1)
        sheet4.write(rowx, 8, Formula("'Balance'!B144"),sheet3_style1)
        sheet4.write(rowx, 9, Formula('SUM(C8:G8)'),sheet3_style1)
        sheet4.write(rowx+1, 1, u'Нягтлан бодох бүртгэлийн бодлогын өөрчлөлтийн нөлөө, алдааны залруулга',sheet3_style3 )
        sheet4.write(rowx+1, 2, u'', sheet3_style2)
        sheet4.write(rowx+1, 3, u'', sheet3_style2)
        sheet4.write(rowx+1, 4, u'', sheet3_style2)
        sheet4.write(rowx+1, 5, u'', sheet3_style2)
        sheet4.write(rowx+1, 6, u'', sheet3_style2)
        sheet4.write(rowx+1, 7, u'', sheet3_style2)
        sheet4.write(rowx+1, 8, u'', sheet3_style2)
        sheet4.write(rowx+1, 9, u'', sheet3_style2)
        
        sheet4.write(rowx+2, 1, u'Залруулсан үлдэгдэл', sheet3_style)
        sheet4.write(rowx+2, 2, Formula('SUM(C8:C9)'), sheet3_style1)
        sheet4.write(rowx+2, 3, Formula('SUM(D8:D9)'), sheet3_style1)
        sheet4.write(rowx+2, 4, Formula('SUM(E8:D9)'), sheet3_style1)
        sheet4.write(rowx+2, 5, Formula('SUM(F8:F9)'), sheet3_style1)
        sheet4.write(rowx+2, 6, Formula('SUM(G8:G9)'), sheet3_style1)
        sheet4.write(rowx+2, 7, Formula('SUM(C10:G10)'), sheet3_style1)
        sheet4.write(rowx+2, 8, u'', sheet3_style2)
        sheet4.write(rowx+2, 9, u'', sheet3_style2)
        
        sheet4.write(rowx+3, 1, u'Тайлант үеийн цэвэр ашиг (алдагдал)', sheet3_style3)
        sheet4.write(rowx+3, 2, u'', sheet3_style2)
        sheet4.write(rowx+3, 3, u'', sheet3_style2)
        sheet4.write(rowx+3, 4, u'', sheet3_style2)
        sheet4.write(rowx+3, 5, u'', sheet3_style2)
        sheet4.write(rowx+3, 6, u'', sheet3_style2)
        sheet4.write(rowx+3, 7, u'', sheet3_style2)
        sheet4.write(rowx+3, 8, u'', sheet3_style2)
        sheet4.write(rowx+3, 9, u'', sheet3_style2)
        
        sheet4.write(rowx+4, 1, u'Бусад дэлгэрэнгүй орлого', sheet3_style3)
        sheet4.write(rowx+4, 2, u'', sheet3_style2)
        sheet4.write(rowx+4, 3, u'', sheet3_style2)
        sheet4.write(rowx+4, 4, u'', sheet3_style2)
        sheet4.write(rowx+4, 5, u'', sheet3_style2)
        sheet4.write(rowx+4, 6, u'', sheet3_style2)
        sheet4.write(rowx+4, 7, u'', sheet3_style2)
        sheet4.write(rowx+4, 8, u'', sheet3_style2)
        sheet4.write(rowx+4, 9, u'', sheet3_style2)
        
        sheet4.write(rowx+5, 1, u'Өмчид гарсан өөрчлөлт', sheet3_style3)
        sheet4.write(rowx+5, 2, u'', sheet3_style2)
        sheet4.write(rowx+5, 3, u'', sheet3_style2)
        sheet4.write(rowx+5, 4, u'', sheet3_style2)
        sheet4.write(rowx+5, 5, u'', sheet3_style2)
        sheet4.write(rowx+5, 6, u'', sheet3_style2)
        sheet4.write(rowx+5, 7, u'', sheet3_style2)
        sheet4.write(rowx+5, 8, u'', sheet3_style2)
        sheet4.write(rowx+5, 9, u'', sheet3_style2)
        
        sheet4.write(rowx+6, 1, u'Зарласан ногдол ашиг', sheet3_style3)
        sheet4.write(rowx+6, 2, u'', sheet3_style2)
        sheet4.write(rowx+6, 3, u'', sheet3_style2)
        sheet4.write(rowx+6, 4, u'', sheet3_style2)
        sheet4.write(rowx+6, 5, u'', sheet3_style2)
        sheet4.write(rowx+6, 6, Formula("'Orlogo'!D30"), data_styless)
        sheet4.write(rowx+6, 7, u'', sheet3_style2)
        sheet4.write(rowx+6, 8, u'', sheet3_style2)
        sheet4.write(rowx+6, 9, u'', sheet3_style2)
        
        sheet4.write(rowx+7, 1, u'Дахин үнэлгээний хэрэгжсэн дүн:',sheet3_style3 )
        sheet4.write(rowx+7, 2, u'', sheet3_style2)
        sheet4.write(rowx+7, 3, u'', sheet3_style2)
        sheet4.write(rowx+7, 4, u'', sheet3_style2)
        sheet4.write(rowx+7, 5, u'', sheet3_style2)
        sheet4.write(rowx+7, 6, Formula("'Balance'!C145"), sheet3_style2)
        sheet4.write(rowx+7, 7, u'', sheet3_style2)
        sheet4.write(rowx+7, 8, u'', sheet3_style2)
        sheet4.write(rowx+7, 9, u'', sheet3_style2)
        
        
        sheet4.write(rowx+8, 1, u'%s оны %s сарын %s-ний үлдэгдэл' % (date_to_obj.strftime('%Y'),date_to_obj.strftime('%m'),date_to_obj.strftime('%d')), sheet3_style)
        sheet4.write(rowx+8, 2, Formula('SUM(C19:C26)'), sheet3_style1)
        sheet4.write(rowx+8, 3, Formula('SUM(D19:D26)'), sheet3_style1)
        sheet4.write(rowx+8, 4, Formula('SUM(E19:D26)'), sheet3_style1)
        sheet4.write(rowx+8, 5, Formula('SUM(F19:F26)'), sheet3_style1)
        sheet4.write(rowx+8, 6, Formula('SUM(G19:G26)'), sheet3_style1)
        sheet4.write(rowx+8, 7, Formula('SUM(C28:G28)'), sheet3_style1)
        sheet4.write(rowx+8, 8, Formula('SUM(C28:G28)'), sheet3_style1)
        sheet4.write(rowx+8, 9, Formula('SUM(C28:G28)'), sheet3_style1)


        inch = 1000
        sheet4.col(0).width = 1*inch
        sheet4.col(1).width = 10*inch
        sheet4.col(2).width = 3*inch
        sheet4.col(3).width = 4*inch
        sheet4.col(4).width = 5*inch
        sheet4.col(5).width = 4*inch
        sheet4.col(6).width = 4*inch
        sheet4.col(7).width = 4*inch
        sheet4.row(6).height = 1000
        rowx += 10
        
        sheet4.write(rowx, 1, u'/Тамга тэмдэг, гарын үсэг/ :',ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet4.write(rowx, 2, u'ГҮЙЦЭТГЭХ ЗАХИРАЛ....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        sheet4.write(rowx+2, 2, u'НЯГТЛАН БОДОГЧ....................................../\
                                                 /', ezxf('font: bold off; align: wrap off, vert centre, horiz left;'))
        
        '''----------------------------- Санхүүгийн тайлангийн тодруулга ---------------------'''
        
        sheet5.write(0, 4, u'Санхүүгийн зохицуулах хорооны', ezxf('font:bold on;align:wrap off,vert centre,horiz left;'))
        sheet5.write(1, 7, u'2008 оны 03 дугаар тогтоолын хавсралт 5', ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        
        sheet5.write(2, 2, u'САНХҮҮГИЙН ТАЙЛАНГИЙН ТОДРУУЛГА', ezxf('font: bold on; align: wrap off, vert centre, horiz left;font: height 250'))
        sheet5.row(2).height = 400
        
        sheet5.write(4, 0, u"“%s” ХХК" %(company.name,), ezxf('font:bold off;align:wrap off,vert centre,horiz left;'))
        sheet5.write(4, 7, u'%s оны %s сарын %s өдөр' %(date_to_obj.strftime('%Y'), date_to_obj.strftime('%m'), date_to_obj.strftime('%d')), ezxf('font:bold off;align:wrap off,vert centre,horiz right;'))
        sheet5.write(6, 0, u'1. ТАНИЛЦУУЛГА', ezxf('font: bold on; align: wrap off, vert centre, horiz left;font: height 250'))
        sheet5.write(7, 0, u'1. Байршил', stylex)
        sheet5.write(8, 0, u'2. Шуудангийн хаяг:\t\t утас:\t\t факс:\t',stylex)
        sheet5.write(9, 0, u'3. Өмчийн хэлбэр:',stylex)
        sheet5.write(10, 0, u'4. Үйл ажиллагаа явуулж эхэлсэн огноо:',stylex)
        sheet5.write(11, 0, u'5. Харилцдаг санхүү, татварын байгууллага: ',stylex)
        sheet5.write(12, 0, u'6. Үндсэн үйл ажиллагааны чиглэл/ төрөл/:',stylex)
        sheet5.write(13, 0, u'7. Салбар, төлөөлөгчийн газрын нэр, байршил:',stylex)
        sheet5.write(14, 0, u'\t1.',stylex)
        sheet5.write(15, 0, u'\t2.',stylex)
        sheet5.write(16, 0, u'\t3.',stylex)
        sheet5.write(17, 0, u'8. Гүйцэтгэх захирлын овог \t, нэр',stylex)
        sheet5.write(18, 0, u'Утас.',stylex)
        sheet5.write(19, 0, u'9. Ерөнхий санхүүчийн овог, нэр',stylex)
        sheet5.write(20, 0, u'Утас.',stylex)
        sheet5.write(21, 0, u'2. НЯГТЛАН БОДОХ БҮРТГЭЛИЙН БОДЛОГО',stylex)
        inch = 1000
        sheet5.col(0).width = 9 * inch
        
        
        sheet6.write_merge(0, 0, 0, 1, u'3. МӨНГӨН ХӨРӨНГӨ',stylex)
        sheet6.write_merge(1, 1, 0, 1, u'Касс дахь мөнгө',stylex)
        sheet6.write_merge(3, 4, 0, 0, u'',data_styless)
        sheet6.write_merge(3, 4, 1, 1,u'Төрөл',data_styless)
        sheet6.write_merge(3, 3, 2, 3, u'Эхний үлдэгдэл',data_styless)
        sheet6.write_merge(3, 3, 4, 5, u'Эцсийн үлдэгдэл',data_styless)
        sheet6.write_merge(4, 4, 2, 2, u'валют',data_styless)
        sheet6.write_merge(4, 4, 3, 3, u'төгрөг',data_styless)
        sheet6.write_merge(4, 4, 4, 4, u'валют',data_styless)
        sheet6.write_merge(4, 4, 5, 5, u'төгрөг',data_styless)
        sheet6.write(5, 0, u'1',data_styless)
        sheet6.write(5, 1, u'Касс дахь бэлэн мөнгө /төгрөг/',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1101')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1101')]),ctx)
        if not datas:
            sheet6.write(5, 2, 0, data_styless)
        else:
            sheet6.write(5, 3, datas[0]['balance'], data_styless)
        sheet6.write(5, 5, datas1[0]['balance'], data_styless)
        sheet6.write(5, 4, u'',data_styless)
        sheet6.write(5, 2, u'',data_styless)
        
        sheet6.write(6, 0, u'2',data_styless)
        sheet6.write(6, 1, u'Касс дахь бэлэн мөнгө /гадаад валют/',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','1102')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '1102')]),ctx)
        if not datas:
            sheet6.write(6, 2, 0, data_styless)
        else:
            sheet6.write(6, 3, datas[0]['balance'], data_styless)
        sheet6.write(6, 5, datas1[0]['balance'], data_styless)
        sheet6.write(6, 4, u'',data_styless)
        sheet6.write(6, 2, u'',data_styless)
        sheet6.write(7, 0, u'3',data_styless)
        sheet6.write(7, 1, u'',data_styless)
        sheet6.write(7, 2, u'',data_styless)
        sheet6.write(7, 3, u'',data_styless)
        sheet6.write(7, 4, u'',data_styless)
        sheet6.write(7, 5, u'',data_styless)
        
        sheet6.write(10, 1, u'Банк, санхүүгийн байгууллага дахь мөнгө',data_styless)
        sheet6.write_merge(12, 13, 0, 0, u'',data_styless)
        sheet6.write_merge(12, 13, 1, 1, u'Харилцагч банкны нэр',data_styless)
        sheet6.write_merge(12, 13, 2, 2, u'Дансны дугаар',data_styless)
        sheet6.write_merge(12, 12, 3, 4, u'Эхний үлдэгдэл',data_styless)
        sheet6.write_merge(12, 12, 5, 6, u'Эцсийн үлдэгдэл',data_styless)
        sheet6.write_merge(13, 13, 3, 3, u'валют',data_styless)
        sheet6.write_merge(13, 13, 4, 4, u'төгрөг',data_styless)
        sheet6.write_merge(13, 13, 5, 5, u'валют',data_styless)
        sheet6.write_merge(13, 13, 6, 6, u'төгрөг',data_styless)
        sheet6.write(14, 0, u'1',data_styless)
        sheet6.write(14, 1, u'Банк дахь харилцах /төгрөг/',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','11')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '11')]),ctx)
        sheet6.write(14, 2, datas[0]['code'], data_styless)
        if not datas:
            sheet6.write(14, 4, 0, data_styless)
        else:
            sheet6.write(14, 4, datas[0]['balance'], data_styless)
        sheet6.write(14, 6, datas1[0]['balance'], data_styless)
        sheet6.write(14, 3, u'',data_styless)
        sheet6.write(14, 5, u'',data_styless)
        sheet6.write(15, 0, u'2',data_styless)
        sheet6.write(15, 1, u'',data_styless)
        sheet6.write(15, 2, u'',data_styless)
        sheet6.write(15, 3, u'',data_styless)
        sheet6.write(15, 4, u'',data_styless)
        sheet6.write(15, 5, u'',data_styless)
        sheet6.write(15, 6, u'',data_styless)
        sheet6.write(16, 0, u'3',data_styless)
        sheet6.write(16, 1, u'',data_styless)
        sheet6.write(16, 2, u'',data_styless)
        sheet6.write(16, 3, u'',data_styless)
        sheet6.write(16, 4, u'',data_styless)
        sheet6.write(16, 5, u'',data_styless)
        sheet6.write(16, 6, u'',data_styless)

        
        
        sheet6.write(18, 1, u'4. БОГИНО ХУГАЦААТ ХӨРӨНГӨ ОРУУЛАЛТ',data_styless)
        sheet6.write(20, 1, u'Хөрөнгө оруулалтын төрөл',datas_styles)
        sheet6.write(20, 2, u'Эхний үлдэгдэл',datas_styles)
        sheet6.write(20, 3, u'Эцсийн үлдэгдэл',datas_styles)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','11')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '11')]),ctx)
        sheet6.write(21, 1, datas[0]['name'], data_styless)
        if not datas:
            sheet6.write(21, 2, 0, data_styless)
        else:
            sheet6.write(21, 2, datas[0]['balance'], data_styless)
        sheet6.write(21, 3, datas1[0]['balance'], data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','11')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '11')]),ctx)
        sheet6.write(22, 1, datas[0]['name'], data_styless)
        if not datas:
            sheet6.write(22, 2, 0, data_styless)
        else:
            sheet6.write(22, 2, datas[0]['balance'], data_styless)
        sheet6.write(22, 3, datas1[0]['balance'], data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','11')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '11')]),ctx)
        sheet6.write(23, 1, datas[0]['name'], data_styless)
        if not datas:
            sheet6.write(23, 2, 0, data_styless)
        else:
            sheet6.write(23, 2, datas[0]['balance'], data_styless)
        sheet6.write(23, 3, datas1[0]['balance'], data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','11')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','11')]),ctx)
        sheet6.write(24, 1, datas[0]['name'], data_styless)
        if not datas:
            sheet6.write(24, 2, 0, data_styless)
        else:
            sheet6.write(24, 2, datas[0]['balance'], data_styless)
        sheet6.write(24, 3, datas1[0]['balance'], data_styless)
        
        
        sheet6.write(25, 1, u'5. ЗЭЭЛ',data_styless)
        sheet6.write_merge(26, 27, 0, 0, u'',data_styless)
        sheet6.write_merge(26, 27, 1, 1, u'Зээл төрлөөрөө',datas_styles)
        sheet6.write_merge(26, 26, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet6.write_merge(26, 26, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet6.write(27, 2, u'валют',data_styless)
        sheet6.write(27, 3, u'төгрөг',data_styless)
        sheet6.write(27, 4, u'валют',data_styless)
        sheet6.write(27, 5, u'төгрөг',data_styless)
        sheet6.write(28, 0, u'1',data_styless)
        sheet6.write(28, 1, u'Нийт зээл',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(28, 3, 0, data_styless)
        else:
            sheet6.write(28, 3, datas[0]['balance'], data_styless)
        sheet6.write(28, 5, datas1[0]['balance'], data_styless)
        sheet6.write(28, 2, u'',data_styless)
        sheet6.write(28, 4, u'',data_styless)
        sheet6.write(29, 0, u'2',data_styless)
        sheet6.write(29, 1, u'Хэвийн зээл',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(29, 3, 0, data_styless)
        else:
            sheet6.write(29, 3, datas[0]['balance'], data_styless)
        sheet6.write(29, 5, datas1[0]['balance'], data_styless)
        sheet6.write(29, 2, u'',data_styless)
        sheet6.write(29, 4, u'',data_styless)
        sheet6.write(30, 0, u'3',data_styless)
        sheet6.write(30, 1, u'Хугацаа хэтэрсэн зээл',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(30, 3, 0, data_styless)
        else:
            sheet6.write(30, 3, datas[0]['balance'], data_styless)
        sheet6.write(30, 5, datas1[0]['balance'], data_styless)
        sheet6.write(30, 2, u'',data_styless)
        sheet6.write(30, 4, u'',data_styless)
        sheet6.write(31, 0, u'4',data_styless)
        sheet6.write(31, 1, u'Чанаргүй зээл',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(31, 3, 0, data_styless)
        else:
            sheet6.write(31, 3, datas[0]['balance'], data_styless)
        sheet6.write(31, 5, datas1[0]['balance'], data_styless)
        sheet6.write(31, 2, u'',data_styless)
        sheet6.write(31, 4, u'',data_styless)
        sheet6.write(32, 0, u'5',data_styless)
        sheet6.write(32, 1, u'\t-Хэвийн бус',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(32, 3, 0, data_styless)
        else:
            sheet6.write(32, 3, datas[0]['balance'], data_styless)
        sheet6.write(32, 5, datas1[0]['balance'], data_styless)
        sheet6.write(32, 2, u'',data_styless)
        sheet6.write(32, 4, u'',data_styless)
        sheet6.write(33, 0, u'6',data_styless)
        sheet6.write(33, 1, u'\t- Эргэлзээтэй',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(33, 3, 0, data_styless)
        else:
            sheet6.write(33, 3, datas[0]['balance'], data_styless)
        sheet6.write(33, 5, datas1[0]['balance'], data_styless)
        sheet6.write(33, 2, u'',data_styless)
        sheet6.write(33, 4, u'',data_styless)
        sheet6.write(34, 0, u'7',data_styless)
        sheet6.write(34, 1, u'\t- Муу',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(34, 3, 0, data_styless)
        else:
            sheet6.write(34, 3, datas[0]['balance'], data_styless)
        sheet6.write(34, 5, datas1[0]['balance'], data_styless)
        sheet6.write(34, 2, u'',data_styless)
        sheet6.write(34, 4, u'',data_styless)
        sheet6.write(35, 0, u'8',data_styless)
        sheet6.write(35, 1, u'Зээлийн эрсдэлийн сан',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','13')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '13')]),ctx)
        if not datas:
            sheet6.write(35, 3, 0, data_styless)
        else:
            sheet6.write(35, 3, datas[0]['balance'], data_styless)
        sheet6.write(35, 5, datas1[0]['balance'], data_styless)
        sheet6.write(35, 2, u'',data_styless)
        sheet6.write(35, 4, u'',data_styless)


        sheet6.write(37, 1, u'6. САНХҮҮГИЙН ТҮРЭЭС ТООЦООНЫ АВЛАГА',data_styless)
        sheet6.write_merge(38, 39, 0, 0, u'',data_styless)
        sheet6.write_merge(38, 39, 1, 1, u'Зээл төрлөөрөө',datas_styles)
        sheet6.write_merge(38, 38, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet6.write_merge(38, 38, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet6.write(39, 2, u'валют',data_styless)
        sheet6.write(39, 3, u'төгрөг',data_styless)
        sheet6.write(39, 4, u'валют',data_styless)
        sheet6.write(39, 5, u'төгрөг',data_styless)
        sheet6.write(40, 0, u'1',data_styless)
        sheet6.write(40, 1, u'Нийт санхүүгийн түрээсийн авлага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(40, 3, 0, data_styless)
        else:
            sheet6.write(40, 3, datas[0]['balance'], data_styless)
        sheet6.write(40, 5, datas1[0]['balance'], data_styless)
        sheet6.write(40, 2, u'',data_styless)
        sheet6.write(40, 4, u'',data_styless)
        sheet6.write(41, 0, u'2',data_styless)
        sheet6.write(41, 1, u'Хэвийн санхүүгийн түрээсийн авлага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(41, 3, 0, data_styless)
        else:
            sheet6.write(41, 3, datas[0]['balance'], data_styless)
        sheet6.write(41, 5, datas1[0]['balance'], data_styless)
        sheet6.write(41, 2, datas2[0]['balance'], data_styless)
        sheet6.write(41, 4, datas3[0]['balance'], data_styless)
        sheet6.write(42, 0, u'3',data_styless)
        sheet6.write(42, 1, u'Хугацаа хэтэрсэн санхүүгийн түрээсийн авлага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(42, 3, 0, data_styless)
        else:
            sheet6.write(42, 3, datas[0]['balance'], data_styless)
        sheet6.write(42, 5, datas1[0]['balance'], data_styless)
        sheet6.write(42, 2, datas2[0]['balance'], data_styless)
        sheet6.write(42, 4, datas3[0]['balance'], data_styless)
        sheet6.write(43, 0, u'4',data_styless)
        sheet6.write(43, 1, u'Чанаргүй санхүүгийн түрээсийн',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(43, 3, 0, data_styless)
        else:
            sheet6.write(43, 3, datas[0]['balance'], data_styless)
        sheet6.write(43, 5, datas1[0]['balance'], data_styless)
        sheet6.write(43, 2, u'', data_styless)
        sheet6.write(43, 4, u'', data_styless)
        sheet6.write(44, 0, u'5',data_styless)
        sheet6.write(44, 1, u'\t-Хэвийн бус',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(44, 3, 0, data_styless)
        else:
            sheet6.write(44, 3, datas[0]['balance'], data_styless)
        sheet6.write(44, 5, datas1[0]['balance'], data_styless)
        sheet6.write(44, 2, datas2[0]['balance'], data_styless)
        sheet6.write(44, 4, datas3[0]['balance'], data_styless)
        sheet6.write(45, 0, u'6',data_styless)
        sheet6.write(45, 1, u'\t- Эргэлзээтэй',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(45, 3, 0, data_styless)
        else:
            sheet6.write(45, 3, datas[0]['balance'], data_styless)
        sheet6.write(45, 5, datas1[0]['balance'], data_styless)
        sheet6.write(45, 2, datas2[0]['balance'], data_styless)
        sheet6.write(45, 4, datas3[0]['balance'], data_styless)
        sheet6.write(46, 0, u'7',data_styless)
        sheet6.write(46, 1, u'\t- Муу',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(46, 3, 0, data_styless)
        else:
            sheet6.write(46, 3, datas[0]['balance'], data_styless)
        sheet6.write(46, 5, datas1[0]['balance'], data_styless)
        sheet6.write(46, 2, datas2[0]['balance'], data_styless)
        sheet6.write(46, 4, datas3[0]['balance'], data_styless)
        sheet6.write(47, 0, u'8',data_styless)
        sheet6.write(47, 1, u'Санхүүгийн түрээсийн эрсдэлийн сан',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','14')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '14')]),ctx)
        if not datas:
            sheet6.write(47, 3, 0, data_styless)
        else:
            sheet6.write(47, 3, datas[0]['balance'], data_styless)
        sheet6.write(47, 5, datas1[0]['balance'], data_styless)
        sheet6.write(47, 2, u'',data_styless)
        sheet6.write(47, 4, u'',data_styless)

        inch = 1000
        sheet6.col(0).width = 1*inch
        sheet6.col(1).width = 9*inch
        sheet6.col(2).width = 6*inch
        sheet6.col(3).width = 6*inch


        sheet7.write(0, 1, u'7. ФАКТОРИНГИЙН ТООЦООНЫ АВЛАГА',data_styless)
        sheet7.write_merge(2, 3, 0, 0, u'',data_styless)
        sheet7.write_merge(2, 3, 1, 1, u'Зээл төрлөөрөө',datas_styles)
        sheet7.write_merge(2, 2, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet7.write_merge(2, 2, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet7.write(3, 2, u'валют',data_styless)
        sheet7.write(3, 3, u'төгрөг',data_styless)
        sheet7.write(3, 4, u'валют',data_styless)
        sheet7.write(3, 5, u'төгрөг',data_styless)
        sheet7.write(4, 0, u'1',data_styless)
        sheet7.write(4, 1, u'Нийт факторингийн тооцооны авлага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet7.write(4, 3, 0, data_styless)
        else:
            sheet7.write(4, 3, datas[0]['balance'], data_styless)
        sheet7.write(4, 5, datas1[0]['balance'], data_styless)
        sheet7.write(4, 2, u'',data_styless)
        sheet7.write(4, 4, u'',data_styless)
        sheet7.write(5, 0, u'2',data_styless)
        sheet7.write(5, 1, u'Хэвийн факторингийн тооцооны влага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet7.write(5, 3, 0, data_styless)
        else:
            sheet7.write(5, 3, datas[0]['balance'], data_styless)
        sheet7.write(5, 5, datas1[0]['balance'], data_styless)
        sheet7.write(5, 2, datas2[0]['balance'], data_styless)
        sheet7.write(5, 4, datas3[0]['balance'], data_styless)
        sheet7.write(6, 0, u'3',data_styless)
        sheet7.write(6, 1, u'Хугацаа хэтэрсэн факторингийн тооцооны авлага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet7.write(6, 3, 0, data_styless)
        else:
            sheet7.write(6, 3, datas[0]['balance'], data_styless)
        sheet7.write(6, 5, datas1[0]['balance'], data_styless)
        sheet7.write(6, 2, datas2[0]['balance'], data_styless)
        sheet7.write(6, 4, datas3[0]['balance'], data_styless)
        sheet7.write(7, 0, u'4',data_styless)
        sheet7.write(7, 1, u'Чанаргүй факторингийн тооцооны авлага',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet7.write(7, 3, 0, data_styless)
        else:
            sheet7.write(7, 3, datas[0]['balance'], data_styless)
        sheet7.write(7, 5, datas1[0]['balance'], data_styless)
        sheet7.write(7, 2, u'', data_styless)
        sheet7.write(7, 4, u'', data_styless)
        sheet7.write(8, 0, u'5',data_styless)
        sheet7.write(8, 1, u'\t-Хэвийн бус',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet7.write(8, 3, 0, data_styless)
        else:
            sheet7.write(8, 3, datas[0]['balance'], data_styless)
        sheet7.write(8, 5, datas1[0]['balance'], data_styless)
        sheet7.write(8, 2, datas2[0]['balance'], data_styless)
        sheet7.write(8, 4, datas3[0]['balance'], data_styless)
        sheet7.write(9, 0, u'6',data_styless)
        sheet7.write(9, 1, u'\t- Эргэлзээтэй',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '15')]),ctx)
        if not datas:
            sheet7.write(9, 3, 0, data_styless)
        else:
            sheet7.write(9, 3, datas[0]['balance'], data_styless)
        sheet7.write(9, 5, datas1[0]['balance'], data_styless)
        sheet7.write(9, 2, datas2[0]['balance'], data_styless)
        sheet7.write(9, 4, datas3[0]['balance'], data_styless)
        sheet7.write(10, 0, u'7',data_styless)
        sheet7.write(10, 1, u'\t- Муу',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx)
        if not datas:
            sheet7.write(10, 3, 0, data_styless)
        else:
            sheet7.write(10, 3, datas[0]['balance'], data_styless)
        sheet7.write(10, 5, datas1[0]['balance'], data_styless)
        sheet7.write(10, 2, datas2[0]['balance'], data_styless)
        sheet7.write(10, 4, datas3[0]['balance'], data_styless)
        sheet7.write(11, 0, u'8',data_styless)
        sheet7.write(11, 1, u'Факторингийн тооцооны эрсдэлийн сан',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','15')]),ctx)
        if not datas:
            sheet7.write(11, 3, 0, data_styless)
        else:
            sheet7.write(11, 3, datas[0]['balance'], data_styless)
        sheet7.write(11, 5, datas1[0]['balance'], data_styless)
        sheet7.write(11, 2, u'', data_styless)
        sheet7.write(11, 4, u'', data_styless)

        
        sheet7.write(14, 1, u'8. САНХҮҮГИЙН ДЕРИВАТИВЫН АВЛАГА',data_styless)
        sheet7.write(16, 1, u'Санхүүгийн деривативын авлагын төрөл',data_styless)
        sheet7.write(16, 2, u'Эхний үлдэгдэл',datas_styles)
        sheet7.write(16, 3, u'Эцсийн үлдэгдэл',datas_styles)
        sheet7.write(17, 1, u'Гадаад валютын',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','18')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','18')]),ctx)
        if not datas:
            sheet7.write(17, 2, 0, data_styless)
        else:
            sheet7.write(17, 2, datas[0]['balance'], data_styless)
        sheet7.write(17, 3, datas1[0]['balance'], data_styless)
        sheet7.write(18, 1, u'Үнэт цаасны',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','18')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','18')]),ctx)
        if not datas:
            sheet7.write(18, 2, 0, data_styless)
        else:
            sheet7.write(18, 2, datas[0]['balance'], data_styless)
        sheet7.write(18, 3, datas1[0]['balance'], data_styless)
        sheet7.write(19, 1, u'',data_styless)
        sheet7.write(19, 2, u'',data_styless)
        sheet7.write(19, 3, u'',data_styless)

        sheet7.write(21, 1, u'9. БУСАД АВЛАГА',data_styless)
        sheet7.write(23, 1, u'Авлагын төрөл',datas_styles)
        sheet7.write(23, 2, u'Эхний үлдэгдэл',datas_styles)
        sheet7.write(23, 3, u'Эцсийн үлдэгдэл',datas_styles)
        sheet7.write(24, 1, u'Холбоотой  талаас авах авлага',data_styless)
        sheet7.write(24, 2, u'',data_styless)
        sheet7.write(24, 3, u'',data_styless)
        sheet7.write(25, 1, u'Хараат компаниас авах авлага',data_styless)
        sheet7.write(25, 2, u'',data_styless)
        sheet7.write(25, 3, u'',data_styless)
        sheet7.write(26, 1, u'Татварын авлага',data_styless)
        sheet7.write(26, 2, u'',data_styless)
        sheet7.write(26, 3, u'',data_styless)
        sheet7.write(27, 1, u'Бусад авлага',data_styless)
        sheet7.write(27, 2, Formula("'Balance'!B63"),data_styless)
        sheet7.write(27, 3, Formula("'Balance'!C63"),data_styless)
        sheet7.write(28, 1, u'Дүн',data_styless)
        sheet7.write(28, 2, Formula('SUM(C25:C28)'),data_styless)
        sheet7.write(28, 3, Formula('SUM(D25:D28)'),data_styless)

        sheet7.write(30, 1, u'10. УРЬДЧИЛГАА ТӨЛБӨРҮҮД',data_styless)
        sheet7.write(32, 1, u'Урьдчилгаа төлбөрийн төрөл',datas_styles)
        sheet7.write(32, 2, u'Эхний үлдэгдэл',datas_styles)
        sheet7.write(32, 3, u'Эцсийн үлдэгдэл',datas_styles)
        sheet7.write(33, 1, u'Урьдчилж төлсөн зардал',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','17')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','17')]),ctx)
        if not datas:
            sheet7.write(33, 2, 0, data_styless)
        else:
            sheet7.write(33, 2, datas[0]['balance'], data_styless)
        sheet7.write(33, 3, datas1[0]['balance'], data_styless)
        sheet7.write(34, 1, u'Бэлтгэн нийлүүлэгчдэд төлсөн урьдчилгаа',data_styless)
        sheet7.write(34, 2, u'',data_styless)
        sheet7.write(34, 3, u'',data_styless)
        sheet7.write(35, 1, u'Урьдчилж төлсөн түрээс',data_styless)
        sheet7.write(35, 2, u'',data_styless)
        sheet7.write(35, 3, u'',data_styless)
        sheet7.write(36, 1, u'Дүн',data_styless)
        sheet7.write(36, 2, Formula('SUM(C32:C34)'),data_styless)
        sheet7.write(36, 3, Formula('SUM(D32:D34)'),data_styless)
        
        sheet7.write(38, 1, u'11. ХУРИМТЛУУЛЖ ТООЦСОН ХҮҮГИЙН АВЛАГА',data_styless)
        sheet7.write(40, 1, u'Хуримтлуулж тооцсон хүүгийн авлагын төрөл',datas_styles)
        sheet7.write(40, 2, u'Эхний үлдэгдэл',datas_styles)
        sheet7.write(40, 3, u'Эцсийн үлдэгдэл',datas_styles)
        sheet7.write(41, 1, u'ХУРИМТЛУУЛЖ ТООЦСОН ЗЭЭЛИЙН ХҮҮГИЙН АВЛАГА',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','17')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','17')]),ctx)
        if not datas:
            sheet7.write(41, 2, 0, data_styless)
        else:
            sheet7.write(41, 2, datas[0]['balance'], data_styless)
        sheet7.write(41, 3, datas1[0]['balance'], data_styless)
        sheet7.write(42, 1, u'',data_styless)
        sheet7.write(42, 2, u'',data_styless)
        sheet7.write(42, 3, u'',data_styless)
        sheet7.write(43, 1, u'Дүн',data_styless)
        sheet7.write(43, 2, Formula('SUM(C39:C40)'),data_styless)
        sheet7.write(43, 3, Formula('SUM(D39:D40)'),data_styless)
        
        sheet7.write(45, 1, u'12. БАРАА МАТЕРИАЛ',data_styless)
        sheet7.write(47, 1, u'Бараа, материал, хангамжийн зүйлсийн төрөл',datas_styles)
        sheet7.write(47, 2, u'Эхний үлдэгдэл',datas_styles)
        sheet7.write(47, 3, u'Эцсийн үлдэгдэл',datas_styles)
        sheet7.write(48, 1, u'Материал, үнэ бүхий зүйлс',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','17')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','17')]),ctx)
        if not datas:
            sheet7.write(48, 2, 0, data_styless)
        else:
            sheet7.write(48, 2, datas[0]['balance'], data_styless)
        sheet7.write(48, 3, datas1[0]['balance'], data_styless)
        sheet7.write(49, 1, u'',data_styless)
        sheet7.write(49, 2, u'',data_styless)
        sheet7.write(49, 3, u'',data_styless)
        sheet7.write(50, 1, u'',data_styless)
        sheet7.write(50, 2, u'',data_styless)
        sheet7.write(50, 3, u'',data_styless)
        
        inch = 1000
        sheet7.col(0).width = 1*inch
        sheet7.col(1).width = 9*inch
        sheet7.col(2).width = 6*inch
        sheet7.col(3).width = 6*inch
        sheet7.row(38).height = 500
        
        sheet8.write(0, 1, u'13. ӨМЧЛӨХ БУСАД ҮЛ ХӨДЛӨХ ХӨРӨНГӨ',data_styless)
        sheet8.write(1, 0, u'',data_styless)
        sheet8.write(1, 1, u'Өмчлөх бусад үл хөдлөх хөрөнгө төрлөөрөө',datas_style)
        sheet8.write(1, 2, u'Эхний үлдэгдэл',datas_style)
        sheet8.write(1, 3, u'Эцсийн үлдэгдэл',datas_styles)
        sheet8.write(2, 0, u'1',data_styless)
        sheet8.write(2, 1, u'Нийт өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(2, 2, 0, data_styless)
        else:
            sheet8.write(2, 2, datas[0]['balance'], data_styless)
        sheet8.write(2, 3, datas1[0]['balance'], data_styless)
        sheet8.write(3, 0, u'2',data_styless)
        sheet8.write(3, 1, u'Хэвийн өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(3, 2, 0, data_styless)
        else:
            sheet8.write(3, 2, datas[0]['balance'], data_styless)
        sheet8.write(3, 3, datas1[0]['balance'], data_styless)
        sheet8.write(4, 0, u'3',data_styless)
        sheet8.write(4, 1, u'Хугацаа хэтэрсэн өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(4, 2, 0, data_styless)
        else:
            sheet8.write(4, 2, datas[0]['balance'], data_styless)
        sheet8.write(4, 3, datas1[0]['balance'], data_styless)
        sheet8.write(5, 0, u'4',data_styless)
        sheet8.write(5, 1, u'Чанаргүй өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(5, 2, 0, data_styless)
        else:
            sheet8.write(5, 2, datas[0]['balance'], data_styless)
        sheet8.write(5, 3, datas1[0]['balance'], data_styless)
        sheet8.write(6, 0, u'5',data_styless)
        sheet8.write(6, 1, u'\t-Хэвийн бус өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(6, 2, 0, data_styless)
        else:
            sheet8.write(6, 2, datas[0]['balance'], data_styless)
        sheet8.write(6, 3, datas1[0]['balance'], data_styless)
        sheet8.write(7, 0, u'6',data_styless)
        sheet8.write(7, 1, u'\t- Эргэлзээтэй өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(7, 2, 0, data_styless)
        else:
            sheet8.write(7, 2, datas[0]['balance'], data_styless)
        sheet8.write(7, 3, datas1[0]['balance'], data_styless)
        sheet8.write(8, 0, u'7',data_styless)
        sheet8.write(8, 1, u'\t- Муу өмчлөх бусад үл хөдлөх хөрөнгө',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(8, 2, 0, data_styless)
        else:
            sheet8.write(8, 2, datas[0]['balance'], data_styless)
        sheet8.write(8, 3, datas1[0]['balance'], data_styless)
        sheet8.write(9, 0, u'8',data_styless)
        sheet8.write(9, 1, u'\t-Өмчлөх бусад үл хөдлөх хөрөнгийн эрсдэлийн сан',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','16')]),ctx)
        if not datas:
            sheet8.write(9, 2, 0, data_styless)
        else:
            sheet8.write(9, 2, datas[0]['balance'], data_styless)
        sheet8.write(9, 3, datas1[0]['balance'], data_styless)
        
        
        sheet8.write(11, 1, u'14. ДУУСААГҮЙ БАРИЛГА',datas_styles)
        sheet8.write(13, 1, u'Дуусаагүй барилгын нэр',datas_styles)
        sheet8.write(13, 2, u'Эхэлсэн он',datas_styles)
        sheet8.write(13, 3, u'Дуусгалтын хувь',datas_styles)
        sheet8.write(13, 4, u'Нийт төсөвт өртөг',datas_styles)
        sheet8.write(13, 5, u'Ашиглалтад орох хугацаа',datas_styles)
        sheet8.write(14, 1, u'',data_styless)
        sheet8.write(14, 2, u'',data_styless)
        sheet8.write(14, 3, u'',data_styless)
        sheet8.write(14, 4, u'',data_styless)
        sheet8.write(14, 5, u'',data_styless)
        sheet8.write(15, 1, u'',data_styless)
        sheet8.write(15, 2, u'',data_styless)        
        sheet8.write(15, 3, u'',data_styless)
        sheet8.write(15, 4, u'',data_styless)        
        sheet8.write(15, 5, u'',data_styless)
        sheet8.write(16, 1, u'',data_styless)        
        sheet8.write(16, 2, u'',data_styless)
        sheet8.write(16, 3, u'',data_styless)        
        sheet8.write(16, 4, u'',data_styless)        
        sheet8.write(16, 5, u'',data_styless)
        
        
        sheet8.write(18, 1, u'15. ҮНДСЭН ХӨРӨНГӨ БА ЭЛЭГДЭЛ',data_styless)
        sheet8.write(20, 1, u'',data_styless)
        sheet8.write(20, 2, u'Газар',datas_styles)
        sheet8.write(20, 3, u'Барилга байгууламж',datas_styles)
        sheet8.write(20, 4, u'Тавилга Эд хогшил',datas_styles)
        sheet8.write(20, 5, u'Комьютер, дагалдах хэрэгсэл',datas_styles)
        sheet8.write(20, 6, u'Тээврийн хэрэгсэл',datas_styles)
        sheet8.write(20, 7, u'Бүгд',datas_styles)
        sheet8.write(21, 1, u'Эхний үлдэгдэл',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','19')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '19')]),ctx1)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','19')]),ctx1)
        if not datas:
            sheet8.write(21, 3, 0, data_styless)
        else:
            sheet8.write(21, 3, datas[0]['balance'], data_styless)
        sheet8.write(21, 4, datas1[0]['balance'], data_styless)
        sheet8.write(21, 2, u'', data_styless)
        sheet8.write(21, 5, datas2[0]['balance'], data_styless)
        sheet8.write(21, 6, u'', data_styless)
        sheet8.write(21, 7, Formula('SUM(C22:G22)'), data_styless)
        sheet8.write(22, 1, u'Нэмэгдсэн',datas_style)
        sheet8.write(22, 2, Formula('SUM(C24:C25)'),data_styless)
        sheet8.write(22, 3, Formula('SUM(D24:D25)'),data_styless)
        sheet8.write(22, 4, Formula('SUM(E24:E25)'),data_styless)
        sheet8.write(22, 5, Formula('SUM(F24:F25)'),data_styless)
        sheet8.write(22, 6, Formula('SUM(G24:G25)'),data_styless)
        sheet8.write(22, 7, Formula('SUM(H24:H25)'),data_styless)
        sheet8.write(23, 1, u'Худалдан авсан',datas_style)
        sheet8.write(23, 2, u'',data_styless)
        sheet8.write(23, 3, u'',data_styless)
        sheet8.write(23, 4, u'',data_styless)
        sheet8.write(23, 5, u'',data_styless)
        sheet8.write(23, 6, u'',data_styless)
        sheet8.write(23, 7, u'',data_styless)
        sheet8.write(24, 1, u'Үнэ төлбөргүй авсан',datas_style)
        sheet8.write(24, 2, u'',data_styless)
        sheet8.write(24, 3, u'',data_styless)
        sheet8.write(24, 4, u'',data_styless)
        sheet8.write(24, 5, u'',data_styless)
        sheet8.write(24, 6, u'',data_styless)
        sheet8.write(24, 7, u'',data_styless)
        sheet8.write(25, 1, u'Хасагдсан',datas_style)
        sheet8.write(25, 2, u'',data_styless)
        sheet8.write(25, 3, u'',data_styless)
        sheet8.write(25, 4, u'',data_styless)
        sheet8.write(25, 5, u'',data_styless)
        sheet8.write(25, 6, u'',data_styless)
        sheet8.write(25, 7, u'',data_styless)
        sheet8.write(26, 1, u'Худалдсан',datas_style)
        sheet8.write(26, 2, u'',data_styless)
        sheet8.write(26, 3, u'',data_styless)
        sheet8.write(26, 4, u'',data_styless)
        sheet8.write(26, 5, u'',data_styless)
        sheet8.write(26, 6, u'',data_styless)
        sheet8.write(26, 7, u'',data_styless)
        sheet8.write(27, 1, u'Акталсан',datas_style)
        sheet8.write(27, 2, u'',data_styless)
        sheet8.write(27, 3, u'',data_styless)
        sheet8.write(27, 4, u'',data_styless)
        sheet8.write(27, 5, u'',data_styless)
        sheet8.write(27, 6, u'',data_styless)
        sheet8.write(27, 7, u'',data_styless)
        sheet8.write(28, 1, u'Үнэгүй шилжүүлсэн',datas_style)
        sheet8.write(28, 2, u'',data_styless)
        sheet8.write(28, 3, u'',data_styless)
        sheet8.write(28, 4, u'',data_styless)
        sheet8.write(28, 5, u'',data_styless)
        sheet8.write(28, 6, u'',data_styless)
        sheet8.write(28, 7, u'',data_styless)
        sheet8.write(29, 1, u'Эцсийн үлдэгдэл',datas_style)
        sheet8.write(29, 2, Formula('C22+C23-C26'),data_styless)
        sheet8.write(29, 3, Formula('D22+D23-D26'),data_styless)
        sheet8.write(29, 4, Formula('E22+E23-E26'),data_styless)
        sheet8.write(29, 5, Formula('F22+F23-F26'),data_styless)
        sheet8.write(29, 6, Formula('G22+G23-G26'),data_styless)
        sheet8.write(29, 7, Formula('SUM(C30:G30)'),data_styless)
        sheet8.write(30, 1, u'Хуримтлагдсан элэгдэл',datas_style)
        sheet8.write(30, 2, u'',data_styless)
        sheet8.write(30, 3, u'',data_styless)
        sheet8.write(30, 4, u'',data_styless)
        sheet8.write(30, 5, u'',data_styless)
        sheet8.write(30, 6, u'',data_styless)
        sheet8.write(30, 7, u'',data_styless)
        sheet8.write(31, 1, u'Эхний үлдэгдэл',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','19')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '19')]),ctx1)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','19')]),ctx1)
        if not datas:
            sheet8.write(31, 3, 0, data_styless)
        else:
             sheet8.write(31, 3, -datas[0]['balance'], data_styless)
        sheet8.write(31, 4, -datas1[0]['balance'], data_styless)
        sheet8.write(31, 2, u'', data_styless)
        sheet8.write(31, 5, -datas2[0]['balance'], data_styless)
        sheet8.write(31, 6, u'', data_styless)
        sheet8.write(31, 7, Formula('SUM(C32:G32)'), data_styless)
        sheet8.write(32, 1, u'Тайланд жилд байгуулсан',datas_style)
        sheet8.write(32, 2, u'',data_styless)
        sheet8.write(32, 3, u'',data_styless)
        sheet8.write(32, 4, u'',data_styless)
        sheet8.write(32, 5, u'',data_styless)
        sheet8.write(32, 6, u'',data_styless)
        sheet8.write(32, 7, u'',data_styless)      
        sheet8.write(33, 1, u'Хасагдсан',datas_style)
        sheet8.write(33, 2, u'',data_styless)
        sheet8.write(33, 3, u'',data_styless)
        sheet8.write(33, 4, u'',data_styless)
        sheet8.write(33, 5, u'',data_styless)
        sheet8.write(33, 6, u'',data_styless)
        sheet8.write(33, 7, Formula('SUM(C34:G34)'),data_styless)
        sheet8.write(34, 1, u'Эцсийн үлдэгдэл',datas_style)
        sheet8.write(34, 2, Formula('C32+C33-C34'),data_styless)
        sheet8.write(34, 3, Formula('D32+D33-D34'),data_styless)
        sheet8.write(34, 4, Formula('E32+E33-E34'),data_styless)
        sheet8.write(34, 5, Formula('F32+F33-F34'),data_styless)
        sheet8.write(34, 6, Formula('G32+G33-G34'),data_styless)
        sheet8.write(34, 7, Formula('SUM(C35:G35)'),data_styless)
        
        inch = 1000
        sheet8.col(0).width = 1*inch
        sheet8.col(1).width = 11*inch
        sheet8.col(2).width = 3*inch
        sheet8.col(3).width = 4*inch
        sheet8.col(5).width = 4*inch
        sheet8.row(13).height = 500
        sheet8.row(20).height = 500
     
        sheet9.write(0, 1, u'16. БИЕТ БУС ХӨРӨНГӨ',datas_style)
        sheet9.write(2, 1, u'',data_styless)
        sheet9.write(2, 2, u'Программ хангамж',datas_styles)
        sheet9.write(2, 3, u'Патент',datas_styles)
        sheet9.write(2, 4, u'Зохиогчийн эрх',datas_styles)
        sheet9.write(2, 5, u'Бусад',datas_styles)
        sheet9.write(2, 6, u'Бүгд',datas_styles)
        sheet9.write(3, 1, u'Эхний үлдэгдэл',datas_style)
        sheet9.write(3, 2, u'',data_styless)
        sheet9.write(3, 3, u'',data_styless)
        sheet9.write(3, 4, u'',data_styless)
        sheet9.write(3, 5, u'',data_styless)
        sheet9.write(3, 6, u'',data_styless)
        sheet9.write(4, 1, u'Нэмэгдсэн',datas_style)
        sheet9.write(4, 2, u'',data_styless)
        sheet9.write(4, 3, u'',data_styless)
        sheet9.write(4, 4, u'',data_styless)
        sheet9.write(4, 5, u'',data_styless)
        sheet9.write(4, 6, u'',data_styless)
        sheet9.write(5, 1, u'Худалдан авсан',datas_style)
        sheet9.write(5, 2, u'',data_styless)
        sheet9.write(5, 3, u'',data_styless)
        sheet9.write(5, 4, u'',data_styless)
        sheet9.write(5, 5, u'',data_styless)
        sheet9.write(5, 6, u'',data_styless)
        sheet9.write(6, 1, u'Үнэ төлбөргүй авсан',datas_style)
        sheet9.write(6, 2, u'',data_styless)
        sheet9.write(6, 3, u'',data_styless)
        sheet9.write(6, 4, u'',data_styless)
        sheet9.write(6, 5, u'',data_styless)
        sheet9.write(6, 6, u'',data_styless)
        sheet9.write(7, 1, u'Хасагдсан',datas_style)
        sheet9.write(7, 2, u'',data_styless)
        sheet9.write(7, 3, u'',data_styless)
        sheet9.write(7, 4, u'',data_styless)
        sheet9.write(7, 5, u'',data_styless)
        sheet9.write(7, 6, u'',data_styless)
        sheet9.write(8, 1, u'Худалдсан',datas_style)
        sheet9.write(8, 2, u'',data_styless)
        sheet9.write(8, 3, u'',data_styless)
        sheet9.write(8, 4, u'',data_styless)
        sheet9.write(8, 5, u'',data_styless)
        sheet9.write(8, 6, u'',data_styless)
        sheet9.write(9, 1, u'Акталсан',datas_style)
        sheet9.write(9, 2, u'',data_styless)
        sheet9.write(9, 3, u'',data_styless)
        sheet9.write(9, 4, u'',data_styless)
        sheet9.write(9, 5, u'',data_styless)
        sheet9.write(9, 6, u'',data_styless)
        sheet9.write(10, 1, u'Үнэгүй шилжүүлсэн',datas_style)
        sheet9.write(10, 2, u'',data_styless)
        sheet9.write(10, 3, u'',data_styless)
        sheet9.write(10, 4, u'',data_styless)
        sheet9.write(10, 5, u'',data_styless)
        sheet9.write(10, 6, u'',data_styless)
        sheet9.write(11, 1, u'Эцсийн үлдэгдэл',datas_style)
        sheet9.write(11, 2, u'',data_styless)
        sheet9.write(11, 3, u'',data_styless)
        sheet9.write(11, 4, u'',data_styless)
        sheet9.write(11, 5, u'',data_styless)
        sheet9.write(11, 6, u'',data_styless)
        sheet9.write(12, 1, u'Хуримтлагдсан элэгдэл',datas_style)
        sheet9.write(12, 2, u'',data_styless)
        sheet9.write(12, 3, u'',data_styless)
        sheet9.write(12, 4, u'',data_styless)
        sheet9.write(12, 5, u'',data_styless)
        sheet9.write(12, 6, u'',data_styless)
        sheet9.write(13, 1, u'Эхний үлдэгдэл',datas_style)
        sheet9.write(13, 2, u'',data_styless)
        sheet9.write(13, 3, u'',data_styless)
        sheet9.write(13, 4, u'',data_styless)
        sheet9.write(13, 5, u'',data_styless)
        sheet9.write(13, 6, u'',data_styless)
        sheet9.write(14, 1, u'Тайланд жилд байгуулсан',datas_style)
        sheet9.write(14, 2, u'',data_styless)
        sheet9.write(14, 3, u'',data_styless)
        sheet9.write(14, 4, u'',data_styless)
        sheet9.write(14, 5, u'',data_styless)
        sheet9.write(14, 6, u'',data_styless)
        sheet9.write(15, 1, u'Хасагдсан',datas_style)
        sheet9.write(15, 2, u'',data_styless)
        sheet9.write(15, 3, u'',data_styless)
        sheet9.write(15, 4, u'',data_styless)
        sheet9.write(15, 5, u'',data_styless)
        sheet9.write(15, 6, u'',data_styless)
        sheet9.write(16, 1, u'Эхний үлдэгдэл',datas_style)
        sheet9.write(16, 2, u'',data_styless)
        sheet9.write(16, 3, u'',data_styless)
        sheet9.write(16, 4, u'',data_styless)
        sheet9.write(16, 5, u'',data_styless)
        sheet9.write(16, 6, u'',data_styless)
        
        
        sheet9.write(17, 1, u'17. БОГИНО ХУГАЦААТ ӨГЛӨГ',datas_style)
        sheet9.write_merge(19, 20, 0, 0, u'',data_styless)
        sheet9.write_merge(19, 20, 1, 1, u'Өглөгийн төрөл',datas_styles)
        sheet9.write_merge(19, 19, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet9.write_merge(19, 19, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet9.write(20, 2, u'валют',data_styless)
        sheet9.write(20, 3, u'төгрөг',data_styless)
        sheet9.write(20, 4, u'валют',data_styless)
        sheet9.write(20, 5, u'төгрөг',data_styless)
        sheet9.write(21, 0, u'1',datas_style)
        sheet9.write(21, 1, u'Богино хугацаат зээлийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','21')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','21')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        if not datas:
            sheet9.write(21, 3, 0, data_styless)
        else:
            sheet9.write(21, 3, -datas[0]['balance'], data_styless)
        sheet9.write(21, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(21, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(21, 4, -datas3[0]['balance'], data_styless)
        sheet9.write(22, 0, u'2',datas_style)
        sheet9.write(22, 1, u'-Төлөгдөх хугацаандаа байгаа',datas_style)
        sheet9.write(22, 2, u'',data_styless)
        sheet9.write(22, 3, u'',data_styless)
        sheet9.write(22, 4, u'',data_styless)
        sheet9.write(22, 5, u'',data_styless)
        sheet9.write(23, 0, u'3',datas_style)
        sheet9.write(23, 1, u'-Хугацаа хэтэрсэн',datas_style)
        sheet9.write(23, 2, u'',data_styless)
        sheet9.write(23, 3, u'',data_styless)
        sheet9.write(23, 4, u'',data_styless)
        sheet9.write(23, 5, u'',data_styless)
        sheet9.write(24, 0, u'4',datas_style)
        sheet9.write(24, 1, u'Өрийн бичгийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','24')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '24')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','24')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '24')]),ctx)
        if not datas:
            sheet9.write(24, 3, 0, data_styless)
        else:
            sheet9.write(24, 3, -datas[0]['balance'], data_styless)
        sheet9.write(24, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(24, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(24, 4, -datas3[0]['balance'], data_styless)
        
        sheet9.write(25, 0, u'5',datas_style)
        sheet9.write(25, 1, u'Санхүүгийн деривативын өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','28')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '28')]),ctx)
        if not datas:
            sheet9.write(25, 3, 0, data_styless)
        else:
            sheet9.write(25, 3, datas[0]['balance'], data_styless)
        sheet9.write(25, 5, datas1[0]['balance'], data_styless)
        sheet9.write(25, 2, u'', data_styless)
        sheet9.write(25, 4, u'', data_styless)
        sheet9.write(26, 0, u'6',datas_style)
        sheet9.write(26, 1, u'Төслийн зээлийн богино хугацаат санхүүжилт',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','22')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '22')]),ctx)
        if not datas:
            sheet9.write(26, 3, 0, -data_styless)
        else:
            sheet9.write(26, 3, -datas[0]['balance'], data_styless)
        sheet9.write(26, 5, -datas1[0]['balance'], data_styless)
        
        sheet9.write(26, 2, u'', data_styless)
        sheet9.write(26, 4, u'', data_styless)
        sheet9.write(27, 0, u'7',datas_style)
        sheet9.write(27, 1, u'Бусад өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','27')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '27')]),ctx)
        if not datas:
            sheet9.write(27, 3, 0, -data_styless)
        else:
            sheet9.write(27, 3, -datas[0]['balance'], data_styless)
        sheet9.write(27, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(27, 2, u'',data_styless)
        sheet9.write(27, 4, u'',data_styless)
        sheet9.write(28, 0, u'',data_styless)
        sheet9.write(28, 1, u'Нийт дүн',datas_styles)
        sheet9.write(28, 2, Formula('SUM(C22:C28)'),data_styless)
        sheet9.write(28, 3, Formula('SUM(D22:D28)'),data_styless)
        sheet9.write(28, 4, Formula('SUM(E22:E28)'),data_styless)
        sheet9.write(28, 5, Formula('SUM(F22:F28)'),data_styless)     
        
        sheet9.write(33, 1, u'18. БУСАД ЭХ ҮҮСВЭРИЙН ӨГЛӨГ',datas_style)
        sheet9.write_merge(35, 36, 0, 0, u'',data_styless)
        sheet9.write_merge(35, 36, 1, 1, u'Өглөгийн төрөл',datas_styles)
        sheet9.write_merge(35, 35, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet9.write_merge(35, 35, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet9.write(36, 2, u'валют',data_styless)
        sheet9.write(36, 3, u'төгрөг',data_styless)
        sheet9.write(36, 4, u'валют',data_styless)
        sheet9.write(36, 5, u'төгрөг',data_styless)
        sheet9.write(37, 0, u'1',datas_style)
        sheet9.write(37, 1, u'Санхүүгийн түрээсийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','21')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','21')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        if not datas:
            sheet9.write(37, 3, 0, data_styless)
        else:
            sheet9.write(37, 3, -datas[0]['balance'], data_styless)
        sheet9.write(37, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(37, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(37, 4, -datas3[0]['balance'], data_styless)
        
        sheet9.write(38, 0, u'2',datas_style)
        sheet9.write(38, 1, u'Итгэлцлийн үйлчилгээний өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','21')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','21')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '21')]),ctx)
        if not datas:
            sheet9.write(38, 3, 0, data_styless)
        else:
            sheet9.write(38, 3, -datas[0]['balance'], data_styless)
        sheet9.write(38, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(38, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(38, 4, -datas3[0]['balance'], data_styless)
        
        sheet9.write(39, 0, u'3',datas_style)
        sheet9.write(39, 1, u'Факторингийн үйлчилгээний өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        if not datas:
            sheet9.write(39, 3, 0, data_styless)
        else:
            sheet9.write(39, 3, -datas[0]['balance'], data_styless)
        sheet9.write(39, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(39, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(39, 4, -datas3[0]['balance'], data_styless)
        
        sheet9.write(40, 0, u'4',data_styless)
        sheet9.write(40, 1, u'Мөнгөн гуйвуулгын өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        if not datas:
            sheet9.write(40, 3, 0, data_styless)
        else:
            sheet9.write(40, 3, -datas[0]['balance'], data_styless)
        sheet9.write(40, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(40, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(40, 4, -datas3[0]['balance'], data_styless)
        
        sheet9.write(41, 0, u'5',datas_style)
        sheet9.write(41, 1, u'Зээлийн батлан даалтанд байршуулсан эх үүсвэр',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        if not datas:
            sheet9.write(41, 3, 0, data_styless)
        else:
            sheet9.write(41, 3, datas[0]['balance'], data_styless)
        sheet9.write(41, 5, datas1[0]['balance'], data_styless)
        sheet9.write(41, 2, datas2[0]['balance'], data_styless)
        sheet9.write(41, 4, datas3[0]['balance'], data_styless)
        sheet9.write(42, 0, u'6',datas_style)
        sheet9.write(42, 1, u'Төлбөрийн болон зээлийн картын үйлчилгээтэй холбоотой өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','23')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '23')]),ctx)
        if not datas:
            sheet9.write(42, 3, 0, data_styless)
        else:
            sheet9.write(42, 3, -datas[0]['balance'], data_styless)
        sheet9.write(42, 5, -datas1[0]['balance'], data_styless)
        sheet9.write(42, 2, -datas2[0]['balance'], data_styless)
        sheet9.write(42, 4, -datas3[0]['balance'], data_styless)
        sheet9.write(43, 0, u'',data_styless)
        sheet9.write(43, 1, u'Дүн',data_styless)
        sheet9.write(43, 2, Formula('SUM(C38:C43)'),data_styless)
        sheet9.write(43, 3, Formula('SUM(D38:D43)'),data_styless)
        sheet9.write(43, 4, Formula('SUM(E38:E43)'),data_styless)
        sheet9.write(43, 5, Formula('SUM(F38:F43)'),data_styless)

        inch = 1000
        sheet9.col(0).width = 1*inch
        sheet9.col(1).width = 9*inch
        sheet9.col(2).width = 6*inch
        sheet9.col(3).width = 6*inch
        sheet9.row(42).height = 500

        sheet10.write_merge(0, 0, 0, 2, u'19. ХУРИМТЛУУЛЖ ТООЦСОН ХҮҮГИЙН ӨГЛӨГ',datas_style)
        sheet10.write_merge(2, 2, 0, 0, u'',data_styless)
        sheet10.write_merge(2, 2, 1, 1, u'Хуримтлуулж тооцсон хүүгийн өглөгийн төрөл',datas_styles)
        sheet10.write_merge(2, 2, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet10.write_merge(2, 2, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet10.write(3, 2, u'валют',data_styless)
        sheet10.write(3, 3, u'төгрөг',data_styless)
        sheet10.write(3, 4, u'валют',data_styless)
        sheet10.write(3, 5, u'төгрөг',data_styless)
        sheet10.write(4, 0, u'1',datas_style)
        sheet10.write(4, 1, u'Хуримтлуулж тооцсон зээлийн хүүгийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        if not datas:
            sheet10.write(4, 3, 0, data_styless)
        else:
            sheet10.write(4, 3, datas[0]['balance'], data_styless)
        sheet10.write(4, 5, datas1[0]['balance'], data_styless)
        sheet10.write(4, 2, datas2[0]['balance'], data_styless)
        sheet10.write(4, 4, datas3[0]['balance'], data_styless)
        sheet10.write(5, 0, u'2',datas_style)
        sheet10.write(5, 1, u'Үнэт цаасны хүүгийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        if not datas:
            sheet10.write(5, 3, 0, data_styless)
        else:
            sheet10.write(5, 3, -datas[0]['balance'], data_styless)
        sheet10.write(5, 5, -datas1[0]['balance'], data_styless)
        sheet10.write(5, 2, -datas2[0]['balance'], data_styless)
        sheet10.write(5, 4, -datas3[0]['balance'], data_styless)
        sheet10.write(6, 0, u'3',datas_style)
        sheet10.write(6, 1, u'Хуримтлуулж тооцсон бусад хүүгийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        if not datas:
            sheet10.write(6, 3, 0, data_styless)
        else:
            sheet10.write(6, 3, -datas[0]['balance'], data_styless)
        sheet10.write(6, 5, -datas1[0]['balance'], data_styless)
        sheet10.write(6, 2, -datas2[0]['balance'], data_styless)
        sheet10.write(6, 4, -datas3[0]['balance'], data_styless)

        sheet10.write(7, 0, u'4',datas_style)
        sheet10.write(7, 1, u'Бусад',datas_style)
        sheet10.write(7, 2, u'',data_styless)
        sheet10.write(7, 3, u'',data_styless)
        sheet10.write(7, 4, u'',data_styless)
        sheet10.write(7, 5, u'',data_styless)
        sheet10.write(8, 0, u'',data_styless)
        sheet10.write(8, 1, u'Дүн',datas_styles)
        sheet10.write(8, 2, Formula('SUM(C5:C8)'),data_styless)
        sheet10.write(8, 3, Formula('SUM(D5:D8)'),data_styless)
        sheet10.write(8, 4, Formula('SUM(E5:E8)'),data_styless)
        sheet10.write(8, 5, Formula('SUM(F5:F8)'),data_styless)

        sheet10.write_merge(11, 11, 0, 2, u'20. УРТ ХУГАЦААТ ӨГЛӨГ',datas_style)
        sheet10.write_merge(13, 14, 0, 0, u'',data_styless)
        sheet10.write_merge(13, 14, 1, 1, u'Өглөгийн төрөл',datas_styles)
        sheet10.write_merge(13, 13, 2, 3, u'Эхний үлдэгдэл',datas_styles)
        sheet10.write_merge(13, 13, 4, 5, u'Эцсийн үлдэгдэл',datas_styles)
        sheet10.write(14, 2, u'валют',data_styless)
        sheet10.write(14, 3, u'төгрөг',data_styless)
        sheet10.write(14, 4, u'валют',data_styless)
        sheet10.write(14, 5, u'төгрөг',data_styless)
        sheet10.write(15, 0, u'1',datas_style)
        sheet10.write(15, 1, u'Урт хугацаат зээлийн өглөг',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        datas2 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','26')]),ctx1)
        datas3 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '26')]),ctx)
        if not datas:
            sheet10.write(15, 3, 0, data_styless)
        else:
            sheet10.write(15, 3, -datas[0]['balance'], data_styless)
        sheet10.write(15, 5, -datas1[0]['balance'], data_styless)
        sheet10.write(15, 2, -datas2[0]['balance'], data_styless)
        sheet10.write(15, 4, -datas3[0]['balance'], data_styless)
        sheet10.write(16, 0, u'2',datas_style)
        sheet10.write(16, 1, u'\t-төлөгдөх хугацаандаа байгаа',datas_style)
        sheet10.write(16, 2, u'',data_styless)
        sheet10.write(16, 3, u'',data_styless)
        sheet10.write(16, 4, u'',data_styless)
        sheet10.write(16, 5, u'',data_styless)
        sheet10.write(17, 0, u'3',datas_style)
        sheet10.write(17, 1, u'\t-Хугацаа хэтэрсэн',datas_style)
        sheet10.write(17, 2, u'',data_styless)
        sheet10.write(17, 3, u'',data_styless)
        sheet10.write(17, 4, u'',data_styless)
        sheet10.write(17, 5, u'',data_styless)
        sheet10.write(18, 0, u'4',datas_style)
        sheet10.write(18, 1, u'Төслийн зээлийн урт хугацаат санхүүжилт',datas_style)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','25')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '25')]),ctx)
        if not datas:
            sheet10.write(18, 3, 0, data_styless)
        else:
            sheet10.write(18, 3, -datas[0]['balance'], data_styless)
        sheet10.write(18, 5, -datas1[0]['balance'], data_styless)
        sheet10.write(18, 2, u'', data_styless)
        sheet10.write(18, 4, u'', data_styless)
        sheet10.write(19, 0, u'',data_styless)
        sheet10.write(19, 1, u'Дүн',datas_styles)
        sheet10.write(19, 2, Formula('SUM(C16:C19)'),data_styless)
        sheet10.write(19, 3, Formula('SUM(D16:D19)'),data_styless)
        sheet10.write(19, 4, Formula('SUM(E16:E19)'),data_styless)
        sheet10.write(19, 5, Formula('SUM(F16:F19)'),data_styless)

        sheet10.write_merge(21, 21, 1, 1, u'21. ЭЗЭМШИГЧИЙН ӨМЧ',datas_style)     
        sheet10.write_merge(23, 23, 1, 3, u'Дүрмийн санд орсон өөрчлөлт, энгийн ба давуу эрхтэй хувьцааны тоо ширхэг, хувь нийлүүлэгчийн талаар',data_styless)     
        sheet10.write_merge(24, 24, 1, 3, u'тэмдэглэл хийнэ. Хоёрдогч өглөгийн талаар нэмж тодруулна.',data_styless)     


        sheet10.write_merge(33, 33, 0, 1, u'22. ОРЛОГО',datas_style)
        sheet10.write_merge(35, 36, 1, 1, u'Орлогын төрөл',datas_styles)
        sheet10.write_merge(35, 36, 2, 2, u'Өмнөх он',datas_styles)
        sheet10.write_merge(35, 36, 3, 3, u'Тайлант он',datas_styles)
        sheet10.write(37, 1, u'Зээлийн хүүгийн орлого',data_styless)
        datas = account_obj.browse(cr, uid, account_obj.search(cr, uid, [('code','=','41')]),ctx1)
        datas1 = account_obj.browse(cr, uid, account_obj.search(cr, uid, [( 'code', '=', '41')]),ctx)
        if not datas:
            sheet10.write(37, 2, 0, data_styless)
        else:
            sheet10.write(37, 2, -datas[0]['balance'], data_styless)
        sheet10.write(37, 3, -datas1[0]['balance'], data_styless)
        sheet10.write(38, 1, u'',data_styless)
        sheet10.write(38, 2, u'',data_styless)
        sheet10.write(38, 3, u'',data_styless)
        sheet10.write(39, 1, u'',data_styless)
        sheet10.write(39, 2, u'',data_styless)
        sheet10.write(39, 3, u'',data_styless)
        sheet10.write(40, 1, u'',data_styless)
        sheet10.write(40, 2, u'',data_styless)
        sheet10.write(40, 3, u'',data_styless)
        sheet10.write(41, 1, u'Дүн',datas_styles)
        sheet10.write(41, 2, Formula('SUM(C38:C41)'),data_styless)
        sheet10.write(41, 3, Formula('SUM(D38:D41)'),data_styless)
        
        inch = 1000
        sheet10.col(0).width = 1*inch
        sheet10.col(1).width = 9*inch
        sheet10.col(2).width = 6*inch
        sheet10.col(3).width = 6*inch
#         sheet10.row().height = 300
#         sheet10.row(8).height = 300

        
        return {'data':book}

account_balance_sheet()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: